/********************************************************************/
/*                                                                  */
/*  Cyrene_Engine.h :  Header de gestion du simulateur du 65c816.   */
/*                                                                  */
/********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Mai 2018  */
/********************************************************************/

struct s65816
{
  /* CPU */
  struct cpu *current_cpu;

  /* Mmoire (RAM, ROM, DOC RAM, DOC Registers, Battery RAM) */
  struct mmu *current_mmu;

  /* Liste des Handle Mmoires */
  int nb_used_handle;
  int size_used_handle;
  struct gs_handle *list_used_handle;
  char error_used[1024];

  int nb_purged_handle;
  int size_purged_handle;
  struct gs_handle *list_purged_handle;
  char error_purged[1024];

  int nb_free_handle;
  int size_free_handle;
  struct gs_handle *list_free_handle;
  char error_free[1024];

  /* Liste des Fichiers en Mmoire */
  struct file_id_table *table_file_id;

  /* Liste des CDA */
  struct cda_table *current_cda;

  /* Liste des NDA */
  struct nda_table *current_nda;

  /* Vecteurs Quickdraw */
  struct quickdraw_vector_table *current_quickdraw_vector;

  /* Vecteurs Interrupt */
  struct interrupt_vector_table *current_interrupt_vector;

  /* Liste des File Segment */
  struct file_segment_table *current_file_segment;

  /* Liste des Init (TIF + PIF + ToolSetup + FinderExtra) */
  struct init_table *current_init;

  /* Liste des Driver */
  struct driver_table *current_driver;

  /* Liste des FST */
  struct fst_table *current_fst;
};

void Initialize65816Engine(struct snapshot *);
void mem_free_s65816(struct s65816 *);

/********************************************************************/
