/**************************************************************************/
/*                                                                        */
/*  Dc_CPU.c :  Module pour la bibliothque de gestion du Microprocessor. */
/*                                                                        */
/**************************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe Software   *  Avril 2018  */
/**************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <windows.h>

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_CPU.h"


/**********************************************************/
/*  GetCPUData() :  Rcupre une information dans le CPU. */
/**********************************************************/
int GetCPUData(int type_data, struct cpu *current_cpu)
{
  int value;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_cpu_threadhandle); 
  
  /** Rcupre une information du CPU **/
  switch(type_data)
    {
      /** Register **/
      case CPU_REGISTER_A :
        value = current_cpu->proc_a;
        break;

      case CPU_REGISTER_X :
        value = current_cpu->proc_x;
        break;

      case CPU_REGISTER_Y :
        value = current_cpu->proc_y;
        break;

      case CPU_REGISTER_DBR :
        value = current_cpu->proc_dbr;
        break;

      case CPU_REGISTER_PBR :
        value = current_cpu->proc_pbr;
        break;

      case CPU_REGISTER_PC :
        value = current_cpu->proc_pc;
        break;

      case CPU_REGISTER_STACK :
        value = current_cpu->proc_stack;
        break;

      case CPU_REGISTER_DIRECTPAGE :
        value = current_cpu->proc_directpage;
        break;

      /** Status Register **/
      case CPU_STATUS_C :
        value = current_cpu->status_c;
        break;

      case CPU_STATUS_Z :
        value = current_cpu->status_z;
        break;

      case CPU_STATUS_I :
        value = current_cpu->status_i;
        break;

      case CPU_STATUS_D :
        value = current_cpu->status_d;
        break;

      case CPU_STATUS_X :
        value = current_cpu->status_x;
        break;

      case CPU_STATUS_M :
        value = current_cpu->status_m;
        break;

      case CPU_STATUS_V :
        value = current_cpu->status_v;
        break;

      case CPU_STATUS_N :
        value = current_cpu->status_n;
        break;

      case CPU_STATUS_E :
        value = current_cpu->status_e;
        break;

      case CPU_STATUS_B :
        value = current_cpu->status_b;
        break;

      /** Adresses Stack et Direct Page **/
      case CPU_ADDRESS_STACK :
        value = (current_cpu->directpage_stack_bank << 16) | current_cpu->proc_stack;
        break;

      case CPU_ADDRESS_DIRECTPAGE :
        value = (current_cpu->directpage_stack_bank << 16) | current_cpu->proc_directpage;
        break;

      case CPU_BANK_DIRECTPAGE_STACK :
        value = current_cpu->directpage_stack_bank;
        break;
    }

  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_cpu_threadhandle);

  /* Renvoi la valeur */
  return(value);
}


/************************************************************/
/*  SetCPUData() :  Mat  jour une information dans le CPU. */
/************************************************************/
void SetCPUData(int type_data, int value, struct cpu *current_cpu)
{

}

/**************************************************************************/
