/************************************************************************/
/*                                                                      */
/*  Dc_Disassembler.h :  Header pour la bibliothque du dsassemblage.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Avril 2018  */
/************************************************************************/

#define AM_UNKOWN                                0  /* Not recognize */
#define AM_IMPLICIT                              1  /* A          Implicit                             */
#define AM_ABSOLUTE                              2  /* addr2      Absolute                             */
#define AM_ABSOLUTE_INDEXED_X_INDIRECT           3  /* (addr2,X)  Absolute Indexed,X Indirect          */
#define AM_ABSOLUTE_INDEXED_X                    4  /* addr2,X    Absolute Indexed,X                   */
#define AM_ABSOLUTE_INDEXED_Y                    5  /* addr2,Y    Absolute Indexed,Y                   */
#define AM_ABSOLUTE_INDIRECT                     6  /* (addr2)    Absolute Indirect                    */
#define AM_ABSOLUTE_INDIRECT_LONG                7  /* [addr2]    Absolute Indirect Long               */
#define AM_ABSOLUTE_LONG                         8  /* addr3      Absolute Long                        */
#define AM_ABSOLUTE_LONG_INDEXED_X               9  /* addr3,X    Absolute Long Indexed,X              */
#define AM_DIRECT_PAGE                          10  /* dp         Direct Page                          */
#define AM_DIRECT_PAGE_INDEXED_X                11  /* dp,X       Direct Page Indexed,X                */
#define AM_DIRECT_PAGE_INDEXED_Y                12  /* dp,Y       Direct Page Indexed,Y                */
#define AM_DIRECT_PAGE_INDIRECT                 13  /* (dp)       Direct Page Indirect                 */
#define AM_DIRECT_PAGE_INDIRECT_LONG            14  /* [dp]       Direct Page Indirect Long            */
#define AM_DIRECT_PAGE_INDEXED_X_INDIRECT       15  /* (dp,X)     Direct Page Indexed,X Indirect       */
#define AM_DIRECT_PAGE_INDIRECT_INDEXED_Y       16  /* (dp),Y     Direct Page Indirect Indexed,Y       */
#define AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y  17  /* [dp],Y     Direct Page Indirect Long Indexed,Y  */
#define AM_IMMEDIATE_8                          18  /* #const 8   Immediate 8 bit                      */
#define AM_IMMEDIATE_16                         19  /* #const 16  Immediate 16 bit                      */
#define AM_PC_RELATIVE                          20  /* relative1  Program Counter Relative 8 bit (BRA, BEQ, BPL...) */
#define AM_PC_RELATIVE_LONG                     21  /* relative2  Program Counter Relative Long (BRL)  */
#define AM_STACK_RELATIVE_INDIRECT_INDEXED_Y    22  /* (sr,S),Y   Stack Relative Indirect Indexed,Y    */
#define AM_STACK_PC_RELATIVE_LONG               23  /* label      Stack PC Relative Long (PER) =  AM_PC_RELATIVE_LONG) */
#define AM_STACK_RELATIVE                       24  /* sr,S       Stack Relative                       */
#define AM_BLOCK_MOVE                           25  /* Src,Dst    Block Move (MVN, MVP)                */

struct code_line
{
  int is_directive;   /* Ce n'est pas une ligne de code */

  int m;              /* Taille des Registres */
  int x;

  int address;        /* Bank / Address */

  int data_size;      /* Taille des donnes de cette ligne */
  unsigned char data[4];
  unsigned int data_value;    /* Valeur Principale */
  unsigned int aux_value;     /* Valeur Secondaire (pour les MVP,MVN) */

  unsigned int operand_address;  /* Adresse Operande sur 24 bit avec le PBR (JMP) ou le DBR (LDA) */

  int address_mode;       /* Mode d'adressage : AM_XXX */

  char label[256];        /* Label */

  char opcode[16];        /* Opcode */

  char operand[64];       /* Operand */

  char comment[32];       /* Comment */

  int wrap_address_bank;  /* L'adresse a t contrainte au mme banc */

  int line_symbol;        /* Cette ligne est lie  un symbol */
  int data_symbol;        /*  */
  int operand_symbol;     /* L'operand de cette ligne est lie  un symbol */

  struct code_line *previous;
  struct code_line *next;
};

#define DIRECTIVE_TYPE_MX    1

struct directive_line
{
  int address;        /* Adresse o insrer la directive */

  int type;           /* DIRECTIVE_TYPE_MX */

  int m;              /* Taille des Registres */
  int x;

  struct directive_line *next;
};

struct code_line *DisassembleDataBlock(int,unsigned char *,int,int,int,int,int,int *,struct directive_line *,int *);
struct code_line *DisassembleOneLine(int,unsigned char *,int,int,int,int,int,WORD,struct code_line *);
void mem_free_codeline(struct code_line *);
void mem_free_directiveline(struct directive_line *);

/************************************************************************/
