/************************************************************************/
/*                                                                      */
/*  Dc_Graphic.c :  Module pour la bibliothque de gestion des Graphic. */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Avril 2018  */
/************************************************************************/

#include <stdio.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Dc_Graphic.h"


/**********************************************************************************************/
/*  GetPictureFromResource() :  Rcupre une resource image sous la forme d'une picture_true. */
/**********************************************************************************************/
struct picture_true *GetPictureFromResource(int resource_id)
{
  HBITMAP hBitmap;
  BITMAP bitmap_bmp;
  BITMAPINFOHEADER bitmap_infoheader;
  HDC hDC;
  int result;
  struct picture_true *current_picture;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On va convertir une image Resource en Pixels **/
  hBitmap = LoadBitmap(param->hInstance,MAKEINTRESOURCE(resource_id));
  if(hBitmap == NULL)
    return(NULL);

  /* Rcupre les informations BMP header (bmWidth et bmHeight) */
  GetObject(hBitmap,sizeof(bitmap_bmp),&bitmap_bmp);

  /* Cration du Bitmap Info Header */
  memset(&bitmap_infoheader,0,sizeof(BITMAPINFOHEADER));
  bitmap_infoheader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_infoheader.biWidth = bitmap_bmp.bmWidth;
  bitmap_infoheader.biHeight = (-1)*(bitmap_bmp.bmHeight);
  bitmap_infoheader.biPlanes = 1;
  bitmap_infoheader.biBitCount = 32;
  bitmap_infoheader.biCompression = BI_RGB;
  bitmap_infoheader.biSizeImage = bitmap_bmp.bmWidth * sizeof(DWORD) * bitmap_bmp.bmHeight;
  bitmap_infoheader.biClrUsed = 0;
  bitmap_infoheader.biClrImportant = 0;

  /* Allocation mmoire de la picture_true */
  current_picture = (struct picture_true *) calloc(1,sizeof(struct picture_true));
  if(current_picture == NULL)
    {
      DeleteObject(hBitmap);
      return(NULL);
    }
  current_picture->width = (int) bitmap_bmp.bmWidth;
  current_picture->height = (int) bitmap_bmp.bmHeight;
  current_picture->data_dwd = (DWORD *) calloc(current_picture->width*current_picture->height,sizeof(DWORD));
  if(current_picture->data_dwd == NULL)
    {
      DeleteObject(hBitmap);
      free(current_picture);
      return(NULL);
    }

  /* Cration d'un Device Context */
  hDC = CreateCompatibleDC(NULL);

  /* Transfre les Pixels */
  result = GetDIBits(hDC,hBitmap,0,(UINT)bitmap_bmp.bmHeight,(void *)current_picture->data_dwd,(BITMAPINFO*)&bitmap_infoheader,DIB_RGB_COLORS);

  /* Libration mmoire */
  DeleteDC(hDC);
  DeleteObject(hBitmap);

  /* Renvoi l'image */
  return(current_picture);
}


/****************************************************************************/
/*  ConvertTrueColorTo256() : Conversion du true color en 256 couleurs GIF. */
/****************************************************************************/
struct picture_256 *ConvertTrueColorTo256(struct picture_true *current_picture_true)
{
  int i, j, k, found;
  int offset, nb_real_color;
  int first, last, x, color_ind;
  int palette_ind[256];
  int palette_found[256];
  unsigned char red, green, blue;
  struct picture_256 *current_picture_256;

  /* Allocation mmoire */
  current_picture_256 = (struct picture_256 *) calloc(1,sizeof(struct picture_256));
  if(current_picture_256 == NULL)
    return(NULL);
  current_picture_256->width = current_picture_true->width;
  current_picture_256->height = current_picture_true->height;
  current_picture_256->nb_color = 0;

  /* Init Palette */
  for(i=0; i<256; i++)
    palette_found[i] = -1;
  memset(&current_picture_256->palette_red[0],0,256*sizeof(int));
  memset(&current_picture_256->palette_green[0],0,256*sizeof(int));
  memset(&current_picture_256->palette_blue[0],0,256*sizeof(int));

  /** Cration de la palette **/
  nb_real_color = 0;
  for(j=0; j<current_picture_true->height; j++)
    for(i=0; i<current_picture_true->width; i++)
      {
        /* Couleur du point */
        if(current_picture_true->data != NULL)
          {
            offset = (i+j*current_picture_true->width)*3;
            red = current_picture_true->data[offset];
            green = current_picture_true->data[offset+1];
            blue = current_picture_true->data[offset+2];
            color_ind = 65536*red + 256*green + blue;
          }
        else
          {
            offset = i+j*current_picture_true->width;
            red =   (unsigned char) ((current_picture_true->data_dwd[offset] & 0x00FF0000) >> 16);
            green = (unsigned char) ((current_picture_true->data_dwd[offset] & 0x0000FF00) >> 8);
            blue =  (unsigned char)  (current_picture_true->data_dwd[offset] & 0x000000FF);
            color_ind = (int) current_picture_true->data_dwd[offset];          
          }

        /** Recherche la couleur dans la palette donne **/
        if(current_picture_true->nb_color > 0)
          {
            /* Vrifie la couleur */
            for(k=0,found=0; k<current_picture_true->nb_color; k++)
              if(current_picture_true->palette_red[k]   == red &&
                 current_picture_true->palette_green[k] == green &&
                 current_picture_true->palette_blue[k]  == blue)
                {
                  palette_found[k] = 1;
                  found = 1;
                  break;
                }

            /* Couleur non trouve => erreur */
            if(found == 0)
              {
                free(current_picture_256);
                return(NULL);
              }
          }
        else 
          {
            /** On insert/recherche la couleur (dichotomie) **/
            if(i == 0 && j == 0)
              {
                palette_ind[0] = color_ind;
                nb_real_color = 1;
              }
            else
              {
                for(first=0,last=nb_real_color; last>=first;)
                  {
                    x = (first+last)/2;
                    if(color_ind < palette_ind[x])
                      last = x - 1;
                    else
                      first = x + 1;

                    if(color_ind == palette_ind[x])
                      break;
                  }

                /* A t'on trouv la couleur ou doit on l'ajouter */
                if(color_ind != palette_ind[x])
                  {
                    /* Gestion des images > 256 couleurs */
                    if(nb_real_color == 256)
                      {
                        free(current_picture_256);
                        return(NULL);
                      }

                    /* Dcalage du tableau */
                    if(x < nb_real_color)
                      if(palette_ind[x] < color_ind)
                        x++;
                    memmove(&palette_ind[x+1],&palette_ind[x],(nb_real_color-x)*sizeof(int));
                    palette_ind[x] = color_ind;
                    nb_real_color++;
                  }
              }
          }
      }
  /** Palette spare RGB **/
  if(current_picture_true->nb_color > 0)
    {
      for(i=0; i<256; i++)
        if(palette_found[i] == 1)
          {
            current_picture_256->palette_red[current_picture_256->nb_color] = current_picture_true->palette_red[i];
            current_picture_256->palette_green[current_picture_256->nb_color] = current_picture_true->palette_green[i];
            current_picture_256->palette_blue[current_picture_256->nb_color] = current_picture_true->palette_blue[i];
            palette_ind[current_picture_256->nb_color] = 65536*current_picture_true->palette_red[i] + 256*current_picture_true->palette_green[i] + current_picture_true->palette_blue[i];
            current_picture_256->nb_color++;
          }
      nb_real_color = current_picture_256->nb_color;
    }
  else
    {
      for(i=0; i<nb_real_color; i++)
        {
          current_picture_256->palette_red[i] =   (palette_ind[i] & 0x00FF0000) >> 16;
          current_picture_256->palette_green[i] = (palette_ind[i] & 0x0000FF00) >> 8;
          current_picture_256->palette_blue[i] =  (palette_ind[i] & 0x000000FF);
        }
      current_picture_256->nb_color = nb_real_color;
    }

  /* Allocation mmoire de l'image en 256 couleurs */
  current_picture_256->data = (unsigned char *) calloc(1,current_picture_true->height*current_picture_true->width);
  if(current_picture_256->data == NULL)
    {
      free(current_picture_256);
      return(NULL);
    }

  /** Points **/
  for(j=0; j<current_picture_true->height; j++)
    for(i=0; i<current_picture_true->width; i++)
      {
        if(current_picture_true->data != NULL)
          {
            offset = (i+j*current_picture_true->width)*3;
            red = current_picture_true->data[offset];
            green = current_picture_true->data[offset+1];
            blue = current_picture_true->data[offset+2];
            color_ind = 65536*red + 256*green + blue;
          }
        else
          {
            offset = (i+j*current_picture_true->width);
            red =   (unsigned char) ((current_picture_true->data_dwd[offset] & 0x00FF0000) >> 16);
            green = (unsigned char) ((current_picture_true->data_dwd[offset] & 0x0000FF00) >> 8);
            blue =  (unsigned char)  (current_picture_true->data_dwd[offset] & 0x000000FF);
            color_ind = current_picture_true->data_dwd[offset];          
          }

        if(current_picture_true->nb_color > 0)
          {
            for(x=0; x<nb_real_color; x++)
              if(color_ind == palette_ind[x])
                break;
          }
        else
          {
            /** Recherche dichotomique de la valeur **/
            for(first=0,last=nb_real_color; last>=first;)
              {
                x = (first+last)/2;
                if(color_ind < palette_ind[x])
                  last = x - 1;
                else
                  first = x + 1;

                if(color_ind == palette_ind[x])
                  break;
              }
          }

        /* Cration du point */
        current_picture_256->data[i+j*current_picture_true->width] = (unsigned char) x;
      }
 
  /** Renvoi l'image 256 **/  
  return(current_picture_256);
}


/**************************************************************************/
/*  mem_free_picture_true() :  Libration mmoire structure picture_true. */
/**************************************************************************/
void mem_free_picture_true(struct picture_true *current_picture)
{
  if(current_picture)
    {
      if(current_picture->data)
        free(current_picture->data);

      if(current_picture->data_dwd)
        free(current_picture->data_dwd);

      free(current_picture);
    }
}


/************************************************************************/
/*  mem_free_picture_256() :  Libration mmoire structure picture_256. */
/************************************************************************/
void mem_free_picture_256(struct picture_256 *current_picture)
{
  if(current_picture)
    {
      if(current_picture->data)
        free(current_picture->data);

      free(current_picture);
    }
}

/************************************************************************/
