/************************************************************************/
/*                                                                      */
/*  Dc_IIgsInternals.h :  Header pour les structures internes du IIgs.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Aot 2023   */
/************************************************************************/

#define GS_FIRST_HANDLE_USED_ADDR    0xE11600   /* E1/1600 */
#define GS_FIRST_HANDLE_PURGED_ADDR  0xE11604   /* E1/1604 */
#define GS_FIRST_HANDLE_FREE_ADDR    0xE11608   /* E1/1608 */

#define GS_NB_CDA_ADDR               0xE01D7C   /* E0/1D7C */
#define GS_CDA_TABLE_ADDR            0xE01D84   /* E0/1D84 */

#define GS_WAP_TABLE_ADDR_PTR        0xE103C8   /* E1/03C8 */
#define GS_SIZE_OF_GLOBAL_DATA       14
#define GS_SIZE_OF_NDA_RECORD        36
#define GS_NDA_NAME_OFFSET           0x15

#define GS_SYSTEM_LOADER_ID          0x7F01     /* 7F01 (x1) -> Table 7201 (x1) => MultiSeg 7101 (xN, MAX = 80) */
#define GS_TABLE_ID_HANDLE_OFFSET    0x78
                                                /* 7004 (xN) : Liste doublement chaine */

#define GS_MAX_ALLOC_SIZE            0x00C00000   /* Taille Max pour une allocation mmoire = 12 MB */
#define GS_MAX_ADDRESS               0x00FFFFFF   /* Adresse Max = 16 MB */

struct gs_handle
{
  /* Handle */
  DWORD handle;

  /* Valeurs */
  DWORD address;
  WORD attributes;
  WORD id;              /* 0001 = RAM Disk */
  DWORD size;
  DWORD prev_handle;
  DWORD next_handle;

  /* Le Handle est t'il valide ? (adresse, size...) */
  int is_valid;

  /* Liste Chaine */
  struct gs_handle *next;
};

struct id_file_7004
{
  struct gs_handle *current_handle;   /* Handle SystemLoader 7004 */

  unsigned char *data;
  WORD id;
  char pathname[1024];
};

struct id_file_7101
{
  WORD file_id;
  struct gs_handle *express_handle;

  unsigned char *express_load_segment_data;
  WORD ref_num;
  char pathname[1024];
};

struct file_id_table
{
  /* Table 7201 */
  struct gs_handle *table_id_handle_7201;
  unsigned char *table_id_data_7201;

  /* Tableau des entres 7004 */
  int nb_file_7004;
  struct id_file_7004 **tab_id_file_7004;

  /* Tableau des entres 7101 */
  int nb_file_7101;
  struct id_file_7101 **tab_id_file_7101;
};

struct cda
{
  DWORD handle;
  DWORD cda_entry_address;

  char *name;
  DWORD entry_addr;
  DWORD exit_addr;
  WORD user_id;
};

struct cda_table
{
  struct gs_handle *cda_table_handle;
  unsigned char *cda_table_data;

  WORD nb_cda;
  struct cda *tab_cda;
};

struct nda
{
  char *name;

  WORD nda_status;
  DWORD open_addr;
  DWORD close_addr;
  DWORD action_addr;
  DWORD init_addr;
  WORD period;
  WORD event_mask;
  DWORD last_serviced;
  DWORD window_ptr;
  DWORD handle;
  WORD user_id;

  struct gs_handle *current_handle;
  unsigned char *data;
};

struct nda_table
{
  WORD nb_nda;
  struct nda *tab_nda;
};

#define NB_QUICKDRAW_VECTOR              0x7F 
#define GS_QUICKDRAW_VECTOR_ADDR_RAM     0xE01E00    /* Vecteurs en RAM */
#define GS_QUICKDRAW_VECTOR_ADDR_ROM01   0xFE0EFD    /* Vecteurs en ROM 01 */
#define GS_QUICKDRAW_VECTOR_ADDR_ROM03   0xFE0EFD    /* Vecteurs en ROM 03 */

struct quickdraw_vector
{
  char name[256];
  WORD number;
  DWORD location;
  DWORD vector_address_rom;
  DWORD vector_address;
};

struct quickdraw_vector_table
{
  int nb_quickdraw_vector;
  struct quickdraw_vector tab_quickdraw_vector[NB_QUICKDRAW_VECTOR];
};

#define NB_INTERRUPT_VECTOR_00           7
#define NB_INTERRUPT_VECTOR_E1          47
#define NB_INTERRUPT_VECTOR_FF          10

#define NB_INTERRUPT_VECTOR              (NB_INTERRUPT_VECTOR_00 + NB_INTERRUPT_VECTOR_E1 + NB_INTERRUPT_VECTOR_FF)
#define GS_INTERRUPT_VECTOR_ADDR_RAM     0xE00000    /* Vecteurs en RAM */
#define GS_INTERRUPT_VECTOR_ADDR_ROM01   0xFE0EFD    /* Vecteurs en ROM 01 */
#define GS_INTERRUPT_VECTOR_ADDR_ROM03   0xFE0EFD    /* Vecteurs en ROM 03 */

#define GS_INTERRUPT_FORMAT_BYTE         1
#define GS_INTERRUPT_FORMAT_ADR          2
#define GS_INTERRUPT_FORMAT_JMP          3
#define GS_INTERRUPT_FORMAT_ADRL         4
#define GS_INTERRUPT_FORMAT_JMPL         5

struct interrupt_vector
{
  DWORD location;                 /* Adresse du Vecteur */

  char name[256];
  char information[256];

  int nb_byte;                   /* Nombre de Byte du Vecteur 1-4 */
  int format;                    /* Formattage des donnes */
  unsigned char data[4];         /* Valeur brute */
  unsigned char data_rom[4];     /* Valeur Init */

  DWORD vector_address;          /* Adresse Numrique : $AA/BBCC */
  char vector_address_txt[16];   /* Adresse Textuelle : JMP $AA/BBCC */

  DWORD vector_address_rom;
  char vector_address_rom_txt[16]; /* Adresse Textuelle : JMP $AA/BBCC */
};

struct interrupt_vector_table
{
  int nb_interrupt_vector;
  struct interrupt_vector tab_interrupt_vector[NB_INTERRUPT_VECTOR];
};

struct file_segment
{
  char *file_path;      /* Chemin complet du fichier */
  char *file_name;      /* Nom du fichier */
  char *segment_name;   /* Nom du Segment */
  int segment_num;      /* Numro du Segment */

  DWORD address;        /* Adresse en mmoire de ce Segment */
  DWORD handle;         /* Adresse en mmoire du Handle */
  DWORD size;
  WORD user_id;
  struct gs_handle *current_handle;

  struct file_segment *next;
};

struct file_segment_table
{
  int nb_segment_table;
  struct file_segment **tab_file_segment;
};

struct init
{
  char *file_name;
  char *file_path;
  char type[256];

  DWORD handle;
  DWORD address;
  DWORD size;
  WORD user_id;
};

struct init_table
{
  WORD nb_init;
  struct init **tab_init;
};

struct driver
{
  char *file_name;
  char *file_path;

  DWORD handle;
  DWORD address;
  DWORD size;
  WORD user_id;
};

struct driver_table
{
  WORD nb_driver;
  struct driver **tab_driver;
};

struct fst
{
  char *file_name;
  char *file_path;

  DWORD handle;
  DWORD address;
  DWORD size;
  WORD user_id;
};

struct fst_table
{
  WORD nb_fst;
  struct fst **tab_fst;
};

struct gs_handle *BuildHandleList(DWORD,int *,int *,char *,struct mmu *);
struct file_id_table *LoadIDTable(struct gs_handle *,char *,struct mmu *);
struct gs_handle *GetMemoryHandle(DWORD,struct mmu *);
int CheckMemoryHandle(struct gs_handle *,char *);
void GetHandleIDType(WORD,char *);
void GetHandleAttributes(WORD,char *);
void GetHandleGroup(DWORD,char *);
void GetHandleFile(struct gs_handle *,char *,char *,int *,int *,char *);
void GetHandleInfo(struct gs_handle *,char *);
void GetHandleListInfo(struct gs_handle *,int *,int *);
unsigned char *LoadHandleData(struct gs_handle *,struct mmu *);
void GetProdosFileName(char *,char *);
struct cda_table *GetCDATable(char *,struct gs_handle *,struct mmu *);
struct nda_table *GetNDATable(char *,struct mmu *);
struct quickdraw_vector_table *GetQuickdrawVectorTable(char *,struct mmu *);
struct interrupt_vector_table *GetInterruptVectorTable(char *,struct mmu *);
struct file_segment_table *GetFileSegmentList(struct gs_handle *,struct mmu *);
struct init_table *GetInitTable(struct s65816 *);
struct driver_table *GetDriverTable(struct s65816 *);
struct fst_table *GetFSTTable(struct s65816 *);
void mem_free_handle_list(struct gs_handle *);
void mem_free_fileidtable(struct file_id_table *);
void mem_free_cdatable(struct cda_table *);
void mem_free_ndatable(struct nda_table *);
void mem_free_quickdrawvectortable(struct quickdraw_vector_table *);
void mem_free_interruptvectortable(struct interrupt_vector_table *);
void mem_free_filesegmenttable(struct file_segment_table *);
void mem_free_filesegment(struct file_segment *);
void mem_free_inittable(struct init_table *);
void mem_free_init(struct init *);
void mem_free_drivertable(struct driver_table *);
void mem_free_driver(struct driver *);
void mem_free_fsttable(struct fst_table *);
void mem_free_fst(struct fst *);

/************************************************************************/
