/************************************************************************/
/*                                                                      */
/* Dc_Library.c : Module pour la bibliothque de gestion de ressources. */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *   Aug 2013   */
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>
#include <setjmp.h>
#include <io.h>

#include <windows.h>

#include "Dc_Shared.h"
#include "s65816_Init.h"
#include "Dc_Library.h"

int compare_breakpoint(const void *,const void *);

/***************************************************************/
/*  my_Memory() :  Fonction de gestion des ressources mmoire. */
/***************************************************************/
void my_Memory(int code, void *data, void *value)
{
  int i, address;
  static struct parameter *param;
  static int nb_breakpoint;  
  static struct breakpoint *first_breakpoint;
  static struct breakpoint *last_breakpoint;
  static struct breakpoint **tab_breakpoint;
  static struct breakpoint local_breakpoint;
  static struct breakpoint *local_breakpoint_ptr;
  struct breakpoint **found_breakpoint_ptr;
  struct breakpoint *current_breakpoint;
  struct breakpoint *next_breakpoint;
  /* Windows */
  static struct window_main *window_main;
  static struct window_shr *window_shr01;

  switch(code)
    {
      case MEMORY_INIT :
        param = NULL;
        nb_breakpoint = 0;
        first_breakpoint = NULL;
        last_breakpoint = NULL;
        tab_breakpoint = NULL;
        local_breakpoint_ptr = &local_breakpoint;
        window_main = NULL;
        window_shr01 = NULL;
        break;

      case MEMORY_FREE :
        my_Memory(MEMORY_FREE_BREAKPOINT,NULL,NULL);
        mem_free_param(param);
        param = NULL;
        break;

      /*************************/
      /*  Structure Parameter  */
      /*************************/
      case MEMORY_SET_PARAM :
        param = (struct parameter *) data;
        break;

      case MEMORY_GET_PARAM :
        *((struct parameter **) data) = param;
        break;

      /****************/
      /*  Breakpoint  */
      /****************/
      case MEMORY_ADD_BREAKPOINT :
        current_breakpoint = (struct breakpoint *) data;
        
        /* Attache la structure */
        if(first_breakpoint == NULL)
          first_breakpoint = current_breakpoint;
        else
          {
            current_breakpoint->previous = last_breakpoint;
            last_breakpoint->next = current_breakpoint;
          }
        last_breakpoint = current_breakpoint;
        nb_breakpoint++;
        break;

      case MEMORY_GET_BREAKPOINT_NB :
        *((int *) data) = nb_breakpoint;
        break;

      case MEMORY_GET_BREAKPOINT :
        /* Init */
        *((char **)value) = NULL;

        /* Vrification d'un dpassement */
        if(*((int *)data) <= 0 || *((int *)data) > nb_breakpoint)
          return;

        /* Accs direct par le tableau */
        if(tab_breakpoint)
          {
            *((struct breakpoint **)value) = tab_breakpoint[(*((int *)data))-1];
            return;
          }

        /* Localise la strucure */
        for(i=0,current_breakpoint=first_breakpoint; i<(*((int *)data))-1; i++)
          current_breakpoint = current_breakpoint->next;
        *((struct breakpoint **)value) = current_breakpoint;
        break;

      case MEMORY_SORT_BREAKPOINT :
        if(nb_breakpoint == 0)
          return;

        /* Allocation mmoire */
        if(tab_breakpoint)
          free(tab_breakpoint);
        tab_breakpoint = (struct breakpoint **) calloc(nb_breakpoint,sizeof(struct breakpoint *));
        if(tab_breakpoint == NULL)
          {
            printf("    Error: Impossible to allocate memory for tab_breakpoint table.\n");
            return;
          }

        /* Place les item */
        for(i=0,current_breakpoint=first_breakpoint; current_breakpoint; current_breakpoint=current_breakpoint->next,i++)
          tab_breakpoint[i] = current_breakpoint;

        /* Tri */
        qsort(tab_breakpoint,nb_breakpoint,sizeof(struct breakpoint *),compare_breakpoint);

        /* Replace les liens NEXT */
        for(i=0; i<nb_breakpoint; i++)
          {
            if(i == nb_breakpoint-1)
              tab_breakpoint[i]->next = NULL;
            else
              tab_breakpoint[i]->next = tab_breakpoint[i+1];
          }
        /* Replace les liens PREVIOUS */
        for(i=0; i<nb_breakpoint; i++)
          {
            if(i == 0)
              tab_breakpoint[i]->previous = NULL;
            else
              tab_breakpoint[i]->previous = tab_breakpoint[i-1];
          }
        /* First et Last */       
        first_breakpoint = tab_breakpoint[0];
        last_breakpoint = tab_breakpoint[nb_breakpoint-1];
        break;

      case MEMORY_SEARCH_BREAKPOINT :
        /* Init */
        *((struct breakpoint **)value) = NULL;
        address = *((int *) data);

        /** Recherche par le tableau **/
        if(tab_breakpoint != NULL)
          {
            local_breakpoint.address = address;
            found_breakpoint_ptr = (struct breakpoint **) bsearch(&local_breakpoint_ptr,(void *)tab_breakpoint,nb_breakpoint,sizeof(struct breakpoint *),compare_breakpoint);
            if(found_breakpoint_ptr != NULL)
              {
                /* On va remonter au premier de la liste */
                current_breakpoint = *found_breakpoint_ptr;
                while(current_breakpoint->previous != NULL)
                  if(current_breakpoint->previous->address == current_breakpoint->address)
                    current_breakpoint = current_breakpoint->previous;
                  else
                    break;
                    
                /* Renvoie le 1er Breakpoint */  
                *((struct breakpoint **)value) = current_breakpoint;
              }
          }
        else
          {
            /* Recherche via la liste chaine */
            for(current_breakpoint=first_breakpoint; current_breakpoint; current_breakpoint=current_breakpoint->next)
              if(current_breakpoint->address == address)
                {
                  *((struct breakpoint **)value) = current_breakpoint;
                  break;
                }
          }
        break;

      case MEMORY_FREE_BREAKPOINT :
        for(current_breakpoint = first_breakpoint; current_breakpoint; )
          {
            next_breakpoint = current_breakpoint->next;
            mem_free_breakpoint(current_breakpoint);
            current_breakpoint = next_breakpoint;
          }
        nb_breakpoint = 0;
        first_breakpoint = NULL;
        last_breakpoint = NULL;
        if(tab_breakpoint)
          free(tab_breakpoint);
        tab_breakpoint = NULL;
        break;

      /************************/
      /*  Gestion des Window  */
      /************************/
      case MEMORY_SET_WINDOW_MAIN :
        window_main = (struct window_main *) data;
        break;

      case MEMORY_GET_WINDOW_MAIN :
        *((struct window_main **) data) = window_main;
        break;

      case MEMORY_SET_WINDOW_SHR01 :
        window_shr01 = (struct window_shr *) data;
        break;

      case MEMORY_GET_WINDOW_SHR01 :
        *((struct window_shr **) data) = window_shr01;
        break;

      default :
        break;
    }
}


/***********************************************************************/
/*  compare_breakpoint() : Fonction de comparaison pour le Quick Sort. */
/***********************************************************************/
int compare_breakpoint(const void *data_1, const void *data_2)
{
  struct breakpoint *breakpoint_1;
  struct breakpoint *breakpoint_2;

  /* Rcupration des paramtres */
  breakpoint_1 = *((struct breakpoint **) data_1);
  breakpoint_2 = *((struct breakpoint **) data_2);

  /* Comparaison les adresses */
  if(breakpoint_1->address < breakpoint_2->address)
    return(-1);
  else if(breakpoint_1->address == breakpoint_2->address)
    return(0);
  else
    return(1);
}

/************************************************************************/
