/**************************************************************************/
/*                                                                        */
/*  Dc_MMU.h :  Header pour la bibliothque de gestion de la Mmoire.     */
/*                                                                        */
/**************************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe Software   *  Avril 2018  */
/**************************************************************************/

#define RAM_BANK_SIZE        (64*1024)    /* 64 KB */
#define DOC_RAM_SIZE         (64*1024)    /* 64 KB */
#define DOC_REGISTER_SIZE     (32*7+3)    /* 227 Bytes */
#define BATTERY_RAM_SIZE           256    /* 256 Bytes */

#define ROM_APPLEIIGS_01             1    /* Rom 01 */
#define ROM_APPLEIIGS_03             3    /* Rom 03 */

struct mmu
{
  /* Rom 01 / 03 */
  int rom_version;

  /* RAM IIgs 00 01 */
  unsigned char ram_00[RAM_BANK_SIZE];
  unsigned char ram_01[RAM_BANK_SIZE];
  
  /* RAM IIgs Extended */
  int nb_extended_bank;               /* Nombre de bank */
  unsigned char *ram_extended;
  
  /* RAM IIgs E0 E1 */
  unsigned char ram_e0[RAM_BANK_SIZE];
  unsigned char ram_e1[RAM_BANK_SIZE];
  
  /* ROM IIgs : FC FD FE FF */
  unsigned char rom_fc[RAM_BANK_SIZE];
  unsigned char rom_fd[RAM_BANK_SIZE];
  unsigned char rom_fe[RAM_BANK_SIZE];
  unsigned char rom_ff[RAM_BANK_SIZE];
  
  /* RAM SON */
  unsigned char ram_sound[DOC_RAM_SIZE];
  
  /* DOC Register */
  unsigned char doc_register[DOC_REGISTER_SIZE];  /* 7*32 + 3 (E0 / E1 / E2) */

  /* Battery RAM */
  unsigned char battery_ram[BATTERY_RAM_SIZE];

  /** Zone Mmoire RWX pour MemoryMap **/
  unsigned char map_ram_00[RAM_BANK_SIZE];
  unsigned char map_ram_01[RAM_BANK_SIZE];
  unsigned char *map_ram_extended;
  unsigned char map_ram_e0[RAM_BANK_SIZE];
  unsigned char map_ram_e1[RAM_BANK_SIZE];
  unsigned char map_rom_fc[RAM_BANK_SIZE];
  unsigned char map_rom_fd[RAM_BANK_SIZE];
  unsigned char map_rom_fe[RAM_BANK_SIZE];
  unsigned char map_rom_ff[RAM_BANK_SIZE];
  unsigned char map_ram_sound[DOC_RAM_SIZE];
};

BYTE ReadMemoryByte(DWORD,struct mmu *);
void ReadMemoryBlock(DWORD,int,unsigned char *,struct mmu *);
void WriteMemoryByte(DWORD,BYTE,struct mmu *);

BYTE ReadDOCRAMByte(DWORD,struct mmu *);
void ReadDOCRAMBlock(DWORD,int,unsigned char *,struct mmu *);
void WriteDOCRAMByte(DWORD,BYTE,struct mmu *);

BYTE ReadDOCRegisterByte(DWORD,struct mmu *);
void ReadDOCRegisterBlock(DWORD,int,unsigned char *,struct mmu *);
void WriteDOCRegisterByte(DWORD,BYTE,struct mmu *);

BYTE ReadBatteryRAMByte(DWORD,struct mmu *);
void ReadBatteryRAMBlock(DWORD,int,unsigned char *,struct mmu *);
void WriteBatteryRAMByte(DWORD,BYTE,struct mmu *);

unsigned char *GetRamRomPointer(DWORD,int,struct mmu *);
unsigned char *GetMapRamRomRWXPointer(DWORD,int,struct mmu *);

/**************************************************************************/
