/************************************************************************/
/*                                                                      */
/*  Dc_Memory.c :  Module pour la bibliothque de gestion Mmoire.      */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Avril 2018  */
/************************************************************************/

#include <stdio.h>
#include <sys/timeb.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <Commdlg.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Gif.h"
#include "Dc_MMU.h"
#include "Dc_Name.h"
#include "Dc_Graphic.h"
#include "Dc_Request.h"
#include "Dc_Snapshot.h"
#include "Cyrene_Engine.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_About.h"
#include "Window_Stack.h"
#include "Window_DirectPage.h"
#include "Window_Softswitch.h"
#include "Window_Processor.h"
#include "Window_MemoryArea.h"
#include "Window_RamSound.h"
#include "Window_TextPage40.h"
#include "Window_TextPage80.h"
#include "Window_GraphicGR.h"
#include "Window_GraphicHGR.h"
#include "Window_GraphicDHGRMono.h"
#include "Window_GraphicDHGRColor.h"
#include "Window_GraphicDGR.h"
#include "Window_GraphicHGRText40.h"
#include "Window_GraphicHGRText80.h"
#include "Window_GraphicGRText40.h"
#include "Window_GraphicGRText80.h"
#include "Window_GraphicDGRText80.h"
#include "Window_GraphicDHGRMonoText80.h"
#include "Window_GraphicDHGRColorText80.h"
#include "Window_GraphicSHR.h"
#include "Window_Disassembler.h"
#include "Window_MemoryMap.h"
#include "Window_DOCRegister.h"
#include "Window_Search.h"
#include "Window_Replace.h"
#include "Window_LoadFile.h"
#include "Window_SaveMemory.h"
#include "Window_Compare.h"
#include "Window_Modify.h"
#include "Window_Move.h"
#include "Window_GraphicScreen.h"
#include "Window_Attach.h"
#include "Window_IPC.h"
#include "Window_BatteryRam.h"
#include "Window_Internals.h"
#include "Window_Symbol.h"
#include "Window_Breakpoint.h"
#include "Window_Statistic.h"
#include "Window_Profile.h"
#include "Window_SymbolPickup.h"
#include "Window_Preference.h"
#include "Window_ProgressBar.h"
#include "Dc_Memory.h"

static int compare_filepath(const void *,const void *);
static int compare_filepath_by_size(const void *,const void *);

/****************************************************************/
/*  my_Memory() :  Fonction de gestion des ressources mmoires. */
/****************************************************************/
void my_Memory(int code, void *data, void *value)
{
  char *path;
  DWORD thread_id;
  HWND window_handle;
  int nb_window_handle;
  HWND tab_window_handle[256];
  int i, j, file_size, request_number, index, operation_number, nb_extra_operation;
  static struct parameter *param;
  static struct snapshot *current_snapshot;
  static struct s65816 *current_s65816;
  static int nb_filepath;                                 /* Chemin des fichiers */
  static struct file_path *first_filepath;
  static struct file_path *last_filepath;
  static struct file_path **file_path_tab;  
  struct file_path *current_filepath;
  struct file_path *next_filepath;  
  static int nb_request;                                  /* Liste des Request */
  static struct request *first_request;
  static struct request *last_request;
  struct request *current_request;
  struct request *next_request;
  static int nb_operation;                                /* Nombre d'Opration */
  static int nb_operation_set;                            /* Liste des Operation Set */
  static struct operation_set *first_operation_set;
  static struct operation_set *last_operation_set;
  struct operation_set *current_operation_set;
  struct operation_set *next_operation_set;
  struct operation *current_operation;
  struct operation tab_operation[256];
  static struct window_main *window_main;                 /* Fentre Main */
  static struct window_about *window_about;
  static struct window_stack *window_stack;
  static struct window_directpage *window_directpage;
  static struct window_softswitch *window_softswitch;
  static struct window_processor *window_processor;
  static struct window_memoryarea *first_window_memoryarea;    /* Liste des Memory Area */
  static struct window_memoryarea *last_window_memoryarea;
  struct window_memoryarea *current_window_memoryarea;
  struct window_memoryarea *next_window_memoryarea;
  struct window_memoryarea *delete_window_memoryarea;
  static struct window_ramsound *first_window_ramsound;        /* Liste des Ram Sound */
  static struct window_ramsound *last_window_ramsound;
  struct window_ramsound *current_window_ramsound;
  struct window_ramsound *next_window_ramsound;
  struct window_ramsound *delete_window_ramsound;
  static struct window_textpage40 *first_window_textpage40;    /* Liste des Text Page 40 */
  static struct window_textpage40 *last_window_textpage40;
  struct window_textpage40 *current_window_textpage40;
  struct window_textpage40 *next_window_textpage40;
  struct window_textpage40 *delete_window_textpage40;
  static struct window_textpage80 *first_window_textpage80;    /* Liste des Text Page 80 */
  static struct window_textpage80 *last_window_textpage80;
  struct window_textpage80 *current_window_textpage80;
  struct window_textpage80 *next_window_textpage80;
  struct window_textpage80 *delete_window_textpage80;
  static struct window_graphicgr *first_window_graphicgr;      /* Liste des Graphic GR */
  static struct window_graphicgr *last_window_graphicgr;
  struct window_graphicgr *current_window_graphicgr;
  struct window_graphicgr *next_window_graphicgr;
  struct window_graphicgr *delete_window_graphicgr;
  static struct window_graphichgr *first_window_graphichgr;    /* Liste des Graphic HGR */
  static struct window_graphichgr *last_window_graphichgr;
  struct window_graphichgr *current_window_graphichgr;
  struct window_graphichgr *next_window_graphichgr;
  struct window_graphichgr *delete_window_graphichgr;
  static struct window_graphicdhgrmono *first_window_graphicdhgrmono;    /* Liste des Graphic DHGR Mono */
  static struct window_graphicdhgrmono *last_window_graphicdhgrmono;
  struct window_graphicdhgrmono *current_window_graphicdhgrmono;
  struct window_graphicdhgrmono *next_window_graphicdhgrmono;
  struct window_graphicdhgrmono *delete_window_graphicdhgrmono;
  static struct window_graphicdhgrcolor *first_window_graphicdhgrcolor;    /* Liste des Graphic DHGR Color */
  static struct window_graphicdhgrcolor *last_window_graphicdhgrcolor;
  struct window_graphicdhgrcolor *current_window_graphicdhgrcolor;
  struct window_graphicdhgrcolor *next_window_graphicdhgrcolor;
  struct window_graphicdhgrcolor *delete_window_graphicdhgrcolor;            
  static struct window_graphicdgr *first_window_graphicdgr;          /* Liste des Graphic DGR */
  static struct window_graphicdgr *last_window_graphicdgr;
  struct window_graphicdgr *current_window_graphicdgr;
  struct window_graphicdgr *next_window_graphicdgr;
  struct window_graphicdgr *delete_window_graphicdgr;
  static struct window_graphichgrtext40 *first_window_graphichgrtext40;  /* Liste des GraphicHGR Text40 */
  static struct window_graphichgrtext40 *last_window_graphichgrtext40;
  struct window_graphichgrtext40 *current_window_graphichgrtext40;
  struct window_graphichgrtext40 *next_window_graphichgrtext40;
  struct window_graphichgrtext40 *delete_window_graphichgrtext40;            
  static struct window_graphichgrtext80 *first_window_graphichgrtext80;  /* Liste des GraphicHGR Text80 */
  static struct window_graphichgrtext80 *last_window_graphichgrtext80;
  struct window_graphichgrtext80 *current_window_graphichgrtext80;
  struct window_graphichgrtext80 *next_window_graphichgrtext80;
  struct window_graphichgrtext80 *delete_window_graphichgrtext80;
  static struct window_graphicgrtext40 *first_window_graphicgrtext40;    /* Liste des GraphicGR Text40 */
  static struct window_graphicgrtext40 *last_window_graphicgrtext40;
  struct window_graphicgrtext40 *current_window_graphicgrtext40;
  struct window_graphicgrtext40 *next_window_graphicgrtext40;
  struct window_graphicgrtext40 *delete_window_graphicgrtext40;
  static struct window_graphicgrtext80 *first_window_graphicgrtext80;    /* Liste des GraphicGR Text80 */
  static struct window_graphicgrtext80 *last_window_graphicgrtext80;
  struct window_graphicgrtext80 *current_window_graphicgrtext80;
  struct window_graphicgrtext80 *next_window_graphicgrtext80;
  struct window_graphicgrtext80 *delete_window_graphicgrtext80;
  static struct window_graphicdgrtext80 *first_window_graphicdgrtext80;    /* Liste des GraphicDGR Text80 */
  static struct window_graphicdgrtext80 *last_window_graphicdgrtext80;
  struct window_graphicdgrtext80 *current_window_graphicdgrtext80;
  struct window_graphicdgrtext80 *next_window_graphicdgrtext80;
  struct window_graphicdgrtext80 *delete_window_graphicdgrtext80;
  static struct window_graphicdhgrmonotext80 *first_window_graphicdhgrmonotext80;    /* Liste des GraphicDHGRMono Text80 */
  static struct window_graphicdhgrmonotext80 *last_window_graphicdhgrmonotext80;
  struct window_graphicdhgrmonotext80 *current_window_graphicdhgrmonotext80;
  struct window_graphicdhgrmonotext80 *next_window_graphicdhgrmonotext80;
  struct window_graphicdhgrmonotext80 *delete_window_graphicdhgrmonotext80;
  static struct window_graphicdhgrcolortext80 *first_window_graphicdhgrcolortext80;  /* Liste des GraphicDHGRColor Text80 */
  static struct window_graphicdhgrcolortext80 *last_window_graphicdhgrcolortext80;
  struct window_graphicdhgrcolortext80 *current_window_graphicdhgrcolortext80;
  struct window_graphicdhgrcolortext80 *next_window_graphicdhgrcolortext80;
  struct window_graphicdhgrcolortext80 *delete_window_graphicdhgrcolortext80;
  static struct window_graphicshr *first_window_graphicshr;               /* Liste des GraphicSHR */
  static struct window_graphicshr *last_window_graphicshr;
  struct window_graphicshr *current_window_graphicshr;
  struct window_graphicshr *next_window_graphicshr;
  struct window_graphicshr *delete_window_graphicshr;
  static struct window_disassembler *first_window_disassembler;           /* Liste des Disassembler */
  static struct window_disassembler *last_window_disassembler;
  struct window_disassembler *current_window_disassembler;
  struct window_disassembler *next_window_disassembler;
  struct window_disassembler *delete_window_disassembler;
  static struct window_memorymap *window_memorymap;                       /* Fentre Memory Map */
  static struct window_docregister *window_docregister;                   /* Fentre DOC Register */
  static struct window_search *window_search;                             /* Fentre Search */
  static struct window_replace *window_replace;                           /* Fentre Replace */
  static struct window_loadfile *first_window_loadfile;                   /* Liste des Load File */
  static struct window_loadfile *last_window_loadfile;
  struct window_loadfile *current_window_loadfile;
  struct window_loadfile *next_window_loadfile;
  struct window_loadfile *delete_window_loadfile;  
  static struct window_savememory *first_window_savememory;               /* Liste des Save Memory */
  static struct window_savememory *last_window_savememory;
  struct window_savememory *current_window_savememory;
  struct window_savememory *next_window_savememory;
  struct window_savememory *delete_window_savememory;  
  static struct window_compare *first_window_compare;                     /* Liste des Compare */
  static struct window_compare *last_window_compare;
  struct window_compare *current_window_compare;
  struct window_compare *next_window_compare;
  struct window_compare *delete_window_compare;  
  static struct window_modify *first_window_modify;                       /* Liste des Modify */
  static struct window_modify *last_window_modify;
  struct window_modify *current_window_modify;
  struct window_modify *next_window_modify;
  struct window_modify *delete_window_modify;
  static struct window_move *first_window_move;                           /* Liste des Move */
  static struct window_move *last_window_move;
  struct window_move *current_window_move;
  struct window_move *next_window_move;
  struct window_move *delete_window_move;
  static struct window_quickwatch *window_quickwatch;                     /* Fentre Quickwatch */
  static struct window_graphicscreen *window_graphicscreen;               /* Fentre GraphicScreen */
  static struct window_attach *window_attach;                             /* Fentre Attach */
  static struct window_ipc *window_ipc;                                   /* Fentre IPC */
  static struct window_batteryram *window_batteryram;                     /* Fentre Battery Ram */
  static struct window_internals *window_internals;                       /* Fentre Apple IIgs Internals */
  static struct window_symbol *window_symbol;                             /* Fentre Symbol */
  static struct window_breakpoint *window_breakpoint;                     /* Fentre Breakpoint */
  static struct window_statistic *window_statistic;                       /* Fentre Statistic */
  static struct window_profile *window_profile;                           /* Fentre Profile */
  static struct window_symbol_pickup *window_symbol_pickup;               /* Fentre SymbolPickup */
  static struct window_preference *window_preference;                     /* Fentre Preference */
  static struct window_progressbar *window_progressbar;                   /* Fentre ProgressBar */

  switch(code)
    {
      /**************************************/
      /*  Gestions des ressources mmoires  */
      /**************************************/
      case MEMORY_INIT_ALL :
        param = NULL;
        current_snapshot = NULL;
        current_s65816 = NULL;
        nb_filepath = 0;
        first_filepath = NULL;
        last_filepath = NULL;
        file_path_tab = NULL;        
        nb_request = 0;
        first_request = NULL;
        last_request = NULL;
        nb_operation = 0;
        nb_operation_set = 0;
        first_operation_set = NULL;
        last_operation_set = NULL;
        window_main = NULL;
        window_about = NULL;
        window_stack = NULL;
        window_directpage = NULL;
        window_softswitch = NULL;
        window_processor = NULL;
        first_window_memoryarea = NULL;
        last_window_memoryarea = NULL;
        first_window_ramsound = NULL;
        last_window_ramsound = NULL;
        first_window_textpage40 = NULL;
        last_window_textpage40 = NULL;
        first_window_textpage80 = NULL;
        last_window_textpage80 = NULL;
        first_window_graphicgr = NULL;
        last_window_graphicgr = NULL;
        first_window_graphichgr = NULL;
        last_window_graphichgr = NULL;
        first_window_graphicdhgrmono = NULL;
        last_window_graphicdhgrmono = NULL;
        first_window_graphicdhgrcolor = NULL;
        last_window_graphicdhgrcolor = NULL;
        first_window_graphicdgr = NULL;
        last_window_graphicdgr = NULL;
        first_window_graphichgrtext40 = NULL;
        last_window_graphichgrtext40 = NULL;
        first_window_graphichgrtext80 = NULL;
        last_window_graphichgrtext80 = NULL;
        first_window_graphicgrtext40 = NULL;
        last_window_graphicgrtext40 = NULL;
        first_window_graphicgrtext80 = NULL;
        last_window_graphicgrtext80 = NULL;
        first_window_graphicdgrtext80 = NULL;
        last_window_graphicdgrtext80 = NULL;
        first_window_graphicdhgrmonotext80 = NULL;
        last_window_graphicdhgrmonotext80 = NULL;
        first_window_graphicdhgrcolortext80 = NULL;
        last_window_graphicdhgrcolortext80 = NULL;
        first_window_graphicshr = NULL;
        last_window_graphicshr = NULL;
        first_window_disassembler = NULL;
        last_window_disassembler = NULL;
        window_memorymap = NULL;
        window_docregister = NULL;
        window_search = NULL;
        window_replace = NULL;
        first_window_loadfile = NULL;
        last_window_loadfile = NULL;
        first_window_savememory = NULL;
        last_window_savememory = NULL;
        first_window_compare = NULL;
        last_window_compare = NULL;
        window_quickwatch = NULL;
        first_window_modify = NULL;
        last_window_modify = NULL;
        first_window_move = NULL;
        last_window_move = NULL;
        window_graphicscreen = NULL;
        window_attach = NULL;
        window_ipc = NULL;
        window_batteryram = NULL;
        window_internals = NULL;
        window_symbol = NULL;
        window_breakpoint = NULL;
        window_statistic = NULL;
        window_profile = NULL;
        window_symbol_pickup = NULL;
        window_preference = NULL;
        window_progressbar = NULL;
        break;

      case MEMORY_CLOSE_WINDOW_ALL :
        /** Ferme toutes les fentres ouvertes **/
        my_Memory(MEMORY_CLOSE_WINDOW_ABOUT,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_STACK,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_DIRECTPAGE,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_SOFTSWITCH,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_PROCESSOR,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_MEMORYAREA,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_RAMSOUND,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_TEXTPAGE40,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_TEXTPAGE80,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICGR,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICHGR,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICDHGRMONO,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICDHGRCOLOR,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICDGR,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICHGRTEXT40,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICHGRTEXT80,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICGRTEXT40,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICGRTEXT80,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICDGRTEXT80,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICDHGRMONOTEXT80,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICDHGRCOLORTEXT80,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICSHR,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_DISASSEMBLER,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_MEMORYMAP,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_DOCREGISTER,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_SEARCH,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_REPLACE,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_LOADFILE,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_SAVEMEMORY,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_COMPARE,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_QUICKWATCH,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_MODIFY,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_MOVE,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_GRAPHICSCREEN,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_ATTACH,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_BATTERYRAM,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_INTERNALS,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_SYMBOL,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_BREAKPOINT,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_STATISTIC,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_PROFILE,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_SYMBOL_PICKUP,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_PREFERENCE,NULL,NULL);
        my_Memory(MEMORY_CLOSE_WINDOW_PROGRESSBAR,NULL,NULL);
        break;

      case MEMORY_FREE_ALL :
        my_Memory(MEMORY_FREE_WINDOW_ABOUT,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_STACK,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_DIRECTPAGE,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_SOFTSWITCH,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_PROCESSOR,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_MEMORYAREA,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_RAMSOUND,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_TEXTPAGE40,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_TEXTPAGE80,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICGR,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICHGR,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICDHGRMONO,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICDHGRCOLOR,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICDGR,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICHGRTEXT40,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICHGRTEXT80,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICGRTEXT40,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICGRTEXT80,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICDGRTEXT80,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICDHGRMONOTEXT80,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICDHGRCOLORTEXT80,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICSHR,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_DISASSEMBLER,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_MEMORYMAP,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_DOCREGISTER,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_SEARCH,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_REPLACE,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_LOADFILE,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_SAVEMEMORY,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_COMPARE,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_QUICKWATCH,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_MODIFY,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_MOVE,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICSCREEN,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_MAIN,NULL,NULL);
        my_Memory(MEMORY_FREE_SNAPSHOT,NULL,NULL);
        my_Memory(MEMORY_FREE_S65816,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_ATTACH,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_IPC,NULL,NULL);
        my_Memory(MEMORY_FREE_FILE,NULL,NULL);
        my_Memory(MEMORY_FREE_REQUEST,NULL,NULL);
        my_Memory(MEMORY_FREE_OPERATION,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_BATTERYRAM,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_INTERNALS,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_SYMBOL,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_BREAKPOINT,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_STATISTIC,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_PROFILE,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_SYMBOL_PICKUP,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_PREFERENCE,NULL,NULL);
        my_Memory(MEMORY_FREE_WINDOW_PROGRESSBAR,NULL,NULL);
        my_Memory(MEMORY_FREE_PARAM,NULL,NULL);
        break;

      /*********************/
      /*  Structure param  */
      /*********************/
      case MEMORY_SET_PARAM :
        param = (struct parameter *) data;
        break;

      case MEMORY_GET_PARAM :
        *((struct parameter **) data) = param;
        break;

      case MEMORY_FREE_PARAM :
        mem_free_param(param);
        param = NULL;
        break;


      /************************/
      /*  Structure Snapshot  */
      /************************/
      case MEMORY_SET_SNAPSHOT :
        if(current_snapshot != NULL)
          mem_free_snapshot(current_snapshot);
        current_snapshot = (struct snapshot *) data;
        break;

      case MEMORY_GET_SNAPSHOT :
        *((struct snapshot **) data) = current_snapshot;
        break;

      case MEMORY_FREE_SNAPSHOT :
        mem_free_snapshot(current_snapshot);
        current_snapshot = NULL;
        break;

      /**********************/
      /*  Structure s65816  */
      /**********************/
      case MEMORY_SET_S65816 :
        current_s65816 = (struct s65816 *) data;
        break;

      case MEMORY_GET_S65816 :
        *((struct s65816 **) data) = current_s65816;
        break;

      case MEMORY_GET_S65816_CPU :
        if(current_s65816 != NULL)
          *((struct cpu **) data) = current_s65816->current_cpu;
        else
          *((struct cpu **) data) = NULL;
        break;
        
      case MEMORY_GET_S65816_MMU :
        if(current_s65816 != NULL)
          *((struct mmu **) data) = current_s65816->current_mmu;
        else
          *((struct mmu **) data) = NULL;
        break;

      case MEMORY_FREE_S65816 :
        mem_free_s65816(current_s65816);
        current_s65816 = NULL;
        break;

      /*******************************************/
      /*  Liste chaine des structure file_path  */
      /*******************************************/
      case MEMORY_INIT_FILE :
        nb_filepath = 0;
        first_filepath = NULL;
        last_filepath = NULL;
        if(file_path_tab)
          free(file_path_tab);
        file_path_tab = NULL;      
        break;
        
      case MEMORY_ADD_FILE :
        path = (char *) data;
        file_size = *((int *) value);

        /* Allocation mmoire */
        current_filepath = (struct file_path *) calloc(1,sizeof(struct file_path));
        if(current_filepath == NULL)
          return;
        current_filepath->size = file_size;
        current_filepath->path = strdup(path);
        if(current_filepath->path == NULL)
          {
            free(current_filepath);
            return;
          }

        /* Ajoute  la fin de la liste */
        if(first_filepath == NULL)
          first_filepath = current_filepath;
        else
          last_filepath->next = current_filepath;
        last_filepath = current_filepath;
        nb_filepath++;
        break;

      case MEMORY_GET_FILE_NB :
        *((int *)data) = nb_filepath;
        break;

      case MEMORY_GET_FILE :
        index = *((int *) data);
        *((struct file_path **) value) = NULL;
        if(index <= 0 || index > nb_filepath)
          return;

        /* Recherche rapide */
        if(file_path_tab != NULL)
          {
            *((struct file_path **) value) = file_path_tab[index-1];
            return;
          }

        /* Recherche le index-nth entry */
        for(i=1, current_filepath = first_filepath; i<index; i++)
          current_filepath = current_filepath->next;
        *((struct file_path **) value) = current_filepath;
        break;

      case MEMORY_SORT_FILE :
        if(nb_filepath == 0)
          return;
          
        /* Allocation */
        if(file_path_tab != NULL)
          {
            free(file_path_tab);
            file_path_tab = NULL;
          }
        file_path_tab = (struct file_path **) calloc(1+nb_filepath,sizeof(struct file_path *));
        if(file_path_tab == NULL)
          return;

        /* Remplissage du tableau */
        for(i=0,current_filepath = first_filepath; current_filepath; current_filepath=current_filepath->next,i++)
          file_path_tab[i] = current_filepath;

        /* Tri Qsort du tableau */
        qsort((void *)file_path_tab,(size_t)nb_filepath,sizeof(struct file_path *),compare_filepath);
 
        /** On refait le cablage des pointeurs next des structures **/
        first_filepath = file_path_tab[0];
        last_filepath = file_path_tab[nb_filepath-1];
        file_path_tab[nb_filepath-1]->next = NULL;
        if(nb_filepath > 1)
          for(i=0; i<nb_filepath-1; i++)
            file_path_tab[i]->next = file_path_tab[i+1];
        break;

      case MEMORY_SORT_FILE_BY_SIZE :
        if(nb_filepath == 0)
          return;
          
        /* Allocation */
        if(file_path_tab != NULL)
          {
            free(file_path_tab);
            file_path_tab = NULL;
          }
        file_path_tab = (struct file_path **) calloc(1+nb_filepath,sizeof(struct file_path *));
        if(file_path_tab == NULL)
          return;

        /* Remplissage du tableau */
        for(i=0,current_filepath = first_filepath; current_filepath; current_filepath=current_filepath->next,i++)
          file_path_tab[i] = current_filepath;

        /* Tri Qsort du tableau */
        qsort((void *)file_path_tab,(size_t)nb_filepath,sizeof(struct file_path *),compare_filepath_by_size);
 
        /** On refait le cablage des pointeurs next des structures **/
        first_filepath = file_path_tab[0];
        last_filepath = file_path_tab[nb_filepath-1];
        file_path_tab[nb_filepath-1]->next = NULL;
        if(nb_filepath > 1)
          for(i=0; i<nb_filepath-1; i++)
            file_path_tab[i]->next = file_path_tab[i+1];      
        break;

      case MEMORY_FREE_FILE :
        for(current_filepath = first_filepath; current_filepath; )
          {
            next_filepath = current_filepath->next;
            mem_free_filepath(current_filepath);
            current_filepath = next_filepath;
          }
        nb_filepath = 0;
        first_filepath = NULL;
        last_filepath = NULL;
        if(file_path_tab)
          free(file_path_tab);
        file_path_tab = NULL;
        break;


      /*************************/
      /*  Gestion des Request  */
      /*************************/
      case MEMORY_ADD_REQUEST :
        /* On ne traite pas les structures vides */
        if((struct request *)data == NULL)
          break;

        /* Rcupre la structure */
        current_request = (struct request *) data;

        /* Stocke la structure */
        if(nb_request == 0)
          first_request = current_request;
        else
          last_request->next = current_request;
        last_request = current_request;
        nb_request++;
        break;

      case MEMORY_GET_REQUEST :
        /* Vrification d'un dpassement */
        *((struct request **)value) = NULL;
        request_number = *((int *) data);
        if(request_number <= 0 || request_number > nb_request)
          return;

        /* Parcours de la liste chaine */
        for(i=0,current_request=first_request; i<request_number-1; i++)
          current_request = current_request->next;
        *((struct request **)value) = current_request;
        break;

      case MEMORY_GET_REQUEST_NB :
        *((int *) data) = nb_request;
        break;

      case MEMORY_EXTRACT_REQUEST :
        *((struct request **)data) = first_request;
        if(first_request == NULL)
          return;
        /* Une seule Request */
        if(first_request == last_request)
          {
            first_request = NULL;
            last_request = NULL;
            nb_request = 0;
          }
        else
          {
            first_request = first_request->next;
            nb_request--;
          }
        break;

      case MEMORY_FREE_REQUEST :
        /* On efface toutes les structures enregistres (Liste chane ) */
        for(current_request = first_request; current_request!=NULL;)
          {
            next_request = current_request->next;
            mem_free_request(current_request);
            current_request = next_request;
          }

        /* On r-initalise */
        nb_request = 0;
        first_request = NULL;
        last_request = NULL;
        break;


      /***************************/
      /*  Gestion des Operation  */
      /***************************/
      case MEMORY_ADD_OPERATION :
        /* Init : No Error */
        *((int *) value) = 0;

        /* On ne traite pas les structures vides */
        if((struct operation *)data == NULL)
          break;

        /* Rcupre la structure */
        current_operation = (struct operation *) data;

        /* Stocke la structure */
        if(nb_operation_set == 0)
          {
            /* Allocation mmoire 1ere structure */
            current_operation_set = (struct operation_set *) calloc(1,sizeof(struct operation_set));
            if(current_operation_set == NULL)
              {
                *((int *) value) = 1;   /* Error */
                break;
              }
            current_operation_set->tab_operation = (struct operation *) calloc(MAX_OPERATION,sizeof(struct operation));
            if(current_operation_set->tab_operation == NULL)
              {
                *((int *) value) = 1;   /* Error */
                free(current_operation_set);
                break;
              }

            /* Copie des Data */
            memcpy(&current_operation_set->tab_operation[current_operation_set->nb_operation],current_operation,sizeof(struct operation));
            current_operation_set->nb_operation++;
            nb_operation++;

            /* Liste chaine */
            first_operation_set = current_operation_set;
            last_operation_set = current_operation_set;
            nb_operation_set++;
          }
        else
          {
            /* Encore de la place dans le tableau */
            if(last_operation_set->nb_operation < MAX_OPERATION)
              {
                memcpy(&last_operation_set->tab_operation[last_operation_set->nb_operation],current_operation,sizeof(struct operation));
                last_operation_set->nb_operation++;
                nb_operation++;
              }
            else
              {
                /* Allocation mmoire nouvelle structure */
                current_operation_set = (struct operation_set *) calloc(1,sizeof(struct operation_set));
                if(current_operation_set == NULL)
                  {
                    *((int *) value) = 1;   /* Error */
                    break;
                  }
                current_operation_set->tab_operation = (struct operation *) calloc(MAX_OPERATION,sizeof(struct operation));
                if(current_operation_set->tab_operation == NULL)
                  {
                    *((int *) value) = 1;   /* Error */
                    free(current_operation_set);
                    break;
                  }

                /* Copie les Data */
                memcpy(&current_operation_set->tab_operation[current_operation_set->nb_operation],current_operation,sizeof(struct operation));
                current_operation_set->nb_operation++;
                nb_operation++;

                /* Liste chaine */
                last_operation_set->next = current_operation_set;
                last_operation_set = current_operation_set;
                nb_operation_set++;
              }
          }
        break;

      case MEMORY_ADD_EXTRA_OPERATION :
        /* Init : No Error */
        *((int *) value) = 0;

        /* On ne traite pas les structures vides */
        if((struct operation *)data == NULL)
          break;

        /* Rcupre la structure */
        current_operation = (struct operation *) data;

        /* Stocke la structure */
        if(nb_operation_set == 0)
          {
            /* Allocation mmoire 1ere structure */
            current_operation_set = (struct operation_set *) calloc(1,sizeof(struct operation_set));
            if(current_operation_set == NULL)
              {
                *((int *) value) = 1;   /* Error */
                break;
              }
            current_operation_set->tab_operation = (struct operation *) calloc(MAX_OPERATION,sizeof(struct operation));
            if(current_operation_set->tab_operation == NULL)
              {
                *((int *) value) = 1;   /* Error */
                free(current_operation_set);
                break;
              }

            /* Copie des Data */
            memcpy(&current_operation_set->tab_operation[current_operation_set->nb_operation],current_operation,sizeof(struct operation));
            current_operation_set->nb_operation++;
            nb_operation++;

            /* Liste chaine */
            first_operation_set = current_operation_set;
            last_operation_set = current_operation_set;
            nb_operation_set++;
          }
        else
          {
            /* Encore de la place dans le tableau */
            if(first_operation_set->nb_operation < window_main->operation_text_data_row)
              {
                /* Ajoute l'entre  la fin */
                memcpy(&first_operation_set->tab_operation[first_operation_set->nb_operation],current_operation,sizeof(struct operation));
                first_operation_set->nb_operation++;
                nb_operation++;
              }
            else
              {
                /* Dcale les entres */
                for(i=0; i<(nb_operation-1); i++)
                  memcpy(&first_operation_set->tab_operation[i],&first_operation_set->tab_operation[i+1],sizeof(struct operation));
                /* Remplace le dernire entres */
                memcpy(&first_operation_set->tab_operation[first_operation_set->nb_operation-1],current_operation,sizeof(struct operation));
              }
          }
        break;

      case MEMORY_GET_OPERATION :
        /* Vrification d'un dpassement */
        *((struct operation **)value) = NULL;
        operation_number = *((int *) data);

        /* Parcours de la liste chaine */
        for(i=1,current_operation_set = first_operation_set; current_operation_set; i+=current_operation_set->nb_operation, current_operation_set = current_operation_set->next)
          {
            if(operation_number >= i && operation_number < (i+current_operation_set->nb_operation))
              {
                *((struct operation **)value) = &current_operation_set->tab_operation[operation_number-i];
                break;
              }
          }
        break;

      case MEMORY_GET_OPERATION_NB :
        *((int *) data) = nb_operation;
        break;

      case MEMORY_FREE_EXTRA_OPERATION :
        nb_extra_operation = *((int *) data);
        if(nb_operation <= nb_extra_operation)
          break;

        /* Rcupre les X dernires entres */
        for(j=0, i=(nb_operation-nb_extra_operation)+1; i<=nb_operation; i++, j++)
          {
            my_Memory(MEMORY_GET_OPERATION,&i,&current_operation);
            memcpy(&tab_operation[j],current_operation,sizeof(struct operation));
          }

        /* On efface toutes les structures enregistres (Liste chane ) en trop */
        for(current_operation_set = first_operation_set->next; current_operation_set!=NULL;)
          {
            next_operation_set = current_operation_set->next;
            if(current_operation_set->tab_operation)
              free(current_operation_set->tab_operation);
            free(current_operation_set);
            current_operation_set = next_operation_set;
          }

        /* Remplissage */
        for(i=0; i<nb_extra_operation; i++)
          memcpy(&first_operation_set->tab_operation[i],&tab_operation[i],sizeof(struct operation));

        /* On r-initalise */
        nb_operation = nb_extra_operation;
        nb_operation_set = 1;
        last_operation_set = first_operation_set;
        first_operation_set->next = NULL;
        first_operation_set->nb_operation = nb_extra_operation;
        break;

      case MEMORY_FREE_OPERATION :
        /* On efface toutes les structures enregistres (Liste chane ) */
        for(current_operation_set = first_operation_set; current_operation_set!=NULL;)
          {
            next_operation_set = current_operation_set->next;
            if(current_operation_set->tab_operation)
              free(current_operation_set->tab_operation);
            free(current_operation_set);
            current_operation_set = next_operation_set;
          }

        /* On r-initalise */
        nb_operation = 0;
        nb_operation_set = 0;
        first_operation_set = NULL;
        last_operation_set = NULL;
        break;


      /*************************************************/
      /** Gestion de la fentre Main de l'application **/
      /*************************************************/
      case MEMORY_SET_WINDOW_MAIN :
        window_main = (struct window_main *) data;
        break;

      case MEMORY_GET_WINDOW_MAIN :
        *((struct window_main **) data) = window_main;
        break;

      case MEMORY_FREE_WINDOW_MAIN :
        mem_free_window_main(window_main);
        window_main = NULL;
        break;

      /**************************************************/
      /** Gestion de la fentre About de l'application **/
      /**************************************************/
      case MEMORY_SET_WINDOW_ABOUT :
        window_about = (struct window_about *) data;
        break;

      case MEMORY_GET_WINDOW_ABOUT :
        *((struct window_about **) data) = window_about;
        break;

      case MEMORY_CLOSE_WINDOW_ABOUT :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_about == NULL) ? NULL : window_about->hwnd_about;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_about != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_ABOUT :
        EnterCriticalSection(&param->cs_windowclose_threadhandle); 
        mem_free_window_about(window_about);
        window_about = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

        
      /**************************************************/
      /** Gestion de la fentre Stack de l'application **/
      /**************************************************/
      case MEMORY_SET_WINDOW_STACK :
        window_stack = (struct window_stack *) data;
        break;

      case MEMORY_GET_WINDOW_STACK :
        *((struct window_stack **) data) = window_stack;
        break;

      case MEMORY_CLOSE_WINDOW_STACK :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_stack == NULL) ? NULL : window_stack->hwnd_stack;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_stack != NULL)
          ;
        break;
        
      case MEMORY_FREE_WINDOW_STACK :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_stack(window_stack);
        window_stack = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      /*******************************************************/
      /** Gestion de la fentre DirectPage de l'application **/
      /*******************************************************/
      case MEMORY_SET_WINDOW_DIRECTPAGE :
        window_directpage = (struct window_directpage *) data;
        break;

      case MEMORY_GET_WINDOW_DIRECTPAGE :
        *((struct window_directpage **) data) = window_directpage;
        break;

      case MEMORY_CLOSE_WINDOW_DIRECTPAGE :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_directpage == NULL) ? NULL : window_directpage->hwnd_directpage;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);        
        while(window_directpage != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_DIRECTPAGE :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_directpage(window_directpage);
        window_directpage = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      /*******************************************************/
      /** Gestion de la fentre Softswitch de l'application **/
      /*******************************************************/
      case MEMORY_SET_WINDOW_SOFTSWITCH :
        window_softswitch = (struct window_softswitch *) data;
        break;

      case MEMORY_GET_WINDOW_SOFTSWITCH :
        *((struct window_softswitch **) data) = window_softswitch;
        break;

      case MEMORY_CLOSE_WINDOW_SOFTSWITCH :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_softswitch == NULL) ? NULL : window_softswitch->hwnd_softswitch;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_softswitch != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_SOFTSWITCH :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_softswitch(window_softswitch);
        window_softswitch = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;
        
      /******************************************************/
      /** Gestion de la fentre Processor de l'application **/
      /******************************************************/
      case MEMORY_SET_WINDOW_PROCESSOR :
        window_processor = (struct window_processor *) data;
        break;

      case MEMORY_GET_WINDOW_PROCESSOR :
        *((struct window_processor **) data) = window_processor;
        break;

      case MEMORY_CLOSE_WINDOW_PROCESSOR :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_processor == NULL) ? NULL : window_processor->hwnd_processor;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_processor != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_PROCESSOR :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_processor(window_processor);
        window_processor = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      /*******************************************************/
      /** Gestion des fentres Memory Area de l'application **/
      /*******************************************************/
      case MEMORY_ADD_WINDOW_MEMORYAREA :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_memoryarea_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_memoryarea = (struct window_memoryarea *) data;
        if(first_window_memoryarea == NULL)
          first_window_memoryarea = current_window_memoryarea;
        else
          last_window_memoryarea->next = current_window_memoryarea;
        last_window_memoryarea = current_window_memoryarea;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_memoryarea_threadhandle);
        break;

      case MEMORY_GET_WINDOW_MEMORYAREA :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_memoryarea_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_memoryarea **) data) = NULL;
        for(current_window_memoryarea = first_window_memoryarea; current_window_memoryarea; current_window_memoryarea = current_window_memoryarea->next)
          if(current_window_memoryarea->thread_id == thread_id)
            *((struct window_memoryarea **) data) = current_window_memoryarea;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_memoryarea_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_MEMORYAREA :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_memoryarea_threadhandle); 

        /* Structure  Supprimer */
        delete_window_memoryarea = (struct window_memoryarea *) data;

        /** Supprime la structure Memory Area de la liste **/
        if(first_window_memoryarea == delete_window_memoryarea && last_window_memoryarea == delete_window_memoryarea)
          {
            /* Structure unique */
            first_window_memoryarea = NULL;
            last_window_memoryarea = NULL;

            /* Libration mmoire */
            mem_free_window_memoryarea(delete_window_memoryarea);
          }
        else if(first_window_memoryarea == delete_window_memoryarea)
          {
            /* Premire Structure */
            first_window_memoryarea = delete_window_memoryarea->next;

            /* Libration mmoire */
            mem_free_window_memoryarea(delete_window_memoryarea);
          }
        else if(last_window_memoryarea == delete_window_memoryarea)
          {
            /* Dernire Structure */
            for(current_window_memoryarea = first_window_memoryarea; current_window_memoryarea; current_window_memoryarea = current_window_memoryarea->next)
              if(current_window_memoryarea->next == delete_window_memoryarea)
                {
                  last_window_memoryarea = current_window_memoryarea;
                  current_window_memoryarea->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_memoryarea(delete_window_memoryarea);
          }
        else
          {
            /* Au milieu */
            for(current_window_memoryarea = first_window_memoryarea; current_window_memoryarea; current_window_memoryarea = current_window_memoryarea->next)
              if(current_window_memoryarea->next == delete_window_memoryarea)
                {
                  current_window_memoryarea->next = delete_window_memoryarea->next;

                  /* Libration mmoire */
                  mem_free_window_memoryarea(delete_window_memoryarea);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_memoryarea_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_MEMORYAREA :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_memoryarea = first_window_memoryarea; current_window_memoryarea; current_window_memoryarea = current_window_memoryarea->next)
          tab_window_handle[nb_window_handle++] = current_window_memoryarea->hwnd_memoryarea;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_memoryarea != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_MEMORYAREA :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_memoryarea_threadhandle); 

        /** Supprime la liste des Memory Area **/
        for(current_window_memoryarea = first_window_memoryarea; current_window_memoryarea; current_window_memoryarea = next_window_memoryarea)
          {
            next_window_memoryarea = current_window_memoryarea->next;
            mem_free_window_memoryarea(current_window_memoryarea);
          }
        first_window_memoryarea = NULL;
        last_window_memoryarea = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_memoryarea_threadhandle);
        break;
        
      /*****************************************************/
      /** Gestion des fentres Ram Sound de l'application **/
      /*****************************************************/
      case MEMORY_ADD_WINDOW_RAMSOUND :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_ramsound_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_ramsound = (struct window_ramsound *) data;
        if(first_window_ramsound == NULL)
          first_window_ramsound = current_window_ramsound;
        else
          last_window_ramsound->next = current_window_ramsound;
        last_window_ramsound = current_window_ramsound;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_ramsound_threadhandle);
        break;

      case MEMORY_GET_WINDOW_RAMSOUND :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_ramsound_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_ramsound **) data) = NULL;
        for(current_window_ramsound = first_window_ramsound; current_window_ramsound; current_window_ramsound = current_window_ramsound->next)
          if(current_window_ramsound->thread_id == thread_id)
            *((struct window_ramsound **) data) = current_window_ramsound;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_ramsound_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_RAMSOUND :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_ramsound_threadhandle); 

        /* Structure  Supprimer */
        delete_window_ramsound = (struct window_ramsound *) data;

        /** Supprime la structure Memory Area de la liste **/
        if(first_window_ramsound == delete_window_ramsound && last_window_ramsound == delete_window_ramsound)
          {
            /* Structure unique */
            first_window_ramsound = NULL;
            last_window_ramsound = NULL;

            /* Libration mmoire */
            mem_free_window_ramsound(delete_window_ramsound);
          }
        else if(first_window_ramsound == delete_window_ramsound)
          {
            /* Premire Structure */
            first_window_ramsound = delete_window_ramsound->next;

            /* Libration mmoire */
            mem_free_window_ramsound(delete_window_ramsound);
          }
        else if(last_window_ramsound == delete_window_ramsound)
          {
            /* Dernire Structure */
            for(current_window_ramsound = first_window_ramsound; current_window_ramsound; current_window_ramsound = current_window_ramsound->next)
              if(current_window_ramsound->next == delete_window_ramsound)
                {
                  last_window_ramsound = current_window_ramsound;
                  current_window_ramsound->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_ramsound(delete_window_ramsound);
          }
        else
          {
            /* Au milieu */
            for(current_window_ramsound = first_window_ramsound; current_window_ramsound; current_window_ramsound = current_window_ramsound->next)
              if(current_window_ramsound->next == delete_window_ramsound)
                {
                  current_window_ramsound->next = delete_window_ramsound->next;

                  /* Libration mmoire */
                  mem_free_window_ramsound(delete_window_ramsound);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_ramsound_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_RAMSOUND :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_ramsound = first_window_ramsound; current_window_ramsound; current_window_ramsound = current_window_ramsound->next)
          tab_window_handle[nb_window_handle++] = current_window_ramsound->hwnd_ramsound;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_ramsound != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_RAMSOUND :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_ramsound_threadhandle); 

        /** Supprime la liste des Ram Sound **/
        for(current_window_ramsound = first_window_ramsound; current_window_ramsound; current_window_ramsound = next_window_ramsound)
          {
            next_window_ramsound = current_window_ramsound->next;
            mem_free_window_ramsound(current_window_ramsound);
          }
        first_window_ramsound = NULL;
        last_window_ramsound = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_ramsound_threadhandle);
        break;
        
      /********************************************************/
      /** Gestion des fentres Text Page 40 de l'application **/
      /********************************************************/
      case MEMORY_ADD_WINDOW_TEXTPAGE40 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_textpage40_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_textpage40 = (struct window_textpage40 *) data;
        if(first_window_textpage40 == NULL)
          first_window_textpage40 = current_window_textpage40;
        else
          last_window_textpage40->next = current_window_textpage40;
        last_window_textpage40 = current_window_textpage40;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_textpage40_threadhandle);
        break;

      case MEMORY_GET_WINDOW_TEXTPAGE40 :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_textpage40_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_textpage40 **) data) = NULL;
        for(current_window_textpage40 = first_window_textpage40; current_window_textpage40; current_window_textpage40 = current_window_textpage40->next)
          if(current_window_textpage40->thread_id == thread_id)
            *((struct window_textpage40 **) data) = current_window_textpage40;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_textpage40_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_TEXTPAGE40 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_textpage40_threadhandle); 

        /* Structure  Supprimer */
        delete_window_textpage40 = (struct window_textpage40 *) data;

        /** Supprime la structure Text Page 40 de la liste **/
        if(first_window_textpage40 == delete_window_textpage40 && last_window_textpage40 == delete_window_textpage40)
          {
            /* Structure unique */
            first_window_textpage40 = NULL;
            last_window_textpage40 = NULL;

            /* Libration mmoire */
            mem_free_window_textpage40(delete_window_textpage40);
          }
        else if(first_window_textpage40 == delete_window_textpage40)
          {
            /* Premire Structure */
            first_window_textpage40 = delete_window_textpage40->next;

            /* Libration mmoire */
            mem_free_window_textpage40(delete_window_textpage40);
          }
        else if(last_window_textpage40 == delete_window_textpage40)
          {
            /* Dernire Structure */
            for(current_window_textpage40 = first_window_textpage40; current_window_textpage40; current_window_textpage40 = current_window_textpage40->next)
              if(current_window_textpage40->next == delete_window_textpage40)
                {
                  last_window_textpage40 = current_window_textpage40;
                  current_window_textpage40->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_textpage40(delete_window_textpage40);
          }
        else
          {
            /* Au milieu */
            for(current_window_textpage40 = first_window_textpage40; current_window_textpage40; current_window_textpage40 = current_window_textpage40->next)
              if(current_window_textpage40->next == delete_window_textpage40)
                {
                  current_window_textpage40->next = delete_window_textpage40->next;

                  /* Libration mmoire */
                  mem_free_window_textpage40(delete_window_textpage40);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_textpage40_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_TEXTPAGE40 :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_textpage40 = first_window_textpage40; current_window_textpage40; current_window_textpage40 = current_window_textpage40->next)
          tab_window_handle[nb_window_handle++] = current_window_textpage40->hwnd_textpage40;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_textpage40 != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_TEXTPAGE40 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_textpage40_threadhandle); 

        /** Supprime la liste des Text Page 40 **/
        for(current_window_textpage40 = first_window_textpage40; current_window_textpage40; current_window_textpage40 = next_window_textpage40)
          {
            next_window_textpage40 = current_window_textpage40->next;
            mem_free_window_textpage40(current_window_textpage40);
          }
        first_window_textpage40 = NULL;
        last_window_textpage40 = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_textpage40_threadhandle);
        break;        
        
      /********************************************************/
      /** Gestion des fentres Text Page 80 de l'application **/
      /********************************************************/
      case MEMORY_ADD_WINDOW_TEXTPAGE80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_textpage80_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_textpage80 = (struct window_textpage80 *) data;
        if(first_window_textpage80 == NULL)
          first_window_textpage80 = current_window_textpage80;
        else
          last_window_textpage80->next = current_window_textpage80;
        last_window_textpage80 = current_window_textpage80;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_textpage80_threadhandle);
        break;

      case MEMORY_GET_WINDOW_TEXTPAGE80 :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_textpage80_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_textpage80 **) data) = NULL;
        for(current_window_textpage80 = first_window_textpage80; current_window_textpage80; current_window_textpage80 = current_window_textpage80->next)
          if(current_window_textpage80->thread_id == thread_id)
            *((struct window_textpage80 **) data) = current_window_textpage80;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_textpage80_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_TEXTPAGE80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_textpage80_threadhandle); 

        /* Structure  Supprimer */
        delete_window_textpage80 = (struct window_textpage80 *) data;

        /** Supprime la structure Text Page 80 de la liste **/
        if(first_window_textpage80 == delete_window_textpage80 && last_window_textpage80 == delete_window_textpage80)
          {
            /* Structure unique */
            first_window_textpage80 = NULL;
            last_window_textpage80 = NULL;

            /* Libration mmoire */
            mem_free_window_textpage80(delete_window_textpage80);
          }
        else if(first_window_textpage80 == delete_window_textpage80)
          {
            /* Premire Structure */
            first_window_textpage80 = delete_window_textpage80->next;

            /* Libration mmoire */
            mem_free_window_textpage80(delete_window_textpage80);
          }
        else if(last_window_textpage80 == delete_window_textpage80)
          {
            /* Dernire Structure */
            for(current_window_textpage80 = first_window_textpage80; current_window_textpage80; current_window_textpage80 = current_window_textpage80->next)
              if(current_window_textpage80->next == delete_window_textpage80)
                {
                  last_window_textpage80 = current_window_textpage80;
                  current_window_textpage80->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_textpage80(delete_window_textpage80);
          }
        else
          {
            /* Au milieu */
            for(current_window_textpage80 = first_window_textpage80; current_window_textpage80; current_window_textpage80 = current_window_textpage80->next)
              if(current_window_textpage80->next == delete_window_textpage80)
                {
                  current_window_textpage80->next = delete_window_textpage80->next;

                  /* Libration mmoire */
                  mem_free_window_textpage80(delete_window_textpage80);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_textpage80_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_TEXTPAGE80 :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_textpage80 = first_window_textpage80; current_window_textpage80; current_window_textpage80 = current_window_textpage80->next)
          tab_window_handle[nb_window_handle++] = current_window_textpage80->hwnd_textpage80;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_textpage80 != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_TEXTPAGE80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_textpage80_threadhandle); 

        /** Supprime la liste des Text Page 80 **/
        for(current_window_textpage80 = first_window_textpage80; current_window_textpage80; current_window_textpage80 = next_window_textpage80)
          {
            next_window_textpage80 = current_window_textpage80->next;
            mem_free_window_textpage80(current_window_textpage80);
          }
        first_window_textpage80 = NULL;
        last_window_textpage80 = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_textpage80_threadhandle);
        break;        

      /******************************************************/
      /** Gestion des fentres Graphic GR de l'application **/
      /******************************************************/
      case MEMORY_ADD_WINDOW_GRAPHICGR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicgr_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_graphicgr = (struct window_graphicgr *) data;
        if(first_window_graphicgr == NULL)
          first_window_graphicgr = current_window_graphicgr;
        else
          last_window_graphicgr->next = current_window_graphicgr;
        last_window_graphicgr = current_window_graphicgr;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicgr_threadhandle);
        break;

      case MEMORY_GET_WINDOW_GRAPHICGR :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicgr_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_graphicgr **) data) = NULL;
        for(current_window_graphicgr = first_window_graphicgr; current_window_graphicgr; current_window_graphicgr = current_window_graphicgr->next)
          if(current_window_graphicgr->thread_id == thread_id)
            *((struct window_graphicgr **) data) = current_window_graphicgr;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicgr_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_GRAPHICGR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_graphicgr_threadhandle); 

        /* Structure  Supprimer */
        delete_window_graphicgr = (struct window_graphicgr *) data;

        /** Supprime la structure Graphic GR de la liste **/
        if(first_window_graphicgr == delete_window_graphicgr && last_window_graphicgr == delete_window_graphicgr)
          {
            /* Structure unique */
            first_window_graphicgr = NULL;
            last_window_graphicgr = NULL;

            /* Libration mmoire */
            mem_free_window_graphicgr(delete_window_graphicgr);
          }
        else if(first_window_graphicgr == delete_window_graphicgr)
          {
            /* Premire Structure */
            first_window_graphicgr = delete_window_graphicgr->next;

            /* Libration mmoire */
            mem_free_window_graphicgr(delete_window_graphicgr);
          }
        else if(last_window_graphicgr == delete_window_graphicgr)
          {
            /* Dernire Structure */
            for(current_window_graphicgr = first_window_graphicgr; current_window_graphicgr; current_window_graphicgr = current_window_graphicgr->next)
              if(current_window_graphicgr->next == delete_window_graphicgr)
                {
                  last_window_graphicgr = current_window_graphicgr;
                  current_window_graphicgr->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_graphicgr(delete_window_graphicgr);
          }
        else
          {
            /* Au milieu */
            for(current_window_graphicgr = first_window_graphicgr; current_window_graphicgr; current_window_graphicgr = current_window_graphicgr->next)
              if(current_window_graphicgr->next == delete_window_graphicgr)
                {
                  current_window_graphicgr->next = delete_window_graphicgr->next;

                  /* Libration mmoire */
                  mem_free_window_graphicgr(delete_window_graphicgr);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicgr_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICGR :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_graphicgr = first_window_graphicgr; current_window_graphicgr; current_window_graphicgr = current_window_graphicgr->next)
          tab_window_handle[nb_window_handle++] = current_window_graphicgr->hwnd_graphicgr;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_graphicgr != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICGR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicgr_threadhandle); 

        /** Supprime la liste des Graphic GR **/
        for(current_window_graphicgr = first_window_graphicgr; current_window_graphicgr; current_window_graphicgr = next_window_graphicgr)
          {
            next_window_graphicgr = current_window_graphicgr->next;
            mem_free_window_graphicgr(current_window_graphicgr);
          }
        first_window_graphicgr = NULL;
        last_window_graphicgr = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicgr_threadhandle);
        break;


      /*******************************************************/
      /** Gestion des fentres Graphic HGR de l'application **/
      /*******************************************************/
      case MEMORY_ADD_WINDOW_GRAPHICHGR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphichgr_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_graphichgr = (struct window_graphichgr *) data;
        if(first_window_graphichgr == NULL)
          first_window_graphichgr = current_window_graphichgr;
        else
          last_window_graphichgr->next = current_window_graphichgr;
        last_window_graphichgr = current_window_graphichgr;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphichgr_threadhandle);
        break;

      case MEMORY_GET_WINDOW_GRAPHICHGR :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphichgr_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_graphichgr **) data) = NULL;
        for(current_window_graphichgr = first_window_graphichgr; current_window_graphichgr; current_window_graphichgr = current_window_graphichgr->next)
          if(current_window_graphichgr->thread_id == thread_id)
            *((struct window_graphichgr **) data) = current_window_graphichgr;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphichgr_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_GRAPHICHGR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_graphichgr_threadhandle); 

        /* Structure  Supprimer */
        delete_window_graphichgr = (struct window_graphichgr *) data;

        /** Supprime la structure Graphic HGR de la liste **/
        if(first_window_graphichgr == delete_window_graphichgr && last_window_graphichgr == delete_window_graphichgr)
          {
            /* Structure unique */
            first_window_graphichgr = NULL;
            last_window_graphichgr = NULL;

            /* Libration mmoire */
            mem_free_window_graphichgr(delete_window_graphichgr);
          }
        else if(first_window_graphichgr == delete_window_graphichgr)
          {
            /* Premire Structure */
            first_window_graphichgr = delete_window_graphichgr->next;

            /* Libration mmoire */
            mem_free_window_graphichgr(delete_window_graphichgr);
          }
        else if(last_window_graphichgr == delete_window_graphichgr)
          {
            /* Dernire Structure */
            for(current_window_graphichgr = first_window_graphichgr; current_window_graphichgr; current_window_graphichgr = current_window_graphichgr->next)
              if(current_window_graphichgr->next == delete_window_graphichgr)
                {
                  last_window_graphichgr = current_window_graphichgr;
                  current_window_graphichgr->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_graphichgr(delete_window_graphichgr);
          }
        else
          {
            /* Au milieu */
            for(current_window_graphichgr = first_window_graphichgr; current_window_graphichgr; current_window_graphichgr = current_window_graphichgr->next)
              if(current_window_graphichgr->next == delete_window_graphichgr)
                {
                  current_window_graphichgr->next = delete_window_graphichgr->next;

                  /* Libration mmoire */
                  mem_free_window_graphichgr(delete_window_graphichgr);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphichgr_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICHGR :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_graphichgr = first_window_graphichgr; current_window_graphichgr; current_window_graphichgr = current_window_graphichgr->next)
          tab_window_handle[nb_window_handle++] = current_window_graphichgr->hwnd_graphichgr;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_graphichgr != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICHGR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphichgr_threadhandle); 

        /** Supprime la liste des Graphic HGR **/
        for(current_window_graphichgr = first_window_graphichgr; current_window_graphichgr; current_window_graphichgr = next_window_graphichgr)
          {
            next_window_graphichgr = current_window_graphichgr->next;
            mem_free_window_graphichgr(current_window_graphichgr);
          }
        first_window_graphichgr = NULL;
        last_window_graphichgr = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphichgr_threadhandle);
        break;


      /************************************************************/
      /** Gestion des fentres Graphic DHGRMono de l'application **/
      /************************************************************/
      case MEMORY_ADD_WINDOW_GRAPHICDHGRMONO :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdhgrmono_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_graphicdhgrmono = (struct window_graphicdhgrmono *) data;
        if(first_window_graphicdhgrmono == NULL)
          first_window_graphicdhgrmono = current_window_graphicdhgrmono;
        else
          last_window_graphicdhgrmono->next = current_window_graphicdhgrmono;
        last_window_graphicdhgrmono = current_window_graphicdhgrmono;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrmono_threadhandle);
        break;

      case MEMORY_GET_WINDOW_GRAPHICDHGRMONO :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdhgrmono_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_graphicdhgrmono **) data) = NULL;
        for(current_window_graphicdhgrmono = first_window_graphicdhgrmono; current_window_graphicdhgrmono; current_window_graphicdhgrmono = current_window_graphicdhgrmono->next)
          if(current_window_graphicdhgrmono->thread_id == thread_id)
            *((struct window_graphicdhgrmono **) data) = current_window_graphicdhgrmono;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrmono_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_GRAPHICDHGRMONO :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_graphicdhgrmono_threadhandle); 

        /* Structure  Supprimer */
        delete_window_graphicdhgrmono = (struct window_graphicdhgrmono *) data;

        /** Supprime la structure Graphic DHGR Mono de la liste **/
        if(first_window_graphicdhgrmono == delete_window_graphicdhgrmono && last_window_graphicdhgrmono == delete_window_graphicdhgrmono)
          {
            /* Structure unique */
            first_window_graphicdhgrmono = NULL;
            last_window_graphicdhgrmono = NULL;

            /* Libration mmoire */
            mem_free_window_graphicdhgrmono(delete_window_graphicdhgrmono);
          }
        else if(first_window_graphicdhgrmono == delete_window_graphicdhgrmono)
          {
            /* Premire Structure */
            first_window_graphicdhgrmono = delete_window_graphicdhgrmono->next;

            /* Libration mmoire */
            mem_free_window_graphicdhgrmono(delete_window_graphicdhgrmono);
          }
        else if(last_window_graphicdhgrmono == delete_window_graphicdhgrmono)
          {
            /* Dernire Structure */
            for(current_window_graphicdhgrmono = first_window_graphicdhgrmono; current_window_graphicdhgrmono; current_window_graphicdhgrmono = current_window_graphicdhgrmono->next)
              if(current_window_graphicdhgrmono->next == delete_window_graphicdhgrmono)
                {
                  last_window_graphicdhgrmono = current_window_graphicdhgrmono;
                  current_window_graphicdhgrmono->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_graphicdhgrmono(delete_window_graphicdhgrmono);
          }
        else
          {
            /* Au milieu */
            for(current_window_graphicdhgrmono = first_window_graphicdhgrmono; current_window_graphicdhgrmono; current_window_graphicdhgrmono = current_window_graphicdhgrmono->next)
              if(current_window_graphicdhgrmono->next == delete_window_graphicdhgrmono)
                {
                  current_window_graphicdhgrmono->next = delete_window_graphicdhgrmono->next;

                  /* Libration mmoire */
                  mem_free_window_graphicdhgrmono(delete_window_graphicdhgrmono);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrmono_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICDHGRMONO :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_graphicdhgrmono = first_window_graphicdhgrmono; current_window_graphicdhgrmono; current_window_graphicdhgrmono = current_window_graphicdhgrmono->next)
          tab_window_handle[nb_window_handle++] = current_window_graphicdhgrmono->hwnd_graphicdhgrmono;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_graphicdhgrmono != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICDHGRMONO :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdhgrmono_threadhandle); 

        /** Supprime la liste des Graphic DHGR Mono **/
        for(current_window_graphicdhgrmono = first_window_graphicdhgrmono; current_window_graphicdhgrmono; current_window_graphicdhgrmono = next_window_graphicdhgrmono)
          {
            next_window_graphicdhgrmono = current_window_graphicdhgrmono->next;
            mem_free_window_graphicdhgrmono(current_window_graphicdhgrmono);
          }
        first_window_graphicdhgrmono = NULL;
        last_window_graphicdhgrmono = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrmono_threadhandle);
        break;
        
        
      /*************************************************************/
      /** Gestion des fentres Graphic DHGRColor de l'application **/
      /*************************************************************/
      case MEMORY_ADD_WINDOW_GRAPHICDHGRCOLOR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdhgrcolor_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_graphicdhgrcolor = (struct window_graphicdhgrcolor *) data;
        if(first_window_graphicdhgrcolor == NULL)
          first_window_graphicdhgrcolor = current_window_graphicdhgrcolor;
        else
          last_window_graphicdhgrcolor->next = current_window_graphicdhgrcolor;
        last_window_graphicdhgrcolor = current_window_graphicdhgrcolor;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrcolor_threadhandle);
        break;

      case MEMORY_GET_WINDOW_GRAPHICDHGRCOLOR :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdhgrcolor_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_graphicdhgrcolor **) data) = NULL;
        for(current_window_graphicdhgrcolor = first_window_graphicdhgrcolor; current_window_graphicdhgrcolor; current_window_graphicdhgrcolor = current_window_graphicdhgrcolor->next)
          if(current_window_graphicdhgrcolor->thread_id == thread_id)
            *((struct window_graphicdhgrcolor **) data) = current_window_graphicdhgrcolor;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrcolor_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_GRAPHICDHGRCOLOR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_graphicdhgrcolor_threadhandle); 

        /* Structure  Supprimer */
        delete_window_graphicdhgrcolor = (struct window_graphicdhgrcolor *) data;

        /** Supprime la structure Graphic DHGR Color de la liste **/
        if(first_window_graphicdhgrcolor == delete_window_graphicdhgrcolor && last_window_graphicdhgrcolor == delete_window_graphicdhgrcolor)
          {
            /* Structure unique */
            first_window_graphicdhgrcolor = NULL;
            last_window_graphicdhgrcolor = NULL;

            /* Libration mmoire */
            mem_free_window_graphicdhgrcolor(delete_window_graphicdhgrcolor);
          }
        else if(first_window_graphicdhgrcolor == delete_window_graphicdhgrcolor)
          {
            /* Premire Structure */
            first_window_graphicdhgrcolor = delete_window_graphicdhgrcolor->next;

            /* Libration mmoire */
            mem_free_window_graphicdhgrcolor(delete_window_graphicdhgrcolor);
          }
        else if(last_window_graphicdhgrcolor == delete_window_graphicdhgrcolor)
          {
            /* Dernire Structure */
            for(current_window_graphicdhgrcolor = first_window_graphicdhgrcolor; current_window_graphicdhgrcolor; current_window_graphicdhgrcolor = current_window_graphicdhgrcolor->next)
              if(current_window_graphicdhgrcolor->next == delete_window_graphicdhgrcolor)
                {
                  last_window_graphicdhgrcolor = current_window_graphicdhgrcolor;
                  current_window_graphicdhgrcolor->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_graphicdhgrcolor(delete_window_graphicdhgrcolor);
          }
        else
          {
            /* Au milieu */
            for(current_window_graphicdhgrcolor = first_window_graphicdhgrcolor; current_window_graphicdhgrcolor; current_window_graphicdhgrcolor = current_window_graphicdhgrcolor->next)
              if(current_window_graphicdhgrcolor->next == delete_window_graphicdhgrcolor)
                {
                  current_window_graphicdhgrcolor->next = delete_window_graphicdhgrcolor->next;

                  /* Libration mmoire */
                  mem_free_window_graphicdhgrcolor(delete_window_graphicdhgrcolor);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrcolor_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICDHGRCOLOR :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_graphicdhgrcolor = first_window_graphicdhgrcolor; current_window_graphicdhgrcolor; current_window_graphicdhgrcolor = current_window_graphicdhgrcolor->next)
          tab_window_handle[nb_window_handle++] = current_window_graphicdhgrcolor->hwnd_graphicdhgrcolor;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_graphicdhgrcolor != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICDHGRCOLOR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdhgrcolor_threadhandle); 

        /** Supprime la liste des Graphic DHGR Color **/
        for(current_window_graphicdhgrcolor = first_window_graphicdhgrcolor; current_window_graphicdhgrcolor; current_window_graphicdhgrcolor = next_window_graphicdhgrcolor)
          {
            next_window_graphicdhgrcolor = current_window_graphicdhgrcolor->next;
            mem_free_window_graphicdhgrcolor(current_window_graphicdhgrcolor);
          }
        first_window_graphicdhgrcolor = NULL;
        last_window_graphicdhgrcolor = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrcolor_threadhandle);
        break;        

      /*******************************************************/
      /** Gestion des fentres Graphic DGR de l'application **/
      /*******************************************************/
      case MEMORY_ADD_WINDOW_GRAPHICDGR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdgr_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_graphicdgr = (struct window_graphicdgr *) data;
        if(first_window_graphicdgr == NULL)
          first_window_graphicdgr = current_window_graphicdgr;
        else
          last_window_graphicdgr->next = current_window_graphicdgr;
        last_window_graphicdgr = current_window_graphicdgr;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdgr_threadhandle);
        break;

      case MEMORY_GET_WINDOW_GRAPHICDGR :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdgr_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_graphicdhgr **) data) = NULL;
        for(current_window_graphicdgr = first_window_graphicdgr; current_window_graphicdgr; current_window_graphicdgr = current_window_graphicdgr->next)
          if(current_window_graphicdgr->thread_id == thread_id)
            *((struct window_graphicdgr **) data) = current_window_graphicdgr;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdgr_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_GRAPHICDGR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_graphicdgr_threadhandle); 

        /* Structure  Supprimer */
        delete_window_graphicdgr = (struct window_graphicdgr *) data;

        /** Supprime la structure Graphic DGR de la liste **/
        if(first_window_graphicdgr == delete_window_graphicdgr && last_window_graphicdgr == delete_window_graphicdgr)
          {
            /* Structure unique */
            first_window_graphicdgr = NULL;
            last_window_graphicdgr = NULL;

            /* Libration mmoire */
            mem_free_window_graphicdgr(delete_window_graphicdgr);
          }
        else if(first_window_graphicdgr == delete_window_graphicdgr)
          {
            /* Premire Structure */
            first_window_graphicdgr = delete_window_graphicdgr->next;

            /* Libration mmoire */
            mem_free_window_graphicdgr(delete_window_graphicdgr);
          }
        else if(last_window_graphicdgr == delete_window_graphicdgr)
          {
            /* Dernire Structure */
            for(current_window_graphicdgr = first_window_graphicdgr; current_window_graphicdgr; current_window_graphicdgr = current_window_graphicdgr->next)
              if(current_window_graphicdgr->next == delete_window_graphicdgr)
                {
                  last_window_graphicdgr = current_window_graphicdgr;
                  current_window_graphicdgr->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_graphicdgr(delete_window_graphicdgr);
          }
        else
          {
            /* Au milieu */
            for(current_window_graphicdgr = first_window_graphicdgr; current_window_graphicdgr; current_window_graphicdgr = current_window_graphicdgr->next)
              if(current_window_graphicdgr->next == delete_window_graphicdgr)
                {
                  current_window_graphicdgr->next = delete_window_graphicdgr->next;

                  /* Libration mmoire */
                  mem_free_window_graphicdgr(delete_window_graphicdgr);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdgr_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICDGR :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_graphicdgr = first_window_graphicdgr; current_window_graphicdgr; current_window_graphicdgr = current_window_graphicdgr->next)
          tab_window_handle[nb_window_handle++] = current_window_graphicdgr->hwnd_graphicdgr;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_graphicdgr != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICDGR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdgr_threadhandle); 

        /** Supprime la liste des Graphic DGR **/
        for(current_window_graphicdgr = first_window_graphicdgr; current_window_graphicdgr; current_window_graphicdgr = next_window_graphicdgr)
          {
            next_window_graphicdgr = current_window_graphicdgr->next;
            mem_free_window_graphicdgr(current_window_graphicdgr);
          }
        first_window_graphicdgr = NULL;
        last_window_graphicdgr = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdgr_threadhandle);
        break;

      /*************************************************************/
      /** Gestion des fentres GraphicHGR Text40 de l'application **/
      /*************************************************************/
      case MEMORY_ADD_WINDOW_GRAPHICHGRTEXT40 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphichgrtext40_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_graphichgrtext40 = (struct window_graphichgrtext40 *) data;
        if(first_window_graphichgrtext40 == NULL)
          first_window_graphichgrtext40 = current_window_graphichgrtext40;
        else
          last_window_graphichgrtext40->next = current_window_graphichgrtext40;
        last_window_graphichgrtext40 = current_window_graphichgrtext40;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphichgrtext40_threadhandle);
        break;

      case MEMORY_GET_WINDOW_GRAPHICHGRTEXT40 :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphichgrtext40_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_graphichgrtext40 **) data) = NULL;
        for(current_window_graphichgrtext40 = first_window_graphichgrtext40; current_window_graphichgrtext40; current_window_graphichgrtext40 = current_window_graphichgrtext40->next)
          if(current_window_graphichgrtext40->thread_id == thread_id)
            *((struct window_graphichgrtext40 **) data) = current_window_graphichgrtext40;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphichgrtext40_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_GRAPHICHGRTEXT40 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_graphichgrtext40_threadhandle); 

        /* Structure  Supprimer */
        delete_window_graphichgrtext40 = (struct window_graphichgrtext40 *) data;

        /** Supprime la structure GraphicHGR Text40 de la liste **/
        if(first_window_graphichgrtext40 == delete_window_graphichgrtext40 && last_window_graphichgrtext40 == delete_window_graphichgrtext40)
          {
            /* Structure unique */
            first_window_graphichgrtext40 = NULL;
            last_window_graphichgrtext40 = NULL;

            /* Libration mmoire */
            mem_free_window_graphichgrtext40(delete_window_graphichgrtext40);
          }
        else if(first_window_graphichgrtext40 == delete_window_graphichgrtext40)
          {
            /* Premire Structure */
            first_window_graphichgrtext40 = delete_window_graphichgrtext40->next;

            /* Libration mmoire */
            mem_free_window_graphichgrtext40(delete_window_graphichgrtext40);
          }
        else if(last_window_graphichgrtext40 == delete_window_graphichgrtext40)
          {
            /* Dernire Structure */
            for(current_window_graphichgrtext40 = first_window_graphichgrtext40; current_window_graphichgrtext40; current_window_graphichgrtext40 = current_window_graphichgrtext40->next)
              if(current_window_graphichgrtext40->next == delete_window_graphichgrtext40)
                {
                  last_window_graphichgrtext40 = current_window_graphichgrtext40;
                  current_window_graphichgrtext40->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_graphichgrtext40(delete_window_graphichgrtext40);
          }
        else
          {
            /* Au milieu */
            for(current_window_graphichgrtext40 = first_window_graphichgrtext40; current_window_graphichgrtext40; current_window_graphichgrtext40 = current_window_graphichgrtext40->next)
              if(current_window_graphichgrtext40->next == delete_window_graphichgrtext40)
                {
                  current_window_graphichgrtext40->next = delete_window_graphichgrtext40->next;

                  /* Libration mmoire */
                  mem_free_window_graphichgrtext40(delete_window_graphichgrtext40);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphichgrtext40_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICHGRTEXT40 :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_graphichgrtext40 = first_window_graphichgrtext40; current_window_graphichgrtext40; current_window_graphichgrtext40 = current_window_graphichgrtext40->next)
          tab_window_handle[nb_window_handle++] = current_window_graphichgrtext40->hwnd_graphichgrtext40;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_graphichgrtext40 != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICHGRTEXT40 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphichgrtext40_threadhandle); 

        /** Supprime la liste des GraphicHGR Text40 **/
        for(current_window_graphichgrtext40 = first_window_graphichgrtext40; current_window_graphichgrtext40; current_window_graphichgrtext40 = next_window_graphichgrtext40)
          {
            next_window_graphichgrtext40 = current_window_graphichgrtext40->next;
            mem_free_window_graphichgrtext40(current_window_graphichgrtext40);
          }
        first_window_graphichgrtext40 = NULL;
        last_window_graphichgrtext40 = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphichgrtext40_threadhandle);
        break;

      /*************************************************************/
      /** Gestion des fentres GraphicHGR Text80 de l'application **/
      /*************************************************************/
      case MEMORY_ADD_WINDOW_GRAPHICHGRTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphichgrtext80_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_graphichgrtext80 = (struct window_graphichgrtext80 *) data;
        if(first_window_graphichgrtext80 == NULL)
          first_window_graphichgrtext80 = current_window_graphichgrtext80;
        else
          last_window_graphichgrtext80->next = current_window_graphichgrtext80;
        last_window_graphichgrtext80 = current_window_graphichgrtext80;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphichgrtext80_threadhandle);
        break;

      case MEMORY_GET_WINDOW_GRAPHICHGRTEXT80 :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphichgrtext80_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_graphichgrtext80 **) data) = NULL;
        for(current_window_graphichgrtext80 = first_window_graphichgrtext80; current_window_graphichgrtext80; current_window_graphichgrtext80 = current_window_graphichgrtext80->next)
          if(current_window_graphichgrtext80->thread_id == thread_id)
            *((struct window_graphichgrtext80 **) data) = current_window_graphichgrtext80;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphichgrtext80_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_GRAPHICHGRTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_graphichgrtext80_threadhandle); 

        /* Structure  Supprimer */
        delete_window_graphichgrtext80 = (struct window_graphichgrtext80 *) data;

        /** Supprime la structure GraphicHGR Text80 de la liste **/
        if(first_window_graphichgrtext80 == delete_window_graphichgrtext80 && last_window_graphichgrtext80 == delete_window_graphichgrtext80)
          {
            /* Structure unique */
            first_window_graphichgrtext80 = NULL;
            last_window_graphichgrtext80 = NULL;

            /* Libration mmoire */
            mem_free_window_graphichgrtext80(delete_window_graphichgrtext80);
          }
        else if(first_window_graphichgrtext80 == delete_window_graphichgrtext80)
          {
            /* Premire Structure */
            first_window_graphichgrtext80 = delete_window_graphichgrtext80->next;

            /* Libration mmoire */
            mem_free_window_graphichgrtext80(delete_window_graphichgrtext80);
          }
        else if(last_window_graphichgrtext80 == delete_window_graphichgrtext80)
          {
            /* Dernire Structure */
            for(current_window_graphichgrtext80 = first_window_graphichgrtext80; current_window_graphichgrtext80; current_window_graphichgrtext80 = current_window_graphichgrtext80->next)
              if(current_window_graphichgrtext80->next == delete_window_graphichgrtext80)
                {
                  last_window_graphichgrtext80 = current_window_graphichgrtext80;
                  current_window_graphichgrtext80->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_graphichgrtext80(delete_window_graphichgrtext80);
          }
        else
          {
            /* Au milieu */
            for(current_window_graphichgrtext80 = first_window_graphichgrtext80; current_window_graphichgrtext80; current_window_graphichgrtext80 = current_window_graphichgrtext80->next)
              if(current_window_graphichgrtext80->next == delete_window_graphichgrtext80)
                {
                  current_window_graphichgrtext80->next = delete_window_graphichgrtext80->next;

                  /* Libration mmoire */
                  mem_free_window_graphichgrtext80(delete_window_graphichgrtext80);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphichgrtext80_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICHGRTEXT80 :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_graphichgrtext80 = first_window_graphichgrtext80; current_window_graphichgrtext80; current_window_graphichgrtext80 = current_window_graphichgrtext80->next)
          tab_window_handle[nb_window_handle++] = current_window_graphichgrtext80->hwnd_graphichgrtext80;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_graphichgrtext80 != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICHGRTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphichgrtext80_threadhandle); 

        /** Supprime la liste des GraphicHGR Text80 **/
        for(current_window_graphichgrtext80 = first_window_graphichgrtext80; current_window_graphichgrtext80; current_window_graphichgrtext80 = next_window_graphichgrtext80)
          {
            next_window_graphichgrtext80 = current_window_graphichgrtext80->next;
            mem_free_window_graphichgrtext80(current_window_graphichgrtext80);
          }
        first_window_graphichgrtext80 = NULL;
        last_window_graphichgrtext80 = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphichgrtext80_threadhandle);
        break;

      /************************************************************/
      /** Gestion des fentres GraphicGR Text40 de l'application **/
      /************************************************************/
      case MEMORY_ADD_WINDOW_GRAPHICGRTEXT40 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicgrtext40_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_graphicgrtext40 = (struct window_graphicgrtext40 *) data;
        if(first_window_graphicgrtext40 == NULL)
          first_window_graphicgrtext40 = current_window_graphicgrtext40;
        else
          last_window_graphicgrtext40->next = current_window_graphicgrtext40;
        last_window_graphicgrtext40 = current_window_graphicgrtext40;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicgrtext40_threadhandle);
        break;

      case MEMORY_GET_WINDOW_GRAPHICGRTEXT40 :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicgrtext40_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_graphicgrtext40 **) data) = NULL;
        for(current_window_graphicgrtext40 = first_window_graphicgrtext40; current_window_graphicgrtext40; current_window_graphicgrtext40 = current_window_graphicgrtext40->next)
          if(current_window_graphicgrtext40->thread_id == thread_id)
            *((struct window_graphicgrtext40 **) data) = current_window_graphicgrtext40;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicgrtext40_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_GRAPHICGRTEXT40 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_graphicgrtext40_threadhandle); 

        /* Structure  Supprimer */
        delete_window_graphicgrtext40 = (struct window_graphicgrtext40 *) data;

        /** Supprime la structure GraphicGR Text40 de la liste **/
        if(first_window_graphicgrtext40 == delete_window_graphicgrtext40 && last_window_graphicgrtext40 == delete_window_graphicgrtext40)
          {
            /* Structure unique */
            first_window_graphicgrtext40 = NULL;
            last_window_graphicgrtext40 = NULL;

            /* Libration mmoire */
            mem_free_window_graphicgrtext40(delete_window_graphicgrtext40);
          }
        else if(first_window_graphicgrtext40 == delete_window_graphicgrtext40)
          {
            /* Premire Structure */
            first_window_graphicgrtext40 = delete_window_graphicgrtext40->next;

            /* Libration mmoire */
            mem_free_window_graphicgrtext40(delete_window_graphicgrtext40);
          }
        else if(last_window_graphicgrtext40 == delete_window_graphicgrtext40)
          {
            /* Dernire Structure */
            for(current_window_graphicgrtext40 = first_window_graphicgrtext40; current_window_graphicgrtext40; current_window_graphicgrtext40 = current_window_graphicgrtext40->next)
              if(current_window_graphicgrtext40->next == delete_window_graphicgrtext40)
                {
                  last_window_graphicgrtext40 = current_window_graphicgrtext40;
                  current_window_graphicgrtext40->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_graphicgrtext40(delete_window_graphicgrtext40);
          }
        else
          {
            /* Au milieu */
            for(current_window_graphicgrtext40 = first_window_graphicgrtext40; current_window_graphicgrtext40; current_window_graphicgrtext40 = current_window_graphicgrtext40->next)
              if(current_window_graphicgrtext40->next == delete_window_graphicgrtext40)
                {
                  current_window_graphicgrtext40->next = delete_window_graphicgrtext40->next;

                  /* Libration mmoire */
                  mem_free_window_graphicgrtext40(delete_window_graphicgrtext40);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicgrtext40_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICGRTEXT40 :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_graphicgrtext40 = first_window_graphicgrtext40; current_window_graphicgrtext40; current_window_graphicgrtext40 = current_window_graphicgrtext40->next)
          tab_window_handle[nb_window_handle++] = current_window_graphicgrtext40->hwnd_graphicgrtext40;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_graphicgrtext40 != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICGRTEXT40 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicgrtext40_threadhandle); 

        /** Supprime la liste des GraphicGR Text40 **/
        for(current_window_graphicgrtext40 = first_window_graphicgrtext40; current_window_graphicgrtext40; current_window_graphicgrtext40 = next_window_graphicgrtext40)
          {
            next_window_graphicgrtext40 = current_window_graphicgrtext40->next;
            mem_free_window_graphicgrtext40(current_window_graphicgrtext40);
          }
        first_window_graphicgrtext40 = NULL;
        last_window_graphicgrtext40 = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicgrtext40_threadhandle);
        break;

      /************************************************************/
      /** Gestion des fentres GraphicGR Text80 de l'application **/
      /************************************************************/
      case MEMORY_ADD_WINDOW_GRAPHICGRTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicgrtext80_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_graphicgrtext80 = (struct window_graphicgrtext80 *) data;
        if(first_window_graphicgrtext80 == NULL)
          first_window_graphicgrtext80 = current_window_graphicgrtext80;
        else
          last_window_graphicgrtext80->next = current_window_graphicgrtext80;
        last_window_graphicgrtext80 = current_window_graphicgrtext80;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicgrtext80_threadhandle);
        break;

      case MEMORY_GET_WINDOW_GRAPHICGRTEXT80 :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicgrtext80_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_graphicgrtext80 **) data) = NULL;
        for(current_window_graphicgrtext80 = first_window_graphicgrtext80; current_window_graphicgrtext80; current_window_graphicgrtext80 = current_window_graphicgrtext80->next)
          if(current_window_graphicgrtext80->thread_id == thread_id)
            *((struct window_graphicgrtext80 **) data) = current_window_graphicgrtext80;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicgrtext80_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_GRAPHICGRTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_graphicgrtext80_threadhandle); 

        /* Structure  Supprimer */
        delete_window_graphicgrtext80 = (struct window_graphicgrtext80 *) data;

        /** Supprime la structure GraphicGR Text80 de la liste **/
        if(first_window_graphicgrtext80 == delete_window_graphicgrtext80 && last_window_graphicgrtext80 == delete_window_graphicgrtext80)
          {
            /* Structure unique */
            first_window_graphicgrtext80 = NULL;
            last_window_graphicgrtext80 = NULL;

            /* Libration mmoire */
            mem_free_window_graphicgrtext80(delete_window_graphicgrtext80);
          }
        else if(first_window_graphicgrtext80 == delete_window_graphicgrtext80)
          {
            /* Premire Structure */
            first_window_graphicgrtext80 = delete_window_graphicgrtext80->next;

            /* Libration mmoire */
            mem_free_window_graphicgrtext80(delete_window_graphicgrtext80);
          }
        else if(last_window_graphicgrtext80 == delete_window_graphicgrtext80)
          {
            /* Dernire Structure */
            for(current_window_graphicgrtext80 = first_window_graphicgrtext80; current_window_graphicgrtext80; current_window_graphicgrtext80 = current_window_graphicgrtext80->next)
              if(current_window_graphicgrtext80->next == delete_window_graphicgrtext80)
                {
                  last_window_graphicgrtext80 = current_window_graphicgrtext80;
                  current_window_graphicgrtext80->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_graphicgrtext80(delete_window_graphicgrtext80);
          }
        else
          {
            /* Au milieu */
            for(current_window_graphicgrtext80 = first_window_graphicgrtext80; current_window_graphicgrtext80; current_window_graphicgrtext80 = current_window_graphicgrtext80->next)
              if(current_window_graphicgrtext80->next == delete_window_graphicgrtext80)
                {
                  current_window_graphicgrtext80->next = delete_window_graphicgrtext80->next;

                  /* Libration mmoire */
                  mem_free_window_graphicgrtext80(delete_window_graphicgrtext80);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicgrtext80_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICGRTEXT80 :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_graphicgrtext80 = first_window_graphicgrtext80; current_window_graphicgrtext80; current_window_graphicgrtext80 = current_window_graphicgrtext80->next)
          tab_window_handle[nb_window_handle++] = current_window_graphicgrtext80->hwnd_graphicgrtext80;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_graphicgrtext80 != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICGRTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicgrtext80_threadhandle); 

        /** Supprime la liste des GraphicGR Text80 **/
        for(current_window_graphicgrtext80 = first_window_graphicgrtext80; current_window_graphicgrtext80; current_window_graphicgrtext80 = next_window_graphicgrtext80)
          {
            next_window_graphicgrtext80 = current_window_graphicgrtext80->next;
            mem_free_window_graphicgrtext80(current_window_graphicgrtext80);
          }
        first_window_graphicgrtext80 = NULL;
        last_window_graphicgrtext80 = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicgrtext80_threadhandle);
        break;

      /*************************************************************/
      /** Gestion des fentres GraphicDGR Text80 de l'application **/
      /*************************************************************/
      case MEMORY_ADD_WINDOW_GRAPHICDGRTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdgrtext80_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_graphicdgrtext80 = (struct window_graphicdgrtext80 *) data;
        if(first_window_graphicdgrtext80 == NULL)
          first_window_graphicdgrtext80 = current_window_graphicdgrtext80;
        else
          last_window_graphicdgrtext80->next = current_window_graphicdgrtext80;
        last_window_graphicdgrtext80 = current_window_graphicdgrtext80;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdgrtext80_threadhandle);
        break;

      case MEMORY_GET_WINDOW_GRAPHICDGRTEXT80 :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdgrtext80_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_graphicdgrtext80 **) data) = NULL;
        for(current_window_graphicdgrtext80 = first_window_graphicdgrtext80; current_window_graphicdgrtext80; current_window_graphicdgrtext80 = current_window_graphicdgrtext80->next)
          if(current_window_graphicdgrtext80->thread_id == thread_id)
            *((struct window_graphicdgrtext80 **) data) = current_window_graphicdgrtext80;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdgrtext80_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_GRAPHICDGRTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_graphicdgrtext80_threadhandle); 

        /* Structure  Supprimer */
        delete_window_graphicdgrtext80 = (struct window_graphicdgrtext80 *) data;

        /** Supprime la structure GraphicDGR Text80 de la liste **/
        if(first_window_graphicdgrtext80 == delete_window_graphicdgrtext80 && last_window_graphicdgrtext80 == delete_window_graphicdgrtext80)
          {
            /* Structure unique */
            first_window_graphicdgrtext80 = NULL;
            last_window_graphicdgrtext80 = NULL;

            /* Libration mmoire */
            mem_free_window_graphicdgrtext80(delete_window_graphicdgrtext80);
          }
        else if(first_window_graphicdgrtext80 == delete_window_graphicdgrtext80)
          {
            /* Premire Structure */
            first_window_graphicdgrtext80 = delete_window_graphicdgrtext80->next;

            /* Libration mmoire */
            mem_free_window_graphicdgrtext80(delete_window_graphicdgrtext80);
          }
        else if(last_window_graphicdgrtext80 == delete_window_graphicdgrtext80)
          {
            /* Dernire Structure */
            for(current_window_graphicdgrtext80 = first_window_graphicdgrtext80; current_window_graphicdgrtext80; current_window_graphicdgrtext80 = current_window_graphicdgrtext80->next)
              if(current_window_graphicdgrtext80->next == delete_window_graphicdgrtext80)
                {
                  last_window_graphicdgrtext80 = current_window_graphicdgrtext80;
                  current_window_graphicdgrtext80->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_graphicdgrtext80(delete_window_graphicdgrtext80);
          }
        else
          {
            /* Au milieu */
            for(current_window_graphicdgrtext80 = first_window_graphicdgrtext80; current_window_graphicdgrtext80; current_window_graphicdgrtext80 = current_window_graphicdgrtext80->next)
              if(current_window_graphicdgrtext80->next == delete_window_graphicdgrtext80)
                {
                  current_window_graphicdgrtext80->next = delete_window_graphicdgrtext80->next;

                  /* Libration mmoire */
                  mem_free_window_graphicdgrtext80(delete_window_graphicdgrtext80);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdgrtext80_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICDGRTEXT80 :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_graphicdgrtext80 = first_window_graphicdgrtext80; current_window_graphicdgrtext80; current_window_graphicdgrtext80 = current_window_graphicdgrtext80->next)
          tab_window_handle[nb_window_handle++] = current_window_graphicdgrtext80->hwnd_graphicdgrtext80;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_graphicdgrtext80 != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICDGRTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdgrtext80_threadhandle); 

        /** Supprime la liste des GraphicDGR Text80 **/
        for(current_window_graphicdgrtext80 = first_window_graphicdgrtext80; current_window_graphicdgrtext80; current_window_graphicdgrtext80 = next_window_graphicdgrtext80)
          {
            next_window_graphicdgrtext80 = current_window_graphicdgrtext80->next;
            mem_free_window_graphicdgrtext80(current_window_graphicdgrtext80);
          }
        first_window_graphicdgrtext80 = NULL;
        last_window_graphicdgrtext80 = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdgrtext80_threadhandle);
        break;

      /*******************************************************************/
      /** Gestion des fentres GraphicDHGR Mono Text80 de l'application **/
      /*******************************************************************/
      case MEMORY_ADD_WINDOW_GRAPHICDHGRMONOTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdhgrmonotext80_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_graphicdhgrmonotext80 = (struct window_graphicdhgrmonotext80 *) data;
        if(first_window_graphicdhgrmonotext80 == NULL)
          first_window_graphicdhgrmonotext80 = current_window_graphicdhgrmonotext80;
        else
          last_window_graphicdhgrmonotext80->next = current_window_graphicdhgrmonotext80;
        last_window_graphicdhgrmonotext80 = current_window_graphicdhgrmonotext80;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrmonotext80_threadhandle);
        break;

      case MEMORY_GET_WINDOW_GRAPHICDHGRMONOTEXT80 :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdhgrmonotext80_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_graphicdhgrmonotext80 **) data) = NULL;
        for(current_window_graphicdhgrmonotext80 = first_window_graphicdhgrmonotext80; current_window_graphicdhgrmonotext80; current_window_graphicdhgrmonotext80 = current_window_graphicdhgrmonotext80->next)
          if(current_window_graphicdhgrmonotext80->thread_id == thread_id)
            *((struct window_graphicdhgrmonotext80 **) data) = current_window_graphicdhgrmonotext80;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrmonotext80_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_GRAPHICDHGRMONOTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_graphicdhgrmonotext80_threadhandle); 

        /* Structure  Supprimer */
        delete_window_graphicdhgrmonotext80 = (struct window_graphicdhgrmonotext80 *) data;

        /** Supprime la structure GraphicDHGRMono Text80 de la liste **/
        if(first_window_graphicdhgrmonotext80 == delete_window_graphicdhgrmonotext80 && last_window_graphicdhgrmonotext80 == delete_window_graphicdhgrmonotext80)
          {
            /* Structure unique */
            first_window_graphicdhgrmonotext80 = NULL;
            last_window_graphicdhgrmonotext80 = NULL;

            /* Libration mmoire */
            mem_free_window_graphicdhgrmonotext80(delete_window_graphicdhgrmonotext80);
          }
        else if(first_window_graphicdhgrmonotext80 == delete_window_graphicdhgrmonotext80)
          {
            /* Premire Structure */
            first_window_graphicdhgrmonotext80 = delete_window_graphicdhgrmonotext80->next;

            /* Libration mmoire */
            mem_free_window_graphicdhgrmonotext80(delete_window_graphicdhgrmonotext80);
          }
        else if(last_window_graphicdhgrmonotext80 == delete_window_graphicdhgrmonotext80)
          {
            /* Dernire Structure */
            for(current_window_graphicdhgrmonotext80 = first_window_graphicdhgrmonotext80; current_window_graphicdhgrmonotext80; current_window_graphicdhgrmonotext80 = current_window_graphicdhgrmonotext80->next)
              if(current_window_graphicdhgrmonotext80->next == delete_window_graphicdhgrmonotext80)
                {
                  last_window_graphicdhgrmonotext80 = current_window_graphicdhgrmonotext80;
                  current_window_graphicdhgrmonotext80->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_graphicdhgrmonotext80(delete_window_graphicdhgrmonotext80);
          }
        else
          {
            /* Au milieu */
            for(current_window_graphicdhgrmonotext80 = first_window_graphicdhgrmonotext80; current_window_graphicdhgrmonotext80; current_window_graphicdhgrmonotext80 = current_window_graphicdhgrmonotext80->next)
              if(current_window_graphicdhgrmonotext80->next == delete_window_graphicdhgrmonotext80)
                {
                  current_window_graphicdhgrmonotext80->next = delete_window_graphicdhgrmonotext80->next;

                  /* Libration mmoire */
                  mem_free_window_graphicdhgrmonotext80(delete_window_graphicdhgrmonotext80);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrmonotext80_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICDHGRMONOTEXT80 :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_graphicdhgrmonotext80 = first_window_graphicdhgrmonotext80; current_window_graphicdhgrmonotext80; current_window_graphicdhgrmonotext80 = current_window_graphicdhgrmonotext80->next)
          tab_window_handle[nb_window_handle++] = current_window_graphicdhgrmonotext80->hwnd_graphicdhgrmonotext80;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_graphicdhgrmonotext80 != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICDHGRMONOTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdhgrmonotext80_threadhandle); 

        /** Supprime la liste des GraphicDHGRMono Text80 **/
        for(current_window_graphicdhgrmonotext80 = first_window_graphicdhgrmonotext80; current_window_graphicdhgrmonotext80; current_window_graphicdhgrmonotext80 = next_window_graphicdhgrmonotext80)
          {
            next_window_graphicdhgrmonotext80 = current_window_graphicdhgrmonotext80->next;
            mem_free_window_graphicdhgrmonotext80(current_window_graphicdhgrmonotext80);
          }
        first_window_graphicdhgrmonotext80 = NULL;
        last_window_graphicdhgrmonotext80 = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrmonotext80_threadhandle);
        break;


      /********************************************************************/
      /** Gestion des fentres GraphicDHGR Color Text80 de l'application **/
      /********************************************************************/
      case MEMORY_ADD_WINDOW_GRAPHICDHGRCOLORTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdhgrcolortext80_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_graphicdhgrcolortext80 = (struct window_graphicdhgrcolortext80 *) data;
        if(first_window_graphicdhgrcolortext80 == NULL)
          first_window_graphicdhgrcolortext80 = current_window_graphicdhgrcolortext80;
        else
          last_window_graphicdhgrcolortext80->next = current_window_graphicdhgrcolortext80;
        last_window_graphicdhgrcolortext80 = current_window_graphicdhgrcolortext80;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrcolortext80_threadhandle);
        break;

      case MEMORY_GET_WINDOW_GRAPHICDHGRCOLORTEXT80 :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdhgrcolortext80_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_graphicdhgrcolortext80 **) data) = NULL;
        for(current_window_graphicdhgrcolortext80 = first_window_graphicdhgrcolortext80; current_window_graphicdhgrcolortext80; current_window_graphicdhgrcolortext80 = current_window_graphicdhgrcolortext80->next)
          if(current_window_graphicdhgrcolortext80->thread_id == thread_id)
            *((struct window_graphicdhgrcolortext80 **) data) = current_window_graphicdhgrcolortext80;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrcolortext80_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_GRAPHICDHGRCOLORTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_graphicdhgrcolortext80_threadhandle); 

        /* Structure  Supprimer */
        delete_window_graphicdhgrcolortext80 = (struct window_graphicdhgrcolortext80 *) data;

        /** Supprime la structure GraphicDHGRColor Text80 de la liste **/
        if(first_window_graphicdhgrcolortext80 == delete_window_graphicdhgrcolortext80 && last_window_graphicdhgrcolortext80 == delete_window_graphicdhgrcolortext80)
          {
            /* Structure unique */
            first_window_graphicdhgrcolortext80 = NULL;
            last_window_graphicdhgrcolortext80 = NULL;

            /* Libration mmoire */
            mem_free_window_graphicdhgrcolortext80(delete_window_graphicdhgrcolortext80);
          }
        else if(first_window_graphicdhgrcolortext80 == delete_window_graphicdhgrcolortext80)
          {
            /* Premire Structure */
            first_window_graphicdhgrcolortext80 = delete_window_graphicdhgrcolortext80->next;

            /* Libration mmoire */
            mem_free_window_graphicdhgrcolortext80(delete_window_graphicdhgrcolortext80);
          }
        else if(last_window_graphicdhgrcolortext80 == delete_window_graphicdhgrcolortext80)
          {
            /* Dernire Structure */
            for(current_window_graphicdhgrcolortext80 = first_window_graphicdhgrcolortext80; current_window_graphicdhgrcolortext80; current_window_graphicdhgrcolortext80 = current_window_graphicdhgrcolortext80->next)
              if(current_window_graphicdhgrcolortext80->next == delete_window_graphicdhgrcolortext80)
                {
                  last_window_graphicdhgrcolortext80 = current_window_graphicdhgrcolortext80;
                  current_window_graphicdhgrcolortext80->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_graphicdhgrcolortext80(delete_window_graphicdhgrcolortext80);
          }
        else
          {
            /* Au milieu */
            for(current_window_graphicdhgrcolortext80 = first_window_graphicdhgrcolortext80; current_window_graphicdhgrcolortext80; current_window_graphicdhgrcolortext80 = current_window_graphicdhgrcolortext80->next)
              if(current_window_graphicdhgrcolortext80->next == delete_window_graphicdhgrcolortext80)
                {
                  current_window_graphicdhgrcolortext80->next = delete_window_graphicdhgrcolortext80->next;

                  /* Libration mmoire */
                  mem_free_window_graphicdhgrcolortext80(delete_window_graphicdhgrcolortext80);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrcolortext80_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICDHGRCOLORTEXT80 :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_graphicdhgrcolortext80 = first_window_graphicdhgrcolortext80; current_window_graphicdhgrcolortext80; current_window_graphicdhgrcolortext80 = current_window_graphicdhgrcolortext80->next)
          tab_window_handle[nb_window_handle++] = current_window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_graphicdhgrcolortext80 != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICDHGRCOLORTEXT80 :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicdhgrcolortext80_threadhandle); 

        /** Supprime la liste des GraphicDHGRColor Text80 **/
        for(current_window_graphicdhgrcolortext80 = first_window_graphicdhgrcolortext80; current_window_graphicdhgrcolortext80; current_window_graphicdhgrcolortext80 = next_window_graphicdhgrcolortext80)
          {
            next_window_graphicdhgrcolortext80 = current_window_graphicdhgrcolortext80->next;
            mem_free_window_graphicdhgrcolortext80(current_window_graphicdhgrcolortext80);
          }
        first_window_graphicdhgrcolortext80 = NULL;
        last_window_graphicdhgrcolortext80 = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicdhgrcolortext80_threadhandle);
        break;

      /******************************************************/
      /** Gestion des fentres GraphicSHR de l'application **/
      /******************************************************/
      case MEMORY_ADD_WINDOW_GRAPHICSHR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicshr_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_graphicshr = (struct window_graphicshr *) data;
        if(first_window_graphicshr == NULL)
          first_window_graphicshr = current_window_graphicshr;
        else
          last_window_graphicshr->next = current_window_graphicshr;
        last_window_graphicshr = current_window_graphicshr;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicshr_threadhandle);
        break;

      case MEMORY_GET_WINDOW_GRAPHICSHR :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicshr_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_graphicshr **) data) = NULL;
        for(current_window_graphicshr = first_window_graphicshr; current_window_graphicshr; current_window_graphicshr = current_window_graphicshr->next)
          if(current_window_graphicshr->thread_id == thread_id)
            *((struct window_graphicshr **) data) = current_window_graphicshr;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicshr_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_GRAPHICSHR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_graphicshr_threadhandle); 

        /* Structure  Supprimer */
        delete_window_graphicshr = (struct window_graphicshr *) data;

        /** Supprime la structure GraphicSHR de la liste **/
        if(first_window_graphicshr == delete_window_graphicshr && last_window_graphicshr == delete_window_graphicshr)
          {
            /* Structure unique */
            first_window_graphicshr = NULL;
            last_window_graphicshr = NULL;

            /* Libration mmoire */
            mem_free_window_graphicshr(delete_window_graphicshr);
          }
        else if(first_window_graphicshr == delete_window_graphicshr)
          {
            /* Premire Structure */
            first_window_graphicshr = delete_window_graphicshr->next;

            /* Libration mmoire */
            mem_free_window_graphicshr(delete_window_graphicshr);
          }
        else if(last_window_graphicshr == delete_window_graphicshr)
          {
            /* Dernire Structure */
            for(current_window_graphicshr = first_window_graphicshr; current_window_graphicshr; current_window_graphicshr = current_window_graphicshr->next)
              if(current_window_graphicshr->next == delete_window_graphicshr)
                {
                  last_window_graphicshr = current_window_graphicshr;
                  current_window_graphicshr->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_graphicshr(delete_window_graphicshr);
          }
        else
          {
            /* Au milieu */
            for(current_window_graphicshr = first_window_graphicshr; current_window_graphicshr; current_window_graphicshr = current_window_graphicshr->next)
              if(current_window_graphicshr->next == delete_window_graphicshr)
                {
                  current_window_graphicshr->next = delete_window_graphicshr->next;

                  /* Libration mmoire */
                  mem_free_window_graphicshr(delete_window_graphicshr);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicshr_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICSHR :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_graphicshr = first_window_graphicshr; current_window_graphicshr; current_window_graphicshr = current_window_graphicshr->next)
          tab_window_handle[nb_window_handle++] = current_window_graphicshr->hwnd_graphicshr;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_graphicshr != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICSHR :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_graphicshr_threadhandle); 

        /** Supprime la liste des GraphicSHR **/
        for(current_window_graphicshr = first_window_graphicshr; current_window_graphicshr; current_window_graphicshr = next_window_graphicshr)
          {
            next_window_graphicshr = current_window_graphicshr->next;
            mem_free_window_graphicshr(current_window_graphicshr);
          }
        first_window_graphicshr = NULL;
        last_window_graphicshr = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_graphicshr_threadhandle);
        break;

      /********************************************************/
      /** Gestion des fentres Disassembler de l'application **/
      /********************************************************/
      case MEMORY_ADD_WINDOW_DISASSEMBLER :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_disassembler_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_disassembler = (struct window_disassembler *) data;
        if(first_window_disassembler == NULL)
          first_window_disassembler = current_window_disassembler;
        else
          last_window_disassembler->next = current_window_disassembler;
        last_window_disassembler = current_window_disassembler;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_disassembler_threadhandle);
        break;

      case MEMORY_GET_WINDOW_DISASSEMBLER :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_disassembler_threadhandle); 

        /** Identifie une structure Memory Area via le ThreadID **/
        *((struct window_disassembler **) data) = NULL;
        for(current_window_disassembler = first_window_disassembler; current_window_disassembler; current_window_disassembler = current_window_disassembler->next)
          if(current_window_disassembler->thread_id == thread_id)
            *((struct window_disassembler **) data) = current_window_disassembler;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_disassembler_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_DISASSEMBLER :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_disassembler_threadhandle); 

        /* Structure  Supprimer */
        delete_window_disassembler = (struct window_disassembler *) data;

        /** Supprime la structure Disassembler de la liste **/
        if(first_window_disassembler == delete_window_disassembler && last_window_disassembler == delete_window_disassembler)
          {
            /* Structure unique */
            first_window_disassembler = NULL;
            last_window_disassembler = NULL;

            /* Libration mmoire */
            mem_free_window_disassembler(delete_window_disassembler);
          }
        else if(first_window_disassembler == delete_window_disassembler)
          {
            /* Premire Structure */
            first_window_disassembler = delete_window_disassembler->next;

            /* Libration mmoire */
            mem_free_window_disassembler(delete_window_disassembler);
          }
        else if(last_window_disassembler == delete_window_disassembler)
          {
            /* Dernire Structure */
            for(current_window_disassembler = first_window_disassembler; current_window_disassembler; current_window_disassembler = current_window_disassembler->next)
              if(current_window_disassembler->next == delete_window_disassembler)
                {
                  last_window_disassembler = current_window_disassembler;
                  current_window_disassembler->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_disassembler(delete_window_disassembler);
          }
        else
          {
            /* Au milieu */
            for(current_window_disassembler = first_window_disassembler; current_window_disassembler; current_window_disassembler = current_window_disassembler->next)
              if(current_window_disassembler->next == delete_window_disassembler)
                {
                  current_window_disassembler->next = delete_window_disassembler->next;

                  /* Libration mmoire */
                  mem_free_window_disassembler(delete_window_disassembler);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_disassembler_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_DISASSEMBLER :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_disassembler = first_window_disassembler; current_window_disassembler; current_window_disassembler = current_window_disassembler->next)
          tab_window_handle[nb_window_handle++] = current_window_disassembler->hwnd_disassembler;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_disassembler != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_DISASSEMBLER :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_disassembler_threadhandle); 

        /** Supprime la liste des Disassembler **/
        for(current_window_disassembler = first_window_disassembler; current_window_disassembler; current_window_disassembler = next_window_disassembler)
          {
            next_window_disassembler = current_window_disassembler->next;
            mem_free_window_disassembler(current_window_disassembler);
          }
        first_window_disassembler = NULL;
        last_window_disassembler = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_disassembler_threadhandle);
        break;
        
      /*******************************************************/
      /** Gestion de la fentre Memory Map de l'application **/
      /*******************************************************/
      case MEMORY_SET_WINDOW_MEMORYMAP :
        window_memorymap = (struct window_memorymap *) data;
        break;

      case MEMORY_GET_WINDOW_MEMORYMAP :
        *((struct window_memorymap **) data) = window_memorymap;
        break;

      case MEMORY_CLOSE_WINDOW_MEMORYMAP :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_memorymap == NULL) ? NULL : window_memorymap->hwnd_memorymap;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_memorymap != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_MEMORYMAP :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_memorymap(window_memorymap);
        window_memorymap = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;
        
      /*********************************************************/
      /** Gestion de la fentre DOC Register de l'application **/
      /*********************************************************/
      case MEMORY_SET_WINDOW_DOCREGISTER :
        window_docregister = (struct window_docregister *) data;
        break;

      case MEMORY_GET_WINDOW_DOCREGISTER :
        *((struct window_docregister **) data) = window_docregister;
        break;

      case MEMORY_CLOSE_WINDOW_DOCREGISTER :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_docregister == NULL) ? NULL : window_docregister->hwnd_docregister;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_docregister != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_DOCREGISTER :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_docregister(window_docregister);
        window_docregister = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      /***************************************************/
      /** Gestion de la fentre Search de l'application **/
      /***************************************************/
      case MEMORY_SET_WINDOW_SEARCH :
        window_search = (struct window_search *) data;
        break;

      case MEMORY_GET_WINDOW_SEARCH :
        *((struct window_search **) data) = window_search;
        break;

      case MEMORY_CLOSE_WINDOW_SEARCH :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_search == NULL) ? NULL : window_search->hwnd_search;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_search != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_SEARCH :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_search(window_search);
        window_search = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      /****************************************************/
      /** Gestion de la fentre Replace de l'application **/
      /****************************************************/
      case MEMORY_SET_WINDOW_REPLACE :
        window_replace = (struct window_replace *) data;
        break;

      case MEMORY_GET_WINDOW_REPLACE :
        *((struct window_replace **) data) = window_replace;
        break;

      case MEMORY_CLOSE_WINDOW_REPLACE :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_replace == NULL) ? NULL : window_replace->hwnd_replace;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_replace != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_REPLACE :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_replace(window_replace);
        window_replace = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;
        
        
      /*****************************************************/
      /** Gestion des fentres Load File de l'application **/
      /*****************************************************/
      case MEMORY_ADD_WINDOW_LOADFILE :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_loadfile_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_loadfile = (struct window_loadfile *) data;
        if(first_window_loadfile == NULL)
          first_window_loadfile = current_window_loadfile;
        else
          last_window_loadfile->next = current_window_loadfile;
        last_window_loadfile = current_window_loadfile;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_loadfile_threadhandle);
        break;

      case MEMORY_GET_WINDOW_LOADFILE :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_loadfile_threadhandle); 

        /** Identifie une structure Load File via le ThreadID **/
        *((struct window_loadfile **) data) = NULL;
        for(current_window_loadfile = first_window_loadfile; current_window_loadfile; current_window_loadfile = current_window_loadfile->next)
          if(current_window_loadfile->thread_id == thread_id)
            *((struct window_loadfile **) data) = current_window_loadfile;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_loadfile_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_LOADFILE :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_loadfile_threadhandle); 

        /* Structure  Supprimer */
        delete_window_loadfile = (struct window_loadfile *) data;

        /** Supprime la structure Load File de la liste **/
        if(first_window_loadfile == delete_window_loadfile && last_window_loadfile == delete_window_loadfile)
          {
            /* Structure unique */
            first_window_loadfile = NULL;
            last_window_loadfile = NULL;

            /* Libration mmoire */
            mem_free_window_loadfile(delete_window_loadfile);
          }
        else if(first_window_loadfile == delete_window_loadfile)
          {
            /* Premire Structure */
            first_window_loadfile = delete_window_loadfile->next;

            /* Libration mmoire */
            mem_free_window_loadfile(delete_window_loadfile);
          }
        else if(last_window_loadfile == delete_window_loadfile)
          {
            /* Dernire Structure */
            for(current_window_loadfile = first_window_loadfile; current_window_loadfile; current_window_loadfile = current_window_loadfile->next)
              if(current_window_loadfile->next == delete_window_loadfile)
                {
                  last_window_loadfile = current_window_loadfile;
                  current_window_loadfile->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_loadfile(delete_window_loadfile);
          }
        else
          {
            /* Au milieu */
            for(current_window_loadfile = first_window_loadfile; current_window_loadfile; current_window_loadfile = current_window_loadfile->next)
              if(current_window_loadfile->next == delete_window_loadfile)
                {
                  current_window_loadfile->next = delete_window_loadfile->next;

                  /* Libration mmoire */
                  mem_free_window_loadfile(delete_window_loadfile);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_loadfile_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_LOADFILE :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_loadfile = first_window_loadfile; current_window_loadfile; current_window_loadfile = current_window_loadfile->next)
          tab_window_handle[nb_window_handle++] = current_window_loadfile->hwnd_loadfile;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_loadfile != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_LOADFILE :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_loadfile_threadhandle); 

        /** Supprime la liste des Load File **/
        for(current_window_loadfile = first_window_loadfile; current_window_loadfile; current_window_loadfile = next_window_loadfile)
          {
            next_window_loadfile = current_window_loadfile->next;
            mem_free_window_loadfile(current_window_loadfile);
          }
        first_window_loadfile = NULL;
        last_window_loadfile = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_loadfile_threadhandle);
        break;

      /*******************************************************/
      /** Gestion des fentres Save Memory de l'application **/
      /*******************************************************/
      case MEMORY_ADD_WINDOW_SAVEMEMORY :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_savememory_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_savememory = (struct window_savememory *) data;
        if(first_window_savememory == NULL)
          first_window_savememory = current_window_savememory;
        else
          last_window_savememory->next = current_window_savememory;
        last_window_savememory = current_window_savememory;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_savememory_threadhandle);
        break;

      case MEMORY_GET_WINDOW_SAVEMEMORY :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_savememory_threadhandle); 

        /** Identifie une structure Save Memory via le ThreadID **/
        *((struct window_savememory **) data) = NULL;
        for(current_window_savememory = first_window_savememory; current_window_savememory; current_window_savememory = current_window_savememory->next)
          if(current_window_savememory->thread_id == thread_id)
            *((struct window_savememory **) data) = current_window_savememory;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_savememory_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_SAVEMEMORY :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_savememory_threadhandle); 

        /* Structure  Supprimer */
        delete_window_savememory = (struct window_savememory *) data;

        /** Supprime la structure Save Memory de la liste **/
        if(first_window_savememory == delete_window_savememory && last_window_savememory == delete_window_savememory)
          {
            /* Structure unique */
            first_window_savememory = NULL;
            last_window_savememory = NULL;

            /* Libration mmoire */
            mem_free_window_savememory(delete_window_savememory);
          }
        else if(first_window_savememory == delete_window_savememory)
          {
            /* Premire Structure */
            first_window_savememory = delete_window_savememory->next;

            /* Libration mmoire */
            mem_free_window_savememory(delete_window_savememory);
          }
        else if(last_window_savememory == delete_window_savememory)
          {
            /* Dernire Structure */
            for(current_window_savememory = first_window_savememory; current_window_savememory; current_window_savememory = current_window_savememory->next)
              if(current_window_savememory->next == delete_window_savememory)
                {
                  last_window_savememory = current_window_savememory;
                  current_window_savememory->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_savememory(delete_window_savememory);
          }
        else
          {
            /* Au milieu */
            for(current_window_savememory = first_window_savememory; current_window_savememory; current_window_savememory = current_window_savememory->next)
              if(current_window_savememory->next == delete_window_savememory)
                {
                  current_window_savememory->next = delete_window_savememory->next;

                  /* Libration mmoire */
                  mem_free_window_savememory(delete_window_savememory);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_savememory_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_SAVEMEMORY :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_savememory = first_window_savememory; current_window_savememory; current_window_savememory = current_window_savememory->next)
          tab_window_handle[nb_window_handle++] = current_window_savememory->hwnd_savememory;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_savememory != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_SAVEMEMORY :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_savememory_threadhandle); 

        /** Supprime la liste des Save Memory **/
        for(current_window_savememory = first_window_savememory; current_window_savememory; current_window_savememory = next_window_savememory)
          {
            next_window_savememory = current_window_savememory->next;
            mem_free_window_savememory(current_window_savememory);
          }
        first_window_savememory = NULL;
        last_window_savememory = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_savememory_threadhandle);
        break;

      /***************************************************/
      /** Gestion des fentres Compare de l'application **/
      /***************************************************/
      case MEMORY_ADD_WINDOW_COMPARE :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_compare_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_compare = (struct window_compare *) data;
        if(first_window_compare == NULL)
          first_window_compare = current_window_compare;
        else
          last_window_compare->next = current_window_compare;
        last_window_compare = current_window_compare;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_compare_threadhandle);
        break;

      case MEMORY_GET_WINDOW_COMPARE :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_compare_threadhandle); 

        /** Identifie une structure Compare via le ThreadID **/
        *((struct window_compare **) data) = NULL;
        for(current_window_compare = first_window_compare; current_window_compare; current_window_compare = current_window_compare->next)
          if(current_window_compare->thread_id == thread_id)
            *((struct window_compare **) data) = current_window_compare;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_compare_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_COMPARE :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_compare_threadhandle); 

        /* Structure  Supprimer */
        delete_window_compare = (struct window_compare *) data;

        /** Supprime la structure Compare de la liste **/
        if(first_window_compare == delete_window_compare && last_window_compare == delete_window_compare)
          {
            /* Structure unique */
            first_window_compare = NULL;
            last_window_compare = NULL;

            /* Libration mmoire */
            mem_free_window_compare(delete_window_compare);
          }
        else if(first_window_compare == delete_window_compare)
          {
            /* Premire Structure */
            first_window_compare = delete_window_compare->next;

            /* Libration mmoire */
            mem_free_window_compare(delete_window_compare);
          }
        else if(last_window_compare == delete_window_compare)
          {
            /* Dernire Structure */
            for(current_window_compare = first_window_compare; current_window_compare; current_window_compare = current_window_compare->next)
              if(current_window_compare->next == delete_window_compare)
                {
                  last_window_compare = current_window_compare;
                  current_window_compare->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_compare(delete_window_compare);
          }
        else
          {
            /* Au milieu */
            for(current_window_compare = first_window_compare; current_window_compare; current_window_compare = current_window_compare->next)
              if(current_window_compare->next == delete_window_compare)
                {
                  current_window_compare->next = delete_window_compare->next;

                  /* Libration mmoire */
                  mem_free_window_compare(delete_window_compare);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_compare_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_COMPARE :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_compare = first_window_compare; current_window_compare; current_window_compare = current_window_compare->next)
          tab_window_handle[nb_window_handle++] = current_window_compare->hwnd_compare;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_compare != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_COMPARE :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_compare_threadhandle); 

        /** Supprime la liste des Compare **/
        for(current_window_compare = first_window_compare; current_window_compare; current_window_compare = next_window_compare)
          {
            next_window_compare = current_window_compare->next;
            mem_free_window_compare(current_window_compare);
          }
        first_window_compare = NULL;
        last_window_compare = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_compare_threadhandle);
        break;

      /*******************************************************/
      /** Gestion de la fentre QuickWatch de l'application **/
      /*******************************************************/
      case MEMORY_SET_WINDOW_QUICKWATCH :
        window_quickwatch = (struct window_quickwatch *) data;
        break;

      case MEMORY_GET_WINDOW_QUICKWATCH :
        *((struct window_quickwatch **) data) = window_quickwatch;
        break;

      case MEMORY_CLOSE_WINDOW_QUICKWATCH :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_quickwatch == NULL) ? NULL : window_quickwatch->hwnd_quickwatch;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_quickwatch != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_QUICKWATCH :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_quickwatch(window_quickwatch);
        window_quickwatch = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      /**************************************************/
      /** Gestion des fentres Modify de l'application **/
      /**************************************************/
      case MEMORY_ADD_WINDOW_MODIFY :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_modify_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_modify = (struct window_modify *) data;
        if(first_window_modify == NULL)
          first_window_modify = current_window_modify;
        else
          last_window_modify->next = current_window_modify;
        last_window_modify = current_window_modify;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_modify_threadhandle);
        break;

      case MEMORY_GET_WINDOW_MODIFY :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_modify_threadhandle); 

        /** Identifie une structure QuickWatch via le ThreadID **/
        *((struct window_modify **) data) = NULL;
        for(current_window_modify = first_window_modify; current_window_modify; current_window_modify = current_window_modify->next)
          if(current_window_modify->thread_id == thread_id)
            *((struct window_modify **) data) = current_window_modify;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_modify_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_MODIFY :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_modify_threadhandle); 

        /* Structure  Supprimer */
        delete_window_modify = (struct window_modify *) data;

        /** Supprime la structure Modify de la liste **/
        if(first_window_modify == delete_window_modify && last_window_modify == delete_window_modify)
          {
            /* Structure unique */
            first_window_modify = NULL;
            last_window_modify = NULL;

            /* Libration mmoire */
            mem_free_window_modify(delete_window_modify);
          }
        else if(first_window_modify == delete_window_modify)
          {
            /* Premire Structure */
            first_window_modify = delete_window_modify->next;

            /* Libration mmoire */
            mem_free_window_modify(delete_window_modify);
          }
        else if(last_window_modify == delete_window_modify)
          {
            /* Dernire Structure */
            for(current_window_modify = first_window_modify; current_window_modify; current_window_modify = current_window_modify->next)
              if(current_window_modify->next == delete_window_modify)
                {
                  last_window_modify = current_window_modify;
                  current_window_modify->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_modify(delete_window_modify);
          }
        else
          {
            /* Au milieu */
            for(current_window_modify = first_window_modify; current_window_modify; current_window_modify = current_window_modify->next)
              if(current_window_modify->next == delete_window_modify)
                {
                  current_window_modify->next = delete_window_modify->next;

                  /* Libration mmoire */
                  mem_free_window_modify(delete_window_modify);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_modify_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_MODIFY :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_modify = first_window_modify; current_window_modify; current_window_modify = current_window_modify->next)
          tab_window_handle[nb_window_handle++] = current_window_modify->hwnd_modify;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_modify != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_MODIFY :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_modify_threadhandle); 

        /** Supprime la liste des QuickWatch **/
        for(current_window_modify = first_window_modify; current_window_modify; current_window_modify = next_window_modify)
          {
            next_window_modify = current_window_modify->next;
            mem_free_window_modify(current_window_modify);
          }
        first_window_modify = NULL;
        last_window_modify = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_modify_threadhandle);
        break;

      /************************************************/
      /** Gestion des fentres Move de l'application **/
      /************************************************/
      case MEMORY_ADD_WINDOW_MOVE :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_move_threadhandle); 

        /* Ajoute la structure  la fin */
        current_window_move = (struct window_move *) data;
        if(first_window_move == NULL)
          first_window_move = current_window_move;
        else
          last_window_move->next = current_window_move;
        last_window_move = current_window_move;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_move_threadhandle);
        break;

      case MEMORY_GET_WINDOW_MOVE :
        /* ThreadID du thread demandant la fentre */
        thread_id = GetCurrentThreadId();

        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_move_threadhandle); 

        /** Identifie une structure QuickWatch via le ThreadID **/
        *((struct window_move **) data) = NULL;
        for(current_window_move = first_window_move; current_window_move; current_window_move = current_window_move->next)
          if(current_window_move->thread_id == thread_id)
            *((struct window_move **) data) = current_window_move;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_move_threadhandle);
        break;

      case MEMORY_DELETE_WINDOW_MOVE :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        EnterCriticalSection(&param->cs_move_threadhandle); 

        /* Structure  Supprimer */
        delete_window_move = (struct window_move *) data;

        /** Supprime la structure Move de la liste **/
        if(first_window_move == delete_window_move && last_window_move == delete_window_move)
          {
            /* Structure unique */
            first_window_move = NULL;
            last_window_move = NULL;

            /* Libration mmoire */
            mem_free_window_move(delete_window_move);
          }
        else if(first_window_move == delete_window_move)
          {
            /* Premire Structure */
            first_window_move = delete_window_move->next;

            /* Libration mmoire */
            mem_free_window_move(delete_window_move);
          }
        else if(last_window_move == delete_window_move)
          {
            /* Dernire Structure */
            for(current_window_move = first_window_move; current_window_move; current_window_move = current_window_move->next)
              if(current_window_move->next == delete_window_move)
                {
                  last_window_move = current_window_move;
                  current_window_move->next = NULL;
                  break;
                }

            /* Libration mmoire */
            mem_free_window_move(delete_window_move);
          }
        else
          {
            /* Au milieu */
            for(current_window_move = first_window_move; current_window_move; current_window_move = current_window_move->next)
              if(current_window_move->next == delete_window_move)
                {
                  current_window_move->next = delete_window_move->next;

                  /* Libration mmoire */
                  mem_free_window_move(delete_window_move);
                  break;
                }
          }

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_move_threadhandle);
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      case MEMORY_CLOSE_WINDOW_MOVE :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        for(nb_window_handle = 0, current_window_move = first_window_move; current_window_move; current_window_move = current_window_move->next)
          tab_window_handle[nb_window_handle++] = current_window_move->hwnd_move;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        for(i=0; i<nb_window_handle; i++)
          SendMessage(tab_window_handle[i],WM_CLOSE,0,0);
        while(first_window_move != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_MOVE :
        /* Accs unique  la zone mmoire */
        EnterCriticalSection(&param->cs_move_threadhandle); 

        /** Supprime la liste des QuickWatch **/
        for(current_window_move = first_window_move; current_window_move; current_window_move = next_window_move)
          {
            next_window_move = current_window_move->next;
            mem_free_window_move(current_window_move);
          }
        first_window_move = NULL;
        last_window_move = NULL;

        /* Fin de l'accs unique */
        LeaveCriticalSection(&param->cs_move_threadhandle);
        break;

      /***********************************************************/
      /** Gestion de la fentre Graphic Screen de l'application **/
      /***********************************************************/
      case MEMORY_SET_WINDOW_GRAPHICSCREEN :
        window_graphicscreen = (struct window_graphicscreen *) data;
        break;

      case MEMORY_GET_WINDOW_GRAPHICSCREEN :
        *((struct window_graphicscreen **) data) = window_graphicscreen;
        break;

      case MEMORY_CLOSE_WINDOW_GRAPHICSCREEN :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_graphicscreen == NULL) ? NULL : window_graphicscreen->hwnd_graphicscreen;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_graphicscreen != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_GRAPHICSCREEN :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_graphicscreen(window_graphicscreen);
        window_graphicscreen = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;


      /***************************************************/
      /** Gestion de la fentre Attach de l'application **/
      /***************************************************/
      case MEMORY_SET_WINDOW_ATTACH :
        window_attach = (struct window_attach *) data;
        break;

      case MEMORY_GET_WINDOW_ATTACH :
        *((struct window_attach **) data) = window_attach;
        break;

      case MEMORY_CLOSE_WINDOW_ATTACH :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_attach == NULL) ? NULL : window_attach->hwnd_attach;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_attach != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_ATTACH :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_attach(window_attach);
        window_attach = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;


      /***************************************************/
      /** Gestion de la fentre Attach de l'application **/
      /***************************************************/
      case MEMORY_SET_WINDOW_IPC :
        window_ipc = (struct window_ipc *) data;
        break;

      case MEMORY_GET_WINDOW_IPC :
        *((struct window_ipc **) data) = window_ipc;
        break;

      case MEMORY_FREE_WINDOW_IPC :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_ipc(window_ipc);
        window_ipc = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;


      /********************************************************/
      /** Gestion de la fentre Battery RAM de l'application **/
      /********************************************************/
      case MEMORY_SET_WINDOW_BATTERYRAM :
        window_batteryram = (struct window_batteryram *) data;
        break;

      case MEMORY_GET_WINDOW_BATTERYRAM :
        *((struct window_batteryram **) data) = window_batteryram;
        break;

      case MEMORY_CLOSE_WINDOW_BATTERYRAM :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_batteryram == NULL) ? NULL : window_batteryram->hwnd_batteryram;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_batteryram != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_BATTERYRAM :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_batteryram(window_batteryram);
        window_batteryram = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;


      /*****************************************************************/
      /** Gestion de la fentre Apple IIgs Internals de l'application **/
      /*****************************************************************/
      case MEMORY_SET_WINDOW_INTERNALS :
        window_internals = (struct window_internals *) data;
        break;

      case MEMORY_GET_WINDOW_INTERNALS :
        *((struct window_internals **) data) = window_internals;
        break;

      case MEMORY_CLOSE_WINDOW_INTERNALS :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_internals == NULL) ? NULL : window_internals->hwnd_internals;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_internals != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_INTERNALS :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_internals(window_internals);
        window_internals = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      /***************************************************/
      /** Gestion de la fentre Symbol de l'application **/
      /***************************************************/
      case MEMORY_SET_WINDOW_SYMBOL :
        window_symbol = (struct window_symbol *) data;
        break;

      case MEMORY_GET_WINDOW_SYMBOL :
        *((struct window_symbol **) data) = window_symbol;
        break;

      case MEMORY_CLOSE_WINDOW_SYMBOL :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_symbol == NULL) ? NULL : window_symbol->hwnd_symbol;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_symbol != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_SYMBOL :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_symbol(window_symbol);
        window_symbol = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      /*******************************************************/
      /** Gestion de la fentre Breakpoint de l'application **/
      /*******************************************************/
      case MEMORY_SET_WINDOW_BREAKPOINT :
        window_breakpoint = (struct window_breakpoint *) data;
        break;

      case MEMORY_GET_WINDOW_BREAKPOINT :
        *((struct window_breakpoint **) data) = window_breakpoint;
        break;

      case MEMORY_CLOSE_WINDOW_BREAKPOINT :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_breakpoint == NULL) ? NULL : window_breakpoint->hwnd_breakpoint;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_breakpoint != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_BREAKPOINT :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_breakpoint(window_breakpoint);
        window_breakpoint = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      /******************************************************/
      /** Gestion de la fentre Statistic de l'application **/
      /******************************************************/
      case MEMORY_SET_WINDOW_STATISTIC :
        window_statistic = (struct window_statistic *) data;
        break;

      case MEMORY_GET_WINDOW_STATISTIC :
        *((struct window_statistic **) data) = window_statistic;
        break;

      case MEMORY_CLOSE_WINDOW_STATISTIC :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_statistic == NULL) ? NULL : window_statistic->hwnd_statistic;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_statistic != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_STATISTIC :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_statistic(window_statistic);
        window_statistic = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      /****************************************************/
      /** Gestion de la fentre Profile de l'application **/
      /****************************************************/
      case MEMORY_SET_WINDOW_PROFILE :
        window_profile = (struct window_profile *) data;
        break;

      case MEMORY_GET_WINDOW_PROFILE :
        *((struct window_profile **) data) = window_profile;
        break;

      case MEMORY_CLOSE_WINDOW_PROFILE :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_profile == NULL) ? NULL : window_profile->hwnd_profile;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_profile != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_PROFILE :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_profile(window_profile);
        window_profile = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      /*********************************************************/
      /** Gestion de la fentre SymbolPickup de l'application **/
      /*********************************************************/
      case MEMORY_SET_WINDOW_SYMBOL_PICKUP :
        window_symbol_pickup = (struct window_symbol_pickup *) data;
        break;

      case MEMORY_GET_WINDOW_SYMBOL_PICKUP :
        *((struct window_symbol_pickup **) data) = window_symbol_pickup;
        break;

      case MEMORY_CLOSE_WINDOW_SYMBOL_PICKUP :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_symbol_pickup == NULL) ? NULL : window_symbol_pickup->hwnd_symbol_pickup;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_symbol_pickup != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_SYMBOL_PICKUP :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_symbol_pickup(window_symbol_pickup);
        window_symbol_pickup = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      /*******************************************************/
      /** Gestion de la fentre Preference de l'application **/
      /*******************************************************/
      case MEMORY_SET_WINDOW_PREFERENCE :
        window_preference = (struct window_preference *) data;
        break;

      case MEMORY_GET_WINDOW_PREFERENCE :
        *((struct window_preference **) data) = window_preference;
        break;

      case MEMORY_CLOSE_WINDOW_PREFERENCE :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_preference == NULL) ? NULL : window_preference->hwnd_preference;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_preference != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_PREFERENCE :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_preference(window_preference);
        window_preference = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;

      /********************************************************/
      /** Gestion de la fentre ProgressBar de l'application **/
      /********************************************************/
      case MEMORY_SET_WINDOW_PROGRESSBAR :
        window_progressbar = (struct window_progressbar *) data;
        break;

      case MEMORY_GET_WINDOW_PROGRESSBAR :
        *((struct window_progressbar **) data) = window_progressbar;
        break;

      case MEMORY_CLOSE_WINDOW_PROGRESSBAR :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        window_handle = (window_progressbar == NULL) ? NULL : window_progressbar->hwnd_progressbar;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        if(window_handle != NULL)
          SendMessage(window_handle,WM_CLOSE,0,0);
        while(window_progressbar != NULL)
          ;
        break;

      case MEMORY_FREE_WINDOW_PROGRESSBAR :
        EnterCriticalSection(&param->cs_windowclose_threadhandle);
        mem_free_window_progressbar(window_progressbar);
        window_progressbar = NULL;
        LeaveCriticalSection(&param->cs_windowclose_threadhandle);
        break;
    }
}


/*********************************************************************/
/*  compare_filepath() : Fonction de comparaison pour le Quick Sort. */
/*********************************************************************/
static int compare_filepath(const void *data_1, const void *data_2)
{
  struct file_path *file_path_1;
  struct file_path *file_path_2;

  /* Rcupration des paramtres */
  file_path_1 = *((struct file_path **) data_1);
  file_path_2 = *((struct file_path **) data_2);

  /* Comparaison des chemins */
  return(stricmp(file_path_1->path,file_path_2->path));
}


/*****************************************************************************/
/*  compare_filepath_by_size() : Fonction de comparaison pour le Quick Sort. */
/*****************************************************************************/
static int compare_filepath_by_size(const void *data_1, const void *data_2)
{
  struct file_path *file_path_1;
  struct file_path *file_path_2;

  /* Rcupration des paramtres */
  file_path_1 = *((struct file_path **) data_1);
  file_path_2 = *((struct file_path **) data_2);

  /* Comparaison des tailles puis des chemins */
  if(file_path_1->size > file_path_2->size)
    return(1);
  else if(file_path_1->size < file_path_2->size)
    return(-1);
  else
    return(stricmp(file_path_1->path,file_path_2->path));
}


/***********************************************************************/
/*  mem_alloc_parameter() :  Allocation mmoire de la structure param. */
/***********************************************************************/
struct parameter *mem_alloc_parameter(HINSTANCE hInstance)
{
  time_t clock;
  struct _timeb timebuffer;
  struct tm *time_date;
  HRSRC resource;
  HGLOBAL resource_data;
  unsigned char *resource_data_ptr;
  struct parameter *param;

  /* Allocation mmoire param */
  param = (struct parameter *) calloc(1,sizeof(struct parameter));
  if(param == NULL)
    return(NULL);

  /* Instance */
  param->hInstance = hInstance;

  /* PID */
  param->cyrene_pid = GetCurrentProcessId();

  /* Initialisation des CS */
  InitializeCriticalSection(&param->cs_notification_threadhandle);
  InitializeCriticalSection(&param->cs_mmu_threadhandle);
  InitializeCriticalSection(&param->cs_cpu_threadhandle);
  InitializeCriticalSection(&param->cs_memoryarea_threadhandle);
  InitializeCriticalSection(&param->cs_ramsound_threadhandle);
  InitializeCriticalSection(&param->cs_textpage40_threadhandle);
  InitializeCriticalSection(&param->cs_textpage80_threadhandle);
  InitializeCriticalSection(&param->cs_graphicgr_threadhandle);
  InitializeCriticalSection(&param->cs_graphichgr_threadhandle);
  InitializeCriticalSection(&param->cs_graphicdhgrmono_threadhandle);
  InitializeCriticalSection(&param->cs_graphicdhgrcolor_threadhandle);
  InitializeCriticalSection(&param->cs_graphicdgr_threadhandle);
  InitializeCriticalSection(&param->cs_graphichgrtext40_threadhandle);
  InitializeCriticalSection(&param->cs_graphichgrtext80_threadhandle);
  InitializeCriticalSection(&param->cs_graphicgrtext40_threadhandle);
  InitializeCriticalSection(&param->cs_graphicgrtext80_threadhandle);
  InitializeCriticalSection(&param->cs_graphicdgrtext80_threadhandle);
  InitializeCriticalSection(&param->cs_graphicdhgrmonotext80_threadhandle);
  InitializeCriticalSection(&param->cs_graphicdhgrcolortext80_threadhandle);
  InitializeCriticalSection(&param->cs_graphicshr_threadhandle);
  InitializeCriticalSection(&param->cs_disassembler_threadhandle);
  InitializeCriticalSection(&param->cs_loadfile_threadhandle);
  InitializeCriticalSection(&param->cs_savememory_threadhandle);
  InitializeCriticalSection(&param->cs_compare_threadhandle);
  InitializeCriticalSection(&param->cs_quickwatch_threadhandle);
  InitializeCriticalSection(&param->cs_modify_threadhandle);
  InitializeCriticalSection(&param->cs_move_threadhandle);
  InitializeCriticalSection(&param->cs_windowclose_threadhandle);
  InitializeCriticalSection(&param->cs_request);
  InitializeCriticalSection(&param->cs_operation);
  InitializeCriticalSection(&param->cs_shared_memory);
  InitializeCriticalSection(&param->cs_callstack);
  InitializeCriticalSection(&param->cs_profile_stack);
  InitializeCriticalSection(&param->cs_output);
  InitializeCriticalSection(&param->cs_progress_bar);
  InitializeCriticalSection(&param->cs_symbol);

  /** Cration la liste des Events Request en mode Manual-Reset **/
  param->thread_request_event = CreateEvent(NULL,TRUE,FALSE,TEXT("ThreadRequest")); 
  if(param->thread_request_event == NULL)
    {
      mem_free_param(param);
      return(NULL);
    }

  /** Rcupre les images GIF des polices de caractres **/
  /* DE */
  resource = FindResource(NULL,MAKEINTRESOURCE(IDR_RCDATA_CHAR_DE),RT_RCDATA);
  resource_data = LoadResource(NULL,resource);
  param->char_de_gif_size = SizeofResource(NULL,resource);
  resource_data_ptr = (unsigned char *) LockResource(resource_data);
  memcpy(param->char_de_gif,resource_data_ptr,param->char_de_gif_size);
  /* DK */
  resource = FindResource(NULL,MAKEINTRESOURCE(IDR_RCDATA_CHAR_DK),RT_RCDATA);
  resource_data = LoadResource(NULL,resource);
  param->char_dk_gif_size = SizeofResource(NULL,resource);
  resource_data_ptr = (unsigned char *) LockResource(resource_data);
  memcpy(param->char_dk_gif,resource_data_ptr,param->char_dk_gif_size);
  /* ES */
  resource = FindResource(NULL,MAKEINTRESOURCE(IDR_RCDATA_CHAR_ES),RT_RCDATA);
  resource_data = LoadResource(NULL,resource);
  param->char_es_gif_size = SizeofResource(NULL,resource);
  resource_data_ptr = (unsigned char *) LockResource(resource_data);
  memcpy(param->char_es_gif,resource_data_ptr,param->char_es_gif_size);
  /* FR */
  resource = FindResource(NULL,MAKEINTRESOURCE(IDR_RCDATA_CHAR_FR),RT_RCDATA);
  resource_data = LoadResource(NULL,resource);
  param->char_fr_gif_size = SizeofResource(NULL,resource);
  resource_data_ptr = (unsigned char *) LockResource(resource_data);
  memcpy(param->char_fr_gif,resource_data_ptr,param->char_fr_gif_size);
  /* IT */
  resource = FindResource(NULL,MAKEINTRESOURCE(IDR_RCDATA_CHAR_IT),RT_RCDATA);
  resource_data = LoadResource(NULL,resource);
  param->char_it_gif_size = SizeofResource(NULL,resource);
  resource_data_ptr = (unsigned char *) LockResource(resource_data);
  memcpy(param->char_it_gif,resource_data_ptr,param->char_it_gif_size);
  /* SE */
  resource = FindResource(NULL,MAKEINTRESOURCE(IDR_RCDATA_CHAR_SE),RT_RCDATA);
  resource_data = LoadResource(NULL,resource);
  param->char_se_gif_size = SizeofResource(NULL,resource);
  resource_data_ptr = (unsigned char *) LockResource(resource_data);
  memcpy(param->char_se_gif,resource_data_ptr,param->char_se_gif_size);
  /* UK */
  resource = FindResource(NULL,MAKEINTRESOURCE(IDR_RCDATA_CHAR_UK),RT_RCDATA);
  resource_data = LoadResource(NULL,resource);
  param->char_uk_gif_size = SizeofResource(NULL,resource);
  resource_data_ptr = (unsigned char *) LockResource(resource_data);
  memcpy(param->char_uk_gif,resource_data_ptr,param->char_uk_gif_size);
  /* US */
  resource = FindResource(NULL,MAKEINTRESOURCE(IDR_RCDATA_CHAR_US),RT_RCDATA);
  resource_data = LoadResource(NULL,resource);
  param->char_us_gif_size = SizeofResource(NULL,resource);
  resource_data_ptr = (unsigned char *) LockResource(resource_data);
  memcpy(param->char_us_gif,resource_data_ptr,param->char_us_gif_size);
  /* MouseText */
  resource = FindResource(NULL,MAKEINTRESOURCE(IDR_RCDATA_CHAR_MOUSETEXT),RT_RCDATA);
  resource_data = LoadResource(NULL,resource);
  param->char_mousetext_gif_size = SizeofResource(NULL,resource);
  resource_data_ptr = (unsigned char *) LockResource(resource_data);
  memcpy(param->char_mousetext_gif,resource_data_ptr,param->char_mousetext_gif_size);
  
  /** Dcodage des images GIF en Pixmap True color **/
  param->picture_char_de = GIFReadBufferToRGB(param->char_de_gif,4);    /* On veut du DWORD */
  param->picture_char_dk = GIFReadBufferToRGB(param->char_dk_gif,4);
  param->picture_char_es = GIFReadBufferToRGB(param->char_es_gif,4);
  param->picture_char_fr = GIFReadBufferToRGB(param->char_fr_gif,4);
  param->picture_char_it = GIFReadBufferToRGB(param->char_it_gif,4);
  param->picture_char_se = GIFReadBufferToRGB(param->char_se_gif,4);
  param->picture_char_uk = GIFReadBufferToRGB(param->char_uk_gif,4);
  param->picture_char_us = GIFReadBufferToRGB(param->char_us_gif,4);
  param->picture_char_mousetext = GIFReadBufferToRGB(param->char_mousetext_gif,4);

  /** Fichier de Trace **/
  param->insert_log_break = 1;
  _ftime(&timebuffer);
  time(&clock);
  time_date = localtime(&clock);
  sprintf(param->trace_file_name,"CyreneLog_%04d%02d%02d_%02d%02d%02d",1900+time_date->tm_year,time_date->tm_mon+1,time_date->tm_mday,time_date->tm_hour,time_date->tm_min,time_date->tm_sec);

  /* On va stocker la premire opration */
  param->is_first_operation = 1;

  /* Rcupre la liste des Appels systmes */
  param->tab_systemcall = BuildSystemCallTable(&param->nb_systemcall);

  /* OK */
  return(param);
}


/******************************************************************/
/*  mem_free_param() :  Libration mmoire de la structure param. */
/******************************************************************/
void mem_free_param(struct parameter *param)
{
  if(param)
    {
      /* Libration des sections  accs controles */
      DeleteCriticalSection(&param->cs_notification_threadhandle);
      DeleteCriticalSection(&param->cs_mmu_threadhandle);
      DeleteCriticalSection(&param->cs_cpu_threadhandle);
      DeleteCriticalSection(&param->cs_memoryarea_threadhandle);
      DeleteCriticalSection(&param->cs_ramsound_threadhandle);
      DeleteCriticalSection(&param->cs_textpage40_threadhandle);
      DeleteCriticalSection(&param->cs_textpage80_threadhandle);
      DeleteCriticalSection(&param->cs_graphicgr_threadhandle);
      DeleteCriticalSection(&param->cs_graphichgr_threadhandle);
      DeleteCriticalSection(&param->cs_graphicdhgrmono_threadhandle);
      DeleteCriticalSection(&param->cs_graphicdhgrcolor_threadhandle);
      DeleteCriticalSection(&param->cs_graphicdgr_threadhandle);
      DeleteCriticalSection(&param->cs_graphichgrtext40_threadhandle);
      DeleteCriticalSection(&param->cs_graphichgrtext80_threadhandle);
      DeleteCriticalSection(&param->cs_graphicgrtext40_threadhandle);
      DeleteCriticalSection(&param->cs_graphicgrtext80_threadhandle);
      DeleteCriticalSection(&param->cs_graphicdgrtext80_threadhandle);
      DeleteCriticalSection(&param->cs_graphicdhgrmonotext80_threadhandle);
      DeleteCriticalSection(&param->cs_graphicdhgrcolortext80_threadhandle);
      DeleteCriticalSection(&param->cs_graphicshr_threadhandle);
      DeleteCriticalSection(&param->cs_disassembler_threadhandle);
      DeleteCriticalSection(&param->cs_loadfile_threadhandle);
      DeleteCriticalSection(&param->cs_savememory_threadhandle);
      DeleteCriticalSection(&param->cs_compare_threadhandle);
      DeleteCriticalSection(&param->cs_quickwatch_threadhandle);
      DeleteCriticalSection(&param->cs_modify_threadhandle);
      DeleteCriticalSection(&param->cs_move_threadhandle);
      DeleteCriticalSection(&param->cs_windowclose_threadhandle);
      DeleteCriticalSection(&param->cs_request);
      DeleteCriticalSection(&param->cs_operation);
      DeleteCriticalSection(&param->cs_shared_memory);
      DeleteCriticalSection(&param->cs_callstack);
      DeleteCriticalSection(&param->cs_profile_stack);
      DeleteCriticalSection(&param->cs_output);
      DeleteCriticalSection(&param->cs_progress_bar);
      DeleteCriticalSection(&param->cs_symbol);

      /** Libration des Request Events **/
      if(param->thread_request_event != NULL)
        CloseHandle(param->thread_request_event);

      /* Libre la DLL */
      if(param->rich_edit_dll)
        FreeLibrary(param->rich_edit_dll);

      /* Libration picture char */
      mem_free_picture_true(param->picture_char_de);
      mem_free_picture_true(param->picture_char_dk);
      mem_free_picture_true(param->picture_char_es);
      mem_free_picture_true(param->picture_char_fr);
      mem_free_picture_true(param->picture_char_it);
      mem_free_picture_true(param->picture_char_se);
      mem_free_picture_true(param->picture_char_uk);
      mem_free_picture_true(param->picture_char_us);
      mem_free_picture_true(param->picture_char_mousetext);

      /* Libration du tableau des Appels Systmes */
      mem_free_systemcall_table(param->nb_systemcall,param->tab_systemcall);

      free(param);
    }
}


/*****************************************************************/
/*  mem_free_filepath() :  Libration de la structure file_path. */
/*****************************************************************/
void mem_free_filepath(struct file_path *current_filepath)
{    
  if(current_filepath)
    {
      if(current_filepath->path != NULL)
        free(current_filepath->path);

      if(current_filepath->data != NULL)
        free(current_filepath->data);

      free(current_filepath);
    }
}

/**********************************************************************/
