/**********************************************************************/
/*                                                                    */
/*  Dc_Name.c :  Module pour la bibliothque de gestion des Noms.     */
/*                                                                    */
/**********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Mai 2024  */
/**********************************************************************/

#include <stdio.h>

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <Commdlg.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_MMU.h"
#include "Dc_Snapshot.h"
#include "Dc_IIgsInternals.h"
#include "Cyrene_Engine.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Symbol.h"
#include "Dc_Memory.h"
#include "Dc_Name.h"

char *tab_rom_00_name[] = 
{
  /* BFxx */
  "0xBF00", "Prodos8 MLI",
  "0xBF2F", "INIT",
  /* F8xx */
  "0xF800", "PLOT",
  "0xF80E", "PLOT1",
  "0xF819", "HLINE",
  "0xF828", "VLINE",
  "0xF832", "CLRSCR",
  "0xF836", "CLRTOP",
  "0xF847", "GBASCALC",
  "0xF85F", "NXTCOL",
  "0xF864", "SETCOL",
  "0xF871", "SCRN",
  "0xF88C", "INSDS1.2",
  "0xF88E", "INSDS2",
  "0xF890", "GET816LEN",
  "0xF8D0", "INSTDSP",
  /* F9xx */
  "0xF940", "PRNTYX",
  "0xF941", "PRNTAX",
  "0xF944", "PRNTX",
  "0xF948", "PRBLNK",
  "0xF94A", "PRBL2",
  "0xF953", "PCADJ",
  "0xF962", "TEXT2COPY",
  /* FAxx */   
  "0xFA40", "OLDIRQ",
  "0xFA4C", "BREAK",
  "0xFA59", "OLDBRK",
  "0xFA62", "RESET",
  "0xFAA6", "PWRUP",
  "0xFABA", "SLOOP",
  "0xFAD7", "REGDSP",
  /* FBxx */
  "0xFB19", "RTBL",
  "0xFB1E", "PREAD",
  "0xFB21", "PREAD4",
  "0xFB2F", "INIT",
  "0xFB39", "SETTXT",
  "0xFB40", "SETGR",
  "0xFB4B", "SETWND",
  "0xFB51", "SETWND2",
  "0xFB5B", "TABV",
  "0xFB60", "APPLEII",
  "0xFB6F", "SETPWRC",
  "0xFB78", "VIDWAIT",
  "0xFB88", "KBDWAIT",
  "0xFBB3", "VERSION",
  "0xFBBF", "ZIDBYTE2",
  "0xFBC0", "ZIDBYTE",
  "0xFBC1", "BASCALC",
  "0xFBDD", "BELL1",
  "0xFBE2", "BELL1.2",
  "0xFBE4", "BELL2",
  "0xFBF0", "STORADV",
  "0xFBF4", "ADVANCE",
  "0xFBFD", "VIDOUT",
  /* FCxx */
  "0xFC10", "BS",
  "0xFC1A", "UP",
  "0xFC22", "VTAB",
  "0xFC24", "VTABZ",
  "0xFC42", "CLREOP",
  "0xFC58", "HOME",
  "0xFC62", "CR",
  "0xFC66", "LF",
  "0xFC70", "SCROLL",
  "0xFC9C", "CLREOL",   
  "0xFC9E", "CLREOLZ",
  "0xFCA8", "WAIT",
  "0xFCB4", "NXTA4",
  "0xFCBA", "NXTA1",
  "0xFCC9", "HEADR",         
  /* FDxx */
  "0xFD0C", "RDKEY",
  "0xFD10", "FD10",
  "0xFD18", "RDKEY1",
  "0xFD1B", "KEYIN",  
  "0xFD35", "RDCHAR",
  "0xFD67", "GETLNZ",
  "0xFD6A", "GETLN", 
  "0xFD6C", "GETLN0",
  "0xFD6F", "GETLN1", 
  "0xFD8B", "CROUT1",
  "0xFD8E", "CROUT",
  "0xFD92", "PRA1",
  "0xFDDA", "PRBYTE",
  "0xFDE3", "PRHEX",
  "0xFDED", "COUT",
  "0xFDF0", "COUT1",
  "0xFDF6", "COUTZ",
  /* FExx */
  "0xFE1F", "IDROUTINE",
  "0xFE2C", "MOVE",
  "0xFE5E", "LIST",
  "0xFE80", "SETINV", 
  "0xFE84", "SETNORM",
  "0xFE89", "SETKBD",
  "0xFE8B", "INPORT", 
  "0xFE93", "SETVID",
  "0xFE95", "OUTPORT",
  "0xFEB6", "GO",
  "0xFECD", "WRITE",
  "0xFEFD", "READ",
  /* FFxx */
  "0xFF2D", "PRERR",
  "0xFF3A", "BELL",
  "0xFF3F", "RESTORE",
  "0xFF4A", "SAVE",
  "0xFF58", "IORTS",
  "0xFF59", "OLDRST",
  "0xFF65", "MON",
  "0xFF69", "MONZ",
  "0xFF6C", "MONZ2",
  "0xFF70", "MONZ4",
  "0xFF8A", "DIG",
  "0xFFA7", "GETNUM",
  "0xFFAD", "NXTCHR",
  "0xFFBE", "TOSUB",
  "0xFFC7", "ZMODE",
  "0xFFFA", "NMI vector",
  "0xFFFC", "Reset vector",
  "0xFFFE", "IRQ/BRK vector",
  NULL, NULL
};

char *tab_softwitch_name[] = 
{
  /* C000 */  "KBD CLR80COL", "SET80COL",   "RDMAINRAM",   "RDCARDRAM",    "WRMAINRAM",       "WRCARDRAM",  "SETSLOTCXROM",     "SETINTCXROM",
  /* C008 */  "SETSTDZP",     "SETALTZP",   "SETINTC3ROM", "SETSLOTC3ROM", "CLR80VID",        "SET80VID",   "CLRALTCHAR",       "SETALTCHAR",
  /* C010 */  "KBDSTRB",      "RDLCBNK2",   "RDLCRAM",     "RDRAMRD",      "RDRAMWRT",        "RDCXROM",    "RDALTZP",          "RDC3ROM",
  /* C018 */  "RD80COL",      "RDVBLBAR",   "RTEXT",       "RDMIX",        "RDPAGE2",         "RDHIRES",    "ALTCHARSET",       "RD80VID",
  /* C020 */  "TAPEOUT",      "MONOCOLOR",  "TBCOLOR",     "VGCINT",       "MOUSEDATA",       "KEYMODREG",  "DATAREG",          "KMSTATUS",     
  /* C028 */  "ROMBANK",      "NEWVIDEO",   "",            "LANGSEL",      "CHARROM",         "SLTROMSEL",  "VERTCNT",          "HORIZCNT",
  /* C030 */  "SPKR",         "DISKREG",    "SCANINT",     "CLOCKDATA",    "CLOCKCTL BORDER", "SHADOW",     "CYAREG",           "DMAREG",
  /* C038 */  "SCCBREG",      "SCCAREG",    "SCCBDATA",    "SCCADATA",     "SOUNDCTL",        "SOUNDDATA",  "SOUNDADRL",        "SOUNDADRH",
  /* C040 */  "",             "INTEN",      "",            "",             "MMDELTAX",        "MMDELTAY",   "DIAGTYPE INTFLAG", "CLRBVLINT",
  /* C048 */  "CLRXYINT",     "",           "",            "",             "",                 "",          "",                 "",
  /* C050 */  "TXTCLR",       "TXTSET",     "MIXCLR",      "MIXSET",       "TXTPAGE1",         "TXTPAGE2",  "LORES",            "HIRES",
  /* C058 */  "SETAN0",       "CLRAN0",     "SETAN1",      "CLRAN1",       "SETAN2",           "CLRAN2",    "SETAN3",           "CLRAN3",
  /* C060 */  "BUTN3",        "BUTN0",      "BUTN1",       "BUTN2",        "PADDL0",           "",          "",                 "",
  /* C068 */  "STATEREG",     "",           "",            "",             "",                 "TESTREG",   "CLRTM",            "ENTM",
  /* C070 */  "PTRIG",        "",           "",            "",             "ROM Interrupt Jump Table",    "",          "",                 "",
  /* C078 */  "",             "",           "",            "",             "",                 "",          "",                 "",
  /* C080 */  "",             "ROMIN",      "",            "LCBANK2",      "",                 "",          "",                 "",
  /* C088 */  "",             "",           "",            "LCBANK1"
};
#define MAX_SOFTSWITCH_ADDRESS  0xC08B

char *tab_iwm_name[] = 
{
  /* C0E0 */  "IWM:PH0 off",   "IWM:PH0 on",   "IWM:PH1 off",  "IWM:PH1 on",  "IWM:PH2 off",       "IWM:PH2 on",            "IWM:PH3 off",                "IWM:PH3 on",
  /* C0E8 */  "IWM:motor off", "IWM:motor on", "IWM:drive 1",  "IWM:drive 2", "IWM:Q6 OFF (Read)", "IWM:Q6 ON  (WP-sense)", "IWM:Q7 OFF (WP-sense/Read)", "IWM:Q7 ON  (Write)"
};

char *tab_tools_name[] =
{
  "Tool Locator",      "0x01", "Memory Manager",           "0x02", "Misc",             "0x03",
  "QuickDraw II",      "0x04", "Desk Manager",             "0x05", "Event Manager",    "0x06",
  "Scheduler",         "0x07", "Sound Manager",            "0x08", "Desktop Bus",      "0x09",
  "Sane",              "0x0A", "Integer Math",             "0x0B", "Text",             "0x0C",
  "RAMDisk",           "0x0D", "Window Manager",           "0x0E", "Menu Manager",     "0x0F", 
  "Control Manager",   "0x10", "Load Manager",             "0x11", "QuickDraw Aux",    "0x12", 
  "Print Manager",     "0x13", "LineEdit",                 "0x14", "Dialog Manager",   "0x15", 
  "Scrap Manager",     "0x16", "Standard File Operations", "0x17", "Disk Utilities",   "0x18",
  "Note Synthesizer",  "0x19", "Note Sequencer",           "0x1A", "Font Manager",     "0x1B", 
  "List Manager",      "0x1C", "ACE",                      "0x1D", "Resource Manager", "0x1E", 
  "MIDI",              "0x20", "Video Overlay",            "0x21", "TextEdit",         "0x22", 
  "Midi Synth",        "0x23", "Anim Tool Set",            "0x25", "Media Controller", "0x26", 
  "Male Voice",        "0x32", "Female Voice",             "0x33", "Parser Speech",    "0x34", 
  "Speech Toolkit",    "0x35", "Hash Tool",                "0x80", "Crypto Tool Set",  "0x81", 
  "HTML Tool Set",     "0x82", "FTP Tool Set",             "0x83", "Undo Manager",     "0x84", 
  "SoundSmith",        "0xDB", "NoiseTracker",             "0xDC", "NinjaTracker",     "0xDD", 
  "NinjaTracker Plus", "0xDE", "ZPack",                    "0xE6", "GS/Bug",           "0xFF", 
  NULL, NULL
};

char *tab_toolbox_name[] = 
{
  /** Tool Locator : 0x01 **/
  "_TLBootInit",    "0x0101", "_TLStartUp",     "0x0201", "_TLShutDown",   "0x0301", "_TLVersion",     "0x0401", "_TLReset",       "0x0501", "_TLStatus",          "0x0601", "_GetTSPtr",      "0x0901", "_SetTSPtr",         "0x0A01", "_GetFuncPtr",    "0x0B01", "_GetWAP", "0x0C01", 
  "_SetWAP",        "0x0D01", "_LoadTools",     "0x0E01", "_LoadOneTool",  "0x0F01", "_UnloadOneTool", "0x1001", "_TLMountVolume", "0x1101", "_TLTextMountVolume", "0x1201", "_SaveTextState", "0x1301", "_RestoreTextState", "0x1401", "_MessageCenter", "0x1501", 
  "_SetDefaultTPT", "0x1601", "_MessageByName", "0x1701", "_StartUpTools", "0x1801", "_ShutDownTools", "0x1901", "_GetMsgHandle",  "0x1A01", "_AcceptRequests",    "0x1B01", "_SendRequest",   "0x1C01", 

  /** Memory Manager : 0x02 **/
  "_MMBootInit",    "0x0102", "_MMStartUp",          "0x0202", "_MMShutDown",    "0x0302", "_MMVersion",   "0x0402", "_MMReset",     "0x0502", "_MMStatus",    "0x0602", "_NewHandle",     "0x0902", "_ReallocHandle", "0x0A02", "_RestoreHandle", "0x0B02", 
  "_AddToOOMQueue", "0x0C02", "_DeleteFromOOMQueue", "0x0D02", "_DisposeHandle", "0x1002", "_DisposeAll",  "0x1102", "_PurgeHandle", "0x1202", "_PurgeAll",    "0x1302", "_GetHandleSize", "0x1802", "_SetHandleSize", "0x1902", "_FindHandle",    "0x1A02", 
  "_FreeMem",       "0x1B02", "_MaxBlock",           "0x1C02", "_TotalMem",      "0x1D02", "_CheckHandle", "0x1E02", "_CompactMem",  "0x1F02", "_HLock",       "0x2002", "_HLockAll",      "0x2102", "_HUnlock",       "0x2202", "_HUnlockAll",    "0x2302", 
  "_SetPurge",      "0x2402", "_SetPurgeAll",        "0x2502", "_PtrToHand",     "0x2802", "_HandToPtr",   "0x2902", "_HandToHand",  "0x2A02", "_BlockMove",   "0x2B02", "_RealFreeMem",   "0x2F02", "_SetHandleID",   "0x3002", 

  /** Misc : 0x03 **/
  "_MTBootInit",          "0x0103", "_MTStartUp",      "0x0203", "_MTShutDown",         "0x0303", "_MTVersion",   "0x0403", "_MTReset",         "0x0503", "_MTStatus",          "0x0603", "_WriteBRam",         "0x0903", "_ReadBRam",           "0x0A03", "_WriteBParam", "0x0B03", "_ReadBParam",   "0x0C03", 
  "_ReadTimeHex",         "0x0D03", "_WriteTimeHex",   "0x0E03", "_ReadAsciiTime",      "0x0F03", "_SetVector",   "0x1003", "_GetVector",       "0x1103", "_SetHeartBeat",      "0x1203", "_DelHeartBeat",      "0x1303", "_ClrHeartBeat",       "0x1403", "_SysFailMgr",  "0x1503", 
  "_GetAddr",             "0x1603", "_ReadMouse",      "0x1703", "_InitMouse",          "0x1803", "_SetMouse",    "0x1903", "_HomeMouse",       "0x1A03", "_ClearMouse",        "0x1B03", "_ClampMouse",        "0x1C03", "_GetMouseClamp",      "0x1D03", "_PosMouse",    "0x1E03", "_ServeMouse",   "0x1F03", 
  "_GetNewID",            "0x2003", "_DeleteID",       "0x2103", "_StatusID",           "0x2203", "_IntSource",   "0x2303", "_FWEntry",         "0x2403", "_GetTick",           "0x2503", "_PackBytes",         "0x2603", "_UnPackBytes",        "0x2703", "_Munger",      "0x2803", "_GetIRQEnable", "0x2903", 
  "_SetAbsClamp",         "0x2A03", "_GetAbsClamp",    "0x2B03", "_SysBeep",            "0x2C03", "_AddToQueue",  "0x2E03", "_DeleteFromQueue", "0x2F03", "_SetInterruptState", "0x3003", "_GetInterruptState", "0x3103", "_GetIntStateRecSize", "0x3203", "_ReadMouse2",  "0x3303", 
  "_GetCodeResConverter", "0x3403", "_GetROMResource", "0x3503", "_ReleaseROMResource", "0x3603", "_ConvSeconds", "0x3703", "_SysBeep2",        "0x3803", "_VersionString",     "0x3903", "_WaitUntil",         "0x3A03", "_StringToText",       "0x3B03", 
  "_ShowBootInfo",        "0x3C03", "_ScanDevices",    "0x3D03", "_AlertMessage",       "0x3E03", "_DoSysPrefs",  "0x3F03", 

  /** QuickDraw II : 0x04 **/
  "_QDBootInit",     "0x0104", "_QDStartUp",         "0x0204", "_QDShutDown",     "0x0304", "_QDVersion",     "0x0404", "_QDReset",        "0x0504", "_QDStatus",        "0x0604", "_GetAddress",    "0x0904", "_GrafOn",        "0x0A04", "_GrafOff",      "0x0B04", "_GetStandardSCB", "0x0C04", 
  "_InitColorTable", "0x0D04", "_SetColorTable",     "0x0E04", "_GetColorTable",  "0x0F04", "_SetColorEntry", "0x1004", "_GetColorEntry",  "0x1104", "_SetSCB",          "0x1204", "_GetSCB",        "0x1304", "_SetAllSCBs",    "0x1404", "_ClearScreen",  "0x1504", 
  "_SetMasterSCB",   "0x1604", "_GetMasterSCB",      "0x1704", "_OpenPort",       "0x1804", "_InitPort",      "0x1904", "_ClosePort",      "0x1A04", "_SetPort",         "0x1B04", "_GetPort",       "0x1C04", "_SetPortLoc",    "0x1D04", "_GetPortLoc",   "0x1E04", "_SetPortRect",    "0x1F04", 
  "_GetPortRect",    "0x2004", "_SetPortSize",       "0x2104", "_MovePortTo",     "0x2204", "_SetOrigin",     "0x2304", "_SetClip",        "0x2404", "_GetClip",         "0x2504", "_ClipRect",      "0x2604", "_HidePen",       "0x2704", "_ShowPen",      "0x2804", "_GetPen",         "0x2904", 
  "_SetPenState",    "0x2A04", "_GetPenState",       "0x2B04", "_SetPenSize",     "0x2C04", "_GetPenSize",    "0x2D04", "_SetPenMode",     "0x2E04", "_GetPenMode",      "0x2F04", "_SetPenPat",     "0x3004", "_GetPenPat",     "0x3104", "_SetPenMask",   "0x3204", 
  "_GetPenMask",     "0x3304", "_SetBackPat",        "0x3404", "_GetBackPat",     "0x3504", "_PenNormal",     "0x3604", "_SetSolidPenPat", "0x3704", "_SetSolidBackPat", "0x3804", "_SolidPattern",  "0x3904", "_MoveTo",        "0x3A04", "_Move",         "0x3B04", 
  "_LineTo",         "0x3C04", "_Line",              "0x3D04", "_SetPicSave",     "0x3E04", "_GetPicSave",    "0x3F04", "_SetRgnSave",     "0x4004", "_GetRgnSave",      "0x4104", "_SetPolySave",   "0x4204", "_GetPolySave",   "0x4304", "_SetGrafProcs", "0x4404", "_GetGrafProcs",   "0x4504", 
  "_SetUserField",   "0x4604", "_GetUserField",      "0x4704", "_SetSysField",    "0x4804", "_GetSysField",   "0x4904", "_SetRect",        "0x4A04", "_OffsetRect",      "0x4B04", "_InsetRect",     "0x4C04", "_SectRect",      "0x4D04", "_UnionRect",    "0x4E04", 
  "_PtInRect",       "0x4F04", "_Pt2Rect",           "0x5004", "_EqualRect",      "0x5104", "_EmptyRect",     "0x5204", "_FrameRect",      "0x5304", "_PaintRect",       "0x5404", "_EraseRect",     "0x5504", "_InvertRect",    "0x5604", "_FillRect",     "0x5704", "_FrameOval",      "0x5804", 
  "_PaintOval",      "0x5904", "_EraseOval",         "0x5A04", "_InvertOval",     "0x5B04", "_FillOval",      "0x5C04", "_FrameRRect",     "0x5D04", "_PaintRRect",      "0x5E04", "_EraseRRect",    "0x5F04", "_InvertRRect",   "0x6004", "_FillRRect",    "0x6104", "_FrameArc",       "0x6204", 
  "_PaintArc",       "0x6304", "_EraseArc",          "0x6404", "_InvertArc",      "0x6504", "_FillArc",       "0x6604", "_NewRgn",         "0x6704", "_DisposeRgn",      "0x6804", "_CopyRgn",       "0x6904", "_SetEmptyRgn",   "0x6A04", "_SetRectRgn",   "0x6B04", "_RectRgn",        "0x6C04", "_OpenRgn", "0x6D04", 
  "_CloseRgn",       "0x6E04", "_OffsetRgn",         "0x6F04", "_InsetRgn",       "0x7004", "_SectRgn",       "0x7104", "_UnionRgn",       "0x7204", "_DiffRgn",         "0x7304", "_XorRgn",        "0x7404", "_PtInRgn",       "0x7504", "_RectInRgn",    "0x7604", "_EqualRgn",       "0x7704", 
  "_EmptyRgn",       "0x7804", "_FrameRgn",          "0x7904", "_PaintRgn",       "0x7A04", "_EraseRgn",      "0x7B04", "_InvertRgn",      "0x7C04", "_FillRgn",         "0x7D04", "_ScrollRect",    "0x7E04", "_PaintPixels",   "0x7F04", "_AddPt",        "0x8004", "_SubPt",          "0x8104", 
  "_SetPt",          "0x8204", "_EqualPt",           "0x8304", "_LocalToGlobal",  "0x8404", "_GlobalToLocal", "0x8504", "_Random",         "0x8604", "_SetRandSeed",     "0x8704", "_GetPixel",      "0x8804", "_ScalePt",       "0x8904", "_MapPt",        "0x8A04", "_MapRect",        "0x8B04", 
  "_MapRgn",         "0x8C04", "_SetStdProcs",       "0x8D04", "_SetCursor",      "0x8E04", "_GetCursorAdr",  "0x8F04", "_HideCursor",     "0x9004", "_ShowCursor",      "0x9104", "_ObscureCursor", "0x9204", "_SetMouseLoc",   "0x9304", "_SetFont",      "0x9404", 
  "_GetFont",        "0x9504", "_GetFontInfo",       "0x9604", "_GetFontGlobals", "0x9704", "_SetFontFlags",  "0x9804", "_GetFontFlags",   "0x9904", "_SetTextFace",     "0x9A04", "_GetTextFace",   "0x9B04", "_SetTextMode",   "0x9C04", "_GetTextMode",  "0x9D04", 
  "_SetSpaceExtra",  "0x9E04", "_GetSpaceExtra",     "0x9F04", "_SetForeColor",   "0xA004", "_GetForeColor",  "0xA104", "_SetBackColor",   "0xA204", "_GetBackColor",    "0xA304", "_DrawChar",      "0xA404", "_DrawString",    "0xA504", "_DrawCString",  "0xA604", 
  "_DrawText",       "0xA704", "_CharWidth",         "0xA804", "_StringWidth",    "0xA904", "_CStringWidth",  "0xAA04", "_TextWidth",      "0xAB04", "_CharBounds",      "0xAC04", "_StringBounds",  "0xAD04", "_CStringBounds", "0xAE04", "_TextBounds",   "0xAF04", 
  "_SetArcRot",      "0xB004", "_GetArcRot",         "0xB104", "_SetSysFont",     "0xB204", "_GetSysFont",    "0xB304", "_SetVisRgn",      "0xB404", "_GetVisRgn",       "0xB504", "_SetIntUse",     "0xB604", "_OpenPicture",   "0xB704", "_PicComment",   "0xB804", "_ClosePicture",   "0xB904", 
  "_DrawPicture",    "0xBA04", "_KillPicture",       "0xBB04", "_FramePoly",      "0xBC04", "_PaintPoly",     "0xBD04", "_ErasePoly",      "0xBE04", "_InvertPoly",      "0xBF04", "_FillPoly",      "0xC004", "_OpenPoly",      "0xC104", "_ClosePoly",    "0xC204", "_KillPoly",       "0xC304", 
  "_OffsetPoly",     "0xC404", "_MapPoly",           "0xC504", "_SetClipHandle",  "0xC604", "_GetClipHandle", "0xC704", "_SetVisHandle",   "0xC804", "_GetVisHandle",    "0xC904", "_InitCursor",    "0xCA04", "_SetBufDims",    "0xCB04", "_ForceBufDims", "0xCC04", 
  "_SaveBufDims",    "0xCD04", "_RestoreBufDims",    "0xCE04", "_GetFGSize",      "0xCF04", "_SetFontID",     "0xD004", "_GetFontID",      "0xD104", "_SetTextSize",     "0xD204", "_GetTextSize",   "0xD304", "_SetCharExtra",  "0xD404", "_GetCharExtra", "0xD504", 
  "_PPToPort",       "0xD604", "_InflateTextBuffer", "0xD704", "_GetRomFont",     "0xD804", "_GetFontLore",   "0xD904", "_Get640Colors",   "0xDA04", "_Set640Color",     "0xDB04", 

  /** Desk Manager : 0x05 **/
  "_DeskBootInit", "0x0105", "_DeskStartUp", "0x0205", "_DeskShutDown",   "0x0305", "_DeskVersion", "0x0405", "_DeskReset",        "0x0505", "_DeskStatus",   "0x0605", "_SaveScrn",     "0x0905", "_RestScrn",  "0x0A05", "_SaveAll",        "0x0B05",
  "_RestAll",      "0x0C05", "_InstallNDA",  "0x0E05", "_InstallCDA",     "0x0F05", "_ChooseCDA",   "0x1105", "_SetDAStrPtr",      "0x1305", "_GetDAStrPtr",  "0x1405", "_OpenNDA",      "0x1505", "_CloseNDA",  "0x1605", "_SystemClick",    "0x1705",
  "_SystemEdit",   "0x1805", "_SystemTask",  "0x1905", "_SystemEvent",    "0x1A05", "_GetNumNDAs",  "0x1B05", "_CloseNDAbyWinPtr", "0x1C05", "_CloseAllNDAs", "0x1D05", "_FixAppleMenu", "0x1E05", "_AddToRunQ", "0x1F05", "_RemoveFromRunQ", "0x2005",
  "_RemoveCDA",    "0x2105", "_RemoveNDA",   "0x2205", "_GetDeskAccInfo", "0x2305", "_CallDeskAcc", "0x2405", "_GetDeskGlobal",    "0x2505",

  /** Event Manager : 0x06 **/
  "_EMBootInit",  "0x0106", "_EMStartUp",  "0x0206", "_EMShutDown",   "0x0306", "_EMVersion",    "0x0406", "_EMReset",   "0x0506", "_EMStatus",        "0x0606", "_DoWindows",         "0x0906", "_GetNextEvent",      "0x0A06", "_EventAvail", "0x0B06", 
  "_GetMouse",    "0x0C06", "_Button",     "0x0D06", "_StillDown",    "0x0E06", "_WaitMouseUp",  "0x0F06", "_TickCount", "0x1006", "_GetDblTime",      "0x1106", "_GetCaretTime",      "0x1206", "_SetSwitch",         "0x1306", "_PostEvent",  "0x1406", 
  "_FlushEvents", "0x1506", "_GetOSEvent", "0x1606", "_OSEventAvail", "0x1706", "_SetEventMask", "0x1806", "_FakeMouse", "0x1906", "_SetAutoKeyLimit", "0x1A06", "_GetKeyTranslation", "0x1B06", "_SetKeyTranslation", "0x1C06", 

  /** Scheduler : 0x07 **/
  "_SchBootInit", "0x0107", "_SchStartUp", "0x0207", "_SchShutDown", "0x0307", "_SchVersion", "0x0407", "_SchReset", "0x0507", "_SchStatus", "0x0607", "_SchAddTask", "0x0907", "_SchFlush", "0x0A07", 

  /** Sound manager : 0x08 **/
  "_SoundBootInit",     "0x0108", "_SoundStartUp",   "0x0208", "_SoundShutDown",  "0x0308", "_SoundVersion", "0x0408", "_SoundReset",    "0x0508", "_SoundToolStatus",   "0x0608", "_WriteRamBlock", "0x0908", "_ReadRamBlock",     "0x0A08", "_GetTableAddress", "0x0B08", 
  "_GetSoundVolume",    "0x0C08", "_SetSoundVolume", "0x0D08", "_FFStartSound",   "0x0E08", "_FFStopSound",  "0x0F08", "_FFSoundStatus", "0x1008", "_FFGeneratorStatus", "0x1108", "_SetSoundMIRQV", "0x1208", "_SetUserSoundIRQV", "0x1308", 
  "_FFSoundDoneStatus", "0x1408", "_FFSetUpSound",   "0x1508", "_FFStartPlaying", "0x1608", "_SetDocReg",    "0x1708", "_ReadDocReg",    "0x1808", 

  /** Desktop Bus : 0x09 **/
  "_ADBBootInit",     "0x0109", "_ADBStartUp",     "0x0209", "_ADBShutDown", "0x0309", "_ADBVersion", "0x0409", "_ADBReset", "0x0509", "_ADBStatus",   "0x0609", "_SendInfo",    "0x0909", "_ReadKeyMicroData", "0x0A09", "_ReadKeyMicroMem", "0x0B09",
  "_AsyncADBReceive", "0x0D09", "_SyncADBReceive", "0x0E09", "_AbsOn",       "0x0F09", "_AbsOff",     "0x1009", "_ReadAbs",  "0x1109", "_SetAbsScale", "0x1209", "_GetAbsScale", "0x1309", "_SRQPoll",          "0x1409", "_SRQRemove",       "0x1509", "_ClearSRQTable", "0x1609",

  /** Sane : 0x0A **/
  "_SANEBootInit", "0x010A", "_SANEStartUp", "0x020A", "_SANEShutDown", "0x030A", "_SANEVersion", "0x040A", "_SANEReset", "0x050A", "_SANEStatus", "0x060A", "_SANEFP816", "0x090A", "_SANEDecStr816", "0x0A0A", "_SANEElems816", "0x0B0A", 

  /** Integer Math : 0x0B **/
  "_IMBootInit", "0x010B", "_IMStartUp", "0x020B", "_IMShutDown", "0x030B", "_IMVersion", "0x040B", "_IMReset",  "0x050B", "_IMStatus", "0x060B", "_Multiply", "0x090B", "_SDivide",  "0x0A0B", "_UDivide",  "0x0B0B", "_LongMul", "0x0C0B", 
  "_LongDivide", "0x0D0B", "_FixRatio",  "0x0E0B", "_FixMul",     "0x0F0B", "_FracMul",   "0x100B", "_FixDiv",   "0x110B", "_FracDiv",  "0x120B", "_FixRound", "0x130B", "_FracSqrt", "0x140B", "_FracCos",  "0x150B", "_FracSin", "0x160B", 
  "_FixATan2",   "0x170B", "_HiWord",    "0x180B", "_LoWord",     "0x190B", "_Long2Fix",  "0x1A0B", "_Fix2Long", "0x1B0B", "_Fix2Frac", "0x1C0B", "_Frac2Fix", "0x1D0B", "_Fix2X",    "0x1E0B", "_Frac2X",   "0x1F0B", "_X2Fix",   "0x200B", 
  "_X2Frac",     "0x210B", "_Int2Hex",   "0x220B", "_Long2Hex",   "0x230B", "_Hex2Int",   "0x240B", "_Hex2Long", "0x250B", "_Int2Dec",  "0x260B", "_Long2Dec", "0x270B", "_Dec2Int",  "0x280B", "_Dec2Long", "0x290B", "_HexIt",   "0x2A0B", 

  /** Text : 0x0C **/
  "_TextBootInit",   "0x010C", "_TextStartUp",    "0x020C", "_TextShutDown",  "0x030C", "_TextVersion",    "0x040C", "_TextReset",        "0x050C", "_TextStatus",     "0x060C", "_SetInGlobals",   "0x090C", "_SetOutGlobals",   "0x0A0C", "_SetErrGlobals", "0x0B0C", 
  "_GetInGlobals",   "0x0C0C", "_GetOutGlobals",  "0x0D0C", "_GetErrGlobals", "0x0E0C", "_SetInputDevice", "0x0F0C",  "_SetOutputDevice", "0x100C", "_SetErrorDevice", "0x110C", "_GetInputDevice", "0x120C", "_GetOutputDevice", "0x130C", 
  "_GetErrorDevice", "0x140C", "_InitTextDev",    "0x150C", "_CtlTextDev",    "0x160C", "_StatusTextDev",  "0x170C", "_WriteChar",        "0x180C", "_ErrWriteChar",   "0x190C", "_WriteLine",      "0x1A0C", "_ErrWriteLine",    "0x1B0C", "_WriteString",   "0x1C0C", 
  "_ErrWriteString", "0x1D0C", "_TextWriteBlock", "0x1E0C", "_ErrWriteBlock", "0x1F0C", "_WriteCString",   "0x200C", "_ErrWriteCString",  "0x210C", "_ReadChar",       "0x220C", "_TextReadBlock",  "0x230C", "_ReadLine",        "0x240C", 

  /** RAMDisk : 0x0D **/
  "_RDBootInit", "0x010D", "_RDStartUp",   "0x020D", "_RDShutDown",    "0x030D", "_RDVersion",     "0x040D", "_RDReset", "0x050D", "_RDStatus", "0x060D",
  "_Kill",       "0x090D", "_RDExecPCCmd", "0x0A0D", "_RDSetDiskSize", "0x0B0D", "_RDGetDiskSize", "0x0C0D",

  /** Window_Manager : 0x0E **/
  "_WindBootInit",      "0x010E", "_WindStartUp",   "0x020E", "_WindShutDown",      "0x030E", "_WindVersion",         "0x040E", "_WindReset",        "0x050E", "_WindStatus",        "0x060E", "_NewWindow",       "0x090E", "_CheckUpdate",   "0x0A0E", "_CloseWindow",    "0x0B0E", 
  "_Desktop",           "0x0C0E", "_SetWTitle",     "0x0D0E", "_GetWTitle",         "0x0E0E", "_SetFrameColor",       "0x0F0E", "_GetFrameColor",    "0x100E", "_SelectWindow",      "0x110E", "_HideWindow",      "0x120E", "_ShowWindow",    "0x130E", "_SendBehind",     "0x140E", 
  "_FrontWindow",       "0x150E", "_SetInfoDraw",   "0x160E", "_FindWindow",        "0x170E", "_TrackGoAway",         "0x180E", "_MoveWindow",       "0x190E", "_DragWindow",        "0x1A0E", "_GrowWindow",      "0x1B0E", "_SizeWindow",    "0x1C0E", "_TaskMaster",     "0x1D0E", 
  "_BeginUpdate",       "0x1E0E", "_EndUpdate",     "0x1F0E", "_GetWMgrPort",       "0x200E", "_PinRect",             "0x210E", "_HiliteWindow",     "0x220E", "_ShowHide",          "0x230E", "_BringToFront",    "0x240E", "_WindNewRes",    "0x250E", "_TrackZoom",      "0x260E", "_ZoomWindow", "0x270E", 
  "_SetWRefCon",        "0x280E", "_GetWRefCon",    "0x290E", "_GetNextWindow",     "0x2A0E", "_GetWKind",            "0x2B0E", "_SetWFrame",        "0x2D0E", "_GetWFrame",         "0x2C0E", "_GetStructRgn",    "0x2E0E", "_GetContentRgn", "0x2F0E", "_GetUpdateRgn",   "0x300E", 
  "_GetDefProc",        "0x310E", "_SetDefProc",    "0x320E", "_GetWControls",      "0x330E", "_SetOriginMask",       "0x340E", "_GetInfoRefCon",    "0x350E", "_SetInfoRefCon",     "0x360E", "_GetZoomRect",     "0x370E", "_SetZoomRect",   "0x380E", "_RefreshDesktop", "0x390E", 
  "_InvalRect",         "0x3A0E", "_InvalRgn",      "0x3B0E", "_ValidRect",         "0x3C0E", "_ValidRgn",            "0x3D0E", "_GetContentOrigin", "0x3E0E", "_SetContentOrigin",  "0x3F0E", "_GetDataSize",     "0x400E", "_SetDataSize",   "0x410E", "_GetMaxGrow",     "0x420E", 
  "_SetMaxGrow",        "0x430E", "_GetScroll",     "0x440E", "_SetScroll",         "0x450E", "_GetPage",             "0x460E", "_SetPage",          "0x470E", "_GetContentDraw",    "0x480E", "_SetContentDraw",  "0x490E", "_GetInfoDraw",   "0x4A0E", "_SetSysWindow",   "0x4B0E", 
  "_GetSysWFlag",       "0x4C0E", "_StartDrawing",  "0x4D0E", "_SetWindowIcons",    "0x4E0E", "_GetRectInfo",         "0x4F0E", "_StartInfoDrawing", "0x500E", "_EndInfoDrawing",    "0x510E", "_GetFirstWindow",  "0x520E", "_WindDragRect",  "0x530E", "_Private01",      "0x540E", 
  "_DrawInfoBar",       "0x550E", "_WindowGlobal",  "0x560E", "_SetContentOrigin2", "0x570E", "_GetWindowMgrGlobals", "0x580E", "_AlertWindow",      "0x590E", "_StartFrameDrawing", "0x5A0E", "_EndFrameDrawing", "0x5B0E", "_ResizeWindow",  "0x5C0E", 
  "_TaskMasterContent", "0x5D0E", "_TaskMasterKey", "0x5E0E", "_TaskMasterDA",      "0x5F0E", "_CompileText",         "0x600E", "_NewWindow2",       "0x610E", "_ErrorWindow",       "0x620E", "_GetAuxWindInfo",  "0x630E", "_DoModalWindow", "0x640E", "_MWGetCtlPart",   "0x650E", 
  "_MWSetMenuProc",     "0x660E", "_MWStdDrawProc", "0x670E", "_MWSetUpEditMenu",   "0x680E", "_FindCursorCtl",       "0x690E", "_ResizeInfoBar",    "0x6A0E", "_HandleDiskInsert",  "0x6B0E", "_UpdateWindow",    "0x6C0E",

  /** Menu Manager : 0x0F **/
  "_MenuBootInit",      "0x010F", "_MenuStartUp",   "0x020F", "_MenuShutDown",    "0x030F", "_MenuVersion",      "0x040F", "_MenuReset",        "0x050F", "_MenuStatus",     "0x060F", "_MenuKey",        "0x090F", "_GetMenuBar",     "0x0A0F", "_MenuRefresh",       "0x0B0F", 
  "_FlashMenuBar",      "0x0C0F", "_InsertMenu",    "0x0D0F", "_DeleteMenu",      "0x0E0F", "_InsertMItem",      "0x0F0F", "_DeleteMItem",      "0x100F", "_GetSysBar",      "0x110F", "_SetSysBar",      "0x120F", "_FixMenuBar",     "0x130F", "_CountMItems",       "0x140F", 
  "_NewMenuBar",        "0x150F", "_GetMHandle",    "0x160F", "_SetBarColors",    "0x170F", "_GetBarColors",     "0x180F", "_SetMTitleStart",   "0x190F", "_GetMTitleStart", "0x1A0F", "_GetMenuMgrPort", "0x1B0F", "_CalcMenuSize",   "0x1C0F", "_SetMTitleWidth",    "0x1D0F", 
  "_GetMTitleWidth",    "0x1E0F", "_SetMenuFlag",   "0x1F0F", "_GetMenuFlag",     "0x200F", "_SetMenuTitle",     "0x210F", "_GetMenuTitle",     "0x220F", "_MenuGlobal",     "0x230F", "_SetMItem",       "0x240F", "_GetMItem",       "0x250F", "_SetMItemFlag",      "0x260F", 
  "_GetMItemFlag",      "0x270F", "_SetMItemBlink", "0x280F", "_MenuNewRes",      "0x290F", "_DrawMenuBar",      "0x2A0F", "_MenuSelect",       "0x2B0F", "_HiliteMenu",     "0x2C0F", "_NewMenu",        "0x2D0F", "_DisposeMenu",    "0x2E0F", "_InitPalette",       "0x2F0F", 
  "_EnableMItem",       "0x300F", "_DisableMItem",  "0x310F", "_CheckMItem",      "0x320F", "_SetMItemMark",     "0x330F", "_GetMItemMark",     "0x340F", "_SetMItemStyle",  "0x350F", "_GetMItemStyle",  "0x360F", "_SetMenuID",      "0x370F", "_SetMItemID",        "0x380F", 
  "_SetMenuBar",        "0x390F", "_SetMItemName",  "0x3A0F", "_GetPopUpDefProc", "0x3B0F", "_PopUpMenuSelect",  "0x3C0F", "_DrawPopUp",        "0x3D0F", "_NewMenu2",       "0x3E0F", "_InsertMItem2",   "0x3F0F", "_SetMenuTitle2",  "0x400F", "_SetMItem2",         "0x410F", 
  "_SetMItemName2",     "0x420F", "_NewMenuBar2",   "0x430F", "_GetResMTitle",    "0x440F", "_HideMenuBar",      "0x450F", "_ShowMenuBar",      "0x460F", "_SetMItemIcon",   "0x470F", "_GetMItemIcon",   "0x480F", "_SetMItemStruct", "0x490F", "_GetMItemStruct",    "0x4A0F", 
  "_RemoveMItemStruct", "0x4B0F", "_GetMItemFlag2", "0x4C0F", "_SetMItemFlag2",   "0x4D0F", "_GetMItemBlink",    "0x4F0F", "_InsertPathMItems", "0x500F", 

  /** Control Manager : 0x10 **/
  "_CtlBootInit",        "0x0110", "_CtlStartUp",        "0x0210", "_CtlShutDown",     "0x0310", "_CtlVersion",        "0x0410", "_CtlReset",        "0x0510", "_CtlStatus",      "0x0610", "_NewControl",   "0x0910", "_DisposeControl",  "0x0A10", "_KillControls",    "0x0B10", 
  "_SetCtlTitle",        "0x0C10", "_GetCtlTitle",       "0x0D10", "_HideControl",     "0x0E10", "_ShowControl",       "0x0F10", "_DrawControls",    "0x1010", "_HiliteControl",  "0x1110", "_CtlNewRes",    "0x1210", "_FindControl",     "0x1310", "_TestControl",     "0x1410", 
  "_TrackControl",       "0x1510", "_MoveControl",       "0x1610", "_DragControl",     "0x1710", "_SetCtlIcons",       "0x1810", "_SetCtlValue",     "0x1910", "_GetCtlValue",    "0x1A10", "_SetCtlParams", "0x1B10", "_GetCtlParams",    "0x1C10", "_DragRect",        "0x1D10", 
  "_GrowSize",           "0x1E10", "_GetCtlDpage",       "0x1F10", "_SetCtlAction",    "0x2010", "_GetCtlAction",      "0x2110", "_SetCtlRefCon",    "0x2210", "_GetCtlRefCon",   "0x2310", "_EraseControl", "0x2410", "_DrawOneCtl",      "0x2510", "_FindTargetCtl",   "0x2610", 
  "_MakeNextCtlTarget",  "0x2710", "_MakeThisCtlTarget", "0x2810", "_SendEventToCtl",  "0x2910", "_GetCtlID",          "0x2A10", "_SetCtlID",        "0x2B10", "_CallCtlDefProc", "0x2C10", "_NotifyCtls",   "0x2D10", "_GetCtlMoreFlags", "0x2E10", "_SetCtlMoreFlags", "0x2F10", 
  "_GetCtlHandleFromID", "0x3010", "_NewControl2",       "0x3110", "_CMLoadResource",  "0x3210", "_CMReleaseResource", "0x3310", "_SetCtlParamPtr",  "0x3410", "_GetCtlParamPtr", "0x3510", "_InvalCtls",    "0x3710", "_FindRadioButton", "0x3910", 
  "_SetLETextByID",      "0x3A10", "_GetLETextByID",     "0x3B10", "_SetCtlValueByID", "0x3C10", "_GetCtlValueByID",   "0x3D10", "_InvalOneCtlByID", "0x3E10", "_HiliteCtlByID",  "0x3F10", 

  /** Load Manager : 0x11 **/
  "_LoaderInitialization", "0x0111", "_LoaderStartUp", "0x0211", "_LoaderShutDown", "0x0311", "_LoaderVersion", "0x0411", "_LoaderReset", "0x0511", "_LoaderStatus",   "0x0611", "_InitialLoad", "0x0911", "_InitialLoad2", "0x2011", 
  "_Restart",              "0x0A11", "_LoadSegNum",    "0x0B11", "_UnloadSegNum",   "0x0C11", "_LoadSegName",   "0x0D11", "_UnloadSeg",   "0x0E11", "_GetLoadSegInfo", "0x0F11", "_GetUserID",   "0x1011", "_GetUserID2",   "0x2111", "_LGetPathname", "0x1111", 
  "_LGetPathname2",        "0x2211", "_UserShutDown",  "0x1211", "_RenamePathname", "0x1311", 

  /** QuickDraw Aux : 0x12 **/
  "_QDAuxBootInit",     "0x0112", "_QDAuxStartUp", "0x0212", "_QDAuxShutDown", "0x0312", "_QDAuxVersion", "0x0412", "_QDAuxReset",   "0x0512", "_QDAuxStatus", "0x0612", "_CopyPixels", "0x0912", "_WaitCursor",      "0x0A12", "_DrawIcon",      "0x0B12", 
  "_SpecialRect",       "0x0C12", "_SeedFill",     "0x0D12", "_CalcMask",      "0x0E12", "_GetSysIcon",   "0x0F12", "_PixelMap2Rgn", "0x1012", "_IBeamCursor", "0x1312", "_WhooshRect", "0x1412", "_DrawStringWidth", "0x1512", "_UseColorTable", "0x1612", 
  "_RestoreColorTable", "0x1712", 
  
  /** Print Manager : 0x13 **/
  "_PMBootInit",    "0x0113", "_PMStartUp",          "0x0213", "_PMShutDown",           "0x0313", "_PMVersion",    "0x0413", "_PMReset",       "0x0513", "_PMStatus",       "0x0613", "_PrDefault",           "0x0913", "_PrValidate",       "0x0A13", "_PrStlDialog",  "0x0B13", "_PrJobDialog",     "0x0C13", 
  "_PrPixelMap",    "0x0D13", "_PrOpenDoc",          "0x0E13", "_PrCloseDoc",           "0x0F13", "_PrOpenPage",   "0x1013", "_PrClosePage",   "0x1113", "_PrPicFile",      "0x1213", "_PrControl",           "0x1313", "_PrError",          "0x1413", "_PrSetError",   "0x1513", "_PrChoosePrinter", "0x1613", 
  "_GetDeviceName", "0x1703", "_PrGetPrinterSpecs",  "0x1813", "_PrDevPrChanged",       "0x1913", "_PrDevStartup", "0x1A13", "_PrDevShutDown", "0x1B13", "_PrDevOpen",      "0x1C13", "_PrDevRead",           "0x1D13", "_PrDevWrite",       "0x1E13", "_PrDevClose",   "0x1F13", 
  "_PrDevStatus",   "0x2013", "_PrDevAsyncRead",     "0x2113", "_PrDevWriteBackground", "0x2213", "_PrDriverVer",  "0x2313", "_PrPortVer",     "0x2413", "_PrGetZoneName",  "0x2513", "_PrGetPrinterDvrName", "0x2813", "_PrGetPortDvrName", "0x2913", 
  "_PrGetUserName", "0x2A13", "_PrGetNetworkName",   "0x2B13", "_PrDevIsItSafe",        "0x3013", "_GetZoneList",  "0x3113", "_GetMyZone",     "0x3213", "_GetPrinterList", "0x3313", "_PMUnloadDriver",      "0x3413", "_PMLoadDriver",     "0x3513", "_PrGetDocName", "0x3613", 
  "_PrSetDocName",  "0x3713", "_PrGetPgOrientation", "0x3813", 

  /** LineEdit : 0x14 **/
  "_LEBootInit", "0x0114", "_LEStartUp",   "0x0214", "_LEShutDown",    "0x0314", "_LEVersion",    "0x0414", "_LEReset",       "0x0514", "_LEStatus",      "0x0614", "_LENew",         "0x0914", "_LEDispose",   "0x0A14", "_LESetText",  "0x0B14", "_LEIdle",   "0x0C14", 
  "_LEClick",    "0x0D14", "_LESetSelect", "0x0E14", "_LEActivate",    "0x0F14", "_LEDeactivate", "0x1014", "_LEKey",         "0x1114", "_LECut",         "0x1214", "_LECopy",        "0x1314", "_LEPaste",     "0x1414", "_LEDelete",   "0x1514", "_LEInsert", "0x1614", 
  "_LEUpdate",   "0x1714", "_LETextBox",   "0x1814", "_LEFromScrap",   "0x1914", "_LEToScrap",    "0x1A14", "_LEScrapHandle", "0x1B14", "_LEGetScrapLen", "0x1C14", "_LESetScrapLen", "0x1D14", "_LESetHilite", "0x1E14", "_LESetCaret", "0x1F14", 
  "_LETextBox2", "0x2014", "_LESetJust",   "0x2114", "_LEGetTextHand", "0x2214", "_LEGetTextLen", "0x2314", "_GetLEDefProc",  "0x2414", "_LEClassifyKey", "0x2514", 

  /** Dialog Manager : 0x15 **/
  "_DialogBootInit",    "0x0115", "_DialogStartUp", "0x0215", "_DialogShutDown",  "0x0315", "_DialogVersion", "0x0415", "_DialogReset",  "0x0515", "_DialogStatus",  "0x0615", "_ErrorSound",    "0x0915", "_NewModalDialog",    "0x0A15",
  "_NewModelessDialog", "0x0B15", "_CloseDialog",   "0x0C15", "_NewDItem",        "0x0D15", "_RemoveDItem",   "0x0E15", "_ModalDialog",  "0x0F15", "_IsDialogEvent", "0x1015", "_DialogSelect",  "0x1115", "_DlgCut",            "0x1215", "_DlgCopy",      "0x1315",
  "_DlgPaste",          "0x1415", "_DlgDelete",     "0x1515", "_DrawDialog",      "0x1615", "_Alert",         "0x1715", "_StopAlert",    "0x1815", "_NoteAlert",     "0x1915", "_CautionAlert",  "0x1A15", "_ParamText",         "0x1B15", "_SetDAFont",    "0x1C15", "_GetControlDItem", "0x1E15",
  "_GetIText",          "0x1F15", "_SetIText",      "0x2015", "_SelectIText",     "0x2115", "_HideDItem",     "0x2215", "_ShowDItem",    "0x2315", "_FindDItem",     "0x2415", "_UpdateDialog",  "0x2515", "_GetDItemType",      "0x2615", "_SetDItemType", "0x2715",
  "_GetDItemBox",       "0x2815", "_SetDItemBox",   "0x2915", "_GetFirstDItem",   "0x2A15", "_GetNextDItem",  "0x2B15", "_ModalDialog2", "0x2C15", "_GetDItemValue", "0x2E15", "_SetDItemValue", "0x2F15", "_GetNewModalDialog", "0x3215",
  "_GetNewDItem",       "0x3315", "_GetAlertStage", "0x3415", "_ResetAlertStage", "0x3515", "_DefaultFilter", "0x3615", "_GetDefButton", "0x3715", "_SetDefButton",  "0x3815", "_DisableDItem",  "0x3915", "_EnableDItem",       "0x3A15",

  /** Scrap manager : 0x16 **/
  "_ScrapBootInit", "0x0116", "_ScrapStartUp", "0x0216", "_ScrapShutDown",  "0x0316", "_ScrapVersion", "0x0416", "_ScrapReset",   "0x0516", "_ScrapStatus",  "0x0616", "_UnloadScrap",   "0x0916", "_LoadScrap",     "0x0A16", "_ZeroScrap",   "0x0B16", 
  "_PutScrap",      "0x0C16", "_GetScrap",     "0x0D16", "_GetScrapHandle", "0x0E16", "_GetScrapSize", "0x0F16", "_GetScrapPath", "0x1016", "_SetScrapPath", "0x1116", "_GetScrapCount", "0x1216", "_GetScrapState", "0x1316", "_GetIndScrap", "0x1416", 
  "_ShowClipboard", "0x1516", 

  /** Standard File Operations : 0x17 **/
  "_SFBootInit", "0x0117", "_SFStartUp",  "0x0217", "_SFShutDown", "0x0317", "_SFVersion",   "0x0417", "_SFReset",     "0x0517", "_SFStatus",        "0x0617", "_SFGetFile", "0x0917", "_SFPutFile",   "0x0A17", "_SFPGetFile",   "0x0B17", "_SFPPutFile", "0x0C17", 
  "_SFAllCaps",  "0x0D17", "_SFGetFile2", "0x0E17", "_SFPutFile2", "0x0F17", "_SFPGetFile2", "0x1017", "_SFPPutFile2", "0x1117", "_SFShowInvisible", "0x1217", "_SFReScan",  "0x1317", "_SFMultiGet2", "0x1417", "_SFPMultiGet2", "0x1517", 

  /** Disk Utilities : 0x18 **/

  /** Note synthesizer : 0x19 **/
  "_NSBootInit",  "0x0119", "_NSStartUp",       "0x0219", "_NSShutDown",         "0x0319", "_NSVersion", "0x0419", "_NSReset", "0x0519", "_NSStatus", "0x0619", "_AllocGen", "0x0919", "_DeallocGen", "0x0A19", "_NoteOn", "0x0B19", "_NoteOff", "0x0C19", 
  "_AllNotesOff", "0x0D19", "_NSSetUpdateRate", "0x0E19", "_NSSetUserUpdateRtn", "0x0F19", 

  /** Note sequencer : 0x1A **/
  "_SeqBootInit",    "0x011A", "_SeqStartUp", "0x021A", "_SeqShutDown", "0x031A", "_SeqVersion", "0x041A", "_SeqReset", "0x051A", "_SeqStatus",    "0x061A", "_SetIncr",   "0x091A", "_ClearIncr", "0x0A1A", "_GetTimer",    "0x0B1A", "_GetLoc", "0x0C1A", 
  "_SeqAllNotesOff", "0x0D1A", "_SetTrkInfo", "0x0E1A", "_StartSeq",    "0x0F1A", "_StepSeq",    "0x101A", "_StopSeq",  "0x111A", "_SetInstTable", "0x121A", "_StartInts", "0x131A", "_StopInts",  "0x141A", "_StartSeqRel", "0x151A", 

  /** Font Manager : 0x1B **/
  "_FMBootInit",  "0x011B", "_FMStartUp",  "0x021B", "_FMShutDown",    "0x031B", "_FMVersion",    "0x041B", "_FMReset",     "0x051B", "_FMStatus",      "0x061B", "_CountFamilies", "0x091B", "_FindFamily",       "0x0A1B", "_GetFamInfo", "0x0B1B", 
  "_GetFamNum",   "0x0C1B", "_AddFamily",  "0x0D1B", "_InstallFont",   "0x0E1B", "_SetPurgeStat", "0x0F1B", "_CountFonts",  "0x101B", "_FindFontStats", "0x111B", "_LoadFont",      "0x121B", "_LoadSysFont",      "0x131B", "_AddFontVar", "0x141B", 
  "_FixFontMenu", "0x151B", "_ChooseFont", "0x161B", "_ItemID2FamNum", "0x171B", "_FMSetSysFont", "0x181B", "_FMGetSysFID", "0x191B", "_FMGetCurFID",   "0x1A1B", "_FamNum2ItemID", "0x1B1B", "_InstallWithStats", "0x1C1B", 

  /** List Manager : 0x1C **/
  "_ListBootInit", "0x011C", "_ListStartup",  "0x021C", "_ListShutDown",   "0x031C", "_ListVersion",    "0x041C", "_ListReset", "0x051C", "_ListStatus",  "0x061C", "_CreateList",  "0x091C", "_SortList",     "0x0A1C", "_NextMember",    "0x0B1C", 
  "_DrawMember",   "0x0C1C", "_SelectMember", "0x0D1C", "_GetListDefProc", "0x0E1C", "_ResetMember",    "0x0F1C", "_NewList",   "0x101C", "_DrawMember2", "0x111C", "_NextMember2", "0x121C", "_ResetMember2", "0x131C", "_SelectMember2", "0x141C", 
  "_SortList2",    "0x151C", "_NewList2",     "0x161C", "_ListKey",        "0x171C", "_CompareStrings", "0x181C", 

  /** ACE : 0x1D **/
  "_ACEBootInit",  "0x011D", "_ACEStartUp",  "0x021D", "_ACEShutDown",    "0x031D", "_ACEVersion",     "0x041D", "_ACEReset", "0x051D", "_ACEStatus", "0x061D", "_ACEInfo", "0x071D", "_ACECompress", "0x091D", "_ACEExpand", "0x0A1D",
  "_ACECompBegin", "0x0B1D", "_ACEExpBegin", "0x0C1D", "_GetACEExpState", "0x0D1D", "_SetACEExpState", "0x0E1D",

  /** Resource : 0x1E **/
  "_ResourceBootInit",     "0x011E", "_ResourceStartUp",   "0x021E", "_ResourceShutDown",    "0x031E", "_ResourceVersion",      "0x041E", "_ResourceReset",        "0x051E", "_ResourceStatus",      "0x061E", "_CreateResourceFile", "0x091E", "_OpenResourceFile",   "0x0A1E", 
  "_CloseResourceFile",    "0x0B1E", "_AddResource",       "0x0C1E", "_UpdateResourceFile",  "0x0D1E", "_LoadResource",         "0x0E1E", "_RemoveResource",       "0x0F1E", "_MarkResourceChange",  "0x101E", "_SetCurResourceFile", "0x111E", "_GetCurResourceFile", "0x121E", 
  "_SetCurResourceApp",    "0x131E", "_GetCurResourceApp", "0x141E", "_HomeResourceFile",    "0x151E", "_WriteResource",        "0x161E", "_ReleaseResource",      "0x171E", "_DetachResource",      "0x181E", "_UniqueResourceID",   "0x191E", 
  "_SetResourceID",        "0x1A1E", "_GetResourceAttr",   "0x1B1E", "_SetResourceAttr",     "0x1C1E", "_GetResourceSize",      "0x1D1E", "_MatchResourceHandle",  "0x1E1E", "_GetOpenFileRefNum",   "0x1F1E", "_CountTypes",         "0x201E", "_GetIndType",         "0x211E", 
  "_CountResources",       "0x221E", "_GetIndResource",    "0x231E", "_SetResourceLoad",     "0x241E", "_SetResourceFileDepth", "0x251E", "_GetMapHandle",         "0x261E", "_LoadAbsResource",     "0x271E", "_ResourceConverter",  "0x281E", "_LoadResource2",      "0x291E", 
  "_RMFindeNamedResource", "0x2A1E", "_RMGetResourceName", "0x2B1E", "_RMLoadNamedResource", "0x2C1E", "_RMSetResourceName",    "0x2D1E", "_OpenResourceFileByID", "0x2E1E", "_CompactResourceFile", "0x2F1E", 

  /** MIDI : 0x20 **/
  "_MidiBootInit", "0x0120", "_MidiStartUp",    "0x0220", "_MidiShutDown",    "0x0320", "_MidiVersion",   "0x0420", "_MidiReset",      "0x0520", "_MidiStatus",  "0x0620", "_MidiControl",  "0x0920", "_MidiDevice",  "0x0A20", "_MidiClock", "0x0B20", 
  "_MidiInfo",     "0x0C20", "_MidiReadPacket", "0x0D20", "_MidiWritePacket", "0x0E20", "_MidiRecordSeq", "0x0F20", "_MidiStopRecord", "0x1020", "_MidiPlaySeq", "0x1120", "_MidiStopPlay", "0x1220", "_MidiConvert", "0x1320", 

  /** Video Overlay : 0x21 **/
  "_VDBootInit",     "0x0121", "_VDStartUp",   "0x0221", "_VDShutDown",   "0x0321", "_VDVersion",     "0x0421", "_VDReset",       "0x0521", "_VDStatus",      "0x0621", "_VDInStatus",    "0x0921", "_VDInSetStd",    "0x0A21", "_VDInGetStd",     "0x0B21", "_VDInConvAdj", "0x0C21", 
  "_VDKeyControl",   "0x0D21", "_VDKeyStatus", "0x0E21", "_VDKeySetKCol", "0x0F21", "_VDKeyGetKRCol", "0x1021", "_VDKeyGetKGCol", "0x1121", "_VDKeyGetKBCol", "0x1221", "_VDKeySetKDiss", "0x1321", "_VDKeyGetKDiss", "0x1421", "_VDKeySetNKDiss", "0x1521", 
  "_VDKeyGetNKDiss", "0x1621", "_VDOutSetStd", "0x1721", "_VDOutGetStd",  "0x1821", "_VDOutControl",  "0x1921", "_VDOutStatus",   "0x1A21", "_VDGetFeatures", "0x1B21", "_VDInControl",   "0x1C21", "_VDGGControl",   "0x1D21", "_VDGGStatus",     "0x1E21", 

  /** TextEdit : 0x22 **/
  "_TEBootInit",      "0x0122", "_TEStartUp",    "0x0222", "_TEShutDown", "0x0322", "_TEVersion",    "0x0422", "_TEReset",   "0x0522", "_TEStatus",          "0x0622", "_TENew",          "0x0922", "_TEKill",              "0x0A22", "_TESetText",     "0x0B22", "_TEGetText",       "0x0C22", 
  "_TEGetTextInfo",   "0x0D22", "_TEIdle",       "0x0E22", "_TEActivate", "0x0F22", "_TEDeactivate", "0x1022", "_TEClick",   "0x1122", "_TEUpdate",          "0x1222", "_TEPaintText",    "0x1322", "_TEKey",               "0x1422", "_TEUndo",        "0x1522", "_TECut",           "0x1622", 
  "_TECopy",          "0x1722", "_TEPaste",      "0x1822", "_TEClear",    "0x1922", "_TEInsert",     "0x1A22", "_TEReplace", "0x1B22", "_TEGetSelection",    "0x1C22", "_TESetSelection", "0x1D22", "_TEGetSelectionStyle", "0x1E22", "_TEStyleChange", "0x1F22", "_TEOffsetToPoint", "0x2022", 
  "_TEPointToOffset", "0x2122", "_TEGetDefProc", "0x2222", "_TEGetRuler", "0x2322", "_TESetRuler",   "0x2422", "_TEScroll",  "0x2522", "_TEGetInternalProc", "0x2622", "_TEGetLastError", "0x2722", "_TECompactRecord",     "0x2822", 

  /** Midi Synth : 0x23 **/
  "_MSBootInit",       "0x0123", "_MSStartUp",     "0x0223", "_MSShutDown",       "0x0323", "_MSVersion",    "0x0423", "_MSReset",        "0x0523", "_MSStatus",       "0x0623", "_SetBasicChannel", "0x0923", "_SetMIDIMode", "0x0A23", "_PlayNote",       "0x0B23", 
  "_StopNote",         "0x0C23", "_KillAllNotes",  "0x0D23", "_SetRecTrack",      "0x0E23", "_SetPlayTrack", "0x0F23", "_TrackToChannel", "0x1023", "_Locate",         "0x1123", "_SetVelComp",      "0x1223", "_SetMIDIPort", "0x1323", "_SetInstrument",  "0x1423", 
  "_SeqPlayer",        "0x1523", "_SetTempo",      "0x1623", "_SetCallBack",      "0x1723", "_SysExOut",     "0x1823", "_SetBeat",        "0x1923", "_MIDIMessage",    "0x1A23", "_LocateEnd",       "0x1B23", "_Merge",       "0x1C23", "_DeleteTrack",    "0x1D23", "_SetMetro", "0x1E23", 
  "_GetMSData",        "0x1F23", "_ConvertToTime", "0x2023", "_ConvertToMeasure", "0x2123", "_MSSuspend",    "0x2223", "_MSResume",       "0x2323", "_SetTuningTable", "0x2423", "_GetTuningTable",  "0x2523", "_SetTrackOut", "0x2623", "_InitMIDIDriver", "0x2723", 
  "_RemoveMIDIDriver", "0x2823", 

  /** Anim Tool Set : 0x25 **/
  "_AnimBootInit",    "0x0125", "_AnimStartUp",    "0x0225", "_AnimShutDown",   "0x0325", "_AnimVersion", "0x0425", "_AnimReset",     "0x0525", "_AnimStatus",   "0x0625", "_AnimIdleDebug",  "0x0825", "_StartScene",    "0x0925", "_StopScene", "0x0A25",
  "_StartFrameTimer", "0x0B25", "_StopFrameTimer", "0x0C25", "_SetBackGndPort", "0x0D25", "_RefreshBack", "0x0E25", "_StartChar",     "0x0F25", "_MoveChar",     "0x1025", "_GetCharRecPtr",  "0x1125", "_KillChar",      "0x1225", "_LoadActor", "0x1325",
  "_SetCharScript",   "0x1425", "_RunAnimScripts", "0x1525", "_FillAddrTable",  "0x1625", "_CompileRect", "0x1725", "_StartTockTask", "0x1825", "_FireTockTask", "0x1925", "_SetForeGndPort", "0x1A25", "_SetAnimWindow", "0x1B25",

  /** Media Controller : 0x26 **/
  "_MCBootInit",  "0x0126", "_MCStartUp",    "0x0226", "_MCShutDown",      "0x0326", "_MCVersion",       "0x0426", "_MCReset",      "0x0526", "_MCStatus",      "0x0626", "_MCGetErrorMsg",  "0x0926", "_MCLoadDriver",   "0x0A26", "_MCUnLoadDriver", "0x0B26", 
  "_MCTime2Bin",  "0x0C26", "_MCBin2Time",   "0x0D26", "_MCGetTrackTitle", "0x0E26", "_MCSetTrackTitle", "0x0F26", "_MCGetProgram", "0x1026", "_MCSetProgram",  "0x1126", "_MCGetDiscTitle", "0x1226", "_MCSetDiscTitle", "0x1326", 
  "_MCDStartUp",  "0x1426", "_MCDShutDown",  "0x1526", "_MCGetFeatures",   "0x1626", "_MCPlay",          "0x1726", "_MCPause",      "0x1826", "_MCSendRawData", "0x1926", "_MCGetStatus",    "0x1A26", "_MCControl",      "0x1B26", "_MCScan",         "0x1C26", 
  "_MCGetSpeeds", "0x1D26", "_MCSpeed",      "0x1E26", "_MCStopAt",        "0x1F26", "_MCJog",           "0x2026", "_MCSearchTo",   "0x2126", "_MCSearchDone",  "0x2226", "_MCSearchWait",   "0x2326", "_MCGetPosition",  "0x2426", "_MCSetAudio",     "0x2526", 
  "_MCGetTimes",  "0x2626", "_MCGetDiscTOC", "0x2726", "_MCGetDiscID",     "0x2826", "_MCGetNoTracks",   "0x2926", "_MCRecord",     "0x2A26", "_MCStop",        "0x2B26", "_MCWaitRawData",  "0x2C26", "_MCGetName",      "0x2D26", "_MCSetVolume",    "0x2E26", 

  /** Male Voice : 0x32 **/
  "_MaleBootInit", "0x0132", "_MaleStartUp", "0x0232", "_MaleShutDown", "0x0332", "_MaleVersion", "0x0432", "_MaleReset", "0x0532", "_MaleStatus", "0x0632", "_MaleSpeak", "0x0932", 

  /** Female Voice : 0x33 **/
  "_FemaleBootInit", "0x0133", "_FemaleStartUp", "0x0233", "_FemaleShutDown", "0x0333", "_FemaleVersion", "0x0433", "_FemaleReset", "0x0533", "_FemaleStatus", "0x0633", "_FemaleSpeak", "0x0933", 

  /** Parser Speech Tool Set : 0x34 **/
  "_SpeechBootInit", "0x0134", "_SpeechStartUp", "0x0234", "_SpeechShutDown", "0x0334", "_SpeechVersion", "0x0434", "_SpeechReset", "0x0534", "_SpeechStatus", "0x0634", "_Parse", "0x0934", "_DictInsert", "0x0A34", "_DictDelete", "0x0B34", 
  "_DictDump",       "0x0C34", "_SetSayGlobals", "0x0D34", "_DictInit",       "0x0E34", "_Say",           "0x0F34", "_Activate",    "0x1034", 

  /** Speech Tool Set : 0x35 **/
  "_SpeechBootInit", "0x0135", "_SpeechStartUp", "0x0235", "_SpeechShutDown", "0x0335", "_SpeechVersion", "0x0435", "_SpeechReset", "0x0535", "_SpeechStatus", "0x0635",

  /** Marinetti TCP/IP : 0x36 **/
  "_TCPIPBootInit",            "0x0136", "_TCPIPStartUp",            "0x0236", "_TCPIPShutDown",            "0x0336", 
  "_TCPIPVersion",             "0x0436", "_TCPIPReset",              "0x0536", "_TCPIPStatus",              "0x0636", 
  "_TCPIPLongVersion",         "0x0836", "_TCPIPGetConnectStatus",   "0x0936", "_TCPIPGetErrorTable",       "0x0A36", 
  "_TCPIPGetReconnectStatus",  "0x0B36", "_TCPIPReconnect",          "0x0C36", "_TCPIPConvertIPToHex",      "0x0D36", 
  "_TCPIPConvertIPToASCII",    "0x0E36", "_TCPIPGetMyIPAddress",     "0x0F36", "_TCPIPGetConnectMethod",    "0x1036", 
  "_TCPIPSetConnectMethod",    "0x1136", "_TCPIPConnect",            "0x1236", "_TCPIPDisconnect",          "0x1336", 
  "_TCPIPGetMTU",              "0x1436", "_TCPIPValidateIPCString",  "0x1536", "_TCPIPGetConnectData",      "0x1636", 
  "_TCPIPSetConnectData",      "0x1736", "_TCPIPGetDisconnectData",  "0x1836", "_TCPIPSetDisconnectData",   "0x1936", 
  "_TCPIPLoadPreferences",     "0x1A36", "_TCPIPSavePreferences",    "0x1B36", "_TCPIPGetTuningTable",      "0x1C36", 
  "_TCPIPSetDNS",              "0x1D36", "_TCPIPGetDNS",             "0x1E36", "_TCPIPSetTuningTable",      "0x1F36", 
  "_TCPIPCancelDNR",           "0x2036", "_TCPIPDNRNameToIP",        "0x2136", "_TCPIPPoll",                "0x2236", 
  "_TCPIPLogin",               "0x2336", "_TCPIPLogout",             "0x2436", "_TCPIPSendICMP",            "0x2536", 
  "_TCPIPSendUDP",             "0x2636", "_TCPIPGetDatagramCount",   "0x2736", "_TCPIPGetNextDatagram",     "0x2836", 
  "_TCPIPGetLoginCount",       "0x2936", "_TCPIPSendICMPEcho",       "0x2A36", "_TCPIPReceiveICMPEcho",     "0x2B36", 
  "_TCPIPOpenTCP",             "0x2C36", "_TCPIPWriteTCP",           "0x2D36", "_TCPIPReadTCP",             "0x2E36", 
  "_TCPIPCloseTCP",            "0x2F36", "_TCPIPAbortTCP",           "0x3036", "_TCPIPStatusTCP",           "0x3136", 
  "_TCPIPGetSourcePort",       "0x3236", "_TCPIPGetTOS",             "0x3336", "_TCPIPSetTOS",              "0x3436", 
  "_TCPIPGetTTL",              "0x3536", "_TCPIPSetTTL",             "0x3636", "_TCPIPSetSourcePort",       "0x3736", 
  "_TCPIPConvertIPCToHex",     "0x3F36", "_TCPIPSendIPDatagram",     "0x4036", "_TCPIPConvertIPToClass",    "0x4136", 
  "_TCPIPGetConnectMsgFlag",   "0x4236", "_TCPIPSetConnectMsgFlag",  "0x4336", "_TCPIPGetUsername",         "0x4436", 
  "_TCPIPSetUsername",         "0x4536", "_TCPIPGetPassword",        "0x4636", "_TCPIPSetPassword",         "0x4736", 
  "_TCPIPValidateIPString",    "0x4836", "_TCPIPGetUserStatistic",   "0x4936", "_TCPIPGetLinkVariables",    "0x4A36", 
  "_TCPIPEditLinkConfig",      "0x4B36", "_TCPIPGetModuleNames",     "0x4C36", "_TCPIPListenTCP",           "0x4E36", 
  "_TCPIPAcceptTCP",           "0x4F36", "_TCPIPSetNewDestination",  "0x5036", "_TCPIPGetHostName",         "0x5136", 
  "_TCPIPSetHostName",         "0x5236", "_TCPIPStatusUDP",          "0x5336", "_TCPIPGetLinkLayer",        "0x5436", 
  "_TCPIPPtrToPtr",            "0x5536", "_TCPIPPtrToPtrNeg",        "0x5636", "_TCPIPGetAuthMessage",      "0x5736", 
  "_TCPIPConvertIPToCASCII",   "0x5836", "_TCPIPMangleDomainName",   "0x5936", "_TCPIPGetAliveFlag",        "0x5A36", 
  "_TCPIPSetAliveFlag",        "0x5B36", "_TCPIPGetAliveMinutes",    "0x5C36", "_TCPIPSetAliveMinutes",     "0x5D36", 
  "_TCPIPReadLineTCP",         "0x5E36", "_TCPIPGetBootConnectFlag", "0x5F36", "_TCPIPSetBootConnectFlag",  "0x6036", 
  "_TCPIPSetUDPDispatch",      "0x6136", "_TCPIPGetDestination",     "0x6236", "_TCPIPGetUserEventTrigger", "0x6336",  
  "_TCPIPSetUserEventTrigger", "0x6436", "_TCPIPGetSysEventTrigger", "0x6536", "_TCPIPSetSysEventTrigger",  "0x6636", 
  "_TCPIPGetDNRTimeouts",      "0x6736", "_TCPIPSetDNRTimeouts",     "0x6836",
  
  /** Time Tool Set : 0x38 **/
  "_tiBootInit",        "0x0138", "_tiStartUp",               "0x0238", "_tiShutDown",              "0x0338", "_tiVersion",           "0x0438", 
  "_tiReset",           "0x0538", "_tiStatus",                "0x0638", "_tiGetTimePrefs",          "0x0938", "_tiSetTimePrefs",      "0x0A38", 
  "_tiDateString2Sec",  "0x0B38", "_tiDateString2Epoch",      "0x0C38", "_tiToday2GMTString",       "0x0D38", "_tiToday2LocalString", "0x0E38", 
  "_tiParseDateString", "0x0F38", "_tiTimeZoneString2Offset", "0x1038", "_tiOffset2TimeZoneString", "0x1138", 

  /** MJuke Tool Set : 0x45 **/
  "_MJBootInit", "0x0145", "_MJStartUp", "0x0245", "_MJShutDown", "0x0345", "_MJVersion", "0x0445", "_MJReset", "0x0545", "_MJStatus", "0x0645",
  "_MJMount",    "0x0945", "_MJPlay",    "0x0A45", "_MJJuke",     "0x0B45", "_MJCont",    "0x0C45", "_MJStop",  "0x0D45", "_MJWave",   "0x0E45",
  "_MJNext",     "0x0F45", "_MJSong",    "0x1045", "_MJTimer",    "0x1145",

  /** Hash Tool : 0x80 **/
  "_hashBootInit", "0x0180", "_hashStartUp", "0x0280", "_hashShutDown", "0x0380", "_hashVersion", "0x0480", "_hashReset",  "0x0580",  "_hashStatus", "0x0680",
  "_md5Init",      "0x0980", "_md5Append",   "0x0A80", "_md5Finish",    "0x0B80", "_md4Init",     "0x0C80", "_md4Append",  "0x0D80",  "_md4Finish",  "0x0E80", 
  "_md2Init",      "0x0F80", "_md2Update",   "0x1080", "_md2Final",     "0x1180", "_sha1Init",    "0x1280", "_sha1Update", "0x1380",  "_sha1Final",  "0x1480",
  
  /** Crypto Tool Set : 0x81 **/
  "_cryptoBootInit", "0x0181", "_cryptoStartUp", "0x0281", "_cryptoShutDown", "0x0381", "_cryptoVersion", "0x0481", "_cryptoReset", "0x0581", "_cryptoStatus", "0x0681",
  "_desCipher",      "0x0981", "_desAddParity",  "0x0A81",

  /** HTML : 0x82 **/
  "_HTMLBootInit",    "0x0182", "_HTMLStartUp",      "0x0282", "_HTMLShutDown",       "0x0382", "_HTMLVersion",         "0x0482", "_HTMLReset",       "0x0582", "_HTMLStatus",      "0x0682", "_HTMLInit",         "0x0982", "_HTMLParse",          "0x0A82", 
  "_HTMLOpenDisplay", "0x0B82", "_HTMLCloseDisplay", "0x0C82", "_HTMLInsertTERecord", "0x0D82", "_HTMLClearDisplay",    "0x0E82", "_HTMLEventTask",   "0x0F82", "_HTMLGetTitle",    "0x1082", "_HTMLGoToName",     "0x1182", "_HTMLFindString",     "0x1282", 
  "_HTMLGetForm",     "0x1382", "_HTMLCheckForForm", "0x1482", "_HTMLGetError",       "0x1582", "_HTMLParse2",          "0x1682", "_HTMLEventTask2",  "0x1782", "_HTMLGetForm2",    "0x1882", "_HTMLOpenDisplay2", "0x1982", "_HTMLEventToDisplay", "0x1A82", 
  "_HTMLGetPage",     "0x1B82", "_HTMLPostForm",     "0x1C82", "_HTMLCacheControl",   "0x1D82", "_HTMLSetTextTERecord", "0x1E82", "_HTMLFindString2", "0x1F82", "_HTMLExtractText", "0x2082", "_HTMLWriteToLog",   "0x2182", "_HTMLSetLogPath",     "0x2282", 

  /** FTP : 0x83 **/
  "_FTPBootInit", "0x0183", "_FTPStartUp",     "0x0283", "_FTPShutDown",   "0x0383", "_FTPVersion",    "0x0483", "_FTPReset",     "0x0583", "_FTPStatus",  "0x0683",
  "_FTPSendFile", "0x0983", "_FTPReceiveFile", "0x0A83", "_FTPDeleteFile", "0x0B83", "_FTPRenameFile", "0x0C83", "_FTPDeleteDir", "0x0D83", "_FTPMakeDir", "0x0E83",
  "_FTPListDir",  "0x0F83", "_FTPLogIn",       "0x1083", "_FTPLogOut",     "0x1183",

  /** Undo Manager : 0x84 **/
  "_UMBootInit",      "0x0184", "_UMStartUp",          "0x0284", "_UMShutDown",  "0x0384", "_UMVersion",    "0x0484", "_UMReset",      "0x0584", "_UMStatus",         "0x0684",
  "_UMLogIn",         "0x0984", "_UMLogOut",           "0x0A84", "_UMKeyAction", "0x0B84", "_UMMenuAction", "0x0C84", "_UMSaveClip",   "0x0D84", "_UMClearClips",     "0x0E84",
  "_UMClear",         "0x0F84", "_UMClearWindowClips", "0x1084", "_UMLogOutAll", "0x1184", "_UMClipStatus", "0x1284", "_UMKeyAction2", "0x1384", "_UMPushUndoObject", "0x1484",
  "_UMPopUndoObject", "0x1584",

  /** SoundSmith : 0xDB **/
  "_STBootInit",   "0x01DB", "_STStartUp",       "0x02DB", "_STShutDown",   "0x03DB", "_STVersion",     "0x04DB", "_STReset",     "0x05DB", "_STStatus",          "0x06DB", "_STLoadOneMusic", "0x09DB", "_STPlayMusic",  "0x0ADB", 
  "_STStopMusic",  "0x0BDB", "_STGetEOfMusic",   "0x0CDB", "_STAddToBatch", "0x0DDB", "_STSelectBatch", "0x0EDB", "_STKillBatch", "0x0FDB", "_STGetPlayingMusic", "0x10DB", "_STPlayBatch",    "0x11DB", "_STGetTrackVu", "0x12DB", 
  "_STPauseMusic", "0x13DB", "_STContinueMusic", "0x14DB", "_STinternal15", "0x15DB", "_STinternal16",  "0x16DB", 

  /** NoiseTracker : 0xDC **/
  "_NTBootInit", "0x01DC", "_NTStartUp", "0x02DC", "_NTShutDown", "0x03DC", "_NTVersion", "0x04DC", "_NTReset", "0x05DC", "_NTStatus", "0x06DC", "_NTInitMusic", "0x09DC", "_NTLaunchMusic", "0x0ADC", "_NTUpdateSound", "0x0BDC", "_NTStopMusic", "0x0CDC", 

  /** NinjaTracker : 0xDD **/
  "_NTBootInit",    "0x01DD", "_NTStartUp",    "0x02DD", "_NTShutDown",    "0x03DD", "_NTVersion",   "0x04DD", "_NTReset",           "0x05DD", "_NTStatus",    "0x06DD", "_NTLoadOneMusic", "0x09DD", "_NTPlayMusic",  "0x0ADD", "_NTStopMusic",     "0x0BDD", 
  "_NTGetEOfMusic", "0x0CDD", "_NTAddToBatch", "0x0DDD", "_NTSelectBatch", "0x0EDD", "_NTKillBatch", "0x0FDD", "_NTGetPlayingMusic", "0x10DD", "_NTPlayBatch", "0x11DD", "_NTGetTrackVu",   "0x12DD", "_NTPauseMusic", "0x13DD", "_NTContinueMusic", "0x14DD", 
  "_NTinternal15",  "0x15DD", "_NTinternal16", "0x16DD", 

  /** NinjaTracker Plus : 0xDE **/
  "_NTBootInit",     "0x01DE", "_NTPStartUp",    "0x02DE", "_NTPShutDown",    "0x03DE", "_NTPVersion",   "0x04DE", "_NTPReset",           "0x05DE", "_NTPStatus",    "0x06DE", "_NTPLoadOneMusic", "0x09DE", "_NTPPlayMusic",  "0x0ADE", "_NTPStopMusic",     "0x0BDE", 
  "_NTPGetEOfMusic", "0x0CDE", "_NTPAddToBatch", "0x0DDE", "_NTPSelectBatch", "0x0EDE", "_NTPKillBatch", "0x0FDE", "_NTPGetPlayingMusic", "0x10DE", "_NTPPlayBatch", "0x11DE", "_NTPGetTrackVu",   "0x12DE", "_NTPPauseMusic", "0x13DE", "_NTPContinueMusic", "0x14DE", 

  /** ZPack : 0xE6 **/
  "_ZBootInit", "0x01E6", "_ZPPStartUp", "0x02E6", "_ZPPShutDown", "0x03E6", "_ZPPVersion", "0x04E6", "_ZPPReset",   "0x05E6", "_ZPPStatus",     "0x06E6", 
  "_ZUnPack",   "0x09E6", "_ZPack",      "0x0AE6", "_ZP1Pack",     "0x0BE6", "_ZP1UnPack",  "0x0CE6", "_ZP1LoadPic", "0x0DE6", "_APFUnPackMain", "0x0FE6",

  /** GS/Bug : 0xFF **/
  "_DebugBootInit", "0x01FF", "_DebugStartUp", "0x02FF", "_DebugShutDown", "0x03FF", "_DebugVersion",      "0x04FF", "_DebugReset",   "0x05FF", "_DebugStatus", "0x06FF", 
  "_DebugStr",      "0x09FF", "_SetMileStone", "0x0AFF", "_DebugSetHook",  "0x0BFF", "_DebugDebugGetInfo", "0x0CFF", "_DebugControl", "0x0DFF", "_DebugQuery",  "0x0EFF",

  NULL, NULL
};

char *tab_prodos8_name[] = 
{
  /* Prodos 8 */
  "0xC0", "CREATE",          "0xC1", "DESTROY",           "0xC2", "RENAME",      "0xC3", "SET_FILE_INFO",
  "0xC4", "GET_FILE_INFO",   "0xC5", "ON_LINE",           "0xC6", "SET_PREFIX",  "0xC7", "GET_PREFIX",
  "0xC8", "OPEN",            "0xC9", "NEWLINE",           "0xCA", "READ",        "0xCB", "WRITE",
  "0xCC", "CLOSE",           "0xCD", "FLUSH",             "0xCE", "SET_MARK",    "0xCF", "GET_MARK",
  "0xD0", "SET_EOF",         "0xD1", "GET_EOF",           "0xD2", "SET_BUF",     "0xD3", "GET_BUF",
  "0x40", "ALLOC_INTERRUPT", "0x41", "DEALLOC_INTERRUPT", "0x65", "QUIT",        "0x80", "READ_BLOCK",
  "0x81", "WRITE_BLOCK",     "0x82", "GET_TIME",
  NULL, NULL
};

char *tab_prodos16_name[] = 
{
  /* Prodos 16 */
  "0x0031", "ALLOC_INTERRUPT", "0x0004", "CHANGE_PATH",  "0x000B", "CLEAR_BACKUP_BIT", "0x0014", "CLOSE",      "0x0001", "CREATE",        "0x0032", "DEALLOC_INTERRUPT", "0x0002", "DESTROY",       "0x002C", "D_INFO",
  "0x0025", "ERASE_DISK",      "0x000E", "EXPAND_PATH",  "0x0015", "FLUSH",            "0x0024", "FORMAT",     "0x0028", "GET_BOOT_VOL",  "0x0020", "GET_DEV_NUM",       "0x001C", "GET_DIR_ENTRY", "0x0019", "GET_EOF",
  "0x0006", "GET_FILE_INFO",   "0x0021", "GET_LAST_DEV", "0x001B", "GET_LEVEL",        "0x0017", "GET_MARK",   "0x0027", "GET_NAME",      "0x000A", "GET_PREFIX",        "0x002A", "GET_VERSION",   "0x0011", "NEWLINE",
  "0x0010", "OPEN",            "0x0029", "QUIT",         "0x0012", "READ",             "0x0022", "READ_BLOCK", "0x0018", "SET_EOF",       "0x0005", "SET_FILE_INFO",     "0x001A", "SET_LEVEL",     "0x0016", "SET_MARK",
  "0x0009", "SET_PREFIX",      "0x0008", "VOLUME",       "0x0013", "WRITE",            "0x0023", "WRITE_BLOCK",
  NULL, NULL
};

char *tab_gsos_name[] = 
{
  /* GS/OS */
  "0x2001", "Create",        "0x2002", "Destroy",       "0x2005", "SetFileInfo",   "0x2006", "GetFileInfo", "0x200B", "ClearBackupBit", "0x2010", "Open",          "0x2011", "NewLine",    "0x2012", "Read",       "0x2013", "Write",
  "0x2014", "Close",         "0x2015", "Flush",         "0x2016", "SetMark",       "0x2017", "GetMark",     "0x2018", "SetEOF",         "0x2019", "GetEOF",        "0x201A", "SetLevel",   "0x201B", "GetLevel",   "0x201C", "GetDirEntry",
  "0x201D", "BeginSession",  "0x201E", "EndSession",    "0x201F", "SessionStatus", "0x2026", "ResetCache",  "0x2004", "ChangePath",     "0x2008", "Volume",        "0x2009", "SetPrefix",  "0x200A", "GetPrefix",  "0x200E", "ExpandPath",
  "0x2024", "Format",        "0x2025", "EraseDisk",     "0x2028", "GetBoolVol",    "0x200C", "SetSysPrefs", "0x200F", "GetSysPrefs",    "0x2027", "GetName",       "0x202A", "GetVersion", "0x202B", "GetFSTInfo", "0x2033", "FSTSpecific",
  "0x2037", "GetStdRefNum",  "0x2038", "GetRefNum",     "0x2039", "GetRefInfo",    "0x2029", "Quit",        "0x2034", "AddNotifyProc",  "0x2035", "DelNotifyProc", "0x200D", "Null",       "0x2003", "OSShutdown", "0x2031", "BindInt",
  "0x2032", "UnbindInt",     "0x202E", "DControl",      "0x202C", "DInfo",         "0x202F", "DRead",       "0x202D", "DStatus",        "0x2030", "DWrite",        "0x2036", "DRename",    "0x2020", "GetDevNumber",
  NULL, NULL
};

static int IsProdos16Call(DWORD,WORD);
static int compare_systemcall(const void *,const void *);


/******************************************************************************/
/*  GetNameFromAddress() :  On va identifier le nom d'une adresse en mmoire. */
/******************************************************************************/
void GetNameFromAddress(DWORD bank_address, WORD call_number, char *name_rtn, char *type_rtn, char *application_rtn)
{
  int i;
  BYTE bank;
  WORD address;
  char buffer[1024];
  struct symbol *current_symbol;
  struct gs_handle *current_gs_handle;
  struct s65816 *current_65816;

  /* Init */
  strcpy(name_rtn,"");
  strcpy(type_rtn,"");
  strcpy(application_rtn,"");
  bank = (BYTE) ((bank_address >> 16) & 0x00FF);
  address = (WORD) (bank_address & 0xFFFF);

  /* Rcupration des Internals */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);
  if(current_65816 == NULL)
    return;

  /***************/
  /**  ToolBox  **/
  /***************/
  if(bank_address == 0x00E10000)
    {
      /* Name */
      sprintf(buffer,"0x%04X",call_number);
      for(i=0; tab_toolbox_name[i+1] != NULL; i+=2)
        if(!strcmp(tab_toolbox_name[i+1],buffer))
          {
            strcpy(name_rtn,tab_toolbox_name[i]);
            break;
          }
      /* Type */
      strcpy(type_rtn,"Toolbox");
      /* Application */
      sprintf(buffer,"0x%02X",call_number & 0x00FF);
      for(i=0; tab_tools_name[i+1] != NULL; i+=2)
        if(!strcmp(tab_tools_name[i+1],buffer))
          {
            sprintf(application_rtn,"Tool %03d : %s",call_number & 0x00FF,tab_tools_name[i]);
            break;
          }
      return;
    }

  /****************************************************/
  /**  GS/OS | Prodos16 : In-Line Call / Call Stack  **/
  /****************************************************/
  if(bank_address == 0x00E100A8 || bank_address == 0x00E100B0)
    {
      /* Name */
      sprintf(buffer,"0x%04X",call_number);
      for(i=0; tab_gsos_name[i] != NULL; i+=2)
        if(!strcmp(tab_gsos_name[i],buffer))
          {
            strcpy(name_rtn,tab_gsos_name[i+1]);
            strcpy(type_rtn,"GS/OS");

            /* Application */
            return;
          }
    }

  /********************************************/
  /**  Prodos16 : In-Line Call / Call Stack  **/
  /********************************************/
  if(bank_address == 0x00E100A8 || bank_address == 0x00E100B0)
    {
      /* Name */
      sprintf(buffer,"0x%04X",call_number);
      for(i=0; tab_prodos16_name[i] != NULL; i+=2)
        if(!strcmp(tab_prodos16_name[i],buffer))
          {
            strcpy(name_rtn,tab_prodos16_name[i+1]);
            strcpy(type_rtn,"Prodos16");

            /* Application */
            return;
          }
    }

  /******************************/
  /**  Prodos8 : In-Line Call  **/
  /******************************/
  if(bank_address == 0x0000BF00)
    {
      /* Name */
      sprintf(buffer,"0x%02X",call_number & 0x00FF);
      for(i=0; tab_prodos8_name[i] != NULL; i+=2)
        if(!strcmp(tab_prodos8_name[i],buffer))
          {
            strcpy(name_rtn,tab_prodos8_name[i+1]);
            break;
          }
      /* Type */
      strcpy(type_rtn,"Prodos8");
      
      /* Application */
      return;
    }

  /*****************/
  /**  ROM Banks  **/
  /*****************/
  if(bank == 0xFC || bank == 0xFD || bank == 0xFE || bank == 0xFF)
    {
      /* Name */
      /* Type */
      strcpy(type_rtn,"ROM");
      /* Application */
      return;
    }

  /*******************/
  /**  ROM Bank 00  **/
  /*******************/
  if(bank == 0x00)
    {
      /* Name */
      sprintf(buffer,"0x%04X",address);
      for(i=0; tab_rom_00_name[i] != NULL; i+=2)
        if(!strcmp(tab_rom_00_name[i],buffer))
          {
            strcpy(name_rtn,tab_rom_00_name[i+1]);
            /* Type */
            strcpy(type_rtn,"ROM");
            /* Application */
            return;
          }
    }

  /******************/
  /**  Softswitch  **/
  /******************/
  if(bank == 0x00 || bank == 0x01 || bank == 0xE0 || bank == 0xE1)
    if(address >= 0xC000 && address <= MAX_SOFTSWITCH_ADDRESS)
      {
        /* Name */
        strcpy(name_rtn,tab_softwitch_name[address - 0xC000]);
        /* Type */
        strcpy(type_rtn,"Softswitch");
        /* Application */
        return;
      }
       
  /***********/
  /**  IWM  **/
  /***********/
  if(bank == 0x00 || bank == 0x01 || bank == 0xE0 || bank == 0xE1)
    if(address >= 0xC0E0 && address <= 0xC0EF)
      {
        /* Name */
        strcpy(name_rtn,tab_iwm_name[address - 0xC0E0]);
        /* Type */
        strcpy(type_rtn,"IWM");
        /* Application */
        return;
      }

  /*****************************************/
  /**  Points d'entres Interrupt Vector  **/
  /*****************************************/
  if(current_65816->current_interrupt_vector != NULL)
    {
      for(i=0; i<current_65816->current_interrupt_vector->nb_interrupt_vector; i++)
        if(current_65816->current_interrupt_vector->tab_interrupt_vector[i].location == bank_address)
          {
            /* Name */
            strcpy(name_rtn,current_65816->current_interrupt_vector->tab_interrupt_vector[i].name);
            /* Type */
            strcpy(type_rtn,"Interrupt Vector");
            /* Application */
            return;
          }
    }

  /********************************************/
  /**  Points d'entres QuickDraw II Vector  **/
  /********************************************/
  if(current_65816->current_quickdraw_vector != NULL)
    {
      for(i=0; i<current_65816->current_quickdraw_vector->nb_quickdraw_vector; i++)
        if(current_65816->current_quickdraw_vector->tab_quickdraw_vector[i].location == bank_address)
          {
            /* Name */
            strcpy(name_rtn,current_65816->current_quickdraw_vector->tab_quickdraw_vector[i].name);
            /* Type */
            strcpy(type_rtn,"QuickDraw II Vector");
            /* Application */
            return;
          }
    }

  /**************************************************/
  /**  Identifiants provenant des fichiers Symbol  **/
  /**************************************************/
  current_symbol = GetSymbolByAddress(bank_address);
  if(current_symbol)
    strcpy(name_rtn,current_symbol->name);    /* Name */

  /**************************************************/
  /**  Recherche l'adresse dans un Handle Mmoire  **/
  /**************************************************/
  for(current_gs_handle=current_65816->list_used_handle; current_gs_handle; current_gs_handle=current_gs_handle->next)
    if(bank_address >= current_gs_handle->address && bank_address < (current_gs_handle->address + current_gs_handle->size))
      {
        /* Rcupration des informations : Type + Application */
        GetHandleIDType(current_gs_handle->id,type_rtn);
        GetHandleInfo(current_gs_handle,application_rtn);
        break;
      }
}


/*********************************************************************/
/*  IsProdos16Call() :  Dtermine s'il s'agit d'un appel Prodos 16 . */
/*********************************************************************/
static int IsProdos16Call(DWORD call_address, WORD call_number)
{
  /* GS/OS | Prodos16 */
  if(call_address != 0xE100A8 && call_address != 0xE100B0)
    return(0);

  /** Prodos 16 Calls **/
  if(call_number == 0x0001 || /* CREATE */
     call_number == 0x0002 || /* DESTROY */
     call_number == 0x0004 || /* CHANGE_PATH */ 
     call_number == 0x0005 || /* SET_FILE_INFO */ 
     call_number == 0x0006 || /* GET_FILE_INFO */
     call_number == 0x0008 || /* VOLUME */
     call_number == 0x0009 || /* SET_PREFIX */
     call_number == 0x000A || /* GET_PREFIX */
     call_number == 0x000B || /* CLEAR_BACKUP_BIT */
     call_number == 0x000E || /* EXPAND_PATH */
     call_number == 0x0010 || /* OPEN */
     call_number == 0x0011 || /* NEWLINE */ 
     call_number == 0x0012 || /* READ */
     call_number == 0x0013 || /* WRITE */ 
     call_number == 0x0014 || /* CLOSE */
     call_number == 0x0015 || /* FLUSH */
     call_number == 0x0016 || /* SET_MARK */ 
     call_number == 0x0017 || /* GET_MARK */
     call_number == 0x0018 || /* SET_EOF */
     call_number == 0x0019 || /* GET_EOF */
     call_number == 0x001A || /* SET_LEVEL */
     call_number == 0x001B || /* GET_LEVEL */
     call_number == 0x001C || /* GET_DIR_ENTRY */
     call_number == 0x0020 || /* GET_DEV_NUM */
     call_number == 0x0021 || /* GET_LAST_DEV */ 
     call_number == 0x0022 || /* READ_BLOCK */
     call_number == 0x0023 || /* WRITE_BLOCK */
     call_number == 0x0024 || /* FORMAT */
     call_number == 0x0025 || /* ERASE_DISK */
     call_number == 0x0027 || /* GET_NAME */
     call_number == 0x0028 || /* GET_BOOT_VOL */
     call_number == 0x0029 || /* QUIT */
     call_number == 0x002A || /* GET_VERSION */
     call_number == 0x002C || /* D_INFO */
     call_number == 0x0031 || /* ALLOC_INTERRUPT */
     call_number == 0x0032) /* DEALLOC_INTERRUPT */
      return(1);

  /* Other */
  return(0);
}


/*********************************************************************/
/*  BuildSystemCallTable() :  Construit la table des Appels Systme. */
/*********************************************************************/
struct system_call **BuildSystemCallTable(int *nb_systemcall_rtn)
{
  DWORD call_number;
  int i, nb_systemcall, table_size;
  struct system_call **tab_systemcall;

  /* Taille du tableau */
  nb_systemcall = 0;
  for(i=0; tab_prodos8_name[i] != NULL; i+=2)
    nb_systemcall++;
  for(i=0; tab_prodos16_name[i] != NULL; i+=2)
    nb_systemcall++;
  for(i=0; tab_gsos_name[i] != NULL; i+=2)
    nb_systemcall++;
  for(i=0; tab_toolbox_name[i] != NULL; i+=2)
    nb_systemcall++;
  table_size = nb_systemcall;

  /* Allocation mmoire */
  tab_systemcall = (struct system_call **) calloc(table_size,sizeof(struct system_call *));
  if(tab_systemcall == NULL)
    return(NULL);
  for(i=0; i<table_size; i++)
    {
      tab_systemcall[i] = (struct system_call *) calloc(1,sizeof(struct system_call));
      if(tab_systemcall[i] == NULL)
        {
          mem_free_systemcall_table(table_size,tab_systemcall);
          return(NULL);
        }
    }

  /** Remplissage du Tableau **/
  nb_systemcall = 0;
  /* Prodos 8 */
  for(i=0; tab_prodos8_name[i] != NULL; i+=2)
    {
      tab_systemcall[nb_systemcall]->type = TYPE_SYSTEM_CALL_P8;
      sscanf(tab_prodos8_name[i],"%X",&call_number);
      tab_systemcall[nb_systemcall]->call_number = (WORD) (call_number & 0x000000FF);
      tab_systemcall[nb_systemcall]->name = strdup(tab_prodos8_name[i+1]);
      sprintf(tab_systemcall[nb_systemcall]->text,"P8 %s [%02X]",tab_prodos8_name[i+1],call_number & 0x000000FF);
      if(tab_systemcall[nb_systemcall]->name == NULL)
        {
          mem_free_systemcall_table(table_size,tab_systemcall);
          return(NULL);
        }
      nb_systemcall++;
    }
  /* Prodos 16 */
  for(i=0; tab_prodos16_name[i] != NULL; i+=2)
    {
      tab_systemcall[nb_systemcall]->type = TYPE_SYSTEM_CALL_P16;
      sscanf(tab_prodos16_name[i],"%X",&call_number);
      tab_systemcall[nb_systemcall]->call_number = (WORD) (call_number & 0x0000FFFF);
      tab_systemcall[nb_systemcall]->name = strdup(tab_prodos16_name[i+1]);
      sprintf(tab_systemcall[nb_systemcall]->text,"P16 %s [%04X]",tab_prodos16_name[i+1],call_number & 0x0000FFFF);
      if(tab_systemcall[nb_systemcall]->name == NULL)
        {
          mem_free_systemcall_table(table_size,tab_systemcall);
          return(NULL);
        }
      nb_systemcall++;
    }
  /* GS/OS */
  for(i=0; tab_gsos_name[i] != NULL; i+=2)
    {
      tab_systemcall[nb_systemcall]->type = TYPE_SYSTEM_CALL_GSOS;
      sscanf(tab_gsos_name[i],"%X",&call_number);
      tab_systemcall[nb_systemcall]->call_number = (WORD) (call_number & 0x0000FFFF);
      tab_systemcall[nb_systemcall]->name = strdup(tab_gsos_name[i+1]);
      sprintf(tab_systemcall[nb_systemcall]->text,"GSOS %s [%04X]",tab_gsos_name[i+1],call_number & 0x0000FFFF);
      if(tab_systemcall[nb_systemcall]->name == NULL)
        {
          mem_free_systemcall_table(table_size,tab_systemcall);
          return(NULL);
        }
      nb_systemcall++;
    }
  /* Toolbox */
  for(i=0; tab_toolbox_name[i] != NULL; i+=2)
    {
      tab_systemcall[nb_systemcall]->type = TYPE_SYSTEM_CALL_TOOLBOX;
      sscanf(tab_toolbox_name[i+1],"%X",&call_number);
      tab_systemcall[nb_systemcall]->call_number = (WORD) (call_number & 0x0000FFFF);
      tab_systemcall[nb_systemcall]->name = strdup(&tab_toolbox_name[i][1]);
      sprintf(tab_systemcall[nb_systemcall]->text,"T%03d %s [%04X]",call_number & 0x000000FF,&tab_toolbox_name[i][1],call_number & 0x0000FFFF);
      if(tab_systemcall[nb_systemcall]->name == NULL)
        {
          mem_free_systemcall_table(table_size,tab_systemcall);
          return(NULL);
        }
      nb_systemcall++;
    }

  /** Tri du tableau **/
  qsort((void *)tab_systemcall,(size_t)nb_systemcall,sizeof(struct system_call *),compare_systemcall);

  /* Renvoie la table */
  *nb_systemcall_rtn = nb_systemcall;
  return(tab_systemcall);
}


/***********************************************************************/
/*  compare_systemcall() : Fonction de comparaison pour le Quick Sort. */
/***********************************************************************/
static int compare_systemcall(const void *data_1, const void *data_2)
{
  struct system_call *system_call_1;
  struct system_call *system_call_2;

  /* Rcupration des paramtres */
  system_call_1 = *((struct system_call **) data_1);
  system_call_2 = *((struct system_call **) data_2);

  /* Comparaison des tailles puis des chemins */
  if(system_call_1->type > system_call_2->type)
    return(1);
  else if(system_call_1->type < system_call_2->type)
    return(-1);
  else
    {
      if(system_call_1->type == TYPE_SYSTEM_CALL_TOOLBOX)
        {
          if((system_call_1->call_number & 0x00FF) > (system_call_2->call_number & 0x00FF))
            return(1);
          else if((system_call_1->call_number & 0x00FF) < (system_call_2->call_number & 0x00FF))
            return(-1);
          else
            return(stricmp(system_call_1->name,system_call_2->name));
        }
      else
        return(stricmp(system_call_1->name,system_call_2->name));
    }
}


/*****************************************************************************/
/*  mem_free_systemcall_table() :  Libration mmoire du tableau SystemCall. */
/*****************************************************************************/
void mem_free_systemcall_table(int nb_systemcall, struct system_call **table_systemcall)
{
  int i;

  if(table_systemcall)
    {
      for(i=0; i<nb_systemcall; i++)
        mem_free_systemcall(table_systemcall[i]);

      free(table_systemcall);
    }
}


/****************************************************************************/
/*  mem_free_systemcall() :  Libration mmoire de la structure SystemCall. */
/****************************************************************************/
void mem_free_systemcall(struct system_call *current_systemcall)
{
  if(current_systemcall)
    {
      if(current_systemcall->name)
        free(current_systemcall->name);

      free(current_systemcall);
    }
}

/**********************************************************************/
