/**********************************************************************/
/*                                                                    */
/*  Dc_Notification.c :  Module pour la gestion des Notifications.    */
/*                                                                    */
/**********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Mai 2018  */
/**********************************************************************/

#include <stdio.h>

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <Commdlg.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_MMU.h"
#include "Dc_Snapshot.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_About.h"
#include "Window_Stack.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"


/****************************************************/
/*  my_Notification() :  Gestion des Notifications. */
/****************************************************/
void my_Notification(int code, void *data, void *value)
{
  int i, index, search_timer_id;
  static int nb_notification;
  static struct notification *first_notification;
  static struct notification *last_notification;
  struct notification *current_notification;
  struct notification *remove_notification;
  struct notification *next_notification;
  static int timer_id;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Rcupration de la fentre principale */  
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
    
  /* Accs unique aux notifications */
  if(param != NULL)
    EnterCriticalSection(&param->cs_notification_threadhandle);   
  
  switch(code)
    {
      case NOTIFICATION_INIT :
        /* Init */
        nb_notification = 0;
        first_notification = NULL;
        last_notification = NULL;
        timer_id = 1;
        break;
        
      case NOTIFICATION_ADD :
        current_notification = (struct notification *) data;
        
        /* Ajoute la structure  la liste */
        if(first_notification == NULL)
          first_notification = current_notification;
        else
          last_notification->next = current_notification;
        last_notification = current_notification;
        nb_notification++;
        
        /* Si c'est une notification TIMER, on envoi un message  la fentre principale pour l'activation */
        if(current_notification->type == NOTIFICATION_TYPE_VIEW && current_notification->sub_type == NOTIFICATION_SUBTYPE_TIMER)
          {
            current_notification->timer_id = timer_id++;
            SendMessage(window_main->hwnd_main,HM_NOTIFICATION_TIMER_CREATE,0,(LPARAM)current_notification);
          }
        break;
        
      case NOTIFICATION_GET_NB :
        *((int *) data) = nb_notification;
        break;
        
      case NOTIFICATION_GET :
        index = *((int *) data);
        *((struct notification **) value) = NULL;
        if(index <= 0 || index > nb_notification)
          break;

        /* Recherche la index-nth entry */
        for(i=1, current_notification = first_notification; i<index; i++)
          current_notification = current_notification->next;
        *((struct notification **) value) = current_notification;
        break;
        
      case NOTIFICATION_GET_BY_ID :
        /* Init */
        *((struct notification **) value) = NULL;
      
        /* Identifiant */
        search_timer_id = *((int *) data);
        
        /* Recherche */
        for(current_notification = first_notification; current_notification; current_notification = current_notification->next)
          if(current_notification->timer_id == search_timer_id)
            {
              *((struct notification **) value) = current_notification;
              break;
            }
        break;
        
      case NOTIFICATION_BROADCAST :
        /* Notifie toutes les fentres ouvertes pour forcer un Refresh */
        for(current_notification = first_notification; current_notification; current_notification = current_notification->next)
          if(current_notification->type == NOTIFICATION_TYPE_VIEW && current_notification->sub_type != NOTIFICATION_SUBTYPE_TIMER)
            SendMessage(current_notification->hwnd_window,current_notification->message,0,0);
        break;

      case NOTIFICATION_REMOVE :
        remove_notification = (struct notification *) data;
        if(remove_notification == NULL)
          break;

        /** Recherche la structure **/
        if(nb_notification == 1 && first_notification == remove_notification)
          {
            /* Unique */
            nb_notification = 0;
            first_notification = NULL;
            last_notification = NULL;
          }
        else if(nb_notification > 1 && first_notification == remove_notification)
          {
            /* Premire */
            nb_notification--;
            first_notification = first_notification->next;
          }
        else if(nb_notification > 1 && last_notification == remove_notification)
          {
            /* Dernire */
            nb_notification--;
            for(current_notification = first_notification; current_notification; current_notification=current_notification->next)
              if(current_notification->next == remove_notification)
                {
                  current_notification->next = NULL;
                  last_notification = current_notification;
                  break;
                }
          }
        else
          {
            /* QQpart au milieu */
            for(current_notification = first_notification; current_notification; current_notification=current_notification->next)
              if(current_notification->next == remove_notification)
                {
                  nb_notification--;
                  current_notification->next = current_notification->next->next;
                  break;
                }
          }
        
        /* Supprime le Timer */
        if(remove_notification->type == NOTIFICATION_TYPE_VIEW && remove_notification->sub_type == NOTIFICATION_SUBTYPE_TIMER)
          SendMessage(window_main->hwnd_main,HM_NOTIFICATION_TIMER_DELETE,0,(LPARAM)remove_notification);
          
        /* Libration mmoire */
        mem_free_notification(remove_notification);
        break;
        
      case NOTIFICATION_FREE :
        for(current_notification = first_notification; current_notification; current_notification = next_notification)
          {
            next_notification = current_notification->next;
            mem_free_notification(current_notification);
          }
        /* Init */
        nb_notification = 0;
        first_notification = NULL;
        last_notification = NULL;      
        break;
    }
    
  /* Fin de l'accs unique */
  if(param != NULL)
    LeaveCriticalSection(&param->cs_notification_threadhandle);     
}


/****************************************************************/
/*  mem_free_notification() :  Libration mmoire notification. */
/****************************************************************/
void mem_free_notification(struct notification *current_notification)
{
  if(current_notification)
    {
      free(current_notification);
    }
}

/**********************************************************************/
