/**********************************************************************/
/*                                                                    */
/*  Dc_Notification.h :  Header pour la gestion des Notifications.    */
/*                                                                    */
/**********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Mai 2018  */
/**********************************************************************/

#define NOTIFICATION_INIT                10
#define NOTIFICATION_ADD                 11
#define NOTIFICATION_GET_NB              12
#define NOTIFICATION_GET                 13
#define NOTIFICATION_GET_BY_ID           14
#define NOTIFICATION_REMOVE              15
#define NOTIFICATION_BROADCAST           16
#define NOTIFICATION_FREE                17

#define NOTIFICATION_TYPE_VIEW            1     /* On va rafraichir une fentre View */
#define NOTIFICATION_TYPE_BREAKPOINT      2     /* On va activer un Break Point */

#define NOTIFICATION_SUBTYPE_MEMORYAREA               1     /* Modification d'une Zone mmoire */
#define NOTIFICATION_SUBTYPE_STACK                    2     /* Modification de la Stack */
#define NOTIFICATION_SUBTYPE_DIRECTPAGE               3     /* Modification de la Direct Page */
#define NOTIFICATION_SUBTYPE_SOFTSWITCH               4     /* Modification du Softswitch */
#define NOTIFICATION_SUBTYPE_PROCESSOR                5     /* Modification du Processeur */
#define NOTIFICATION_SUBTYPE_RAMSOUND                 6     /* Modification d'une Zone mmoire de la RAM Son */
#define NOTIFICATION_SUBTYPE_TEXTPAGE40               7     /* Modification d'une Zone mmoire de la Text Page 40 */
#define NOTIFICATION_SUBTYPE_TEXTPAGE80               8     /* Modification d'une Zone mmoire de la Text Page 80 */
#define NOTIFICATION_SUBTYPE_GRAPHICGR                9     /* Modification d'une Zone mmoire de la Graphic GR */
#define NOTIFICATION_SUBTYPE_GRAPHICHGR              10     /* Modification d'une Zone mmoire de la Graphic HGR */
#define NOTIFICATION_SUBTYPE_GRAPHICDHGRMONO         11     /* Modification d'une Zone mmoire de la Graphic DHGR Mono */
#define NOTIFICATION_SUBTYPE_GRAPHICDHGRCOLOR        12     /* Modification d'une Zone mmoire de la Graphic DHGR Color */
#define NOTIFICATION_SUBTYPE_GRAPHICDGR              13     /* Modification d'une Zone mmoire de la Graphic DGR */
#define NOTIFICATION_SUBTYPE_GRAPHICHGRTEXT40        14     /* Modification d'une Zone mmoire de la Graphic HGR Text 40 */
#define NOTIFICATION_SUBTYPE_GRAPHICHGRTEXT80        15     /* Modification d'une Zone mmoire de la Graphic HGR Text 80 */
#define NOTIFICATION_SUBTYPE_GRAPHICGRTEXT40         16     /* Modification d'une Zone mmoire de la Graphic GR Text 40 */
#define NOTIFICATION_SUBTYPE_GRAPHICGRTEXT80         17     /* Modification d'une Zone mmoire de la Graphic GR Text 80 */
#define NOTIFICATION_SUBTYPE_GRAPHICDGRTEXT80        18     /* Modification d'une Zone mmoire de la Graphic DGR Text 80 */
#define NOTIFICATION_SUBTYPE_GRAPHICDHGRMONOTEXT80   19     /* Modification d'une Zone mmoire de la Graphic DHGR Mono Text 80 */
#define NOTIFICATION_SUBTYPE_GRAPHICDHGRCOLORTEXT80  20     /* Modification d'une Zone mmoire de la Graphic DHGR Color Text 80 */
#define NOTIFICATION_SUBTYPE_GRAPHICSHR              21     /* Modification d'une Zone mmoire de la Graphic SHR */
#define NOTIFICATION_SUBTYPE_DISASSEMBLER            22     /* Modification d'une Zone mmoire de la Disassembler */
#define NOTIFICATION_SUBTYPE_MEMORYMAP               23     /* Modification d'une Zone mmoire de la Memory Map */
#define NOTIFICATION_SUBTYPE_DOCREGISTER             24     /* Modification d'une Zone mmoire de DOC Register */
#define NOTIFICATION_SUBTYPE_QUICKWATCH              25     /* Modification d'une Zone mmoire de Quick Watch */
#define NOTIFICATION_SUBTYPE_GRAPHICSCREEN           26     /* Modification de l'cran Graphique */
#define NOTIFICATION_SUBTYPE_BATTERYRAM              27     /* Modification d'une Zone mmoire de Battery Ram */
#define NOTIFICATION_SUBTYPE_INTERNALS               28     /* Modification d'une Zone mmoire de Internals */
#define NOTIFICATION_SUBTYPE_STATISTIC               29     /* Modification d'une Zone mmoire de Statistic */
#define NOTIFICATION_SUBTYPE_PROFILE                 30     /* Modification d'une Zone mmoire de Profile */
#define NOTIFICATION_SUBTYPE_TIMER                   31     /* Notification Timer */

#define MEMORY_ACTION_READ                         0x01
#define MEMORY_ACTION_WRITE                        0x02
#define MEMORY_ACTION_EXECUTE                      0x04

struct notification
{
  int type;               /* NOTIFICATION_TYPE_XXX */
  int sub_type;           /* NOTIFICATION_SUBTYPE_XXX */
  
  /** View **/
  HWND hwnd_window;       /* Fentre cible */
  int message;            /* Message  envoyer */
 
  /** **/
  
  /** Pour les Notifications TIMER, on indique la frquence **/
  int timer_id;
  int timer_milliseconds;

  struct notification *next;
};

void my_Notification(int,void *,void *);
void mem_free_notification(struct notification *);

/**********************************************************************/
