/******************************************************************************/
/*                                                                            */
/*  Dc_Output.h :  Header de gestion des lignes Output de la Shared Memory.   */
/*                                                                            */
/******************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Date : June 2023  */
/******************************************************************************/

/* Trace System Calls */
#define TRACE_CALL_ORIGIN_GSOS      1
#define TRACE_CALL_ORIGIN_P16       2
#define TRACE_CALL_ORIGIN_P8        3
#define TRACE_CALL_ORIGIN_TOOL      4
#define TRACE_CALL_ORIGIN_RETURN    5

/* Trace System Calls Parameter Type */
#define TRACE_CALL_PARAM_TYPE_IN    1
#define TRACE_CALL_PARAM_TYPE_OUT   2

struct output
{
  int type;                        /* BREAKPOINT_ACTION_PRINT / BREAKPOINT_ACTION_FILE / BREAKPOINT_ACTION_TRACE_SYSTEM_CALL */
  char *line;
  DWORD breakpoint_id;
  QWORD goid;
  DWORD address;
  int wait_for_return_value;       /* A 1 si les param de retour ne sont pas encore dispo */

  /** System Call **/
  int origin;                      /* GS/OS | P16 | P8 | Tools */
  WORD call_number;
  DWORD nb_cycle;                  /* Total number of cycle for this call */
  WORD error_code;                 /* Error */
  int is_return;                   /* System Calls Return Values */
  char *function_name;             /* Nom de la fonction */

  /* Tableau des Paramtres */
  int nb_param;
  char *tab_param_value_utf8[32];  /* Valeurs */
  int tab_param_type[32];          /* Type (0=In,1=Out) */

  int published;                   /* 1 : Si dj ajout dans la ListBox */

  DWORD offset_sm;                 /* Position dans la Shared Memory */
  DWORD length_sm;

  struct output *previous;
  struct output *next;
};

int StartOutputManager(void);
void mem_free_output(struct output *);

/******************************************************************************/
