/************************************************************************/
/*                                                                      */
/*  Dc_Project.h :  Header pour les fonctions de gestion des Projets.   */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Avril 2018  */
/************************************************************************/

/* Types de ROM de l'Apple IIgs */
#define PROJECT_APPLEIIGS_ROM01   1
#define PROJECT_APPLEIIGS_ROM03   3
#define PROJECT_APPLEIIGS_NOROM   0

/* Code Couleurs de l'Apple IIgs */
#define COLOR_DISPLAY_CODE_BLACK         0
#define COLOR_DISPLAY_CODE_DEEPRED       1
#define COLOR_DISPLAY_CODE_DARKBLUE      2
#define COLOR_DISPLAY_CODE_PURPLE        3
#define COLOR_DISPLAY_CODE_DARKGREEN     4
#define COLOR_DISPLAY_CODE_DARKGRAY      5
#define COLOR_DISPLAY_CODE_MEDIUMBLUE    6
#define COLOR_DISPLAY_CODE_LIGHTBLUE     7
#define COLOR_DISPLAY_CODE_BROWN         8
#define COLOR_DISPLAY_CODE_ORANGE        9
#define COLOR_DISPLAY_CODE_LIGHTGRAY    10
#define COLOR_DISPLAY_CODE_PINK         11
#define COLOR_DISPLAY_CODE_LIGHTGREEN   12
#define COLOR_DISPLAY_CODE_YELLOW       13
#define COLOR_DISPLAY_CODE_AQUAMARINE   14
#define COLOR_DISPLAY_CODE_WHITE        15

/* Couleurs RGB de l'Apple IIgs */
#define COLOR_DISPLAY_RGB_BLACK         0x000000
#define COLOR_DISPLAY_RGB_DEEPRED       0xDD0033
#define COLOR_DISPLAY_RGB_DARKBLUE      0x000099
#define COLOR_DISPLAY_RGB_PURPLE        0xDD00DD
#define COLOR_DISPLAY_RGB_DARKGREEN     0x007700
#define COLOR_DISPLAY_RGB_DARKGRAY      0x555555
#define COLOR_DISPLAY_RGB_MEDIUMBLUE    0x2222FF
#define COLOR_DISPLAY_RGB_LIGHTBLUE     0x66AAFF
#define COLOR_DISPLAY_RGB_BROWN         0x885522
#define COLOR_DISPLAY_RGB_ORANGE        0xFF6600
#define COLOR_DISPLAY_RGB_LIGHTGRAY     0xAAAAAA
#define COLOR_DISPLAY_RGB_PINK          0xFF9988
#define COLOR_DISPLAY_RGB_LIGHTGREEN    0x00DD00
#define COLOR_DISPLAY_RGB_YELLOW        0xFFFF00
#define COLOR_DISPLAY_RGB_AQUAMARINE    0x00FF99
#define COLOR_DISPLAY_RGB_WHITE         0xFFFFFF

struct project
{
  /* Nom du fichier contenant les informations du projet */
  char project_file_path[BUFFER_SIZE];

  /** Files 0-8 **/
  BYTE tab_file_bank[9];
  WORD tab_file_address[9];
  struct file_path *tab_file[9];

  /** File Sound **/
  WORD file_sound_address;
  struct file_path *file_sound;

  /** Merlin 32 output files **/
  char file_merlin32_path[BUFFER_SIZE];
  int nb_merlin32_file;
  struct file_path *first_merlin32_file;
  struct file_path *last_merlin32_file;
  
  /* Processor */
  WORD proc_a;
  WORD proc_x;
  WORD proc_y;
  BYTE proc_dbr;
  BYTE proc_pbr;
  WORD proc_pc;
  WORD proc_stack;
  WORD proc_directpage;

  /* Status Register */
  int status_c;
  int status_z;
  int status_i;
  int status_d;
  int status_x;
  int status_m;
  int status_v;
  int status_n;
  int status_e;
  int status_b;  

  /* RAM Size */
  int ram_size;               /* Nombre de banc mmoire extended (en plus des 256 KB de base) */

  /* Apple II ROM */
  int appleii_rom;

  /* Apple IIgs ROM */
  int appleiigs_rom;          /* PROJECT_APPLEIIGS_XXX */

  /* Display Preferences */
  int color_text;
  int color_background;
  int color_border;
};

int SaveProjectFile(struct project *,char *);
struct project *LoadProjectFile(char *,char *);
void mem_free_project(struct project *);

/************************************************************************/
