/*****************************************************************************
 *                                                                           *
 *  Dc_Request.h :  Header regroupant les fonctions des Requtes.            *
 *                                                                           *
 *****************************************************************************
 *  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Date : May 2023  *
 *****************************************************************************/

/** Kegs / Cyrene Communication Protocol **/
#define IPC_REQUEST_TYPE_C2K_OPEN_CONNECTION      1   /* Cyrene -> Kegs : Open Connection (Send Connection information) */
#define IPC_REQUEST_TYPE_C2K_CLOSE_CONNECTION     2   /* Cyrene -> Kegs : Stop Connection */
#define IPC_REQUEST_TYPE_C2K_GET_SNAPSHOT         3   /* Cyrene -> Kegs : Ask for Snapshot IIgs Data */
#define IPC_REQUEST_TYPE_C2K_GET_OPERATION        4   /* Cyrene -> Kegs : Ask for Next Operation Data */
#define IPC_REQUEST_TYPE_C2K_PAUSE                5   /* Cyrene -> Kegs : Ask for Waiting */
#define IPC_REQUEST_TYPE_C2K_WRITE_DATA           6   /* Cyrene -> Kegs : Write Data into Kegs Memory (RAM,DOC,...) */

#define IPC_REQUEST_TYPE_K2C_SEND_SNAPSHOT        6   /* Kegs -> Cyrene : Send Snapshot IIgs Data (Memory) */
#define IPC_REQUEST_TYPE_K2C_SEND_OPERATION       7   /* Kegs -> Cyrene : Send Next Operation Data (Opcode,Operand,Hardware,Register) */
#define IPC_REQUEST_TYPE_K2C_CLOSE_CONNECTION     9   /* Kegs -> Cyrene : Close Connection (Kegs is exiting) */

/* Cyrene / Kegs Connection Status */
#define CONNECTION_STATUS_CLOSED                  0   /* No connection open between Cyrene and Kegs */
#define CONNECTION_STATUS_WAIT_FOR_ACCEPT         1   /* Cyrene is waiting Kegs answer */
#define CONNECTION_STATUS_OPEN                    2   /* Connection Open between Cyrene and Kegs */

/* Operation Data Buffer Size */
#define OPERATION_DATA_SIZE                      59   /* 1 Operation Data Size */

/* Operation Data Buffer */
#define OPERATION_DATA_OFFSET_A_W                 0   /* Offset of A Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_X_W                 2   /* Offset of X Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_Y_W                 4   /* Offset of Y Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_PC_W                6   /* Offset of PC Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_PBR_B               8   /* Offset of PBR Byte in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_DBR_B               9   /* Offset of DBR Byte in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_STACK_W            10   /* Offset of STACK Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_DIRECT_W           12   /* Offset of DIRECT Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_STATUS_W           14   /* Offset of STATUS Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_OPCODE_B           16   /* Offset of Opcode Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_OPERAND1_B         17   /* Offset of Operand #1 Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_OPERAND2_B         18   /* Offset of Operand #2 Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_OPERAND3_B         19   /* Offset of Operand #3 Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_DATASIZE_CYCLE_B   20   /* Offset of DataSize and #Cycle Byte in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_READ_ADDRESSB_B    21   /* Offset of Read Address Bank Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_READ_ADDRESSH_B    22   /* Offset of Read Address High Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_READ_ADDRESSL_B    23   /* Offset of Read Address Low Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_WRITE_ADDRESSB_B   24   /* Offset of Write Address Bank Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_WRITE_ADDRESSH_B   25   /* Offset of Write Address High Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_WRITE_ADDRESSL_B   26   /* Offset of Write Address Low Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_JUMP_ADDRESSB_B    27   /* Offset of Jump Address Bank Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_JUMP_ADDRESSH_B    28   /* Offset of Jump Address High Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_JUMP_ADDRESSL_B    29   /* Offset of Jump Address Low Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_READ_VALUE_W       30   /* Offset of Read Value Bytes in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_WRITE_VALUE_W      32   /* Offset of Write Value Bytes in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_FLAGS_B            34   /* Offset of Flags Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_CALL_NUMBER_W      35   /* Offset of Call Number Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_ROM_B              37   /* Offset of ROM (01/03) Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_RAM_B              38   /* Offset of RAM Banks (4-255) Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_C02E_B             39   /* Offset of Softswitch $C02E Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_C02F_B             40   /* Offset of Softswitch $C02F Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_C035_B             41   /* Offset of Softswitch $C035 Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_C068_B             42   /* Offset of Softswitch $C068 Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_GLOBALID_Q         43   /* Offset of Global ID Bytes in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_GLOBALCYCLE_Q      51   /* Offset of Global Cycle Bytes in the Operation Data buffer */

#define SNAPSHOT_DATA_OFFSET_NEXT_OPERATION       0   /* Offset of Next Operation Data in the Snapshot Data buffer */
#define SNAPSHOT_DATA_OFFSET_STOP               100   /* Offset of Stop Data in the Snapshot Data buffer v*/
#define SNAPSHOT_DATA_OFFSET_DOC_REGISTER       512   /* Offset of DOC Register 224+3 bytes in the Snapshot Data buffer */
#define SNAPSHOT_DATA_OFFSET_BATTERY_RAM        768   /* Offset of Battery Ram 256 bytes in the Snapshot Data buffer */
#define SNAPSHOT_DATA_OFFSET_RAM               1024   /* Offset of RAM 00 -> XX + E0 + E1 bytes in the Snapshot Data buffer */
                                                      /* ROM (128 KB or 256 KB) */
                                                      /* DOC RAM (64 KB) */
/* Operation Flags */
#define OPERATION_FLAG_INTERRUPT               0x80   /* Interruption has occurred */

/* Operation Stop */
#define STOP_CONDITION_GOID                       1   /* Next XXX operations */     
#define STOP_CONDITION_JMP                        2   /* Next JMP BRA BEQ BPL... */
#define STOP_CONDITION_JSR                        3   /* Next JSR JSL */
#define STOP_CONDITION_RTS                        4   /* Next RTS RTL RTI */
#define STOP_CONDITION_ADDRESS                    5   /* Next Specified Address */
#define STOP_CONDITION_VBL                        6   /* Next VBL */
#define STOP_CONDITION_INTERRUPT                  7   /* Next Interrupt */
#define STOP_CONDITION_TOOLBOX                    8   /* Next Toolbox */
#define STOP_CONDITION_OS                         9   /* Next P8 / P16/ GS/OS */
#define STOP_CONDITION_BREAKPOINT                10   /* Next Breakpoint */

/* RAM Type */
#define RAMTYPE_RAM                               1   /* RAM / ROM : 00/0000 - FF/FFFF */
#define RAMTYPE_DOC                               2   /* DOC RAM   : 0000 - FFFF */

struct request
{
  /* Information brute provenant du message COPYDATA */
  int request_type;
  int request_data_size;
  unsigned char *request_data;

  struct request *next;
};

int StartRequestManager();
void AddRequest(int,int,unsigned char *);
void mem_free_request(struct request *);
void WriteDataToEmulatorMemory(int,DWORD,int,unsigned char *);

int SendDataToKegs(int,int,unsigned char *);

/**********************************************************************/
