/***********************************************************************/
/*                                                                     */
/*  Dc_Shared.c : Module pour la bibliothque gnrique de fonctions.  */
/*                                                                     */
/***********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Aug 2013   */
/***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <direct.h>
#include <fcntl.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/timeb.h>
#include <math.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"

static void LogLine(char *,char *,int,int,time_t *);


/********************************************************************/
/*  GetFolderFiles() :  Rcupre tous les fichiers d'un rpertoire. */
/********************************************************************/
int GetFolderFiles(char *folder_path, char *hierarchy, int is_recursive)
{
  int error, rc, file_size;
  intptr_t hFile;
  int first_time = 1;
  struct _finddata_t c_file;
  char *buffer_folder_path = NULL;
  char *buffer_file_path = NULL;

  /* Rien  faire */
  if(folder_path == NULL)
    return(0);
  if(strlen(folder_path) == 0)
    return(0);
  error = 0;

  /* Allocation mmoire */
  buffer_folder_path = (char *) calloc(1,1024);
  buffer_file_path = (char *) calloc(1,1024);
  if(buffer_folder_path == NULL || buffer_file_path == NULL)
    return(1);
  strcpy(buffer_folder_path,folder_path);
  if(buffer_folder_path[strlen(buffer_folder_path)-1] != '\\' && buffer_folder_path[strlen(buffer_folder_path)-1] != '/')
    strcat(buffer_folder_path,FOLDER_CHARACTER);
  strcat(buffer_folder_path,"*.*");

  /** On boucle sur tous les fichiers prsents **/
  while(1)
    {
      if(first_time == 1)
        {
          hFile = _findfirst(buffer_folder_path,&c_file);
          rc = (int) hFile;
        }
      else
        rc = _findnext(hFile,&c_file);

        /* On analyse le rsultat */
    	if(rc == -1)
          break;    /* no more files */
 
      /** On traite cette entre **/
      first_time++;
      strcpy(buffer_file_path,folder_path);
      if(buffer_file_path[strlen(buffer_file_path)-1] != '\\' && buffer_file_path[strlen(buffer_file_path)-1] != '/')
        strcat(buffer_file_path,FOLDER_CHARACTER);
      strcat(buffer_file_path,c_file.name);
      file_size = (int) c_file.size;

      /** Traite le dossier de faon rcursive **/
      if((c_file.attrib & _A_SUBDIR) == _A_SUBDIR)
        {
          /* On ne traite ni . ni .. */
          if(!stricmp(c_file.name,".") || !stricmp(c_file.name,".."))
            continue;

          /* Recherche dans le contenu du dossier (appel rcursif) */
          if(is_recursive == 1)
            {
              error = GetFolderFiles(buffer_file_path,hierarchy,is_recursive);
              if(error)
                break;
            }
        }
      else
        {
          /* Conserve le nom du fichier */
          if(MatchHierarchie(buffer_file_path,hierarchy))
            my_Memory(MEMORY_ADD_FILE,buffer_file_path,&file_size);
        }
    }

  /* On ferme */
  _findclose(hFile);

  /* Libration mmoire */
  free(buffer_folder_path);
  free(buffer_file_path);

  return(error);
}


/***********************************************************************/
/*  MatchHierarchie() : Indique si un nom appartient  une hirarchie. */
/***********************************************************************/
int MatchHierarchie(char *name, char *hierarchie)
{
  int i,j,k;
  int length;
  char *hier_ptr;
  char *name_ptr;
  int result;
  int count;
  int offset;
  char buffer[BUFFER_SIZE];

  /*** On parcours les deux chanes ***/
  for(i=0,j=0; i<(int)strlen(hierarchie); i++)
    {
      if(hierarchie[i] != '*')
        {
          if(toupper(hierarchie[i]) !=  toupper(name[j]))
            return(0);
          j++;
        }
      else if(hierarchie[i] == '?')
        j++;
      else
        {
          /* Si '*' dernier caractre de la chane => OK */
          if(hierarchie[i+1] == '\0')
            return(1);

          /** S'il ne reste pas d'autre '*' : On compare la fin **/
          hier_ptr = strchr(&hierarchie[i+1],'*');
          if(hier_ptr == NULL)
            {
              length = (int) strlen(&hierarchie[i+1]);
              if((int)strlen(&name[j]) < length)
                return(0);
              if(!mh_stricmp(&name[strlen(name)-length],&hierarchie[i+1]))
                return(1);
              else
                return(0);
            }

          /** On compte le nb d'occurences de la partie entre les deux '*' dans name **/
          strncpy(buffer,&hierarchie[i+1],hier_ptr-&hierarchie[i+1]);
          buffer[hier_ptr-&hierarchie[i+1]] = '\0';
          length = (int) strlen(buffer);
          for(count = 0,offset = j;;count++)
            {
              name_ptr = mh_stristr(&name[offset],buffer);
              if(name_ptr == NULL)
                break;
              offset = (int)((name_ptr - name) + 1);
            }
          /* Si aucune occurence => pas de matching */
          if(count == 0)
            return(0);

          /** On lance la rcursivit sur toutes les occurences trouves **/
          for(k=0,offset=j; k<count; k++)
            {
              name_ptr = mh_stristr(&name[offset],buffer);
              result = MatchHierarchie(name_ptr+length,&hierarchie[i+1+length]);
              if(result)
                return(1);
              offset = (int)((name_ptr - name) + 1);
            }
          return(0);
        }
    }

  /* Est t'on galement  la fin de name ? */
  if(j != (int)strlen(name))
    return(0);

  /* On est arriv au bout : OK */
  return(1);
}


/****************************************************************/
/* CleanHierarchie() : Supprime les '*' en trop dans la chane. */
/****************************************************************/
void CleanHierarchie(char *hierarchie)
{
  int i,j;
  char buffer[BUFFER_SIZE];

  for(i=0,j=0; i<=(int)strlen(hierarchie); i++)
    {
      if(hierarchie[i] != '*')
        buffer[j++] = hierarchie[i];
      else
        {
          /* On ne laisse pas plusieurs '*' conscutifs */
          if(j == 0)
            buffer[j++] = hierarchie[i];
          else
            {
              if(buffer[j-1] == '*')
                continue;
              else
                buffer[j++] = hierarchie[i];
            }
        }
    }

  /* On recopie la chaine nettoye */
  strcpy(hierarchie,buffer);
}


/***********************************************************************/
/*  mh_stristr() : Recherche l'occurence d'une chane dans une autre.  */
/***********************************************************************/
char *mh_stristr(char *name, char *hierarchie)
{
  int i;
  int length_n = (int) strlen(name);
  int length_h = (int) strlen(hierarchie);

  /* On limine les cas extrmes */
  if(length_n < length_h)
    return(NULL);

  /** On parcours la chane 'name' afin de localiser la chaine 'hierarchie' **/
  for(i=0; i<length_n-length_h+1; i++)
    {
      /* La recherche ne tient pas compte de la casse */
      if(!strnicmp(&name[i],hierarchie,length_h))
        return(&name[i]);
    }

  /* On a rien trouv */
  return(NULL);
}


/********************************************************************/
/*  mh_stricmp() : Compare deux chanes de caractres avec les '?'. */
/********************************************************************/
int mh_stricmp(char *string_n, char *string_h)
{
  int i;
  int length_n = (int) strlen(string_n);
  int length_h = (int) strlen(string_h);

  /* On limine immdiatement les cas dfavorables */
  if(length_n != length_h)
    return(1);

  /** On compare tous les caractres avec gestion du '?' **/
  for(i=0; i<length_n; i++)
    {
      if(string_h[i] == '?')
        continue;
      else if(toupper(string_n[i]) != toupper(string_h[i]))
        return(1);
    }

  /* On a deux chanes identiques */
  return(0);
}


/******************************************************************/
/*  IsFileExist() :  Vrifie l'existance d'un fichier sur disque. */
/******************************************************************/
int IsFileExist(char *file_folder_path, int is_folder, int *file_size_rtn)
{
  int error;
  struct stat sts;

  /* Vrifie s'il existe */
  error = stat(file_folder_path,&sts);
  if(error == 0)
    {
      /* Taille du fichier */
      *file_size_rtn = (int) sts.st_size;
          
      /* Est-ce un dossier ? */
      if(S_ISDIR(sts.st_mode))
        {
          if(is_folder == 1)
            return(1);
          else
            return(0);
        }

      /* OK */
      return(1);
    }

  /* N'existe pas */
  return(0);
}


/********************************************************************/
/*  GetFileSizeInt() :  Rcupre la taille d'un fichier sur disque. */
/********************************************************************/
int GetFileSizeInt(char *file_path)
{
  int error;
  struct stat sts;

  /* Vrifie s'il existe */
  error = stat(file_path,&sts);
  if(error == 0)
    return((int) sts.st_size);
  else
    return(0);   /* Le fichier n'exite pas */
}


/***********************************************************************/
/*  GetFileNumber() :  Rcupre le nombre de fichiers d'une hirachie. */
/***********************************************************************/
int GetFileNumber(char *file_path_param, int free_list)
{
  int i, result, nb_file, file_size;
  char folder_path[1024];
  char file_path[1024];

  /* Init */
  my_Memory(MEMORY_FREE_FILE,NULL,NULL);

  /** Y a t'il des caractres joker **/
  if(strchr(file_path_param,'*') == NULL && strchr(file_path_param,'?') == NULL)
    {
      /* Teste l'existance du fichier */
      result = IsFileExist(file_path_param,0,&file_size);
      if(result == 1)
        {
          /* Conserve le chemin du fichier */
          my_Memory(MEMORY_ADD_FILE,file_path_param,&file_size);
          return(1);
        }

      /* Pas trouv */
      return(0);
    }

  /** On doit compter le nombre de fichiers **/
  strcpy(file_path,file_path_param);
  CleanHierarchie(file_path);
  strcpy(folder_path,file_path);
  for(i=(int)strlen(folder_path)-1; i>=0; i--)
    if(folder_path[i] == '\\' || folder_path[i] == '/')
      {
        folder_path[i+1] = '\0';
        break;
      }

  /* Rcupre les chemmins des fichiers */
  result = GetFolderFiles(folder_path,file_path,0);
  my_Memory(MEMORY_GET_FILE_NB,&nb_file,NULL);
  
  /* Libration mmoire */
  if(free_list == 1)
    my_Memory(MEMORY_FREE_FILE,NULL,NULL);

  /* Trie les fichiers par ordre alphabtique */
  my_Memory(MEMORY_SORT_FILE,NULL,NULL);

  /* Renvoi le nb de fichier */
  return(nb_file);
}


/************************************************************************************/
/*  GetOneFilePath() :  Rcupre le nom d'un fichier slectionn par l'utilisateur. */
/************************************************************************************/
void GetOneFilePath(HWND hwnd, char *title, char *filter, char *current_path, char *new_path_rtn)
{
  BOOL result;
  OPENFILENAME opfn;
  char file_path[2048] = "";

  /* Init */
  strcpy(file_path,current_path);
  strcpy(new_path_rtn,"");

  /* Init de la structure */
  memset(&opfn,0,sizeof(opfn));
  opfn.lStructSize = sizeof(opfn);
  opfn.hwndOwner = hwnd;
  opfn.lpstrFilter = filter;
  opfn.lpstrFile = &file_path[0];
  opfn.nMaxFile = 2048;
  opfn.lpstrTitle = title;
  opfn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR;

  /** Rcupre les noms de fichier **/
  result = GetOpenFileName(&opfn);
  if(result != TRUE)
    return;
  if(strlen(file_path) == 0)
    return;

  /* Nom du fichier */
  strcpy(new_path_rtn,file_path);
}


/*************************************************************************************/
/*  GetSaveFilePath() :  Rcupre le nom d'un fichier slectionn par l'utilisateur. */
/*************************************************************************************/
BOOL GetSaveFilePath(HWND hwnd, char *title, char *filter, char *file_path_rtn)
{
  BOOL result;
  OPENFILENAME opfn;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Controler erreur */
  if(param == NULL)
    return(FALSE);

  /* Init de la structure */
  memset(&opfn,0,sizeof(opfn));
  opfn.lStructSize = sizeof(opfn);
  opfn.hwndOwner = hwnd;
  opfn.lpstrFilter = filter;
  opfn.lpstrFile = file_path_rtn;
  opfn.nMaxFile = MAX_PATH;
  opfn.lpstrTitle = title;
  opfn.Flags = OFN_EXPLORER | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;

  /** Rcupre les noms de fichier **/
  result = GetSaveFileName(&opfn);

  /* return result */
  return(result);
}


/**********************************************************/
/*  LoadTextFile() :  Charge un fichier Texte en mmoire. */
/**********************************************************/
unsigned char *LoadTextFile(char *file_path, int *file_length_rtn, char *error_buffer_rtn)
{
  FILE *fd;
  int file_length, nb_read;
  unsigned char *file_data;

  /* Init */
  strcpy(error_buffer_rtn,"");

  /* Ouverture du fichier Texte */
  fd = fopen(file_path,"r");
  if(fd == NULL)
    {
      strcpy(error_buffer_rtn,"Can't open file");
      return(NULL);
    }

  /* Taille du fichier */
  fseek(fd,0L,SEEK_END);
  file_length = ftell(fd);
  fseek(fd,0L,SEEK_SET);
   
  /* Allocation mmoire */
  file_data = (unsigned char *) calloc(1,file_length+1);
  if(file_data == NULL)
    {
      strcpy(error_buffer_rtn,"Memory allocation error");
      fclose(fd);
      return(NULL);
    }

  /** Lecture des donnes **/
  nb_read = (int) fread(file_data,1,file_length,fd);
  if(nb_read < 0)
    {
      strcpy(error_buffer_rtn,"Can't read file data");
      free(file_data);
      fclose(fd);
      return(NULL);
    }
  file_data[nb_read] = '\0';

  /* Fermeture du fichier */
  fclose(fd);

  /* Renvoi le fichier et sa taille */
  *file_length_rtn = nb_read;
  return(file_data);
}


/**************************************************************/
/*  LoadBinaryFile() :  Charge un fichier Binaire en mmoire. */
/**************************************************************/
unsigned char *LoadBinaryFile(char *file_path, int *file_length_rtn, char *error_buffer_rtn)
{
  FILE *fd;
  int file_length, nb_read;
  unsigned char *file_data;
  
  /* Init */
  strcpy(error_buffer_rtn,"");

  /* Ouverture du fichier Binaire */
  fd = fopen(file_path,"rb");
  if(fd == NULL)
    {
      strcpy(error_buffer_rtn,"Can't open file");
      return(NULL);
    }

  /* Taille du fichier */
  fseek(fd,0L,SEEK_END);
  file_length = ftell(fd);
  fseek(fd,0L,SEEK_SET);
   
  /* Allocation mmoire */
  file_data = (unsigned char *) calloc(1,file_length+1);
  if(file_data == NULL)
    {
      strcpy(error_buffer_rtn,"Memory allocation error");
      fclose(fd);
      return(NULL);
    }

  /** Lecture des donnes **/
  nb_read = (int) fread(file_data,1,file_length,fd);
  if(nb_read < 0)
    {
      strcpy(error_buffer_rtn,"Can't read file data");
      free(file_data);
      fclose(fd);
      return(NULL);
    }

  /* Fermeture du fichier */
  fclose(fd);

  /* Renvoi le fichier et sa taille */
  *file_length_rtn = nb_read;
  return(file_data);
}


/******************************************************************/
/*  SaveBinaryFile() :  Cration d'un fichier Binaire sur disque. */
/******************************************************************/
void SaveBinaryFile(char *file_path, int buffer_length, unsigned char *buffer_data, char *error_buffer_rtn)
{
  FILE *fd;
  int nb_write;

  /* Init */
  strcpy(error_buffer_rtn,"");
  
  /* Ouverture du fichier Binaire */
  fd = fopen(file_path,"wb");
  if(fd == NULL)
    {
      strcpy(error_buffer_rtn,"Can't create file");
      return;
    }
   
  /** Ecriture des donnes **/
  nb_write = (int) fwrite(buffer_data,1,buffer_length,fd);
  if(nb_write != buffer_length)
    {
      strcpy(error_buffer_rtn,"Can't write file data");
      fclose(fd);
      return;
    }

  /* Fermeture du fichier */
  fclose(fd);
}


/**************************************************************/
/*  SaveTextFile() :  Cration d'un fichier texte sur disque. */
/**************************************************************/
void SaveTextFile(char *file_path, char *buffer_data, char *error_buffer_rtn)
{
  FILE *fd;
  int nb_write, buffer_length;

  /* Init */
  strcpy(error_buffer_rtn,"");
  
  /* Ouverture du fichier Texte */
  fd = fopen(file_path,"w");
  if(fd == NULL)
    {
      strcpy(error_buffer_rtn,"Can't create file");
      return;
    }
   
  /** Ecriture des donnes **/
  buffer_length = (int) strlen(buffer_data);
  nb_write = (int) fwrite(buffer_data,1,buffer_length,fd);
  if(nb_write != buffer_length)
    {
      strcpy(error_buffer_rtn,"Can't write file data");
      fclose(fd);
      return;
    }

  /* Fermeture du fichier */
  fclose(fd);
}


/*****************************************************/
/*  CleanBuffer() : On nettoy un buffer genre @Trim. */
/*****************************************************/
void CleanBuffer(char *buffer)
{
  int i, j;
  int length;

  /* Nettoyage arrire */
  length = (int) strlen(buffer);
  for(i=length-1; i>=0; i--)
    if(buffer[i] == '\0' || buffer[i] == ' ' || buffer[i] == '\n' || buffer[i] == '\t')
      buffer[i] = '\0';
    else
      break;

  /* Nettoyage avant */
  length = (int) strlen(buffer);
  for(i=0,j=0; i<length; i++)
    if(buffer[i] == ' ' || buffer[i] == '\n' || buffer[i] == '\t')
      j++;
    else
      break;

  /* On dplace si besoin */
  if(j > 0)
    memmove(&buffer[0],&buffer[j],(length-j)+1);
}


/************************************************************************/
/*  EvaluateAsOffsetOrLength() :  Rcupre une valeur dans un controle. */
/************************************************************************/
int EvaluateAsOffsetOrLength(HWND hwnd, int id)
{
  int is_hexa, is_odd, value_int;
  char value_txt[1024];

  /* Rcupre la valeur dans le controle */
  GetWindowText(GetDlgItem(hwnd,id),value_txt,1024-1);

  /* Vide */
  if(strlen(value_txt) == 0)
    return(0);

  /* Format hexadcimal ? */
  if(value_txt[0] == '$')
    {
      memmove(&value_txt[0],&value_txt[1],strlen(&value_txt[1])+1);
      is_hexa = 1;
    }
  else if(value_txt[0] == '0' || toupper(value_txt[1]) == 'X')
    {
      memmove(&value_txt[0],&value_txt[2],strlen(&value_txt[2])+1);
      is_hexa = 1;
    }
  else
    is_hexa = IsHexaDecimal(value_txt,&is_odd);

  /* Dcode la valeur */
  if(is_hexa == 1)
    sscanf(value_txt,"%x",&value_int);
  else
    value_int = atoi(value_txt);

  /* Renvoi la valeur */
  return(value_int);
}


/*************************************************************/
/*  EvaluateAsAddress() :  Evalue une expression en Address. */
/*************************************************************/
int EvaluateAsAddress(char *value_txt, int *is_error_rtn)
{
  int address, is_hexa, is_odd;

  /* Init */
  *is_error_rtn = 0;

  /* Teste la longueur */
  if(strlen(value_txt) != 6 && strlen(value_txt) != 7)
    {
      *is_error_rtn = 1;
      return(0);
    }
  if(strlen(value_txt) == 7 && value_txt[2] != ':' && value_txt[2] != '/')
    {
      *is_error_rtn = 1;
      return(0);
    }

  /* Compresse l'expression (E1/2000 -> E12000) */
  if(strlen(value_txt) == 7)
    memmove(&value_txt[2],&value_txt[3],strlen(&value_txt[3])+1);

  /* Teste les caractres */
  is_hexa = IsHexaDecimal(value_txt,&is_odd);
  if(is_hexa == 0 || is_odd == 1)
    {
      *is_error_rtn = 1;
      return(0);
    }

  /* Conversion */
  sscanf(value_txt,"%x",&address);
  if(address < 0 || address > 0xFFFFFF)
    {
      /* Overflow */
      *is_error_rtn = 1;
      return(0);
    }

  /* Renvoi la valeur */
  return(address);
}


/***********************************************************/
/*  EvaluateAsLength() :  Evalue une expression en Length. */
/***********************************************************/
int EvaluateAsLength(char *value_txt, int *is_error_rtn)
{
  int length, is_odd;

  /* Init */
  *is_error_rtn = 0;

  /* Teste la longueur */
  if(strlen(value_txt) == 0)
    {
      *is_error_rtn = 1;
      return(0);
    }

  /* Teste les caractres */
  if(IsHexaDecimal(value_txt,&is_odd) == 0)
    {
      *is_error_rtn = 1;
      return(0);
    }

  /* Conversion */
  sscanf(value_txt,"%x",&length);
  if(length < 0 || length > 0xFFFFFF)
    {
      /* Overflow */
      *is_error_rtn = 1;
      return(0);
    }

  /* Renvoi la valeur */
  return(length);
}


/*******************************************************/
/*  EvaluateAsWord() :  Evalue une expression en WORD. */
/*******************************************************/
WORD EvaluateAsWord(char *value_txt, int *is_error_rtn)
{
  int i, is_hexa, is_odd;
  WORD value_wd;

  /* Init */
  *is_error_rtn = 0;

  /* Teste les caractres */
  is_hexa = IsHexaDecimal(value_txt,&is_odd);
  if(is_hexa == 0 || is_odd == 1)
    {
      *is_error_rtn = 1;
      return(0);
    }

  /* Conversion */
  sscanf(value_txt,"%x",&i);
  if(i < 0 || i > 65535)
    {
      /* Overflow */
      *is_error_rtn = 1;
      return(0);
    }
  value_wd = (WORD) i;

  /* Renvoi la valeur */
  return(value_wd);
}


/*******************************************************/
/*  EvaluateAsByte() :  Evalue une expression en BYTE. */
/*******************************************************/
BYTE EvaluateAsByte(char *value_txt, int *is_error_rtn)
{
  int i, is_hexa, is_odd;
  BYTE value_b;

  /* Init */
  *is_error_rtn = 0;

  /* Teste les caractres */
  is_hexa = IsHexaDecimal(value_txt,&is_odd);
  if(is_hexa == 0 || is_odd == 1)
    {
      *is_error_rtn = 1;
      return(0);
    }

  /* Conversion */
  sscanf(value_txt,"%x",&i);
  if(i < 0 || i > 255)
    {
      /* Overflow */
      *is_error_rtn = 1;
      return(0);
    }
  value_b = (BYTE) i;

  /* Renvoi la valeur */
  return(value_b);
}


/**********************************************************/
/*  EvaluateAsOneByte() :  Evalue une expression en BYTE. */
/**********************************************************/
BYTE EvaluateAsOneByte(char *value_txt_param, int *is_error_rtn)
{
  int i, is_hexa, is_odd;
  BYTE value_b;
  char value_txt[3];

  /* Init */
  *is_error_rtn = 0;
  memcpy(value_txt,value_txt_param,2);
  value_txt[2] = '\0';

  /* Teste les caractres */
  is_hexa = IsHexaDecimal(value_txt,&is_odd);
  if(is_hexa == 0 || is_odd == 1)
    {
      *is_error_rtn = 1;
      return(0);
    }

  /* Conversion */
  sscanf(value_txt,"%x",&i);
  if(i < 0 || i > 255)
    {
      /* Overflow */
      *is_error_rtn = 1;
      return(0);
    }
  value_b = (BYTE) i;

  /* Renvoi la valeur */
  return(value_b);
}


/*****************************************************/
/*  EvaluateAsBit() :  Evalue une expression en Bit. */
/*****************************************************/
int EvaluateAsBit(char *value_txt, int *is_error_rtn)
{
  /* Init */
  *is_error_rtn = 0;

  /* Teste les caractres */
  if(!stricmp(value_txt,"0"))
    return(0);
  else if(!stricmp(value_txt,"1"))
    return(1);

  /* Erreur */
  *is_error_rtn = 1;
  return(0);
}


/*******************************************************/
/*  IsHexaDecimal() :  Est-ce une valeur hexadcimal ? */
/*******************************************************/
int IsHexaDecimal(char *value, int *is_odd_rtn)
{
  int i;

  /* Init */
  *is_odd_rtn = 0;

  /* Valeur vide */
  if(strlen(value) == 0)
    return(0);

  /* On teste la plage de caractre 0-F */
  for(i=1; i<(int)strlen(value); i++)
    if(!((toupper(value[i]) >= '0' && toupper(value[i]) <= '9') || (toupper(value[i]) >= 'A' && toupper(value[i]) <= 'F')))
      return(0);

  /* Nombre pair d'octets */
  if((strlen(value) % 2) == 1)
    *is_odd_rtn = 1;

  /* OK */
  return(1);
}


/******************************************************************/
/*  IsHexaDecimalChar() :  Dtermine si ce caractre est en Hexa. */
/******************************************************************/
int IsHexaDecimalChar(char one_char)
{
  if(one_char >= '0' && one_char <= '9')
    return(1);
  if(toupper(one_char) >= 'A' && toupper(one_char) <= 'F')
    return(1);
  return(0);
}


/*****************************************************************/
/*  IsDecimalChar() :  Dtermine si ce caractre est en Dcimal. */
/*****************************************************************/
int IsDecimalChar(char one_char)
{
  if(one_char >= '0' && one_char <= '9')
    return(1);
  return(0);
}


/************************************************/
/*  IsDecimal() :  Est-ce une valeur dcimale ? */
/************************************************/
int IsDecimal(char *value, int *is_odd_rtn)
{
  int i;

  /* Init */
  *is_odd_rtn = 0;

  /* Valeur vide */
  if(strlen(value) == 0)
    return(0);

  /* On teste la plage de caractre 0-9 */
  for(i=1; i<(int)strlen(value); i++)
    if(!(toupper(value[i]) >= '0' && toupper(value[i]) <= '9'))
      return(0);

  /* Nombre pair d'octets */
  if((strlen(value) % 2) == 1)
    *is_odd_rtn = 1;

  /* OK */
  return(1);
}


/**********************************************************************/
/*  RemoveSpace() :  Supprime les espaces de la chaine de caractres. */
/**********************************************************************/
void RemoveSpace(char *buffer_space)
{
  int i, j;
  char buffer_no_space[1024];
  
  /* Supprime les espaces */
  for(i=0,j=0; i<(int)strlen(buffer_space); i++)
    if(buffer_space[i] != ' ')
      buffer_no_space[j++] = buffer_space[i];
  buffer_no_space[j] = '\0';

  /* Renvoi le buffer */
  strcpy(buffer_space,buffer_no_space);
}


/**********************************************************/
/*  ExchangeByte() :  Echange les bytes Intel / Motorola. */
/**********************************************************/
void ExchangeByte(void *data, int nb_byte)
{
  int i;
  unsigned char data_src[16];
  unsigned char data_dst[16];

  /* Inverse les octets */
  memcpy(&data_src[0],data,nb_byte);
  for(i=0; i<nb_byte; i++)
    data_dst[nb_byte-1-i] = data_src[i];
  memcpy(data,&data_dst[0],nb_byte);
}


/************************************************************************/
/*  my_stristr() :  Recherche une chaine dans une autre, sans la casse. */
/************************************************************************/
char *my_stristr(char *string1, char *string2)
{
  int i, l_1, l_2;

  /* Calcule les longueurs */
  l_1 = (int) strlen(string1);
  l_2 = (int) strlen(string2);

  /* Cas extrmes */
  if(l_2 > l_1)
    return(NULL);
  else if(l_2 == l_1)
    {
      if(!stricmp(string1,string2))
        return(string1);
      return(NULL);
    }

  /* On parcours la chaine string1 */
  for(i=0; i<(l_1-l_2+1); i++)
    if(!strnicmp(&string1[i],string2,(size_t)l_2))
      return(&string1[i]);

  /* Pas trouv */
  return(NULL);
}


/********************************************************/
/*  GetHexaValue() :  Extrait une valeur code en Hexa. */
/********************************************************/
void GetHexaValue(char *begin, char *buffer_rtn)
{
  int i, j;

  /* Init */
  strcpy(buffer_rtn,"");

  /** Extrait les valeurs jusqu' tomber sur un sparateur **/
  for(i=0,j=0; i<(int)strlen(begin); i++)
    if((begin[i] >= '0' && begin[i] <= '9') || (toupper(begin[i]) >= 'A' && toupper(begin[i]) <= 'F'))
      buffer_rtn[j++] = begin[i];
    else
      break;
  buffer_rtn[j] = '\0';
}


/*************************************************/
/*  dump_memory() :  Cration d'un dump mmoire. */
/*************************************************/
void dump_memory(unsigned char *memory, int first_address, int length, char *file_name)
{
  FILE *fd;

  /* Cration du fichier */
  fd = fopen(file_name,"wb");
  if(fd == NULL)
    return;

  /* Dump */
  fwrite(&memory[first_address],1,length,fd);

  /* Fermeture du fichier */
  fclose(fd);
}


/***********************************************************/
/*  BuildFilePath() :  Cration d'une structure file_path. */
/***********************************************************/
struct file_path *BuildFilePath(char *file_path)
{
  int size;
  unsigned char *data;
  struct file_path *current_filepath;
  char error_buffer[BUFFER_SIZE];
  
  /* Chargement du fichier */
  data = LoadBinaryFile(file_path,&size,&error_buffer[0]);
  if(data == NULL)
    return(NULL);
    
  /* Allocation mmoire */
  current_filepath = (struct file_path *) calloc(1,sizeof(struct file_path));
  if(current_filepath == NULL)
    {
      free(data);
      return(NULL);
    }
  current_filepath->path = strdup(file_path);
  if(current_filepath->path == NULL)
    {
      free(data);    
      free(current_filepath);
      return(NULL);
    }
    
  /* Remplissage */
  current_filepath->data = data;
  current_filepath->size = size;
    
  /* Renvoi la structure */
  return(current_filepath);
}


/**********************************************************/
/*  GetHGRColor() :  Dtermine la couleur d'un point HGR. */
/**********************************************************/
int GetHGRColor(int bit_previous, int bit_current, int bit_next, int bit_color, int x)
{
  if(bit_current == 1) 
    {
      if((bit_previous == 0) && (bit_next == 0))
        {
          if((x & 0x01) == 1)    /* Impair */
            {
              if(bit_color == 1)
                return(HGR_COLOR_ORANGE);
              else 
                return(HGR_COLOR_GREEN);
            }
          else
            {
              if(bit_color == 1)
                return(HGR_COLOR_BLUE);
              else 
                return(HGR_COLOR_PURPLE);
            }
        } 
      else
        return(HGR_COLOR_WHITE);
    }
  else
    {
      if((bit_previous == 1) && (bit_next == 1))
        {
          if((x & 0x01) == 1)     /* Impair */
            {
              if(bit_color == 1)
                return(HGR_COLOR_BLUE);
              else 
                return(HGR_COLOR_PURPLE);
            } 
          else
            {
              if(bit_color == 1)
                return(HGR_COLOR_ORANGE);
              else 
                return(HGR_COLOR_GREEN);
            }
        }
      else 
        return(HGR_COLOR_BLACK);
    }
}


/**********************************************************************************/
/*  LoadFileAsBinary() :  Fonction gnrique de chargement de fichier en mmoire. */
/**********************************************************************************/
unsigned char *LoadFileAsBinary(char *src_path, int *file_size_rtn)
{
  int nb_read, file_size;
  FILE *fd;
  unsigned char *file_buffer;

  /* Init */
  *file_size_rtn = 0;

  /* Ouverture du fichier */
  fd = fopen(src_path,"rb");
  if(fd == NULL)
    {
      printf("  Error : Impossible to open file '%s'.\n",src_path);
      return(NULL);
    }

  /* Dtermine la taille du fichier */
  fseek(fd,0L,SEEK_END);
  file_size = ftell(fd);
  fseek(fd,0L,SEEK_SET);

  /* Cration d'un buffer de la taille du fichier */
  file_buffer = (unsigned char *) calloc(1,file_size+1);
  if(file_buffer == NULL)
    {
      printf("  Error : Impossible to allocate memory (%d).\n",file_size);
      fclose(fd);
      return(NULL);
    }

  /* Lecture du contenu */
  nb_read = (int) fread(file_buffer,1,file_size,fd);
 
  /* Fermetutre du fichier */
  fclose(fd);

  /* Vrification de la lecture */
  if(nb_read != file_size)
    {   
      printf("  Error : Should have read %d bytes, actually read: %d bytes, file_size: %d\n",file_size,nb_read);
      free(file_buffer);
      fclose(fd);
      return(NULL);
    }

  /* Renvoi les donnes */
  *file_size_rtn = file_size;
  return(file_buffer);
}


/*********************************************************************************/
/*  GetFilePath() :  Rcupre le nom d'un fichier slectionn par l'utilisateur. */
/*********************************************************************************/
char **GetFilePath(HWND hwnd, char *title, char *filter, int *nb_path_rtn, int is_multi)
{
  int i,nb_path,length,offset;
  char **file_path_tab;
  BOOL result;
  OPENFILENAME opfn;
  char file_path[2048] = "";
  char folder_path[1024];
  char file_name[1024];

  /* Init de la structure */
  memset(&opfn,0,sizeof(opfn));
  opfn.lStructSize = sizeof(opfn);
  opfn.hwndOwner = hwnd;
  opfn.lpstrFilter = filter;
  opfn.lpstrFile = &file_path[0];
  opfn.nMaxFile = 2048;
  opfn.lpstrTitle = title;
  opfn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR | ((is_multi == 1) ? OFN_ALLOWMULTISELECT : 0);

  /** Rcupre les noms de fichier **/
  result = GetOpenFileName(&opfn);
  if(result != TRUE)
    return(NULL);

  /* Combien de chemins ont t saisis */
  for(nb_path=0,i=0; !(file_path[i] == '\0' && file_path[i+1] == '\0'); i++)
    if(file_path[i] == '\0')
      nb_path++;

  /** Un seul fichier **/
  if(nb_path == 0 && strlen(file_path) > 0)
    {
      /* Allocation */
      file_path_tab = (char **) calloc(1,sizeof(char *));
      if(file_path_tab == NULL)
        return(NULL);
      /* Copie du chemin */
      file_path_tab[0] = _strdup(file_path);
      if(file_path_tab[0] == NULL)
        {
          free(file_path_tab);
          return(NULL);
        }
      /* Renvoi du chemin */
      *nb_path_rtn = 1;
      return(file_path_tab);
    }

  /* Allocation du tableau de pointeur en fonction du nombre d'lments */
  file_path_tab = (char **) calloc(nb_path,sizeof(char *));
  if(file_path_tab == NULL)
    return(NULL);

  /** Construction des chemins complets **/
  /* Isole le rpertoire */
  strcpy(folder_path,&file_path[0]);
  length = (int) strlen(folder_path);
  if(folder_path[length-1] != '\\')
    strcat(folder_path,"\\");

  /** Extrait la liste de tous les noms de fichiers **/
  offset = (int) strlen(file_path) + 1;
  nb_path = 0;
  while(1)
    {
      /* Isole le file_name */
      strcpy(file_name,&file_path[offset]);
      if(strlen(file_name) == 0)
        break;

      /* Allocation du buffer de rception */
      file_path_tab[nb_path] = (char *) calloc(1,strlen(folder_path)+strlen(file_name)+1);
      if(file_path_tab[nb_path] == NULL)
        {
          mem_free_list(nb_path,file_path_tab);
          return(NULL);
        }

      /* Copie */
      sprintf(file_path_tab[nb_path],"%s%s",folder_path,file_name);

      /* File_name suivant */
      offset += (int) strlen(file_name) + 1;
      nb_path++;
    }

  /* Renvoi le tableau des chemins */
  *nb_path_rtn = nb_path;
  return(file_path_tab);
}


/***********************************************************************/
/*  Extract_filename() :  Extrait le nom de fichier au bout du chemin. */
/***********************************************************************/
void Extract_filename(char *filepath, char *buffer_rtn)
{
  int length,i;

  /* Pas de pointeur null */
  if(filepath == NULL)
    return;

  /* Taille du chemin */
  length = (int) strnlen(filepath,2048);

  /* Il y a bien une fin */
  filepath[length] = '\0';

  /* Parcours depuis la fin */
  for(i=length; i>0; i--)
    if(filepath[i] == '\\')
      {
        /* Insre un espace pour la prsentation */
        sprintf(&buffer_rtn[0],"%c",' ');
        /* Sans le '\' */
        strcpy(&buffer_rtn[1],&filepath[i+1]);
        break;
      }
}


/*************************************************************************/
/*  mem_free_list() :  libration des lments d'un tableau de pointeur. */
/*************************************************************************/
void mem_free_list(int nb_item, char **tab)
{
  int i;
  
  if(tab)
    {
      for(i=0; i<nb_item; i++)
        if(tab[i])
          free(tab[i]);

      free(tab);
    }
}


/*****************************************************************/
/*  mem_free_palette() :  Libration mmoire pour la structure.  */
/*****************************************************************/
void mem_free_palette(struct palette *current_palette)
{
  if(current_palette)
    {
      free(current_palette);
    }
}


/*************************************************************/
/*  mem_alloc_palette_arne() :  Cration de la palette Arne. */
/*************************************************************/
struct palette *mem_alloc_palette_arne(void)
{
  int i;
  struct palette *arne_palette;

  /* Allocation mmoire */
  arne_palette = mem_alloc_palette("Arne","ARNE.pal");
  if(arne_palette == NULL)
    return(NULL);

  /** Couleurs RGB Arne **/
  arne_palette->nb_color = 16;
  arne_palette->palette_red[0]    = 0;
  arne_palette->palette_green[0]  = 0;
  arne_palette->palette_blue[0]   = 0;
  arne_palette->palette_red[1]    = 33;
  arne_palette->palette_green[1]  = 49;
  arne_palette->palette_blue[1]   = 57;
  arne_palette->palette_red[2]    = 57;
  arne_palette->palette_green[2]  = 82;
  arne_palette->palette_blue[2]   = 82;
  arne_palette->palette_red[3]    = 82;
  arne_palette->palette_green[3]  = 148;
  arne_palette->palette_blue[3]   = 33;
  arne_palette->palette_red[4]    = 165;
  arne_palette->palette_green[4]  = 206;
  arne_palette->palette_blue[4]   = 49;
  arne_palette->palette_red[5]    = 82;
  arne_palette->palette_green[5]  = 66;
  arne_palette->palette_blue[5]   = 49;
  arne_palette->palette_red[6]    = 165;
  arne_palette->palette_green[6]  = 115;
  arne_palette->palette_blue[6]   = 41;
  arne_palette->palette_red[7]    = 239;
  arne_palette->palette_green[7]  = 148;
  arne_palette->palette_blue[7]   = 57;
  arne_palette->palette_red[8]    = 247;
  arne_palette->palette_green[8]  = 231;
  arne_palette->palette_blue[8]   = 115;
  arne_palette->palette_red[9]    = 57;
  arne_palette->palette_green[9]  = 66;
  arne_palette->palette_blue[9]   = 189;
  arne_palette->palette_red[10]   = 57;
  arne_palette->palette_green[10] = 165;
  arne_palette->palette_blue[10]  = 247;
  arne_palette->palette_red[11]   = 181;
  arne_palette->palette_green[11] = 222;
  arne_palette->palette_blue[11]  = 239;
  arne_palette->palette_red[12]   = 189;
  arne_palette->palette_green[12] = 49;
  arne_palette->palette_blue[12]  = 66;
  arne_palette->palette_red[13]   = 222;
  arne_palette->palette_green[13] = 123;
  arne_palette->palette_blue[13]  = 148;
  arne_palette->palette_red[14]   = 165;
  arne_palette->palette_green[14] = 165;
  arne_palette->palette_blue[14]  = 165;
  arne_palette->palette_red[15]   = 247;
  arne_palette->palette_green[15] = 247;
  arne_palette->palette_blue[15]  = 231;

  /* Calcul les valeurs TSL */
  for(i=0; i<arne_palette->nb_color; i++)
    RGB2HSL(arne_palette->palette_red[i],arne_palette->palette_green[i],arne_palette->palette_blue[i],
            &arne_palette->palette_h[i],&arne_palette->palette_s[i],&arne_palette->palette_l[i]);

  /* OK */
  return(arne_palette);
} 


/******************************************************/
/*  RGB2HSL() :  Conversion en HSL d'une couleur RVB. */
/******************************************************/
void RGB2HSL(int r, int g, int b, int *t_rtn, int *s_rtn, int *l_rtn)
{
  int h,s,l;
  double r_percent, g_percent, b_percent, max_color, min_color, H, S, L;

  /* Pourcentage de couleurs */
  r_percent = ((double)r) / 255.0;
  g_percent = ((double)g) / 255.0;
  b_percent = ((double)b) / 255.0;

  /* Calcul des Min / Max */
  max_color =  MY_MAX(r_percent,MY_MAX(g_percent,b_percent));
  min_color =  MY_MIN(r_percent,MY_MIN(g_percent,b_percent));

  /* Luminosit */
  L = (max_color + min_color)/2.0;

  /* Cas particulier */
  if(max_color == min_color)
    {
      S = 0.0;
      H = 0.0;
    }
  else
    {
      /* Saturation */
      if(L < 0.5)
        S = (max_color - min_color)/(max_color + min_color);
      else
        S = (max_color - min_color)/(2.0 - max_color - min_color);

      /* Teinte */
      if(max_color == r_percent)
        H = (g_percent - b_percent)/(max_color - min_color);
      else if(max_color == g_percent)
        H = 2.0 + (b_percent - r_percent)/(max_color - min_color);
      else if(max_color == b_percent)
        H = 4.0 + (r_percent - g_percent)/(max_color - min_color);
    }

  /* Passage en 0-255 */
  s = (unsigned int)(S*255.0);
  l = (unsigned int)(L*255.0);
  H = H*60.0;
  if(H < 0)
    H += 360.0;
  h = (unsigned int) ((H*255.0)/359.0);

  /* Renvoi les valeurs */
  *t_rtn = h; 
  *s_rtn = s; 
  *l_rtn = l;
}


/*****************************************************************/
/*  mem_alloc_palette() :  Allocation mmoire pour la structure. */
/*****************************************************************/
struct palette *mem_alloc_palette(char *palette_name, char *palette_path)
{
  struct palette *current_palette;

  /* Allocation mmoire */
  current_palette = (struct palette *) calloc(1,sizeof(struct palette));
  if(current_palette == NULL)
    return(NULL);

  /* Item */
  strcpy(current_palette->name,palette_name);
  strcpy(current_palette->path,palette_path);

  /* OK */
  return(current_palette);
} 


/*********************************************************************************/
/*  SetWindowTextHexaByte() :  On va mettre une valeur Hexa dans un champ Texte. */
/*********************************************************************************/
void SetWindowTextHexaByte(HWND hwnd, int static_text_id, unsigned char value_byte, unsigned char value_byte_old, int force)
{
  char buffer[16];
  
  /* Rien n'a chang */
  if(value_byte == value_byte_old && force == 0)
    return;
  
  /* Passage de la valeur en Hexa Ascii */
  sprintf(buffer,"%02X",value_byte);
  SetWindowText(GetDlgItem(hwnd,static_text_id),buffer);
}


/***********************************************************************/
/*  FormatWordAsTextInfo() :  Convetit un WORD en buffer Text complet. */
/***********************************************************************/
void FormatByteAsBinary(BYTE one_byte, char *output_buffer)
{
  int i, offset;

  /* Binary */
  strcpy(output_buffer,"0000.0000");
  for(i=0,offset=8; i<8; i++)
    {
      /* Le 0 est dj en place => on ne met que les 1 */
      if(((one_byte >> i) & 0x01) == 0x01)
        output_buffer[offset] = '1';

      /* Dcale */
      offset--;

      /* Gre les espaces */
      if(i == 3)
        offset--;
    }
}


/*******************************************************************************************/
/*  GetNamedValueText() :  Rcupre la valeur Texte d'une variable dont on fournit le nom. */
/*******************************************************************************************/
void GetNamedValueText(char *value_list, char *name, char *value_rtn)
{
  int length_value;
  char *begin_value;
  char *end_value;

  /* Init */
  strcpy(value_rtn,"");

  /* Recherche le nom */
  begin_value = strstr(value_list,name);
  if(begin_value)
    {
      begin_value += strlen(name);
      end_value = strchr(begin_value,',');
      if(end_value)
        {
          /* Traite la valeur Texte */
          length_value = (int) (end_value - begin_value);
          memcpy(value_rtn,begin_value,length_value);
          value_rtn[length_value] = '\0';
        }
    }
}

/****************************************************************************************/
/*  GetNamedValueInt() :  Rcupre la valeur Int d'une variable dont on fournit le nom. */
/****************************************************************************************/
int GetNamedValueInt(char *value_list, char *name)
{
  int length_value;
  char *begin_value;
  char *end_value;
  char value[1024];

  /* Init */
  strcpy(value,"");

  /* Recherche le nom */
  begin_value = strstr(value_list,name);
  if(begin_value)
    {
      begin_value += strlen(name);
      end_value = strchr(begin_value,',');
      if(end_value)
        {
          /* Traite la valeur Texte */
          length_value = (int) (end_value - begin_value);
          memcpy(value,begin_value,length_value);
          value[length_value] = '\0';

          /* Conversion */
          return(atoi(value));
        }
    }

  /* Pas trouv */
  return(0);
}


/******************************************************************************************/
/*  GetNamedValue8bit() :  Rcupre la valeur Byte d'une variable dont on fournit le nom. */
/******************************************************************************************/
BYTE GetNamedValue8bit(char *value_list, char *name)
{
  int length_value, is_error;
  char *begin_value;
  char *end_value;
  char value[1024];
  BYTE byte;

  /* Init */
  strcpy(value,"");

  /* Recherche le nom */
  begin_value = strstr(value_list,name);
  if(begin_value)
    {
      begin_value += strlen(name);
      end_value = strchr(begin_value,',');
      if(end_value)
        {
          /* Traite la valeur Texte */
          length_value = (int) (end_value - begin_value);
          memcpy(value,begin_value,length_value);
          value[length_value] = '\0';

          /* Conversion */
          byte = EvaluateAsByte(value,&is_error);
          if(is_error)
            return(0x00);
          return(byte);
        }
    }

  /* Pas trouv */
  return(0x00);
}


/*******************************************************************************************/
/*  GetNamedValue16bit() :  Rcupre la valeur Word d'une variable dont on fournit le nom. */
/*******************************************************************************************/
WORD GetNamedValue16bit(char *value_list, char *name)
{
  int length_value, is_error;
  char *begin_value;
  char *end_value;
  char value[1024];
  WORD word;

  /* Init */
  strcpy(value,"");

  /* Recherche le nom */
  begin_value = strstr(value_list,name);
  if(begin_value)
    {
      begin_value += strlen(name);
      end_value = strchr(begin_value,',');
      if(end_value)
        {
          /* Traite la valeur Texte */
          length_value = (int) (end_value - begin_value);
          memcpy(value,begin_value,length_value);
          value[length_value] = '\0';

          /* Conversion */
          word = EvaluateAsWord(value,&is_error);
          if(is_error)
            return(0x0000);
          return(word);
        }
    }

  /* Pas trouv */
  return(0x0000);
}


/*****************************************************/
/*  BuildListView() :  Construction d'une List View. */
/*****************************************************/
void BuildListView(HWND hwnd_parent, HFONT hFont, int listview_x, int listview_y, int nb_row, struct list_view *current_listview)
{
  INITCOMMONCONTROLSEX icex;                 /* Structure for control initialization */
  LVCOLUMN listview_column;
  DWORD dwStyle;
  HWND headerWnd;
  HD_ITEM item;
  int bufLen = 256;
  char headerText[256];
  int i, result, nb_listview_line;
  int aElements[1] = {COLOR_WINDOW};
  DWORD aOldColors[1];
  DWORD aNewColors[1];
  DWORD dialog_background_color;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration du control ListView **/
  memset(&icex,0,sizeof(INITCOMMONCONTROLSEX));
  icex.dwICC = ICC_LISTVIEW_CLASSES;
  InitCommonControlsEx(&icex);

  /* Create the list-view window in report view with label editing enabled */
  current_listview->listview_x = listview_x;
  current_listview->listview_y = listview_y;
  current_listview->listview_width = 18;    /* Largeur pour l'ascenceur */
  for(i=0; i<current_listview->nb_column; i++)
    current_listview->listview_width += current_listview->tab_column[i].length;
  nb_listview_line = nb_row;
  current_listview->listview_height = INTERNALS_LISTVIEW_HEADER_HEIGHT + nb_listview_line*INTERNALS_LISTVIEW_LINE_HEIGHT + 0;
  current_listview->hwnd_listview = CreateWindow(WC_LISTVIEW, 
                                                 "",
                                                 WS_CHILD | LVS_REPORT | /*LVS_EDITLABELS |*/ WS_VISIBLE,
                                                 current_listview->listview_x, current_listview->listview_y,
                                                 current_listview->listview_width, current_listview->listview_height,
                                                 hwnd_parent,
                                                 (HMENU) current_listview->idc_listview,
                                                 param->hInstance,
                                                 NULL);

  /* Police de caractre de la ListView */
  SendMessage(current_listview->hwnd_listview,WM_SETFONT,(WPARAM)hFont,MAKELPARAM(TRUE,0));

  /* Style Report */
  dwStyle = GetWindowLong(current_listview->hwnd_listview,GWL_STYLE); 
  if((dwStyle & LVS_TYPEMASK) != LVS_REPORT) 
    SetWindowLongPtr(current_listview->hwnd_listview,GWL_STYLE,(dwStyle & ~LVS_TYPEMASK) | LVS_REPORT);

  /* Slection Simple */
  dwStyle = GetWindowLong(current_listview->hwnd_listview,GWL_STYLE); 
  SetWindowLongPtr(current_listview->hwnd_listview,GWL_STYLE,(dwStyle & ~LVS_SINGLESEL) | LVS_SINGLESEL);

  /* Affichage avec Grille */
  ListView_SetExtendedListViewStyleEx(current_listview->hwnd_listview,LVS_EX_GRIDLINES,LVS_EX_GRIDLINES);

  /* Autorise les images dans les Sub Items */
  ListView_SetExtendedListViewStyleEx(current_listview->hwnd_listview,LVS_EX_SUBITEMIMAGES,LVS_EX_SUBITEMIMAGES);

  /* Slection de toute la ligne d'un coup */
  SendMessage(current_listview->hwnd_listview,LVM_SETEXTENDEDLISTVIEWSTYLE,0,LVS_EX_FULLROWSELECT);

  /** Cration des Colonnes **/
  /* Commun */
  listview_column.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
  /* Col 0 -> N */
  for(i=0; i<current_listview->nb_column; i++)
    {
      listview_column.iSubItem = i;  
      listview_column.pszText  = current_listview->tab_column[i].title;
      listview_column.cx       = current_listview->tab_column[i].length;      // Width of column in pixels.
      listview_column.fmt      = current_listview->tab_column[i].align;       // Left / Center / Right-aligned column.
      result = ListView_InsertColumn(current_listview->hwnd_listview,i,&listview_column);
    }

  /* Couleur de Fond du control : Gris comme la fentre d'accueil */
  dialog_background_color = GetSysColor(COLOR_3DFACE);
  aOldColors[0] = GetSysColor(COLOR_WINDOW); 
  aNewColors[0] = dialog_background_color;

  /** Ajoute les images dans les Header des Colonnes **/
  SetSysColors(1,aElements,aNewColors); 
  current_listview->hBitmap_none = LoadImageA(param->hInstance,MAKEINTRESOURCE(IDB_BITMAP_NONE),IMAGE_BITMAP,0,0,LR_LOADTRANSPARENT);
  current_listview->hBitmap_up   = LoadImageA(param->hInstance,MAKEINTRESOURCE(IDB_BITMAP_UP),IMAGE_BITMAP,0,0,LR_LOADTRANSPARENT);
  current_listview->hBitmap_down = LoadImageA(param->hInstance,MAKEINTRESOURCE(IDB_BITMAP_DOWN),IMAGE_BITMAP,0,0,LR_LOADTRANSPARENT);
  SetSysColors(1,aElements,aOldColors); 

  /* Reset All */
  headerWnd = ListView_GetHeader(current_listview->hwnd_listview);
  for(i=0; i<current_listview->nb_column; i++)
    {
      item.mask = HDI_FORMAT | HDI_TEXT | HDI_BITMAP;
      item.pszText = headerText;
      item.cchTextMax = bufLen - 1;
      SendMessage(headerWnd,HDM_GETITEM,i,(LPARAM)&item);

      item.hbm = current_listview->hBitmap_none;
      item.fmt |= HDF_STRING | HDF_BITMAP_ON_RIGHT | HDF_BITMAP;
      item.mask = HDI_TEXT | HDI_BITMAP | HDI_FORMAT;
      SendMessage(headerWnd,HDM_SETITEM,i,(LPARAM)&item);      
    }
  for(i=0; i<current_listview->nb_column; i++)
    current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
}


/********************************************************************************************/
/*  ProcessListViewWindowCustomDraw() :  Fonction de dessin du fond de la fentre ListView. */
/********************************************************************************************/
LRESULT ProcessListViewWindowCustomDraw(LPARAM lParam)
{
  LPNMLVCUSTOMDRAW lplvcd = (LPNMLVCUSTOMDRAW) lParam;

  switch(lplvcd->nmcd.dwDrawStage) 
    {
      /* Before the paint cycle begins */
      case CDDS_PREPAINT :
        //request notifications for individual listview items
        return(CDRF_NOTIFYITEMDRAW);
          
      /** On peut changer le couleur du Text ou du Fond pour l'Item **/
      case CDDS_ITEMPREPAINT :
        if(((int)lplvcd->nmcd.dwItemSpec%2)==0)
          {
            /* Text Color : Noir */
            lplvcd->clrText   = COLOR_LISTVIEW_BLACK;
            
            /* Background Color : Gris Clair */
            lplvcd->clrTextBk = COLOR_LISTVIEW_GREY;
            
            /* Si On veut personaliser au niveau des SubItem il faut renvoyer CDRF_NOTIFYSUBITEMDRAW */
            return(CDRF_NEWFONT);
          }
        else
          {
            /* Text Color : Noir */
            lplvcd->clrText   = COLOR_LISTVIEW_BLACK;
              
            /* Background Color : Blanc */
            lplvcd->clrTextBk = COLOR_LISTVIEW_WHITE;
            
            /* Si On veut personaliser au niveau des SubItem il faut renvoyer CDRF_NOTIFYSUBITEMDRAW */
            return(CDRF_NEWFONT);
          }
        break;
      
      /** On peut changer le couleur du Text ou du Fond pour les Sous Item **/
      case CDDS_SUBITEM | CDDS_ITEMPREPAINT :
        /** On alterne Blanc et Gris **/
        if(((int)lplvcd->nmcd.dwItemSpec%2)==0)
          {
            /* Text Color : Noir */
            lplvcd->clrText   = COLOR_LISTVIEW_BLACK;
                
            /* Background Color : Gris Clair */
            lplvcd->clrTextBk = COLOR_LISTVIEW_GREY;
          }
        else
          {
            /* Text Color = Noir */
            lplvcd->clrText   = COLOR_LISTVIEW_BLACK;
                  
            /* Background Color : Blanc */
            lplvcd->clrTextBk = COLOR_LISTVIEW_WHITE;
          }
        return(CDRF_NEWFONT);
    }

  return(CDRF_DODEFAULT);
}


/*****************************************************************************************************/
/*  setListViewSortIcon() :  On va changer les Icones affiches dans les Header des colonnes (Tris). */
/*****************************************************************************************************/
void setListViewSortIcon(struct list_view *current_list_view)
{
  HWND headerWnd;
  int bufLen = 256;
  char headerText[256];
  HD_ITEM item;
  int numColumns, curCol;
    
  /* On rcupre le Handle sur le Header du ListView */
  headerWnd = ListView_GetHeader(current_list_view->hwnd_listview);
  
  /* Nombre de colonnes du List View */
  numColumns = Header_GetItemCount(headerWnd);

  /** Traite toutes les colonnes pour y affecter la bonne icone ^v **/
  for(curCol=0; curCol<numColumns; curCol++)
    {
      /* Rcupre les valeurs */
      item.mask = HDI_FORMAT | HDI_TEXT;
      item.pszText = headerText;
      item.cchTextMax = bufLen - 1;
      SendMessage(headerWnd,HDM_GETITEM,curCol,(LPARAM)&item);

      /** Modifie les Images des Tri **/
      if(current_list_view->tab_column[curCol].sort == INTERNALS_COLUMN_SORT_NONE)
        item.hbm = current_list_view->hBitmap_none;
      else if(current_list_view->tab_column[curCol].sort == INTERNALS_COLUMN_SORT_UP)
        item.hbm = current_list_view->hBitmap_up;
      else if(current_list_view->tab_column[curCol].sort == INTERNALS_COLUMN_SORT_DOWN)
        item.hbm = current_list_view->hBitmap_down;
      item.fmt |= HDF_STRING | HDF_BITMAP_ON_RIGHT | HDF_BITMAP;
      item.mask = HDI_TEXT | HDI_BITMAP | HDI_FORMAT;
      SendMessage(headerWnd,HDM_SETITEM,curCol,(LPARAM)&item);      
    }
}


/********************************************************************************************************/
/*  ProcessListViewCompareProc() :  Fonction de CallBack pour le tri des donnes des colonnes ListView. */
/********************************************************************************************************/
LRESULT CALLBACK ProcessListViewCompareProc(LPARAM lp1, LPARAM lp2, LPARAM sortParam)
{
  int i, item1_id, item2_id;
  struct list_view *current_listview;

  /* Identifiant de la ligne */
  item1_id = (int) lp1;
  item2_id = (int) lp2;
  current_listview = (struct list_view *) sortParam;

  /*** Tri selon la colonne ***/
  for(i=0; i<current_listview->nb_column; i++)
    {
      /** Gestion des Tri Numriques / Textuelles **/
      if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_UP)
        {
          if(current_listview->tab_column[i].type == INTERNALS_COLUMN_TYPE_TXT)
            {
              return(stricmp(current_listview->tab_column[i].tab_value_txt[item1_id],
                             current_listview->tab_column[i].tab_value_txt[item2_id]));
            }
          else if(current_listview->tab_column[i].type == INTERNALS_COLUMN_TYPE_NUM)
            {
              if(current_listview->tab_column[i].tab_value_num[item1_id] > 
                 current_listview->tab_column[i].tab_value_num[item2_id])
                return(1);
              else if(current_listview->tab_column[i].tab_value_num[item1_id] < 
                      current_listview->tab_column[i].tab_value_num[item2_id])
                return(-1);
              else
                return(0);
            }
        }
      else if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_DOWN)
        {
          if(current_listview->tab_column[i].type == INTERNALS_COLUMN_TYPE_TXT)
            {
              return(stricmp(current_listview->tab_column[i].tab_value_txt[item2_id],
                             current_listview->tab_column[i].tab_value_txt[item1_id]));
            }
          else if(current_listview->tab_column[i].type == INTERNALS_COLUMN_TYPE_NUM)
            {
              if(current_listview->tab_column[i].tab_value_num[item2_id] > 
                 current_listview->tab_column[i].tab_value_num[item1_id])
                return(1);
              else if(current_listview->tab_column[i].tab_value_num[item2_id] < 
                      current_listview->tab_column[i].tab_value_num[item1_id])
                return(-1);
              else
                return(0);
            }
        }
    }

  /* Never Here */
  return(0);
}


/*************************************************************************************/
/*  mem_free_listviewcolumn() :  Libration mmoire de la partie column du listview. */
/*************************************************************************************/
void mem_free_listviewcolumn(struct list_view *current_listview)
{
  int i, j;

  /** Supprime les donnes des colonnes **/
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Valeur au format texte */
      if(current_listview->tab_column[i].tab_value_txt != NULL)
        {
          for(j=0; j<current_listview->nb_row; j++)
            if(current_listview->tab_column[i].tab_value_txt[j])
              free(current_listview->tab_column[i].tab_value_txt[j]);

          free(current_listview->tab_column[i].tab_value_txt);
          current_listview->tab_column[i].tab_value_txt = NULL;
        }

      /* Valeur au format numrique */
      if(current_listview->tab_column[i].tab_value_num != NULL)
        {
          free(current_listview->tab_column[i].tab_value_num);
          current_listview->tab_column[i].tab_value_num = NULL;
        }
    }

  /* Plus de ligne de donnes */
  current_listview->nb_row = 0;
}


/********************************************************************/
/*  CyreneLogLine() :  Ajout d'une ligne dans le fichier CyreneLog. */
/********************************************************************/
void CyreneLogLine(char *command)
{
  int file_size;
  static time_t previous_clock;
  char temp_path[2048] = "";  
  char file_path[2048] = "";
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Vrification des paramtres */
  if(command == NULL)
    return;

  /* Rpertoire Temp */  
  GetEnvironmentVariable("Temp",temp_path,1024);
  if(strlen(temp_path) > 0)
    if(temp_path[strlen(temp_path)-1] != '\\')
      strcat(temp_path,"\\");
  
  /* Chemin du fichier */
  sprintf(file_path,"%s%s.txt",temp_path,param->trace_file_name);

  /** Gestion des commandes : Suppression des fichiers (Principal + Threads) **/
  if(!stricmp(command,"InitFile") || !stricmp(command,"DeleteFile"))
    { 
      /* Suppression du fichier */
      if(IsFileExist(file_path,0,&file_size))
        _unlink(file_path);
        
      /* Init de l'horloge */
      LogLine(NULL,NULL,0,1,&previous_clock);
      return;        
    }
    
  /** Ajout d'une ligne dans le fichier **/
  LogLine(file_path,command,1,0,&previous_clock);
}


/********************************************************/
/* LogLine() : Insre une ligne dans le fichier de Log. */
/********************************************************/
static void LogLine(char *file_name, char *line, int display_time, int first_time, time_t *previous_clock)
{
  FILE *stream;
  time_t clock;
  int diff, insert_log_break;
  struct _timeb timebuffer;
  struct tm *time_date;
  char buffer[256];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Ajoute le sparateur de seconde */
  insert_log_break = 1;
  if(param != NULL)
    if(param->insert_log_break == 0)
      insert_log_break = 0;
      
  /* La 1re fois, on initialise juste la date */
  if(first_time)
    {
      time(previous_clock);
      time_date = localtime(previous_clock);
      return;
    }
    
  /* Ouverture du fichier */
  stream = fopen(file_name,"a+");
  if(stream != NULL)
    {
      /* Ecriture de la date courante */
      if(display_time)
        {
          _ftime(&timebuffer);
          /* Date actuelle */
          time(&clock);
          time_date = localtime(&clock);

          /* Diffrence */
          if(*previous_clock != 0)
            {
              diff = (int) difftime(clock,*previous_clock);
              if(diff > 0 && insert_log_break == 1)
                {
                  sprintf(buffer,"- %d\n",diff);
                  fwrite(buffer,strlen(buffer),1,stream);      
                }
            }
            
          /* Conserve la date prcdente */
          time(previous_clock);
          time_date = localtime(previous_clock);

          /* Cration du message */
          sprintf(buffer,"%04d/%02d/%02d %02d:%02d:%02d [%03d]   ",
                  1900+time_date->tm_year,time_date->tm_mon+1,time_date->tm_mday,
                  time_date->tm_hour,time_date->tm_min,time_date->tm_sec,timebuffer.millitm);
          fwrite(buffer,strlen(buffer),1,stream);
        }
        
      /* Ecriture du message */
      fwrite(line,strlen(line),1,stream);
      fwrite("\n",1,1,stream);
      fclose(stream);
    }
  else
      perror("stream is NULL");
}


/*********************************************************************************/
/*  BuildUniqueListFromText() :  Rcupre la liste des valeurs unique d'un Text. */
/*********************************************************************************/
char **BuildUniqueListFromText(char *text, char separator, int *nb_value_rtn)
{
  int i, j, nb_found, nb_value, value_length;
  char **tab;
  char buffer_value[1024];

  /* Compte le nombre de sparateur */
  nb_value = 1;
  for(i=0; text[i] != '\0'; i++)
    if(text[i] == separator)
      nb_value++;

  /* Allocation du tableau */
  tab = (char **) calloc(nb_value,sizeof(char *));
  if(tab == NULL)
    return(NULL);

  /** Lecture des valeurs **/
  nb_found = 0;
  value_length = 0;
  for(i=0; text[i] != '\0'; i++)
    {
      if(text[i] == separator)
        {
          if(value_length > 0)
            {
              /* Termine la valeur */
              buffer_value[value_length] = '\0';
              
              /* Vrifie qu'elle n'existe pas dj */
              for(j=0; j<nb_found; j++)
                if(!strcmp(tab[j],buffer_value))
                  {
                    value_length = 0;
                    break;
                  }
                  
              /* Stocke la valeur */
              if(value_length > 0)
                {
                  tab[nb_found] = strdup(buffer_value);
                  if(tab[nb_found] == NULL)
                    {
                      for(j=0; j<nb_found; j++)
                        if(tab[j] != NULL)
                          free(tab[j]);
                      free(tab);
                      return(NULL);
                    }
                  nb_found++;
                  value_length = 0;
                }
            }
        }
      else
        buffer_value[value_length++] = text[i];
    }

  /** Dernire valeur **/
  if(value_length > 0)
    {
      /* Termine la valeur */
      buffer_value[value_length] = '\0';
      
      /* Vrifie qu'elle n'existe pas dj */
      for(j=0; j<nb_found; j++)
        if(!strcmp(tab[j],buffer_value))
          {
            value_length = 0;
            break;
          }
          
      /* Stocke la valeur */
      if(value_length > 0)
        {
          tab[nb_found] = strdup(buffer_value);
          if(tab[nb_found] == NULL)
            {
              for(j=0; j<nb_found; j++)
                if(tab[j] != NULL)
                  free(tab[j]);
              free(tab);
              return(NULL);
            }
          nb_found++;
          value_length = 0;
        }
    }

  /* Renvoi le tableau */
  *nb_value_rtn = nb_found;
  return(tab);
}


/****************************************************************/
/*  PunctuateAtThoudand() :  Ajoute les sparateurs de millier. */
/****************************************************************/
void PunctuateAtThoudand(char *buffer)
{
  int i,j,k;
  char buffer_punctuate[256];

  /* Ajoute les sparateurs */
  for(k=1,j=0,i=(int)strlen(buffer)-1; i>=0; i--,k++)
    {
      buffer_punctuate[j++] = buffer[i];
      if(k%3 == 0)
        buffer_punctuate[j++] = ' ';
    }
  buffer_punctuate[j] = '\0';
  if(buffer_punctuate[strlen(buffer_punctuate)-1] == ' ')
    buffer_punctuate[strlen(buffer_punctuate)-1] = '\0';

  /* Renvoi le buffer */
  for(j=0,i=(int)strlen(buffer_punctuate)-1; i>=0; i--)
    buffer[j++] = buffer_punctuate[i];
  buffer[j] = '\0';
}


/************************************************************************/
/*  LogTextLineDirect() :  Ecriture d'une ligne de log dans un fichier. */
/************************************************************************/
void LogTextLineDirect(char *log_file_path, char *log_line, int add_eol)
{
  FILE *fd;

  /* Ouverture du fichier */
  fd = fopen(log_file_path,"a+");
  if(fd == NULL)
    return;

  /* Ecriture de la ligne */
  fwrite(log_line,1,strlen(log_line),fd);

  /* Sparateur */
  if(add_eol == 1)
    fwrite("\n",1,strlen("\n"),fd);

  /* Fermeture du fichier */
  fclose(fd);
}


/**************************************************************/
/*  GetTextValue() :  Rcupre la valeur d'un buffer mmoire. */
/**************************************************************/
void GetTextValue(char *data, char *key_name, char *value_rtn)
{
  int length;
  char *begin_value;
  char *end_value;

  /* Init */
  strcpy(value_rtn,"");

  /* Rcupre la valeur */
  begin_value = my_stristr(data,key_name);
  if(begin_value)
    {
      begin_value += strlen(key_name);
      end_value = strchr(begin_value,'\n');
      length = (int) (end_value - begin_value);
      memcpy(value_rtn,begin_value,length);
      value_rtn[length] = '\0';
    }
}


/********************************************************************************/
/*  GetCurrentTimeMS() :  On rcupre l'heure actuelle (avec les milliseconds). */
/********************************************************************************/
char *GetCurrentTimeMS(char *buffer_time)
{
  time_t clock;
  struct _timeb timebuffer;
  struct tm *time_date;

  /* Date actuelle */
  _ftime(&timebuffer);
  time(&clock);
  time_date = localtime(&clock);

  /* Formate l'heure */
  sprintf(buffer_time,"[%02d:%02d:%02d-%03d]",time_date->tm_hour,time_date->tm_min,time_date->tm_sec,timebuffer.millitm);

  /* Renvoi l'heure */
  return(&buffer_time[0]);
}


/*********************************************************************************/
/*  GetCurrentTimeDwd() :  On rcupre l'heure actuelle (avec les milliseconds). */
/*********************************************************************************/
DWORD GetCurrentTimeDwd(void)
{
  time_t clock;
  struct _timeb timebuffer;
  struct tm *time_date;
  DWORD time_ms;

  /* Date actuelle */
  _ftime(&timebuffer);
  time(&clock);
  time_date = localtime(&clock);

  /* Formate l'heure */
  time_ms = time_date->tm_hour * 10000000 +
            time_date->tm_min  * 100000   +
            time_date->tm_sec  * 1000     +
            timebuffer.millitm;

  /* Renvoi l'heure */
  return(time_ms);
}


/*******************************************************************************************/
/*  GetCurrentTimeDiff() :  Calcule la diffrence entre maintenant et une date prcdente. */
/*******************************************************************************************/
DWORD GetCurrentTimeDiff(DWORD previous_td)
{
  DWORD current_td;

  /* Get Current Time Date */
  current_td = GetCurrentTimeDwd();
  
  /* Return Diff */
  return(current_td - previous_td);
}


/**************************************************************************************************/
/*  ConvertAppleIIgsHexaAsciiAsUTF8() :  Conversion d'une chaine de caractre Apple IIgs en UTF8. */
/**************************************************************************************************/
void ConvertAppleIIgsHexaAsciiAsUTF8(char *apple_string, char *utf8_string)
{
  int i, j, length;
  DWORD one_dword;
  unsigned char one_byte;
  char buffer_hexa[3] = {'0','0','\0'};

  /* Check Error */
  strcpy(utf8_string,"");
  length = (int) strlen(apple_string);
  if((length % 2) != 0)
    return;
  if(length < 2)
    return;
  if(apple_string[0] != '"')
    return;
  if(apple_string[length-1] != '"')
    return;

  /* Init */
  j = 0;
  utf8_string[j++] = '"';

  /** Conversion Hexa Ascii Apple IIgs => UTF8 **/
  for(i=1,j=1; i<length-1; i+=2)
    {
      /* HexaAscii => Apple IIgs Byte */
      memcpy(buffer_hexa,&apple_string[i],2);
      sscanf(buffer_hexa,"%02X",&one_dword);
      one_byte = (BYTE) (one_dword & 0x000000FF);

      /** Apple IIgs Byte => UTF8 **/
      /* Control Character */
      if(one_byte >= 0x00 && one_byte < 0x20)
        {
          /* \XX */
          utf8_string[j++] = '\\';
          utf8_string[j++] = buffer_hexa[0];
          utf8_string[j++] = buffer_hexa[1];
        }
      /* Ascii */
      else if(one_byte >= 0x20 && one_byte <= 0x7F)
        {
          /* Same Encoding */
          utf8_string[j++] = one_byte;
        }
      /* High Ascii */
      else  
        {
          switch (one_byte)
            {
              /*  (A with diaeresis) */
              case 0x80 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x84;
                break;
              /*  (A with ring) */
              case 0x81 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x85;
                break;
              /*  (C with cedilla) */
              case 0x82 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x87;
                break;
              /*  (E with acute accent) */
              case 0x83 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x89;
                break;
              /*  (N with tilde) */
              case 0x84 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x91;
                break;
              /*  (O with diaeresis) */
              case 0x85 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x96;
                break;
              /*  (U with diaeresis) */
              case 0x86 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x9C;
                break;
              /*  (a with acute accent) */
              case 0x87 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xA1;
                break;
              /*  (a with grave accent) */
              case 0x88 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xA0;
                break;
              /*  (a with circumflex) */
              case 0x89 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xA2;
                break;
              /*  (a with diaeresis) */
              case 0x8A :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xA4;
                break;
              /*  (a with tilde) */
              case 0x8B :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xA3;
                break;
              /*  (a with ring) */
              case 0x8C :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xA5;
                break;
              /*  (c with cedilla) */
              case 0x8D :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xA7;
                break;
              /*  (e with acute accent) */
              case 0x8E :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xA9;
                break;
              /*  (e with grave accent) */
              case 0x8F :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xA8;
                break;

              /*  (e with circumflex) */
              case 0x90 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xAA;
                break;
              /*  (e with diaeresis) */
              case 0x91 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xAB;
                break;
              /*  (i with acute accent) */
              case 0x92 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xAD;
                break;
              /*  (i with grave accent) */
              case 0x93 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xAC;
                break;
              /*  (i with circumflex) */
              case 0x94 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xAE;
                break;
              /*  (i with diaeresis) */
              case 0x95 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xAF;
                break;
              /*  (n with tilde) */
              case 0x96 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xC1;
                break;
              /*  (o with acute accent) */
              case 0x97 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xB3;
                break;
              /*  (o with grave accent) */
              case 0x98 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xB2;
                break;
              /*  (o with circumflex) */
              case 0x99 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xB4;
                break;
              /*  (o with diaeresis) */
              case 0x9A :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xB6;
                break;
              /*  (o with tilde) */
              case 0x9B :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xB5;
                break;
              /*  (u with acute accent) */
              case 0x9C :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xBA;
                break;
              /*  (u with grave accent) */
              case 0x9D :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xB9;
                break;
              /*  (u with circumflex) */
              case 0x9E :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xBB;
                break;
              /*  (u with diaeresis) */
              case 0x9F :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xBC;
                break;

              /*  (dagger) */
              case 0xA0 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x86;
                break;
              /*  (degree) */
              case 0xA1 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xB0;
                break;
              /*  (cent) */
              case 0xA2 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xA2;
                break;
              /*  (pound sterling) */
              case 0xA3 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xA3;
                break;
              /*  (section) */
              case 0xA4 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xA7;
                break;
              /*  (bullet) */
              case 0xA5 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x95;
                break;
              /*  (pilcrow [paragraph sign]) */
              case 0xA6 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xB6;
                break;
              /*  (small sharp s) */
              case 0xA7 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x9F;
                break;
              /*  (registered trademark) */
              case 0xA8 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xAE;
                break;
              /*  (copyright) */
              case 0xA9 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xA9;
                break;
              /*  (trademark) */
              case 0xAA :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x99;
                break;
              /*  (acute accent) */
              case 0xAB :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xB4;
                break;
              /*  (diaeresis) (umlaut) */
              case 0xAC :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xA8;
                break;
              /* not equal to */
              case 0xAD :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x89;
                utf8_string[j++] = 0xA0;
                break;
              /*  (AE ligature) */
              case 0xAE :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x86;
                break;
              /*  (O with slash) */
              case 0xAF :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x98;
                break;

              /* infinity */
              case 0xB0 :
                utf8_string[j++] = 0x22;
                utf8_string[j++] = 0x1E;
                break;
              /*  (plus or minus) */
              case 0xB1 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xB1;
                break;
              /* less than or equal to */
              case 0xB2 :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x89;
                utf8_string[j++] = 0xA4;
                break;
              /* greater than or equal to */
              case 0xB3 :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x89;
                utf8_string[j++] = 0xA5;
                break;
              /*  (yen) */
              case 0xB4 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xA5;
                break;
              /*  (micro) */
              case 0xB5 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xB5;
                break;
              /* ? partial differential */
              case 0xB6 :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x88;
                utf8_string[j++] = 0x82;
                break;
              /* ? summation */
              case 0xB7 :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x88;
                utf8_string[j++] = 0x91;
                break;
              /* ? product (capital pi) */
              case 0xB8 :
                utf8_string[j++] = 0xCE;
                utf8_string[j++] = 0xA0;
                break;
              /* small pi */
              case 0xB9 :
                utf8_string[j++] = 0xCF;
                utf8_string[j++] = 0x80;
                break;
              /* ? integral */
              case 0xBA :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x88;
                utf8_string[j++] = 0xAB;
                break;
              /*  (feminine ordinal) */
              case 0xBB :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xAA;
                break;
              /*  (masculine ordinal) */
              case 0xBC :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xBA;
                break;
              /* ohm (omega) */
              case 0xBD :
                utf8_string[j++] = 0xCE;
                utf8_string[j++] = 0xA9;
                break;
              /*  (ae ligature) */
              case 0xBE :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xA6;
                break;
              /*  (o with slash) */
              case 0xBF :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xB8;
                break;

              /*  (inverted question mark) */
              case 0xC0 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xBF;
                break;
              /*  (inverted exclamation mark) */
              case 0xC1 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xA1;
                break;
              /*  (not) */
              case 0xC2 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xAC;
                break;
              /* ? square root */
              case 0xC3 :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x88;
                utf8_string[j++] = 0x9A;
                break;
              /*  (small script f) */
              case 0xC4 :
                utf8_string[j++] = 0xF0;
                utf8_string[j++] = 0x9D;
                utf8_string[j++] = 0x91;
                utf8_string[j++] = 0x93;
                break;
              /* ? almost equal to */
              case 0xC5 :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x89;
                utf8_string[j++] = 0x88;
                break;
              /* ? increment (delta) */
              case 0xC6 :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x88;
                utf8_string[j++] = 0x86;
                break;
              /*  (left pointing guillemet) */
              case 0xC7 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xAB;
                break;
              /*  (right pointing guillemet) */
              case 0xC8 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xBB;
                break;
              /*  (horizontal ellipsis) */
              case 0xC9 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x85;
                break;
              /* (non-breaking space) */
              case 0xCA :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xA0;
                break;
              /*  (A with grave accent) */
              case 0xCB :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x80;
                break;
              /*  (A with tilde) */
              case 0xCC :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x83;
                break;
              /*  (O with tilde) */
              case 0xCD :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x95;
                break;
              /*  (OE ligature) */
              case 0xCE :
                utf8_string[j++] = 0xC5;
                utf8_string[j++] = 0x92;
                break;
              /*  (oe ligature) */
              case 0xCF :
                utf8_string[j++] = 0xC5;
                utf8_string[j++] = 0x93;
                break;

              /*  (endash) */
              case 0xD0 :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x80;
                utf8_string[j++] = 0x93;
                break;
              /*  (emdash) */
              case 0xD1 :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x80;
                utf8_string[j++] = 0x94;
                break;
              /*  (left double quote) */
              case 0xD2 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x93;
                break;
              /*  (right double quote) */
              case 0xD3 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x94;
                break;
              /*  (left single quote) */
              case 0xD4 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x91;
                break;
              /*  (right single quote) */
              case 0xD5 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x92;
                break;
              /*  (division) */
              case 0xD6 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xB7;
                break;
              /* ? lozenge */
              case 0xD7 :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x97;
                utf8_string[j++] = 0x8A;
                break;
              /*  (y with diaeresis) */
              case 0xD8 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xBF;
                break;
              /*  (Y with diaeresis) */
              case 0xD9 :
                utf8_string[j++] = 0xC5;
                utf8_string[j++] = 0xB8;
                break;
              /* ? fraction slash */
              case 0xDA :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x81;
                utf8_string[j++] = 0x84;
                break;
              /*  (euro) */
              case 0xDB :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x80;
                break;
              /*  (left pointing single guillemet) */
              case 0xDC :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x8B;
                break;
              /*  (right pointing single guillemet) */
              case 0xDD :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x9B;
                break;
              /*  (small fi ligature [capital thorn]) */
              case 0xDE :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x9E;
                break;
              /*  (small fl ligature [small thorn)] */
              case 0xDF :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xBE;
                break;

              /*  (double dagger) */
              case 0xE0 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x87;
                break;
              /*  (middle dot) */
              case 0xE1 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xB7;
                break;
              /*  (low single quote) */
              case 0xE2 :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x80;
                utf8_string[j++] = 0x9A;
                break;
              /*  (low double quote) */
              case 0xE3 :
                utf8_string[j++] = 0xE2;
                utf8_string[j++] = 0x80;
                utf8_string[j++] = 0x9E;
                break;
              /*  (per mille) */
              case 0xE4 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x89;
                break;
              /*  (A with circumflex) */
              case 0xE5 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x82;
                break;
              /*  (E with circumflex) */
              case 0xE6 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x8A;
                break;
              /*  (A with acute accent) */
              case 0xE7 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x81;
                break;
              /*  (E with diaeresis) */
              case 0xE8 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x8B;
                break;
              /*  (E with grave accent) */
              case 0xE9 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x88;
                break;
              /*  (I with acute accent) */
              case 0xEA :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x8D;
                break;
              /*  (I with circumflex) */
              case 0xEB :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x8E;
                break;
              /*  (I with diaeresis) */
              case 0xEC :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x8F;
                break;
              /*  (I with grave accent) */
              case 0xED :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x8C;
                break;
              /*  (O with acute accent) */
              case 0xEE :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xB3;
                break;
              /*  (O with circumflex) */
              case 0xEF :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0xB4;
                break;

              /* Apple Computer logo */
              case 0xF0 :
                utf8_string[j++] = 0xEF;
                utf8_string[j++] = 0xA3;
                utf8_string[j++] = 0xBF;
                break;
              /*  (O with grave accent) */
              case 0xF1 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x92;
                break;
              /*  (U with acute accent) */
              case 0xF2 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x9A;
                break;
              /*  (U with circumflex) */
              case 0xF3 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x9B;
                break;
              /*  (U with grave accent) */
              case 0xF4 :
                utf8_string[j++] = 0xC3;
                utf8_string[j++] = 0x99;
                break;
              /* i without a dot */
              case 0xF5 :
                utf8_string[j++] = 0xC4;
                utf8_string[j++] = 0xB1;
                break;
              /*  (circumflex accent) */
              case 0xF6 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x88;
                break;
              /*  (tilde accent) */
              case 0xF7 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0x98;
                break;
              /*  (macron) */
              case 0xF8 :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xAF;
                break;
              /* ? breve */
              case 0xF9 :
                utf8_string[j++] = 0xCB;
                utf8_string[j++] = 0x98;
                break;
              /* dot accent */
              case 0xFA :
                utf8_string[j++] = 0xCB;
                utf8_string[j++] = 0x99;
                break;
              /* ring */
              case 0xFB :
                utf8_string[j++] = 0xCB;
                utf8_string[j++] = 0x9A;
                break;
              /*  (cedilla accent) */
              case 0xFC :
                utf8_string[j++] = 0xC2;
                utf8_string[j++] = 0xB8;
                break;
              /* ? double acute accent */
              case 0xFD :
                utf8_string[j++] = 0xCB;
                utf8_string[j++] = 0x9D;
                break;
              /* ? ogonek */
              case 0xFE :
                utf8_string[j++] = 0xCB;
                utf8_string[j++] = 0x9B;
                break;
              /* ? hacek */
              case 0xFF :
                utf8_string[j++] = 0xCB;
                utf8_string[j++] = 0x87;
                break;
            }
        }
    }

  /* Fin de la chaine */
  utf8_string[j++] = '"';
  utf8_string[j] = '\0';
}


/***********************************************************************************/
/*  GetWindowControlInfo() :  Rcupre la position d'un controle dans une fentre. */
/***********************************************************************************/
void GetWindowControlInfo(HWND hwnd_container, HWND hwnd_control, int *x_rtn, int *y_rtn, int *width_rtn, int *height_rtn)
{
  RECT button1_rect;
  int width, height, x, y;

  /* Taille de la fentre */
  GetWindowRect(hwnd_control,&button1_rect);
  width = (button1_rect.right - button1_rect.left);
  height = (button1_rect.bottom - button1_rect.top);

  /* Position de la fentre */
  GetWindowRect(hwnd_control,&button1_rect);
  MapWindowPoints(HWND_DESKTOP,hwnd_container,(LPPOINT)&button1_rect,2);
  x = button1_rect.left;
  y = button1_rect.top;

  /* Renvoie les informations */
  *x_rtn = x;
  *y_rtn = y;
  *width_rtn = width;
  *height_rtn = height;
}

/***************************************************************************/
