/***********************************************************************/
/*                                                                     */
/*  Dc_Shared.h : Header pour la bibliothque gnrique de fonctions.  */
/*                                                                     */
/***********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Aug 2013   */
/***********************************************************************/

#define BUFFER_SIZE       1024

#define FOLDER_CHARACTER  "\\"

typedef unsigned __int64  QWORD;    /* Unsigned 64 bit */
typedef unsigned long     DWORD;    /* Unsigned 32 bit */
typedef unsigned short    WORD;     /* Unsigned 16 bit */
typedef unsigned char     BYTE;     /* Unsigned 8 bit */

#define MAKECOLOR(r,g,b)           ((COLORREF)(((BYTE)(b)|((WORD)((BYTE)(g))<<8))|(((DWORD)(BYTE)(r))<<16)))
#define REDVALUE(color)            (LOBYTE((color)>>16))
#define GREENVALUE(color)          (LOBYTE(((WORD)(color)) >> 8))
#define BLUEVALUE(color)           (LOBYTE(color))

#define COLOR_IIGS_BLACK        0
#define COLOR_IIGS_DEEP_RED     1
#define COLOR_IIGS_DARK_BLUE    2
#define COLOR_IIGS_PURPLE       3
#define COLOR_IIGS_DARK_GREEN   4
#define COLOR_IIGS_DARK_GRAY    5
#define COLOR_IIGS_MEDIUM_BLUE  6
#define COLOR_IIGS_LIGHT_BLUE   7
#define COLOR_IIGS_BROWN        8
#define COLOR_IIGS_ORANGE       9
#define COLOR_IIGS_LIGHT_GRAY  10
#define COLOR_IIGS_PINK        11
#define COLOR_IIGS_LIGHT_GREEN 12
#define COLOR_IIGS_YELLOW      13
#define COLOR_IIGS_AQUAMARINE  14
#define COLOR_IIGS_WHITE       15

#define LANGUAGE_IIGS_USA       0
#define LANGUAGE_IIGS_UK        1
#define LANGUAGE_IIGS_FRENCH    2
#define LANGUAGE_IIGS_DANISH    3
#define LANGUAGE_IIGS_SPANISH   4
#define LANGUAGE_IIGS_ITALIAN   5
#define LANGUAGE_IIGS_GERMAN    6
#define LANGUAGE_IIGS_SWEDISH   7
#define LANGUAGE_IIGS_MOUSETEXT 8

#define HGR_COLOR_BLACK       0
#define HGR_COLOR_GREEN       1
#define HGR_COLOR_PURPLE      2
#define HGR_COLOR_WHITE       3
#define HGR_COLOR_BLACK2      4    /* Noir #2 */
#define HGR_COLOR_ORANGE      5
#define HGR_COLOR_BLUE        6
#define HGR_COLOR_WHITE2      7    /* Blanc #2 */

#define GETASCIICODE(c) ((c >= 32 && c <= 126)?c:'.')

#define MY_MIN(a,b)     ((a) > (b) ? (b) : (a))
#define MY_MAX(a,b)     ((a) > (b) ? (a) : (b))

#if !defined(S_ISDIR) && defined(S_IFDIR)
#define S_ISDIR(m) (((m) & S_IFMT) == S_IFDIR)
#endif

#define PALETTE_COLOR_BORDER        4
#define PALETTE_COLOR_SIZE_WIDTH   13     /* Carr 11*20 */
#define PALETTE_COLOR_SIZE_HEIGHT  20

/* Structure Palette */
struct palette
{
  /* Nom de Palette */
  char path[1024];
  char name[1024];

  /* Nb de couleur */
  int nb_color;

  /* RGB */
  int palette_red[256];
  int palette_green[256];
  int palette_blue[256];

  /* HSL */
  int palette_h[256];    /* Teinte */
  int palette_s[256];    /* Saturation */
  int palette_l[256];    /* Lumience */

  struct palette *next;
};

/* List View */
#define INTERNALS_LISTVIEW_HEADER_HEIGHT    21 
#define INTERNALS_LISTVIEW_LINE_HEIGHT      15    /* 15 Pixels de Haut */
#define LISTVIEW_FONT_HEIGHT                14

#define INTERNALS_COLUMN_SORT_NONE      0
#define INTERNALS_COLUMN_SORT_UP        1
#define INTERNALS_COLUMN_SORT_DOWN      2

#define INTERNALS_COLUMN_TYPE_TXT       1
#define INTERNALS_COLUMN_TYPE_NUM       2

#define RESULT_LISTVIEW_HEADER_HEIGHT  21
#define RESULT_LISTVIEW_LINE_HEIGHT    16
#define RESULT_WINDOW_EXTRA_WIDTH      33
#define RESULT_WINDOW_EXTRA_HEIGHT     38

#define COLOR_LISTVIEW_GREY            RGB(240,240,240)
#define COLOR_LISTVIEW_WHITE           RGB(255,255,255)
#define COLOR_LISTVIEW_BLACK           RGB(0,0,0)
#define COLOR_LISTVIEW_BLUE            RGB(0,120,215)

#define MAX_LISTVIEW_COLUMN            20    /* Nombre MAx de colonne dans un ListView */

struct list_column
{
  int length;        /* Column length (in Pixels) */
  char title[256];   /* Titre de la colonnes */
  int align;         /* LVCFMT_LEFT / LVCFMT_CENTER / LVCFMT_RIGHT */
  int sort;          /* INTERNALS_COLUMN_SORT_UP / INTERNALS_COLUMN_SORT_NONE / INTERNALS_COLUMN_SORT_DOWN */
  int type;          /* INTERNALS_COLUMN_TYPE_TXT, INTERNALS_COLUMN_TYPE_NUM */

  /* Valeurs */
  char **tab_value_txt;           /* Version Texte (affiche dans la colonne) */
  DWORD *tab_value_num;           /* Version Numrique (utilise pour le tri) */
};

struct list_view
{
  int listview_x;
  int listview_y;
  int listview_width;
  int listview_height;

  HWND hwnd_listview;
  int idc_listview;

  /* Image de tri pour les colonnes */
  HBITMAP hBitmap_none;
  HBITMAP hBitmap_up;
  HBITMAP hBitmap_down;

  /* A t'on une colonne trie ? */
  int has_column_sort;

  /** Colonnes **/
  int nb_row;           /* Nombre de lignes */
  int nb_column;        /* Nombre de colonnes */
  struct list_column tab_column[MAX_LISTVIEW_COLUMN];
};

int GetFolderFiles(char *,char *,int);
int MatchHierarchie(char *,char *);
void CleanHierarchie(char *);
char *mh_stristr(char *,char *);
int mh_stricmp(char *,char *);
int IsFileExist(char *,int,int *);
int GetFileNumber(char *,int);
void GetOneFilePath(HWND,char *,char *,char *,char *);
BOOL GetSaveFilePath(HWND,char *,char *,char *);
unsigned char *LoadTextFile(char *,int *,char *);
unsigned char *LoadBinaryFile(char *,int *,char *);
void SaveTextFile(char *,char *,char *);
void SaveBinaryFile(char *,int,unsigned char *,char *);
int GetFileSizeInt(char *);
void CleanBuffer(char *);
int EvaluateAsOffsetOrLength(HWND,int);
int EvaluateAsAddress(char *,int *);
int EvaluateAsLength(char *,int *);
WORD EvaluateAsWord(char *,int *);
BYTE EvaluateAsByte(char *,int *);
BYTE EvaluateAsOneByte(char *,int *);
int EvaluateAsBit(char *,int *);
int IsHexaDecimal(char *,int *);
int IsDecimal(char *,int *);
int IsHexaDecimalChar(char);
int IsDecimalChar(char);
void RemoveSpace(char *);
void ExchangeByte(void *,int);
char *my_stristr(char *,char *);
void GetHexaValue(char *,char *);
void dump_memory(unsigned char *,int,int,char *);
struct file_path *BuildFilePath(char *);
int GetHGRColor(int,int,int,int,int);
void Extract_filename(char *,char *);
unsigned char *LoadFileAsBinary(char *,int *);
char **GetFilePath(HWND,char *,char *,int *,int);
void mem_free_list(int,char **);
struct palette *mem_alloc_palette(char *,char *);
struct palette *mem_alloc_palette_arne(void);
void mem_free_palette(struct palette *);
void RGB2HSL(int,int,int,int *,int *,int *);
void SetWindowTextHexaByte(HWND,int,unsigned char,unsigned char,int);
void FormatByteAsBinary(BYTE,char *);
void GetNamedTextValue(char *,char *,char *);
int GetNamedValueInt(char *,char *);
BYTE GetNamedValue8bit(char *,char *);
WORD GetNamedValue16bit(char *,char *);
void BuildListView(HWND,HFONT,int,int,int,struct list_view *);
LRESULT ProcessListViewWindowCustomDraw(LPARAM);
void setListViewSortIcon(struct list_view *);
LRESULT CALLBACK ProcessListViewCompareProc(LPARAM,LPARAM,LPARAM);
void mem_free_listviewcolumn(struct list_view *);
void CyreneLogLine(char *);
char **BuildUniqueListFromText(char *,char,int *);
void PunctuateAtThoudand(char *);
void LogTextLineDirect(char *,char *,int);
void GetTextValue(char *,char *,char *);
char *GetCurrentTimeMS(char *);
DWORD GetCurrentTimeDwd(void);
DWORD GetCurrentTimeDiff(DWORD);
void ConvertAppleIIgsHexaAsciiAsUTF8(char *, char *);
void GetWindowControlInfo(HWND,HWND,int *,int *,int *,int *);

/***********************************************************************/
