/**********************************************************************/
/*                                                                    */
/*   Window_About.c :  Module de gestion de la fentre About.         */
/*                                                                    */
/**********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Avril 2018  */
/**********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Graphic.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_About.h"

LRESULT CALLBACK ProcHandlerAboutWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateAboutChildWindows(HWND,struct window_about *);
static void UpdateAboutClientAllArea(HDC,struct window_about *);
static void UpdateAboutClientBrutalDeluxeArea(HDC,struct window_about *);
static void DeleteAboutChildWindows(struct window_about *);

static DWORD BuildAboutWindowThread(void *);

/************************************************************************/
/*  BuildAboutWindow() :  Construction / Affichage de la fentre About. */
/************************************************************************/
struct window_about *BuildAboutWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_about *window_about;

  /** Allocation de la structure window_about **/
  window_about = (struct window_about *) calloc(1,sizeof(struct window_about));
  if(window_about == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_ABOUT,window_about,NULL);

  /* Lance un thread pour grer la fentre About */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildAboutWindowThread,window_about,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_about);
}


/**********************************************************************************/
/*  BuildAboutWindowThread() :  Cration de la fentre About via un Thread ddi. */
/**********************************************************************************/
static DWORD BuildAboutWindowThread(void *window_about_param)
{
  HWND hwndAbout;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_about *window_about;
  struct parameter *param;

  /* Init */
  window_about = (struct window_about *) window_about_param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre About **/
  hwndAbout = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_ABOUT),window_main->hwnd_main,(DLGPROC)ProcHandlerAboutWindow);
  if(hwndAbout == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndAbout,SW_SHOWDEFAULT);
  UpdateWindow(hwndAbout);

  /** Message Loop pour toutes les fentres de ce Thread (fentre About...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/**********************************************************************/
/*  ProcHandlerAboutWindow() :  Gestion des vnements de la fentre. */
/**********************************************************************/
LRESULT CALLBACK ProcHandlerAboutWindow(HWND hwndAbout, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;
  struct window_about *window_about;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_ABOUT,&window_about,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateAboutChildWindows(hwndAbout,window_about);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_about->hBitmap_about != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndAbout,&ps);

            /** MAJ Bitmap Brutal Deluxe **/
            UpdateAboutClientAllArea(hDC,window_about);

            /* End Paint */
            EndPaint(hwndAbout,&ps);
          }
        break;

      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            case IDC_BUTTON_OK :
              /* Quite la fentre */
              SendMessage(hwndAbout,WM_CLOSE,0,0);
              break;
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre About */
        DestroyWindow(hwndAbout);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteAboutChildWindows(window_about);
        CheckMainWindowMenuItem(ID_MENU_ABOUT,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_ABOUT,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*************************************************************************************************/
/*  CreateAboutChildWindows() : Crer les fentres enfants et remplir la structure window_about. */
/*************************************************************************************************/
static int CreateAboutChildWindows(HWND hwndAbout, struct window_about *window_about)
{
  int i;
  BITMAPINFO bitmap_info;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_about->hwnd_about = hwndAbout;

  /** Device Context **/
  window_about->hDC = GetDC(window_about->hwnd_about);
  window_about->hCompatibleDC = CreateCompatibleDC(window_about->hDC);

  /** Hauteur et Largeur des Bitmap couleurs **/
  /* Image Logo Brutal Deluxe */
  window_about->bitmap_about_width = 428;
  window_about->bitmap_about_height = 104;

  /* Position des Bitmap dans la fenetre */
  window_about->bitmap_about_x = 10;
  window_about->bitmap_about_y = 10;

  /* Charge l'image de Log Brutal Deluxe */
  window_about->brutal_deluxe_picture = GetPictureFromResource(IDB_BITMAP_BRUTALDELUXE);
  if(window_about->brutal_deluxe_picture != NULL)
    {
      window_about->bitmap_about_width = window_about->brutal_deluxe_picture->width;
      window_about->bitmap_about_height = window_about->brutal_deluxe_picture->height;
    }

  /** Bitmap affichant le Loge Brutal Deluxe **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_about->bitmap_about_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_about->bitmap_about_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_about->hBitmap_about = CreateDIBSection(window_about->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_about->bitmap_data_about),NULL,0);
  if(window_about->brutal_deluxe_picture != NULL)
    memcpy(window_about->bitmap_data_about,window_about->brutal_deluxe_picture->data_dwd,window_about->bitmap_about_width*window_about->bitmap_about_height*sizeof(DWORD));
  else
    {
      /* Couleur Unie */
      for(i=0; i<window_about->bitmap_about_width*window_about->bitmap_about_height; i++)
        window_about->bitmap_data_about[i] = BITMAP_BACKGROUND_COLOR;
    }

  /** MAJ des Pixmap Brutal Deluxe de la fentre About **/
  InvalidateRect(hwndAbout,NULL,TRUE);  

  /* OK */
  return(0);
}


/****************************************************************************/
/*  UpdateAboutClientAllArea() :  Affichage des Bitmap de la fentre About. */
/****************************************************************************/
static void UpdateAboutClientAllArea(HDC hDC, struct window_about *window_about)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Image Brutal Deluxe **/
  UpdateAboutClientBrutalDeluxeArea(hDC,window_about);
}


/********************************************************************************/
/*  UpdateAboutClientBrutalDeluxeArea() :  Affichage de la Bitmap BrutalDeluxe. */
/********************************************************************************/
static void UpdateAboutClientBrutalDeluxeArea(HDC hDC, struct window_about *window_about)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec le logo Brutal Deluxe **/
  bitmap_old = SelectObject(window_about->hCompatibleDC,window_about->hBitmap_about);
  StretchBlt(hDC,window_about->bitmap_about_x,window_about->bitmap_about_y,window_about->bitmap_about_width,window_about->bitmap_about_height,
             window_about->hCompatibleDC,0,0,window_about->bitmap_about_width,window_about->bitmap_about_height,SRCCOPY);
  SelectObject(window_about->hCompatibleDC,bitmap_old);
}


/*******************************************************************/
/*  DeleteAboutChildWindows() :  Destruction des fentres enfants. */
/*******************************************************************/
static void DeleteAboutChildWindows(struct window_about *window_about)
{
  DeleteObject(window_about->hBitmap_about);
  ReleaseDC(window_about->hwnd_about,window_about->hDC);
  DeleteDC(window_about->hCompatibleDC);   
}


/*********************************************************************/
/*  mem_free_window_about() :  Libration mmoire pour la structure. */
/*********************************************************************/
void mem_free_window_about(struct window_about *window_about)
{
  if(window_about)
    {
      if(window_about->brutal_deluxe_picture != NULL)
        mem_free_picture_true(window_about->brutal_deluxe_picture);

      free(window_about);
    }
}

/**********************************************************************/
