/***********************************************************************/
/*                                                                     */
/*  Window_Breakpoint.h :  Header de gestion de la fentre Breakpoint. */
/*                                                                     */
/***********************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : Nov 2023  */
/***********************************************************************/

/* Un message destin  la fentre Breakpoint pour demander la maj de l'affichage */
#define HM_BREAKPOINT_REFRESHSYMBOL      (WM_USER + 24)

#define BREAKPOINT_EDIT_FONT_HEIGHT       15    /* Taille de la Fonte en Pixel */

#define BREAKPOINT_LISTVIEW_NB_ROW        12    /* Nombre de ligne de donnes dans la ListView  */
#define BREAKPOINT_LISTVIEW_MIN_NB_ROW     2    /* Nombre de ligne de donnes Min dans la ListView  */

#define IDC_LISTVIEW_BREAKPOINT         1032
#define IDC_COMBOBOX_SYSTEMCALL         1033
#define IDC_COMBOBOX_REGISTER           1034
#define IDC_COMBOBOX_READ_OPERATOR      1035
#define IDC_COMBOBOX_WRITE_OPERATOR     1036
#define IDC_COMBOBOX_READWRITE_OPERATOR 1037
#define IDC_COMBOBOX_HOLD_OPERATOR      1038
#define IDC_COMBOBOX_SYMBOL             1039

#define BREAKPOINT_NUMBER_MAX            256    /* Maximum number of Breakpoint */

#define BREAKPOINT_SOURCE_ADDRESS          1
#define BREAKPOINT_SOURCE_SYMBOL           2
#define BREAKPOINT_SOURCE_SYSTEMCALL       3
#define BREAKPOINT_SOURCE_REGISTER         4
#define BREAKPOINT_SOURCE_OPERATION        5

#define BREAKPOINT_TRIGGER_READ            1
#define BREAKPOINT_TRIGGER_WRITE           2
#define BREAKPOINT_TRIGGER_READWRITE       3
#define BREAKPOINT_TRIGGER_EXECUTE         4
#define BREAKPOINT_TRIGGER_HOLD            5

#define BREAKPOINT_ACTION_STOP               1
#define BREAKPOINT_ACTION_PRINT              2
#define BREAKPOINT_ACTION_FILE               3
#define BREAKPOINT_ACTION_TRACE_SYSTEM_CALL  4

#define BREAKPOINT_REPEAT_ALWAYS           1
#define BREAKPOINT_REPEAT_ONLY             2
#define BREAKPOINT_REPEAT_AFTER            3

#define BREAKPOINT_REGISTER_A              0
#define BREAKPOINT_REGISTER_X              1
#define BREAKPOINT_REGISTER_Y              2
#define BREAKPOINT_REGISTER_PC             3
#define BREAKPOINT_REGISTER_STACK          4
#define BREAKPOINT_REGISTER_DIRECT         5
#define BREAKPOINT_REGISTER_DBR            6
#define BREAKPOINT_REGISTER_PBR            7
#define BREAKPOINT_REGISTER_STATUS         8

#define BREAKPOINT_OPERATOR_EQUAL          0
#define BREAKPOINT_OPERATOR_DIFFERENT      1
#define BREAKPOINT_OPERATOR_GREATER        2
#define BREAKPOINT_OPERATOR_GREATER_EQUAL  3
#define BREAKPOINT_OPERATOR_LOWER          4
#define BREAKPOINT_OPERATOR_LOWER_EQUAL    5

#define BREAKPOINT_LINE_TARGET_KEGS        1
#define BREAKPOINT_LINE_TARGET_FILE        2
#define BREAKPOINT_LINE_TARGET_STATUS      3


struct cyrene_breakpoint              /* A synchroniser avec Dc_Cyene.h dans KEGS */
{
  int status;                         /* 0:Disable / 1:Enable */

  DWORD id;                           /* Identifiant unique d'un Breakpoint (1-N) */

  /* Source */
  int source;                         /* BREAKPOINT_SOURCE_XXX */
  char source_address_bank[3];        /* 03 ou ** */
  char source_address_address[5];     /* 20FA ou 20F* */
  char source_address_bank_dst[3];    /* 03 ou ** */
  char source_address_address_dst[5]; /* 20FA ou 20F* */
  DWORD source_symbol_begin_address;  /* 0xFFFFFFFF si inconnu, Adresse du Symbol */
  DWORD source_symbol_end_address;    /* 0xFFFFFFFF si non utilis, Adresse de fin des Data */
  char source_symbol_program[256];
  char source_symbol_segment[256];
  char source_symbol_name[256];
  int source_systemcall_type;         /* SystemCall Type TYPE_SYSTEM_CALL_XXX */
  WORD source_systemcall_number;      /* SystemCall Call Number */
  char source_systemcall_text[256];
  int source_systemcall_index;        /* Indice 0-N dans le tableau param->tab_systemcall[] */
  int source_register;                /* BREAKPOINT_REGISTER_XXX */
  char source_operation_text[256];
  int source_operation_data_nb;
  BYTE source_operation_data[4];

  /* Trigger */
  int trigger;                        /* BREAKPOINT_TRIGGER_XXX */
  int trigger_read_operator;          /* BREAKPOINT_OPERATOR_XXX */
  int trigger_read_value_size;        /* 0:None / 1:8bit / 2:16bit */
  WORD trigger_read_value;
  int trigger_write_operator;         /* BREAKPOINT_OPERATOR_XXX */
  int trigger_write_value_size;       /* 0:None / 1:8bit / 2:16bit */
  WORD trigger_write_value;
  int trigger_readwrite_operator;     /* BREAKPOINT_OPERATOR_XXX */
  int trigger_readwrite_value_size;   /* 0:None / 1:8bit / 2:16bit */
  WORD trigger_readwrite_value;
  int trigger_hold_operator;          /* BREAKPOINT_OPERATOR_XXX */
  int trigger_hold_value_size;        /* 0:None / 1:8bit / 2:16bit */
  WORD trigger_hold_value;

  /* Action */
  int action;                         /* BREAKPOINT_ACTION_XXX */
  char action_print_text[1024];       /* [A] [X] [Y] [PC] ... [012000] */
  char action_file_path[1024];        /* File Path */

  /* Repeat */
  int repeat;                         /* BREAKPOINT_REPEAT_XXX */
  int repeat_only_times;

  struct cyrene_breakpoint *next;
};

struct breakpoint_table
{
  int nb_breakpoint;
  struct cyrene_breakpoint *first_breakpoint;
  struct cyrene_breakpoint *last_breakpoint;

  struct cyrene_breakpoint **tab_breakpoint;
};

struct window_breakpoint
{
  HWND hwnd_breakpoint;
  HFONT hFont;
  HWND hwnd_combo_register;
  HWND hwnd_combo_systemcall;
  HWND hwnd_combo_read_operator;
  HWND hwnd_combo_write_operator;
  HWND hwnd_combo_readwrite_operator;
  HWND hwnd_combo_hold_operator;

  /* Surchage des Control Edit pour rcuprer les vnements Souris + Entre */
  WNDPROC oldEditProcSrcBank;
  WNDPROC oldEditProcSrcAddress;
  WNDPROC oldEditProcDstBank;
  WNDPROC oldEditProcDstAddress;
  WNDPROC oldEditProcOperationData;
  WNDPROC oldEditProcReadValue;
  WNDPROC oldEditProcWriteValue;
  WNDPROC oldEditProcReadWriteValue;
  WNDPROC oldEditProcHoldValue;
  WNDPROC oldEditProcPrintData;
  WNDPROC oldEditProcFilePath;
  WNDPROC oldEditProcRepeatOnly;

  /* Mode de retaille de la fentre (user/executable) */
  int user_resize;

  /* Dimension de la fentre  la cration */
  int window_init_width;
  int window_init_height;
  int nb_init_data_row;

  /* List View */
  int nb_row;                      /* Nombre de lignes affiches dans la ListBox */
  int breakpoint_x0;
  int breakpoint_y0;
  struct list_view listview_breakpoint;
  
  /** Breakpoint en cours **/
  int edit_mode;                            /* 1 si Edit Mode (New / Modify) */

  /* Source */
  int source_radio;                         /* BREAKPOINT_SOURCE_XXX */
  char source_address_bank[3];              /* XX ou *X ou *X ou ** */
  char source_address_address[5];           /* XXXX ou XX*X ou **** */
  char source_address_bank_dst[3];          /* XX ou *X ou X* ou ** ou VIDE */
  char source_address_address_dst[5];       /* XXXX ou XX*X ou **** ou VIDE */
  DWORD source_symbol_begin_address;        /* Adresse rsolue du Symbol en mmoire XX/XXXX */
  DWORD source_symbol_end_address;          /* Adresse rsolue de la fin des Data du Symbol en mmoire XX/XXXX */
  char source_symbol_program[256];
  char source_symbol_segment[256];
  char source_symbol_name[256];
  int source_systemcall_type;               /* TYPE_SYSTEM_CALL_XXX */
  WORD source_systemcall_number;
  char source_systemcall_text[256];
  int source_systemcall_index;              /* Indice 0-N dans le tableau param->tab_systemcall[] */
  int source_register;                      /* BREAKPOINT_REGISTER_XXX */
  char source_operation_text[256];
  int source_operation_data_nb;
  BYTE source_operation_data[4];

  /* Trigger */
  int trigger_radio;                        /* BREAKPOINT_TRIGGER_XXX */
  int trigger_read_operator;                /* BREAKPOINT_OPERATOR_XXX */
  int trigger_read_operator_value_size;
  WORD trigger_read_operator_value;
  int trigger_write_operator;               /* BREAKPOINT_OPERATOR_XXX */
  int trigger_write_operator_value_size;
  WORD trigger_write_operator_value;
  int trigger_readwrite_operator;           /* BREAKPOINT_OPERATOR_XXX */
  int trigger_readwrite_operator_value_size;
  WORD trigger_readwrite_operator_value;
  int trigger_hold_operator;                /* BREAKPOINT_OPERATOR_XXX */
  int trigger_hold_operator_value_size;
  WORD trigger_hold_operator_value;

  /* Action */
  int action_radio;                         /* BREAKPOINT_ACTION_XXX */
  char action_print_data[1024];
  char action_file_path[1024];

  /* Repeat */
  int repeat_radio;                         /* BREAKPOINT_REPEAT_XXX */
  int repeat_only_times;

  /* Breakpoint slectionn */
  struct cyrene_breakpoint *selected_breakpoint;

  /* Buffer */
  char buffer_log[BUFFER_SIZE];  
};

struct window_breakpoint *BuildBreakpointWindow(HINSTANCE,int);
void LoadAppendBreakpointFile(char *,struct window_breakpoint *);
struct cyrene_breakpoint *GetBreakpointByID(DWORD);
void BuildBreakpointTextLine(struct cyrene_breakpoint *,int,char *);
void ClearBreakpointTable(struct window_breakpoint *);
void RefreshBreakpointWindow(struct window_breakpoint *);
void UpdateBreakpointSymbolAddress(void);
void mem_free_window_breakpoint(struct window_breakpoint *);
void mem_free_breakpoint(struct cyrene_breakpoint *);
void mem_free_breakpoint_table(struct breakpoint_table *);

/********************************************************/
