/********************************************************************/
/*                                                                  */
/*  Window_Compare.h :  Header de gestion de la fentre Compare.    */
/*                                                                  */
/********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Jan 2019  */
/********************************************************************/

/* Control Edit pour les Compare Result */
#define IDC_RICHEDIT_COMPARE_DATA       1625

#define COMPARE_EDIT_FONT_HEIGHT          14   /* Taille de la Fonte en Pixel */

#define COMPARE_RICHEDIT_FONT_HEIGHT      14   /* Taille de la Fonte en Pixel */
#define COMPARE_RICHEDIT_FONT_HEIGHT_RTF "16"  /* Taille de la Fonte dans le RTF */

#define COMPARE_RICHEDIT_HEADER_SIZE    1024   /* Nombre d'octet MAX pour encoder le header du RichEdit */
#define COMPARE_RICHEDIT_MAX_CHAR_SIZE    24   /* Nombre d'octet MAX pour encoder un caractre (y compris sa couleur...) */
#define COMPARE_RICHEDIT_FOOTER_SIZE     512   /* Nombre d'octet MAX pour encoder le footer du RichEdit */

#define COMPARE_RICHEDIT_COLOR_BLACK       0   /* Couleurs pour le RichEdit */
#define COMPARE_RICHEDIT_COLOR_RED         1
#define COMPARE_RICHEDIT_COLOR_BLUE        2
#define COMPARE_RICHEDIT_COLOR_GREEN       3
#define COMPARE_RICHEDIT_COLOR_DARKRED     4
#define COMPARE_RICHEDIT_COLOR_DARKGREY    5

#define COMPARE_RAMTYPE_RAM                0
#define COMPARE_RAMTYPE_DOCRAM             1

struct compare_result
{ 
  int compare_area1;        /* COMPARE_RAMTYPE_XXX */
  int compare_area2;
  
  int bank1;                /* Addresse o se trouve les donnes #1 */
  int address1;

  int bank2;                /* Addresse o se trouve les donnes #2 */
  int address2;

  struct compare_result *next;
};

struct window_compare
{
  HWND hwnd_compare;
  HWND hwnd_richedit_data;      /* Control multi-lignes RTF */

  DWORD thread_id;              /* Le ThreadID va permettre de diffrencier les diffrentes fentres */

  /** Surcharge des Control Edit **/
  WNDPROC oldEditProcBank1;
  WNDPROC oldEditProcAddress1;
  WNDPROC oldEditProcBank2;
  WNDPROC oldEditProcAddress2;
  WNDPROC oldEditProcAreaLength;

  /** Zones de Comparaison **/
  int ram_type_1;                /* COMPARE_RAMTYPE_XXX */
  int bank_1;
  int address_1;
  int ram_type_2;                /* COMPARE_RAMTYPE_XXX */
  int bank_2;
  int address_2;
  int area_length;
  
  /** Rsultat de la comparaison **/
  int nb_found;
  int current_result;
  struct compare_result *first_compareresult;
  struct compare_result *last_compareresult;

  /* Posiiton / Taille du Control Group Box */
  int group_data_x;
  int group_data_y;

  /* Position / Taille du Control Text Zone Compare Result Data */
  int compare_data_x;
  int compare_data_y;
  int compare_data_width;
  int compare_data_height;

  /** Control Edit : Zone Texte o sont affiches les Data **/  
  int nb_line;                     /* 2 */  
  int nb_column;                   /* 16 */
  int text_data_width;             /* "00/0000___00_00_00_00_00_00_00_00.00_00_00_00_00_00_00_00\r\n" */
  int rtf_data_size;               /* Taille de la Zone Texte RTF */
  char *rtf_data_hexadecimal;      /* Zone Texte RTF */  
  int rtf_write_offset;            /* Offset de l'criture de la Zone Texte RTF vers le Rich Edit */

  /* Dimension de la fentre = Title + Zone Texte */
  int title_height;
  int window_width;
  int window_height;
  
  /* Buffer */
  char buffer_log[BUFFER_SIZE];

  /* Liste chaine des Compare */
  struct window_compare *next;
};

struct window_compare *BuildCompareWindow(HINSTANCE,int);
void mem_free_compareresult(struct compare_result *);
void mem_free_window_compare(struct window_compare *);

/********************************************************/
