/****************************************************************************/
/*                                                                          */
/*  Window_Disassembler.h :  Header de gestion de la fentre Disassembler.  */
/*                                                                          */
/****************************************************************************/
/*   Auteur : Olivier ZARDINI    *   Brutal Deluxe   *   Date : Aout 2018   */
/****************************************************************************/

/* Un message destin  la fentre Disassembler pour demander la maj de l'affichage */
#define HM_DISASSEMBLER_REFRESHWINDOW               (WM_USER + 24)
#define HM_DISASSEMBLER_REFRESHSYMBOL               (WM_USER + 25)
#define IDC_RICHEDIT_DISASSEMBLER_DATA      1623

#define IDM_CONTEXT_DISASSEMBLER_00         1624    /* Menu Contextuel pour les directives */
#define IDM_CONTEXT_DISASSEMBLER_01         1625
#define IDM_CONTEXT_DISASSEMBLER_10         1626
#define IDM_CONTEXT_DISASSEMBLER_11         1627
#define IDM_CONTEXT_DISASSEMBLER_CLEAR      1628

#define DISASSEMBLER_REFRESH_AUTOMATIC  0
#define DISASSEMBLER_REFRESH_MANUAL     1 

#define DISASSEMBLER_PROC_6502          0
#define DISASSEMBLER_PROC_65C02         1
#define DISASSEMBLER_PROC_65C816        2

#define DISASSEMBLER_REGISTER_00        0
#define DISASSEMBLER_REGISTER_01        1
#define DISASSEMBLER_REGISTER_10        2
#define DISASSEMBLER_REGISTER_11        3

#define DISASSEMBLER_RICHEDIT_FONT_HEIGHT      14   /* Taille de la Fonte en Pixel */
#define DISASSEMBLER_RICHEDIT_FONT_HEIGHT_RTF "16"  /* Taille de la Fonte dans le RTF */

#define DISASSEMBLER_RICHEDIT_HEADER_SIZE    1024   /* Nombre d'octet MAX pour encoder le header du RichEdit */
#define DISASSEMBLER_RICHEDIT_MAX_CHAR_SIZE    24   /* Nombre d'octet MAX pour encoder un caractre (y compris sa couleur...) */
#define DISASSEMBLER_RICHEDIT_FOOTER_SIZE     512   /* Nombre d'octet MAX pour encoder le footer du RichEdit */

#define DISASSEMBLER_RICHEDIT_COLOR_BLACK       0   /* Couleurs pour le RichEdit */
#define DISASSEMBLER_RICHEDIT_COLOR_RED         1
#define DISASSEMBLER_RICHEDIT_COLOR_BLUE        2
#define DISASSEMBLER_RICHEDIT_COLOR_GREEN       3
#define DISASSEMBLER_RICHEDIT_COLOR_DARKRED     4
#define DISASSEMBLER_RICHEDIT_COLOR_DARKGREY    5

#define DISASSEMBLER_AREA_MX            "11 "                   /*  3 */
#define DISASSEMBLER_AREA_ADDRESS       "00/0000: "             /*  9 */
#define DISASSEMBLER_AREA_DATA          "00 00 00 00  "         /* 13 */
#define DISASSEMBLER_AREA_LABEL         "                    "  /* 20 */
#define DISASSEMBLER_AREA_OPCODE        "LDAL "                 /*  5 */
#define DISASSEMBLER_AREA_OPERAND       "$012000,X    "         /* 13 */
#define DISASSEMBLER_AREA_COMMENT       "; Comment-----------"  /* 20 */

struct window_disassembler
{
  HWND hwnd_disassembler;
  HWND hwnd_richedit_data;      /* Control multi-lignes RTF */

  DWORD thread_id;              /* Le ThreadID va permettre de diffrencier les diffrentes fentres */

  /* Surchage des Control Edit pour rcuprer les vnements Souris + Entre */
  WNDPROC oldEditProcDisassemblerData;
  WNDPROC oldEditProcBank;
  WNDPROC oldEditProcAddress;
  WNDPROC oldEditProcRow;
  WNDPROC oldEditProcDataBank;

  /* Mode de rafraichissement de la fentre */
  int refresh_mode;        /* DISASSEMBLER_REFRESH_XXX */
  
  int processor;           /* DISASSEMBLER_PROC_XXX */

  int radio_register_mx;   /* Radio Bouton DISASSEMBLER_REGISTER_XXX */

  /* Posiiton / Taille du Control Group Box */
  int group_data_x;
  int group_data_y;
  int group_data_width;
  int group_data_height;

  /* Position / Taille du Control Text Zone Disassembler Data */
  int disassembler_data_x;
  int disassembler_data_y;
  int disassembler_data_width;
  int disassembler_data_height;

  int text_min_row;                /* Nombre minimum de ligne */

  /** Control Edit : Zone Texte o sont affiches les Data **/   
  int text_data_column;            /* Nombre de colonne Data */
  int text_data_row;               /* Nombre de ligne Data */
  int text_data_width;             /* "11  00/0000  00_00_00_00  LDAL $012000,X   ; COMMENT\r\n" */
  int rtf_data_size;               /* Taille de la Zone Texte RTF */
  char *rtf_data_hexadecimal;      /* Zone Texte RTF */  
  int rtf_write_offset;            /* Offset de l'criture de la Zone Texte RTF vers le Rich Edit */

  /** Donnes actuellement affiches **/
  DWORD init_address;              /* Adresse */
  int disassembler_bank;
  int disassembler_address;
  int disassembler_data_bank;
  int nb_row;
  unsigned char disassembler_data[1024];    /* nb_row * 4 char MAX */

  /** Version dsassembl du code **/
  int current_register_mx;
  struct code_line *first_code_line;
  struct code_line *last_code_line;

  /* Liste des directives d'assemblage */
  struct directive_line *first_directive_line;
  struct directive_line *last_directive_line;

  int context_directive_address;     /* Addresse de la ligne o insrer/supprimer une directive */

  /* Dimension de la fentre = Title + Zone Texte */
  int window_border_size;       /* Taille en Pixel de la bordure de la fentre */
  int title_height;
  int window_width;
  int window_height;

  /* Buffer */
  char buffer_log[BUFFER_SIZE];

  /* Notification */
  struct notification *current_notification;

  /* Liste chaine des Disassembler */
  struct window_disassembler *next;
};

struct window_disassembler *BuildDisassemblerWindow(HINSTANCE,int,DWORD);
void mem_free_window_disassembler(struct window_disassembler *);

/***************************************************************/
