/************************************************************************/
/*                                                                      */
/*  Window_GraphicDGR.c :  Module de gestion de la fentre GraphicDGR.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Juillet 2018  */
/************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_GraphicDGR.h"

static DWORD BuildGraphicDGRWindowThread(void *);
LRESULT CALLBACK ProcHandlerGraphicDGRWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateGraphicDGRChildWindows(HWND,struct window_graphicdgr *);
LRESULT CALLBACK subEditGraphicDGRProcControl(HWND,UINT,WPARAM,LPARAM);
void UpdateGraphicDGRClientAllArea(HDC,struct window_graphicdgr *);
static void UpdateGraphicDGRClientBitmapArea(HDC,struct window_graphicdgr *);
static void UpdateGraphicDGRContextInformation(struct window_graphicdgr *,int,int);
static void UpdateGraphicDGRAddress(struct window_graphicdgr *,int);
static void UpdateGraphicDGRRefreshMode(struct window_graphicdgr *,int);
static void UpdateGraphicDGRZoomLevel(struct window_graphicdgr *,int);
static void RefreshGraphicDGRWindowData(struct window_graphicdgr *,int);
static int IsGraphicDGRDataModified(int,int,int,int,struct window_graphicdgr *);
static void ResizeGraphicDGRWindow(struct window_graphicdgr *);
static void BuildGraphicDGRFromData(struct window_graphicdgr *);
static void DeleteGraphicDGRChildWindows(struct window_graphicdgr *);


/**********************************************************************************/
/*  BuildGraphicDGRWindow() :  Construction / Affichage de la fentre GraphicDGR. */
/**********************************************************************************/
struct window_graphicdgr *BuildGraphicDGRWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_graphicdgr *window_graphicdgr;

  /** Allocation de la structure window_graphicdgr **/
  window_graphicdgr = (struct window_graphicdgr *) calloc(1,sizeof(struct window_graphicdgr));
  if(window_graphicdgr == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_GRAPHICDGR,window_graphicdgr,NULL);

  /* Lance un thread pour grer la fentre GraphicDGR */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildGraphicDGRWindowThread,window_graphicdgr,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_graphicdgr);
}


/********************************************************************************************/
/*  BuildGraphicDGRWindowThread() :  Cration de la fentre GraphicDGR via un Thread ddi. */
/********************************************************************************************/
static DWORD BuildGraphicDGRWindowThread(void *window_graphicdgr_param)
{
  HWND hwndGraphicDGR;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_graphicdgr *window_graphicdgr;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_graphicdgr = (struct window_graphicdgr *) window_graphicdgr_param;
  window_graphicdgr->thread_id = GetCurrentThreadId();         /* Le ThreaID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Graphic GR **/
  hwndGraphicDGR = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_GRAPHICDGR),window_main->hwnd_main,(DLGPROC)ProcHandlerGraphicDGRWindow);
  if(hwndGraphicDGR == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndGraphicDGR,SW_SHOWDEFAULT);
  UpdateWindow(hwndGraphicDGR);

  /** Message Loop pour toutes les fentres de ce Thread (fentre GraphicDGR + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***************************************************************************/
/*  ProcHandlerGraphicDGRWindow() :  Gestion des vnements de la fentre. */
/***************************************************************************/
LRESULT CALLBACK ProcHandlerGraphicDGRWindow(HWND hwndGraphicDGR, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;
  int mouse_x, mouse_y;
  struct window_graphicdgr *window_graphicdgr;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_GRAPHICDGR,&window_graphicdgr,NULL);
  if(window_graphicdgr == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateGraphicDGRChildWindows(hwndGraphicDGR,window_graphicdgr);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        UpdateGraphicDGRContextInformation(window_graphicdgr,mouse_x,mouse_y);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_graphicdgr->hBitmap_graphicdgr != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndGraphicDGR,&ps);

            /** MAJ Bitmap **/
            UpdateGraphicDGRClientAllArea(hDC,window_graphicdgr);

            /* End Paint */
            EndPaint(hwndGraphicDGR,&ps);
          }
        break;
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdgr->refresh_mode != GRAPHICDGR_REFRESH_AUTOMATIC)
                UpdateGraphicDGRRefreshMode(window_graphicdgr,GRAPHICDGR_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdgr->refresh_mode != GRAPHICDGR_REFRESH_MANUAL)
                UpdateGraphicDGRRefreshMode(window_graphicdgr,GRAPHICDGR_REFRESH_MANUAL);
              break;

            /** Bouton Radio : On change le Zoom de la fentre **/
            case IDC_RADIO_ZOOM_X1 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdgr->zoom_level != GRAPHICDGR_ZOOM_X1)
                UpdateGraphicDGRZoomLevel(window_graphicdgr,GRAPHICDGR_ZOOM_X1);
              break;

            case IDC_RADIO_ZOOM_X2 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdgr->zoom_level != GRAPHICDGR_ZOOM_X2)
                UpdateGraphicDGRZoomLevel(window_graphicdgr,GRAPHICDGR_ZOOM_X2);
              break;

            /** Bouton de dplacement de l'adresse **/
            case IDC_BUTTON_LEFT :
              UpdateGraphicDGRAddress(window_graphicdgr,-1);
              break;

            case IDC_BUTTON_RIGHT :
              UpdateGraphicDGRAddress(window_graphicdgr,1);
              break;

            case IDC_BUTTON_UP :
              UpdateGraphicDGRAddress(window_graphicdgr,-40);
              break;

            case IDC_BUTTON_DOWN :
              UpdateGraphicDGRAddress(window_graphicdgr,40);
              break;

            case IDC_BUTTON_PAGEUP :
              UpdateGraphicDGRAddress(window_graphicdgr,-1024);
              break;

            case IDC_BUTTON_PAGEDOWN :
              UpdateGraphicDGRAddress(window_graphicdgr,1024);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshGraphicDGRWindowData(window_graphicdgr,1);
              break;              
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_GRAPHICDGR_REFRESHWINDOW :
        RefreshGraphicDGRWindowData(window_graphicdgr,0);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre GraphicDGR */
        DestroyWindow(hwndGraphicDGR);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteGraphicDGRChildWindows(window_graphicdgr);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_GRAPHICDGR,window_graphicdgr,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************/
/*  CreateGraphicDGRChildWindows() : Crer les fentres enfants et remplir la structure window_graphicdgr. */
/***********************************************************************************************************/
static int CreateGraphicDGRChildWindows(HWND hwndGraphicDGR, struct window_graphicdgr *window_graphicdgr)
{
  BITMAPINFO bitmap_info;
  int i;
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_graphicdgr->hwnd_graphicdgr = hwndGraphicDGR;

  /** Device Context **/
  window_graphicdgr->hDC = GetDC(window_graphicdgr->hwnd_graphicdgr);
  window_graphicdgr->hCompatibleDC = CreateCompatibleDC(window_graphicdgr->hDC);

  /** Surchage des Control Edit pour capter les vnements **/
  window_graphicdgr->oldEditProcBank    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDGR,IDC_EDIT_BANK),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDGRProcControl);
  window_graphicdgr->oldEditProcAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDGR,IDC_EDIT_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDGRProcControl);

  /** Ajoute les images sur les boutons **/
  SendMessage(GetDlgItem(hwndGraphicDGR,IDC_BUTTON_LEFT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicDGR,IDC_BUTTON_RIGHT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicDGR,IDC_BUTTON_UP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicDGR,IDC_BUTTON_DOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicDGR,IDC_BUTTON_PAGEUP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicDGR,IDC_BUTTON_PAGEDOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Adresse de la Zone de Data **/
  window_graphicdgr->graphicdgr_bank = 0x00;
  window_graphicdgr->graphicdgr_address = 0x0400; 
  sprintf(buffer,"%02X",window_graphicdgr->graphicdgr_bank);
  SetDlgItemText(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",window_graphicdgr->graphicdgr_address);
  SetDlgItemText(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_ADDRESS,buffer);

  /* Refresh Mode : Automatic */
  UpdateGraphicDGRRefreshMode(window_graphicdgr,GRAPHICDGR_REFRESH_AUTOMATIC);

  /* Zoom Level : 1 */
  window_graphicdgr->zoom_level = GRAPHICDGR_ZOOM_X1;
  SendDlgItemMessage(window_graphicdgr->hwnd_graphicdgr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_graphicdgr->hwnd_graphicdgr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);

  /* Dimension en Pixel de la Bitmap GraphicDGR Data */
  window_graphicdgr->bitmap_graphicdgr_width = 560;
  window_graphicdgr->bitmap_graphicdgr_height = 192;
  
  /** Bitmap affichant la zone de slection **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicdgr->bitmap_graphicdgr_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicdgr->bitmap_graphicdgr_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicdgr->hBitmap_graphicdgr = CreateDIBSection(window_graphicdgr->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicdgr->bitmap_data_graphicdgr),NULL,0);
  for(i=0; i<window_graphicdgr->bitmap_graphicdgr_width*window_graphicdgr->bitmap_graphicdgr_height; i++)
    window_graphicdgr->bitmap_data_graphicdgr[i] = BITMAP_BACKGROUND_COLOR;

  /* Retaille la fentre */
  ResizeGraphicDGRWindow(window_graphicdgr);

  /** Construction de la Page Texte + Rafraichissement de la fentre **/
  RefreshGraphicDGRWindowData(window_graphicdgr,1);
    
  /* OK */
  return(0);
}


/*************************************************************************************************/
/*  subEditGraphicDGRProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*************************************************************************************************/
LRESULT CALLBACK subEditGraphicDGRProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_graphicdgr *window_graphicdgr;
  my_Memory(MEMORY_GET_WINDOW_GRAPHICDGR,&window_graphicdgr,NULL);

  /* Rien  faire */
  if(window_graphicdgr == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_BANK) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_ADDRESS) == hwnd_control)
    index = 2;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement du Return dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_graphicdgr->hwnd_graphicdgr);

      /* On force la MAJ de la fentre */
      RefreshGraphicDGRWindowData(window_graphicdgr,1);
    }
  else if(index == 1)
    return(CallWindowProc(window_graphicdgr->oldEditProcBank,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_graphicdgr->oldEditProcAddress,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
} 


/**************************************************************************************/
/*  UpdateGraphicDGRClientAllArea() :  Affichage des Bitmap de la fentre GraphicDGR. */
/**************************************************************************************/
void UpdateGraphicDGRClientAllArea(HDC hDC, struct window_graphicdgr *window_graphicdgr)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap **/
  UpdateGraphicDGRClientBitmapArea(hDC,window_graphicdgr);
}


/*****************************************************************************/
/*  UpdateGraphicDGRClientBitmapArea() :  Affichage de la Bitmap GraphicDGR. */
/*****************************************************************************/
static void UpdateGraphicDGRClientBitmapArea(HDC hDC, struct window_graphicdgr *window_graphicdgr)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec la zone mmoire et la zone slectionne **/
  bitmap_old = SelectObject(window_graphicdgr->hCompatibleDC,window_graphicdgr->hBitmap_graphicdgr);
  StretchBlt(hDC,window_graphicdgr->bitmap_graphicdgr_x,window_graphicdgr->bitmap_graphicdgr_y,
             window_graphicdgr->bitmap_graphicdgr_width*window_graphicdgr->zoom_level,
             window_graphicdgr->bitmap_graphicdgr_height*2*window_graphicdgr->zoom_level,    /* x2 */
             window_graphicdgr->hCompatibleDC,0,0,window_graphicdgr->bitmap_graphicdgr_width,window_graphicdgr->bitmap_graphicdgr_height,SRCCOPY);
  SelectObject(window_graphicdgr->hCompatibleDC,bitmap_old);
}


/*******************************************************************************/
/*  UpdateGraphicDGRAddress() :  Dplace l'adresse pour la fentre GraphicDGR. */
/*******************************************************************************/
static void UpdateGraphicDGRAddress(struct window_graphicdgr *window_graphicdgr, int delta)
{
  int current_address, new_address, graphicdgr_bank, graphicdgr_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphicdgr->graphicdgr_bank << 16) | window_graphicdgr->graphicdgr_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  graphicdgr_bank = (new_address >> 16) & 0xFF;
  graphicdgr_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphicdgr_bank);
  SetDlgItemText(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",graphicdgr_address);
  SetDlgItemText(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_ADDRESS,buffer);

  /* MAJ de la fentre */
  RefreshGraphicDGRWindowData(window_graphicdgr,1);
}


/*******************************************************************************************************/
/*  UpdateGraphicDGRContextInformation() :  MAJ des informations de contexte de la fentre GraphicDGR. */
/*******************************************************************************************************/
static void UpdateGraphicDGRContextInformation(struct window_graphicdgr *window_graphicdgr, int mouse_x, int mouse_y)
{
  int pixmap_x, pixmap_y, grid_x, grid_y, grix_pixel_y;
  int graphicdgr_data_address;
  unsigned char graphicdgr_data;
  int tab_text80_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };

  /* Rien  faire */
  if(window_graphicdgr == NULL)
    return;

  /* Est t'on dans l'image ? */
  if(mouse_x < window_graphicdgr->bitmap_graphicdgr_x || mouse_x >= (window_graphicdgr->bitmap_graphicdgr_x + window_graphicdgr->bitmap_graphicdgr_width*window_graphicdgr->zoom_level) ||
     mouse_y < window_graphicdgr->bitmap_graphicdgr_y || mouse_y >= (window_graphicdgr->bitmap_graphicdgr_y + window_graphicdgr->bitmap_graphicdgr_height*window_graphicdgr->zoom_level*2))
    {
      SetWindowText(GetDlgItem(window_graphicdgr->hwnd_graphicdgr,IDC_STATIC_CONTEXT),"");
      return;
    }

  /* Coordonnes dans la Pixmap */
  pixmap_x = (mouse_x - window_graphicdgr->bitmap_graphicdgr_x)/window_graphicdgr->zoom_level;
  pixmap_y = (mouse_y - window_graphicdgr->bitmap_graphicdgr_y)/window_graphicdgr->zoom_level;

  /* Coordonnes dans la grille 80x24 */
  grid_x = pixmap_x/7;
  grid_y = pixmap_y/(8*2);
  grix_pixel_y = pixmap_y/8;   /* 0-47 */

  /* Valeur */
  graphicdgr_data = window_graphicdgr->graphicdgr_data[grid_x + grid_y*80];

  /* Adresse */
  graphicdgr_data_address = ((window_graphicdgr->graphicdgr_bank+(((grid_x%2)==0)?1:0)) << 16) + window_graphicdgr->graphicdgr_address;
  graphicdgr_data_address += (grid_x/2);
  graphicdgr_data_address += tab_text80_address[grid_y];

  /* Message */
  sprintf(window_graphicdgr->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdgr_data_address>>16,graphicdgr_data_address&0xFFFF,graphicdgr_data,grid_x,grix_pixel_y);
  SetWindowText(GetDlgItem(window_graphicdgr->hwnd_graphicdgr,IDC_STATIC_CONTEXT),window_graphicdgr->buffer_log);
}


/***********************************************************************************************/
/*  UpdateGraphicDGRRefreshMode() :  MAJ du mode de rafraichissement de la fentre GraphicDGR. */
/***********************************************************************************************/
static void UpdateGraphicDGRRefreshMode(struct window_graphicdgr *window_graphicdgr, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == GRAPHICDGR_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_graphicdgr->refresh_mode = GRAPHICDGR_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_graphicdgr->hwnd_graphicdgr,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicdgr->hwnd_graphicdgr,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_graphicdgr->hwnd_graphicdgr;
      current_notification->message = HM_GRAPHICDGR_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_GRAPHICDGR;

      /* Ajout de la notification */
      window_graphicdgr->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_graphicdgr->refresh_mode = GRAPHICDGR_REFRESH_MANUAL;
      SendDlgItemMessage(window_graphicdgr->hwnd_graphicdgr,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicdgr->hwnd_graphicdgr,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicdgr->current_notification,NULL);
      window_graphicdgr->current_notification = NULL;
    }
}


/***********************************************************************************/
/*  UpdateGraphicDGRZoomLevel() :  MAJ du nieau de Zomm de la fenetre GraphicDGR. */
/***********************************************************************************/
static void UpdateGraphicDGRZoomLevel(struct window_graphicdgr *window_graphicdgr, int zoom_level)
{
  if(zoom_level == GRAPHICDGR_ZOOM_X1)
    {
      /* Visuel */
      window_graphicdgr->zoom_level = GRAPHICDGR_ZOOM_X1;
      SendDlgItemMessage(window_graphicdgr->hwnd_graphicdgr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicdgr->hwnd_graphicdgr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICDGR_ZOOM_X2)
    {
      /* Visuel */
      window_graphicdgr->zoom_level = GRAPHICDGR_ZOOM_X2;
      SendDlgItemMessage(window_graphicdgr->hwnd_graphicdgr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicdgr->hwnd_graphicdgr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,1,0);
    }
        
  /* Retaille la fentre */
  ResizeGraphicDGRWindow(window_graphicdgr);  
  
  /* On force la MAJ de la fentre */  
  RefreshGraphicDGRWindowData(window_graphicdgr,1);    
}


/******************************************************************************/
/*  RefreshGraphicDGRWindowData() :  MAJ du contenu de la fentre GraphicDGR. */
/******************************************************************************/
static void RefreshGraphicDGRWindowData(struct window_graphicdgr *window_graphicdgr, int force)
{
  int length, is_modified;
  int graphicdgr_bank, graphicdgr_address, nb_column, nb_row;
  char buffer[1024];

  /* Rcupre la valeur du GraphicDGR Pointer */
  length = GetDlgItemText(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_BANK,buffer,1024);
  sscanf(buffer,"%X",&graphicdgr_bank);
  length = GetDlgItemText(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_ADDRESS,buffer,1024);
  sscanf(buffer,"%X",&graphicdgr_address);
  nb_column = GetDlgItemInt(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_COLUMN,NULL,FALSE);
  if(nb_column < 1)
    nb_column = 1;
  nb_row = GetDlgItemInt(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_ROW,NULL,FALSE);
  if(nb_row < 1)
    nb_row = 1;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphicdgr_bank);
  SetDlgItemText(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",graphicdgr_address);
  SetDlgItemText(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_ADDRESS,buffer);
  SetDlgItemInt(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_COLUMN,nb_column,FALSE);
  SetDlgItemInt(window_graphicdgr->hwnd_graphicdgr,IDC_EDIT_ROW,nb_row,FALSE);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsGraphicDGRDataModified(graphicdgr_bank,graphicdgr_address,nb_column,nb_row,window_graphicdgr);
  if(is_modified == 0 && force == 0)
    return;

  /** Cration de la Zone Graphique  partir des nouvelles donnes **/
  BuildGraphicDGRFromData(window_graphicdgr);

  /* Rafraichit la fentre */
  InvalidateRect(window_graphicdgr->hwnd_graphicdgr,NULL,TRUE);
  UpdateWindow(window_graphicdgr->hwnd_graphicdgr);
}


/************************************************************************************************************/
/*  IsGraphicDGRDataModified() :  Les donnes GraphicDGR ont t'elles t modifies depuis la dernire fois. */
/************************************************************************************************************/
static int IsGraphicDGRDataModified(int graphicdgr_bank, int graphicdgr_address, int nb_column, int nb_row, struct window_graphicdgr *window_graphicdgr)
{
  int x, y, graphicdgr_data_address;
  unsigned char graphicdgr_raw_bank0_data[1024];
  unsigned char graphicdgr_raw_bank1_data[1024];
  unsigned char graphicdgr_data[80*24];
  int tab_text40_address[24] =
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /* Rcupre les donnes brutes : 2*1024 chars */
  graphicdgr_data_address = ((graphicdgr_bank+1) << 16) | graphicdgr_address;
  ReadMemoryBlock(graphicdgr_data_address,1024,graphicdgr_raw_bank1_data,current_mmu);
  graphicdgr_data_address = (graphicdgr_bank << 16) | graphicdgr_address;
  ReadMemoryBlock(graphicdgr_data_address,1024,graphicdgr_raw_bank0_data,current_mmu);

  /* R-organise les donnes : Linarisation (passage du block de 2*1024 char en 80*24) */
  for(y=0; y<24; y++)
    for(x=0; x<40; x++)
      {
        graphicdgr_data[2*x+0+y*80] = graphicdgr_raw_bank1_data[tab_text40_address[y]+x];
        graphicdgr_data[2*x+1+y*80] = graphicdgr_raw_bank0_data[tab_text40_address[y]+x];
      }

  /* Compare les donnes */
  if(window_graphicdgr->graphicdgr_data != NULL)
    if(graphicdgr_bank == window_graphicdgr->graphicdgr_bank &&
       graphicdgr_address == window_graphicdgr->graphicdgr_address &&
       memcmp(&graphicdgr_data[0],&window_graphicdgr->graphicdgr_data[0],80*24) == 0)
      return(0);

  /* Les donnes sont diffrentes => MAJ */
  window_graphicdgr->graphicdgr_bank = graphicdgr_bank;
  window_graphicdgr->graphicdgr_address = graphicdgr_address;
  memcpy(window_graphicdgr->graphicdgr_data,graphicdgr_data,80*24);

  /* Donnes diffrentes */
  return(1);
}


/***********************************************************************/
/*  ResizeGraphicDGRWindow() : Resize GraphicDGR window by given size. */
/***********************************************************************/
static void ResizeGraphicDGRWindow(struct window_graphicdgr *window_graphicdgr)
{
  HWND hwnd_group;
  RECT window_rect, client_rect, group_rect;
  int window_x, window_y;
  
  /** On va ajuster la fentre par rapport au Zoom demand **/  
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_graphicdgr->hwnd_graphicdgr,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_graphicdgr->hwnd_graphicdgr,&client_rect);
  window_graphicdgr->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select */
  hwnd_group = GetDlgItem(window_graphicdgr->hwnd_graphicdgr,IDC_GROUP_GRAPHICDGR);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_GRAPHICDGR */
  GetWindowRect(hwnd_group,&group_rect);
  window_graphicdgr->group_data_x = (group_rect.left - window_rect.left) - 3;
  window_graphicdgr->group_data_y = ((group_rect.top - window_rect.top) - window_graphicdgr->title_height) + 3;

  /* Position de la Zone BipMap dans la fenetre */
  window_graphicdgr->bitmap_graphicdgr_x = window_graphicdgr->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_graphicdgr->bitmap_graphicdgr_y = window_graphicdgr->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_graphicdgr->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_graphicdgr->bitmap_graphicdgr_width*window_graphicdgr->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_graphicdgr->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicdgr->bitmap_graphicdgr_height*2*window_graphicdgr->zoom_level + WINDOW_GROUP_BORDER_BOTTOM_SIZE;   /* x2 */
  MoveWindow(hwnd_group,window_graphicdgr->group_data_x,window_graphicdgr->group_data_y,window_graphicdgr->group_data_width,window_graphicdgr->group_data_height,TRUE);

  /* Largeur de la fentre */
  window_graphicdgr->window_width = window_graphicdgr->group_data_x + window_graphicdgr->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* On doit garantir une largeur minimale de la fentre */
  if(window_graphicdgr->window_width < GRAPHICDGR_DIALOGBOX_WIDTH)
    window_graphicdgr->window_width = GRAPHICDGR_DIALOGBOX_WIDTH;

  /* Hauteur de la fentre */
  window_graphicdgr->window_height = window_graphicdgr->title_height + window_graphicdgr->group_data_y + window_graphicdgr->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Resize window */
  MoveWindow(window_graphicdgr->hwnd_graphicdgr,window_x,window_y,window_graphicdgr->window_width,window_graphicdgr->window_height,TRUE);
}


/************************************************************************************************************/
/*  BuildGraphicDGRFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/************************************************************************************************************/
static void BuildGraphicDGRFromData(struct window_graphicdgr *window_graphicdgr)
{
  unsigned char one_byte;
  int x, y, offset_pixmap_screen, pixel_x, pixel_y;
  char buffer_title[256];
  DWORD color_screen;
  DWORD tab_rgb_color[16] = 
    {
      /* 16 couleurs pour le mode GR */
      MAKECOLOR(0,0,0), MAKECOLOR(221,0,51), MAKECOLOR(0,0,153), MAKECOLOR(221,0,221),
      MAKECOLOR(0,119,0), MAKECOLOR(85,85,85), MAKECOLOR(34,34,255), MAKECOLOR(102,170,255),
      MAKECOLOR(136,85,34), MAKECOLOR(255,102,0), MAKECOLOR(170,170,170), MAKECOLOR(255,153,136),
      MAKECOLOR(0,221,0), MAKECOLOR(255,255,0), MAKECOLOR(0,255,153), MAKECOLOR(255,255,255)
    };
  DWORD tab_rgb_color_aux[16] = 
    {
      /* 16 couleurs pour le mode DGR */
      MAKECOLOR(0,0,0),     MAKECOLOR(0,0,153),     MAKECOLOR(0,119,0),   MAKECOLOR(34,34,255),
      MAKECOLOR(136,85,34), MAKECOLOR(170,170,170), MAKECOLOR(0,221,0),   MAKECOLOR(0,255,153), 
      MAKECOLOR(221,0,51),  MAKECOLOR(221,0,221),   MAKECOLOR(85,85,85),  MAKECOLOR(102,170,255), 
      MAKECOLOR(255,102,0), MAKECOLOR(255,153,136), MAKECOLOR(255,255,0), MAKECOLOR(255,255,255)
    };
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rien  faire */
  if(window_graphicdgr->bitmap_data_graphicdgr == NULL)
    return;    

  /** Cration de la bitmap 560*192 correspondant au 80*48 du DGR **/
  for(y=0; y<24; y++)
    {
      for(x=0; x<80; x++)
        {
          /* 1 Byte = 2 Pixels  */
          one_byte = window_graphicdgr->graphicdgr_data[x + y*80];

          /* Adresse du Bloc */
          offset_pixmap_screen = x*7 + y*560*8;

          /* Bloc Haut de 7*4 */
          color_screen = (((x%2) == 0) ? tab_rgb_color_aux[one_byte & 0x0F] : tab_rgb_color[one_byte & 0x0F]);      /* Couleur du Pixel final */
          for(pixel_y=0; pixel_y<4; pixel_y++)
            for(pixel_x=0; pixel_x<7; pixel_x++)
              window_graphicdgr->bitmap_data_graphicdgr[offset_pixmap_screen + pixel_x + pixel_y*560] = color_screen;

          /* Bloc Bas de 7*4 */
          color_screen = (((x%2) == 0) ? tab_rgb_color_aux[one_byte >> 4] : tab_rgb_color[one_byte >> 4]);      /* Couleur du Pixel final */
          for(pixel_y=4; pixel_y<8; pixel_y++)
            for(pixel_x=0; pixel_x<7; pixel_x++)
              window_graphicdgr->bitmap_data_graphicdgr[offset_pixmap_screen + pixel_x + pixel_y*560] = color_screen;
        }
    }

  /* Titre de la fentre */
  sprintf(buffer_title,"Graphic DGR : %02X/%04X",window_graphicdgr->graphicdgr_bank,window_graphicdgr->graphicdgr_address);
  SetWindowText(window_graphicdgr->hwnd_graphicdgr,buffer_title);
}


/************************************************************************/
/*  DeleteGraphicDGRChildWindows() :  Destruction des fentres enfants. */
/************************************************************************/
static void DeleteGraphicDGRChildWindows(struct window_graphicdgr *window_graphicdgr)
{
  DeleteObject(window_graphicdgr->hBitmap_graphicdgr);
  ReleaseDC(window_graphicdgr->hwnd_graphicdgr,window_graphicdgr->hDC);
  DeleteDC(window_graphicdgr->hCompatibleDC);
}


/**************************************************************************/
/*  mem_free_window_graphicdgr() :  Libration mmoire pour la structure. */
/**************************************************************************/
void mem_free_window_graphicdgr(struct window_graphicdgr *window_graphicdgr)
{
  if(window_graphicdgr)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicdgr->current_notification,NULL);
      window_graphicdgr->current_notification = NULL;
      
      free(window_graphicdgr);
    }
}

/*******************************************************************************/
