/************************************************************************************/
/*                                                                                  */
/*  Window_GraphicDGRText80.c :  Module de gestion de la fentre GraphicDGRText80.  */
/*                                                                                  */
/************************************************************************************/
/*   Auteur : Olivier ZARDINI     *     Brutal Deluxe     *     Date :  Aout 2018   */
/************************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_GraphicDGRText80.h"

static DWORD BuildGraphicDGRText80WindowThread(void *);
LRESULT CALLBACK ProcHandlerGraphicDGRText80Window(HWND,UINT,WPARAM,LPARAM);
static int CreateGraphicDGRText80ChildWindows(HWND,struct window_graphicdgrtext80 *);
LRESULT CALLBACK subEditGraphicDGRText80ProcControl(HWND,UINT,WPARAM,LPARAM);
void UpdateGraphicDGRText80ClientAllArea(HDC,struct window_graphicdgrtext80 *);
static void UpdateGraphicDGRText80ClientBitmapArea(HDC,struct window_graphicdgrtext80 *);
static void UpdateGraphicDGRText80ContextInformation(struct window_graphicdgrtext80 *,int,int);
static void UpdateGraphicDGRText80AddressGraphic(struct window_graphicdgrtext80 *,int);
static void UpdateGraphicDGRText80AddressText(struct window_graphicdgrtext80 *,int);
static void UpdateGraphicDGRText80RefreshMode(struct window_graphicdgrtext80 *,int);
static void UpdateGraphicDGRText80ZoomLevel(struct window_graphicdgrtext80 *,int);
static void RefreshGraphicDGRText80WindowData(struct window_graphicdgrtext80 *,int);
static int IsGraphicDGRText80DataModified(int,int,int,int,struct window_graphicdgrtext80 *);
static void ResizeGraphicDGRText80Window(struct window_graphicdgrtext80 *);
static void BuildGraphicDGRText80FromData(struct window_graphicdgrtext80 *);
static void DeleteGraphicDGRText80ChildWindows(struct window_graphicdgrtext80 *);


/**********************************************************************************************/
/*  BuildGraphicDGRText80Window() :  Construction / Affichage de la fentre GraphicDGRText80. */
/**********************************************************************************************/
struct window_graphicdgrtext80 *BuildGraphicDGRText80Window(HINSTANCE hInstance, int menu_id)
{
  struct window_graphicdgrtext80 *window_graphicdgrtext80;

  /** Allocation de la structure window_graphicdgrtext80 **/
  window_graphicdgrtext80 = (struct window_graphicdgrtext80 *) calloc(1,sizeof(struct window_graphicdgrtext80));
  if(window_graphicdgrtext80 == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_GRAPHICDGRTEXT80,window_graphicdgrtext80,NULL);

  /* Lance un thread pour grer la fentre GraphicDGRText80 */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildGraphicDGRText80WindowThread,window_graphicdgrtext80,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_graphicdgrtext80);
}


/********************************************************************************************************/
/*  BuildGraphicDGRText80WindowThread() :  Cration de la fentre GraphicDGRText80 via un Thread ddi. */
/********************************************************************************************************/
static DWORD BuildGraphicDGRText80WindowThread(void *window_graphicdgrtext80_param)
{
  HWND hwndGraphicDGRText80;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_graphicdgrtext80 *window_graphicdgrtext80;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_graphicdgrtext80 = (struct window_graphicdgrtext80 *) window_graphicdgrtext80_param;
  window_graphicdgrtext80->thread_id = GetCurrentThreadId();         /* Le ThreaID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Text Page 80 Column **/
  hwndGraphicDGRText80 = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_GRAPHICDGRTEXT80),window_main->hwnd_main,(DLGPROC)ProcHandlerGraphicDGRText80Window);
  if(hwndGraphicDGRText80 == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndGraphicDGRText80,SW_SHOWDEFAULT);
  UpdateWindow(hwndGraphicDGRText80);

  /** Message Loop pour toutes les fentres de ce Thread (fentre GraphicDGRText80 + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/*********************************************************************************/
/*  ProcHandlerGraphicDGRText80Window() :  Gestion des vnements de la fentre. */
/*********************************************************************************/
LRESULT CALLBACK ProcHandlerGraphicDGRText80Window(HWND hwndGraphicDGRText80, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;
  LRESULT result;
  WORD param_wd;
  int mouse_x, mouse_y;
  struct window_graphicdgrtext80 *window_graphicdgrtext80;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_GRAPHICDGRTEXT80,&window_graphicdgrtext80,NULL);
  if(window_graphicdgrtext80 == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateGraphicDGRText80ChildWindows(hwndGraphicDGRText80,window_graphicdgrtext80);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        UpdateGraphicDGRText80ContextInformation(window_graphicdgrtext80,mouse_x,mouse_y);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_graphicdgrtext80->hBitmap_graphicdgrtext80 != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndGraphicDGRText80,&ps);

            /** MAJ Bitmap **/
            UpdateGraphicDGRText80ClientAllArea(hDC,window_graphicdgrtext80);

            /* End Paint */
            EndPaint(hwndGraphicDGRText80,&ps);
          }
        break;
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdgrtext80->refresh_mode != GRAPHICDGRTEXT80_REFRESH_AUTOMATIC)
                UpdateGraphicDGRText80RefreshMode(window_graphicdgrtext80,GRAPHICDGRTEXT80_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdgrtext80->refresh_mode != GRAPHICDGRTEXT80_REFRESH_MANUAL)
                UpdateGraphicDGRText80RefreshMode(window_graphicdgrtext80,GRAPHICDGRTEXT80_REFRESH_MANUAL);
              break;

            /** Bouton Radio : On change le Zoom de la fentre **/
            case IDC_RADIO_ZOOM_X1 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdgrtext80->zoom_level != GRAPHICDGRTEXT80_ZOOM_X1)
                UpdateGraphicDGRText80ZoomLevel(window_graphicdgrtext80,GRAPHICDGRTEXT80_ZOOM_X1);
              break;

            case IDC_RADIO_ZOOM_X2 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdgrtext80->zoom_level != GRAPHICDGRTEXT80_ZOOM_X2)
                UpdateGraphicDGRText80ZoomLevel(window_graphicdgrtext80,GRAPHICDGRTEXT80_ZOOM_X2);
              break;

            /** Bouton de dplacement de l'adresse Graphic **/
            case IDC_BUTTON_LEFT_GRAPHIC :
              UpdateGraphicDGRText80AddressGraphic(window_graphicdgrtext80,-1);
              break;

            case IDC_BUTTON_RIGHT_GRAPHIC :
              UpdateGraphicDGRText80AddressGraphic(window_graphicdgrtext80,1);
              break;

            case IDC_BUTTON_UP_GRAPHIC :
              UpdateGraphicDGRText80AddressGraphic(window_graphicdgrtext80,-40);
              break;

            case IDC_BUTTON_DOWN_GRAPHIC :
              UpdateGraphicDGRText80AddressGraphic(window_graphicdgrtext80,40);
              break;

            case IDC_BUTTON_PAGEUP_GRAPHIC :
              UpdateGraphicDGRText80AddressGraphic(window_graphicdgrtext80,-1024);
              break;

            case IDC_BUTTON_PAGEDOWN_GRAPHIC :
              UpdateGraphicDGRText80AddressGraphic(window_graphicdgrtext80,1024);
              break;

            /** Bouton de dplacement de l'adresse Text **/
            case IDC_BUTTON_LEFT_TEXT :
              UpdateGraphicDGRText80AddressText(window_graphicdgrtext80,-1);
              break;

            case IDC_BUTTON_RIGHT_TEXT :
              UpdateGraphicDGRText80AddressText(window_graphicdgrtext80,1);
              break;

            case IDC_BUTTON_UP_TEXT :
              UpdateGraphicDGRText80AddressText(window_graphicdgrtext80,-40);
              break;

            case IDC_BUTTON_DOWN_TEXT :
              UpdateGraphicDGRText80AddressText(window_graphicdgrtext80,40);
              break;

            case IDC_BUTTON_PAGEUP_TEXT :
              UpdateGraphicDGRText80AddressText(window_graphicdgrtext80,-1024);
              break;

            case IDC_BUTTON_PAGEDOWN_TEXT :
              UpdateGraphicDGRText80AddressText(window_graphicdgrtext80,1024);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshGraphicDGRText80WindowData(window_graphicdgrtext80,1);
              break;
              
            /** Slection dans la ComboBox **/
            case IDC_COMBOBOX_GRAPHICDGRTEXT80_COLOR_TEXT :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphicdgrtext80->hwnd_combo_color_text,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphicdgrtext80->color_text)
                      {
                        window_graphicdgrtext80->color_text = (int) result;
                        RefreshGraphicDGRText80WindowData(window_graphicdgrtext80,1);
                      }
                }
              break;              
             
            case IDC_COMBOBOX_GRAPHICDGRTEXT80_COLOR_BACKGROUND :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphicdgrtext80->hwnd_combo_color_background,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphicdgrtext80->color_background)
                      {
                        window_graphicdgrtext80->color_background = (int) result;
                        RefreshGraphicDGRText80WindowData(window_graphicdgrtext80,1);
                      }
                }
              break; 
             
            case IDC_COMBOBOX_GRAPHICDGRTEXT80_DISPLAY_LANGUAGE :
              param_wd = HIWORD(wParam);
              if(param_wd == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphicdgrtext80->hwnd_combo_display_language,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphicdgrtext80->display_language)
                      {
                        window_graphicdgrtext80->display_language = (int) result;
                        RefreshGraphicDGRText80WindowData(window_graphicdgrtext80,1);
                      }
                }
              break;                           
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_GRAPHICDGRTEXT80_REFRESHWINDOW :
        RefreshGraphicDGRText80WindowData(window_graphicdgrtext80,0);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre GraphicDGRText80 */
        DestroyWindow(hwndGraphicDGRText80);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteGraphicDGRText80ChildWindows(window_graphicdgrtext80);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_GRAPHICDGRTEXT80,window_graphicdgrtext80,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************************/
/*  CreateGraphicDGRText80ChildWindows() : Crer les fentres enfants et remplir la structure window_graphicdgrtext80. */
/***********************************************************************************************************************/
static int CreateGraphicDGRText80ChildWindows(HWND hwndGraphicDGRText80, struct window_graphicdgrtext80 *window_graphicdgrtext80)
{
  BITMAPINFO bitmap_info;
  int i;
  char buffer[BUFFER_SIZE];
  char *color_name[16] = {"Black","Deep Red","Dark Blue","Purple","Dark Green","Dark Gray","Medium Blue","Light Blue","Brown","Orange","Light Gray","Pink","Light Green","Yellow","Aquamarine","White"};
  char *display_language[9] = {"U.S.A.","U.K.","French","Danish","Spanish","Italian","German","Swedish","MouseText"};
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_graphicdgrtext80->hwnd_graphicdgrtext80 = hwndGraphicDGRText80;

  /** Device Context **/
  window_graphicdgrtext80->hDC = GetDC(window_graphicdgrtext80->hwnd_graphicdgrtext80);
  window_graphicdgrtext80->hCompatibleDC = CreateCompatibleDC(window_graphicdgrtext80->hDC);

  /** Surchage des Control Edit pour capter les vnements **/
  window_graphicdgrtext80->oldEditProcBankGraphic    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDGRText80,IDC_EDIT_BANK_GRAPHIC),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDGRText80ProcControl);
  window_graphicdgrtext80->oldEditProcAddressGraphic = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDGRText80,IDC_EDIT_ADDRESS_GRAPHIC),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDGRText80ProcControl);
  window_graphicdgrtext80->oldEditProcBankText       = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDGRText80,IDC_EDIT_BANK_TEXT),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDGRText80ProcControl);
  window_graphicdgrtext80->oldEditProcAddressText    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDGRText80,IDC_EDIT_ADDRESS_TEXT),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDGRText80ProcControl);

  /** Ajoute les images sur les boutons **/
  /* Graphic */
  SendMessage(GetDlgItem(hwndGraphicDGRText80,IDC_BUTTON_LEFT_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicDGRText80,IDC_BUTTON_RIGHT_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicDGRText80,IDC_BUTTON_UP_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicDGRText80,IDC_BUTTON_DOWN_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicDGRText80,IDC_BUTTON_PAGEUP_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicDGRText80,IDC_BUTTON_PAGEDOWN_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));
  /* Text */
  SendMessage(GetDlgItem(hwndGraphicDGRText80,IDC_BUTTON_LEFT_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicDGRText80,IDC_BUTTON_RIGHT_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicDGRText80,IDC_BUTTON_UP_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicDGRText80,IDC_BUTTON_DOWN_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicDGRText80,IDC_BUTTON_PAGEUP_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicDGRText80,IDC_BUTTON_PAGEDOWN_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Cration des ComboBox **/
  /* Text Color */
  window_graphicdgrtext80->hwnd_combo_color_text = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                 99,102,100,250,hwndGraphicDGRText80,(HMENU)IDC_COMBOBOX_GRAPHICDGRTEXT80_COLOR_TEXT,param->hInstance,0);
  for(i=0; i<16; i++)
    SendMessage(window_graphicdgrtext80->hwnd_combo_color_text,CB_ADDSTRING,0,(LPARAM)color_name[i]);

  /* Background Color */
  window_graphicdgrtext80->hwnd_combo_color_background = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                                       304,102,100,250,hwndGraphicDGRText80,(HMENU)IDC_COMBOBOX_GRAPHICDGRTEXT80_COLOR_BACKGROUND,param->hInstance,0);
  for(i=0; i<16; i++)
    SendMessage(window_graphicdgrtext80->hwnd_combo_color_background,CB_ADDSTRING,0,(LPARAM)color_name[i]);

  /* Display Language */
  window_graphicdgrtext80->hwnd_combo_display_language = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                                       304,134,100,220,hwndGraphicDGRText80,(HMENU)IDC_COMBOBOX_GRAPHICDGRTEXT80_DISPLAY_LANGUAGE,param->hInstance,0);
  for(i=0; i<9; i++)
    SendMessage(window_graphicdgrtext80->hwnd_combo_display_language,CB_ADDSTRING,0,(LPARAM)display_language[i]);

  /* 1res valeurs */
  window_graphicdgrtext80->color_text = COLOR_IIGS_WHITE;
  window_graphicdgrtext80->color_background = COLOR_IIGS_MEDIUM_BLUE;
  window_graphicdgrtext80->display_language = LANGUAGE_IIGS_USA;
  SendMessage(window_graphicdgrtext80->hwnd_combo_color_text,CB_SETCURSEL,window_graphicdgrtext80->color_text,0);
  SendMessage(window_graphicdgrtext80->hwnd_combo_color_background,CB_SETCURSEL,window_graphicdgrtext80->color_background,0);
  SendMessage(window_graphicdgrtext80->hwnd_combo_display_language,CB_SETCURSEL,window_graphicdgrtext80->display_language,0);

  /** Cration de la Police de caractre + Affectation de la police au Control Edit **/
  window_graphicdgrtext80->hFont = CreateFont(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");
  SendMessage(window_graphicdgrtext80->hwnd_combo_color_text,WM_SETFONT,(WPARAM)window_graphicdgrtext80->hFont,TRUE);
  SendMessage(window_graphicdgrtext80->hwnd_combo_color_background,WM_SETFONT,(WPARAM)window_graphicdgrtext80->hFont,TRUE);
  SendMessage(window_graphicdgrtext80->hwnd_combo_display_language,WM_SETFONT,(WPARAM)window_graphicdgrtext80->hFont,TRUE);

  /** Adresse de la Zone de Data **/
  /* Graphic */
  window_graphicdgrtext80->graphicdgr_bank = 0x00;
  window_graphicdgrtext80->graphicdgr_address = 0x0400; 
  sprintf(buffer,"%02X",window_graphicdgrtext80->graphicdgr_bank);
  SetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_BANK_GRAPHIC,buffer);
  sprintf(buffer,"%04X",window_graphicdgrtext80->graphicdgr_address);
  SetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_ADDRESS_GRAPHIC,buffer);
  /* Text */
  window_graphicdgrtext80->textpage80_bank = 0x00;
  window_graphicdgrtext80->textpage80_address = 0x0400; 
  sprintf(buffer,"%02X",window_graphicdgrtext80->textpage80_bank);
  SetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_BANK_TEXT,buffer);
  sprintf(buffer,"%04X",window_graphicdgrtext80->textpage80_address);
  SetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_ADDRESS_TEXT,buffer);

  /* Refresh Mode : Automatic */
  UpdateGraphicDGRText80RefreshMode(window_graphicdgrtext80,GRAPHICDGRTEXT80_REFRESH_AUTOMATIC);

  /* Zoom Level : 1 */
  window_graphicdgrtext80->zoom_level = GRAPHICDGRTEXT80_ZOOM_X1;
  SendDlgItemMessage(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);

  /* Dimension en Pixel de la Bitmap GraphicDGRText80 Data */
  window_graphicdgrtext80->bitmap_graphicdgrtext80_width = 560;
  window_graphicdgrtext80->bitmap_graphicdgrtext80_height = 192;
  
  /** Bitmap affichant la zone de slection **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicdgrtext80->bitmap_graphicdgrtext80_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicdgrtext80->bitmap_graphicdgrtext80_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicdgrtext80->hBitmap_graphicdgrtext80 = CreateDIBSection(window_graphicdgrtext80->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicdgrtext80->bitmap_data_graphicdgrtext80),NULL,0);
  for(i=0; i<window_graphicdgrtext80->bitmap_graphicdgrtext80_width*window_graphicdgrtext80->bitmap_graphicdgrtext80_height; i++)
    window_graphicdgrtext80->bitmap_data_graphicdgrtext80[i] = BITMAP_BACKGROUND_COLOR;

  /* Retaille la fentre */
  ResizeGraphicDGRText80Window(window_graphicdgrtext80);

  /** Construction de la Page Texte + Rafraichissement de la fentre **/
  RefreshGraphicDGRText80WindowData(window_graphicdgrtext80,1);
    
  /* OK */
  return(0);
}


/*******************************************************************************************************/
/*  subEditGraphicDGRText80ProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*******************************************************************************************************/
LRESULT CALLBACK subEditGraphicDGRText80ProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_graphicdgrtext80 *window_graphicdgrtext80;
  my_Memory(MEMORY_GET_WINDOW_GRAPHICDGRTEXT80,&window_graphicdgrtext80,NULL);

  /* Rien  faire */
  if(window_graphicdgrtext80 == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_BANK_GRAPHIC) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_ADDRESS_GRAPHIC) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_BANK_TEXT) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_ADDRESS_TEXT) == hwnd_control)
    index = 4;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement du Return dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_graphicdgrtext80->hwnd_graphicdgrtext80);

      /* On force la MAJ de la fentre */
      RefreshGraphicDGRText80WindowData(window_graphicdgrtext80,1);
    }
  else if(index == 1)
    return(CallWindowProc(window_graphicdgrtext80->oldEditProcBankGraphic,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_graphicdgrtext80->oldEditProcAddressGraphic,hwnd_control,msg,wParam,lParam)); 
  else if(index == 3)
    return(CallWindowProc(window_graphicdgrtext80->oldEditProcBankText,hwnd_control,msg,wParam,lParam)); 
  else if(index == 4)
    return(CallWindowProc(window_graphicdgrtext80->oldEditProcAddressText,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
} 


/**************************************************************************************************/
/*  UpdateGraphicDGRText80ClientAllArea() :  Affichage des Bitmap de la fentre GraphicDGRText80. */
/**************************************************************************************************/
void UpdateGraphicDGRText80ClientAllArea(HDC hDC, struct window_graphicdgrtext80 *window_graphicdgrtext80)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap **/
  UpdateGraphicDGRText80ClientBitmapArea(hDC,window_graphicdgrtext80);
}


/*****************************************************************************************/
/*  UpdateGraphicDGRText80ClientBitmapArea() :  Affichage de la Bitmap GraphicDGRText80. */
/*****************************************************************************************/
static void UpdateGraphicDGRText80ClientBitmapArea(HDC hDC, struct window_graphicdgrtext80 *window_graphicdgrtext80)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec la zone mmoire et la zone slectionne **/
  bitmap_old = SelectObject(window_graphicdgrtext80->hCompatibleDC,window_graphicdgrtext80->hBitmap_graphicdgrtext80);
  StretchBlt(hDC,window_graphicdgrtext80->bitmap_graphicdgrtext80_x,window_graphicdgrtext80->bitmap_graphicdgrtext80_y,
             window_graphicdgrtext80->bitmap_graphicdgrtext80_width*window_graphicdgrtext80->zoom_level,
             window_graphicdgrtext80->bitmap_graphicdgrtext80_height*window_graphicdgrtext80->zoom_level*2,  /* Double la hauteur */
             window_graphicdgrtext80->hCompatibleDC,0,0,window_graphicdgrtext80->bitmap_graphicdgrtext80_width,window_graphicdgrtext80->bitmap_graphicdgrtext80_height,SRCCOPY);
  SelectObject(window_graphicdgrtext80->hCompatibleDC,bitmap_old);
}


/**********************************************************************************************************/
/*  UpdateGraphicDGRText80AddressGraphic() :  Dplace l'adresse Graphic pour la fentre GraphicDGRText80. */
/**********************************************************************************************************/
static void UpdateGraphicDGRText80AddressGraphic(struct window_graphicdgrtext80 *window_graphicdgrtext80, int delta)
{
  int current_address, new_address, graphicdgr_bank, graphicdgr_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphicdgrtext80->graphicdgr_bank << 16) | window_graphicdgrtext80->graphicdgr_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  graphicdgr_bank = (new_address >> 16) & 0xFF;
  graphicdgr_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphicdgr_bank);
  SetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_BANK_GRAPHIC,buffer);
  sprintf(buffer,"%04X",graphicdgr_address);
  SetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_ADDRESS_GRAPHIC,buffer);

  /* MAJ de la fentre */
  RefreshGraphicDGRText80WindowData(window_graphicdgrtext80,1);
}


/*****************************************************************************************************/
/*  UpdateGraphicDGRText80AddressText() :  Dplace l'adresse Texte pour la fentre GraphicDGRText80. */
/*****************************************************************************************************/
static void UpdateGraphicDGRText80AddressText(struct window_graphicdgrtext80 *window_graphicdgrtext80, int delta)
{
  int current_address, new_address, textpage80_bank, textpage80_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphicdgrtext80->textpage80_bank << 16) | window_graphicdgrtext80->textpage80_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  textpage80_bank = (new_address >> 16) & 0xFF;
  textpage80_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",textpage80_bank);
  SetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_BANK_TEXT,buffer);
  sprintf(buffer,"%04X",textpage80_address);
  SetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_ADDRESS_TEXT,buffer);

  /* MAJ de la fentre */
  RefreshGraphicDGRText80WindowData(window_graphicdgrtext80,1);
}


/*******************************************************************************************************************/
/*  UpdateGraphicDGRText80ContextInformation() :  MAJ des informations de contexte de la fentre GraphicDGRText80. */
/*******************************************************************************************************************/
static void UpdateGraphicDGRText80ContextInformation(struct window_graphicdgrtext80 *window_graphicdgrtext80, int mouse_x, int mouse_y)
{
  int pixmap_x, pixmap_y, grid_x, grid_y, grid_pixel_y;
  int graphicdgr_data_address;
  int textpage80_data_address;
  unsigned char graphicdgr_data;
  unsigned char textpage80_data;
  int tab_dgr_address[24] =
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page GR */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };
  int tab_text80_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };

  /* Rien  faire */
  if(window_graphicdgrtext80 == NULL)
    return;

  /* Est t'on dans l'image ? */
  if(mouse_x < window_graphicdgrtext80->bitmap_graphicdgrtext80_x || mouse_x >= (window_graphicdgrtext80->bitmap_graphicdgrtext80_x + window_graphicdgrtext80->bitmap_graphicdgrtext80_width*window_graphicdgrtext80->zoom_level) ||
     mouse_y < window_graphicdgrtext80->bitmap_graphicdgrtext80_y || mouse_y >= (window_graphicdgrtext80->bitmap_graphicdgrtext80_y + window_graphicdgrtext80->bitmap_graphicdgrtext80_height*window_graphicdgrtext80->zoom_level*2))
    {
      SetWindowText(GetDlgItem(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_STATIC_CONTEXT),"");
      return;
    }

  /* Coordonnes dans la Pixmap */
  pixmap_x = (mouse_x - window_graphicdgrtext80->bitmap_graphicdgrtext80_x)/window_graphicdgrtext80->zoom_level;
  pixmap_y = (mouse_y - window_graphicdgrtext80->bitmap_graphicdgrtext80_y)/window_graphicdgrtext80->zoom_level;

  /* Graphic */
  if(pixmap_y < 160*2)   /* 192 - 8*4 */
    {
      /* Coordonnes dans la grille 80x24 */
      grid_x = pixmap_x/7;
      grid_y = pixmap_y/(8*2);
      grid_pixel_y = pixmap_y/8;   /* 0-47 */

      /* Valeur */
      graphicdgr_data = window_graphicdgrtext80->graphicdgr_data[grid_x + grid_y*80];

      /* Adresse */
      graphicdgr_data_address = ((window_graphicdgrtext80->graphicdgr_bank+(((grid_x%2)==0)?1:0)) << 16) + window_graphicdgrtext80->graphicdgr_address;
      graphicdgr_data_address += (grid_x/2);
      graphicdgr_data_address += tab_text80_address[grid_y];

      /* Message */
      sprintf(window_graphicdgrtext80->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdgr_data_address>>16,graphicdgr_data_address&0xFFFF,graphicdgr_data,grid_x,grid_pixel_y);
      SetWindowText(GetDlgItem(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_STATIC_CONTEXT),window_graphicdgrtext80->buffer_log);
    }
  else   /* Text */
    {
      /* Coordonnes dans la grille 80x24 */
      grid_x = pixmap_x/7;
      grid_y = pixmap_y/(8*2);

      /* Valeur */
      textpage80_data = window_graphicdgrtext80->textpage80_data[grid_x + grid_y*80];

      /* Adresse */
      textpage80_data_address = ((window_graphicdgrtext80->textpage80_bank+(((grid_x%2)==0)?1:0)) << 16) + window_graphicdgrtext80->textpage80_address;
      textpage80_data_address += (grid_x/2);
      textpage80_data_address += tab_text80_address[grid_y];

      /* Message */
      sprintf(window_graphicdgrtext80->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",textpage80_data_address>>16,textpage80_data_address&0xFFFF,textpage80_data,grid_x,grid_y);
      SetWindowText(GetDlgItem(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_STATIC_CONTEXT),window_graphicdgrtext80->buffer_log);
    }
}


/************************************************************************************************************/
/*  UpdateGraphicDGRText80RefreshMode() :  MAJ du mode de rafraichissement de la fenetre GraphicDGRText80. */
/************************************************************************************************************/
static void UpdateGraphicDGRText80RefreshMode(struct window_graphicdgrtext80 *window_graphicdgrtext80, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == GRAPHICDGRTEXT80_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_graphicdgrtext80->refresh_mode = GRAPHICDGRTEXT80_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_graphicdgrtext80->hwnd_graphicdgrtext80;
      current_notification->message = HM_GRAPHICDGRTEXT80_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_GRAPHICDGRTEXT80;

      /* Ajout de la notification */
      window_graphicdgrtext80->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_graphicdgrtext80->refresh_mode = GRAPHICDGRTEXT80_REFRESH_MANUAL;
      SendDlgItemMessage(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicdgrtext80->current_notification,NULL);
      window_graphicdgrtext80->current_notification = NULL;
    }
}


/***********************************************************************************************/
/*  UpdateGraphicDGRText80ZoomLevel() :  MAJ du niveau de Zoom de la fentre GraphicDGRText80. */
/***********************************************************************************************/
static void UpdateGraphicDGRText80ZoomLevel(struct window_graphicdgrtext80 *window_graphicdgrtext80, int zoom_level)
{
  if(zoom_level == GRAPHICDGRTEXT80_ZOOM_X1)
    {
      /* Visuel */
      window_graphicdgrtext80->zoom_level = GRAPHICDGRTEXT80_ZOOM_X1;
      SendDlgItemMessage(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICDGRTEXT80_ZOOM_X2)
    {
      /* Visuel */
      window_graphicdgrtext80->zoom_level = GRAPHICDGRTEXT80_ZOOM_X2;
      SendDlgItemMessage(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_RADIO_ZOOM_X2,BM_SETCHECK,1,0);
    }
    
  /* Retaille la fentre */
  ResizeGraphicDGRText80Window(window_graphicdgrtext80);  
  
  /* On force la MAJ de la fentre */  
  RefreshGraphicDGRText80WindowData(window_graphicdgrtext80,1);    
}


/******************************************************************************************/
/*  RefreshGraphicDGRText80WindowData() :  MAJ du contenu de la fentre GraphicDGRText80. */
/******************************************************************************************/
static void RefreshGraphicDGRText80WindowData(struct window_graphicdgrtext80 *window_graphicdgrtext80, int force)
{
  int length, is_modified;
  int graphicdgr_bank, graphicdgr_address, textpage80_bank, textpage80_address;
  char buffer[1024];

  /** Rcupre la valeur du GraphicDGRText80 Pointer **/
  /* Graphic */
  length = GetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_BANK_GRAPHIC,buffer,1024);
  sscanf(buffer,"%X",&graphicdgr_bank);
  length = GetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_ADDRESS_GRAPHIC,buffer,1024);
  sscanf(buffer,"%X",&graphicdgr_address);
  /* Text */
  length = GetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_BANK_TEXT,buffer,1024);
  sscanf(buffer,"%X",&textpage80_bank);
  length = GetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_ADDRESS_TEXT,buffer,1024);
  sscanf(buffer,"%X",&textpage80_address);

  /** Replace les valeurs dans les Control pour montrer ce qui a t compris **/
  /* Graphic */
  sprintf(buffer,"%02X",graphicdgr_bank);
  SetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_BANK_GRAPHIC,buffer);
  sprintf(buffer,"%04X",graphicdgr_address);
  SetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_ADDRESS_GRAPHIC,buffer);
  /* Text */
  sprintf(buffer,"%02X",textpage80_bank);
  SetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_BANK_TEXT,buffer);
  sprintf(buffer,"%04X",textpage80_address);
  SetDlgItemText(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_EDIT_ADDRESS_TEXT,buffer);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsGraphicDGRText80DataModified(graphicdgr_bank,graphicdgr_address,textpage80_bank,textpage80_address,window_graphicdgrtext80);
  if(is_modified == 0 && force == 0)
    return;

  /** Cration de la Zone Text Graphique  partir des nouvelles donnes **/
  BuildGraphicDGRText80FromData(window_graphicdgrtext80);

  /* Rafraichit la fentre */
  InvalidateRect(window_graphicdgrtext80->hwnd_graphicdgrtext80,NULL,TRUE);
  UpdateWindow(window_graphicdgrtext80->hwnd_graphicdgrtext80);
}


/************************************************************************************************************************/
/*  IsGraphicDGRText80DataModified() :  Les donnes GraphicDGRText80 ont t'elles t modifies depuis la dernire fois. */
/************************************************************************************************************************/
static int IsGraphicDGRText80DataModified(int graphicdgr_bank, int graphicdgr_address, int textpage80_bank, int textpage80_address, struct window_graphicdgrtext80 *window_graphicdgrtext80)
{
  int x, y, graphicdgr_data_address, textpage80_data_address;
  unsigned char graphicdgr_raw_data_main[1024];
  unsigned char graphicdgr_raw_data_aux[1024];
  unsigned char textpage80_raw_data_main[1024];
  unsigned char textpage80_raw_data_aux[1024];
  unsigned char graphicdgr_data[80*24];
  unsigned char textpage80_data[80*24];
  int tab_dgr_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page DGR */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };
  int tab_text80_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /** Graphic : Rcupre les donnes brutes : 2x1024 bytes **/
  graphicdgr_data_address = (graphicdgr_bank << 16) | graphicdgr_address;
  ReadMemoryBlock(graphicdgr_data_address,1024,graphicdgr_raw_data_main,current_mmu);
  graphicdgr_data_address = ((graphicdgr_bank+1) << 16) | graphicdgr_address;
  ReadMemoryBlock(graphicdgr_data_address,1024,graphicdgr_raw_data_aux,current_mmu);

  /** Text : Rcupre les donnes brutes : 2x1024 chars **/
  textpage80_data_address = (textpage80_bank << 16) | textpage80_address;
  ReadMemoryBlock(textpage80_data_address,1024,textpage80_raw_data_main,current_mmu);
  textpage80_data_address = ((textpage80_bank+1) << 16) | textpage80_address;
  ReadMemoryBlock(textpage80_data_address,1024,textpage80_raw_data_aux,current_mmu);

  /** Graphic : R-organise les donnes : Linarisation (passage du block de 2*1024 byte en 80*24) **/
  for(y=0; y<24; y++)
    for(x=0; x<40; x++)
      {
        graphicdgr_data[(2*x+0)+y*80] = graphicdgr_raw_data_aux[tab_dgr_address[y]+x];
        graphicdgr_data[(2*x+1)+y*80] = graphicdgr_raw_data_main[tab_dgr_address[y]+x];
      }

  /** Text : R-organise les donnes : Linarisation Linarisation (passage du block de 2x1024 char en 80*24) **/
  for(y=0; y<24; y++)
    for(x=0; x<40; x++)
      {
        textpage80_data[(2*x+0)+y*80] = textpage80_raw_data_aux[tab_text80_address[y]+x];
        textpage80_data[(2*x+1)+y*80] = textpage80_raw_data_main[tab_text80_address[y]+x];
      }
      
  /** Compare les donnes **/
  if(window_graphicdgrtext80->graphicdgr_data != NULL && window_graphicdgrtext80->textpage80_data != NULL)
    if(graphicdgr_bank == window_graphicdgrtext80->graphicdgr_bank &&
       graphicdgr_address == window_graphicdgrtext80->graphicdgr_address &&
       memcmp(&graphicdgr_data[0],&window_graphicdgrtext80->graphicdgr_data[0],80*24) == 0 && 
       textpage80_bank == window_graphicdgrtext80->textpage80_bank &&
       textpage80_address == window_graphicdgrtext80->textpage80_address &&
       memcmp(&textpage80_data[0],&window_graphicdgrtext80->textpage80_data[0],80*24) == 0)
      return(0);

  /** Les donnes sont diffrentes => MAJ **/
  /* Graphic */
  window_graphicdgrtext80->graphicdgr_bank = graphicdgr_bank;
  window_graphicdgrtext80->graphicdgr_address = graphicdgr_address;
  memcpy(window_graphicdgrtext80->graphicdgr_data,graphicdgr_data,80*24);
  /* Text */
  window_graphicdgrtext80->textpage80_bank = textpage80_bank;
  window_graphicdgrtext80->textpage80_address = textpage80_address;
  memcpy(window_graphicdgrtext80->textpage80_data,textpage80_data,80*24);

  /* Donnes diffrentes */
  return(1);
}


/************************************************************************************/
/*  ResizeGraphicDGRText80Window() :  Resize GraphicDGRText80 window by given size. */
/************************************************************************************/
static void ResizeGraphicDGRText80Window(struct window_graphicdgrtext80 *window_graphicdgrtext80)
{
  HWND hwnd_group;
  RECT window_rect, client_rect, group_rect;
  int window_x, window_y;

  /** On va ajuster la fentre par rapport au Zoom demand **/  
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_graphicdgrtext80->hwnd_graphicdgrtext80,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_graphicdgrtext80->hwnd_graphicdgrtext80,&client_rect);
  window_graphicdgrtext80->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select */
  hwnd_group = GetDlgItem(window_graphicdgrtext80->hwnd_graphicdgrtext80,IDC_GROUP_GRAPHICDGRTEXT80);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_GRAPHICDGRTEXT80 */
  GetWindowRect(hwnd_group,&group_rect);
  window_graphicdgrtext80->group_data_x = (group_rect.left - window_rect.left) - 3;
  window_graphicdgrtext80->group_data_y = ((group_rect.top - window_rect.top) - window_graphicdgrtext80->title_height) + 3;

  /* Position de la Zone BitMap dans la fenetre */
  window_graphicdgrtext80->bitmap_graphicdgrtext80_x = window_graphicdgrtext80->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_graphicdgrtext80->bitmap_graphicdgrtext80_y = window_graphicdgrtext80->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_graphicdgrtext80->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_graphicdgrtext80->bitmap_graphicdgrtext80_width*window_graphicdgrtext80->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_graphicdgrtext80->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicdgrtext80->bitmap_graphicdgrtext80_height*window_graphicdgrtext80->zoom_level*2 + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_graphicdgrtext80->group_data_x,window_graphicdgrtext80->group_data_y,window_graphicdgrtext80->group_data_width,window_graphicdgrtext80->group_data_height,TRUE);

  /* Largeur de la fentre */
  window_graphicdgrtext80->window_width = window_graphicdgrtext80->group_data_x + window_graphicdgrtext80->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* On doit garantir une largeur minimale de la fentre */
  if(window_graphicdgrtext80->window_width < GRAPHICDGRTEXT80_DIALOGBOX_WIDTH)
    window_graphicdgrtext80->window_width = GRAPHICDGRTEXT80_DIALOGBOX_WIDTH;

  /* Hauteur de la fentre */
  window_graphicdgrtext80->window_height = window_graphicdgrtext80->title_height + window_graphicdgrtext80->group_data_y + window_graphicdgrtext80->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Resize window */
  MoveWindow(window_graphicdgrtext80->hwnd_graphicdgrtext80,window_x,window_y,window_graphicdgrtext80->window_width,window_graphicdgrtext80->window_height,TRUE);
}


/******************************************************************************************************************/
/*  BuildGraphicDGRText80FromData() :  Construire les donnes hexadcimal  afficher  partir des donnes brutes. */
/******************************************************************************************************************/
static void BuildGraphicDGRText80FromData(struct window_graphicdgrtext80 *window_graphicdgrtext80)
{
  int one_byte;
  int x, y, pixmap_char_x, pixmap_char_y, pixel_x, pixel_y, offset_pixmap_char, offset_pixmap_screen;
  char buffer_title[256];
  DWORD color_text, color_background, color_pixmap, color_screen;
  DWORD tab_rgb_color_graphic[16] = 
    {
      /* 16 couleurs pour le mode DGR */
      MAKECOLOR(0,0,0), MAKECOLOR(221,0,51), MAKECOLOR(0,0,153), MAKECOLOR(221,0,221),
      MAKECOLOR(0,119,0), MAKECOLOR(85,85,85), MAKECOLOR(34,34,255), MAKECOLOR(102,170,255),
      MAKECOLOR(136,85,34), MAKECOLOR(255,102,0), MAKECOLOR(170,170,170), MAKECOLOR(255,153,136),
      MAKECOLOR(0,221,0), MAKECOLOR(255,255,0), MAKECOLOR(0,255,153), MAKECOLOR(255,255,255)
    };
  DWORD tab_rgb_color_graphic_aux[16] = 
    {
      /* 16 couleurs pour le mode DGR */
      MAKECOLOR(0,0,0),     MAKECOLOR(0,0,153),     MAKECOLOR(0,119,0),   MAKECOLOR(34,34,255),
      MAKECOLOR(136,85,34), MAKECOLOR(170,170,170), MAKECOLOR(0,221,0),   MAKECOLOR(0,255,153), 
      MAKECOLOR(221,0,51),  MAKECOLOR(221,0,221),   MAKECOLOR(85,85,85),  MAKECOLOR(102,170,255), 
      MAKECOLOR(255,102,0), MAKECOLOR(255,153,136), MAKECOLOR(255,255,0), MAKECOLOR(255,255,255)
    };
  DWORD tab_rgb_color_text[16] = 
    {
      /* 16 couleurs de la Bordure / Fond Ecran / Text */
      MAKECOLOR(0,0,0), MAKECOLOR(221,0,51), MAKECOLOR(0,0,153), MAKECOLOR(221,34,221),
      MAKECOLOR(0,119,34), MAKECOLOR(85,85,85), MAKECOLOR(34,34,255), MAKECOLOR(102,170,255),
      MAKECOLOR(136,85,0), MAKECOLOR(255,102,0), MAKECOLOR(170,170,170), MAKECOLOR(255,153,136),
      MAKECOLOR(17,221,0), MAKECOLOR(255,255,0), MAKECOLOR(68,255,153), MAKECOLOR(255,255,255)
    };
  struct picture_true *picture_char;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rien  faire */
  if(window_graphicdgrtext80->bitmap_data_graphicdgrtext80 == NULL)
    return;

  /* Adresse de la picture */
  if(window_graphicdgrtext80->display_language == LANGUAGE_IIGS_USA)
    picture_char = param->picture_char_us;
  else if(window_graphicdgrtext80->display_language == LANGUAGE_IIGS_UK)
    picture_char = param->picture_char_uk;
  else if(window_graphicdgrtext80->display_language == LANGUAGE_IIGS_FRENCH)
    picture_char = param->picture_char_fr;
  else if(window_graphicdgrtext80->display_language == LANGUAGE_IIGS_DANISH)
    picture_char = param->picture_char_dk;
  else if(window_graphicdgrtext80->display_language == LANGUAGE_IIGS_SPANISH)
    picture_char = param->picture_char_se;
  else if(window_graphicdgrtext80->display_language == LANGUAGE_IIGS_ITALIAN)
    picture_char = param->picture_char_it;
  else if(window_graphicdgrtext80->display_language == LANGUAGE_IIGS_GERMAN)
    picture_char = param->picture_char_de;
  else if(window_graphicdgrtext80->display_language == LANGUAGE_IIGS_SWEDISH)
    picture_char = param->picture_char_se;
  else if(window_graphicdgrtext80->display_language == LANGUAGE_IIGS_MOUSETEXT)
    picture_char = param->picture_char_mousetext;
    
  /* Couleur du Texte */
  color_text = tab_rgb_color_text[window_graphicdgrtext80->color_text];

  /* Couleur du Fond */
  color_background = tab_rgb_color_text[window_graphicdgrtext80->color_background];

  /** Graphic : Cration de la bitmap 560*192 correspondant au 80*48 du DGR **/
  for(y=0; y<20; y++)
    {
      for(x=0; x<80; x++)
        {
          /* 1 Byte = 2 Pixels  */
          one_byte = window_graphicdgrtext80->graphicdgr_data[x + y*80];

          /* Adresse du Bloc */
          offset_pixmap_screen = x*7 + y*560*8;

          /* Bloc Haut de 7*4 */
          color_screen = (((x%2) == 0) ? tab_rgb_color_graphic_aux[one_byte & 0x0F] : tab_rgb_color_graphic[one_byte & 0x0F]);      /* Couleur du Pixel final */
          for(pixel_y=0; pixel_y<4; pixel_y++)
            for(pixel_x=0; pixel_x<7; pixel_x++)
              window_graphicdgrtext80->bitmap_data_graphicdgrtext80[offset_pixmap_screen + pixel_x + pixel_y*560] = color_screen;

          /* Bloc Bas de 7*4 */
          color_screen = (((x%2) == 0) ? tab_rgb_color_graphic_aux[one_byte >> 4] : tab_rgb_color_graphic[one_byte >> 4]);      /* Couleur du Pixel final */
          for(pixel_y=4; pixel_y<8; pixel_y++)
            for(pixel_x=0; pixel_x<7; pixel_x++)
              window_graphicdgrtext80->bitmap_data_graphicdgrtext80[offset_pixmap_screen + pixel_x + pixel_y*560] = color_screen;
        }
    }

  /** Text : Cration du bas de la bitmap 560*192 correspondant au 40*24 **/
  for(y=20; y<24; y++)
    {
      for(x=0; x<80; x++)
        {
          /* Caractre */
          one_byte = (int) window_graphicdgrtext80->textpage80_data[x + y*80];

          /* Offset dans la Pixmap Charset */
          pixmap_char_y = one_byte / 40;
          pixmap_char_x = one_byte - (pixmap_char_y*40);

          /* Copie ce bloc de 7*8 */
          offset_pixmap_char  = pixmap_char_x*7 + pixmap_char_y*(40*7)*8;
          offset_pixmap_screen = x*7 + y*(80*7)*8;
          for(pixel_y=0; pixel_y<8; pixel_y++)
            for(pixel_x=0; pixel_x<7; pixel_x++)
              {
                /* Pixel dans la Char Tab */
                color_pixmap = picture_char->data_dwd[offset_pixmap_char + pixel_x + pixel_y*40*7];

                /* Couleur du Pixel final */
                color_screen = (color_pixmap == 0xFFFFFF) ? color_text : color_background;

                /* Place le point dans la Pixmap */
                window_graphicdgrtext80->bitmap_data_graphicdgrtext80[offset_pixmap_screen + pixel_x + pixel_y*80*7] = color_screen;
              }
        }
    }

  /* Titre de la fentre */
  sprintf(buffer_title,"Graphic DGR Text 80 Column : %02X/%04X   %02X/%04X",window_graphicdgrtext80->graphicdgr_bank,window_graphicdgrtext80->graphicdgr_address,
                                                                            window_graphicdgrtext80->textpage80_bank,window_graphicdgrtext80->textpage80_address);
  SetWindowText(window_graphicdgrtext80->hwnd_graphicdgrtext80,buffer_title);
}


/******************************************************************************/
/*  DeleteGraphicDGRText80ChildWindows() :  Destruction des fentres enfants. */
/******************************************************************************/
static void DeleteGraphicDGRText80ChildWindows(struct window_graphicdgrtext80 *window_graphicdgrtext80)
{
  DeleteObject(window_graphicdgrtext80->hBitmap_graphicdgrtext80);
  DeleteObject(window_graphicdgrtext80->hwnd_combo_color_text);
  DeleteObject(window_graphicdgrtext80->hwnd_combo_color_background);
  DeleteObject(window_graphicdgrtext80->hwnd_combo_display_language);
  DeleteObject(window_graphicdgrtext80->hFont);
  ReleaseDC(window_graphicdgrtext80->hwnd_graphicdgrtext80,window_graphicdgrtext80->hDC);
  DeleteDC(window_graphicdgrtext80->hCompatibleDC);
}


/********************************************************************************/
/*  mem_free_window_graphicdgrtext80() :  Libration mmoire pour la structure. */
/********************************************************************************/
void mem_free_window_graphicdgrtext80(struct window_graphicdgrtext80 *window_graphicdgrtext80)
{
  if(window_graphicdgrtext80)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicdgrtext80->current_notification,NULL);
      window_graphicdgrtext80->current_notification = NULL;
      
      free(window_graphicdgrtext80);
    }
}

/******************************************************************************/
