/************************************************************************************/
/*                                                                                  */
/*  Window_GraphicDHGRColor.c :  Module de gestion de la fentre GraphicDHGRColor.  */
/*                                                                                  */
/************************************************************************************/
/*    Auteur : Olivier ZARDINI    *    Brutal Deluxe    *    Date : Juillet 2018    */
/************************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_GraphicDHGRColor.h"

static DWORD BuildGraphicDHGRColorWindowThread(void *);
LRESULT CALLBACK ProcHandlerGraphicDHGRColorWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateGraphicDHGRColorChildWindows(HWND,struct window_graphicdhgrcolor *);
LRESULT CALLBACK subEditGraphicDHGRColorProcControl(HWND,UINT,WPARAM,LPARAM);
void UpdateGraphicDHGRColorClientAllArea(HDC,struct window_graphicdhgrcolor *);
static void UpdateGraphicDHGRColorClientBitmapArea(HDC,struct window_graphicdhgrcolor *);
static void UpdateGraphicDHGRColorContextInformation(struct window_graphicdhgrcolor *,int,int);
static void UpdateGraphicDHGRColorAddress(struct window_graphicdhgrcolor *,int);
static void UpdateGraphicDHGRColorRefreshMode(struct window_graphicdhgrcolor *,int);
static void UpdateGraphicDHGRColorZoomLevel(struct window_graphicdhgrcolor *,int);
static void RefreshGraphicDHGRColorWindowData(struct window_graphicdhgrcolor *,int);
static int IsGraphicDHGRColorDataModified(int,int,int,int,struct window_graphicdhgrcolor *);
static void ResizeGraphicDHGRColorWindow(struct window_graphicdhgrcolor *);
static void BuildGraphicDHGRColorFromData(struct window_graphicdhgrcolor *);
static void DeleteGraphicDHGRColorChildWindows(struct window_graphicdhgrcolor *);


/**********************************************************************************************/
/*  BuildGraphicDHGRColorWindow() :  Construction / Affichage de la fentre GraphicDHGRColor. */
/**********************************************************************************************/
struct window_graphicdhgrcolor *BuildGraphicDHGRColorWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_graphicdhgrcolor *window_graphicdhgrcolor;

  /** Allocation de la structure window_graphicdhgrcolor **/
  window_graphicdhgrcolor = (struct window_graphicdhgrcolor *) calloc(1,sizeof(struct window_graphicdhgrcolor));
  if(window_graphicdhgrcolor == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_GRAPHICDHGRCOLOR,window_graphicdhgrcolor,NULL);

  /* Lance un thread pour grer la fentre GraphicDHGRColor */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildGraphicDHGRColorWindowThread,window_graphicdhgrcolor,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_graphicdhgrcolor);
}


/********************************************************************************************************/
/*  BuildGraphicDHGRColorWindowThread() :  Cration de la fentre GraphicDHGRColor via un Thread ddi. */
/********************************************************************************************************/
static DWORD BuildGraphicDHGRColorWindowThread(void *window_graphicdhgrcolor_param)
{
  HWND hwndGraphicDHGRColor;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_graphicdhgrcolor *window_graphicdhgrcolor;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_graphicdhgrcolor = (struct window_graphicdhgrcolor *) window_graphicdhgrcolor_param;
  window_graphicdhgrcolor->thread_id = GetCurrentThreadId();         /* Le ThreaID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Graphic GR **/
  hwndGraphicDHGRColor = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_GRAPHICDHGRCOLOR),window_main->hwnd_main,(DLGPROC)ProcHandlerGraphicDHGRColorWindow);
  if(hwndGraphicDHGRColor == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndGraphicDHGRColor,SW_SHOWDEFAULT);
  UpdateWindow(hwndGraphicDHGRColor);

  /** Message Loop pour toutes les fentres de ce Thread (fentre GraphicDHGRColor + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/*********************************************************************************/
/*  ProcHandlerGraphicDHGRColorWindow() :  Gestion des vnements de la fentre. */
/*********************************************************************************/
LRESULT CALLBACK ProcHandlerGraphicDHGRColorWindow(HWND hwndGraphicDHGRColor, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;
  int mouse_x, mouse_y;
  struct window_graphicdhgrcolor *window_graphicdhgrcolor;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_GRAPHICDHGRCOLOR,&window_graphicdhgrcolor,NULL);
  if(window_graphicdhgrcolor == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateGraphicDHGRColorChildWindows(hwndGraphicDHGRColor,window_graphicdhgrcolor);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        UpdateGraphicDHGRColorContextInformation(window_graphicdhgrcolor,mouse_x,mouse_y);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_graphicdhgrcolor->hBitmap_graphicdhgrcolor != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndGraphicDHGRColor,&ps);

            /** MAJ Bitmap **/
            UpdateGraphicDHGRColorClientAllArea(hDC,window_graphicdhgrcolor);

            /* End Paint */
            EndPaint(hwndGraphicDHGRColor,&ps);
          }
        break;
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdhgrcolor->refresh_mode != GRAPHICDHGRCOLOR_REFRESH_AUTOMATIC)
                UpdateGraphicDHGRColorRefreshMode(window_graphicdhgrcolor,GRAPHICDHGRCOLOR_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdhgrcolor->refresh_mode != GRAPHICDHGRCOLOR_REFRESH_MANUAL)
                UpdateGraphicDHGRColorRefreshMode(window_graphicdhgrcolor,GRAPHICDHGRCOLOR_REFRESH_MANUAL);
              break;

            /** Bouton Radio : On change le Zoom de la fentre **/
            case IDC_RADIO_ZOOM_X1 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdhgrcolor->zoom_level != GRAPHICDHGRCOLOR_ZOOM_X1)
                UpdateGraphicDHGRColorZoomLevel(window_graphicdhgrcolor,GRAPHICDHGRCOLOR_ZOOM_X1);
              break;

            case IDC_RADIO_ZOOM_X2 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdhgrcolor->zoom_level != GRAPHICDHGRCOLOR_ZOOM_X2)
                UpdateGraphicDHGRColorZoomLevel(window_graphicdhgrcolor,GRAPHICDHGRCOLOR_ZOOM_X2);
              break;

            case IDC_RADIO_ZOOM_X3 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdhgrcolor->zoom_level != GRAPHICDHGRCOLOR_ZOOM_X3)
                UpdateGraphicDHGRColorZoomLevel(window_graphicdhgrcolor,GRAPHICDHGRCOLOR_ZOOM_X3);
              break;

            /** Bouton de dplacement de l'adresse **/
            case IDC_BUTTON_LEFT :
              UpdateGraphicDHGRColorAddress(window_graphicdhgrcolor,-1);
              break;

            case IDC_BUTTON_RIGHT :
              UpdateGraphicDHGRColorAddress(window_graphicdhgrcolor,1);
              break;

            case IDC_BUTTON_UP :
              UpdateGraphicDHGRColorAddress(window_graphicdhgrcolor,-40);
              break;

            case IDC_BUTTON_DOWN :
              UpdateGraphicDHGRColorAddress(window_graphicdhgrcolor,40);
              break;

            case IDC_BUTTON_PAGEUP :
              UpdateGraphicDHGRColorAddress(window_graphicdhgrcolor,-8192);
              break;

            case IDC_BUTTON_PAGEDOWN :
              UpdateGraphicDHGRColorAddress(window_graphicdhgrcolor,8192);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshGraphicDHGRColorWindowData(window_graphicdhgrcolor,1);
              break;              
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_GRAPHICDHGRCOLOR_REFRESHWINDOW :
        RefreshGraphicDHGRColorWindowData(window_graphicdhgrcolor,0);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre GraphicDHGRColor */
        DestroyWindow(hwndGraphicDHGRColor);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteGraphicDHGRColorChildWindows(window_graphicdhgrcolor);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_GRAPHICDHGRCOLOR,window_graphicdhgrcolor,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************************/
/*  CreateGraphicDHGRColorChildWindows() : Crer les fentres enfants et remplir la structure window_graphicdhgrcolor. */
/***********************************************************************************************************************/
static int CreateGraphicDHGRColorChildWindows(HWND hwndGraphicDHGRColor, struct window_graphicdhgrcolor *window_graphicdhgrcolor)
{
  BITMAPINFO bitmap_info;
  int i;
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_graphicdhgrcolor->hwnd_graphicdhgrcolor = hwndGraphicDHGRColor;

  /** Device Context **/
  window_graphicdhgrcolor->hDC = GetDC(window_graphicdhgrcolor->hwnd_graphicdhgrcolor);
  window_graphicdhgrcolor->hCompatibleDC = CreateCompatibleDC(window_graphicdhgrcolor->hDC);

  /** Surchage des Control Edit pour capter les vnements **/
  window_graphicdhgrcolor->oldEditProcBank    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDHGRColor,IDC_EDIT_BANK),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDHGRColorProcControl);
  window_graphicdhgrcolor->oldEditProcAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDHGRColor,IDC_EDIT_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDHGRColorProcControl);

  /** Ajoute les images sur les boutons **/
  SendMessage(GetDlgItem(hwndGraphicDHGRColor,IDC_BUTTON_LEFT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColor,IDC_BUTTON_RIGHT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColor,IDC_BUTTON_UP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColor,IDC_BUTTON_DOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColor,IDC_BUTTON_PAGEUP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColor,IDC_BUTTON_PAGEDOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Adresse de la Zone de Data **/
  window_graphicdhgrcolor->graphicdhgrcolor_bank = 0x00;
  window_graphicdhgrcolor->graphicdhgrcolor_address = 0x2000; 
  sprintf(buffer,"%02X",window_graphicdhgrcolor->graphicdhgrcolor_bank);
  SetDlgItemText(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",window_graphicdhgrcolor->graphicdhgrcolor_address);
  SetDlgItemText(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_ADDRESS,buffer);

  /* Refresh Mode : Automatic */
  UpdateGraphicDHGRColorRefreshMode(window_graphicdhgrcolor,GRAPHICDHGRCOLOR_REFRESH_AUTOMATIC);

  /* Zoom Level : 1 */
  window_graphicdhgrcolor->zoom_level = GRAPHICDHGRCOLOR_ZOOM_X1;
  SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);

  /* Dimension en Pixel de la Bitmap GraphicDHGRColor Data */
  window_graphicdhgrcolor->bitmap_graphicdhgrcolor_width = 280;
  window_graphicdhgrcolor->bitmap_graphicdhgrcolor_height = 192;
  
  /** Bitmap affichant la zone de slection **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicdhgrcolor->bitmap_graphicdhgrcolor_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicdhgrcolor->bitmap_graphicdhgrcolor_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicdhgrcolor->hBitmap_graphicdhgrcolor = CreateDIBSection(window_graphicdhgrcolor->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor),NULL,0);
  for(i=0; i<window_graphicdhgrcolor->bitmap_graphicdhgrcolor_width*window_graphicdhgrcolor->bitmap_graphicdhgrcolor_height; i++)
    window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[i] = BITMAP_BACKGROUND_COLOR;

  /* Retaille la fentre */
  ResizeGraphicDHGRColorWindow(window_graphicdhgrcolor);

  /** Construction de la Page Texte + Rafraichissement de la fentre **/
  RefreshGraphicDHGRColorWindowData(window_graphicdhgrcolor,1);
    
  /* OK */
  return(0);
}


/*******************************************************************************************************/
/*  subEditGraphicDHGRColorProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*******************************************************************************************************/
LRESULT CALLBACK subEditGraphicDHGRColorProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_graphicdhgrcolor *window_graphicdhgrcolor;
  my_Memory(MEMORY_GET_WINDOW_GRAPHICDHGRCOLOR,&window_graphicdhgrcolor,NULL);

  /* Rien  faire */
  if(window_graphicdhgrcolor == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_BANK) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_ADDRESS) == hwnd_control)
    index = 2;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement du Return dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_graphicdhgrcolor->hwnd_graphicdhgrcolor);

      /* On force la MAJ de la fentre */
      RefreshGraphicDHGRColorWindowData(window_graphicdhgrcolor,1);
    }
  else if(index == 1)
    return(CallWindowProc(window_graphicdhgrcolor->oldEditProcBank,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_graphicdhgrcolor->oldEditProcAddress,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
}


/**************************************************************************************************/
/*  UpdateGraphicDHGRColorClientAllArea() :  Affichage des Bitmap de la fentre GraphicDHGRColor. */
/**************************************************************************************************/
void UpdateGraphicDHGRColorClientAllArea(HDC hDC, struct window_graphicdhgrcolor *window_graphicdhgrcolor)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap **/
  UpdateGraphicDHGRColorClientBitmapArea(hDC,window_graphicdhgrcolor);
}


/*****************************************************************************************/
/*  UpdateGraphicDHGRColorClientBitmapArea() :  Affichage de la Bitmap GraphicDHGRColor. */
/*****************************************************************************************/
static void UpdateGraphicDHGRColorClientBitmapArea(HDC hDC, struct window_graphicdhgrcolor *window_graphicdhgrcolor)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec la zone mmoire et la zone slectionne **/
  bitmap_old = SelectObject(window_graphicdhgrcolor->hCompatibleDC,window_graphicdhgrcolor->hBitmap_graphicdhgrcolor);
  StretchBlt(hDC,window_graphicdhgrcolor->bitmap_graphicdhgrcolor_x,window_graphicdhgrcolor->bitmap_graphicdhgrcolor_y,
             window_graphicdhgrcolor->bitmap_graphicdhgrcolor_width*window_graphicdhgrcolor->zoom_level,
             window_graphicdhgrcolor->bitmap_graphicdhgrcolor_height*window_graphicdhgrcolor->zoom_level,
             window_graphicdhgrcolor->hCompatibleDC,0,0,window_graphicdhgrcolor->bitmap_graphicdhgrcolor_width,window_graphicdhgrcolor->bitmap_graphicdhgrcolor_height,SRCCOPY);
  SelectObject(window_graphicdhgrcolor->hCompatibleDC,bitmap_old);
}


/*******************************************************************************************/
/*  UpdateGraphicDHGRColorAddress() :  Dplace l'adresse pour la fentre GraphicDHGRColor. */
/*******************************************************************************************/
static void UpdateGraphicDHGRColorAddress(struct window_graphicdhgrcolor *window_graphicdhgrcolor, int delta)
{
  int current_address, new_address, graphicdhgrcolor_bank, graphicdhgrcolor_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphicdhgrcolor->graphicdhgrcolor_bank << 16) | window_graphicdhgrcolor->graphicdhgrcolor_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  graphicdhgrcolor_bank = (new_address >> 16) & 0xFF;
  graphicdhgrcolor_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphicdhgrcolor_bank);
  SetDlgItemText(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",graphicdhgrcolor_address);
  SetDlgItemText(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_ADDRESS,buffer);

  /* MAJ de la fentre */
  RefreshGraphicDHGRColorWindowData(window_graphicdhgrcolor,1);
}


/*******************************************************************************************************************/
/*  UpdateGraphicDHGRColorContextInformation() :  MAJ des informations de contexte de la fentre GraphicDHGRColor. */
/*******************************************************************************************************************/
static void UpdateGraphicDHGRColorContextInformation(struct window_graphicdhgrcolor *window_graphicdhgrcolor, int mouse_x, int mouse_y)
{
  int pixmap_x, pixmap_y, grid_x, byte_group_x;
  int graphicdhgrcolor_data_address, graphicdhgrcolor_data_bank;
  unsigned char byte0_bank0, byte0_bank1, byte1_bank0, byte1_bank1;
  int tab_hgr_address[192] =     /* Adresse du dbut des 192 lignes de la Page HGR */
    {
      0x0000+0x0000, 0x0000+0x0400, 0x0000+0x0800, 0x0000+0x0C00, 0x0000+0x1000, 0x0000+0x1400, 0x0000+0x1800, 0x0000+0x1C00, 
      0x0080+0x0000, 0x0080+0x0400, 0x0080+0x0800, 0x0080+0x0C00, 0x0080+0x1000, 0x0080+0x1400, 0x0080+0x1800, 0x0080+0x1C00, 
      0x0100+0x0000, 0x0100+0x0400, 0x0100+0x0800, 0x0100+0x0C00, 0x0100+0x1000, 0x0100+0x1400, 0x0100+0x1800, 0x0100+0x1C00, 
      0x0180+0x0000, 0x0180+0x0400, 0x0180+0x0800, 0x0180+0x0C00, 0x0180+0x1000, 0x0180+0x1400, 0x0180+0x1800, 0x0180+0x1C00, 
      0x0200+0x0000, 0x0200+0x0400, 0x0200+0x0800, 0x0200+0x0C00, 0x0200+0x1000, 0x0200+0x1400, 0x0200+0x1800, 0x0200+0x1C00, 
      0x0280+0x0000, 0x0280+0x0400, 0x0280+0x0800, 0x0280+0x0C00, 0x0280+0x1000, 0x0280+0x1400, 0x0280+0x1800, 0x0280+0x1C00, 
      0x0300+0x0000, 0x0300+0x0400, 0x0300+0x0800, 0x0300+0x0C00, 0x0300+0x1000, 0x0300+0x1400, 0x0300+0x1800, 0x0300+0x1C00, 
      0x0380+0x0000, 0x0380+0x0400, 0x0380+0x0800, 0x0380+0x0C00, 0x0380+0x1000, 0x0380+0x1400, 0x0380+0x1800, 0x0380+0x1C00, 
      0x0028+0x0000, 0x0028+0x0400, 0x0028+0x0800, 0x0028+0x0C00, 0x0028+0x1000, 0x0028+0x1400, 0x0028+0x1800, 0x0028+0x1C00, 
      0x00A8+0x0000, 0x00A8+0x0400, 0x00A8+0x0800, 0x00A8+0x0C00, 0x00A8+0x1000, 0x00A8+0x1400, 0x00A8+0x1800, 0x00A8+0x1C00, 
      0x0128+0x0000, 0x0128+0x0400, 0x0128+0x0800, 0x0128+0x0C00, 0x0128+0x1000, 0x0128+0x1400, 0x0128+0x1800, 0x0128+0x1C00, 
      0x01A8+0x0000, 0x01A8+0x0400, 0x01A8+0x0800, 0x01A8+0x0C00, 0x01A8+0x1000, 0x01A8+0x1400, 0x01A8+0x1800, 0x01A8+0x1C00, 
      0x0228+0x0000, 0x0228+0x0400, 0x0228+0x0800, 0x0228+0x0C00, 0x0228+0x1000, 0x0228+0x1400, 0x0228+0x1800, 0x0228+0x1C00, 
      0x02A8+0x0000, 0x02A8+0x0400, 0x02A8+0x0800, 0x02A8+0x0C00, 0x02A8+0x1000, 0x02A8+0x1400, 0x02A8+0x1800, 0x02A8+0x1C00, 
      0x0328+0x0000, 0x0328+0x0400, 0x0328+0x0800, 0x0328+0x0C00, 0x0328+0x1000, 0x0328+0x1400, 0x0328+0x1800, 0x0328+0x1C00, 
      0x03A8+0x0000, 0x03A8+0x0400, 0x03A8+0x0800, 0x03A8+0x0C00, 0x03A8+0x1000, 0x03A8+0x1400, 0x03A8+0x1800, 0x03A8+0x1C00, 
      0x0050+0x0000, 0x0050+0x0400, 0x0050+0x0800, 0x0050+0x0C00, 0x0050+0x1000, 0x0050+0x1400, 0x0050+0x1800, 0x0050+0x1C00, 
      0x00D0+0x0000, 0x00D0+0x0400, 0x00D0+0x0800, 0x00D0+0x0C00, 0x00D0+0x1000, 0x00D0+0x1400, 0x00D0+0x1800, 0x00D0+0x1C00, 
      0x0150+0x0000, 0x0150+0x0400, 0x0150+0x0800, 0x0150+0x0C00, 0x0150+0x1000, 0x0150+0x1400, 0x0150+0x1800, 0x0150+0x1C00, 
      0x01D0+0x0000, 0x01D0+0x0400, 0x01D0+0x0800, 0x01D0+0x0C00, 0x01D0+0x1000, 0x01D0+0x1400, 0x01D0+0x1800, 0x01D0+0x1C00, 
      0x0250+0x0000, 0x0250+0x0400, 0x0250+0x0800, 0x0250+0x0C00, 0x0250+0x1000, 0x0250+0x1400, 0x0250+0x1800, 0x0250+0x1C00, 
      0x02D0+0x0000, 0x02D0+0x0400, 0x02D0+0x0800, 0x02D0+0x0C00, 0x02D0+0x1000, 0x02D0+0x1400, 0x02D0+0x1800, 0x02D0+0x1C00, 
      0x0350+0x0000, 0x0350+0x0400, 0x0350+0x0800, 0x0350+0x0C00, 0x0350+0x1000, 0x0350+0x1400, 0x0350+0x1800, 0x0350+0x1C00, 
      0x03D0+0x0000, 0x03D0+0x0400, 0x03D0+0x0800, 0x03D0+0x0C00, 0x03D0+0x1000, 0x03D0+0x1400, 0x03D0+0x1800, 0x03D0+0x1C00
    };

  /* Rien  faire */
  if(window_graphicdhgrcolor == NULL)
    return;

  /* Est t'on dans l'image ? */
  if(mouse_x < window_graphicdhgrcolor->bitmap_graphicdhgrcolor_x || mouse_x >= (window_graphicdhgrcolor->bitmap_graphicdhgrcolor_x + window_graphicdhgrcolor->bitmap_graphicdhgrcolor_width*window_graphicdhgrcolor->zoom_level) ||
     mouse_y < window_graphicdhgrcolor->bitmap_graphicdhgrcolor_y || mouse_y >= (window_graphicdhgrcolor->bitmap_graphicdhgrcolor_y + window_graphicdhgrcolor->bitmap_graphicdhgrcolor_height*2*window_graphicdhgrcolor->zoom_level))
    {
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT),"");
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT2),"");
      return;
    }

  /* Coordonnes dans la Pixmap */
  pixmap_x = (mouse_x - window_graphicdhgrcolor->bitmap_graphicdhgrcolor_x)/window_graphicdhgrcolor->zoom_level;    /* 0-279 */
  pixmap_y = (mouse_y - window_graphicdhgrcolor->bitmap_graphicdhgrcolor_y)/window_graphicdhgrcolor->zoom_level;

  /* Coordonnes dans la grille 140x192 */
  grid_x = pixmap_x/2;
  byte_group_x = grid_x/7;

  /* 1 pixel = 4 bit, 4 Bytes = 7 pixels */
  byte0_bank0 = window_graphicdhgrcolor->graphicdhgrcolor_data[byte_group_x + 1 + pixmap_y*80];
  byte0_bank1 = window_graphicdhgrcolor->graphicdhgrcolor_data[byte_group_x + 0 + pixmap_y*80];
  byte1_bank0 = window_graphicdhgrcolor->graphicdhgrcolor_data[byte_group_x + 3 + pixmap_y*80];
  byte1_bank1 = window_graphicdhgrcolor->graphicdhgrcolor_data[byte_group_x + 2 + pixmap_y*80];

  /* Adresse */
  graphicdhgrcolor_data_address = (window_graphicdhgrcolor->graphicdhgrcolor_bank << 16) + window_graphicdhgrcolor->graphicdhgrcolor_address;
  graphicdhgrcolor_data_address += tab_hgr_address[pixmap_y];  
  graphicdhgrcolor_data_address += byte_group_x;
  graphicdhgrcolor_data_bank = graphicdhgrcolor_data_address>>16;

  /** Message **/
  if((grid_x % 7) == 0)
    {
      /* Pixel 0 : byte0_bank1 */
      sprintf(window_graphicdhgrcolor->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgrcolor_data_bank+1,graphicdhgrcolor_data_address&0xFFFF,byte0_bank1,grid_x,pixmap_y);
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT),window_graphicdhgrcolor->buffer_log);
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT2),"");
    }
  else if((grid_x % 7) == 1)
    {
      /* Pixel 1 : byte0_bank1 + byte0_bank0 */
      sprintf(window_graphicdhgrcolor->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgrcolor_data_bank+1,graphicdhgrcolor_data_address&0xFFFF,byte0_bank1,grid_x,pixmap_y);
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT),window_graphicdhgrcolor->buffer_log);  
      sprintf(window_graphicdhgrcolor->buffer_log,"%02X/%04X : %02X",graphicdhgrcolor_data_bank,graphicdhgrcolor_data_address&0xFFFF,byte0_bank0);
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT2),window_graphicdhgrcolor->buffer_log); 
    }
  else if((grid_x % 7) == 2)
    {
      /* Pixel 2 : byte0_bank0 */
      sprintf(window_graphicdhgrcolor->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgrcolor_data_bank,graphicdhgrcolor_data_address&0xFFFF,byte0_bank0,grid_x,pixmap_y);
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT),window_graphicdhgrcolor->buffer_log);
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT2),"");      
    }
  else if((grid_x % 7) == 3)
    {
      /* Pixel 3 : byte0_bank0 + byte1_bank1 */
      sprintf(window_graphicdhgrcolor->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgrcolor_data_bank,graphicdhgrcolor_data_address&0xFFFF,byte0_bank0,grid_x,pixmap_y);
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT),window_graphicdhgrcolor->buffer_log);      
      sprintf(window_graphicdhgrcolor->buffer_log,"%02X/%04X : %02X",graphicdhgrcolor_data_bank+1,(graphicdhgrcolor_data_address+1)&0xFFFF,byte1_bank1);
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT2),window_graphicdhgrcolor->buffer_log);      
    }
  else if((grid_x % 7) == 4)
    {
      /* Pixel 4 : byte1_bank1 */
      sprintf(window_graphicdhgrcolor->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgrcolor_data_bank+1,(graphicdhgrcolor_data_address+1)&0xFFFF,byte1_bank1,grid_x,pixmap_y);
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT),window_graphicdhgrcolor->buffer_log);  
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT2),"");      
    }
  else if((grid_x % 7) == 5)
    {
      /* Pixel 5 : byte1_bank1 + byte1_bank0 */
      sprintf(window_graphicdhgrcolor->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgrcolor_data_bank+1,(graphicdhgrcolor_data_address+1)&0xFFFF,byte1_bank1,grid_x,pixmap_y);
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT),window_graphicdhgrcolor->buffer_log);      
      sprintf(window_graphicdhgrcolor->buffer_log,"%02X/%04X : %02X",graphicdhgrcolor_data_bank,(graphicdhgrcolor_data_address+1)&0xFFFF,byte1_bank0);
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT2),window_graphicdhgrcolor->buffer_log);            
    }
  else
    {
      /* Pixel 6 : byte1_bank0 */
      sprintf(window_graphicdhgrcolor->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgrcolor_data_bank,(graphicdhgrcolor_data_address+1)&0xFFFF,byte1_bank0,grid_x,pixmap_y);
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT),window_graphicdhgrcolor->buffer_log);  
      SetWindowText(GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_STATIC_CONTEXT2),"");      
    }
}


/***********************************************************************************************************/
/*  UpdateGraphicDHGRColorRefreshMode() :  MAJ du mode de rafraichissement de la fentre GraphicDHGRColor. */
/***********************************************************************************************************/
static void UpdateGraphicDHGRColorRefreshMode(struct window_graphicdhgrcolor *window_graphicdhgrcolor, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == GRAPHICDHGRCOLOR_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_graphicdhgrcolor->refresh_mode = GRAPHICDHGRCOLOR_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_graphicdhgrcolor->hwnd_graphicdhgrcolor;
      current_notification->message = HM_GRAPHICDHGRCOLOR_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_GRAPHICDHGRCOLOR;

      /* Ajout de la notification */
      window_graphicdhgrcolor->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_graphicdhgrcolor->refresh_mode = GRAPHICDHGRCOLOR_REFRESH_MANUAL;
      SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicdhgrcolor->current_notification,NULL);
      window_graphicdhgrcolor->current_notification = NULL;
    }
}


/***********************************************************************************************/
/*  UpdateGraphicDHGRColorZoomLevel() :  MAJ du nieau de Zomm de la fenetre GraphicDHGRColor. */
/***********************************************************************************************/
static void UpdateGraphicDHGRColorZoomLevel(struct window_graphicdhgrcolor *window_graphicdhgrcolor, int zoom_level)
{
  if(zoom_level == GRAPHICDHGRCOLOR_ZOOM_X1)
    {
      /* Visuel */
      window_graphicdhgrcolor->zoom_level = GRAPHICDHGRCOLOR_ZOOM_X1;
      SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);      
    }
  else if(zoom_level == GRAPHICDHGRCOLOR_ZOOM_X2)
    {
      /* Visuel */
      window_graphicdhgrcolor->zoom_level = GRAPHICDHGRCOLOR_ZOOM_X2;
      SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_ZOOM_X2,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICDHGRCOLOR_ZOOM_X3)
    {
      /* Visuel */
      window_graphicdhgrcolor->zoom_level = GRAPHICDHGRCOLOR_ZOOM_X3;
      SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_RADIO_ZOOM_X3,BM_SETCHECK,1,0);            
    }
        
  /* Retaille la fentre */
  ResizeGraphicDHGRColorWindow(window_graphicdhgrcolor);  
  
  /* On force la MAJ de la fentre */  
  RefreshGraphicDHGRColorWindowData(window_graphicdhgrcolor,1);    
}


/******************************************************************************************/
/*  RefreshGraphicDHGRColorWindowData() :  MAJ du contenu de la fentre GraphicDHGRColor. */
/******************************************************************************************/
static void RefreshGraphicDHGRColorWindowData(struct window_graphicdhgrcolor *window_graphicdhgrcolor, int force)
{
  int length, is_modified;
  int graphicdhgrcolor_bank, graphicdhgrcolor_address, nb_column, nb_row;
  char buffer[1024];

  /* Rcupre la valeur du GraphicDHGRColor Pointer */
  length = GetDlgItemText(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_BANK,buffer,1024);
  sscanf(buffer,"%X",&graphicdhgrcolor_bank);
  length = GetDlgItemText(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_ADDRESS,buffer,1024);
  sscanf(buffer,"%X",&graphicdhgrcolor_address);
  nb_column = GetDlgItemInt(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_COLUMN,NULL,FALSE);
  if(nb_column < 1)
    nb_column = 1;
  nb_row = GetDlgItemInt(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_ROW,NULL,FALSE);
  if(nb_row < 1)
    nb_row = 1;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphicdhgrcolor_bank);
  SetDlgItemText(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",graphicdhgrcolor_address);
  SetDlgItemText(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_ADDRESS,buffer);
  SetDlgItemInt(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_COLUMN,nb_column,FALSE);
  SetDlgItemInt(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_EDIT_ROW,nb_row,FALSE);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsGraphicDHGRColorDataModified(graphicdhgrcolor_bank,graphicdhgrcolor_address,nb_column,nb_row,window_graphicdhgrcolor);
  if(is_modified == 0 && force == 0)
    return;

  /** Cration de la Zone Graphique  partir des nouvelles donnes **/
  BuildGraphicDHGRColorFromData(window_graphicdhgrcolor);

  /* Rafraichit la fentre */
  InvalidateRect(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,NULL,TRUE);
  UpdateWindow(window_graphicdhgrcolor->hwnd_graphicdhgrcolor);
}


/************************************************************************************************************************/
/*  IsGraphicDHGRColorDataModified() :  Les donnes GraphicDHGRColor ont t'elles t modifies depuis la dernire fois. */
/************************************************************************************************************************/
static int IsGraphicDHGRColorDataModified(int graphicdhgrcolor_bank, int graphicdhgrcolor_address, int nb_column, int nb_row, struct window_graphicdhgrcolor *window_graphicdhgrcolor)
{
  int x, y, graphicdhgrcolor_data_address;
  unsigned char graphicdhgrcolor_raw_bank0_data[8192];
  unsigned char graphicdhgrcolor_raw_bank1_data[8192];
  unsigned char graphicdhgrcolor_data[80*192];
  int tab_hgr_address[192] =     /* Adresse du dbut des 192 lignes de la Page HGR */
    {
      0x0000+0x0000, 0x0000+0x0400, 0x0000+0x0800, 0x0000+0x0C00, 0x0000+0x1000, 0x0000+0x1400, 0x0000+0x1800, 0x0000+0x1C00, 
      0x0080+0x0000, 0x0080+0x0400, 0x0080+0x0800, 0x0080+0x0C00, 0x0080+0x1000, 0x0080+0x1400, 0x0080+0x1800, 0x0080+0x1C00, 
      0x0100+0x0000, 0x0100+0x0400, 0x0100+0x0800, 0x0100+0x0C00, 0x0100+0x1000, 0x0100+0x1400, 0x0100+0x1800, 0x0100+0x1C00, 
      0x0180+0x0000, 0x0180+0x0400, 0x0180+0x0800, 0x0180+0x0C00, 0x0180+0x1000, 0x0180+0x1400, 0x0180+0x1800, 0x0180+0x1C00, 
      0x0200+0x0000, 0x0200+0x0400, 0x0200+0x0800, 0x0200+0x0C00, 0x0200+0x1000, 0x0200+0x1400, 0x0200+0x1800, 0x0200+0x1C00, 
      0x0280+0x0000, 0x0280+0x0400, 0x0280+0x0800, 0x0280+0x0C00, 0x0280+0x1000, 0x0280+0x1400, 0x0280+0x1800, 0x0280+0x1C00, 
      0x0300+0x0000, 0x0300+0x0400, 0x0300+0x0800, 0x0300+0x0C00, 0x0300+0x1000, 0x0300+0x1400, 0x0300+0x1800, 0x0300+0x1C00, 
      0x0380+0x0000, 0x0380+0x0400, 0x0380+0x0800, 0x0380+0x0C00, 0x0380+0x1000, 0x0380+0x1400, 0x0380+0x1800, 0x0380+0x1C00, 
      0x0028+0x0000, 0x0028+0x0400, 0x0028+0x0800, 0x0028+0x0C00, 0x0028+0x1000, 0x0028+0x1400, 0x0028+0x1800, 0x0028+0x1C00, 
      0x00A8+0x0000, 0x00A8+0x0400, 0x00A8+0x0800, 0x00A8+0x0C00, 0x00A8+0x1000, 0x00A8+0x1400, 0x00A8+0x1800, 0x00A8+0x1C00, 
      0x0128+0x0000, 0x0128+0x0400, 0x0128+0x0800, 0x0128+0x0C00, 0x0128+0x1000, 0x0128+0x1400, 0x0128+0x1800, 0x0128+0x1C00, 
      0x01A8+0x0000, 0x01A8+0x0400, 0x01A8+0x0800, 0x01A8+0x0C00, 0x01A8+0x1000, 0x01A8+0x1400, 0x01A8+0x1800, 0x01A8+0x1C00, 
      0x0228+0x0000, 0x0228+0x0400, 0x0228+0x0800, 0x0228+0x0C00, 0x0228+0x1000, 0x0228+0x1400, 0x0228+0x1800, 0x0228+0x1C00, 
      0x02A8+0x0000, 0x02A8+0x0400, 0x02A8+0x0800, 0x02A8+0x0C00, 0x02A8+0x1000, 0x02A8+0x1400, 0x02A8+0x1800, 0x02A8+0x1C00, 
      0x0328+0x0000, 0x0328+0x0400, 0x0328+0x0800, 0x0328+0x0C00, 0x0328+0x1000, 0x0328+0x1400, 0x0328+0x1800, 0x0328+0x1C00, 
      0x03A8+0x0000, 0x03A8+0x0400, 0x03A8+0x0800, 0x03A8+0x0C00, 0x03A8+0x1000, 0x03A8+0x1400, 0x03A8+0x1800, 0x03A8+0x1C00, 
      0x0050+0x0000, 0x0050+0x0400, 0x0050+0x0800, 0x0050+0x0C00, 0x0050+0x1000, 0x0050+0x1400, 0x0050+0x1800, 0x0050+0x1C00, 
      0x00D0+0x0000, 0x00D0+0x0400, 0x00D0+0x0800, 0x00D0+0x0C00, 0x00D0+0x1000, 0x00D0+0x1400, 0x00D0+0x1800, 0x00D0+0x1C00, 
      0x0150+0x0000, 0x0150+0x0400, 0x0150+0x0800, 0x0150+0x0C00, 0x0150+0x1000, 0x0150+0x1400, 0x0150+0x1800, 0x0150+0x1C00, 
      0x01D0+0x0000, 0x01D0+0x0400, 0x01D0+0x0800, 0x01D0+0x0C00, 0x01D0+0x1000, 0x01D0+0x1400, 0x01D0+0x1800, 0x01D0+0x1C00, 
      0x0250+0x0000, 0x0250+0x0400, 0x0250+0x0800, 0x0250+0x0C00, 0x0250+0x1000, 0x0250+0x1400, 0x0250+0x1800, 0x0250+0x1C00, 
      0x02D0+0x0000, 0x02D0+0x0400, 0x02D0+0x0800, 0x02D0+0x0C00, 0x02D0+0x1000, 0x02D0+0x1400, 0x02D0+0x1800, 0x02D0+0x1C00, 
      0x0350+0x0000, 0x0350+0x0400, 0x0350+0x0800, 0x0350+0x0C00, 0x0350+0x1000, 0x0350+0x1400, 0x0350+0x1800, 0x0350+0x1C00, 
      0x03D0+0x0000, 0x03D0+0x0400, 0x03D0+0x0800, 0x03D0+0x0C00, 0x03D0+0x1000, 0x03D0+0x1400, 0x03D0+0x1800, 0x03D0+0x1C00
    };
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /* Rcupre les donnes brutes : 2*8192 chars */
  graphicdhgrcolor_data_address = ((graphicdhgrcolor_bank+1) << 16) | graphicdhgrcolor_address;
  ReadMemoryBlock(graphicdhgrcolor_data_address,8192,graphicdhgrcolor_raw_bank1_data,current_mmu);
  graphicdhgrcolor_data_address = (graphicdhgrcolor_bank << 16) | graphicdhgrcolor_address;
  ReadMemoryBlock(graphicdhgrcolor_data_address,8192,graphicdhgrcolor_raw_bank0_data,current_mmu);

  /* R-organise les donnes : Linarisation (passage du block de 2*8192 char en 80*192) */
  for(y=0; y<192; y++)
    for(x=0; x<40; x++)
      {
        graphicdhgrcolor_data[2*x+0+y*80] = graphicdhgrcolor_raw_bank1_data[tab_hgr_address[y]+x];
        graphicdhgrcolor_data[2*x+1+y*80] = graphicdhgrcolor_raw_bank0_data[tab_hgr_address[y]+x];
      }

  /* Compare les donnes */
  if(window_graphicdhgrcolor->graphicdhgrcolor_data != NULL)
    if(graphicdhgrcolor_bank == window_graphicdhgrcolor->graphicdhgrcolor_bank &&
       graphicdhgrcolor_address == window_graphicdhgrcolor->graphicdhgrcolor_address &&
       memcmp(&graphicdhgrcolor_data[0],&window_graphicdhgrcolor->graphicdhgrcolor_data[0],80*192) == 0)
      return(0);

  /* Les donnes sont diffrentes => MAJ */
  window_graphicdhgrcolor->graphicdhgrcolor_bank = graphicdhgrcolor_bank;
  window_graphicdhgrcolor->graphicdhgrcolor_address = graphicdhgrcolor_address;
  memcpy(window_graphicdhgrcolor->graphicdhgrcolor_data,graphicdhgrcolor_data,80*192);

  /* Donnes diffrentes */
  return(1);
}


/***********************************************************************************/
/*  ResizeGraphicDHGRColorWindow() : Resize GraphicDHGRColor window by given size. */
/***********************************************************************************/
static void ResizeGraphicDHGRColorWindow(struct window_graphicdhgrcolor *window_graphicdhgrcolor)
{
  HWND hwnd_group;
  RECT window_rect, client_rect, group_rect;
  int window_x, window_y;
  
  /** On va ajuster la fentre par rapport au Zoom demand **/  
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,&client_rect);
  window_graphicdhgrcolor->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select */
  hwnd_group = GetDlgItem(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,IDC_GROUP_GRAPHICDHGRCOLOR);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_GRAPHICDHGRCOLOR */
  GetWindowRect(hwnd_group,&group_rect);
  window_graphicdhgrcolor->group_data_x = (group_rect.left - window_rect.left) - 3;
  window_graphicdhgrcolor->group_data_y = ((group_rect.top - window_rect.top) - window_graphicdhgrcolor->title_height) + 3;

  /* Position de la Zone BipMap dans la fenetre */
  window_graphicdhgrcolor->bitmap_graphicdhgrcolor_x = window_graphicdhgrcolor->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_graphicdhgrcolor->bitmap_graphicdhgrcolor_y = window_graphicdhgrcolor->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_graphicdhgrcolor->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_graphicdhgrcolor->bitmap_graphicdhgrcolor_width*window_graphicdhgrcolor->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_graphicdhgrcolor->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicdhgrcolor->bitmap_graphicdhgrcolor_height*window_graphicdhgrcolor->zoom_level + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_graphicdhgrcolor->group_data_x,window_graphicdhgrcolor->group_data_y,window_graphicdhgrcolor->group_data_width,window_graphicdhgrcolor->group_data_height,TRUE);

  /* Largeur de la fentre */
  window_graphicdhgrcolor->window_width = window_graphicdhgrcolor->group_data_x + window_graphicdhgrcolor->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* On doit garantir une largeur minimale de la fentre */
  if(window_graphicdhgrcolor->window_width < GRAPHICDHGRCOLOR_DIALOGBOX_WIDTH)
    window_graphicdhgrcolor->window_width = GRAPHICDHGRCOLOR_DIALOGBOX_WIDTH;

  /* Hauteur de la fentre */
  window_graphicdhgrcolor->window_height = window_graphicdhgrcolor->title_height + window_graphicdhgrcolor->group_data_y + window_graphicdhgrcolor->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Resize window */
  MoveWindow(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,window_x,window_y,window_graphicdhgrcolor->window_width,window_graphicdhgrcolor->window_height,TRUE);
}


/******************************************************************************************************************/
/*  BuildGraphicDHGRColorFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/******************************************************************************************************************/
static void BuildGraphicDHGRColorFromData(struct window_graphicdhgrcolor *window_graphicdhgrcolor)
{
  unsigned char byte0_bank0;
  unsigned char byte1_bank0;
  unsigned char byte0_bank1;
  unsigned char byte1_bank1;
  int x, y, color_index, offset_bitmap;
  char buffer_title[256];
  DWORD tab_rgb_color[16] = 
    {
      /* 16 couleurs pour le mode DHGR */
      MAKECOLOR(0,0,0),     MAKECOLOR(0,0,153),     MAKECOLOR(0,119,0),   MAKECOLOR(34,34,255),
      MAKECOLOR(136,85,34), MAKECOLOR(170,170,170), MAKECOLOR(0,221,0),   MAKECOLOR(0,255,153), 
      MAKECOLOR(221,0,51),  MAKECOLOR(221,0,221),   MAKECOLOR(85,85,85),  MAKECOLOR(102,170,255), 
      MAKECOLOR(255,102,0), MAKECOLOR(255,153,136), MAKECOLOR(255,255,0), MAKECOLOR(255,255,255)
    };
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rien  faire */
  if(window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor == NULL)
    return;    

  /** Cration de la bitmap (2*140)*192 correspondant au 140*192 du DHGR Color **/
  offset_bitmap = 0;
  for(y=0; y<192; y++)      /* 192-(4*8) */
    for(x=0; x<80; x+=4)
      {
        /* 1 pixel = 4 bit, 4 Bytes = 7 pixels */
        byte0_bank0 = window_graphicdhgrcolor->graphicdhgrcolor_data[x + 1 + y*80];
        byte0_bank1 = window_graphicdhgrcolor->graphicdhgrcolor_data[x + 0 + y*80];
        byte1_bank0 = window_graphicdhgrcolor->graphicdhgrcolor_data[x + 3 + y*80];
        byte1_bank1 = window_graphicdhgrcolor->graphicdhgrcolor_data[x + 2 + y*80];
        
        /** Codage des 7 points **/
        /* Pixel 0 */
        color_index = (int) (byte0_bank1 & 0x0F);
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;              
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;   
        /* Pixel 1 */
        color_index = (int) (((byte0_bank1 >> 4) & 0x07) | ((byte0_bank0 << 3) & 0x08));
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;
        /* Pixel 2 */
        color_index = (int) (((byte0_bank0 >> 1) & 0x0F));
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;
        /* Pixel 3 */
        color_index = (int) (((byte0_bank0 >> 5) & 0x03) | ((byte1_bank1 << 2) & 0x0C));
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;
        /* Pixel 4 */
        color_index = (int) ((byte1_bank1 >> 2) & 0x0F);
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;
        /* Pixel 5 */
        color_index = (int) (((byte1_bank1 >> 6) & 0x01) | ((byte1_bank0 << 1)& 0x0E));
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;
        /* Pixel 6 */
        color_index = (int) ((byte1_bank0 >> 3) & 0x0F);
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;  
        window_graphicdhgrcolor->bitmap_data_graphicdhgrcolor[offset_bitmap] = tab_rgb_color[color_index];
        offset_bitmap++;
      }

  /* Titre de la fentre */
  sprintf(buffer_title,"Graphic DHGR Color : %02X/%04X",window_graphicdhgrcolor->graphicdhgrcolor_bank,window_graphicdhgrcolor->graphicdhgrcolor_address);
  SetWindowText(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,buffer_title);
}


/******************************************************************************/
/*  DeleteGraphicDHGRColorChildWindows() :  Destruction des fentres enfants. */
/******************************************************************************/
static void DeleteGraphicDHGRColorChildWindows(struct window_graphicdhgrcolor *window_graphicdhgrcolor)
{
  DeleteObject(window_graphicdhgrcolor->hBitmap_graphicdhgrcolor);
  ReleaseDC(window_graphicdhgrcolor->hwnd_graphicdhgrcolor,window_graphicdhgrcolor->hDC);
  DeleteDC(window_graphicdhgrcolor->hCompatibleDC);
}


/********************************************************************************/
/*  mem_free_window_graphicdhgrcolor() :  Libration mmoire pour la structure. */
/********************************************************************************/
void mem_free_window_graphicdhgrcolor(struct window_graphicdhgrcolor *window_graphicdhgrcolor)
{
  if(window_graphicdhgrcolor)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicdhgrcolor->current_notification,NULL);
      window_graphicdhgrcolor->current_notification = NULL;
      
      free(window_graphicdhgrcolor);
    }
}

/*******************************************************************************/
