/************************************************************************************************/
/*                                                                                              */
/*  Window_GraphicDHGRColorText80.c :  Module de gestion de la fentre GraphicDHGRColorText80.  */
/*                                                                                              */
/************************************************************************************************/
/*    Auteur : Olivier ZARDINI       *       Brutal Deluxe       *       Date :  Aout 2018      */
/************************************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_GraphicDHGRColorText80.h"

static DWORD BuildGraphicDHGRColorText80WindowThread(void *);
LRESULT CALLBACK ProcHandlerGraphicDHGRColorText80Window(HWND,UINT,WPARAM,LPARAM);
static int CreateGraphicDHGRColorText80ChildWindows(HWND,struct window_graphicdhgrcolortext80 *);
LRESULT CALLBACK subEditGraphicDHGRColorText80ProcControl(HWND,UINT,WPARAM,LPARAM);
void UpdateGraphicDHGRColorText80ClientAllArea(HDC,struct window_graphicdhgrcolortext80 *);
static void UpdateGraphicDHGRColorText80ClientBitmapArea(HDC,struct window_graphicdhgrcolortext80 *);
static void UpdateGraphicDHGRColorText80ContextInformation(struct window_graphicdhgrcolortext80 *,int,int);
static void UpdateGraphicDHGRColorText80AddressGraphic(struct window_graphicdhgrcolortext80 *,int);
static void UpdateGraphicDHGRColorText80AddressText(struct window_graphicdhgrcolortext80 *,int);
static void UpdateGraphicDHGRColorText80RefreshMode(struct window_graphicdhgrcolortext80 *,int);
static void UpdateGraphicDHGRColorText80ZoomLevel(struct window_graphicdhgrcolortext80 *,int);
static void RefreshGraphicDHGRColorText80WindowData(struct window_graphicdhgrcolortext80 *,int);
static int IsGraphicDHGRColorText80DataModified(int,int,int,int,struct window_graphicdhgrcolortext80 *);
static void ResizeGraphicDHGRColorText80Window(struct window_graphicdhgrcolortext80 *);
static void BuildGraphicDHGRColorText80FromData(struct window_graphicdhgrcolortext80 *);
static void DeleteGraphicDHGRColorText80ChildWindows(struct window_graphicdhgrcolortext80 *);


/**********************************************************************************************************/
/*  BuildGraphicDHGRColorText80Window() :  Construction / Affichage de la fentre GraphicDHGRColorText80. */
/**********************************************************************************************************/
struct window_graphicdhgrcolortext80 *BuildGraphicDHGRColorText80Window(HINSTANCE hInstance, int menu_id)
{
  struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80;

  /** Allocation de la structure window_graphicdhgrcolortext80 **/
  window_graphicdhgrcolortext80 = (struct window_graphicdhgrcolortext80 *) calloc(1,sizeof(struct window_graphicdhgrcolortext80));
  if(window_graphicdhgrcolortext80 == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_GRAPHICDHGRCOLORTEXT80,window_graphicdhgrcolortext80,NULL);

  /* Lance un thread pour grer la fentre GraphicDHGRColorText80 */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildGraphicDHGRColorText80WindowThread,window_graphicdhgrcolortext80,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_graphicdhgrcolortext80);
}


/********************************************************************************************************************/
/*  BuildGraphicDHGRColorText80WindowThread() :  Cration de la fentre GraphicDHGRColorText80 via un Thread ddi. */
/********************************************************************************************************************/
static DWORD BuildGraphicDHGRColorText80WindowThread(void *window_graphicdhgrcolortext80_param)
{
  HWND hwndGraphicDHGRColorText80;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_graphicdhgrcolortext80 = (struct window_graphicdhgrcolortext80 *) window_graphicdhgrcolortext80_param;
  window_graphicdhgrcolortext80->thread_id = GetCurrentThreadId();         /* Le ThreaID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre DHGR Color Text Page 80 Column **/
  hwndGraphicDHGRColorText80 = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_GRAPHICDHGRCOLORTEXT80),window_main->hwnd_main,(DLGPROC)ProcHandlerGraphicDHGRColorText80Window);
  if(hwndGraphicDHGRColorText80 == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndGraphicDHGRColorText80,SW_SHOWDEFAULT);
  UpdateWindow(hwndGraphicDHGRColorText80);

  /** Message Loop pour toutes les fentres de ce Thread (fentre GraphicDHGRColorText80 + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***************************************************************************************/
/*  ProcHandlerGraphicDHGRColorText80Window() :  Gestion des vnements de la fentre. */
/***************************************************************************************/
LRESULT CALLBACK ProcHandlerGraphicDHGRColorText80Window(HWND hwndGraphicDHGRColorText80, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;
  LRESULT result;
  WORD param_wd;
  int mouse_x, mouse_y;
  struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_GRAPHICDHGRCOLORTEXT80,&window_graphicdhgrcolortext80,NULL);
  if(window_graphicdhgrcolortext80 == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateGraphicDHGRColorText80ChildWindows(hwndGraphicDHGRColorText80,window_graphicdhgrcolortext80);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        UpdateGraphicDHGRColorText80ContextInformation(window_graphicdhgrcolortext80,mouse_x,mouse_y);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_graphicdhgrcolortext80->hBitmap_graphicdhgrcolortext80 != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndGraphicDHGRColorText80,&ps);

            /** MAJ Bitmap **/
            UpdateGraphicDHGRColorText80ClientAllArea(hDC,window_graphicdhgrcolortext80);

            /* End Paint */
            EndPaint(hwndGraphicDHGRColorText80,&ps);
          }
        break;
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdhgrcolortext80->refresh_mode != GRAPHICDHGRCOLORTEXT80_REFRESH_AUTOMATIC)
                UpdateGraphicDHGRColorText80RefreshMode(window_graphicdhgrcolortext80,GRAPHICDHGRCOLORTEXT80_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdhgrcolortext80->refresh_mode != GRAPHICDHGRCOLORTEXT80_REFRESH_MANUAL)
                UpdateGraphicDHGRColorText80RefreshMode(window_graphicdhgrcolortext80,GRAPHICDHGRCOLORTEXT80_REFRESH_MANUAL);
              break;

            /** Bouton Radio : On change le Zoom de la fentre **/
            case IDC_RADIO_ZOOM_X1 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdhgrcolortext80->zoom_level != GRAPHICDHGRCOLORTEXT80_ZOOM_X1)
                UpdateGraphicDHGRColorText80ZoomLevel(window_graphicdhgrcolortext80,GRAPHICDHGRCOLORTEXT80_ZOOM_X1);
              break;

            case IDC_RADIO_ZOOM_X2 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdhgrcolortext80->zoom_level != GRAPHICDHGRCOLORTEXT80_ZOOM_X2)
                UpdateGraphicDHGRColorText80ZoomLevel(window_graphicdhgrcolortext80,GRAPHICDHGRCOLORTEXT80_ZOOM_X2);
              break;

            /** Bouton de dplacement de l'adresse Graphic **/
            case IDC_BUTTON_LEFT_GRAPHIC :
              UpdateGraphicDHGRColorText80AddressGraphic(window_graphicdhgrcolortext80,-1);
              break;

            case IDC_BUTTON_RIGHT_GRAPHIC :
              UpdateGraphicDHGRColorText80AddressGraphic(window_graphicdhgrcolortext80,1);
              break;

            case IDC_BUTTON_UP_GRAPHIC :
              UpdateGraphicDHGRColorText80AddressGraphic(window_graphicdhgrcolortext80,-40);
              break;

            case IDC_BUTTON_DOWN_GRAPHIC :
              UpdateGraphicDHGRColorText80AddressGraphic(window_graphicdhgrcolortext80,40);
              break;

            case IDC_BUTTON_PAGEUP_GRAPHIC :
              UpdateGraphicDHGRColorText80AddressGraphic(window_graphicdhgrcolortext80,-8192);
              break;

            case IDC_BUTTON_PAGEDOWN_GRAPHIC :
              UpdateGraphicDHGRColorText80AddressGraphic(window_graphicdhgrcolortext80,8192);
              break;

            /** Bouton de dplacement de l'adresse Text **/
            case IDC_BUTTON_LEFT_TEXT :
              UpdateGraphicDHGRColorText80AddressText(window_graphicdhgrcolortext80,-1);
              break;

            case IDC_BUTTON_RIGHT_TEXT :
              UpdateGraphicDHGRColorText80AddressText(window_graphicdhgrcolortext80,1);
              break;

            case IDC_BUTTON_UP_TEXT :
              UpdateGraphicDHGRColorText80AddressText(window_graphicdhgrcolortext80,-40);
              break;

            case IDC_BUTTON_DOWN_TEXT :
              UpdateGraphicDHGRColorText80AddressText(window_graphicdhgrcolortext80,40);
              break;

            case IDC_BUTTON_PAGEUP_TEXT :
              UpdateGraphicDHGRColorText80AddressText(window_graphicdhgrcolortext80,-1024);
              break;

            case IDC_BUTTON_PAGEDOWN_TEXT :
              UpdateGraphicDHGRColorText80AddressText(window_graphicdhgrcolortext80,1024);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshGraphicDHGRColorText80WindowData(window_graphicdhgrcolortext80,1);
              break;
              
            /** Slection dans la ComboBox **/
            case IDC_COMBOBOX_GRAPHICDHGRCOLORTEXT80_COLOR_TEXT :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphicdhgrcolortext80->hwnd_combo_color_text,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphicdhgrcolortext80->color_text)
                      {
                        window_graphicdhgrcolortext80->color_text = (int) result;
                        RefreshGraphicDHGRColorText80WindowData(window_graphicdhgrcolortext80,1);
                      }
                }
              break;              
             
            case IDC_COMBOBOX_GRAPHICDHGRCOLORTEXT80_COLOR_BACKGROUND :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphicdhgrcolortext80->hwnd_combo_color_background,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphicdhgrcolortext80->color_background)
                      {
                        window_graphicdhgrcolortext80->color_background = (int) result;
                        RefreshGraphicDHGRColorText80WindowData(window_graphicdhgrcolortext80,1);
                      }
                }
              break; 
             
            case IDC_COMBOBOX_GRAPHICDHGRCOLORTEXT80_DISPLAY_LANGUAGE :
              param_wd = HIWORD(wParam);
              if(param_wd == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphicdhgrcolortext80->hwnd_combo_display_language,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphicdhgrcolortext80->display_language)
                      {
                        window_graphicdhgrcolortext80->display_language = (int) result;
                        RefreshGraphicDHGRColorText80WindowData(window_graphicdhgrcolortext80,1);
                      }
                }
              break;                           
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_GRAPHICDHGRCOLORTEXT80_REFRESHWINDOW :
        RefreshGraphicDHGRColorText80WindowData(window_graphicdhgrcolortext80,0);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre GraphicDHGRColorText80 */
        DestroyWindow(hwndGraphicDHGRColorText80);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteGraphicDHGRColorText80ChildWindows(window_graphicdhgrcolortext80);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_GRAPHICDHGRCOLORTEXT80,window_graphicdhgrcolortext80,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************************************/
/*  CreateGraphicDHGRColorText80ChildWindows() : Crer les fentres enfants et remplir la structure window_graphicdhgrcolortext80. */
/***********************************************************************************************************************************/
static int CreateGraphicDHGRColorText80ChildWindows(HWND hwndGraphicDHGRColorText80, struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80)
{
  BITMAPINFO bitmap_info;
  int i;
  char buffer[BUFFER_SIZE];
  char *color_name[16] = {"Black","Deep Red","Dark Blue","Purple","Dark Green","Dark Gray","Medium Blue","Light Blue","Brown","Orange","Light Gray","Pink","Light Green","Yellow","Aquamarine","White"};
  char *display_language[9] = {"U.S.A.","U.K.","French","Danish","Spanish","Italian","German","Swedish","MouseText"};
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80 = hwndGraphicDHGRColorText80;

  /** Device Context **/
  window_graphicdhgrcolortext80->hDC = GetDC(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80);
  window_graphicdhgrcolortext80->hCompatibleDC = CreateCompatibleDC(window_graphicdhgrcolortext80->hDC);

  /** Surchage des Control Edit pour capter les vnements **/
  window_graphicdhgrcolortext80->oldEditProcBankGraphic    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDHGRColorText80,IDC_EDIT_BANK_GRAPHIC),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDHGRColorText80ProcControl);
  window_graphicdhgrcolortext80->oldEditProcAddressGraphic = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDHGRColorText80,IDC_EDIT_ADDRESS_GRAPHIC),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDHGRColorText80ProcControl);
  window_graphicdhgrcolortext80->oldEditProcBankText       = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDHGRColorText80,IDC_EDIT_BANK_TEXT),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDHGRColorText80ProcControl);
  window_graphicdhgrcolortext80->oldEditProcAddressText    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDHGRColorText80,IDC_EDIT_ADDRESS_TEXT),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDHGRColorText80ProcControl);

  /** Ajoute les images sur les boutons **/
  /* Graphic */
  SendMessage(GetDlgItem(hwndGraphicDHGRColorText80,IDC_BUTTON_LEFT_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColorText80,IDC_BUTTON_RIGHT_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColorText80,IDC_BUTTON_UP_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColorText80,IDC_BUTTON_DOWN_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColorText80,IDC_BUTTON_PAGEUP_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColorText80,IDC_BUTTON_PAGEDOWN_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));
  /* Text */
  SendMessage(GetDlgItem(hwndGraphicDHGRColorText80,IDC_BUTTON_LEFT_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColorText80,IDC_BUTTON_RIGHT_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColorText80,IDC_BUTTON_UP_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColorText80,IDC_BUTTON_DOWN_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColorText80,IDC_BUTTON_PAGEUP_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicDHGRColorText80,IDC_BUTTON_PAGEDOWN_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Cration des ComboBox **/
  /* Text Color */
  window_graphicdhgrcolortext80->hwnd_combo_color_text = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                        99,102,100,250,hwndGraphicDHGRColorText80,(HMENU)IDC_COMBOBOX_GRAPHICDHGRCOLORTEXT80_COLOR_TEXT,param->hInstance,0);
  for(i=0; i<16; i++)
    SendMessage(window_graphicdhgrcolortext80->hwnd_combo_color_text,CB_ADDSTRING,0,(LPARAM)color_name[i]);

  /* Background Color */
  window_graphicdhgrcolortext80->hwnd_combo_color_background = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                                              304,102,100,250,hwndGraphicDHGRColorText80,(HMENU)IDC_COMBOBOX_GRAPHICDHGRCOLORTEXT80_COLOR_BACKGROUND,param->hInstance,0);
  for(i=0; i<16; i++)
    SendMessage(window_graphicdhgrcolortext80->hwnd_combo_color_background,CB_ADDSTRING,0,(LPARAM)color_name[i]);

  /* Display Language */
  window_graphicdhgrcolortext80->hwnd_combo_display_language = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                                              304,134,100,220,hwndGraphicDHGRColorText80,(HMENU)IDC_COMBOBOX_GRAPHICDHGRCOLORTEXT80_DISPLAY_LANGUAGE,param->hInstance,0);
  for(i=0; i<9; i++)
    SendMessage(window_graphicdhgrcolortext80->hwnd_combo_display_language,CB_ADDSTRING,0,(LPARAM)display_language[i]);

  /* 1res valeurs */
  window_graphicdhgrcolortext80->color_text = COLOR_IIGS_WHITE;
  window_graphicdhgrcolortext80->color_background = COLOR_IIGS_MEDIUM_BLUE;
  window_graphicdhgrcolortext80->display_language = LANGUAGE_IIGS_USA;
  SendMessage(window_graphicdhgrcolortext80->hwnd_combo_color_text,CB_SETCURSEL,window_graphicdhgrcolortext80->color_text,0);
  SendMessage(window_graphicdhgrcolortext80->hwnd_combo_color_background,CB_SETCURSEL,window_graphicdhgrcolortext80->color_background,0);
  SendMessage(window_graphicdhgrcolortext80->hwnd_combo_display_language,CB_SETCURSEL,window_graphicdhgrcolortext80->display_language,0);

  /** Cration de la Police de caractre + Affectation de la police au Control Edit **/
  window_graphicdhgrcolortext80->hFont = CreateFont(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");
  SendMessage(window_graphicdhgrcolortext80->hwnd_combo_color_text,WM_SETFONT,(WPARAM)window_graphicdhgrcolortext80->hFont,TRUE);
  SendMessage(window_graphicdhgrcolortext80->hwnd_combo_color_background,WM_SETFONT,(WPARAM)window_graphicdhgrcolortext80->hFont,TRUE);
  SendMessage(window_graphicdhgrcolortext80->hwnd_combo_display_language,WM_SETFONT,(WPARAM)window_graphicdhgrcolortext80->hFont,TRUE);

  /** Adresse de la Zone de Data **/
  /* Graphic */
  window_graphicdhgrcolortext80->graphicdhgr_bank = 0x00;
  window_graphicdhgrcolortext80->graphicdhgr_address = 0x2000; 
  sprintf(buffer,"%02X",window_graphicdhgrcolortext80->graphicdhgr_bank);
  SetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_BANK_GRAPHIC,buffer);
  sprintf(buffer,"%04X",window_graphicdhgrcolortext80->graphicdhgr_address);
  SetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_ADDRESS_GRAPHIC,buffer);
  /* Text */
  window_graphicdhgrcolortext80->textpage80_bank = 0x00;
  window_graphicdhgrcolortext80->textpage80_address = 0x0400; 
  sprintf(buffer,"%02X",window_graphicdhgrcolortext80->textpage80_bank);
  SetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_BANK_TEXT,buffer);
  sprintf(buffer,"%04X",window_graphicdhgrcolortext80->textpage80_address);
  SetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_ADDRESS_TEXT,buffer);

  /* Refresh Mode : Automatic */
  UpdateGraphicDHGRColorText80RefreshMode(window_graphicdhgrcolortext80,GRAPHICDHGRCOLORTEXT80_REFRESH_AUTOMATIC);

  /* Zoom Level : 1 */
  window_graphicdhgrcolortext80->zoom_level = GRAPHICDHGRCOLORTEXT80_ZOOM_X1;
  SendDlgItemMessage(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);

  /* Dimension en Pixel de la Bitmap GraphicDHGRColorText80 Data */
  window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_width = 560;
  window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_height = 192;
  
  /** Bitmap affichant la zone de slection **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicdhgrcolortext80->hBitmap_graphicdhgrcolortext80 = CreateDIBSection(window_graphicdhgrcolortext80->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80),NULL,0);
  for(i=0; i<window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_width*window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_height; i++)
    window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[i] = BITMAP_BACKGROUND_COLOR;

  /* Retaille la fentre */
  ResizeGraphicDHGRColorText80Window(window_graphicdhgrcolortext80);

  /** Construction de la Page Texte + Rafraichissement de la fentre **/
  RefreshGraphicDHGRColorText80WindowData(window_graphicdhgrcolortext80,1);
    
  /* OK */
  return(0);
}


/*************************************************************************************************************/
/*  subEditGraphicDHGRColorText80ProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*************************************************************************************************************/
LRESULT CALLBACK subEditGraphicDHGRColorText80ProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80;
  my_Memory(MEMORY_GET_WINDOW_GRAPHICDHGRCOLORTEXT80,&window_graphicdhgrcolortext80,NULL);

  /* Rien  faire */
  if(window_graphicdhgrcolortext80 == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_BANK_GRAPHIC) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_ADDRESS_GRAPHIC) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_BANK_TEXT) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_ADDRESS_TEXT) == hwnd_control)
    index = 4;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement du Return dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80);

      /* On force la MAJ de la fentre */
      RefreshGraphicDHGRColorText80WindowData(window_graphicdhgrcolortext80,1);
    }
  else if(index == 1)
    return(CallWindowProc(window_graphicdhgrcolortext80->oldEditProcBankGraphic,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_graphicdhgrcolortext80->oldEditProcAddressGraphic,hwnd_control,msg,wParam,lParam)); 
  else if(index == 3)
    return(CallWindowProc(window_graphicdhgrcolortext80->oldEditProcBankText,hwnd_control,msg,wParam,lParam)); 
  else if(index == 4)
    return(CallWindowProc(window_graphicdhgrcolortext80->oldEditProcAddressText,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
} 


/**************************************************************************************************************/
/*  UpdateGraphicDHGRColorText80ClientAllArea() :  Affichage des Bitmap de la fentre GraphicDHGRColorText80. */
/**************************************************************************************************************/
void UpdateGraphicDHGRColorText80ClientAllArea(HDC hDC, struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap **/
  UpdateGraphicDHGRColorText80ClientBitmapArea(hDC,window_graphicdhgrcolortext80);
}


/*****************************************************************************************************/
/*  UpdateGraphicDHGRColorText80ClientBitmapArea() :  Affichage de la Bitmap GraphicDHGRColorText80. */
/*****************************************************************************************************/
static void UpdateGraphicDHGRColorText80ClientBitmapArea(HDC hDC, struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec la zone mmoire et la zone slectionne **/
  bitmap_old = SelectObject(window_graphicdhgrcolortext80->hCompatibleDC,window_graphicdhgrcolortext80->hBitmap_graphicdhgrcolortext80);
  StretchBlt(hDC,window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_x,window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_y,
             window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_width*window_graphicdhgrcolortext80->zoom_level,
             window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_height*window_graphicdhgrcolortext80->zoom_level*2,  /* Double la hauteur */
             window_graphicdhgrcolortext80->hCompatibleDC,0,0,window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_width,window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_height,SRCCOPY);
  SelectObject(window_graphicdhgrcolortext80->hCompatibleDC,bitmap_old);
}


/**********************************************************************************************************************/
/*  UpdateGraphicDHGRColorText80AddressGraphic() :  Dplace l'adresse Graphic pour la fentre GraphicDHGRColorText80. */
/**********************************************************************************************************************/
static void UpdateGraphicDHGRColorText80AddressGraphic(struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80, int delta)
{
  int current_address, new_address, graphicdhgr_bank, graphicdhgr_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphicdhgrcolortext80->graphicdhgr_bank << 16) | window_graphicdhgrcolortext80->graphicdhgr_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  graphicdhgr_bank = (new_address >> 16) & 0xFF;
  graphicdhgr_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphicdhgr_bank);
  SetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_BANK_GRAPHIC,buffer);
  sprintf(buffer,"%04X",graphicdhgr_address);
  SetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_ADDRESS_GRAPHIC,buffer);

  /* MAJ de la fentre */
  RefreshGraphicDHGRColorText80WindowData(window_graphicdhgrcolortext80,1);
}


/*****************************************************************************************************************/
/*  UpdateGraphicDHGRColorText80AddressText() :  Dplace l'adresse Texte pour la fentre GraphicDHGRColorText80. */
/*****************************************************************************************************************/
static void UpdateGraphicDHGRColorText80AddressText(struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80, int delta)
{
  int current_address, new_address, textpage80_bank, textpage80_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphicdhgrcolortext80->textpage80_bank << 16) | window_graphicdhgrcolortext80->textpage80_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  textpage80_bank = (new_address >> 16) & 0xFF;
  textpage80_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",textpage80_bank);
  SetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_BANK_TEXT,buffer);
  sprintf(buffer,"%04X",textpage80_address);
  SetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_ADDRESS_TEXT,buffer);

  /* MAJ de la fentre */
  RefreshGraphicDHGRColorText80WindowData(window_graphicdhgrcolortext80,1);
}


/*******************************************************************************************************************************/
/*  UpdateGraphicDHGRColorText80ContextInformation() :  MAJ des informations de contexte de la fentre GraphicDHGRColorText80. */
/*******************************************************************************************************************************/
static void UpdateGraphicDHGRColorText80ContextInformation(struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80, int mouse_x, int mouse_y)
{
  int pixmap_x, pixmap_y, grid_x, grid_y, byte_group_x;
  int graphicdhgr_data_bank;
  int graphicdhgr_data_address;
  int textpage80_data_address;
  unsigned char textpage80_data;
  unsigned char byte0_bank0, byte0_bank1, byte1_bank0, byte1_bank1;
  int tab_dhgr_address[192] =     /* Adresse du dbut des 192 lignes de la Page DHGR */
    {
      0x0000+0x0000, 0x0000+0x0400, 0x0000+0x0800, 0x0000+0x0C00, 0x0000+0x1000, 0x0000+0x1400, 0x0000+0x1800, 0x0000+0x1C00, 
      0x0080+0x0000, 0x0080+0x0400, 0x0080+0x0800, 0x0080+0x0C00, 0x0080+0x1000, 0x0080+0x1400, 0x0080+0x1800, 0x0080+0x1C00, 
      0x0100+0x0000, 0x0100+0x0400, 0x0100+0x0800, 0x0100+0x0C00, 0x0100+0x1000, 0x0100+0x1400, 0x0100+0x1800, 0x0100+0x1C00, 
      0x0180+0x0000, 0x0180+0x0400, 0x0180+0x0800, 0x0180+0x0C00, 0x0180+0x1000, 0x0180+0x1400, 0x0180+0x1800, 0x0180+0x1C00, 
      0x0200+0x0000, 0x0200+0x0400, 0x0200+0x0800, 0x0200+0x0C00, 0x0200+0x1000, 0x0200+0x1400, 0x0200+0x1800, 0x0200+0x1C00, 
      0x0280+0x0000, 0x0280+0x0400, 0x0280+0x0800, 0x0280+0x0C00, 0x0280+0x1000, 0x0280+0x1400, 0x0280+0x1800, 0x0280+0x1C00, 
      0x0300+0x0000, 0x0300+0x0400, 0x0300+0x0800, 0x0300+0x0C00, 0x0300+0x1000, 0x0300+0x1400, 0x0300+0x1800, 0x0300+0x1C00, 
      0x0380+0x0000, 0x0380+0x0400, 0x0380+0x0800, 0x0380+0x0C00, 0x0380+0x1000, 0x0380+0x1400, 0x0380+0x1800, 0x0380+0x1C00,
      0x0028+0x0000, 0x0028+0x0400, 0x0028+0x0800, 0x0028+0x0C00, 0x0028+0x1000, 0x0028+0x1400, 0x0028+0x1800, 0x0028+0x1C00, 
      0x00A8+0x0000, 0x00A8+0x0400, 0x00A8+0x0800, 0x00A8+0x0C00, 0x00A8+0x1000, 0x00A8+0x1400, 0x00A8+0x1800, 0x00A8+0x1C00, 
      0x0128+0x0000, 0x0128+0x0400, 0x0128+0x0800, 0x0128+0x0C00, 0x0128+0x1000, 0x0128+0x1400, 0x0128+0x1800, 0x0128+0x1C00, 
      0x01A8+0x0000, 0x01A8+0x0400, 0x01A8+0x0800, 0x01A8+0x0C00, 0x01A8+0x1000, 0x01A8+0x1400, 0x01A8+0x1800, 0x01A8+0x1C00, 
      0x0228+0x0000, 0x0228+0x0400, 0x0228+0x0800, 0x0228+0x0C00, 0x0228+0x1000, 0x0228+0x1400, 0x0228+0x1800, 0x0228+0x1C00, 
      0x02A8+0x0000, 0x02A8+0x0400, 0x02A8+0x0800, 0x02A8+0x0C00, 0x02A8+0x1000, 0x02A8+0x1400, 0x02A8+0x1800, 0x02A8+0x1C00, 
      0x0328+0x0000, 0x0328+0x0400, 0x0328+0x0800, 0x0328+0x0C00, 0x0328+0x1000, 0x0328+0x1400, 0x0328+0x1800, 0x0328+0x1C00, 
      0x03A8+0x0000, 0x03A8+0x0400, 0x03A8+0x0800, 0x03A8+0x0C00, 0x03A8+0x1000, 0x03A8+0x1400, 0x03A8+0x1800, 0x03A8+0x1C00, 
      0x0050+0x0000, 0x0050+0x0400, 0x0050+0x0800, 0x0050+0x0C00, 0x0050+0x1000, 0x0050+0x1400, 0x0050+0x1800, 0x0050+0x1C00, 
      0x00D0+0x0000, 0x00D0+0x0400, 0x00D0+0x0800, 0x00D0+0x0C00, 0x00D0+0x1000, 0x00D0+0x1400, 0x00D0+0x1800, 0x00D0+0x1C00, 
      0x0150+0x0000, 0x0150+0x0400, 0x0150+0x0800, 0x0150+0x0C00, 0x0150+0x1000, 0x0150+0x1400, 0x0150+0x1800, 0x0150+0x1C00, 
      0x01D0+0x0000, 0x01D0+0x0400, 0x01D0+0x0800, 0x01D0+0x0C00, 0x01D0+0x1000, 0x01D0+0x1400, 0x01D0+0x1800, 0x01D0+0x1C00, 
      0x0250+0x0000, 0x0250+0x0400, 0x0250+0x0800, 0x0250+0x0C00, 0x0250+0x1000, 0x0250+0x1400, 0x0250+0x1800, 0x0250+0x1C00, 
      0x02D0+0x0000, 0x02D0+0x0400, 0x02D0+0x0800, 0x02D0+0x0C00, 0x02D0+0x1000, 0x02D0+0x1400, 0x02D0+0x1800, 0x02D0+0x1C00, 
      0x0350+0x0000, 0x0350+0x0400, 0x0350+0x0800, 0x0350+0x0C00, 0x0350+0x1000, 0x0350+0x1400, 0x0350+0x1800, 0x0350+0x1C00, 
      0x03D0+0x0000, 0x03D0+0x0400, 0x03D0+0x0800, 0x03D0+0x0C00, 0x03D0+0x1000, 0x03D0+0x1400, 0x03D0+0x1800, 0x03D0+0x1C00
    };
  int tab_text80_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };

  /* Rien  faire */
  if(window_graphicdhgrcolortext80 == NULL)
    return;

  /* Est t'on dans l'image ? */
  if(mouse_x < window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_x || mouse_x >= (window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_x + window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_width*window_graphicdhgrcolortext80->zoom_level) ||
     mouse_y < window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_y || mouse_y >= (window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_y + window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_height*window_graphicdhgrcolortext80->zoom_level*2))
    {
      SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT),"");
      SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT3),"");
      return;
    }

  /* Coordonnes dans la Pixmap */
  pixmap_x = (mouse_x - window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_x)/window_graphicdhgrcolortext80->zoom_level;
  pixmap_y = (mouse_y - window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_y)/window_graphicdhgrcolortext80->zoom_level;

  /* Graphic */
  if(pixmap_y < 160*2)   /* 192 - 8*4 */
    {
      /* Coordonnes dans la grille 140x192 */
      grid_x = pixmap_x/4;
      byte_group_x = grid_x/7;
      grid_y = pixmap_y/2;

      /* 1 pixel = 4 bit, 4 Bytes = 7 pixels */
      byte0_bank0 = window_graphicdhgrcolortext80->graphicdhgr_data[byte_group_x + 1 + grid_y*80];
      byte0_bank1 = window_graphicdhgrcolortext80->graphicdhgr_data[byte_group_x + 0 + grid_y*80];
      byte1_bank0 = window_graphicdhgrcolortext80->graphicdhgr_data[byte_group_x + 3 + grid_y*80];
      byte1_bank1 = window_graphicdhgrcolortext80->graphicdhgr_data[byte_group_x + 2 + grid_y*80];

      /* Adresse */
      graphicdhgr_data_address = (window_graphicdhgrcolortext80->graphicdhgr_bank << 16) + window_graphicdhgrcolortext80->graphicdhgr_address;
      graphicdhgr_data_address += tab_dhgr_address[grid_y];
      graphicdhgr_data_address += byte_group_x;
      graphicdhgr_data_bank = graphicdhgr_data_address>>16;

      /** Message **/
      if((grid_x % 7) == 0)
        {
          /* Pixel 0 : byte0_bank1 */
          sprintf(window_graphicdhgrcolortext80->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgr_data_bank+1,graphicdhgr_data_address&0xFFFF,byte0_bank1,grid_x,grid_y);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT),window_graphicdhgrcolortext80->buffer_log);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT3),"");
        }
      else if((grid_x % 7) == 1)
        {
          /* Pixel 1 : byte0_bank1 + byte0_bank0 */
          sprintf(window_graphicdhgrcolortext80->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgr_data_bank+1,graphicdhgr_data_address&0xFFFF,byte0_bank1,grid_x,grid_y);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT),window_graphicdhgrcolortext80->buffer_log);  
          sprintf(window_graphicdhgrcolortext80->buffer_log,"%02X/%04X : %02X",graphicdhgr_data_bank,graphicdhgr_data_address&0xFFFF,byte0_bank0);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT3),window_graphicdhgrcolortext80->buffer_log); 
        }
      else if((grid_x % 7) == 2)
        {
          /* Pixel 2 : byte0_bank0 */
          sprintf(window_graphicdhgrcolortext80->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgr_data_bank,graphicdhgr_data_address&0xFFFF,byte0_bank0,grid_x,grid_y);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT),window_graphicdhgrcolortext80->buffer_log);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT3),"");
        }
      else if((grid_x % 7) == 3)
        {
          /* Pixel 3 : byte0_bank0 + byte1_bank1 */
          sprintf(window_graphicdhgrcolortext80->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgr_data_bank,graphicdhgr_data_address&0xFFFF,byte0_bank0,grid_x,grid_y);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT),window_graphicdhgrcolortext80->buffer_log);
          sprintf(window_graphicdhgrcolortext80->buffer_log,"%02X/%04X : %02X",graphicdhgr_data_bank+1,(graphicdhgr_data_address+1)&0xFFFF,byte1_bank1);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT3),window_graphicdhgrcolortext80->buffer_log);
        }
      else if((grid_x % 7) == 4)
        {
          /* Pixel 4 : byte1_bank1 */
          sprintf(window_graphicdhgrcolortext80->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgr_data_bank+1,(graphicdhgr_data_address+1)&0xFFFF,byte1_bank1,grid_x,grid_y);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT),window_graphicdhgrcolortext80->buffer_log);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT3),"");
        }
      else if((grid_x % 7) == 5)
        {
          /* Pixel 5 : byte1_bank1 + byte1_bank0 */
          sprintf(window_graphicdhgrcolortext80->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgr_data_bank+1,(graphicdhgr_data_address+1)&0xFFFF,byte1_bank1,grid_x,grid_y);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT),window_graphicdhgrcolortext80->buffer_log);
          sprintf(window_graphicdhgrcolortext80->buffer_log,"%02X/%04X : %02X",graphicdhgr_data_bank,(graphicdhgr_data_address+1)&0xFFFF,byte1_bank0);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT3),window_graphicdhgrcolortext80->buffer_log);
        }
      else
        {
          /* Pixel 6 : byte1_bank0 */
          sprintf(window_graphicdhgrcolortext80->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgr_data_bank,(graphicdhgr_data_address+1)&0xFFFF,byte1_bank0,grid_x,grid_y);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT),window_graphicdhgrcolortext80->buffer_log);
          SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT3),"");
        }
    }
  else   /* Text */
    {
      /* Coordonnes dans la grille 80x24 */
      grid_x = pixmap_x/7;
      grid_y = pixmap_y/(8*2);

      /* Valeur */
      textpage80_data = window_graphicdhgrcolortext80->textpage80_data[grid_x + grid_y*80];

      /* Adresse */
      textpage80_data_address = ((window_graphicdhgrcolortext80->textpage80_bank+(((grid_x%2)==0)?1:0)) << 16) + window_graphicdhgrcolortext80->textpage80_address;
      textpage80_data_address += (grid_x/2);
      textpage80_data_address += tab_text80_address[grid_y];

      /* Message */
      sprintf(window_graphicdhgrcolortext80->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",textpage80_data_address>>16,textpage80_data_address&0xFFFF,textpage80_data,grid_x,grid_y);
      SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT),window_graphicdhgrcolortext80->buffer_log);
      SetWindowText(GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_STATIC_CONTEXT3),"");
    }
}


/************************************************************************************************************************/
/*  UpdateGraphicDHGRColorText80RefreshMode() :  MAJ du mode de rafraichissement de la fenetre GraphicDHGRColorText80. */
/************************************************************************************************************************/
static void UpdateGraphicDHGRColorText80RefreshMode(struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == GRAPHICDHGRCOLORTEXT80_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_graphicdhgrcolortext80->refresh_mode = GRAPHICDHGRCOLORTEXT80_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80;
      current_notification->message = HM_GRAPHICDHGRCOLORTEXT80_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_GRAPHICDHGRCOLORTEXT80;

      /* Ajout de la notification */
      window_graphicdhgrcolortext80->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_graphicdhgrcolortext80->refresh_mode = GRAPHICDHGRCOLORTEXT80_REFRESH_MANUAL;
      SendDlgItemMessage(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicdhgrcolortext80->current_notification,NULL);
      window_graphicdhgrcolortext80->current_notification = NULL;
    }
}


/***********************************************************************************************************/
/*  UpdateGraphicDHGRColorText80ZoomLevel() :  MAJ du niveau de Zoom de la fentre GraphicDHGRColorText80. */
/***********************************************************************************************************/
static void UpdateGraphicDHGRColorText80ZoomLevel(struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80, int zoom_level)
{
  if(zoom_level == GRAPHICDHGRCOLORTEXT80_ZOOM_X1)
    {
      /* Visuel */
      window_graphicdhgrcolortext80->zoom_level = GRAPHICDHGRCOLORTEXT80_ZOOM_X1;
      SendDlgItemMessage(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICDHGRCOLORTEXT80_ZOOM_X2)
    {
      /* Visuel */
      window_graphicdhgrcolortext80->zoom_level = GRAPHICDHGRCOLORTEXT80_ZOOM_X2;
      SendDlgItemMessage(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_RADIO_ZOOM_X2,BM_SETCHECK,1,0);
    }
    
  /* Retaille la fentre */
  ResizeGraphicDHGRColorText80Window(window_graphicdhgrcolortext80);  
  
  /* On force la MAJ de la fentre */  
  RefreshGraphicDHGRColorText80WindowData(window_graphicdhgrcolortext80,1);    
}


/******************************************************************************************************/
/*  RefreshGraphicDHGRColorText80WindowData() :  MAJ du contenu de la fentre GraphicDHGRColorText80. */
/******************************************************************************************************/
static void RefreshGraphicDHGRColorText80WindowData(struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80, int force)
{
  int length, is_modified;
  int graphicdhgr_bank, graphicdhgr_address, textpage80_bank, textpage80_address;
  char buffer[1024];

  /** Rcupre la valeur du GraphicDHGRColorText80 Pointer **/
  /* Graphic */
  length = GetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_BANK_GRAPHIC,buffer,1024);
  sscanf(buffer,"%X",&graphicdhgr_bank);
  length = GetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_ADDRESS_GRAPHIC,buffer,1024);
  sscanf(buffer,"%X",&graphicdhgr_address);
  /* Text */
  length = GetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_BANK_TEXT,buffer,1024);
  sscanf(buffer,"%X",&textpage80_bank);
  length = GetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_ADDRESS_TEXT,buffer,1024);
  sscanf(buffer,"%X",&textpage80_address);

  /** Replace les valeurs dans les Control pour montrer ce qui a t compris **/
  /* Graphic */
  sprintf(buffer,"%02X",graphicdhgr_bank);
  SetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_BANK_GRAPHIC,buffer);
  sprintf(buffer,"%04X",graphicdhgr_address);
  SetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_ADDRESS_GRAPHIC,buffer);
  /* Text */
  sprintf(buffer,"%02X",textpage80_bank);
  SetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_BANK_TEXT,buffer);
  sprintf(buffer,"%04X",textpage80_address);
  SetDlgItemText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_EDIT_ADDRESS_TEXT,buffer);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsGraphicDHGRColorText80DataModified(graphicdhgr_bank,graphicdhgr_address,textpage80_bank,textpage80_address,window_graphicdhgrcolortext80);
  if(is_modified == 0 && force == 0)
    return;

  /** Cration de la Zone Text Graphique  partir des nouvelles donnes **/
  BuildGraphicDHGRColorText80FromData(window_graphicdhgrcolortext80);

  /* Rafraichit la fentre */
  InvalidateRect(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,NULL,TRUE);
  UpdateWindow(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80);
}


/************************************************************************************************************************************/
/*  IsGraphicDHGRColorText80DataModified() :  Les donnes GraphicDHGRColorText80 ont t'elles t modifies depuis la dernire fois. */
/************************************************************************************************************************************/
static int IsGraphicDHGRColorText80DataModified(int graphicdgr_bank, int graphicdgr_address, int textpage80_bank, int textpage80_address, struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80)
{
  int x, y, graphicdhgr_data_address, textpage80_data_address;
  unsigned char graphicdhgr_raw_data_main[8192];
  unsigned char graphicdhgr_raw_data_aux[8192];
  unsigned char textpage80_raw_data_main[1024];
  unsigned char textpage80_raw_data_aux[1024];
  unsigned char graphicdhgr_data[80*192];
  unsigned char textpage80_data[80*24];
  int tab_dhgr_address[192] =     /* Adresse du dbut des 192 lignes de la Page DHGR */
    {
      0x0000+0x0000, 0x0000+0x0400, 0x0000+0x0800, 0x0000+0x0C00, 0x0000+0x1000, 0x0000+0x1400, 0x0000+0x1800, 0x0000+0x1C00, 
      0x0080+0x0000, 0x0080+0x0400, 0x0080+0x0800, 0x0080+0x0C00, 0x0080+0x1000, 0x0080+0x1400, 0x0080+0x1800, 0x0080+0x1C00, 
      0x0100+0x0000, 0x0100+0x0400, 0x0100+0x0800, 0x0100+0x0C00, 0x0100+0x1000, 0x0100+0x1400, 0x0100+0x1800, 0x0100+0x1C00, 
      0x0180+0x0000, 0x0180+0x0400, 0x0180+0x0800, 0x0180+0x0C00, 0x0180+0x1000, 0x0180+0x1400, 0x0180+0x1800, 0x0180+0x1C00, 
      0x0200+0x0000, 0x0200+0x0400, 0x0200+0x0800, 0x0200+0x0C00, 0x0200+0x1000, 0x0200+0x1400, 0x0200+0x1800, 0x0200+0x1C00, 
      0x0280+0x0000, 0x0280+0x0400, 0x0280+0x0800, 0x0280+0x0C00, 0x0280+0x1000, 0x0280+0x1400, 0x0280+0x1800, 0x0280+0x1C00, 
      0x0300+0x0000, 0x0300+0x0400, 0x0300+0x0800, 0x0300+0x0C00, 0x0300+0x1000, 0x0300+0x1400, 0x0300+0x1800, 0x0300+0x1C00, 
      0x0380+0x0000, 0x0380+0x0400, 0x0380+0x0800, 0x0380+0x0C00, 0x0380+0x1000, 0x0380+0x1400, 0x0380+0x1800, 0x0380+0x1C00, 
      0x0028+0x0000, 0x0028+0x0400, 0x0028+0x0800, 0x0028+0x0C00, 0x0028+0x1000, 0x0028+0x1400, 0x0028+0x1800, 0x0028+0x1C00, 
      0x00A8+0x0000, 0x00A8+0x0400, 0x00A8+0x0800, 0x00A8+0x0C00, 0x00A8+0x1000, 0x00A8+0x1400, 0x00A8+0x1800, 0x00A8+0x1C00, 
      0x0128+0x0000, 0x0128+0x0400, 0x0128+0x0800, 0x0128+0x0C00, 0x0128+0x1000, 0x0128+0x1400, 0x0128+0x1800, 0x0128+0x1C00, 
      0x01A8+0x0000, 0x01A8+0x0400, 0x01A8+0x0800, 0x01A8+0x0C00, 0x01A8+0x1000, 0x01A8+0x1400, 0x01A8+0x1800, 0x01A8+0x1C00, 
      0x0228+0x0000, 0x0228+0x0400, 0x0228+0x0800, 0x0228+0x0C00, 0x0228+0x1000, 0x0228+0x1400, 0x0228+0x1800, 0x0228+0x1C00, 
      0x02A8+0x0000, 0x02A8+0x0400, 0x02A8+0x0800, 0x02A8+0x0C00, 0x02A8+0x1000, 0x02A8+0x1400, 0x02A8+0x1800, 0x02A8+0x1C00, 
      0x0328+0x0000, 0x0328+0x0400, 0x0328+0x0800, 0x0328+0x0C00, 0x0328+0x1000, 0x0328+0x1400, 0x0328+0x1800, 0x0328+0x1C00, 
      0x03A8+0x0000, 0x03A8+0x0400, 0x03A8+0x0800, 0x03A8+0x0C00, 0x03A8+0x1000, 0x03A8+0x1400, 0x03A8+0x1800, 0x03A8+0x1C00, 
      0x0050+0x0000, 0x0050+0x0400, 0x0050+0x0800, 0x0050+0x0C00, 0x0050+0x1000, 0x0050+0x1400, 0x0050+0x1800, 0x0050+0x1C00, 
      0x00D0+0x0000, 0x00D0+0x0400, 0x00D0+0x0800, 0x00D0+0x0C00, 0x00D0+0x1000, 0x00D0+0x1400, 0x00D0+0x1800, 0x00D0+0x1C00, 
      0x0150+0x0000, 0x0150+0x0400, 0x0150+0x0800, 0x0150+0x0C00, 0x0150+0x1000, 0x0150+0x1400, 0x0150+0x1800, 0x0150+0x1C00, 
      0x01D0+0x0000, 0x01D0+0x0400, 0x01D0+0x0800, 0x01D0+0x0C00, 0x01D0+0x1000, 0x01D0+0x1400, 0x01D0+0x1800, 0x01D0+0x1C00, 
      0x0250+0x0000, 0x0250+0x0400, 0x0250+0x0800, 0x0250+0x0C00, 0x0250+0x1000, 0x0250+0x1400, 0x0250+0x1800, 0x0250+0x1C00, 
      0x02D0+0x0000, 0x02D0+0x0400, 0x02D0+0x0800, 0x02D0+0x0C00, 0x02D0+0x1000, 0x02D0+0x1400, 0x02D0+0x1800, 0x02D0+0x1C00, 
      0x0350+0x0000, 0x0350+0x0400, 0x0350+0x0800, 0x0350+0x0C00, 0x0350+0x1000, 0x0350+0x1400, 0x0350+0x1800, 0x0350+0x1C00, 
      0x03D0+0x0000, 0x03D0+0x0400, 0x03D0+0x0800, 0x03D0+0x0C00, 0x03D0+0x1000, 0x03D0+0x1400, 0x03D0+0x1800, 0x03D0+0x1C00
    };
  int tab_text80_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /** Graphic : Rcupre les donnes brutes : 2x8192 bytes **/
  graphicdhgr_data_address = (graphicdgr_bank << 16) | graphicdgr_address;
  ReadMemoryBlock(graphicdhgr_data_address,8192,graphicdhgr_raw_data_main,current_mmu);
  graphicdhgr_data_address = ((graphicdgr_bank+1) << 16) | graphicdgr_address;
  ReadMemoryBlock(graphicdhgr_data_address,8192,graphicdhgr_raw_data_aux,current_mmu);

  /** Text : Rcupre les donnes brutes : 2x1024 chars **/
  textpage80_data_address = (textpage80_bank << 16) | textpage80_address;
  ReadMemoryBlock(textpage80_data_address,1024,textpage80_raw_data_main,current_mmu);
  textpage80_data_address = ((textpage80_bank+1) << 16) | textpage80_address;
  ReadMemoryBlock(textpage80_data_address,1024,textpage80_raw_data_aux,current_mmu);

  /** Graphic : R-organise les donnes : Linarisation (passage du block de 2*1024 byte en 80*24) **/
  for(y=0; y<160; y++)
    for(x=0; x<40; x++)
      {
        graphicdhgr_data[2*x+0+y*80] = graphicdhgr_raw_data_aux[tab_dhgr_address[y]+x];
        graphicdhgr_data[2*x+1+y*80] = graphicdhgr_raw_data_main[tab_dhgr_address[y]+x];
      }

  /** Text : R-organise les donnes : Linarisation Linarisation (passage du block de 2x1024 char en 80*24) **/
  for(y=0; y<24; y++)
    for(x=0; x<40; x++)
      {
        textpage80_data[(2*x+0)+y*80] = textpage80_raw_data_aux[tab_text80_address[y]+x];
        textpage80_data[(2*x+1)+y*80] = textpage80_raw_data_main[tab_text80_address[y]+x];
      }
      
  /** Compare les donnes **/
  if(window_graphicdhgrcolortext80->graphicdhgr_data != NULL && window_graphicdhgrcolortext80->textpage80_data != NULL)
    if(graphicdgr_bank == window_graphicdhgrcolortext80->graphicdhgr_bank &&
       graphicdgr_address == window_graphicdhgrcolortext80->graphicdhgr_address &&
       memcmp(&graphicdhgr_data[0],&window_graphicdhgrcolortext80->graphicdhgr_data[0],80*192) == 0 && 
       textpage80_bank == window_graphicdhgrcolortext80->textpage80_bank &&
       textpage80_address == window_graphicdhgrcolortext80->textpage80_address &&
       memcmp(&textpage80_data[0],&window_graphicdhgrcolortext80->textpage80_data[0],80*24) == 0)
      return(0);

  /** Les donnes sont diffrentes => MAJ **/
  /* Graphic */
  window_graphicdhgrcolortext80->graphicdhgr_bank = graphicdgr_bank;
  window_graphicdhgrcolortext80->graphicdhgr_address = graphicdgr_address;
  memcpy(window_graphicdhgrcolortext80->graphicdhgr_data,graphicdhgr_data,80*192);
  /* Text */
  window_graphicdhgrcolortext80->textpage80_bank = textpage80_bank;
  window_graphicdhgrcolortext80->textpage80_address = textpage80_address;
  memcpy(window_graphicdhgrcolortext80->textpage80_data,textpage80_data,80*24);

  /* Donnes diffrentes */
  return(1);
}


/************************************************************************************************/
/*  ResizeGraphicDHGRColorText80Window() :  Resize GraphicDHGRColorText80 window by given size. */
/************************************************************************************************/
static void ResizeGraphicDHGRColorText80Window(struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80)
{
  HWND hwnd_group;
  RECT window_rect, client_rect, group_rect;
  int window_x, window_y;

  /** On va ajuster la fentre par rapport au Zoom demand **/  
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,&client_rect);
  window_graphicdhgrcolortext80->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select */
  hwnd_group = GetDlgItem(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,IDC_GROUP_GRAPHICDHGRCOLORTEXT80);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_GRAPHICDHGRCOLORTEXT80 */
  GetWindowRect(hwnd_group,&group_rect);
  window_graphicdhgrcolortext80->group_data_x = (group_rect.left - window_rect.left) - 3;
  window_graphicdhgrcolortext80->group_data_y = ((group_rect.top - window_rect.top) - window_graphicdhgrcolortext80->title_height) + 3;

  /* Position de la Zone BitMap dans la fenetre */
  window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_x = window_graphicdhgrcolortext80->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_y = window_graphicdhgrcolortext80->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_graphicdhgrcolortext80->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_width*window_graphicdhgrcolortext80->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_graphicdhgrcolortext80->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicdhgrcolortext80->bitmap_graphicdhgrcolortext80_height*window_graphicdhgrcolortext80->zoom_level*2 + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_graphicdhgrcolortext80->group_data_x,window_graphicdhgrcolortext80->group_data_y,window_graphicdhgrcolortext80->group_data_width,window_graphicdhgrcolortext80->group_data_height,TRUE);

  /* Largeur de la fentre */
  window_graphicdhgrcolortext80->window_width = window_graphicdhgrcolortext80->group_data_x + window_graphicdhgrcolortext80->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* On doit garantir une largeur minimale de la fentre */
  if(window_graphicdhgrcolortext80->window_width < GRAPHICDHGRCOLORTEXT80_DIALOGBOX_WIDTH)
    window_graphicdhgrcolortext80->window_width = GRAPHICDHGRCOLORTEXT80_DIALOGBOX_WIDTH;

  /* Hauteur de la fentre */
  window_graphicdhgrcolortext80->window_height = window_graphicdhgrcolortext80->title_height + window_graphicdhgrcolortext80->group_data_y + window_graphicdhgrcolortext80->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Resize window */
  MoveWindow(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,window_x,window_y,window_graphicdhgrcolortext80->window_width,window_graphicdhgrcolortext80->window_height,TRUE);
}


/************************************************************************************************************************/
/*  BuildGraphicDHGRColorText80FromData() :  Construire les donnes hexadcimal  afficher  partir des donnes brutes. */
/************************************************************************************************************************/
static void BuildGraphicDHGRColorText80FromData(struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80)
{
  int one_byte;
  unsigned char byte0_bank0;
  unsigned char byte1_bank0;
  unsigned char byte0_bank1;
  unsigned char byte1_bank1;
  int x, y, pixmap_char_x, pixmap_char_y, pixel_x, pixel_y, offset_pixmap_char, offset_pixmap_screen, offset_bitmap, color_index;
  char buffer_title[256];
  DWORD color_text, color_background, color_pixmap, color_screen;
  DWORD tab_rgb_color_graphic[16] = 
    {
      /* 16 couleurs pour le mode DHGR */
      MAKECOLOR(0,0,0),     MAKECOLOR(0,0,153),     MAKECOLOR(0,119,0),   MAKECOLOR(34,34,255),
      MAKECOLOR(136,85,34), MAKECOLOR(170,170,170), MAKECOLOR(0,221,0),   MAKECOLOR(0,255,153), 
      MAKECOLOR(221,0,51),  MAKECOLOR(221,0,221),   MAKECOLOR(85,85,85),  MAKECOLOR(102,170,255), 
      MAKECOLOR(255,102,0), MAKECOLOR(255,153,136), MAKECOLOR(255,255,0), MAKECOLOR(255,255,255)
    };
  DWORD tab_rgb_color_text[16] = 
    {
      /* 16 couleurs de la Bordure / Fond Ecran / Text */
      MAKECOLOR(0,0,0), MAKECOLOR(221,0,51), MAKECOLOR(0,0,153), MAKECOLOR(221,34,221),
      MAKECOLOR(0,119,34), MAKECOLOR(85,85,85), MAKECOLOR(34,34,255), MAKECOLOR(102,170,255),
      MAKECOLOR(136,85,0), MAKECOLOR(255,102,0), MAKECOLOR(170,170,170), MAKECOLOR(255,153,136),
      MAKECOLOR(17,221,0), MAKECOLOR(255,255,0), MAKECOLOR(68,255,153), MAKECOLOR(255,255,255)
    };
  struct picture_true *picture_char;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rien  faire */
  if(window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80 == NULL)
    return;

  /* Adresse de la picture */
  if(window_graphicdhgrcolortext80->display_language == LANGUAGE_IIGS_USA)
    picture_char = param->picture_char_us;
  else if(window_graphicdhgrcolortext80->display_language == LANGUAGE_IIGS_UK)
    picture_char = param->picture_char_uk;
  else if(window_graphicdhgrcolortext80->display_language == LANGUAGE_IIGS_FRENCH)
    picture_char = param->picture_char_fr;
  else if(window_graphicdhgrcolortext80->display_language == LANGUAGE_IIGS_DANISH)
    picture_char = param->picture_char_dk;
  else if(window_graphicdhgrcolortext80->display_language == LANGUAGE_IIGS_SPANISH)
    picture_char = param->picture_char_se;
  else if(window_graphicdhgrcolortext80->display_language == LANGUAGE_IIGS_ITALIAN)
    picture_char = param->picture_char_it;
  else if(window_graphicdhgrcolortext80->display_language == LANGUAGE_IIGS_GERMAN)
    picture_char = param->picture_char_de;
  else if(window_graphicdhgrcolortext80->display_language == LANGUAGE_IIGS_SWEDISH)
    picture_char = param->picture_char_se;
  else if(window_graphicdhgrcolortext80->display_language == LANGUAGE_IIGS_MOUSETEXT)
    picture_char = param->picture_char_mousetext;
    
  /* Couleur du Texte */
  color_text = tab_rgb_color_text[window_graphicdhgrcolortext80->color_text];

  /* Couleur du Fond */
  color_background = tab_rgb_color_text[window_graphicdhgrcolortext80->color_background];

  /** Graphic : Cration de la bitmap 560*192 correspondant au 140*192 du DHGR **/
  offset_bitmap = 0;
  for(y=0; y<160; y++)               /* 192-(4*8) */
    for(x=0; x<80; x+=4)
      {
        /* 1 pixel = 4 bit, 4 Bytes = 7 pixels */
        byte0_bank0 = window_graphicdhgrcolortext80->graphicdhgr_data[x + 1 + y*80];
        byte0_bank1 = window_graphicdhgrcolortext80->graphicdhgr_data[x + 0 + y*80];
        byte1_bank0 = window_graphicdhgrcolortext80->graphicdhgr_data[x + 3 + y*80];
        byte1_bank1 = window_graphicdhgrcolortext80->graphicdhgr_data[x + 2 + y*80];
        
        /** Codage des 7 points **/
        /* Pixel 0 */
        color_index = (int) (byte0_bank1 & 0x0F);
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        /* Pixel 1 */
        color_index = (int) (((byte0_bank1 >> 4) & 0x07) | ((byte0_bank0 << 3) & 0x08));
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        /* Pixel 2 */
        color_index = (int) (((byte0_bank0 >> 1) & 0x0F));
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        /* Pixel 3 */
        color_index = (int) (((byte0_bank0 >> 5) & 0x03) | ((byte1_bank1 << 2) & 0x0C));
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        /* Pixel 4 */
        color_index = (int) ((byte1_bank1 >> 2) & 0x0F);
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        /* Pixel 5 */
        color_index = (int) (((byte1_bank1 >> 6) & 0x01) | ((byte1_bank0 << 1)& 0x0E));
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        /* Pixel 6 */
        color_index = (int) ((byte1_bank0 >> 3) & 0x0F);
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;  
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;  
        window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_bitmap] = tab_rgb_color_graphic[color_index];
        offset_bitmap++;
      }

  /** Text : Cration du bas de la bitmap 560*192 correspondant au 40*24 **/
  for(y=20; y<24; y++)
    {
      for(x=0; x<80; x++)
        {
          /* Caractre */
          one_byte = (int) window_graphicdhgrcolortext80->textpage80_data[x + y*80];

          /* Offset dans la Pixmap Charset */
          pixmap_char_y = one_byte / 40;
          pixmap_char_x = one_byte - (pixmap_char_y*40);

          /* Copie ce bloc de 7*8 */
          offset_pixmap_char  = pixmap_char_x*7 + pixmap_char_y*(40*7)*8;
          offset_pixmap_screen = x*7 + y*(80*7)*8;
          for(pixel_y=0; pixel_y<8; pixel_y++)
            for(pixel_x=0; pixel_x<7; pixel_x++)
              {
                /* Pixel dans la Char Tab */
                color_pixmap = picture_char->data_dwd[offset_pixmap_char + pixel_x + pixel_y*40*7];

                /* Couleur du Pixel final */
                color_screen = (color_pixmap == 0xFFFFFF) ? color_text : color_background;

                /* Place le point dans la Pixmap */
                window_graphicdhgrcolortext80->bitmap_data_graphicdhgrcolortext80[offset_pixmap_screen + pixel_x + pixel_y*80*7] = color_screen;
              }
        }
    }

  /* Titre de la fentre */
  sprintf(buffer_title,"Graphic DHGR Color Text 80 Column : %02X/%04X   %02X/%04X",window_graphicdhgrcolortext80->graphicdhgr_bank,window_graphicdhgrcolortext80->graphicdhgr_address,
                                                                                   window_graphicdhgrcolortext80->textpage80_bank,window_graphicdhgrcolortext80->textpage80_address);
  SetWindowText(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,buffer_title);
}


/************************************************************************************/
/*  DeleteGraphicDHGRColorText80ChildWindows() :  Destruction des fentres enfants. */
/************************************************************************************/
static void DeleteGraphicDHGRColorText80ChildWindows(struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80)
{
  DeleteObject(window_graphicdhgrcolortext80->hBitmap_graphicdhgrcolortext80);
  DeleteObject(window_graphicdhgrcolortext80->hwnd_combo_color_text);
  DeleteObject(window_graphicdhgrcolortext80->hwnd_combo_color_background);
  DeleteObject(window_graphicdhgrcolortext80->hwnd_combo_display_language);
  DeleteObject(window_graphicdhgrcolortext80->hFont);
  ReleaseDC(window_graphicdhgrcolortext80->hwnd_graphicdhgrcolortext80,window_graphicdhgrcolortext80->hDC);
  DeleteDC(window_graphicdhgrcolortext80->hCompatibleDC);
}


/**************************************************************************************/
/*  mem_free_window_graphicdhgrcolortext80() :  Libration mmoire pour la structure. */
/**************************************************************************************/
void mem_free_window_graphicdhgrcolortext80(struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80)
{
  if(window_graphicdhgrcolortext80)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicdhgrcolortext80->current_notification,NULL);
      window_graphicdhgrcolortext80->current_notification = NULL;
      
      free(window_graphicdhgrcolortext80);
    }
}

/**************************************************************************************/
