/**********************************************************************************/
/*                                                                                */
/*  Window_GraphicDHGRMono.c :  Module de gestion de la fentre GraphicDHGRMono.  */
/*                                                                                */
/**********************************************************************************/
/*   Auteur : Olivier ZARDINI    *    Brutal Deluxe    *    Date : Juillet 2018   */
/**********************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_GraphicDHGRMono.h"

static DWORD BuildGraphicDHGRMonoWindowThread(void *);
LRESULT CALLBACK ProcHandlerGraphicDHGRMonoWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateGraphicDHGRMonoChildWindows(HWND,struct window_graphicdhgrmono *);
LRESULT CALLBACK subEditGraphicDHGRMonoProcControl(HWND,UINT,WPARAM,LPARAM);
void UpdateGraphicDHGRMonoClientAllArea(HDC,struct window_graphicdhgrmono *);
static void UpdateGraphicDHGRMonoClientBitmapArea(HDC,struct window_graphicdhgrmono *);
static void UpdateGraphicDHGRMonoContextInformation(struct window_graphicdhgrmono *,int,int);
static void UpdateGraphicDHGRMonoAddress(struct window_graphicdhgrmono *,int);
static void UpdateGraphicDHGRMonoRefreshMode(struct window_graphicdhgrmono *,int);
static void UpdateGraphicDHGRMonoZoomLevel(struct window_graphicdhgrmono *,int);
static void RefreshGraphicDHGRMonoWindowData(struct window_graphicdhgrmono *,int);
static int IsGraphicDHGRMonoDataModified(int,int,int,int,struct window_graphicdhgrmono *);
static void ResizeGraphicDHGRMonoWindow(struct window_graphicdhgrmono *);
static void BuildGraphicDHGRMonoFromData(struct window_graphicdhgrmono *);
static void DeleteGraphicDHGRMonoChildWindows(struct window_graphicdhgrmono *);


/********************************************************************************************/
/*  BuildGraphicDHGRMonoWindow() :  Construction / Affichage de la fentre GraphicDHGRMono. */
/********************************************************************************************/
struct window_graphicdhgrmono *BuildGraphicDHGRMonoWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_graphicdhgrmono *window_graphicdhgrmono;

  /** Allocation de la structure window_graphicdhgrmono **/
  window_graphicdhgrmono = (struct window_graphicdhgrmono *) calloc(1,sizeof(struct window_graphicdhgrmono));
  if(window_graphicdhgrmono == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_GRAPHICDHGRMONO,window_graphicdhgrmono,NULL);

  /* Lance un thread pour grer la fentre GraphicDHGRMono */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildGraphicDHGRMonoWindowThread,window_graphicdhgrmono,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_graphicdhgrmono);
}


/******************************************************************************************************/
/*  BuildGraphicDHGRMonoWindowThread() :  Cration de la fentre GraphicDHGRMono via un Thread ddi. */
/******************************************************************************************************/
static DWORD BuildGraphicDHGRMonoWindowThread(void *window_graphicdhgrmono_param)
{
  HWND hwndGraphicDHGRMono;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_graphicdhgrmono *window_graphicdhgrmono;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_graphicdhgrmono = (struct window_graphicdhgrmono *) window_graphicdhgrmono_param;
  window_graphicdhgrmono->thread_id = GetCurrentThreadId();         /* Le ThreaID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Graphic GR **/
  hwndGraphicDHGRMono = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_GRAPHICDHGRMONO),window_main->hwnd_main,(DLGPROC)ProcHandlerGraphicDHGRMonoWindow);
  if(hwndGraphicDHGRMono == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndGraphicDHGRMono,SW_SHOWDEFAULT);
  UpdateWindow(hwndGraphicDHGRMono);

  /** Message Loop pour toutes les fentres de ce Thread (fentre GraphicDHGRMono + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/********************************************************************************/
/*  ProcHandlerGraphicDHGRMonoWindow() :  Gestion des vnements de la fentre. */
/********************************************************************************/
LRESULT CALLBACK ProcHandlerGraphicDHGRMonoWindow(HWND hwndGraphicDHGRMono, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;
  int mouse_x, mouse_y;
  struct window_graphicdhgrmono *window_graphicdhgrmono;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_GRAPHICDHGRMONO,&window_graphicdhgrmono,NULL);
  if(window_graphicdhgrmono == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateGraphicDHGRMonoChildWindows(hwndGraphicDHGRMono,window_graphicdhgrmono);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        UpdateGraphicDHGRMonoContextInformation(window_graphicdhgrmono,mouse_x,mouse_y);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_graphicdhgrmono->hBitmap_graphicdhgrmono != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndGraphicDHGRMono,&ps);

            /** MAJ Bitmap **/
            UpdateGraphicDHGRMonoClientAllArea(hDC,window_graphicdhgrmono);

            /* End Paint */
            EndPaint(hwndGraphicDHGRMono,&ps);
          }
        break;
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdhgrmono->refresh_mode != GRAPHICDHGRMONO_REFRESH_AUTOMATIC)
                UpdateGraphicDHGRMonoRefreshMode(window_graphicdhgrmono,GRAPHICDHGRMONO_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdhgrmono->refresh_mode != GRAPHICDHGRMONO_REFRESH_MANUAL)
                UpdateGraphicDHGRMonoRefreshMode(window_graphicdhgrmono,GRAPHICDHGRMONO_REFRESH_MANUAL);
              break;

            /** Bouton Radio : On change le Zoom de la fentre **/
            case IDC_RADIO_ZOOM_X1 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdhgrmono->zoom_level != GRAPHICDHGRMONO_ZOOM_X1)
                UpdateGraphicDHGRMonoZoomLevel(window_graphicdhgrmono,GRAPHICDHGRMONO_ZOOM_X1);
              break;

            case IDC_RADIO_ZOOM_X2 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicdhgrmono->zoom_level != GRAPHICDHGRMONO_ZOOM_X2)
                UpdateGraphicDHGRMonoZoomLevel(window_graphicdhgrmono,GRAPHICDHGRMONO_ZOOM_X2);
              break;

            /** Bouton de dplacement de l'adresse **/
            case IDC_BUTTON_LEFT :
              UpdateGraphicDHGRMonoAddress(window_graphicdhgrmono,-1);
              break;

            case IDC_BUTTON_RIGHT :
              UpdateGraphicDHGRMonoAddress(window_graphicdhgrmono,1);
              break;

            case IDC_BUTTON_UP :
              UpdateGraphicDHGRMonoAddress(window_graphicdhgrmono,-40);
              break;

            case IDC_BUTTON_DOWN :
              UpdateGraphicDHGRMonoAddress(window_graphicdhgrmono,40);
              break;

            case IDC_BUTTON_PAGEUP :
              UpdateGraphicDHGRMonoAddress(window_graphicdhgrmono,-8192);
              break;

            case IDC_BUTTON_PAGEDOWN :
              UpdateGraphicDHGRMonoAddress(window_graphicdhgrmono,8192);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshGraphicDHGRMonoWindowData(window_graphicdhgrmono,1);
              break;              
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_GRAPHICDHGRMONO_REFRESHWINDOW :
        RefreshGraphicDHGRMonoWindowData(window_graphicdhgrmono,0);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre GraphicDHGRMono */
        DestroyWindow(hwndGraphicDHGRMono);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteGraphicDHGRMonoChildWindows(window_graphicdhgrmono);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_GRAPHICDHGRMONO,window_graphicdhgrmono,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*********************************************************************************************************************/
/*  CreateGraphicDHGRMonoChildWindows() : Crer les fentres enfants et remplir la structure window_graphicdhgrmono. */
/*********************************************************************************************************************/
static int CreateGraphicDHGRMonoChildWindows(HWND hwndGraphicDHGRMono, struct window_graphicdhgrmono *window_graphicdhgrmono)
{
  BITMAPINFO bitmap_info;
  int i;
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_graphicdhgrmono->hwnd_graphicdhgrmono = hwndGraphicDHGRMono;

  /** Device Context **/
  window_graphicdhgrmono->hDC = GetDC(window_graphicdhgrmono->hwnd_graphicdhgrmono);
  window_graphicdhgrmono->hCompatibleDC = CreateCompatibleDC(window_graphicdhgrmono->hDC);

  /** Surchage des Control Edit pour capter les vnements **/
  window_graphicdhgrmono->oldEditProcBank    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDHGRMono,IDC_EDIT_BANK),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDHGRMonoProcControl);
  window_graphicdhgrmono->oldEditProcAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicDHGRMono,IDC_EDIT_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditGraphicDHGRMonoProcControl);

  /** Ajoute les images sur les boutons **/
  SendMessage(GetDlgItem(hwndGraphicDHGRMono,IDC_BUTTON_LEFT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicDHGRMono,IDC_BUTTON_RIGHT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicDHGRMono,IDC_BUTTON_UP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicDHGRMono,IDC_BUTTON_DOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicDHGRMono,IDC_BUTTON_PAGEUP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicDHGRMono,IDC_BUTTON_PAGEDOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Adresse de la Zone de Data **/
  window_graphicdhgrmono->graphicdhgrmono_bank = 0x00;
  window_graphicdhgrmono->graphicdhgrmono_address = 0x2000; 
  sprintf(buffer,"%02X",window_graphicdhgrmono->graphicdhgrmono_bank);
  SetDlgItemText(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",window_graphicdhgrmono->graphicdhgrmono_address);
  SetDlgItemText(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_ADDRESS,buffer);

  /* Refresh Mode : Automatic */
  UpdateGraphicDHGRMonoRefreshMode(window_graphicdhgrmono,GRAPHICDHGRMONO_REFRESH_AUTOMATIC);

  /* Zoom Level : 1 */
  window_graphicdhgrmono->zoom_level = GRAPHICDHGRMONO_ZOOM_X1;
  SendDlgItemMessage(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);

  /* Dimension en Pixel de la Bitmap GraphicDHGRMono Data */
  window_graphicdhgrmono->bitmap_graphicdhgrmono_width = 560;
  window_graphicdhgrmono->bitmap_graphicdhgrmono_height = 192;
  
  /** Bitmap affichant la zone de slection **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicdhgrmono->bitmap_graphicdhgrmono_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicdhgrmono->bitmap_graphicdhgrmono_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicdhgrmono->hBitmap_graphicdhgrmono = CreateDIBSection(window_graphicdhgrmono->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicdhgrmono->bitmap_data_graphicdhgrmono),NULL,0);
  for(i=0; i<window_graphicdhgrmono->bitmap_graphicdhgrmono_width*window_graphicdhgrmono->bitmap_graphicdhgrmono_height; i++)
    window_graphicdhgrmono->bitmap_data_graphicdhgrmono[i] = BITMAP_BACKGROUND_COLOR;

  /* Retaille la fentre */
  ResizeGraphicDHGRMonoWindow(window_graphicdhgrmono);

  /** Construction de la Page Texte + Rafraichissement de la fentre **/
  RefreshGraphicDHGRMonoWindowData(window_graphicdhgrmono,1);
    
  /* OK */
  return(0);
}


/******************************************************************************************************/
/*  subEditGraphicDHGRMonoProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/******************************************************************************************************/
LRESULT CALLBACK subEditGraphicDHGRMonoProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_graphicdhgrmono *window_graphicdhgrmono;
  my_Memory(MEMORY_GET_WINDOW_GRAPHICDHGRMONO,&window_graphicdhgrmono,NULL);

  /* Rien  faire */
  if(window_graphicdhgrmono == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_BANK) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_ADDRESS) == hwnd_control)
    index = 2;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement du Return dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_graphicdhgrmono->hwnd_graphicdhgrmono);

      /* On force la MAJ de la fentre */
      RefreshGraphicDHGRMonoWindowData(window_graphicdhgrmono,1);
    }
  else if(index == 1)
    return(CallWindowProc(window_graphicdhgrmono->oldEditProcBank,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_graphicdhgrmono->oldEditProcAddress,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
}


/************************************************************************************************/
/*  UpdateGraphicDHGRMonoClientAllArea() :  Affichage des Bitmap de la fentre GraphicDHGRMono. */
/************************************************************************************************/
void UpdateGraphicDHGRMonoClientAllArea(HDC hDC, struct window_graphicdhgrmono *window_graphicdhgrmono)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap **/
  UpdateGraphicDHGRMonoClientBitmapArea(hDC,window_graphicdhgrmono);
}


/***************************************************************************************/
/*  UpdateGraphicDHGRMonoClientBitmapArea() :  Affichage de la Bitmap GraphicDHGRMono. */
/***************************************************************************************/
static void UpdateGraphicDHGRMonoClientBitmapArea(HDC hDC, struct window_graphicdhgrmono *window_graphicdhgrmono)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec la zone mmoire et la zone slectionne **/
  bitmap_old = SelectObject(window_graphicdhgrmono->hCompatibleDC,window_graphicdhgrmono->hBitmap_graphicdhgrmono);
  StretchBlt(hDC,window_graphicdhgrmono->bitmap_graphicdhgrmono_x,window_graphicdhgrmono->bitmap_graphicdhgrmono_y,
             window_graphicdhgrmono->bitmap_graphicdhgrmono_width*window_graphicdhgrmono->zoom_level,
             window_graphicdhgrmono->bitmap_graphicdhgrmono_height*window_graphicdhgrmono->zoom_level*2,   /* x2 pour la hauteur */
             window_graphicdhgrmono->hCompatibleDC,0,0,window_graphicdhgrmono->bitmap_graphicdhgrmono_width,window_graphicdhgrmono->bitmap_graphicdhgrmono_height,SRCCOPY);
  SelectObject(window_graphicdhgrmono->hCompatibleDC,bitmap_old);
}


/*****************************************************************************************/
/*  UpdateGraphicDHGRMonoAddress() :  Dplace l'adresse pour la fentre GraphicDHGRMono. */
/*****************************************************************************************/
static void UpdateGraphicDHGRMonoAddress(struct window_graphicdhgrmono *window_graphicdhgrmono, int delta)
{
  int current_address, new_address, graphicdhgrmono_bank, graphicdhgrmono_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphicdhgrmono->graphicdhgrmono_bank << 16) | window_graphicdhgrmono->graphicdhgrmono_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  graphicdhgrmono_bank = (new_address >> 16) & 0xFF;
  graphicdhgrmono_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphicdhgrmono_bank);
  SetDlgItemText(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",graphicdhgrmono_address);
  SetDlgItemText(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_ADDRESS,buffer);

  /* MAJ de la fentre */
  RefreshGraphicDHGRMonoWindowData(window_graphicdhgrmono,1);
}


/*****************************************************************************************************************/
/*  UpdateGraphicDHGRMonoContextInformation() :  MAJ des informations de contexte de la fentre GraphicDHGRMono. */
/*****************************************************************************************************************/
static void UpdateGraphicDHGRMonoContextInformation(struct window_graphicdhgrmono *window_graphicdhgrmono, int mouse_x, int mouse_y)
{
  int pixmap_x, pixmap_y, grid_x;
  int graphicdhgrmono_data_address;
  unsigned char graphicdhgrmono_data;
  int tab_hgr_address[192] =     /* Adresse du dbut des 192 lignes de la Page HGR */
    {
      0x0000+0x0000, 0x0000+0x0400, 0x0000+0x0800, 0x0000+0x0C00, 0x0000+0x1000, 0x0000+0x1400, 0x0000+0x1800, 0x0000+0x1C00, 
      0x0080+0x0000, 0x0080+0x0400, 0x0080+0x0800, 0x0080+0x0C00, 0x0080+0x1000, 0x0080+0x1400, 0x0080+0x1800, 0x0080+0x1C00, 
      0x0100+0x0000, 0x0100+0x0400, 0x0100+0x0800, 0x0100+0x0C00, 0x0100+0x1000, 0x0100+0x1400, 0x0100+0x1800, 0x0100+0x1C00, 
      0x0180+0x0000, 0x0180+0x0400, 0x0180+0x0800, 0x0180+0x0C00, 0x0180+0x1000, 0x0180+0x1400, 0x0180+0x1800, 0x0180+0x1C00, 
      0x0200+0x0000, 0x0200+0x0400, 0x0200+0x0800, 0x0200+0x0C00, 0x0200+0x1000, 0x0200+0x1400, 0x0200+0x1800, 0x0200+0x1C00, 
      0x0280+0x0000, 0x0280+0x0400, 0x0280+0x0800, 0x0280+0x0C00, 0x0280+0x1000, 0x0280+0x1400, 0x0280+0x1800, 0x0280+0x1C00, 
      0x0300+0x0000, 0x0300+0x0400, 0x0300+0x0800, 0x0300+0x0C00, 0x0300+0x1000, 0x0300+0x1400, 0x0300+0x1800, 0x0300+0x1C00, 
      0x0380+0x0000, 0x0380+0x0400, 0x0380+0x0800, 0x0380+0x0C00, 0x0380+0x1000, 0x0380+0x1400, 0x0380+0x1800, 0x0380+0x1C00, 
      0x0028+0x0000, 0x0028+0x0400, 0x0028+0x0800, 0x0028+0x0C00, 0x0028+0x1000, 0x0028+0x1400, 0x0028+0x1800, 0x0028+0x1C00, 
      0x00A8+0x0000, 0x00A8+0x0400, 0x00A8+0x0800, 0x00A8+0x0C00, 0x00A8+0x1000, 0x00A8+0x1400, 0x00A8+0x1800, 0x00A8+0x1C00, 
      0x0128+0x0000, 0x0128+0x0400, 0x0128+0x0800, 0x0128+0x0C00, 0x0128+0x1000, 0x0128+0x1400, 0x0128+0x1800, 0x0128+0x1C00, 
      0x01A8+0x0000, 0x01A8+0x0400, 0x01A8+0x0800, 0x01A8+0x0C00, 0x01A8+0x1000, 0x01A8+0x1400, 0x01A8+0x1800, 0x01A8+0x1C00, 
      0x0228+0x0000, 0x0228+0x0400, 0x0228+0x0800, 0x0228+0x0C00, 0x0228+0x1000, 0x0228+0x1400, 0x0228+0x1800, 0x0228+0x1C00, 
      0x02A8+0x0000, 0x02A8+0x0400, 0x02A8+0x0800, 0x02A8+0x0C00, 0x02A8+0x1000, 0x02A8+0x1400, 0x02A8+0x1800, 0x02A8+0x1C00, 
      0x0328+0x0000, 0x0328+0x0400, 0x0328+0x0800, 0x0328+0x0C00, 0x0328+0x1000, 0x0328+0x1400, 0x0328+0x1800, 0x0328+0x1C00, 
      0x03A8+0x0000, 0x03A8+0x0400, 0x03A8+0x0800, 0x03A8+0x0C00, 0x03A8+0x1000, 0x03A8+0x1400, 0x03A8+0x1800, 0x03A8+0x1C00, 
      0x0050+0x0000, 0x0050+0x0400, 0x0050+0x0800, 0x0050+0x0C00, 0x0050+0x1000, 0x0050+0x1400, 0x0050+0x1800, 0x0050+0x1C00, 
      0x00D0+0x0000, 0x00D0+0x0400, 0x00D0+0x0800, 0x00D0+0x0C00, 0x00D0+0x1000, 0x00D0+0x1400, 0x00D0+0x1800, 0x00D0+0x1C00, 
      0x0150+0x0000, 0x0150+0x0400, 0x0150+0x0800, 0x0150+0x0C00, 0x0150+0x1000, 0x0150+0x1400, 0x0150+0x1800, 0x0150+0x1C00, 
      0x01D0+0x0000, 0x01D0+0x0400, 0x01D0+0x0800, 0x01D0+0x0C00, 0x01D0+0x1000, 0x01D0+0x1400, 0x01D0+0x1800, 0x01D0+0x1C00, 
      0x0250+0x0000, 0x0250+0x0400, 0x0250+0x0800, 0x0250+0x0C00, 0x0250+0x1000, 0x0250+0x1400, 0x0250+0x1800, 0x0250+0x1C00, 
      0x02D0+0x0000, 0x02D0+0x0400, 0x02D0+0x0800, 0x02D0+0x0C00, 0x02D0+0x1000, 0x02D0+0x1400, 0x02D0+0x1800, 0x02D0+0x1C00, 
      0x0350+0x0000, 0x0350+0x0400, 0x0350+0x0800, 0x0350+0x0C00, 0x0350+0x1000, 0x0350+0x1400, 0x0350+0x1800, 0x0350+0x1C00, 
      0x03D0+0x0000, 0x03D0+0x0400, 0x03D0+0x0800, 0x03D0+0x0C00, 0x03D0+0x1000, 0x03D0+0x1400, 0x03D0+0x1800, 0x03D0+0x1C00
    };

  /* Rien  faire */
  if(window_graphicdhgrmono == NULL)
    return;

  /* Est t'on dans l'image ? */
  if(mouse_x < window_graphicdhgrmono->bitmap_graphicdhgrmono_x || mouse_x >= (window_graphicdhgrmono->bitmap_graphicdhgrmono_x + window_graphicdhgrmono->bitmap_graphicdhgrmono_width*window_graphicdhgrmono->zoom_level) ||
     mouse_y < window_graphicdhgrmono->bitmap_graphicdhgrmono_y || mouse_y >= (window_graphicdhgrmono->bitmap_graphicdhgrmono_y + window_graphicdhgrmono->bitmap_graphicdhgrmono_height*2*window_graphicdhgrmono->zoom_level))
    {
      SetWindowText(GetDlgItem(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_STATIC_CONTEXT),"");
      return;
    }

  /* Coordonnes dans la Pixmap */
  pixmap_x = (mouse_x - window_graphicdhgrmono->bitmap_graphicdhgrmono_x)/window_graphicdhgrmono->zoom_level;
  pixmap_y = ((mouse_y - window_graphicdhgrmono->bitmap_graphicdhgrmono_y)/window_graphicdhgrmono->zoom_level)/2;

  /* Coordonnes dans la grille 80x192 */
  grid_x = pixmap_x/7;

  /* Valeur */
  graphicdhgrmono_data = window_graphicdhgrmono->graphicdhgrmono_data[grid_x + pixmap_y*80];

  /* Adresse */
  graphicdhgrmono_data_address = (window_graphicdhgrmono->graphicdhgrmono_bank << 16) + window_graphicdhgrmono->graphicdhgrmono_address;
  graphicdhgrmono_data_address += grid_x/2;
  graphicdhgrmono_data_address += tab_hgr_address[pixmap_y];

  /* Message */
  if((grid_x % 2) == 0)
    sprintf(window_graphicdhgrmono->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",(graphicdhgrmono_data_address>>16)+1,graphicdhgrmono_data_address&0xFFFF,graphicdhgrmono_data,pixmap_x,pixmap_y);
  else
    sprintf(window_graphicdhgrmono->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicdhgrmono_data_address>>16,graphicdhgrmono_data_address&0xFFFF,graphicdhgrmono_data,pixmap_x,pixmap_y);
  SetWindowText(GetDlgItem(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_STATIC_CONTEXT),window_graphicdhgrmono->buffer_log);
}


/**********************************************************************************************************/
/*  UpdateGraphicDHGRMonoRefreshMode() :  MAJ du mode de rafraichissement de la fenetre GraphicDHGRMono. */
/**********************************************************************************************************/
static void UpdateGraphicDHGRMonoRefreshMode(struct window_graphicdhgrmono *window_graphicdhgrmono, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == GRAPHICDHGRMONO_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_graphicdhgrmono->refresh_mode = GRAPHICDHGRMONO_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_graphicdhgrmono->hwnd_graphicdhgrmono;
      current_notification->message = HM_GRAPHICDHGRMONO_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_GRAPHICDHGRMONO;

      /* Ajout de la notification */
      window_graphicdhgrmono->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_graphicdhgrmono->refresh_mode = GRAPHICDHGRMONO_REFRESH_MANUAL;
      SendDlgItemMessage(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicdhgrmono->current_notification,NULL);
      window_graphicdhgrmono->current_notification = NULL;
    }
}


/*********************************************************************************************/
/*  UpdateGraphicDHGRMonoZoomLevel() :  MAJ du nieau de Zomm de la fenetre GraphicDHGRMono. */
/*********************************************************************************************/
static void UpdateGraphicDHGRMonoZoomLevel(struct window_graphicdhgrmono *window_graphicdhgrmono, int zoom_level)
{
  if(zoom_level == GRAPHICDHGRMONO_ZOOM_X1)
    {
      /* Visuel */
      window_graphicdhgrmono->zoom_level = GRAPHICDHGRMONO_ZOOM_X1;
      SendDlgItemMessage(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICDHGRMONO_ZOOM_X2)
    {
      /* Visuel */
      window_graphicdhgrmono->zoom_level = GRAPHICDHGRMONO_ZOOM_X2;
      SendDlgItemMessage(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_RADIO_ZOOM_X2,BM_SETCHECK,1,0);
    }
    
  /* Retaille la fentre */
  ResizeGraphicDHGRMonoWindow(window_graphicdhgrmono);  
  
  /* On force la MAJ de la fentre */  
  RefreshGraphicDHGRMonoWindowData(window_graphicdhgrmono,1);    
}


/****************************************************************************************/
/*  RefreshGraphicDHGRMonoWindowData() :  MAJ du contenu de la fentre GraphicDHGRMono. */
/****************************************************************************************/
static void RefreshGraphicDHGRMonoWindowData(struct window_graphicdhgrmono *window_graphicdhgrmono, int force)
{
  int length, is_modified;
  int graphicdhgrmono_bank, graphicdhgrmono_address, nb_column, nb_row;
  char buffer[1024];

  /* Rcupre la valeur du GraphicDHGRMono Pointer */
  length = GetDlgItemText(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_BANK,buffer,1024);
  sscanf(buffer,"%X",&graphicdhgrmono_bank);
  length = GetDlgItemText(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_ADDRESS,buffer,1024);
  sscanf(buffer,"%X",&graphicdhgrmono_address);
  nb_column = GetDlgItemInt(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_COLUMN,NULL,FALSE);
  if(nb_column < 1)
    nb_column = 1;
  nb_row = GetDlgItemInt(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_ROW,NULL,FALSE);
  if(nb_row < 1)
    nb_row = 1;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphicdhgrmono_bank);
  SetDlgItemText(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",graphicdhgrmono_address);
  SetDlgItemText(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_ADDRESS,buffer);
  SetDlgItemInt(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_COLUMN,nb_column,FALSE);
  SetDlgItemInt(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_EDIT_ROW,nb_row,FALSE);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsGraphicDHGRMonoDataModified(graphicdhgrmono_bank,graphicdhgrmono_address,nb_column,nb_row,window_graphicdhgrmono);
  if(is_modified == 0 && force == 0)
    return;

  /** Cration de la Zone Graphique  partir des nouvelles donnes **/
  BuildGraphicDHGRMonoFromData(window_graphicdhgrmono);

  /* Rafraichit la fentre */
  InvalidateRect(window_graphicdhgrmono->hwnd_graphicdhgrmono,NULL,TRUE);
  UpdateWindow(window_graphicdhgrmono->hwnd_graphicdhgrmono);
}


/**********************************************************************************************************************/
/*  IsGraphicDHGRMonoDataModified() :  Les donnes GraphicDHGRMono ont t'elles t modifies depuis la dernire fois. */
/**********************************************************************************************************************/
static int IsGraphicDHGRMonoDataModified(int graphicdhgrmono_bank, int graphicdhgrmono_address, int nb_column, int nb_row, struct window_graphicdhgrmono *window_graphicdhgrmono)
{
  int x, y, graphicdhgrmono_data_address;
  unsigned char graphicdhgrmono_raw_bank0_data[8192];
  unsigned char graphicdhgrmono_raw_bank1_data[8192];
  unsigned char graphicdhgrmono_data[80*192];
  int tab_hgr_address[192] =     /* Adresse du dbut des 192 lignes de la Page HGR */
    {
      0x0000+0x0000, 0x0000+0x0400, 0x0000+0x0800, 0x0000+0x0C00, 0x0000+0x1000, 0x0000+0x1400, 0x0000+0x1800, 0x0000+0x1C00, 
      0x0080+0x0000, 0x0080+0x0400, 0x0080+0x0800, 0x0080+0x0C00, 0x0080+0x1000, 0x0080+0x1400, 0x0080+0x1800, 0x0080+0x1C00, 
      0x0100+0x0000, 0x0100+0x0400, 0x0100+0x0800, 0x0100+0x0C00, 0x0100+0x1000, 0x0100+0x1400, 0x0100+0x1800, 0x0100+0x1C00, 
      0x0180+0x0000, 0x0180+0x0400, 0x0180+0x0800, 0x0180+0x0C00, 0x0180+0x1000, 0x0180+0x1400, 0x0180+0x1800, 0x0180+0x1C00, 
      0x0200+0x0000, 0x0200+0x0400, 0x0200+0x0800, 0x0200+0x0C00, 0x0200+0x1000, 0x0200+0x1400, 0x0200+0x1800, 0x0200+0x1C00, 
      0x0280+0x0000, 0x0280+0x0400, 0x0280+0x0800, 0x0280+0x0C00, 0x0280+0x1000, 0x0280+0x1400, 0x0280+0x1800, 0x0280+0x1C00, 
      0x0300+0x0000, 0x0300+0x0400, 0x0300+0x0800, 0x0300+0x0C00, 0x0300+0x1000, 0x0300+0x1400, 0x0300+0x1800, 0x0300+0x1C00, 
      0x0380+0x0000, 0x0380+0x0400, 0x0380+0x0800, 0x0380+0x0C00, 0x0380+0x1000, 0x0380+0x1400, 0x0380+0x1800, 0x0380+0x1C00, 
      0x0028+0x0000, 0x0028+0x0400, 0x0028+0x0800, 0x0028+0x0C00, 0x0028+0x1000, 0x0028+0x1400, 0x0028+0x1800, 0x0028+0x1C00, 
      0x00A8+0x0000, 0x00A8+0x0400, 0x00A8+0x0800, 0x00A8+0x0C00, 0x00A8+0x1000, 0x00A8+0x1400, 0x00A8+0x1800, 0x00A8+0x1C00, 
      0x0128+0x0000, 0x0128+0x0400, 0x0128+0x0800, 0x0128+0x0C00, 0x0128+0x1000, 0x0128+0x1400, 0x0128+0x1800, 0x0128+0x1C00, 
      0x01A8+0x0000, 0x01A8+0x0400, 0x01A8+0x0800, 0x01A8+0x0C00, 0x01A8+0x1000, 0x01A8+0x1400, 0x01A8+0x1800, 0x01A8+0x1C00, 
      0x0228+0x0000, 0x0228+0x0400, 0x0228+0x0800, 0x0228+0x0C00, 0x0228+0x1000, 0x0228+0x1400, 0x0228+0x1800, 0x0228+0x1C00, 
      0x02A8+0x0000, 0x02A8+0x0400, 0x02A8+0x0800, 0x02A8+0x0C00, 0x02A8+0x1000, 0x02A8+0x1400, 0x02A8+0x1800, 0x02A8+0x1C00, 
      0x0328+0x0000, 0x0328+0x0400, 0x0328+0x0800, 0x0328+0x0C00, 0x0328+0x1000, 0x0328+0x1400, 0x0328+0x1800, 0x0328+0x1C00, 
      0x03A8+0x0000, 0x03A8+0x0400, 0x03A8+0x0800, 0x03A8+0x0C00, 0x03A8+0x1000, 0x03A8+0x1400, 0x03A8+0x1800, 0x03A8+0x1C00, 
      0x0050+0x0000, 0x0050+0x0400, 0x0050+0x0800, 0x0050+0x0C00, 0x0050+0x1000, 0x0050+0x1400, 0x0050+0x1800, 0x0050+0x1C00, 
      0x00D0+0x0000, 0x00D0+0x0400, 0x00D0+0x0800, 0x00D0+0x0C00, 0x00D0+0x1000, 0x00D0+0x1400, 0x00D0+0x1800, 0x00D0+0x1C00, 
      0x0150+0x0000, 0x0150+0x0400, 0x0150+0x0800, 0x0150+0x0C00, 0x0150+0x1000, 0x0150+0x1400, 0x0150+0x1800, 0x0150+0x1C00, 
      0x01D0+0x0000, 0x01D0+0x0400, 0x01D0+0x0800, 0x01D0+0x0C00, 0x01D0+0x1000, 0x01D0+0x1400, 0x01D0+0x1800, 0x01D0+0x1C00, 
      0x0250+0x0000, 0x0250+0x0400, 0x0250+0x0800, 0x0250+0x0C00, 0x0250+0x1000, 0x0250+0x1400, 0x0250+0x1800, 0x0250+0x1C00, 
      0x02D0+0x0000, 0x02D0+0x0400, 0x02D0+0x0800, 0x02D0+0x0C00, 0x02D0+0x1000, 0x02D0+0x1400, 0x02D0+0x1800, 0x02D0+0x1C00, 
      0x0350+0x0000, 0x0350+0x0400, 0x0350+0x0800, 0x0350+0x0C00, 0x0350+0x1000, 0x0350+0x1400, 0x0350+0x1800, 0x0350+0x1C00, 
      0x03D0+0x0000, 0x03D0+0x0400, 0x03D0+0x0800, 0x03D0+0x0C00, 0x03D0+0x1000, 0x03D0+0x1400, 0x03D0+0x1800, 0x03D0+0x1C00
    };
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /* Rcupre les donnes brutes : 2*812 chars */
  graphicdhgrmono_data_address = ((graphicdhgrmono_bank+1) << 16) | graphicdhgrmono_address;
  ReadMemoryBlock(graphicdhgrmono_data_address,8192,graphicdhgrmono_raw_bank1_data,current_mmu);
  graphicdhgrmono_data_address = (graphicdhgrmono_bank << 16) | graphicdhgrmono_address;
  ReadMemoryBlock(graphicdhgrmono_data_address,8192,graphicdhgrmono_raw_bank0_data,current_mmu);

  /* R-organise les donnes : Linarisation (passage du block de 2*8192 char en 80*192) */
  for(y=0; y<192; y++)
    for(x=0; x<40; x++)
      {
        graphicdhgrmono_data[2*x+0+y*80] = graphicdhgrmono_raw_bank1_data[tab_hgr_address[y]+x];
        graphicdhgrmono_data[2*x+1+y*80] = graphicdhgrmono_raw_bank0_data[tab_hgr_address[y]+x];
      }

  /* Compare les donnes */
  if(window_graphicdhgrmono->graphicdhgrmono_data != NULL)
    if(graphicdhgrmono_bank == window_graphicdhgrmono->graphicdhgrmono_bank &&
       graphicdhgrmono_address == window_graphicdhgrmono->graphicdhgrmono_address &&
       memcmp(&graphicdhgrmono_data[0],&window_graphicdhgrmono->graphicdhgrmono_data[0],80*192) == 0)
      return(0);

  /* Les donnes sont diffrentes => MAJ */
  window_graphicdhgrmono->graphicdhgrmono_bank = graphicdhgrmono_bank;
  window_graphicdhgrmono->graphicdhgrmono_address = graphicdhgrmono_address;
  memcpy(window_graphicdhgrmono->graphicdhgrmono_data,graphicdhgrmono_data,80*192);

  /* Donnes diffrentes */
  return(1);
}


/*********************************************************************************/
/*  ResizeGraphicDHGRMonoWindow() : Resize GraphicDHGRMono window by given size. */
/*********************************************************************************/
static void ResizeGraphicDHGRMonoWindow(struct window_graphicdhgrmono *window_graphicdhgrmono)
{
  HWND hwnd_group;
  RECT window_rect, client_rect, group_rect;
  int window_x, window_y;
  
  /** On va ajuster la fentre par rapport au Zoom demand **/  
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_graphicdhgrmono->hwnd_graphicdhgrmono,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_graphicdhgrmono->hwnd_graphicdhgrmono,&client_rect);
  window_graphicdhgrmono->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select */
  hwnd_group = GetDlgItem(window_graphicdhgrmono->hwnd_graphicdhgrmono,IDC_GROUP_GRAPHICDHGRMONO);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_GRAPHICDHGRMONO */
  GetWindowRect(hwnd_group,&group_rect);
  window_graphicdhgrmono->group_data_x = (group_rect.left - window_rect.left) - 3;
  window_graphicdhgrmono->group_data_y = ((group_rect.top - window_rect.top) - window_graphicdhgrmono->title_height) + 3;

  /* Position de la Zone BipMap dans la fenetre */
  window_graphicdhgrmono->bitmap_graphicdhgrmono_x = window_graphicdhgrmono->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_graphicdhgrmono->bitmap_graphicdhgrmono_y = window_graphicdhgrmono->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_graphicdhgrmono->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_graphicdhgrmono->bitmap_graphicdhgrmono_width*window_graphicdhgrmono->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_graphicdhgrmono->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicdhgrmono->bitmap_graphicdhgrmono_height*window_graphicdhgrmono->zoom_level*2 + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_graphicdhgrmono->group_data_x,window_graphicdhgrmono->group_data_y,window_graphicdhgrmono->group_data_width,window_graphicdhgrmono->group_data_height,TRUE);

  /* Largeur de la fentre */
  window_graphicdhgrmono->window_width = window_graphicdhgrmono->group_data_x + window_graphicdhgrmono->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* On doit garantir une largeur minimale de la fentre */
  if(window_graphicdhgrmono->window_width < GRAPHICDHGRMONO_DIALOGBOX_WIDTH)
    window_graphicdhgrmono->window_width = GRAPHICDHGRMONO_DIALOGBOX_WIDTH;

  /* Hauteur de la fentre */
  window_graphicdhgrmono->window_height = window_graphicdhgrmono->title_height + window_graphicdhgrmono->group_data_y + window_graphicdhgrmono->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Resize window */
  MoveWindow(window_graphicdhgrmono->hwnd_graphicdhgrmono,window_x,window_y,window_graphicdhgrmono->window_width,window_graphicdhgrmono->window_height,TRUE);
}


/*****************************************************************************************************************/
/*  BuildGraphicDHGRMonoFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/*****************************************************************************************************************/
static void BuildGraphicDHGRMonoFromData(struct window_graphicdhgrmono *window_graphicdhgrmono)
{
  int one_byte;
  int x, y, offset_pixmap_screen;
  char buffer_title[256];
  DWORD tab_rgb_color[16] = {MAKECOLOR(0,0,0), MAKECOLOR(255,255,255)};
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rien  faire */
  if(window_graphicdhgrmono->bitmap_data_graphicdhgrmono == NULL)
    return;    

  /** Cration de la bitmap 560*192 correspondant au 80*48 du DHGR Mono **/
  for(y=0; y<192; y++)
    for(x=0; x<80; x++)
      {
        /* 1 Byte = 7 Pixels */
        one_byte = (int) window_graphicdhgrmono->graphicdhgrmono_data[x + y*80];

        /* Adresse du block de 7 pixels  l'cran */
        offset_pixmap_screen = x*7 + y*560;
        window_graphicdhgrmono->bitmap_data_graphicdhgrmono[offset_pixmap_screen + 0] = tab_rgb_color[(one_byte >> 0) & 0x01];
        window_graphicdhgrmono->bitmap_data_graphicdhgrmono[offset_pixmap_screen + 1] = tab_rgb_color[(one_byte >> 1) & 0x01];
        window_graphicdhgrmono->bitmap_data_graphicdhgrmono[offset_pixmap_screen + 2] = tab_rgb_color[(one_byte >> 2) & 0x01];
        window_graphicdhgrmono->bitmap_data_graphicdhgrmono[offset_pixmap_screen + 3] = tab_rgb_color[(one_byte >> 3) & 0x01];
        window_graphicdhgrmono->bitmap_data_graphicdhgrmono[offset_pixmap_screen + 4] = tab_rgb_color[(one_byte >> 4) & 0x01];
        window_graphicdhgrmono->bitmap_data_graphicdhgrmono[offset_pixmap_screen + 5] = tab_rgb_color[(one_byte >> 5) & 0x01];
        window_graphicdhgrmono->bitmap_data_graphicdhgrmono[offset_pixmap_screen + 6] = tab_rgb_color[(one_byte >> 6) & 0x01];
      }

  /* Titre de la fentre */
  sprintf(buffer_title,"Graphic DHGR Mono : %02X/%04X",window_graphicdhgrmono->graphicdhgrmono_bank,window_graphicdhgrmono->graphicdhgrmono_address);
  SetWindowText(window_graphicdhgrmono->hwnd_graphicdhgrmono,buffer_title);
}


/*****************************************************************************/
/*  DeleteGraphicDHGRMonoChildWindows() :  Destruction des fentres enfants. */
/*****************************************************************************/
static void DeleteGraphicDHGRMonoChildWindows(struct window_graphicdhgrmono *window_graphicdhgrmono)
{
  DeleteObject(window_graphicdhgrmono->hBitmap_graphicdhgrmono);
  ReleaseDC(window_graphicdhgrmono->hwnd_graphicdhgrmono,window_graphicdhgrmono->hDC);
  DeleteDC(window_graphicdhgrmono->hCompatibleDC);
}


/*******************************************************************************/
/*  mem_free_window_graphicdhgrmono() :  Libration mmoire pour la structure. */
/*******************************************************************************/
void mem_free_window_graphicdhgrmono(struct window_graphicdhgrmono *window_graphicdhgrmono)
{
  if(window_graphicdhgrmono)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicdhgrmono->current_notification,NULL);
      window_graphicdhgrmono->current_notification = NULL;
      
      free(window_graphicdhgrmono);
    }
}

/*******************************************************************************/
