/************************************************************************/
/*                                                                      */
/*  Window_GraphicGR.c :  Module de gestion de la fentre GraphicGR.    */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Juillet 2018  */
/************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_GraphicGR.h"

static DWORD BuildGraphicGRWindowThread(void *);
LRESULT CALLBACK ProcHandlerGraphicGRWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateGraphicGRChildWindows(HWND,struct window_graphicgr *);
LRESULT CALLBACK subEditGraphicGRProcControl(HWND,UINT,WPARAM,LPARAM);
void UpdateGraphicGRClientAllArea(HDC,struct window_graphicgr *);
static void UpdateGraphicGRClientBitmapArea(HDC,struct window_graphicgr *);
static void UpdateGraphicGRContextInformation(struct window_graphicgr *,int,int);
static void UpdateGraphicGRAddress(struct window_graphicgr *,int);
static void UpdateGraphicGRRefreshMode(struct window_graphicgr *,int);
static void UpdateGraphicGRZoomLevel(struct window_graphicgr *,int);
static void RefreshGraphicGRWindowData(struct window_graphicgr *,int);
static int IsGraphicGRDataModified(int,int,int,int,struct window_graphicgr *);
static void ResizeGraphicGRWindow(struct window_graphicgr *);
static void BuildGraphicGRFromData(struct window_graphicgr *);
static void DeleteGraphicGRChildWindows(struct window_graphicgr *);


/********************************************************************************/
/*  BuildGraphicGRWindow() :  Construction / Affichage de la fentre GraphicGR. */
/********************************************************************************/
struct window_graphicgr *BuildGraphicGRWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_graphicgr *window_graphicgr;

  /** Allocation de la structure window_graphicgr **/
  window_graphicgr = (struct window_graphicgr *) calloc(1,sizeof(struct window_graphicgr));
  if(window_graphicgr == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_GRAPHICGR,window_graphicgr,NULL);

  /* Lance un thread pour grer la fentre GraphicGR */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildGraphicGRWindowThread,window_graphicgr,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_graphicgr);
}


/******************************************************************************************/
/*  BuildGraphicGRWindowThread() :  Cration de la fentre GraphicGR via un Thread ddi. */
/******************************************************************************************/
static DWORD BuildGraphicGRWindowThread(void *window_graphicgr_param)
{
  HWND hwndGraphicGR;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_graphicgr *window_graphicgr;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_graphicgr = (struct window_graphicgr *) window_graphicgr_param;
  window_graphicgr->thread_id = GetCurrentThreadId();         /* Le ThreaID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Graphic GR **/
  hwndGraphicGR = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_GRAPHICGR),window_main->hwnd_main,(DLGPROC)ProcHandlerGraphicGRWindow);
  if(hwndGraphicGR == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndGraphicGR,SW_SHOWDEFAULT);
  UpdateWindow(hwndGraphicGR);

  /** Message Loop pour toutes les fentres de ce Thread (fentre GraphicGR + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/**************************************************************************/
/*  ProcHandlerGraphicGRWindow() :  Gestion des vnements de la fentre. */
/**************************************************************************/
LRESULT CALLBACK ProcHandlerGraphicGRWindow(HWND hwndGraphicGR, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;
  int mouse_x, mouse_y;
  struct window_graphicgr *window_graphicgr;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_GRAPHICGR,&window_graphicgr,NULL);
  if(window_graphicgr == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateGraphicGRChildWindows(hwndGraphicGR,window_graphicgr);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        UpdateGraphicGRContextInformation(window_graphicgr,mouse_x,mouse_y);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_graphicgr->hBitmap_graphicgr != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndGraphicGR,&ps);

            /** MAJ Bitmap **/
            UpdateGraphicGRClientAllArea(hDC,window_graphicgr);

            /* End Paint */
            EndPaint(hwndGraphicGR,&ps);
          }
        break;
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicgr->refresh_mode != GRAPHICGR_REFRESH_AUTOMATIC)
                UpdateGraphicGRRefreshMode(window_graphicgr,GRAPHICGR_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicgr->refresh_mode != GRAPHICGR_REFRESH_MANUAL)
                UpdateGraphicGRRefreshMode(window_graphicgr,GRAPHICGR_REFRESH_MANUAL);
              break;

            /** Bouton Radio : On change le Zoom de la fentre **/
            case IDC_RADIO_ZOOM_X1 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicgr->zoom_level != GRAPHICGR_ZOOM_X1)
                UpdateGraphicGRZoomLevel(window_graphicgr,GRAPHICGR_ZOOM_X1);
              break;

            case IDC_RADIO_ZOOM_X2 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicgr->zoom_level != GRAPHICGR_ZOOM_X2)
                UpdateGraphicGRZoomLevel(window_graphicgr,GRAPHICGR_ZOOM_X2);
              break;

            case IDC_RADIO_ZOOM_X3 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicgr->zoom_level != GRAPHICGR_ZOOM_X3)
                UpdateGraphicGRZoomLevel(window_graphicgr,GRAPHICGR_ZOOM_X3);
              break;

            /** Bouton de dplacement de l'adresse **/
            case IDC_BUTTON_LEFT :
              UpdateGraphicGRAddress(window_graphicgr,-1);
              break;

            case IDC_BUTTON_RIGHT :
              UpdateGraphicGRAddress(window_graphicgr,1);
              break;

            case IDC_BUTTON_UP :
              UpdateGraphicGRAddress(window_graphicgr,-40);
              break;

            case IDC_BUTTON_DOWN :
              UpdateGraphicGRAddress(window_graphicgr,40);
              break;

            case IDC_BUTTON_PAGEUP :
              UpdateGraphicGRAddress(window_graphicgr,-1024);
              break;

            case IDC_BUTTON_PAGEDOWN :
              UpdateGraphicGRAddress(window_graphicgr,1024);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshGraphicGRWindowData(window_graphicgr,1);
              break;              
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_GRAPHICGR_REFRESHWINDOW :
        RefreshGraphicGRWindowData(window_graphicgr,0);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre GraphicGR */
        DestroyWindow(hwndGraphicGR);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteGraphicGRChildWindows(window_graphicgr);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_GRAPHICGR,window_graphicgr,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*********************************************************************************************************/
/*  CreateGraphicGRChildWindows() : Crer les fentres enfants et remplir la structure window_graphicgr. */
/*********************************************************************************************************/
static int CreateGraphicGRChildWindows(HWND hwndGraphicGR, struct window_graphicgr *window_graphicgr)
{
  BITMAPINFO bitmap_info;
  int i;
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_graphicgr->hwnd_graphicgr = hwndGraphicGR;

  /** Device Context **/
  window_graphicgr->hDC = GetDC(window_graphicgr->hwnd_graphicgr);
  window_graphicgr->hCompatibleDC = CreateCompatibleDC(window_graphicgr->hDC);

  /** Ajoute les images sur les boutons **/
  SendMessage(GetDlgItem(hwndGraphicGR,IDC_BUTTON_LEFT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicGR,IDC_BUTTON_RIGHT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicGR,IDC_BUTTON_UP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicGR,IDC_BUTTON_DOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicGR,IDC_BUTTON_PAGEUP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicGR,IDC_BUTTON_PAGEDOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Adresse de la Zone de Data **/
  window_graphicgr->graphicgr_bank = 0x00;
  window_graphicgr->graphicgr_address = 0x0400; 
  sprintf(buffer,"%02X",window_graphicgr->graphicgr_bank);
  SetDlgItemText(window_graphicgr->hwnd_graphicgr,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",window_graphicgr->graphicgr_address);
  SetDlgItemText(window_graphicgr->hwnd_graphicgr,IDC_EDIT_ADDRESS,buffer);

  /* Refresh Mode : Automatic */
  UpdateGraphicGRRefreshMode(window_graphicgr,GRAPHICGR_REFRESH_AUTOMATIC);

  /* Zoom Level : 1 */
  window_graphicgr->zoom_level = GRAPHICGR_ZOOM_X1;
  SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);

  /* Dimension en Pixel de la Bitmap GraphicGR Data */
  window_graphicgr->bitmap_graphicgr_width = 280;
  window_graphicgr->bitmap_graphicgr_height = 192;
  
  /** Bitmap affichant la zone de slection **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicgr->bitmap_graphicgr_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicgr->bitmap_graphicgr_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicgr->hBitmap_graphicgr = CreateDIBSection(window_graphicgr->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicgr->bitmap_data_graphicgr),NULL,0);
  for(i=0; i<window_graphicgr->bitmap_graphicgr_width*window_graphicgr->bitmap_graphicgr_height; i++)
    window_graphicgr->bitmap_data_graphicgr[i] = BITMAP_BACKGROUND_COLOR;

  /* Retaille la fentre */
  ResizeGraphicGRWindow(window_graphicgr);

  /** Construction de la Page Texte + Rafraichissement de la fentre **/
  RefreshGraphicGRWindowData(window_graphicgr,1);
    
  /* OK */
  return(0);
}


/************************************************************************************/
/*  UpdateGraphicGRClientAllArea() :  Affichage des Bitmap de la fentre GraphicGR. */
/************************************************************************************/
void UpdateGraphicGRClientAllArea(HDC hDC, struct window_graphicgr *window_graphicgr)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap **/
  UpdateGraphicGRClientBitmapArea(hDC,window_graphicgr);
}


/***************************************************************************/
/*  UpdateGraphicGRClientBitmapArea() :  Affichage de la Bitmap GraphicGR. */
/***************************************************************************/
static void UpdateGraphicGRClientBitmapArea(HDC hDC, struct window_graphicgr *window_graphicgr)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec la zone mmoire et la zone slectionne **/
  bitmap_old = SelectObject(window_graphicgr->hCompatibleDC,window_graphicgr->hBitmap_graphicgr);
  StretchBlt(hDC,window_graphicgr->bitmap_graphicgr_x,window_graphicgr->bitmap_graphicgr_y,
             window_graphicgr->bitmap_graphicgr_width*window_graphicgr->zoom_level,
             window_graphicgr->bitmap_graphicgr_height*window_graphicgr->zoom_level,
             window_graphicgr->hCompatibleDC,0,0,window_graphicgr->bitmap_graphicgr_width,window_graphicgr->bitmap_graphicgr_height,SRCCOPY);
  SelectObject(window_graphicgr->hCompatibleDC,bitmap_old);
}


/*****************************************************************************/
/*  UpdateGraphicGRAddress() :  Dplace l'adresse pour la fentre GraphicGR. */
/*****************************************************************************/
static void UpdateGraphicGRAddress(struct window_graphicgr *window_graphicgr, int delta)
{
  int current_address, new_address, graphicgr_bank, graphicgr_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphicgr->graphicgr_bank << 16) | window_graphicgr->graphicgr_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  graphicgr_bank = (new_address >> 16) & 0xFF;
  graphicgr_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphicgr_bank);
  SetDlgItemText(window_graphicgr->hwnd_graphicgr,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",graphicgr_address);
  SetDlgItemText(window_graphicgr->hwnd_graphicgr,IDC_EDIT_ADDRESS,buffer);

  /* MAJ de la fentre */
  RefreshGraphicGRWindowData(window_graphicgr,1);
}


/*****************************************************************************************************/
/*  UpdateGraphicGRContextInformation() :  MAJ des information de contexte de la fenetre GraphicGR. */
/*****************************************************************************************************/
static void UpdateGraphicGRContextInformation(struct window_graphicgr *window_graphicgr, int mouse_x, int mouse_y)
{
  int pixmap_x, pixmap_y, grid_x, grid_y;
  int graphicgr_data_address;
  unsigned char graphicgr_data;
  int tab_text40_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };

  /* Rien  faire */
  if(window_graphicgr == NULL)
    return;

  /* Est t'on dans l'image ? */
  if(mouse_x < window_graphicgr->bitmap_graphicgr_x || mouse_x >= (window_graphicgr->bitmap_graphicgr_x + window_graphicgr->bitmap_graphicgr_width*window_graphicgr->zoom_level) ||
     mouse_y < window_graphicgr->bitmap_graphicgr_y || mouse_y >= (window_graphicgr->bitmap_graphicgr_y + window_graphicgr->bitmap_graphicgr_height*window_graphicgr->zoom_level))
    {
      SetWindowText(GetDlgItem(window_graphicgr->hwnd_graphicgr,IDC_STATIC_CONTEXT),"");
      return;
    }

  /* Coordonnes dans la Pixmap */
  pixmap_x = (mouse_x - window_graphicgr->bitmap_graphicgr_x)/window_graphicgr->zoom_level;
  pixmap_y = (mouse_y - window_graphicgr->bitmap_graphicgr_y)/window_graphicgr->zoom_level;

  /* Coordonnes dans la grille 40x24 */
  grid_x = pixmap_x/7;
  grid_y = pixmap_y/8;

  /* Valeur */
  graphicgr_data = window_graphicgr->graphicgr_data[grid_x + grid_y*40];

  /* Adresse */
  graphicgr_data_address = (window_graphicgr->graphicgr_bank << 16) + window_graphicgr->graphicgr_address;
  graphicgr_data_address += grid_x;
  graphicgr_data_address += tab_text40_address[grid_y];

  /* Message */
  sprintf(window_graphicgr->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicgr_data_address>>16,graphicgr_data_address&0xFFFF,graphicgr_data,grid_x,pixmap_y/4);
  SetWindowText(GetDlgItem(window_graphicgr->hwnd_graphicgr,IDC_STATIC_CONTEXT),window_graphicgr->buffer_log);
}


/**********************************************************************************************/
/*  UpdateGraphicGRRefreshMode() :  MAJ du mode de rafraichissement de la fenetre GraphicGR. */
/**********************************************************************************************/
static void UpdateGraphicGRRefreshMode(struct window_graphicgr *window_graphicgr, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == GRAPHICGR_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_graphicgr->refresh_mode = GRAPHICGR_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_graphicgr->hwnd_graphicgr;
      current_notification->message = HM_GRAPHICGR_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_GRAPHICGR;

      /* Ajout de la notification */
      window_graphicgr->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_graphicgr->refresh_mode = GRAPHICGR_REFRESH_MANUAL;
      SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicgr->current_notification,NULL);
      window_graphicgr->current_notification = NULL;
    }
}


/*********************************************************************************/
/*  UpdateGraphicGRZoomLevel() :  MAJ du nieau de Zomm de la fenetre GraphicGR. */
/*********************************************************************************/
static void UpdateGraphicGRZoomLevel(struct window_graphicgr *window_graphicgr, int zoom_level)
{
  if(zoom_level == GRAPHICGR_ZOOM_X1)
    {
      /* Visuel */
      window_graphicgr->zoom_level = GRAPHICGR_ZOOM_X1;
      SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICGR_ZOOM_X2)
    {
      /* Visuel */
      window_graphicgr->zoom_level = GRAPHICGR_ZOOM_X2;
      SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICGR_ZOOM_X3)
    {
      /* Visuel */
      window_graphicgr->zoom_level = GRAPHICGR_ZOOM_X3;
      SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicgr->hwnd_graphicgr,IDC_RADIO_ZOOM_X3,BM_SETCHECK,1,0);
    }
    
  /* Retaille la fentre */
  ResizeGraphicGRWindow(window_graphicgr);  
  
  /* On force la MAJ de la fentre */  
  RefreshGraphicGRWindowData(window_graphicgr,1);    
}


/****************************************************************************/
/*  RefreshGraphicGRWindowData() :  MAJ du contenu de la fentre GraphicGR. */
/****************************************************************************/
static void RefreshGraphicGRWindowData(struct window_graphicgr *window_graphicgr, int force)
{
  int length, is_modified;
  int graphicgr_bank, graphicgr_address, nb_column, nb_row;
  char buffer[1024];

  /* Rcupre la valeur du GraphicGR Pointer */
  length = GetDlgItemText(window_graphicgr->hwnd_graphicgr,IDC_EDIT_BANK,buffer,1024);
  sscanf(buffer,"%X",&graphicgr_bank);
  length = GetDlgItemText(window_graphicgr->hwnd_graphicgr,IDC_EDIT_ADDRESS,buffer,1024);
  sscanf(buffer,"%X",&graphicgr_address);
  nb_column = GetDlgItemInt(window_graphicgr->hwnd_graphicgr,IDC_EDIT_COLUMN,NULL,FALSE);
  if(nb_column < 1)
    nb_column = 1;
  nb_row = GetDlgItemInt(window_graphicgr->hwnd_graphicgr,IDC_EDIT_ROW,NULL,FALSE);
  if(nb_row < 1)
    nb_row = 1;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphicgr_bank);
  SetDlgItemText(window_graphicgr->hwnd_graphicgr,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",graphicgr_address);
  SetDlgItemText(window_graphicgr->hwnd_graphicgr,IDC_EDIT_ADDRESS,buffer);
  SetDlgItemInt(window_graphicgr->hwnd_graphicgr,IDC_EDIT_COLUMN,nb_column,FALSE);
  SetDlgItemInt(window_graphicgr->hwnd_graphicgr,IDC_EDIT_ROW,nb_row,FALSE);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsGraphicGRDataModified(graphicgr_bank,graphicgr_address,nb_column,nb_row,window_graphicgr);
  if(is_modified == 0 && force == 0)
    return;

  /** Cration de la Zone Text Graphique  partir des nouvelles donnes **/
  BuildGraphicGRFromData(window_graphicgr);

  /* Rafraichit la fentre */
  InvalidateRect(window_graphicgr->hwnd_graphicgr,NULL,TRUE);
  UpdateWindow(window_graphicgr->hwnd_graphicgr);
}


/***********************************************************************************************************/
/*  IsGraphicGRDataModified() :  Les donnes GraphicGR ont t'elles t modifies depuis la dernirer fois. */
/***********************************************************************************************************/
static int IsGraphicGRDataModified(int graphicgr_bank, int graphicgr_address, int nb_column, int nb_row, struct window_graphicgr *window_graphicgr)
{
  int x, y, graphicgr_data_address;
  unsigned char graphicgr_raw_data[1024];
  unsigned char graphicgr_data[40*24];
  int tab_text40_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /* Rcupre les donnes brutes : 1024 chars */
  graphicgr_data_address = (graphicgr_bank << 16) | graphicgr_address;
  ReadMemoryBlock(graphicgr_data_address,1024,graphicgr_raw_data,current_mmu);

  /* R-organise les donnes : Linarisation (passage du block de 1024 char en 40*24) */
  for(y=0; y<24; y++)
    for(x=0; x<40; x++)
      graphicgr_data[x+y*40] = graphicgr_raw_data[tab_text40_address[y]+x];

  /* Compare les donnes */
  if(window_graphicgr->graphicgr_data != NULL)
    if(graphicgr_bank == window_graphicgr->graphicgr_bank &&
       graphicgr_address == window_graphicgr->graphicgr_address &&
       memcmp(&graphicgr_data[0],&window_graphicgr->graphicgr_data[0],40*24) == 0)
      return(0);

  /* Les donnes sont diffrentes => MAJ */
  window_graphicgr->graphicgr_bank = graphicgr_bank;
  window_graphicgr->graphicgr_address = graphicgr_address;
  memcpy(window_graphicgr->graphicgr_data,graphicgr_data,40*24);

  /* Donnes diffrentes */
  return(1);
}


/*********************************************************************/
/*  ResizeGraphicGRWindow() : Resize GraphicGR window by given size. */
/*********************************************************************/
static void ResizeGraphicGRWindow(struct window_graphicgr *window_graphicgr)
{
  HWND hwnd_group;
  RECT window_rect, client_rect, group_rect;
  int window_x, window_y;
  
  /** On va ajuster la fentre par rapport au Zoom demand **/  
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_graphicgr->hwnd_graphicgr,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_graphicgr->hwnd_graphicgr,&client_rect);
  window_graphicgr->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select */
  hwnd_group = GetDlgItem(window_graphicgr->hwnd_graphicgr,IDC_GROUP_GRAPHICGR);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_GRAPHICGR */
  GetWindowRect(hwnd_group,&group_rect);
  window_graphicgr->group_data_x = (group_rect.left - window_rect.left) - 3;
  window_graphicgr->group_data_y = ((group_rect.top - window_rect.top) - window_graphicgr->title_height) + 3;

  /* Position de la Zone BipMap dans la fenetre */
  window_graphicgr->bitmap_graphicgr_x = window_graphicgr->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_graphicgr->bitmap_graphicgr_y = window_graphicgr->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_graphicgr->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_graphicgr->bitmap_graphicgr_width*window_graphicgr->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_graphicgr->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicgr->bitmap_graphicgr_height*window_graphicgr->zoom_level + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_graphicgr->group_data_x,window_graphicgr->group_data_y,window_graphicgr->group_data_width,window_graphicgr->group_data_height,TRUE);

  /* Largeur de la fentre */
  window_graphicgr->window_width = window_graphicgr->group_data_x + window_graphicgr->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* On doit garantir une largeur minimale de la fentre */
  if(window_graphicgr->window_width < GRAPHICGR_DIALOGBOX_WIDTH)
    window_graphicgr->window_width = GRAPHICGR_DIALOGBOX_WIDTH;

  /* Hauteur de la fentre */
  window_graphicgr->window_height = window_graphicgr->title_height + window_graphicgr->group_data_y + window_graphicgr->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Resize window */
  MoveWindow(window_graphicgr->hwnd_graphicgr,window_x,window_y,window_graphicgr->window_width,window_graphicgr->window_height,TRUE);
}


/***********************************************************************************************************/
/*  BuildGraphicGRFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/***********************************************************************************************************/
static void BuildGraphicGRFromData(struct window_graphicgr *window_graphicgr)
{
  int one_byte;
  int x, y, pixel_x, pixel_y, offset_pixmap_screen;
  char buffer_title[256];
  DWORD color_screen;
  DWORD tab_rgb_color[16] = 
    {
      /* 16 couleurs pour le mode GR */
      MAKECOLOR(0,0,0), MAKECOLOR(221,0,51), MAKECOLOR(0,0,153), MAKECOLOR(221,0,221),
      MAKECOLOR(0,119,0), MAKECOLOR(85,85,85), MAKECOLOR(34,34,255), MAKECOLOR(102,170,255),
      MAKECOLOR(136,85,34), MAKECOLOR(255,102,0), MAKECOLOR(170,170,170), MAKECOLOR(255,153,136),
      MAKECOLOR(0,221,0), MAKECOLOR(255,255,0), MAKECOLOR(0,255,153), MAKECOLOR(255,255,255)
  };
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rien  faire */
  if(window_graphicgr->bitmap_data_graphicgr == NULL)
    return;    

  /** Cration de la bitmap 280*192 correspondant au 40*48 du GR **/
  for(y=0; y<24; y++)
    {
      for(x=0; x<40; x++)
        {
          /* Caractre */
          one_byte = (int) window_graphicgr->graphicgr_data[x + y*40];

          /* Adresse du caractre  l'cran */
          offset_pixmap_screen = x*7 + y*(40*7)*8;

          /* Bloc Haut de 7*4 */
          color_screen = tab_rgb_color[one_byte & 0x0F];    /* Couleur du Pixel final */
          for(pixel_y=0; pixel_y<4; pixel_y++)
            for(pixel_x=0; pixel_x<7; pixel_x++)
              window_graphicgr->bitmap_data_graphicgr[offset_pixmap_screen + pixel_x + pixel_y*40*7] = color_screen;

          /* Bloc Bas de 7*4 */
          color_screen = tab_rgb_color[one_byte >> 4];      /* Couleur du Pixel final */
          for(pixel_y=4; pixel_y<8; pixel_y++)
            for(pixel_x=0; pixel_x<7; pixel_x++)
              window_graphicgr->bitmap_data_graphicgr[offset_pixmap_screen + pixel_x + pixel_y*40*7] = color_screen;
        }
    }

  /* Titre de la fentre */
  sprintf(buffer_title,"Graphic GR : %02X/%04X",window_graphicgr->graphicgr_bank,window_graphicgr->graphicgr_address);
  SetWindowText(window_graphicgr->hwnd_graphicgr,buffer_title);
}


/***********************************************************************/
/*  DeleteGraphicGRChildWindows() :  Destruction des fentres enfants. */
/***********************************************************************/
static void DeleteGraphicGRChildWindows(struct window_graphicgr *window_graphicgr)
{
  DeleteObject(window_graphicgr->hBitmap_graphicgr);
  ReleaseDC(window_graphicgr->hwnd_graphicgr,window_graphicgr->hDC);
  DeleteDC(window_graphicgr->hCompatibleDC);
}


/*************************************************************************/
/*  mem_free_window_graphicgr() :  Libration mmoire pour la structure. */
/*************************************************************************/
void mem_free_window_graphicgr(struct window_graphicgr *window_graphicgr)
{
  if(window_graphicgr)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicgr->current_notification,NULL);
      window_graphicgr->current_notification = NULL;
      
      free(window_graphicgr);
    }
}

/**********************************************************************/
