/**********************************************************************************/
/*                                                                                */
/*  Window_GraphicGRText40.c :  Module de gestion de la fentre GraphicGRText40.  */
/*                                                                                */
/**********************************************************************************/
/*    Auteur : Olivier ZARDINI   *     Brutal Deluxe     *   Date : Juillet 2018  */
/**********************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_GraphicGRText40.h"

static DWORD BuildGraphicGRText40WindowThread(void *);
LRESULT CALLBACK ProcHandlerGraphicGRText40Window(HWND,UINT,WPARAM,LPARAM);
static int CreateGraphicGRText40ChildWindows(HWND,struct window_graphicgrtext40 *);
LRESULT CALLBACK subEditGraphicGRText40ProcControl(HWND,UINT,WPARAM,LPARAM);
void UpdateGraphicGRText40ClientAllArea(HDC,struct window_graphicgrtext40 *);
static void UpdateGraphicGRText40ClientBitmapArea(HDC,struct window_graphicgrtext40 *);
static void UpdateGraphicGRText40ContextInformation(struct window_graphicgrtext40 *,int,int);
static void UpdateGraphicGRText40AddressGraphic(struct window_graphicgrtext40 *,int);
static void UpdateGraphicGRText40AddressText(struct window_graphicgrtext40 *,int);
static void UpdateGraphicGRText40RefreshMode(struct window_graphicgrtext40 *,int);
static void UpdateGraphicGRText40FlashMode(struct window_graphicgrtext40 *);
static void UpdateGraphicGRText40ZoomLevel(struct window_graphicgrtext40 *,int);
static void RefreshGraphicGRText40WindowData(struct window_graphicgrtext40 *,int);
static int IsGraphicGRText40DataModified(int,int,int,int,struct window_graphicgrtext40 *);
static void ResizeGraphicGRText40Window(struct window_graphicgrtext40 *);
static void BuildGraphicGRText40FromData(struct window_graphicgrtext40 *);
static void DeleteGraphicGRText40ChildWindows(struct window_graphicgrtext40 *);


/********************************************************************************************/
/*  BuildGraphicGRText40Window() :  Construction / Affichage de la fentre GraphicGRText40. */
/********************************************************************************************/
struct window_graphicgrtext40 *BuildGraphicGRText40Window(HINSTANCE hInstance, int menu_id)
{
  struct window_graphicgrtext40 *window_graphicgrtext40;

  /** Allocation de la structure window_graphicgrtext40 **/
  window_graphicgrtext40 = (struct window_graphicgrtext40 *) calloc(1,sizeof(struct window_graphicgrtext40));
  if(window_graphicgrtext40 == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_GRAPHICGRTEXT40,window_graphicgrtext40,NULL);

  /* Lance un thread pour grer la fentre GraphicGRText40 */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildGraphicGRText40WindowThread,window_graphicgrtext40,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_graphicgrtext40);
}


/******************************************************************************************************/
/*  BuildGraphicGRText40WindowThread() :  Cration de la fentre GraphicGRText40 via un Thread ddi. */
/******************************************************************************************************/
static DWORD BuildGraphicGRText40WindowThread(void *window_graphicgrtext40_param)
{
  HWND hwndGraphicGRText40;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_graphicgrtext40 *window_graphicgrtext40;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_graphicgrtext40 = (struct window_graphicgrtext40 *) window_graphicgrtext40_param;
  window_graphicgrtext40->thread_id = GetCurrentThreadId();         /* Le ThreaID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Text Page 40 Column **/
  hwndGraphicGRText40 = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_GRAPHICGRTEXT40),window_main->hwnd_main,(DLGPROC)ProcHandlerGraphicGRText40Window);
  if(hwndGraphicGRText40 == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndGraphicGRText40,SW_SHOWDEFAULT);
  UpdateWindow(hwndGraphicGRText40);

  /** Message Loop pour toutes les fentres de ce Thread (fentre GraphicGRText40 + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/********************************************************************************/
/*  ProcHandlerGraphicGRText40Window() :  Gestion des vnements de la fentre. */
/********************************************************************************/
LRESULT CALLBACK ProcHandlerGraphicGRText40Window(HWND hwndGraphicGRText40, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;
  LRESULT result;
  WORD param_wd;
  int mouse_x, mouse_y;
  struct window_graphicgrtext40 *window_graphicgrtext40;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_GRAPHICGRTEXT40,&window_graphicgrtext40,NULL);
  if(window_graphicgrtext40 == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateGraphicGRText40ChildWindows(hwndGraphicGRText40,window_graphicgrtext40);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        UpdateGraphicGRText40ContextInformation(window_graphicgrtext40,mouse_x,mouse_y);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_graphicgrtext40->hBitmap_graphicgrtext40 != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndGraphicGRText40,&ps);

            /** MAJ Bitmap **/
            UpdateGraphicGRText40ClientAllArea(hDC,window_graphicgrtext40);

            /* End Paint */
            EndPaint(hwndGraphicGRText40,&ps);
          }
        break;
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicgrtext40->refresh_mode != GRAPHICGRTEXT40_REFRESH_AUTOMATIC)
                UpdateGraphicGRText40RefreshMode(window_graphicgrtext40,GRAPHICGRTEXT40_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicgrtext40->refresh_mode != GRAPHICGRTEXT40_REFRESH_MANUAL)
                UpdateGraphicGRText40RefreshMode(window_graphicgrtext40,GRAPHICGRTEXT40_REFRESH_MANUAL);
              break;

            /** Bouton Radio : On change le Zoom de la fentre **/
            case IDC_RADIO_ZOOM_X1 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicgrtext40->zoom_level != GRAPHICGRTEXT40_ZOOM_X1)
                UpdateGraphicGRText40ZoomLevel(window_graphicgrtext40,GRAPHICGRTEXT40_ZOOM_X1);
              break;

            case IDC_RADIO_ZOOM_X2 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicgrtext40->zoom_level != GRAPHICGRTEXT40_ZOOM_X2)
                UpdateGraphicGRText40ZoomLevel(window_graphicgrtext40,GRAPHICGRTEXT40_ZOOM_X2);
              break;

            case IDC_RADIO_ZOOM_X3 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicgrtext40->zoom_level != GRAPHICGRTEXT40_ZOOM_X3)
                UpdateGraphicGRText40ZoomLevel(window_graphicgrtext40,GRAPHICGRTEXT40_ZOOM_X3);
              break;

            /** Bouton de dplacement de l'adresse Graphic **/
            case IDC_BUTTON_LEFT_GRAPHIC :
              UpdateGraphicGRText40AddressGraphic(window_graphicgrtext40,-1);
              break;

            case IDC_BUTTON_RIGHT_GRAPHIC :
              UpdateGraphicGRText40AddressGraphic(window_graphicgrtext40,1);
              break;

            case IDC_BUTTON_UP_GRAPHIC :
              UpdateGraphicGRText40AddressGraphic(window_graphicgrtext40,-40);
              break;

            case IDC_BUTTON_DOWN_GRAPHIC :
              UpdateGraphicGRText40AddressGraphic(window_graphicgrtext40,40);
              break;

            case IDC_BUTTON_PAGEUP_GRAPHIC :
              UpdateGraphicGRText40AddressGraphic(window_graphicgrtext40,-1024);
              break;

            case IDC_BUTTON_PAGEDOWN_GRAPHIC :
              UpdateGraphicGRText40AddressGraphic(window_graphicgrtext40,1024);
              break;

            /** Bouton de dplacement de l'adresse Text **/
            case IDC_BUTTON_LEFT_TEXT :
              UpdateGraphicGRText40AddressText(window_graphicgrtext40,-1);
              break;

            case IDC_BUTTON_RIGHT_TEXT :
              UpdateGraphicGRText40AddressText(window_graphicgrtext40,1);
              break;

            case IDC_BUTTON_UP_TEXT :
              UpdateGraphicGRText40AddressText(window_graphicgrtext40,-40);
              break;

            case IDC_BUTTON_DOWN_TEXT :
              UpdateGraphicGRText40AddressText(window_graphicgrtext40,40);
              break;

            case IDC_BUTTON_PAGEUP_TEXT :
              UpdateGraphicGRText40AddressText(window_graphicgrtext40,-1024);
              break;

            case IDC_BUTTON_PAGEDOWN_TEXT :
              UpdateGraphicGRText40AddressText(window_graphicgrtext40,1024);
              break;

            /** Demande d'activation du Flash **/
            case IDC_CHECK_FLASH :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateGraphicGRText40FlashMode(window_graphicgrtext40);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshGraphicGRText40WindowData(window_graphicgrtext40,1);
              break;
              
            /** Slection dans la ComboBox **/
            case IDC_COMBOBOX_GRAPHICGRTEXT40_COLOR_TEXT :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphicgrtext40->hwnd_combo_color_text,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphicgrtext40->color_text)
                      {
                        window_graphicgrtext40->color_text = (int) result;
                        RefreshGraphicGRText40WindowData(window_graphicgrtext40,1);
                      }
                }
              break;              
             
            case IDC_COMBOBOX_GRAPHICGRTEXT40_COLOR_BACKGROUND :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphicgrtext40->hwnd_combo_color_background,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphicgrtext40->color_background)
                      {
                        window_graphicgrtext40->color_background = (int) result;
                        RefreshGraphicGRText40WindowData(window_graphicgrtext40,1);
                      }
                }
              break; 
             
            case IDC_COMBOBOX_GRAPHICGRTEXT40_DISPLAY_LANGUAGE :
              param_wd = HIWORD(wParam);
              if(param_wd == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphicgrtext40->hwnd_combo_display_language,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphicgrtext40->display_language)
                      {
                        window_graphicgrtext40->display_language = (int) result;
                        RefreshGraphicGRText40WindowData(window_graphicgrtext40,1);
                      }
                }
              break;                           
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_GRAPHICGRTEXT40_REFRESHWINDOW :
        RefreshGraphicGRText40WindowData(window_graphicgrtext40,0);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre GraphicGRText40 */
        DestroyWindow(hwndGraphicGRText40);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteGraphicGRText40ChildWindows(window_graphicgrtext40);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_GRAPHICGRTEXT40,window_graphicgrtext40,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*********************************************************************************************************************/
/*  CreateGraphicGRText40ChildWindows() : Crer les fentres enfants et remplir la structure window_graphicgrtext40. */
/*********************************************************************************************************************/
static int CreateGraphicGRText40ChildWindows(HWND hwndGraphicGRText40, struct window_graphicgrtext40 *window_graphicgrtext40)
{
  BITMAPINFO bitmap_info;
  int i;
  char buffer[BUFFER_SIZE];
  char *color_name[16] = {"Black","Deep Red","Dark Blue","Purple","Dark Green","Dark Gray","Medium Blue","Light Blue","Brown","Orange","Light Gray","Pink","Light Green","Yellow","Aquamarine","White"};
  char *display_language[9] = {"U.S.A.","U.K.","French","Danish","Spanish","Italian","German","Swedish","MouseText"};
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_graphicgrtext40->hwnd_graphicgrtext40 = hwndGraphicGRText40;

  /** Device Context **/
  window_graphicgrtext40->hDC = GetDC(window_graphicgrtext40->hwnd_graphicgrtext40);
  window_graphicgrtext40->hCompatibleDC = CreateCompatibleDC(window_graphicgrtext40->hDC);

  /** Surchage des Control Edit pour capter les vnements **/
  window_graphicgrtext40->oldEditProcBankGraphic    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicGRText40,IDC_EDIT_BANK_GRAPHIC),GWLP_WNDPROC,(LONG_PTR)subEditGraphicGRText40ProcControl);
  window_graphicgrtext40->oldEditProcAddressGraphic = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicGRText40,IDC_EDIT_ADDRESS_GRAPHIC),GWLP_WNDPROC,(LONG_PTR)subEditGraphicGRText40ProcControl);
  window_graphicgrtext40->oldEditProcBankText       = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicGRText40,IDC_EDIT_BANK_TEXT),GWLP_WNDPROC,(LONG_PTR)subEditGraphicGRText40ProcControl);
  window_graphicgrtext40->oldEditProcAddressText    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicGRText40,IDC_EDIT_ADDRESS_TEXT),GWLP_WNDPROC,(LONG_PTR)subEditGraphicGRText40ProcControl);

  /** Ajoute les images sur les boutons **/
  /* Graphic */
  SendMessage(GetDlgItem(hwndGraphicGRText40,IDC_BUTTON_LEFT_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicGRText40,IDC_BUTTON_RIGHT_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicGRText40,IDC_BUTTON_UP_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicGRText40,IDC_BUTTON_DOWN_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicGRText40,IDC_BUTTON_PAGEUP_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicGRText40,IDC_BUTTON_PAGEDOWN_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));
  /* Text */
  SendMessage(GetDlgItem(hwndGraphicGRText40,IDC_BUTTON_LEFT_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicGRText40,IDC_BUTTON_RIGHT_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicGRText40,IDC_BUTTON_UP_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicGRText40,IDC_BUTTON_DOWN_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicGRText40,IDC_BUTTON_PAGEUP_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicGRText40,IDC_BUTTON_PAGEDOWN_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Cration des ComboBox **/
  /* Text Color */
  window_graphicgrtext40->hwnd_combo_color_text = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                  104,102,100,250,hwndGraphicGRText40,(HMENU)IDC_COMBOBOX_GRAPHICGRTEXT40_COLOR_TEXT,param->hInstance,0);
  for(i=0; i<16; i++)
    SendMessage(window_graphicgrtext40->hwnd_combo_color_text,CB_ADDSTRING,0,(LPARAM)color_name[i]);

  /* Background Color */
  window_graphicgrtext40->hwnd_combo_color_background = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                                        309,102,100,250,hwndGraphicGRText40,(HMENU)IDC_COMBOBOX_GRAPHICGRTEXT40_COLOR_BACKGROUND,param->hInstance,0);
  for(i=0; i<16; i++)
    SendMessage(window_graphicgrtext40->hwnd_combo_color_background,CB_ADDSTRING,0,(LPARAM)color_name[i]);

  /* Display Language */
  window_graphicgrtext40->hwnd_combo_display_language = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                                        309,134,100,220,hwndGraphicGRText40,(HMENU)IDC_COMBOBOX_GRAPHICGRTEXT40_DISPLAY_LANGUAGE,param->hInstance,0);
  for(i=0; i<9; i++)
    SendMessage(window_graphicgrtext40->hwnd_combo_display_language,CB_ADDSTRING,0,(LPARAM)display_language[i]);

  /* 1res valeurs */
  window_graphicgrtext40->color_text = COLOR_IIGS_WHITE;
  window_graphicgrtext40->color_background = COLOR_IIGS_MEDIUM_BLUE;
  window_graphicgrtext40->display_language = LANGUAGE_IIGS_USA;
  SendMessage(window_graphicgrtext40->hwnd_combo_color_text,CB_SETCURSEL,window_graphicgrtext40->color_text,0);
  SendMessage(window_graphicgrtext40->hwnd_combo_color_background,CB_SETCURSEL,window_graphicgrtext40->color_background,0);
  SendMessage(window_graphicgrtext40->hwnd_combo_display_language,CB_SETCURSEL,window_graphicgrtext40->display_language,0);

  /** Cration de la Police de caractre + Affectation de la police au Control Edit **/
  window_graphicgrtext40->hFont = CreateFont(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");
  SendMessage(window_graphicgrtext40->hwnd_combo_color_text,WM_SETFONT,(WPARAM)window_graphicgrtext40->hFont,TRUE);
  SendMessage(window_graphicgrtext40->hwnd_combo_color_background,WM_SETFONT,(WPARAM)window_graphicgrtext40->hFont,TRUE);
  SendMessage(window_graphicgrtext40->hwnd_combo_display_language,WM_SETFONT,(WPARAM)window_graphicgrtext40->hFont,TRUE);

  /** Adresse de la Zone de Data **/
  /* Graphic */
  window_graphicgrtext40->graphicgr_bank = 0x00;
  window_graphicgrtext40->graphicgr_address = 0x0400; 
  sprintf(buffer,"%02X",window_graphicgrtext40->graphicgr_bank);
  SetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_BANK_GRAPHIC,buffer);
  sprintf(buffer,"%04X",window_graphicgrtext40->graphicgr_address);
  SetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_ADDRESS_GRAPHIC,buffer);
  /* Text */
  window_graphicgrtext40->textpage40_bank = 0x00;
  window_graphicgrtext40->textpage40_address = 0x0400; 
  sprintf(buffer,"%02X",window_graphicgrtext40->textpage40_bank);
  SetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_BANK_TEXT,buffer);
  sprintf(buffer,"%04X",window_graphicgrtext40->textpage40_address);
  SetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_ADDRESS_TEXT,buffer);

  /* Refresh Mode : Automatic */
  UpdateGraphicGRText40RefreshMode(window_graphicgrtext40,GRAPHICGRTEXT40_REFRESH_AUTOMATIC);

  /* Zoom Level : 1 */
  window_graphicgrtext40->zoom_level = GRAPHICGRTEXT40_ZOOM_X1;
  SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);

  /* Fash des caractres */
  window_graphicgrtext40->flash = 0;

  /* Dimension en Pixel de la Bitmap GraphicGRText40 Data */
  window_graphicgrtext40->bitmap_graphicgrtext40_width = 280;
  window_graphicgrtext40->bitmap_graphicgrtext40_height = 192;
  
  /** Bitmap affichant la zone de slection **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicgrtext40->bitmap_graphicgrtext40_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicgrtext40->bitmap_graphicgrtext40_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicgrtext40->hBitmap_graphicgrtext40 = CreateDIBSection(window_graphicgrtext40->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicgrtext40->bitmap_data_graphicgrtext40),NULL,0);
  for(i=0; i<window_graphicgrtext40->bitmap_graphicgrtext40_width*window_graphicgrtext40->bitmap_graphicgrtext40_height; i++)
    window_graphicgrtext40->bitmap_data_graphicgrtext40[i] = BITMAP_BACKGROUND_COLOR;

  /* Retaille la fentre */
  ResizeGraphicGRText40Window(window_graphicgrtext40);

  /** Construction de la Page Texte + Rafraichissement de la fentre **/
  RefreshGraphicGRText40WindowData(window_graphicgrtext40,1);
    
  /* OK */
  return(0);
}


/******************************************************************************************************/
/*  subEditGraphicGRText40ProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/******************************************************************************************************/
LRESULT CALLBACK subEditGraphicGRText40ProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_graphicgrtext40 *window_graphicgrtext40;
  my_Memory(MEMORY_GET_WINDOW_GRAPHICGRTEXT40,&window_graphicgrtext40,NULL);

  /* Rien  faire */
  if(window_graphicgrtext40 == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_BANK_GRAPHIC) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_ADDRESS_GRAPHIC) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_BANK_TEXT) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_ADDRESS_TEXT) == hwnd_control)
    index = 4;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement du Return dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_graphicgrtext40->hwnd_graphicgrtext40);

      /* On force la MAJ de la fentre */
      RefreshGraphicGRText40WindowData(window_graphicgrtext40,1);
    }
  else if(index == 1)
    return(CallWindowProc(window_graphicgrtext40->oldEditProcBankGraphic,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_graphicgrtext40->oldEditProcAddressGraphic,hwnd_control,msg,wParam,lParam)); 
  else if(index == 3)
    return(CallWindowProc(window_graphicgrtext40->oldEditProcBankText,hwnd_control,msg,wParam,lParam)); 
  else if(index == 4)
    return(CallWindowProc(window_graphicgrtext40->oldEditProcAddressText,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
} 


/************************************************************************************************/
/*  UpdateGraphicGRText40ClientAllArea() :  Affichage des Bitmap de la fentre GraphicGRText40. */
/************************************************************************************************/
void UpdateGraphicGRText40ClientAllArea(HDC hDC, struct window_graphicgrtext40 *window_graphicgrtext40)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap **/
  UpdateGraphicGRText40ClientBitmapArea(hDC,window_graphicgrtext40);
}


/***************************************************************************************/
/*  UpdateGraphicGRText40ClientBitmapArea() :  Affichage de la Bitmap GraphicGRText40. */
/***************************************************************************************/
static void UpdateGraphicGRText40ClientBitmapArea(HDC hDC, struct window_graphicgrtext40 *window_graphicgrtext40)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec la zone mmoire et la zone slectionne **/
  bitmap_old = SelectObject(window_graphicgrtext40->hCompatibleDC,window_graphicgrtext40->hBitmap_graphicgrtext40);
  StretchBlt(hDC,window_graphicgrtext40->bitmap_graphicgrtext40_x,window_graphicgrtext40->bitmap_graphicgrtext40_y,
             window_graphicgrtext40->bitmap_graphicgrtext40_width*window_graphicgrtext40->zoom_level,
             window_graphicgrtext40->bitmap_graphicgrtext40_height*window_graphicgrtext40->zoom_level,
             window_graphicgrtext40->hCompatibleDC,0,0,window_graphicgrtext40->bitmap_graphicgrtext40_width,window_graphicgrtext40->bitmap_graphicgrtext40_height,SRCCOPY);
  SelectObject(window_graphicgrtext40->hCompatibleDC,bitmap_old);
}


/********************************************************************************************************/
/*  UpdateGraphicGRText40AddressGraphic() :  Dplace l'adresse Graphic pour la fentre GraphicGRText40. */
/********************************************************************************************************/
static void UpdateGraphicGRText40AddressGraphic(struct window_graphicgrtext40 *window_graphicgrtext40, int delta)
{
  int current_address, new_address, graphichgr_bank, graphichgr_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphicgrtext40->graphicgr_bank << 16) | window_graphicgrtext40->graphicgr_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  graphichgr_bank = (new_address >> 16) & 0xFF;
  graphichgr_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphichgr_bank);
  SetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_BANK_GRAPHIC,buffer);
  sprintf(buffer,"%04X",graphichgr_address);
  SetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_ADDRESS_GRAPHIC,buffer);

  /* MAJ de la fentre */
  RefreshGraphicGRText40WindowData(window_graphicgrtext40,1);
}


/**************************************************************************************************/
/*  UpdateGraphicGRText40AddressText() :  Dplace l'adresse Text pour la fentre GraphicGRText40. */
/**************************************************************************************************/
static void UpdateGraphicGRText40AddressText(struct window_graphicgrtext40 *window_graphicgrtext40, int delta)
{
  int current_address, new_address, textpage40_bank, textpage40_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphicgrtext40->textpage40_bank << 16) | window_graphicgrtext40->textpage40_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  textpage40_bank = (new_address >> 16) & 0xFF;
  textpage40_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",textpage40_bank);
  SetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_BANK_TEXT,buffer);
  sprintf(buffer,"%04X",textpage40_address);
  SetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_ADDRESS_TEXT,buffer);

  /* MAJ de la fentre */
  RefreshGraphicGRText40WindowData(window_graphicgrtext40,1);
}


/*****************************************************************************************************************/
/*  UpdateGraphicGRText40ContextInformation() :  MAJ des information de contexte de la fenetre GraphicGRText40. */
/*****************************************************************************************************************/
static void UpdateGraphicGRText40ContextInformation(struct window_graphicgrtext40 *window_graphicgrtext40, int mouse_x, int mouse_y)
{
  int pixmap_x, pixmap_y, grid_x, grid_y;
  int graphicgr_data_address;
  int textpage40_data_address;
  unsigned char graphicgr_data;
  unsigned char textpage40_data;
  int tab_gr_address[24] =
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page GR */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };
  int tab_text40_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };

  /* Rien  faire */
  if(window_graphicgrtext40 == NULL)
    return;

  /* Est t'on dans l'image ? */
  if(mouse_x < window_graphicgrtext40->bitmap_graphicgrtext40_x || mouse_x >= (window_graphicgrtext40->bitmap_graphicgrtext40_x + window_graphicgrtext40->bitmap_graphicgrtext40_width*window_graphicgrtext40->zoom_level) ||
     mouse_y < window_graphicgrtext40->bitmap_graphicgrtext40_y || mouse_y >= (window_graphicgrtext40->bitmap_graphicgrtext40_y + window_graphicgrtext40->bitmap_graphicgrtext40_height*window_graphicgrtext40->zoom_level))
    {
      SetWindowText(GetDlgItem(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_STATIC_CONTEXT),"");
      return;
    }

  /* Coordonnes dans la Pixmap */
  pixmap_x = (mouse_x - window_graphicgrtext40->bitmap_graphicgrtext40_x)/window_graphicgrtext40->zoom_level;
  pixmap_y = (mouse_y - window_graphicgrtext40->bitmap_graphicgrtext40_y)/window_graphicgrtext40->zoom_level;

  /* Graphic */
  if(pixmap_y < 160)   /* 192 - 8*4 */
    {
      /* Coordonnes dans la grille 40x24 */
      grid_x = pixmap_x/7;
      grid_y = pixmap_y/8;

      /* Valeur */
      graphicgr_data = window_graphicgrtext40->graphicgr_data[grid_x + grid_y*40];

      /* Adresse */
      graphicgr_data_address = (window_graphicgrtext40->graphicgr_bank << 16) + window_graphicgrtext40->graphicgr_address;
      graphicgr_data_address += grid_x;
      graphicgr_data_address += tab_gr_address[grid_y];

      /* Message */
      sprintf(window_graphicgrtext40->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphicgr_data_address>>16,graphicgr_data_address&0xFFFF,graphicgr_data,grid_x,pixmap_y/4);
      SetWindowText(GetDlgItem(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_STATIC_CONTEXT),window_graphicgrtext40->buffer_log);
    }
  else   /* Text */
    {
      /* Coordonnes dans la grille 40x24 */
      grid_x = pixmap_x/7;
      grid_y = pixmap_y/8;

      /* Valeur */
      textpage40_data = window_graphicgrtext40->textpage40_data[grid_x + grid_y*40];

      /* Adresse */
      textpage40_data_address = (window_graphicgrtext40->textpage40_bank << 16) + window_graphicgrtext40->textpage40_address;
      textpage40_data_address += grid_x;
      textpage40_data_address += tab_text40_address[grid_y];

      /* Message */
      sprintf(window_graphicgrtext40->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",textpage40_data_address>>16,textpage40_data_address&0xFFFF,textpage40_data,grid_x,grid_y);
      SetWindowText(GetDlgItem(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_STATIC_CONTEXT),window_graphicgrtext40->buffer_log);
    }
}


/**********************************************************************************************************/
/*  UpdateGraphicGRText40RefreshMode() :  MAJ du mode de rafraichissement de la fenetre GraphicGRText40. */
/**********************************************************************************************************/
static void UpdateGraphicGRText40RefreshMode(struct window_graphicgrtext40 *window_graphicgrtext40, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == GRAPHICGRTEXT40_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_graphicgrtext40->refresh_mode = GRAPHICGRTEXT40_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_graphicgrtext40->hwnd_graphicgrtext40;
      current_notification->message = HM_GRAPHICGRTEXT40_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_GRAPHICGRTEXT40;

      /* Ajout de la notification */
      window_graphicgrtext40->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_graphicgrtext40->refresh_mode = GRAPHICGRTEXT40_REFRESH_MANUAL;
      SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicgrtext40->current_notification,NULL);
      window_graphicgrtext40->current_notification = NULL;
    }
}


/****************************************************************************************************************/
/*  UpdateGraphicGRText40FlashMode() :  MAJ de l'activation des caractres FLASH de la fentre GraphicGRText40. */
/****************************************************************************************************************/
static void UpdateGraphicGRText40FlashMode(struct window_graphicgrtext40 *window_graphicgrtext40)
{
  struct notification *current_notification;

  if(window_graphicgrtext40->flash == 0)
    {
      /* Visuel */
      window_graphicgrtext40->flash = 1;

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_graphicgrtext40->hwnd_graphicgrtext40;
      current_notification->message = HM_GRAPHICGRTEXT40_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_TIMER;
      current_notification->timer_milliseconds = 500;

      /* Ajout de la notification */
      window_graphicgrtext40->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_graphicgrtext40->flash = 0;

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicgrtext40->current_notification,NULL);
      window_graphicgrtext40->current_notification = NULL;
    }
}


/*********************************************************************************************/
/*  UpdateGraphicGRText40ZoomLevel() :  MAJ du niveau de Zoom de la fentre GraphicGRText40. */
/*********************************************************************************************/
static void UpdateGraphicGRText40ZoomLevel(struct window_graphicgrtext40 *window_graphicgrtext40, int zoom_level)
{
  if(zoom_level == GRAPHICGRTEXT40_ZOOM_X1)
    {
      /* Visuel */
      window_graphicgrtext40->zoom_level = GRAPHICGRTEXT40_ZOOM_X1;
      SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICGRTEXT40_ZOOM_X2)
    {
      /* Visuel */
      window_graphicgrtext40->zoom_level = GRAPHICGRTEXT40_ZOOM_X2;
      SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_ZOOM_X2,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICGRTEXT40_ZOOM_X3)
    {
      /* Visuel */
      window_graphicgrtext40->zoom_level = GRAPHICGRTEXT40_ZOOM_X3;
      SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_RADIO_ZOOM_X3,BM_SETCHECK,1,0);
    }
    
  /* Retaille la fentre */
  ResizeGraphicGRText40Window(window_graphicgrtext40);  
  
  /* On force la MAJ de la fentre */  
  RefreshGraphicGRText40WindowData(window_graphicgrtext40,1);    
}


/****************************************************************************************/
/*  RefreshGraphicGRText40WindowData() :  MAJ du contenu de la fentre GraphicGRText40. */
/****************************************************************************************/
static void RefreshGraphicGRText40WindowData(struct window_graphicgrtext40 *window_graphicgrtext40, int force)
{
  int length, is_modified;
  int graphicgr_bank, graphicgr_address, textpage40_bank, textpage40_address;
  char buffer[1024];

  /** Rcupre la valeur des GraphicGRText40 Pointer **/
  /* Graphic */
  length = GetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_BANK_GRAPHIC,buffer,1024);
  sscanf(buffer,"%X",&graphicgr_bank);
  length = GetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_ADDRESS_GRAPHIC,buffer,1024);
  sscanf(buffer,"%X",&graphicgr_address);
  /* Text */
  length = GetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_BANK_TEXT,buffer,1024);
  sscanf(buffer,"%X",&textpage40_bank);
  length = GetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_ADDRESS_TEXT,buffer,1024);
  sscanf(buffer,"%X",&textpage40_address);

  /** Replace les valeurs dans les Control pour montrer ce qui a t compris **/
  /* Graphic */
  sprintf(buffer,"%02X",graphicgr_bank);
  SetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_BANK_GRAPHIC,buffer);
  sprintf(buffer,"%04X",graphicgr_address);
  SetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_ADDRESS_GRAPHIC,buffer);
  /* Text */
  sprintf(buffer,"%02X",textpage40_bank);
  SetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_BANK_TEXT,buffer);
  sprintf(buffer,"%04X",textpage40_address);
  SetDlgItemText(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_EDIT_ADDRESS_TEXT,buffer);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsGraphicGRText40DataModified(graphicgr_bank,graphicgr_address,textpage40_bank,textpage40_address,window_graphicgrtext40);
  if(is_modified == 0 && force == 0)
    return;

  /** Cration de la Zone Graphique + Text  partir des nouvelles donnes **/
  BuildGraphicGRText40FromData(window_graphicgrtext40);

  /* Rafraichit la fentre */
  InvalidateRect(window_graphicgrtext40->hwnd_graphicgrtext40,NULL,TRUE);
  UpdateWindow(window_graphicgrtext40->hwnd_graphicgrtext40);
}


/**********************************************************************************************************************/
/*  IsGraphicGRText40DataModified() :  Les donnes GraphicGRText40 ont t'elles t modifies depuis la dernire fois. */
/**********************************************************************************************************************/
static int IsGraphicGRText40DataModified(int graphicgr_bank, int graphicgr_address, int textpage40_bank, int textpage40_address, struct window_graphicgrtext40 *window_graphicgrtext40)
{
  int x, y, graphicgr_data_address, textpage40_data_address, has_flash_character;
  unsigned char graphicgr_raw_data[1024];
  unsigned char textpage40_raw_data[1024];
  unsigned char graphicgr_data[40*24];
  unsigned char textpage40_data[40*24];
  int tab_gr_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };
  int tab_text40_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /** Graphic : Rcupre les donnes brutes : 1024 bytes **/
  graphicgr_data_address = (graphicgr_bank << 16) | graphicgr_address;
  ReadMemoryBlock(graphicgr_data_address,1024,graphicgr_raw_data,current_mmu);

  /** Text : Rcupre les donnes brutes : 1024 chars **/
  textpage40_data_address = (textpage40_bank << 16) | textpage40_address;
  ReadMemoryBlock(textpage40_data_address,1024,textpage40_raw_data,current_mmu);

  /** Graphic : R-organise les donnes : Linarisation (passage du block de 1024 byte en 40*24) **/
  for(y=0; y<24; y++)
    for(x=0; x<40; x++)
      graphicgr_data[x+y*40] = graphicgr_raw_data[tab_gr_address[y]+x];

  /** Text : R-organise les donnes : Linarisation (passage du block de 1024 char en 40*24) **/
  has_flash_character = 0;
  for(y=0; y<24; y++)
    for(x=0; x<40; x++)
      {
        textpage40_data[x+y*40] = textpage40_raw_data[tab_text40_address[y]+x];
        if(textpage40_data[x+y*40] >= 0x40 && textpage40_data[x+y*40] <= 0x7F)
          has_flash_character = 1;
      }

  /** Compare les donnes **/
  if(window_graphicgrtext40->graphicgr_data != NULL && window_graphicgrtext40->textpage40_data != NULL)
    if(graphicgr_bank == window_graphicgrtext40->graphicgr_bank &&
       graphicgr_address == window_graphicgrtext40->graphicgr_address &&
       memcmp(&graphicgr_data[0],&window_graphicgrtext40->graphicgr_data[0],40*24) == 0 && 
       textpage40_bank == window_graphicgrtext40->textpage40_bank &&
       textpage40_address == window_graphicgrtext40->textpage40_address &&
       memcmp(&textpage40_data[0],&window_graphicgrtext40->textpage40_data[0],40*24) == 0 && 
       (window_graphicgrtext40->flash == 1 && has_flash_character == 0))
      return(0);

  /** Les donnes sont diffrentes => MAJ **/
  /* Graphic */
  window_graphicgrtext40->graphicgr_bank = graphicgr_bank;
  window_graphicgrtext40->graphicgr_address = graphicgr_address;
  memcpy(window_graphicgrtext40->graphicgr_data,graphicgr_data,40*24);
  /* Text */
  window_graphicgrtext40->textpage40_bank = textpage40_bank;
  window_graphicgrtext40->textpage40_address = textpage40_address;
  memcpy(window_graphicgrtext40->textpage40_data,textpage40_data,40*24);
  window_graphicgrtext40->has_flash_character = has_flash_character;

  /* Donnes diffrentes ou FLASH */
  return(1);
}


/*********************************************************************************/
/*  ResizeGraphicGRText40Window() : Resize GraphicGRText40 window by given size. */
/*********************************************************************************/
static void ResizeGraphicGRText40Window(struct window_graphicgrtext40 *window_graphicgrtext40)
{
  HWND hwnd_group;
  RECT window_rect, client_rect, group_rect;
  int window_x, window_y;
  
  /** On va ajuster la fentre par rapport au Zoom demand **/  
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_graphicgrtext40->hwnd_graphicgrtext40,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_graphicgrtext40->hwnd_graphicgrtext40,&client_rect);
  window_graphicgrtext40->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select */
  hwnd_group = GetDlgItem(window_graphicgrtext40->hwnd_graphicgrtext40,IDC_GROUP_GRAPHICGRTEXT40);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_GRAPHICGRTEXT40 */
  GetWindowRect(hwnd_group,&group_rect);
  window_graphicgrtext40->group_data_x = (group_rect.left - window_rect.left) - 3;
  window_graphicgrtext40->group_data_y = ((group_rect.top - window_rect.top) - window_graphicgrtext40->title_height) + 3;

  /* Position de la Zone BitMap dans la fenetre */
  window_graphicgrtext40->bitmap_graphicgrtext40_x = window_graphicgrtext40->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_graphicgrtext40->bitmap_graphicgrtext40_y = window_graphicgrtext40->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_graphicgrtext40->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_graphicgrtext40->bitmap_graphicgrtext40_width*window_graphicgrtext40->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_graphicgrtext40->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicgrtext40->bitmap_graphicgrtext40_height*window_graphicgrtext40->zoom_level + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_graphicgrtext40->group_data_x,window_graphicgrtext40->group_data_y,window_graphicgrtext40->group_data_width,window_graphicgrtext40->group_data_height,TRUE);

  /* Largeur de la fentre */
  window_graphicgrtext40->window_width = window_graphicgrtext40->group_data_x + window_graphicgrtext40->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* On doit garantir une largeur minimale de la fentre */
  if(window_graphicgrtext40->window_width < GRAPHICGRTEXT40_DIALOGBOX_WIDTH)
    window_graphicgrtext40->window_width = GRAPHICGRTEXT40_DIALOGBOX_WIDTH;

  /* Hauteur de la fentre */
  window_graphicgrtext40->window_height = window_graphicgrtext40->title_height + window_graphicgrtext40->group_data_y + window_graphicgrtext40->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Resize window */
  MoveWindow(window_graphicgrtext40->hwnd_graphicgrtext40,window_x,window_y,window_graphicgrtext40->window_width,window_graphicgrtext40->window_height,TRUE);
}


/*****************************************************************************************************************/
/*  BuildGraphicGRText40FromData() :  Construire la page Gaphique + Text  afficher  partir des donnes brutes. */
/*****************************************************************************************************************/
static void BuildGraphicGRText40FromData(struct window_graphicgrtext40 *window_graphicgrtext40)
{
  int one_byte, flash_offset_low, flash_offset_high, hundred_sec;
  int x, y, pixmap_char_x, pixmap_char_y, pixel_x, pixel_y, offset_pixmap_char, offset_pixmap_screen;
  char buffer_title[256];
  DWORD color_text, color_background, color_pixmap, color_screen;
  DWORD tab_rgb_color_graphic[16] = 
    {
      /* 16 couleurs pour le mode GR */
      MAKECOLOR(0,0,0), MAKECOLOR(221,0,51), MAKECOLOR(0,0,153), MAKECOLOR(221,0,221),
      MAKECOLOR(0,119,0), MAKECOLOR(85,85,85), MAKECOLOR(34,34,255), MAKECOLOR(102,170,255),
      MAKECOLOR(136,85,34), MAKECOLOR(255,102,0), MAKECOLOR(170,170,170), MAKECOLOR(255,153,136),
      MAKECOLOR(0,221,0), MAKECOLOR(255,255,0), MAKECOLOR(0,255,153), MAKECOLOR(255,255,255)
    };
  DWORD tab_rgb_color_text[16] = 
    {
      /* 16 couleurs de la Bordure / Fond Ecran / Text */
      MAKECOLOR(0,0,0), MAKECOLOR(221,0,51), MAKECOLOR(0,0,153), MAKECOLOR(221,34,221),
      MAKECOLOR(0,119,34), MAKECOLOR(85,85,85), MAKECOLOR(34,34,255), MAKECOLOR(102,170,255),
      MAKECOLOR(136,85,0), MAKECOLOR(255,102,0), MAKECOLOR(170,170,170), MAKECOLOR(255,153,136),
      MAKECOLOR(17,221,0), MAKECOLOR(255,255,0), MAKECOLOR(68,255,153), MAKECOLOR(255,255,255)
    };
  struct _timeb timebuffer;
  struct picture_true *picture_char;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rien  faire */
  if(window_graphicgrtext40->bitmap_data_graphicgrtext40 == NULL)
    return;

  /* Dtermine le changement pour les FLASH */
  _ftime(&timebuffer);
  hundred_sec = timebuffer.millitm / 10;
  if(hundred_sec < 50)
    {
      /* INVERSE */
      flash_offset_low = (-1)*(0x40);
      flash_offset_high = (-1)*(0x40);
    }
  else
    {
      /* NORMAL */
      flash_offset_low = (0xA0 - 0x40 + 0x20);
      flash_offset_high = (0xA0 - 0x40 - 0x20);
    }

  /* Adresse de la picture */
  if(window_graphicgrtext40->display_language == LANGUAGE_IIGS_USA)
    picture_char = param->picture_char_us;
  else if(window_graphicgrtext40->display_language == LANGUAGE_IIGS_UK)
    picture_char = param->picture_char_uk;
  else if(window_graphicgrtext40->display_language == LANGUAGE_IIGS_FRENCH)
    picture_char = param->picture_char_fr;
  else if(window_graphicgrtext40->display_language == LANGUAGE_IIGS_DANISH)
    picture_char = param->picture_char_dk;
  else if(window_graphicgrtext40->display_language == LANGUAGE_IIGS_SPANISH)
    picture_char = param->picture_char_se;
  else if(window_graphicgrtext40->display_language == LANGUAGE_IIGS_ITALIAN)
    picture_char = param->picture_char_it;
  else if(window_graphicgrtext40->display_language == LANGUAGE_IIGS_GERMAN)
    picture_char = param->picture_char_de;
  else if(window_graphicgrtext40->display_language == LANGUAGE_IIGS_SWEDISH)
    picture_char = param->picture_char_se;
  else if(window_graphicgrtext40->display_language == LANGUAGE_IIGS_MOUSETEXT)
    picture_char = param->picture_char_mousetext;
    
  /* Couleur du Texte */
  color_text = tab_rgb_color_text[window_graphicgrtext40->color_text];

  /* Couleur du Fond */
  color_background = tab_rgb_color_text[window_graphicgrtext40->color_background];

  /** Graphic : Cration de la bitmap 280*192 correspondant au 40*48 du GR **/
  for(y=0; y<20; y++)
    {
      for(x=0; x<40; x++)
        {
          /* Caractre */
          one_byte = (int) window_graphicgrtext40->graphicgr_data[x + y*40];

          /* Adresse du caractre  l'cran */
          offset_pixmap_screen = x*7 + y*(40*7)*8;

          /* Bloc Haut de 7*4 */
          color_screen = tab_rgb_color_graphic[one_byte & 0x0F];    /* Couleur du Pixel final */
          for(pixel_y=0; pixel_y<4; pixel_y++)
            for(pixel_x=0; pixel_x<7; pixel_x++)
              window_graphicgrtext40->bitmap_data_graphicgrtext40[offset_pixmap_screen + pixel_x + pixel_y*40*7] = color_screen;

          /* Bloc Bas de 7*4 */
          color_screen = tab_rgb_color_graphic[one_byte >> 4];      /* Couleur du Pixel final */
          for(pixel_y=4; pixel_y<8; pixel_y++)
            for(pixel_x=0; pixel_x<7; pixel_x++)
              window_graphicgrtext40->bitmap_data_graphicgrtext40[offset_pixmap_screen + pixel_x + pixel_y*40*7] = color_screen;
        }
    }

  /** Text : Cration du bas de la bitmap 280*192 correspondant au 40*24 **/
  for(y=20; y<24; y++)
    {
      for(x=0; x<40; x++)
        {
          /* Caractre */
          one_byte = (int) window_graphicgrtext40->textpage40_data[x + y*40];

          /* Si le caractre FLASH, on switch entre INVERSE et NORMAL (Pas de FLASH en MouseText) */
          if(window_graphicgrtext40->display_language != LANGUAGE_IIGS_MOUSETEXT)
            {
              if(one_byte >= 0x40 && one_byte <= 0x5F)
                one_byte += flash_offset_low;
              else if(one_byte >= 0x60 && one_byte <= 0x7F)
                one_byte += flash_offset_high;
            }

          /* Offset dans la Pixmap */
          pixmap_char_y = one_byte / 40;
          pixmap_char_x = one_byte - (pixmap_char_y*40);

          /* Copie ce bloc de 7*8 */
          offset_pixmap_char  = pixmap_char_x*7 + pixmap_char_y*(40*7)*8;
          offset_pixmap_screen = x*7 + y*(40*7)*8;
          for(pixel_y=0; pixel_y<8; pixel_y++)
            for(pixel_x=0; pixel_x<7; pixel_x++)
              {
                /* Pixel dans la Char Tab */
                color_pixmap = picture_char->data_dwd[offset_pixmap_char + pixel_x + pixel_y*40*7];

                /* Couleur du Pixel final */
                color_screen = (color_pixmap == 0xFFFFFF) ? color_text : color_background;

                /* Place le point dans la Pixmap */
                window_graphicgrtext40->bitmap_data_graphicgrtext40[offset_pixmap_screen + pixel_x + pixel_y*40*7] = color_screen;
              }
        }
    }

  /* Titre de la fentre */
  sprintf(buffer_title,"Graphic GR Text 40 Column : %02X/%04X   %02X/%04X",window_graphicgrtext40->graphicgr_bank,window_graphicgrtext40->graphicgr_address,
                                                                           window_graphicgrtext40->textpage40_bank,window_graphicgrtext40->textpage40_address);
  SetWindowText(window_graphicgrtext40->hwnd_graphicgrtext40,buffer_title);
}


/*****************************************************************************/
/*  DeleteGraphicGRText40ChildWindows() :  Destruction des fentres enfants. */
/*****************************************************************************/
static void DeleteGraphicGRText40ChildWindows(struct window_graphicgrtext40 *window_graphicgrtext40)
{
  DeleteObject(window_graphicgrtext40->hBitmap_graphicgrtext40);
  DeleteObject(window_graphicgrtext40->hwnd_combo_color_text);
  DeleteObject(window_graphicgrtext40->hwnd_combo_color_background);
  DeleteObject(window_graphicgrtext40->hwnd_combo_display_language);
  DeleteObject(window_graphicgrtext40->hFont);
  ReleaseDC(window_graphicgrtext40->hwnd_graphicgrtext40,window_graphicgrtext40->hDC);
  DeleteDC(window_graphicgrtext40->hCompatibleDC);
}


/*******************************************************************************/
/*  mem_free_window_graphicgrtext40() :  Libration mmoire pour la structure. */
/*******************************************************************************/
void mem_free_window_graphicgrtext40(struct window_graphicgrtext40 *window_graphicgrtext40)
{
  if(window_graphicgrtext40)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicgrtext40->current_notification,NULL);
      window_graphicgrtext40->current_notification = NULL;
      
      free(window_graphicgrtext40);
    }
}

/****************************************************************************/
