/************************************************************************/
/*                                                                      */
/*  Window_GraphicHGR.c :  Module de gestion de la fentre GraphicHGR.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Juillet 2018  */
/************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_GraphicHGR.h"

static DWORD BuildGraphicHGRWindowThread(void *);
LRESULT CALLBACK ProcHandlerGraphicHGRWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateGraphicHGRChildWindows(HWND,struct window_graphichgr *);
LRESULT CALLBACK subEditGraphicHGRProcControl(HWND,UINT,WPARAM,LPARAM);
void UpdateGraphicHGRClientAllArea(HDC,struct window_graphichgr *);
static void UpdateGraphicHGRClientBitmapArea(HDC,struct window_graphichgr *);
static void UpdateGraphicHGRContextInformation(struct window_graphichgr *,int,int);
static void UpdateGraphicHGRAddress(struct window_graphichgr *,int);
static void UpdateGraphicHGRRefreshMode(struct window_graphichgr *,int);
static void UpdateGraphicHGRZoomLevel(struct window_graphichgr *,int);
static void RefreshGraphicHGRWindowData(struct window_graphichgr *,int);
static int IsGraphicHGRDataModified(int,int,int,int,struct window_graphichgr *);
static void ResizeGraphicHGRWindow(struct window_graphichgr *);
static void BuildGraphicHGRFromData(struct window_graphichgr *);
static void DeleteGraphicHGRChildWindows(struct window_graphichgr *);


/**********************************************************************************/
/*  BuildGraphicHGRWindow() :  Construction / Affichage de la fentre GraphicHGR. */
/**********************************************************************************/
struct window_graphichgr *BuildGraphicHGRWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_graphichgr *window_graphichgr;

  /** Allocation de la structure window_graphichgr **/
  window_graphichgr = (struct window_graphichgr *) calloc(1,sizeof(struct window_graphichgr));
  if(window_graphichgr == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_GRAPHICHGR,window_graphichgr,NULL);

  /* Lance un thread pour grer la fentre GraphicHGR */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildGraphicHGRWindowThread,window_graphichgr,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_graphichgr);
}


/********************************************************************************************/
/*  BuildGraphicHGRWindowThread() :  Cration de la fentre GraphicHGR via un Thread ddi. */
/********************************************************************************************/
static DWORD BuildGraphicHGRWindowThread(void *window_graphichgr_param)
{
  HWND hwndGraphicHGR;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_graphichgr *window_graphichgr;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_graphichgr = (struct window_graphichgr *) window_graphichgr_param;
  window_graphichgr->thread_id = GetCurrentThreadId();         /* Le ThreaID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Graphic HGR **/
  hwndGraphicHGR = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_GRAPHICHGR),window_main->hwnd_main,(DLGPROC)ProcHandlerGraphicHGRWindow);
  if(hwndGraphicHGR == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndGraphicHGR,SW_SHOWDEFAULT);
  UpdateWindow(hwndGraphicHGR);

  /** Message Loop pour toutes les fentres de ce Thread (fentre GraphicHGR + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***************************************************************************/
/*  ProcHandlerGraphicHGRWindow() :  Gestion des vnements de la fentre. */
/***************************************************************************/
LRESULT CALLBACK ProcHandlerGraphicHGRWindow(HWND hwndGraphicHGR, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;
  int mouse_x, mouse_y;
  struct window_graphichgr *window_graphichgr;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_GRAPHICHGR,&window_graphichgr,NULL);
  if(window_graphichgr == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateGraphicHGRChildWindows(hwndGraphicHGR,window_graphichgr);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        UpdateGraphicHGRContextInformation(window_graphichgr,mouse_x,mouse_y);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_graphichgr->hBitmap_graphichgr != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndGraphicHGR,&ps);

            /** MAJ Bitmap **/
            UpdateGraphicHGRClientAllArea(hDC,window_graphichgr);

            /* End Paint */
            EndPaint(hwndGraphicHGR,&ps);
          }
        break;
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_graphichgr->refresh_mode != GRAPHICHGR_REFRESH_AUTOMATIC)
                UpdateGraphicHGRRefreshMode(window_graphichgr,GRAPHICHGR_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_graphichgr->refresh_mode != GRAPHICHGR_REFRESH_MANUAL)
                UpdateGraphicHGRRefreshMode(window_graphichgr,GRAPHICHGR_REFRESH_MANUAL);
              break;

            /** Bouton Radio : On change le Zoom de la fentre **/
            case IDC_RADIO_ZOOM_X1 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphichgr->zoom_level != GRAPHICHGR_ZOOM_X1)
                UpdateGraphicHGRZoomLevel(window_graphichgr,GRAPHICHGR_ZOOM_X1);
              break;

            case IDC_RADIO_ZOOM_X2 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphichgr->zoom_level != GRAPHICHGR_ZOOM_X2)
                UpdateGraphicHGRZoomLevel(window_graphichgr,GRAPHICHGR_ZOOM_X2);
              break;

            case IDC_RADIO_ZOOM_X3 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphichgr->zoom_level != GRAPHICHGR_ZOOM_X3)
                UpdateGraphicHGRZoomLevel(window_graphichgr,GRAPHICHGR_ZOOM_X3);
              break;

            /** Bouton de dplacement de l'adresse **/
            case IDC_BUTTON_LEFT :
              UpdateGraphicHGRAddress(window_graphichgr,-1);
              break;

            case IDC_BUTTON_RIGHT :
              UpdateGraphicHGRAddress(window_graphichgr,1);
              break;

            case IDC_BUTTON_UP :
              UpdateGraphicHGRAddress(window_graphichgr,-40);
              break;

            case IDC_BUTTON_DOWN :
              UpdateGraphicHGRAddress(window_graphichgr,40);
              break;

            case IDC_BUTTON_PAGEUP :
              UpdateGraphicHGRAddress(window_graphichgr,-8192);
              break;

            case IDC_BUTTON_PAGEDOWN :
              UpdateGraphicHGRAddress(window_graphichgr,8192);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshGraphicHGRWindowData(window_graphichgr,1);
              break;              
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_GRAPHICHGR_REFRESHWINDOW :
        RefreshGraphicHGRWindowData(window_graphichgr,0);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre GraphicHGR */
        DestroyWindow(hwndGraphicHGR);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteGraphicHGRChildWindows(window_graphichgr);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_GRAPHICHGR,window_graphichgr,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************/
/*  CreateGraphicHGRChildWindows() : Crer les fentres enfants et remplir la structure window_graphichgr. */
/***********************************************************************************************************/
static int CreateGraphicHGRChildWindows(HWND hwndGraphicHGR, struct window_graphichgr *window_graphichgr)
{
  BITMAPINFO bitmap_info;
  int i;
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_graphichgr->hwnd_graphichgr = hwndGraphicHGR;

  /** Device Context **/
  window_graphichgr->hDC = GetDC(window_graphichgr->hwnd_graphichgr);
  window_graphichgr->hCompatibleDC = CreateCompatibleDC(window_graphichgr->hDC);

  /** Surchage des Control Edit pour capter les vnements **/
  window_graphichgr->oldEditProcBank    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicHGR,IDC_EDIT_BANK),GWLP_WNDPROC,(LONG_PTR)subEditGraphicHGRProcControl);
  window_graphichgr->oldEditProcAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicHGR,IDC_EDIT_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditGraphicHGRProcControl);

  /** Ajoute les images sur les boutons **/
  SendMessage(GetDlgItem(hwndGraphicHGR,IDC_BUTTON_LEFT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicHGR,IDC_BUTTON_RIGHT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicHGR,IDC_BUTTON_UP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicHGR,IDC_BUTTON_DOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicHGR,IDC_BUTTON_PAGEUP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicHGR,IDC_BUTTON_PAGEDOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Adresse de la Zone de Data **/
  window_graphichgr->graphichgr_bank = 0x00;
  window_graphichgr->graphichgr_address = 0x2000;       /* Adresse HGR Page #1 */
  sprintf(buffer,"%02X",window_graphichgr->graphichgr_bank);
  SetDlgItemText(window_graphichgr->hwnd_graphichgr,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",window_graphichgr->graphichgr_address);
  SetDlgItemText(window_graphichgr->hwnd_graphichgr,IDC_EDIT_ADDRESS,buffer);

  /* Refresh Mode : Automatic */
  UpdateGraphicHGRRefreshMode(window_graphichgr,GRAPHICHGR_REFRESH_AUTOMATIC);

  /* Zoom Level : 1 */
  window_graphichgr->zoom_level = GRAPHICHGR_ZOOM_X1;
  SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);

  /* Dimension en Pixel de la Bitmap GraphicHGR Data */
  window_graphichgr->bitmap_graphichgr_width = 280;
  window_graphichgr->bitmap_graphichgr_height = 192;
  
  /** Bitmap affichant la zone de slection **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphichgr->bitmap_graphichgr_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphichgr->bitmap_graphichgr_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphichgr->hBitmap_graphichgr = CreateDIBSection(window_graphichgr->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphichgr->bitmap_data_graphichgr),NULL,0);
  for(i=0; i<window_graphichgr->bitmap_graphichgr_width*window_graphichgr->bitmap_graphichgr_height; i++)
    window_graphichgr->bitmap_data_graphichgr[i] = BITMAP_BACKGROUND_COLOR;

  /* Retaille la fentre */
  ResizeGraphicHGRWindow(window_graphichgr);

  /** Construction de la Page Texte + Rafraichissement de la fentre **/
  RefreshGraphicHGRWindowData(window_graphichgr,1);
    
  /* OK */
  return(0);
}


/*************************************************************************************************/
/*  subEditGraphicHGRProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*************************************************************************************************/
LRESULT CALLBACK subEditGraphicHGRProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_graphichgr *window_graphichgr;
  my_Memory(MEMORY_GET_WINDOW_GRAPHICHGR,&window_graphichgr,NULL);

  /* Rien  faire */
  if(window_graphichgr == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_graphichgr->hwnd_graphichgr,IDC_EDIT_BANK) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_graphichgr->hwnd_graphichgr,IDC_EDIT_ADDRESS) == hwnd_control)
    index = 2;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement du Return dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_graphichgr->hwnd_graphichgr);

      /* On force la MAJ de la fentre */
      RefreshGraphicHGRWindowData(window_graphichgr,1);
    }
  else if(index == 1)
    return(CallWindowProc(window_graphichgr->oldEditProcBank,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_graphichgr->oldEditProcAddress,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
}


/**************************************************************************************/
/*  UpdateGraphicHGRClientAllArea() :  Affichage des Bitmap de la fentre GraphicHGR. */
/**************************************************************************************/
void UpdateGraphicHGRClientAllArea(HDC hDC, struct window_graphichgr *window_graphichgr)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap **/
  UpdateGraphicHGRClientBitmapArea(hDC,window_graphichgr);
}


/*****************************************************************************/
/*  UpdateGraphicHGRClientBitmapArea() :  Affichage de la Bitmap GraphicHGR. */
/*****************************************************************************/
static void UpdateGraphicHGRClientBitmapArea(HDC hDC, struct window_graphichgr *window_graphichgr)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec la zone mmoire et la zone slectionne **/
  bitmap_old = SelectObject(window_graphichgr->hCompatibleDC,window_graphichgr->hBitmap_graphichgr);
  StretchBlt(hDC,window_graphichgr->bitmap_graphichgr_x,window_graphichgr->bitmap_graphichgr_y,
             window_graphichgr->bitmap_graphichgr_width*window_graphichgr->zoom_level,
             window_graphichgr->bitmap_graphichgr_height*window_graphichgr->zoom_level,
             window_graphichgr->hCompatibleDC,0,0,window_graphichgr->bitmap_graphichgr_width,window_graphichgr->bitmap_graphichgr_height,SRCCOPY);
  SelectObject(window_graphichgr->hCompatibleDC,bitmap_old);
}


/*******************************************************************************/
/*  UpdateGraphicHGRAddress() :  Dplace l'adresse pour la fentre GraphicHGR. */
/*******************************************************************************/
static void UpdateGraphicHGRAddress(struct window_graphichgr *window_graphichgr, int delta)
{
  int current_address, new_address, graphichgr_bank, graphichgr_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphichgr->graphichgr_bank << 16) | window_graphichgr->graphichgr_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  graphichgr_bank = (new_address >> 16) & 0xFF;
  graphichgr_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphichgr_bank);
  SetDlgItemText(window_graphichgr->hwnd_graphichgr,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",graphichgr_address);
  SetDlgItemText(window_graphichgr->hwnd_graphichgr,IDC_EDIT_ADDRESS,buffer);

  /* MAJ de la fentre */
  RefreshGraphicHGRWindowData(window_graphichgr,1);
}


/*******************************************************************************************************/
/*  UpdateGraphicHGRContextInformation() :  MAJ des information de contexte de la fenetre GraphicHGR. */
/*******************************************************************************************************/
static void UpdateGraphicHGRContextInformation(struct window_graphichgr *window_graphichgr, int mouse_x, int mouse_y)
{
  int pixmap_x, pixmap_y, grid_x;
  int graphichgr_data_address;
  unsigned char graphichgr_data;
  int tab_hgr_address[192] =     /* Adresse du dbut des 192 lignes de la Page HGR */
    {
      0x0000+0x0000, 0x0000+0x0400, 0x0000+0x0800, 0x0000+0x0C00, 0x0000+0x1000, 0x0000+0x1400, 0x0000+0x1800, 0x0000+0x1C00, 
      0x0080+0x0000, 0x0080+0x0400, 0x0080+0x0800, 0x0080+0x0C00, 0x0080+0x1000, 0x0080+0x1400, 0x0080+0x1800, 0x0080+0x1C00, 
      0x0100+0x0000, 0x0100+0x0400, 0x0100+0x0800, 0x0100+0x0C00, 0x0100+0x1000, 0x0100+0x1400, 0x0100+0x1800, 0x0100+0x1C00, 
      0x0180+0x0000, 0x0180+0x0400, 0x0180+0x0800, 0x0180+0x0C00, 0x0180+0x1000, 0x0180+0x1400, 0x0180+0x1800, 0x0180+0x1C00, 
      0x0200+0x0000, 0x0200+0x0400, 0x0200+0x0800, 0x0200+0x0C00, 0x0200+0x1000, 0x0200+0x1400, 0x0200+0x1800, 0x0200+0x1C00, 
      0x0280+0x0000, 0x0280+0x0400, 0x0280+0x0800, 0x0280+0x0C00, 0x0280+0x1000, 0x0280+0x1400, 0x0280+0x1800, 0x0280+0x1C00, 
      0x0300+0x0000, 0x0300+0x0400, 0x0300+0x0800, 0x0300+0x0C00, 0x0300+0x1000, 0x0300+0x1400, 0x0300+0x1800, 0x0300+0x1C00, 
      0x0380+0x0000, 0x0380+0x0400, 0x0380+0x0800, 0x0380+0x0C00, 0x0380+0x1000, 0x0380+0x1400, 0x0380+0x1800, 0x0380+0x1C00, 
      0x0028+0x0000, 0x0028+0x0400, 0x0028+0x0800, 0x0028+0x0C00, 0x0028+0x1000, 0x0028+0x1400, 0x0028+0x1800, 0x0028+0x1C00, 
      0x00A8+0x0000, 0x00A8+0x0400, 0x00A8+0x0800, 0x00A8+0x0C00, 0x00A8+0x1000, 0x00A8+0x1400, 0x00A8+0x1800, 0x00A8+0x1C00, 
      0x0128+0x0000, 0x0128+0x0400, 0x0128+0x0800, 0x0128+0x0C00, 0x0128+0x1000, 0x0128+0x1400, 0x0128+0x1800, 0x0128+0x1C00, 
      0x01A8+0x0000, 0x01A8+0x0400, 0x01A8+0x0800, 0x01A8+0x0C00, 0x01A8+0x1000, 0x01A8+0x1400, 0x01A8+0x1800, 0x01A8+0x1C00, 
      0x0228+0x0000, 0x0228+0x0400, 0x0228+0x0800, 0x0228+0x0C00, 0x0228+0x1000, 0x0228+0x1400, 0x0228+0x1800, 0x0228+0x1C00, 
      0x02A8+0x0000, 0x02A8+0x0400, 0x02A8+0x0800, 0x02A8+0x0C00, 0x02A8+0x1000, 0x02A8+0x1400, 0x02A8+0x1800, 0x02A8+0x1C00, 
      0x0328+0x0000, 0x0328+0x0400, 0x0328+0x0800, 0x0328+0x0C00, 0x0328+0x1000, 0x0328+0x1400, 0x0328+0x1800, 0x0328+0x1C00, 
      0x03A8+0x0000, 0x03A8+0x0400, 0x03A8+0x0800, 0x03A8+0x0C00, 0x03A8+0x1000, 0x03A8+0x1400, 0x03A8+0x1800, 0x03A8+0x1C00, 
      0x0050+0x0000, 0x0050+0x0400, 0x0050+0x0800, 0x0050+0x0C00, 0x0050+0x1000, 0x0050+0x1400, 0x0050+0x1800, 0x0050+0x1C00, 
      0x00D0+0x0000, 0x00D0+0x0400, 0x00D0+0x0800, 0x00D0+0x0C00, 0x00D0+0x1000, 0x00D0+0x1400, 0x00D0+0x1800, 0x00D0+0x1C00, 
      0x0150+0x0000, 0x0150+0x0400, 0x0150+0x0800, 0x0150+0x0C00, 0x0150+0x1000, 0x0150+0x1400, 0x0150+0x1800, 0x0150+0x1C00, 
      0x01D0+0x0000, 0x01D0+0x0400, 0x01D0+0x0800, 0x01D0+0x0C00, 0x01D0+0x1000, 0x01D0+0x1400, 0x01D0+0x1800, 0x01D0+0x1C00, 
      0x0250+0x0000, 0x0250+0x0400, 0x0250+0x0800, 0x0250+0x0C00, 0x0250+0x1000, 0x0250+0x1400, 0x0250+0x1800, 0x0250+0x1C00, 
      0x02D0+0x0000, 0x02D0+0x0400, 0x02D0+0x0800, 0x02D0+0x0C00, 0x02D0+0x1000, 0x02D0+0x1400, 0x02D0+0x1800, 0x02D0+0x1C00, 
      0x0350+0x0000, 0x0350+0x0400, 0x0350+0x0800, 0x0350+0x0C00, 0x0350+0x1000, 0x0350+0x1400, 0x0350+0x1800, 0x0350+0x1C00, 
      0x03D0+0x0000, 0x03D0+0x0400, 0x03D0+0x0800, 0x03D0+0x0C00, 0x03D0+0x1000, 0x03D0+0x1400, 0x03D0+0x1800, 0x03D0+0x1C00
    };

  /* Rien  faire */
  if(window_graphichgr == NULL)
    return;

  /* Est t'on dans l'image ? */
  if(mouse_x < window_graphichgr->bitmap_graphichgr_x || mouse_x >= (window_graphichgr->bitmap_graphichgr_x + window_graphichgr->bitmap_graphichgr_width*window_graphichgr->zoom_level) ||
     mouse_y < window_graphichgr->bitmap_graphichgr_y || mouse_y >= (window_graphichgr->bitmap_graphichgr_y + window_graphichgr->bitmap_graphichgr_height*window_graphichgr->zoom_level))
    {
      SetWindowText(GetDlgItem(window_graphichgr->hwnd_graphichgr,IDC_STATIC_CONTEXT),"");
      return;
    }

  /* Coordonnes dans la Pixmap */
  pixmap_x = (mouse_x - window_graphichgr->bitmap_graphichgr_x)/window_graphichgr->zoom_level;
  pixmap_y = (mouse_y - window_graphichgr->bitmap_graphichgr_y)/window_graphichgr->zoom_level;

  /* Coordonnes dans la grille 40x24 */
  grid_x = pixmap_x/7;

  /* Valeur */
  graphichgr_data = window_graphichgr->graphichgr_data[grid_x + pixmap_y*40];

  /* Adresse */
  graphichgr_data_address = (window_graphichgr->graphichgr_bank << 16) + window_graphichgr->graphichgr_address;
  graphichgr_data_address += grid_x;
  graphichgr_data_address += tab_hgr_address[pixmap_y];

  /* Message */
  sprintf(window_graphichgr->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphichgr_data_address>>16,graphichgr_data_address&0xFFFF,graphichgr_data,pixmap_x,pixmap_y);
  SetWindowText(GetDlgItem(window_graphichgr->hwnd_graphichgr,IDC_STATIC_CONTEXT),window_graphichgr->buffer_log);
}


/************************************************************************************************/
/*  UpdateGraphicHGRRefreshMode() :  MAJ du mode de rafraichissement de la fenetre GraphicHGR. */
/************************************************************************************************/
static void UpdateGraphicHGRRefreshMode(struct window_graphichgr *window_graphichgr, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == GRAPHICHGR_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_graphichgr->refresh_mode = GRAPHICHGR_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_graphichgr->hwnd_graphichgr;
      current_notification->message = HM_GRAPHICHGR_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_GRAPHICHGR;

      /* Ajout de la notification */
      window_graphichgr->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_graphichgr->refresh_mode = GRAPHICHGR_REFRESH_MANUAL;
      SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphichgr->current_notification,NULL);
      window_graphichgr->current_notification = NULL;
    }
}


/**********************************************************************************/
/*  UpdateGraphicHGRZoomLevel() :  MAJ du nieau de Zomm de la fentre GraphicHGR. */
/**********************************************************************************/
static void UpdateGraphicHGRZoomLevel(struct window_graphichgr *window_graphichgr, int zoom_level)
{
  if(zoom_level == GRAPHICHGR_ZOOM_X1)
    {
      /* Visuel */
      window_graphichgr->zoom_level = GRAPHICHGR_ZOOM_X1;
      SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICHGR_ZOOM_X2)
    {
      /* Visuel */
      window_graphichgr->zoom_level = GRAPHICHGR_ZOOM_X2;
      SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICHGR_ZOOM_X3)
    {
      /* Visuel */
      window_graphichgr->zoom_level = GRAPHICHGR_ZOOM_X3;
      SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphichgr->hwnd_graphichgr,IDC_RADIO_ZOOM_X3,BM_SETCHECK,1,0);
    }
    
  /* Retaille la fentre */
  ResizeGraphicHGRWindow(window_graphichgr);  
  
  /* On force la MAJ de la fentre */  
  RefreshGraphicHGRWindowData(window_graphichgr,1);    
}


/******************************************************************************/
/*  RefreshGraphicHGRWindowData() :  MAJ du contenu de la fentre GraphicHGR. */
/******************************************************************************/
static void RefreshGraphicHGRWindowData(struct window_graphichgr *window_graphichgr, int force)
{
  int length, is_modified;
  int graphichgr_bank, graphichgr_address, nb_column, nb_row;
  char buffer[1024];

  /* Rcupre la valeur du GraphicHGR Pointer */
  length = GetDlgItemText(window_graphichgr->hwnd_graphichgr,IDC_EDIT_BANK,buffer,1024);
  sscanf(buffer,"%X",&graphichgr_bank);
  length = GetDlgItemText(window_graphichgr->hwnd_graphichgr,IDC_EDIT_ADDRESS,buffer,1024);
  sscanf(buffer,"%X",&graphichgr_address);
  nb_column = GetDlgItemInt(window_graphichgr->hwnd_graphichgr,IDC_EDIT_COLUMN,NULL,FALSE);
  if(nb_column < 1)
    nb_column = 1;
  nb_row = GetDlgItemInt(window_graphichgr->hwnd_graphichgr,IDC_EDIT_ROW,NULL,FALSE);
  if(nb_row < 1)
    nb_row = 1;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphichgr_bank);
  SetDlgItemText(window_graphichgr->hwnd_graphichgr,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",graphichgr_address);
  SetDlgItemText(window_graphichgr->hwnd_graphichgr,IDC_EDIT_ADDRESS,buffer);
  SetDlgItemInt(window_graphichgr->hwnd_graphichgr,IDC_EDIT_COLUMN,nb_column,FALSE);
  SetDlgItemInt(window_graphichgr->hwnd_graphichgr,IDC_EDIT_ROW,nb_row,FALSE);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsGraphicHGRDataModified(graphichgr_bank,graphichgr_address,nb_column,nb_row,window_graphichgr);
  if(is_modified == 0 && force == 0)
    return;

  /** Cration de la Zone Text Graphique  partir des nouvelles donnes **/
  BuildGraphicHGRFromData(window_graphichgr);

  /* Rafraichit la fentre */
  InvalidateRect(window_graphichgr->hwnd_graphichgr,NULL,TRUE);
  UpdateWindow(window_graphichgr->hwnd_graphichgr);
}


/*************************************************************************************************************/
/*  IsGraphicHGRDataModified() :  Les donnes GraphicHGR ont t'elles t modifies depuis la dernirer fois. */
/*************************************************************************************************************/
static int IsGraphicHGRDataModified(int graphichgr_bank, int graphichgr_address, int nb_column, int nb_row, struct window_graphichgr *window_graphichgr)
{
  int x, y, graphichgr_data_address;
  unsigned char graphichgr_raw_data[8192];   /* 8 KB */
  unsigned char graphichgr_data[40*192];
  int tab_hgr_address[192] =     /* Adresse du dbut des 192 lignes de la Page HGR */
    {
      0x0000+0x0000, 0x0000+0x0400, 0x0000+0x0800, 0x0000+0x0C00, 0x0000+0x1000, 0x0000+0x1400, 0x0000+0x1800, 0x0000+0x1C00, 
      0x0080+0x0000, 0x0080+0x0400, 0x0080+0x0800, 0x0080+0x0C00, 0x0080+0x1000, 0x0080+0x1400, 0x0080+0x1800, 0x0080+0x1C00, 
      0x0100+0x0000, 0x0100+0x0400, 0x0100+0x0800, 0x0100+0x0C00, 0x0100+0x1000, 0x0100+0x1400, 0x0100+0x1800, 0x0100+0x1C00, 
      0x0180+0x0000, 0x0180+0x0400, 0x0180+0x0800, 0x0180+0x0C00, 0x0180+0x1000, 0x0180+0x1400, 0x0180+0x1800, 0x0180+0x1C00, 
      0x0200+0x0000, 0x0200+0x0400, 0x0200+0x0800, 0x0200+0x0C00, 0x0200+0x1000, 0x0200+0x1400, 0x0200+0x1800, 0x0200+0x1C00, 
      0x0280+0x0000, 0x0280+0x0400, 0x0280+0x0800, 0x0280+0x0C00, 0x0280+0x1000, 0x0280+0x1400, 0x0280+0x1800, 0x0280+0x1C00, 
      0x0300+0x0000, 0x0300+0x0400, 0x0300+0x0800, 0x0300+0x0C00, 0x0300+0x1000, 0x0300+0x1400, 0x0300+0x1800, 0x0300+0x1C00, 
      0x0380+0x0000, 0x0380+0x0400, 0x0380+0x0800, 0x0380+0x0C00, 0x0380+0x1000, 0x0380+0x1400, 0x0380+0x1800, 0x0380+0x1C00, 
      0x0028+0x0000, 0x0028+0x0400, 0x0028+0x0800, 0x0028+0x0C00, 0x0028+0x1000, 0x0028+0x1400, 0x0028+0x1800, 0x0028+0x1C00, 
      0x00A8+0x0000, 0x00A8+0x0400, 0x00A8+0x0800, 0x00A8+0x0C00, 0x00A8+0x1000, 0x00A8+0x1400, 0x00A8+0x1800, 0x00A8+0x1C00, 
      0x0128+0x0000, 0x0128+0x0400, 0x0128+0x0800, 0x0128+0x0C00, 0x0128+0x1000, 0x0128+0x1400, 0x0128+0x1800, 0x0128+0x1C00, 
      0x01A8+0x0000, 0x01A8+0x0400, 0x01A8+0x0800, 0x01A8+0x0C00, 0x01A8+0x1000, 0x01A8+0x1400, 0x01A8+0x1800, 0x01A8+0x1C00, 
      0x0228+0x0000, 0x0228+0x0400, 0x0228+0x0800, 0x0228+0x0C00, 0x0228+0x1000, 0x0228+0x1400, 0x0228+0x1800, 0x0228+0x1C00, 
      0x02A8+0x0000, 0x02A8+0x0400, 0x02A8+0x0800, 0x02A8+0x0C00, 0x02A8+0x1000, 0x02A8+0x1400, 0x02A8+0x1800, 0x02A8+0x1C00, 
      0x0328+0x0000, 0x0328+0x0400, 0x0328+0x0800, 0x0328+0x0C00, 0x0328+0x1000, 0x0328+0x1400, 0x0328+0x1800, 0x0328+0x1C00, 
      0x03A8+0x0000, 0x03A8+0x0400, 0x03A8+0x0800, 0x03A8+0x0C00, 0x03A8+0x1000, 0x03A8+0x1400, 0x03A8+0x1800, 0x03A8+0x1C00, 
      0x0050+0x0000, 0x0050+0x0400, 0x0050+0x0800, 0x0050+0x0C00, 0x0050+0x1000, 0x0050+0x1400, 0x0050+0x1800, 0x0050+0x1C00, 
      0x00D0+0x0000, 0x00D0+0x0400, 0x00D0+0x0800, 0x00D0+0x0C00, 0x00D0+0x1000, 0x00D0+0x1400, 0x00D0+0x1800, 0x00D0+0x1C00, 
      0x0150+0x0000, 0x0150+0x0400, 0x0150+0x0800, 0x0150+0x0C00, 0x0150+0x1000, 0x0150+0x1400, 0x0150+0x1800, 0x0150+0x1C00, 
      0x01D0+0x0000, 0x01D0+0x0400, 0x01D0+0x0800, 0x01D0+0x0C00, 0x01D0+0x1000, 0x01D0+0x1400, 0x01D0+0x1800, 0x01D0+0x1C00, 
      0x0250+0x0000, 0x0250+0x0400, 0x0250+0x0800, 0x0250+0x0C00, 0x0250+0x1000, 0x0250+0x1400, 0x0250+0x1800, 0x0250+0x1C00, 
      0x02D0+0x0000, 0x02D0+0x0400, 0x02D0+0x0800, 0x02D0+0x0C00, 0x02D0+0x1000, 0x02D0+0x1400, 0x02D0+0x1800, 0x02D0+0x1C00, 
      0x0350+0x0000, 0x0350+0x0400, 0x0350+0x0800, 0x0350+0x0C00, 0x0350+0x1000, 0x0350+0x1400, 0x0350+0x1800, 0x0350+0x1C00, 
      0x03D0+0x0000, 0x03D0+0x0400, 0x03D0+0x0800, 0x03D0+0x0C00, 0x03D0+0x1000, 0x03D0+0x1400, 0x03D0+0x1800, 0x03D0+0x1C00
    };
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /* Rcupre les donnes brutes : 8192 bytes */
  graphichgr_data_address = (graphichgr_bank << 16) | graphichgr_address;
  ReadMemoryBlock(graphichgr_data_address,8192,graphichgr_raw_data,current_mmu);

  /* R-organise les donnes : Linarisation (passage du block de 8192 bytes en 280*192) */
  for(y=0; y<192; y++)
    for(x=0; x<40; x++)
      graphichgr_data[x+y*40] = graphichgr_raw_data[tab_hgr_address[y]+x];

  /* Compare les donnes */
  if(window_graphichgr->graphichgr_data != NULL)
    if(graphichgr_bank == window_graphichgr->graphichgr_bank &&
       graphichgr_address == window_graphichgr->graphichgr_address &&
       memcmp(&graphichgr_data[0],&window_graphichgr->graphichgr_data[0],40*192) == 0)
      return(0);

  /* Les donnes sont diffrentes => MAJ */
  window_graphichgr->graphichgr_bank = graphichgr_bank;
  window_graphichgr->graphichgr_address = graphichgr_address;
  memcpy(window_graphichgr->graphichgr_data,graphichgr_data,40*192);

  /* Donnes diffrentes */
  return(1);
}


/***********************************************************************/
/*  ResizeGraphicHGRWindow() : Resize GraphicHGR window by given size. */
/***********************************************************************/
static void ResizeGraphicHGRWindow(struct window_graphichgr *window_graphichgr)
{
  HWND hwnd_group;
  RECT window_rect, client_rect, group_rect;
  int window_x, window_y;
  
  /** On va ajuster la fentre par rapport au Zoom demand **/  
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_graphichgr->hwnd_graphichgr,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_graphichgr->hwnd_graphichgr,&client_rect);
  window_graphichgr->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select */
  hwnd_group = GetDlgItem(window_graphichgr->hwnd_graphichgr,IDC_GROUP_GRAPHICHGR);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_GRAPHICHGR */
  GetWindowRect(hwnd_group,&group_rect);
  window_graphichgr->group_data_x = (group_rect.left - window_rect.left) - 3;
  window_graphichgr->group_data_y = ((group_rect.top - window_rect.top) - window_graphichgr->title_height) + 3;

  /* Position de la Zone BipMap dans la fenetre */
  window_graphichgr->bitmap_graphichgr_x = window_graphichgr->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_graphichgr->bitmap_graphichgr_y = window_graphichgr->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_graphichgr->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_graphichgr->bitmap_graphichgr_width*window_graphichgr->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_graphichgr->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphichgr->bitmap_graphichgr_height*window_graphichgr->zoom_level + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_graphichgr->group_data_x,window_graphichgr->group_data_y,window_graphichgr->group_data_width,window_graphichgr->group_data_height,TRUE);

  /* Largeur de la fentre */
  window_graphichgr->window_width = window_graphichgr->group_data_x + window_graphichgr->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* On doit garantir une largeur minimale de la fentre */
  if(window_graphichgr->window_width < GRAPHICHGR_DIALOGBOX_WIDTH)
    window_graphichgr->window_width = GRAPHICHGR_DIALOGBOX_WIDTH;

  /* Hauteur de la fentre */
  window_graphichgr->window_height = window_graphichgr->title_height + window_graphichgr->group_data_y + window_graphichgr->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Resize window */
  MoveWindow(window_graphichgr->hwnd_graphichgr,window_x,window_y,window_graphichgr->window_width,window_graphichgr->window_height,TRUE);
}


/************************************************************************************************************/
/*  BuildGraphicHGRFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/************************************************************************************************************/
static void BuildGraphicHGRFromData(struct window_graphichgr *window_graphichgr)
{
  unsigned char one_byte;
  int bit_previous, bit_0, bit_1, bit_2, bit_3, bit_4, bit_5, bit_6, bit_next, bit_color;
  int offset, x, y;
  char buffer_title[256];
  DWORD tab_rgb_color[16] = 
    {
      /* 8 couleurs pour le mode HGR */
      MAKECOLOR(0,0,0), MAKECOLOR(0,221,0),   MAKECOLOR(221,0,221), MAKECOLOR(255,255,255),
      MAKECOLOR(0,0,0), MAKECOLOR(255,102,0), MAKECOLOR(34,34,255), MAKECOLOR(255,255,255)
    };
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rien  faire */
  if(window_graphichgr->bitmap_data_graphichgr == NULL)
    return;    

  /** Cration de la bitmap 280*192 correspondant au 40*192 du HGR **/
  bit_previous = 0;
  offset = 0;
  for(y=0; y<192; y++)
    {
      for(x=0; x<40; x++)
        {
          /* Bit Suivant */
          if(x == 39 && y == 191)
            bit_next = 0;                                                             /* Fin de l'image */
          else if(x == 39)
            bit_next = (window_graphichgr->graphichgr_data[(y+1)*40+0] & 0x01);       /* Ligne suivante, Byte #0 */
          else
            bit_next = (window_graphichgr->graphichgr_data[y*40+(x+1)] & 0x01);       /* Byte suivant sur la mme ligne */

          /* 1 Byte = 7 pixels */
          one_byte = window_graphichgr->graphichgr_data[y*40+x];
          bit_color = (one_byte & 0x80) >> 7;                                         /* 0=Vert+Violet, 1=Bleu+Orange */
          bit_6 = (one_byte & 0x40) >> 6;
          bit_5 = (one_byte & 0x20) >> 5;
          bit_4 = (one_byte & 0x10) >> 4;
          bit_3 = (one_byte & 0x08) >> 3;
          bit_2 = (one_byte & 0x04) >> 2;
          bit_1 = (one_byte & 0x02) >> 1;
          bit_0 = (one_byte & 0x01);
                  
          /* Codage des points */
          window_graphichgr->bitmap_data_graphichgr[offset++] = tab_rgb_color[GetHGRColor(bit_previous,bit_0,bit_1,bit_color,x*7+0)];
          window_graphichgr->bitmap_data_graphichgr[offset++] = tab_rgb_color[GetHGRColor(bit_0,bit_1,bit_2,bit_color,x*7+1)];
          window_graphichgr->bitmap_data_graphichgr[offset++] = tab_rgb_color[GetHGRColor(bit_1,bit_2,bit_3,bit_color,x*7+2)];
          window_graphichgr->bitmap_data_graphichgr[offset++] = tab_rgb_color[GetHGRColor(bit_2,bit_3,bit_4,bit_color,x*7+3)];
          window_graphichgr->bitmap_data_graphichgr[offset++] = tab_rgb_color[GetHGRColor(bit_3,bit_4,bit_5,bit_color,x*7+4)];
          window_graphichgr->bitmap_data_graphichgr[offset++] = tab_rgb_color[GetHGRColor(bit_4,bit_5,bit_6,bit_color,x*7+5)];
          window_graphichgr->bitmap_data_graphichgr[offset++] = tab_rgb_color[GetHGRColor(bit_5,bit_6,bit_next,bit_color,x*7+6)];
                  
          /* Conserve le Bit prcdant */
          bit_previous = bit_6;
        }
    }

  /* Titre de la fentre */
  sprintf(buffer_title,"Graphic HGR : %02X/%04X",window_graphichgr->graphichgr_bank,window_graphichgr->graphichgr_address);
  SetWindowText(window_graphichgr->hwnd_graphichgr,buffer_title);
}


/************************************************************************/
/*  DeleteGraphicHGRChildWindows() :  Destruction des fentres enfants. */
/************************************************************************/
static void DeleteGraphicHGRChildWindows(struct window_graphichgr *window_graphichgr)
{
  DeleteObject(window_graphichgr->hBitmap_graphichgr);
  ReleaseDC(window_graphichgr->hwnd_graphichgr,window_graphichgr->hDC);
  DeleteDC(window_graphichgr->hCompatibleDC);
}


/**************************************************************************/
/*  mem_free_window_graphichgr() :  Libration mmoire pour la structure. */
/**************************************************************************/
void mem_free_window_graphichgr(struct window_graphichgr *window_graphichgr)
{
  if(window_graphichgr)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphichgr->current_notification,NULL);
      window_graphichgr->current_notification = NULL;
      
      free(window_graphichgr);
    }
}

/**********************************************************************/
