/************************************************************************************/
/*                                                                                  */
/*  Window_GraphicHGRText40.c :  Module de gestion de la fentre GraphicHGRText40.  */
/*                                                                                  */
/************************************************************************************/
/*    Auteur : Olivier ZARDINI    *     Brutal Deluxe     *    Date : Juillet 2018  */
/************************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_GraphicHGRText40.h"

static DWORD BuildGraphicHGRText40WindowThread(void *);
LRESULT CALLBACK ProcHandlerGraphicHGRText40Window(HWND,UINT,WPARAM,LPARAM);
static int CreateGraphicHGRText40ChildWindows(HWND,struct window_graphichgrtext40 *);
LRESULT CALLBACK subEditGraphicHGRText40ProcControl(HWND,UINT,WPARAM,LPARAM);
void UpdateGraphicHGRText40ClientAllArea(HDC,struct window_graphichgrtext40 *);
static void UpdateGraphicHGRText40ClientBitmapArea(HDC,struct window_graphichgrtext40 *);
static void UpdateGraphicHGRText40ContextInformation(struct window_graphichgrtext40 *,int,int);
static void UpdateGraphicHGRText40AddressGraphic(struct window_graphichgrtext40 *,int);
static void UpdateGraphicHGRText40AddressText(struct window_graphichgrtext40 *,int);
static void UpdateGraphicHGRText40RefreshMode(struct window_graphichgrtext40 *,int);
static void UpdateGraphicHGRText40FlashMode(struct window_graphichgrtext40 *);
static void UpdateGraphicHGRText40ZoomLevel(struct window_graphichgrtext40 *,int);
static void RefreshGraphicHGRText40WindowData(struct window_graphichgrtext40 *,int);
static int IsGraphicHGRText40DataModified(int,int,int,int,struct window_graphichgrtext40 *);
static void ResizeGraphicHGRText40Window(struct window_graphichgrtext40 *);
static void BuildGraphicHGRText40FromData(struct window_graphichgrtext40 *);
static void DeleteGraphicHGRText40ChildWindows(struct window_graphichgrtext40 *);


/**********************************************************************************************/
/*  BuildGraphicHGRText40Window() :  Construction / Affichage de la fentre GraphicHGRText40. */
/**********************************************************************************************/
struct window_graphichgrtext40 *BuildGraphicHGRText40Window(HINSTANCE hInstance, int menu_id)
{
  struct window_graphichgrtext40 *window_graphichgrtext40;

  /** Allocation de la structure window_graphichgrtext40 **/
  window_graphichgrtext40 = (struct window_graphichgrtext40 *) calloc(1,sizeof(struct window_graphichgrtext40));
  if(window_graphichgrtext40 == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_GRAPHICHGRTEXT40,window_graphichgrtext40,NULL);

  /* Lance un thread pour grer la fentre GraphicHGRText40 */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildGraphicHGRText40WindowThread,window_graphichgrtext40,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_graphichgrtext40);
}


/********************************************************************************************************/
/*  BuildGraphicHGRText40WindowThread() :  Cration de la fentre GraphicHGRText40 via un Thread ddi. */
/********************************************************************************************************/
static DWORD BuildGraphicHGRText40WindowThread(void *window_graphichgrtext40_param)
{
  HWND hwndGraphicHGRText40;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_graphichgrtext40 *window_graphichgrtext40;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_graphichgrtext40 = (struct window_graphichgrtext40 *) window_graphichgrtext40_param;
  window_graphichgrtext40->thread_id = GetCurrentThreadId();         /* Le ThreaID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Text Page 40 Column **/
  hwndGraphicHGRText40 = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_GRAPHICHGRTEXT40),window_main->hwnd_main,(DLGPROC)ProcHandlerGraphicHGRText40Window);
  if(hwndGraphicHGRText40 == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndGraphicHGRText40,SW_SHOWDEFAULT);
  UpdateWindow(hwndGraphicHGRText40);

  /** Message Loop pour toutes les fentres de ce Thread (fentre GraphicHGRText40 + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/*********************************************************************************/
/*  ProcHandlerGraphicHGRText40Window() :  Gestion des vnements de la fentre. */
/*********************************************************************************/
LRESULT CALLBACK ProcHandlerGraphicHGRText40Window(HWND hwndGraphicHGRText40, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;
  LRESULT result;
  WORD param_wd;
  int mouse_x, mouse_y;
  struct window_graphichgrtext40 *window_graphichgrtext40;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_GRAPHICHGRTEXT40,&window_graphichgrtext40,NULL);
  if(window_graphichgrtext40 == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateGraphicHGRText40ChildWindows(hwndGraphicHGRText40,window_graphichgrtext40);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        UpdateGraphicHGRText40ContextInformation(window_graphichgrtext40,mouse_x,mouse_y);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_graphichgrtext40->hBitmap_graphichgrtext40 != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndGraphicHGRText40,&ps);

            /** MAJ Bitmap **/
            UpdateGraphicHGRText40ClientAllArea(hDC,window_graphichgrtext40);

            /* End Paint */
            EndPaint(hwndGraphicHGRText40,&ps);
          }
        break;
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_graphichgrtext40->refresh_mode != GRAPHICHGRTEXT40_REFRESH_AUTOMATIC)
                UpdateGraphicHGRText40RefreshMode(window_graphichgrtext40,GRAPHICHGRTEXT40_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_graphichgrtext40->refresh_mode != GRAPHICHGRTEXT40_REFRESH_MANUAL)
                UpdateGraphicHGRText40RefreshMode(window_graphichgrtext40,GRAPHICHGRTEXT40_REFRESH_MANUAL);
              break;

            /** Bouton Radio : On change le Zoom de la fentre **/
            case IDC_RADIO_ZOOM_X1 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphichgrtext40->zoom_level != GRAPHICHGRTEXT40_ZOOM_X1)
                UpdateGraphicHGRText40ZoomLevel(window_graphichgrtext40,GRAPHICHGRTEXT40_ZOOM_X1);
              break;

            case IDC_RADIO_ZOOM_X2 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphichgrtext40->zoom_level != GRAPHICHGRTEXT40_ZOOM_X2)
                UpdateGraphicHGRText40ZoomLevel(window_graphichgrtext40,GRAPHICHGRTEXT40_ZOOM_X2);
              break;

            case IDC_RADIO_ZOOM_X3 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphichgrtext40->zoom_level != GRAPHICHGRTEXT40_ZOOM_X3)
                UpdateGraphicHGRText40ZoomLevel(window_graphichgrtext40,GRAPHICHGRTEXT40_ZOOM_X3);
              break;

            /** Bouton de dplacement de l'adresse Graphic **/
            case IDC_BUTTON_LEFT_GRAPHIC :
              UpdateGraphicHGRText40AddressGraphic(window_graphichgrtext40,-1);
              break;

            case IDC_BUTTON_RIGHT_GRAPHIC :
              UpdateGraphicHGRText40AddressGraphic(window_graphichgrtext40,1);
              break;

            case IDC_BUTTON_UP_GRAPHIC :
              UpdateGraphicHGRText40AddressGraphic(window_graphichgrtext40,-40);
              break;

            case IDC_BUTTON_DOWN_GRAPHIC :
              UpdateGraphicHGRText40AddressGraphic(window_graphichgrtext40,40);
              break;

            case IDC_BUTTON_PAGEUP_GRAPHIC :
              UpdateGraphicHGRText40AddressGraphic(window_graphichgrtext40,-8192);
              break;

            case IDC_BUTTON_PAGEDOWN_GRAPHIC :
              UpdateGraphicHGRText40AddressGraphic(window_graphichgrtext40,8192);
              break;

            /** Bouton de dplacement de l'adresse Text **/
            case IDC_BUTTON_LEFT_TEXT :
              UpdateGraphicHGRText40AddressText(window_graphichgrtext40,-1);
              break;

            case IDC_BUTTON_RIGHT_TEXT :
              UpdateGraphicHGRText40AddressText(window_graphichgrtext40,1);
              break;

            case IDC_BUTTON_UP_TEXT :
              UpdateGraphicHGRText40AddressText(window_graphichgrtext40,-40);
              break;

            case IDC_BUTTON_DOWN_TEXT :
              UpdateGraphicHGRText40AddressText(window_graphichgrtext40,40);
              break;

            case IDC_BUTTON_PAGEUP_TEXT :
              UpdateGraphicHGRText40AddressText(window_graphichgrtext40,-1024);
              break;

            case IDC_BUTTON_PAGEDOWN_TEXT :
              UpdateGraphicHGRText40AddressText(window_graphichgrtext40,1024);
              break;

            /** Demande d'activation du Flash **/
            case IDC_CHECK_FLASH :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateGraphicHGRText40FlashMode(window_graphichgrtext40);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshGraphicHGRText40WindowData(window_graphichgrtext40,1);
              break;
              
            /** Slection dans la ComboBox **/
            case IDC_COMBOBOX_GRAPHICHGRTEXT40_COLOR_TEXT :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphichgrtext40->hwnd_combo_color_text,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphichgrtext40->color_text)
                      {
                        window_graphichgrtext40->color_text = (int) result;
                        RefreshGraphicHGRText40WindowData(window_graphichgrtext40,1);
                      }
                }
              break;              
             
            case IDC_COMBOBOX_GRAPHICHGRTEXT40_COLOR_BACKGROUND :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphichgrtext40->hwnd_combo_color_background,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphichgrtext40->color_background)
                      {
                        window_graphichgrtext40->color_background = (int) result;
                        RefreshGraphicHGRText40WindowData(window_graphichgrtext40,1);
                      }
                }
              break; 
             
            case IDC_COMBOBOX_GRAPHICHGRTEXT40_DISPLAY_LANGUAGE :
              param_wd = HIWORD(wParam);
              if(param_wd == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphichgrtext40->hwnd_combo_display_language,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphichgrtext40->display_language)
                      {
                        window_graphichgrtext40->display_language = (int) result;
                        RefreshGraphicHGRText40WindowData(window_graphichgrtext40,1);
                      }
                }
              break;                           
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_GRAPHICHGRTEXT40_REFRESHWINDOW :
        RefreshGraphicHGRText40WindowData(window_graphichgrtext40,0);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre GraphicHGRText40 */
        DestroyWindow(hwndGraphicHGRText40);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteGraphicHGRText40ChildWindows(window_graphichgrtext40);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_GRAPHICHGRTEXT40,window_graphichgrtext40,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************************/
/*  CreateGraphicHGRText40ChildWindows() : Crer les fentres enfants et remplir la structure window_graphichgrtext40. */
/***********************************************************************************************************************/
static int CreateGraphicHGRText40ChildWindows(HWND hwndGraphicHGRText40, struct window_graphichgrtext40 *window_graphichgrtext40)
{
  BITMAPINFO bitmap_info;
  int i;
  char buffer[BUFFER_SIZE];
  char *color_name[16] = {"Black","Deep Red","Dark Blue","Purple","Dark Green","Dark Gray","Medium Blue","Light Blue","Brown","Orange","Light Gray","Pink","Light Green","Yellow","Aquamarine","White"};
  char *display_language[9] = {"U.S.A.","U.K.","French","Danish","Spanish","Italian","German","Swedish","MouseText"};
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_graphichgrtext40->hwnd_graphichgrtext40 = hwndGraphicHGRText40;

  /** Device Context **/
  window_graphichgrtext40->hDC = GetDC(window_graphichgrtext40->hwnd_graphichgrtext40);
  window_graphichgrtext40->hCompatibleDC = CreateCompatibleDC(window_graphichgrtext40->hDC);

  /** Surchage des Control Edit pour capter les vnements **/
  window_graphichgrtext40->oldEditProcBankGraphic    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicHGRText40,IDC_EDIT_BANK_GRAPHIC),GWLP_WNDPROC,(LONG_PTR)subEditGraphicHGRText40ProcControl);
  window_graphichgrtext40->oldEditProcAddressGraphic = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicHGRText40,IDC_EDIT_ADDRESS_GRAPHIC),GWLP_WNDPROC,(LONG_PTR)subEditGraphicHGRText40ProcControl);
  window_graphichgrtext40->oldEditProcBankText       = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicHGRText40,IDC_EDIT_BANK_TEXT),GWLP_WNDPROC,(LONG_PTR)subEditGraphicHGRText40ProcControl);
  window_graphichgrtext40->oldEditProcAddressText    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicHGRText40,IDC_EDIT_ADDRESS_TEXT),GWLP_WNDPROC,(LONG_PTR)subEditGraphicHGRText40ProcControl);

  /** Ajoute les images sur les boutons **/
  /* Graphic */
  SendMessage(GetDlgItem(hwndGraphicHGRText40,IDC_BUTTON_LEFT_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicHGRText40,IDC_BUTTON_RIGHT_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicHGRText40,IDC_BUTTON_UP_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicHGRText40,IDC_BUTTON_DOWN_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicHGRText40,IDC_BUTTON_PAGEUP_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicHGRText40,IDC_BUTTON_PAGEDOWN_GRAPHIC),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));
  /* Text */
  SendMessage(GetDlgItem(hwndGraphicHGRText40,IDC_BUTTON_LEFT_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicHGRText40,IDC_BUTTON_RIGHT_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicHGRText40,IDC_BUTTON_UP_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicHGRText40,IDC_BUTTON_DOWN_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicHGRText40,IDC_BUTTON_PAGEUP_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicHGRText40,IDC_BUTTON_PAGEDOWN_TEXT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Cration des ComboBox **/
  /* Text Color */
  window_graphichgrtext40->hwnd_combo_color_text = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                  104,102,100,250,hwndGraphicHGRText40,(HMENU)IDC_COMBOBOX_GRAPHICHGRTEXT40_COLOR_TEXT,param->hInstance,0);
  for(i=0; i<16; i++)
    SendMessage(window_graphichgrtext40->hwnd_combo_color_text,CB_ADDSTRING,0,(LPARAM)color_name[i]);

  /* Background Color */
  window_graphichgrtext40->hwnd_combo_color_background = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                                        309,102,100,250,hwndGraphicHGRText40,(HMENU)IDC_COMBOBOX_GRAPHICHGRTEXT40_COLOR_BACKGROUND,param->hInstance,0);
  for(i=0; i<16; i++)
    SendMessage(window_graphichgrtext40->hwnd_combo_color_background,CB_ADDSTRING,0,(LPARAM)color_name[i]);

  /* Display Language */
  window_graphichgrtext40->hwnd_combo_display_language = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                                        309,134,100,220,hwndGraphicHGRText40,(HMENU)IDC_COMBOBOX_GRAPHICHGRTEXT40_DISPLAY_LANGUAGE,param->hInstance,0);
  for(i=0; i<9; i++)
    SendMessage(window_graphichgrtext40->hwnd_combo_display_language,CB_ADDSTRING,0,(LPARAM)display_language[i]);

  /* 1res valeurs */
  window_graphichgrtext40->color_text = COLOR_IIGS_WHITE;
  window_graphichgrtext40->color_background = COLOR_IIGS_MEDIUM_BLUE;
  window_graphichgrtext40->display_language = LANGUAGE_IIGS_USA;
  SendMessage(window_graphichgrtext40->hwnd_combo_color_text,CB_SETCURSEL,window_graphichgrtext40->color_text,0);
  SendMessage(window_graphichgrtext40->hwnd_combo_color_background,CB_SETCURSEL,window_graphichgrtext40->color_background,0);
  SendMessage(window_graphichgrtext40->hwnd_combo_display_language,CB_SETCURSEL,window_graphichgrtext40->display_language,0);

  /** Cration de la Police de caractre + Affectation de la police au Control Edit **/
  window_graphichgrtext40->hFont = CreateFont(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");
  SendMessage(window_graphichgrtext40->hwnd_combo_color_text,WM_SETFONT,(WPARAM)window_graphichgrtext40->hFont,TRUE);
  SendMessage(window_graphichgrtext40->hwnd_combo_color_background,WM_SETFONT,(WPARAM)window_graphichgrtext40->hFont,TRUE);
  SendMessage(window_graphichgrtext40->hwnd_combo_display_language,WM_SETFONT,(WPARAM)window_graphichgrtext40->hFont,TRUE);

  /** Adresse de la Zone de Data **/
  /* Graphic */
  window_graphichgrtext40->graphichgr_bank = 0x00;
  window_graphichgrtext40->graphichgr_address = 0x2000; 
  sprintf(buffer,"%02X",window_graphichgrtext40->graphichgr_bank);
  SetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_BANK_GRAPHIC,buffer);
  sprintf(buffer,"%04X",window_graphichgrtext40->graphichgr_address);
  SetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_ADDRESS_GRAPHIC,buffer);
  /* Text */
  window_graphichgrtext40->textpage40_bank = 0x00;
  window_graphichgrtext40->textpage40_address = 0x0400; 
  sprintf(buffer,"%02X",window_graphichgrtext40->textpage40_bank);
  SetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_BANK_TEXT,buffer);
  sprintf(buffer,"%04X",window_graphichgrtext40->textpage40_address);
  SetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_ADDRESS_TEXT,buffer);

  /* Refresh Mode : Automatic */
  UpdateGraphicHGRText40RefreshMode(window_graphichgrtext40,GRAPHICHGRTEXT40_REFRESH_AUTOMATIC);

  /* Zoom Level : 1 */
  window_graphichgrtext40->zoom_level = GRAPHICHGRTEXT40_ZOOM_X1;
  SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);

  /* Fash des caractres */
  window_graphichgrtext40->flash = 0;

  /* Dimension en Pixel de la Bitmap GraphicHGRText40 Data */
  window_graphichgrtext40->bitmap_graphichgrtext40_width = 280;
  window_graphichgrtext40->bitmap_graphichgrtext40_height = 192;
  
  /** Bitmap affichant la zone de slection **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphichgrtext40->bitmap_graphichgrtext40_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphichgrtext40->bitmap_graphichgrtext40_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphichgrtext40->hBitmap_graphichgrtext40 = CreateDIBSection(window_graphichgrtext40->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphichgrtext40->bitmap_data_graphichgrtext40),NULL,0);
  for(i=0; i<window_graphichgrtext40->bitmap_graphichgrtext40_width*window_graphichgrtext40->bitmap_graphichgrtext40_height; i++)
    window_graphichgrtext40->bitmap_data_graphichgrtext40[i] = BITMAP_BACKGROUND_COLOR;

  /* Retaille la fentre */
  ResizeGraphicHGRText40Window(window_graphichgrtext40);

  /** Construction de la Page Texte + Rafraichissement de la fentre **/
  RefreshGraphicHGRText40WindowData(window_graphichgrtext40,1);
    
  /* OK */
  return(0);
}


/*******************************************************************************************************/
/*  subEditGraphicHGRText40ProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*******************************************************************************************************/
LRESULT CALLBACK subEditGraphicHGRText40ProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_graphichgrtext40 *window_graphichgrtext40;
  my_Memory(MEMORY_GET_WINDOW_GRAPHICHGRTEXT40,&window_graphichgrtext40,NULL);

  /* Rien  faire */
  if(window_graphichgrtext40 == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_BANK_GRAPHIC) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_ADDRESS_GRAPHIC) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_BANK_TEXT) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_ADDRESS_TEXT) == hwnd_control)
    index = 4;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement du Return dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_graphichgrtext40->hwnd_graphichgrtext40);

      /* On force la MAJ de la fentre */
      RefreshGraphicHGRText40WindowData(window_graphichgrtext40,1);
    }
  else if(index == 1)
    return(CallWindowProc(window_graphichgrtext40->oldEditProcBankGraphic,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_graphichgrtext40->oldEditProcAddressGraphic,hwnd_control,msg,wParam,lParam)); 
  else if(index == 3)
    return(CallWindowProc(window_graphichgrtext40->oldEditProcBankText,hwnd_control,msg,wParam,lParam)); 
  else if(index == 4)
    return(CallWindowProc(window_graphichgrtext40->oldEditProcAddressText,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
} 


/**************************************************************************************************/
/*  UpdateGraphicHGRText40ClientAllArea() :  Affichage des Bitmap de la fentre GraphicHGRText40. */
/**************************************************************************************************/
void UpdateGraphicHGRText40ClientAllArea(HDC hDC, struct window_graphichgrtext40 *window_graphichgrtext40)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap **/
  UpdateGraphicHGRText40ClientBitmapArea(hDC,window_graphichgrtext40);
}


/*****************************************************************************************/
/*  UpdateGraphicHGRText40ClientBitmapArea() :  Affichage de la Bitmap GraphicHGRText40. */
/*****************************************************************************************/
static void UpdateGraphicHGRText40ClientBitmapArea(HDC hDC, struct window_graphichgrtext40 *window_graphichgrtext40)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec la zone mmoire et la zone slectionne **/
  bitmap_old = SelectObject(window_graphichgrtext40->hCompatibleDC,window_graphichgrtext40->hBitmap_graphichgrtext40);
  StretchBlt(hDC,window_graphichgrtext40->bitmap_graphichgrtext40_x,window_graphichgrtext40->bitmap_graphichgrtext40_y,
             window_graphichgrtext40->bitmap_graphichgrtext40_width*window_graphichgrtext40->zoom_level,
             window_graphichgrtext40->bitmap_graphichgrtext40_height*window_graphichgrtext40->zoom_level,
             window_graphichgrtext40->hCompatibleDC,0,0,window_graphichgrtext40->bitmap_graphichgrtext40_width,window_graphichgrtext40->bitmap_graphichgrtext40_height,SRCCOPY);
  SelectObject(window_graphichgrtext40->hCompatibleDC,bitmap_old);
}


/**********************************************************************************************************/
/*  UpdateGraphicHGRText40AddressGraphic() :  Dplace l'adresse Graphic pour la fentre GraphicHGRText40. */
/**********************************************************************************************************/
static void UpdateGraphicHGRText40AddressGraphic(struct window_graphichgrtext40 *window_graphichgrtext40, int delta)
{
  int current_address, new_address, graphichgr_bank, graphichgr_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphichgrtext40->graphichgr_bank << 16) | window_graphichgrtext40->graphichgr_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  graphichgr_bank = (new_address >> 16) & 0xFF;
  graphichgr_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphichgr_bank);
  SetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_BANK_GRAPHIC,buffer);
  sprintf(buffer,"%04X",graphichgr_address);
  SetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_ADDRESS_GRAPHIC,buffer);

  /* MAJ de la fentre */
  RefreshGraphicHGRText40WindowData(window_graphichgrtext40,1);
}


/****************************************************************************************************/
/*  UpdateGraphicHGRText40AddressText() :  Dplace l'adresse Text pour la fentre GraphicHGRText40. */
/****************************************************************************************************/
static void UpdateGraphicHGRText40AddressText(struct window_graphichgrtext40 *window_graphichgrtext40, int delta)
{
  int current_address, new_address, textpage40_bank, textpage40_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphichgrtext40->textpage40_bank << 16) | window_graphichgrtext40->textpage40_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  textpage40_bank = (new_address >> 16) & 0xFF;
  textpage40_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",textpage40_bank);
  SetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_BANK_TEXT,buffer);
  sprintf(buffer,"%04X",textpage40_address);
  SetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_ADDRESS_TEXT,buffer);

  /* MAJ de la fentre */
  RefreshGraphicHGRText40WindowData(window_graphichgrtext40,1);
}


/*******************************************************************************************************************/
/*  UpdateGraphicHGRText40ContextInformation() :  MAJ des information de contexte de la fenetre GraphicHGRText40. */
/*******************************************************************************************************************/
static void UpdateGraphicHGRText40ContextInformation(struct window_graphichgrtext40 *window_graphichgrtext40, int mouse_x, int mouse_y)
{
  int pixmap_x, pixmap_y, grid_x, grid_y;
  int graphichgr_data_address;
  int textpage40_data_address;
  unsigned char graphichgr_data;
  unsigned char textpage40_data;
  int tab_hgr_address[192] =     /* Adresse du dbut des 192 lignes de la Page HGR */
    {
      0x0000+0x0000, 0x0000+0x0400, 0x0000+0x0800, 0x0000+0x0C00, 0x0000+0x1000, 0x0000+0x1400, 0x0000+0x1800, 0x0000+0x1C00, 
      0x0080+0x0000, 0x0080+0x0400, 0x0080+0x0800, 0x0080+0x0C00, 0x0080+0x1000, 0x0080+0x1400, 0x0080+0x1800, 0x0080+0x1C00, 
      0x0100+0x0000, 0x0100+0x0400, 0x0100+0x0800, 0x0100+0x0C00, 0x0100+0x1000, 0x0100+0x1400, 0x0100+0x1800, 0x0100+0x1C00, 
      0x0180+0x0000, 0x0180+0x0400, 0x0180+0x0800, 0x0180+0x0C00, 0x0180+0x1000, 0x0180+0x1400, 0x0180+0x1800, 0x0180+0x1C00, 
      0x0200+0x0000, 0x0200+0x0400, 0x0200+0x0800, 0x0200+0x0C00, 0x0200+0x1000, 0x0200+0x1400, 0x0200+0x1800, 0x0200+0x1C00, 
      0x0280+0x0000, 0x0280+0x0400, 0x0280+0x0800, 0x0280+0x0C00, 0x0280+0x1000, 0x0280+0x1400, 0x0280+0x1800, 0x0280+0x1C00, 
      0x0300+0x0000, 0x0300+0x0400, 0x0300+0x0800, 0x0300+0x0C00, 0x0300+0x1000, 0x0300+0x1400, 0x0300+0x1800, 0x0300+0x1C00, 
      0x0380+0x0000, 0x0380+0x0400, 0x0380+0x0800, 0x0380+0x0C00, 0x0380+0x1000, 0x0380+0x1400, 0x0380+0x1800, 0x0380+0x1C00, 
      0x0028+0x0000, 0x0028+0x0400, 0x0028+0x0800, 0x0028+0x0C00, 0x0028+0x1000, 0x0028+0x1400, 0x0028+0x1800, 0x0028+0x1C00, 
      0x00A8+0x0000, 0x00A8+0x0400, 0x00A8+0x0800, 0x00A8+0x0C00, 0x00A8+0x1000, 0x00A8+0x1400, 0x00A8+0x1800, 0x00A8+0x1C00, 
      0x0128+0x0000, 0x0128+0x0400, 0x0128+0x0800, 0x0128+0x0C00, 0x0128+0x1000, 0x0128+0x1400, 0x0128+0x1800, 0x0128+0x1C00, 
      0x01A8+0x0000, 0x01A8+0x0400, 0x01A8+0x0800, 0x01A8+0x0C00, 0x01A8+0x1000, 0x01A8+0x1400, 0x01A8+0x1800, 0x01A8+0x1C00, 
      0x0228+0x0000, 0x0228+0x0400, 0x0228+0x0800, 0x0228+0x0C00, 0x0228+0x1000, 0x0228+0x1400, 0x0228+0x1800, 0x0228+0x1C00, 
      0x02A8+0x0000, 0x02A8+0x0400, 0x02A8+0x0800, 0x02A8+0x0C00, 0x02A8+0x1000, 0x02A8+0x1400, 0x02A8+0x1800, 0x02A8+0x1C00, 
      0x0328+0x0000, 0x0328+0x0400, 0x0328+0x0800, 0x0328+0x0C00, 0x0328+0x1000, 0x0328+0x1400, 0x0328+0x1800, 0x0328+0x1C00, 
      0x03A8+0x0000, 0x03A8+0x0400, 0x03A8+0x0800, 0x03A8+0x0C00, 0x03A8+0x1000, 0x03A8+0x1400, 0x03A8+0x1800, 0x03A8+0x1C00, 
      0x0050+0x0000, 0x0050+0x0400, 0x0050+0x0800, 0x0050+0x0C00, 0x0050+0x1000, 0x0050+0x1400, 0x0050+0x1800, 0x0050+0x1C00, 
      0x00D0+0x0000, 0x00D0+0x0400, 0x00D0+0x0800, 0x00D0+0x0C00, 0x00D0+0x1000, 0x00D0+0x1400, 0x00D0+0x1800, 0x00D0+0x1C00, 
      0x0150+0x0000, 0x0150+0x0400, 0x0150+0x0800, 0x0150+0x0C00, 0x0150+0x1000, 0x0150+0x1400, 0x0150+0x1800, 0x0150+0x1C00, 
      0x01D0+0x0000, 0x01D0+0x0400, 0x01D0+0x0800, 0x01D0+0x0C00, 0x01D0+0x1000, 0x01D0+0x1400, 0x01D0+0x1800, 0x01D0+0x1C00, 
      0x0250+0x0000, 0x0250+0x0400, 0x0250+0x0800, 0x0250+0x0C00, 0x0250+0x1000, 0x0250+0x1400, 0x0250+0x1800, 0x0250+0x1C00, 
      0x02D0+0x0000, 0x02D0+0x0400, 0x02D0+0x0800, 0x02D0+0x0C00, 0x02D0+0x1000, 0x02D0+0x1400, 0x02D0+0x1800, 0x02D0+0x1C00, 
      0x0350+0x0000, 0x0350+0x0400, 0x0350+0x0800, 0x0350+0x0C00, 0x0350+0x1000, 0x0350+0x1400, 0x0350+0x1800, 0x0350+0x1C00, 
      0x03D0+0x0000, 0x03D0+0x0400, 0x03D0+0x0800, 0x03D0+0x0C00, 0x03D0+0x1000, 0x03D0+0x1400, 0x03D0+0x1800, 0x03D0+0x1C00
    };
  int tab_text40_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };

  /* Rien  faire */
  if(window_graphichgrtext40 == NULL)
    return;

  /* Est t'on dans l'image ? */
  if(mouse_x < window_graphichgrtext40->bitmap_graphichgrtext40_x || mouse_x >= (window_graphichgrtext40->bitmap_graphichgrtext40_x + window_graphichgrtext40->bitmap_graphichgrtext40_width*window_graphichgrtext40->zoom_level) ||
     mouse_y < window_graphichgrtext40->bitmap_graphichgrtext40_y || mouse_y >= (window_graphichgrtext40->bitmap_graphichgrtext40_y + window_graphichgrtext40->bitmap_graphichgrtext40_height*window_graphichgrtext40->zoom_level))
    {
      SetWindowText(GetDlgItem(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_STATIC_CONTEXT),"");
      return;
    }

  /* Coordonnes dans la Pixmap */
  pixmap_x = (mouse_x - window_graphichgrtext40->bitmap_graphichgrtext40_x)/window_graphichgrtext40->zoom_level;
  pixmap_y = (mouse_y - window_graphichgrtext40->bitmap_graphichgrtext40_y)/window_graphichgrtext40->zoom_level;

  /* Graphic */
  if(pixmap_y < 160)   /* 192 - 8*4 */
    {
      /* Coordonnes dans la grille 40x24 */
      grid_x = pixmap_x/7;

      /* Valeur */
      graphichgr_data = window_graphichgrtext40->graphichgr_data[grid_x + pixmap_y*40];

      /* Adresse */
      graphichgr_data_address = (window_graphichgrtext40->graphichgr_bank << 16) + window_graphichgrtext40->graphichgr_address;
      graphichgr_data_address += grid_x;
      graphichgr_data_address += tab_hgr_address[pixmap_y];

      /* Message */
      sprintf(window_graphichgrtext40->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",graphichgr_data_address>>16,graphichgr_data_address&0xFFFF,graphichgr_data,pixmap_x,pixmap_y);
      SetWindowText(GetDlgItem(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_STATIC_CONTEXT),window_graphichgrtext40->buffer_log);
    }
  else   /* Text */
    {
      /* Coordonnes dans la grille 40x24 */
      grid_x = pixmap_x/7;
      grid_y = pixmap_y/8;

      /* Valeur */
      textpage40_data = window_graphichgrtext40->textpage40_data[grid_x + grid_y*40];

      /* Adresse */
      textpage40_data_address = (window_graphichgrtext40->textpage40_bank << 16) + window_graphichgrtext40->textpage40_address;
      textpage40_data_address += grid_x;
      textpage40_data_address += tab_text40_address[grid_y];

      /* Message */
      sprintf(window_graphichgrtext40->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",textpage40_data_address>>16,textpage40_data_address&0xFFFF,textpage40_data,grid_x,grid_y);
      SetWindowText(GetDlgItem(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_STATIC_CONTEXT),window_graphichgrtext40->buffer_log);
    }
}


/************************************************************************************************************/
/*  UpdateGraphicHGRText40RefreshMode() :  MAJ du mode de rafraichissement de la fenetre GraphicHGRText40. */
/************************************************************************************************************/
static void UpdateGraphicHGRText40RefreshMode(struct window_graphichgrtext40 *window_graphichgrtext40, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == GRAPHICHGRTEXT40_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_graphichgrtext40->refresh_mode = GRAPHICHGRTEXT40_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_graphichgrtext40->hwnd_graphichgrtext40;
      current_notification->message = HM_GRAPHICHGRTEXT40_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_GRAPHICHGRTEXT40;

      /* Ajout de la notification */
      window_graphichgrtext40->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_graphichgrtext40->refresh_mode = GRAPHICHGRTEXT40_REFRESH_MANUAL;
      SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphichgrtext40->current_notification,NULL);
      window_graphichgrtext40->current_notification = NULL;
    }
}


/*******************************************************************************************************************/
/*  UpdateGraphicHGRText40FlashMode() :  MAJ de l'activation des caractres FLASH de la fenetre GraphicHGRText40. */
/*******************************************************************************************************************/
static void UpdateGraphicHGRText40FlashMode(struct window_graphichgrtext40 *window_graphichgrtext40)
{
  struct notification *current_notification;

  if(window_graphichgrtext40->flash == 0)
    {
      /* Visuel */
      window_graphichgrtext40->flash = 1;

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_graphichgrtext40->hwnd_graphichgrtext40;
      current_notification->message = HM_GRAPHICHGRTEXT40_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_TIMER;
      current_notification->timer_milliseconds = 500;

      /* Ajout de la notification */
      window_graphichgrtext40->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_graphichgrtext40->flash = 0;

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphichgrtext40->current_notification,NULL);
      window_graphichgrtext40->current_notification = NULL;
    }
}


/***********************************************************************************************/
/*  UpdateGraphicHGRText40ZoomLevel() :  MAJ du nieau de Zomm de la fenetre GraphicHGRText40. */
/***********************************************************************************************/
static void UpdateGraphicHGRText40ZoomLevel(struct window_graphichgrtext40 *window_graphichgrtext40, int zoom_level)
{
  if(zoom_level == GRAPHICHGRTEXT40_ZOOM_X1)
    {
      /* Visuel */
      window_graphichgrtext40->zoom_level = GRAPHICHGRTEXT40_ZOOM_X1;
      SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICHGRTEXT40_ZOOM_X2)
    {
      /* Visuel */
      window_graphichgrtext40->zoom_level = GRAPHICHGRTEXT40_ZOOM_X2;
      SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_ZOOM_X2,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICHGRTEXT40_ZOOM_X3)
    {
      /* Visuel */
      window_graphichgrtext40->zoom_level = GRAPHICHGRTEXT40_ZOOM_X3;
      SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_RADIO_ZOOM_X3,BM_SETCHECK,1,0);
    }
    
  /* Retaille la fentre */
  ResizeGraphicHGRText40Window(window_graphichgrtext40);  
  
  /* On force la MAJ de la fentre */  
  RefreshGraphicHGRText40WindowData(window_graphichgrtext40,1);    
}


/******************************************************************************************/
/*  RefreshGraphicHGRText40WindowData() :  MAJ du contenu de la fentre GraphicHGRText40. */
/******************************************************************************************/
static void RefreshGraphicHGRText40WindowData(struct window_graphichgrtext40 *window_graphichgrtext40, int force)
{
  int length, is_modified;
  int graphichgr_bank, graphichgr_address, textpage40_bank, textpage40_address;
  char buffer[1024];

  /** Rcupre la valeur des GraphicHGRText40 Pointer **/
  /* Graphic */
  length = GetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_BANK_GRAPHIC,buffer,1024);
  sscanf(buffer,"%X",&graphichgr_bank);
  length = GetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_ADDRESS_GRAPHIC,buffer,1024);
  sscanf(buffer,"%X",&graphichgr_address);
  /* Text */
  length = GetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_BANK_TEXT,buffer,1024);
  sscanf(buffer,"%X",&textpage40_bank);
  length = GetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_ADDRESS_TEXT,buffer,1024);
  sscanf(buffer,"%X",&textpage40_address);

  /** Replace les valeurs dans les Control pour montrer ce qui a t compris **/
  /* Graphic */
  sprintf(buffer,"%02X",graphichgr_bank);
  SetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_BANK_GRAPHIC,buffer);
  sprintf(buffer,"%04X",graphichgr_address);
  SetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_ADDRESS_GRAPHIC,buffer);
  /* Text */
  sprintf(buffer,"%02X",textpage40_bank);
  SetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_BANK_TEXT,buffer);
  sprintf(buffer,"%04X",textpage40_address);
  SetDlgItemText(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_EDIT_ADDRESS_TEXT,buffer);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsGraphicHGRText40DataModified(graphichgr_bank,graphichgr_address,textpage40_bank,textpage40_address,window_graphichgrtext40);
  if(is_modified == 0 && force == 0)
    return;

  /** Cration de la Zone Graphique + Text  partir des nouvelles donnes **/
  BuildGraphicHGRText40FromData(window_graphichgrtext40);

  /* Rafraichit la fentre */
  InvalidateRect(window_graphichgrtext40->hwnd_graphichgrtext40,NULL,TRUE);
  UpdateWindow(window_graphichgrtext40->hwnd_graphichgrtext40);
}


/************************************************************************************************************************/
/*  IsGraphicHGRText40DataModified() :  Les donnes GraphicHGRText40 ont t'elles t modifies depuis la dernire fois. */
/************************************************************************************************************************/
static int IsGraphicHGRText40DataModified(int graphichgr_bank, int graphichgr_address, int textpage40_bank, int textpage40_address, struct window_graphichgrtext40 *window_graphichgrtext40)
{
  int x, y, graphichgr_data_address, textpage40_data_address, has_flash_character;
  unsigned char graphichgr_raw_data[8192];
  unsigned char textpage40_raw_data[1024];
  unsigned char graphichgr_data[40*192];
  unsigned char textpage40_data[40*24];
  int tab_hgr_address[192] =     /* Adresse du dbut des 192 lignes de la Page HGR */
    {
      0x0000+0x0000, 0x0000+0x0400, 0x0000+0x0800, 0x0000+0x0C00, 0x0000+0x1000, 0x0000+0x1400, 0x0000+0x1800, 0x0000+0x1C00, 
      0x0080+0x0000, 0x0080+0x0400, 0x0080+0x0800, 0x0080+0x0C00, 0x0080+0x1000, 0x0080+0x1400, 0x0080+0x1800, 0x0080+0x1C00, 
      0x0100+0x0000, 0x0100+0x0400, 0x0100+0x0800, 0x0100+0x0C00, 0x0100+0x1000, 0x0100+0x1400, 0x0100+0x1800, 0x0100+0x1C00, 
      0x0180+0x0000, 0x0180+0x0400, 0x0180+0x0800, 0x0180+0x0C00, 0x0180+0x1000, 0x0180+0x1400, 0x0180+0x1800, 0x0180+0x1C00, 
      0x0200+0x0000, 0x0200+0x0400, 0x0200+0x0800, 0x0200+0x0C00, 0x0200+0x1000, 0x0200+0x1400, 0x0200+0x1800, 0x0200+0x1C00, 
      0x0280+0x0000, 0x0280+0x0400, 0x0280+0x0800, 0x0280+0x0C00, 0x0280+0x1000, 0x0280+0x1400, 0x0280+0x1800, 0x0280+0x1C00, 
      0x0300+0x0000, 0x0300+0x0400, 0x0300+0x0800, 0x0300+0x0C00, 0x0300+0x1000, 0x0300+0x1400, 0x0300+0x1800, 0x0300+0x1C00, 
      0x0380+0x0000, 0x0380+0x0400, 0x0380+0x0800, 0x0380+0x0C00, 0x0380+0x1000, 0x0380+0x1400, 0x0380+0x1800, 0x0380+0x1C00, 
      0x0028+0x0000, 0x0028+0x0400, 0x0028+0x0800, 0x0028+0x0C00, 0x0028+0x1000, 0x0028+0x1400, 0x0028+0x1800, 0x0028+0x1C00, 
      0x00A8+0x0000, 0x00A8+0x0400, 0x00A8+0x0800, 0x00A8+0x0C00, 0x00A8+0x1000, 0x00A8+0x1400, 0x00A8+0x1800, 0x00A8+0x1C00, 
      0x0128+0x0000, 0x0128+0x0400, 0x0128+0x0800, 0x0128+0x0C00, 0x0128+0x1000, 0x0128+0x1400, 0x0128+0x1800, 0x0128+0x1C00, 
      0x01A8+0x0000, 0x01A8+0x0400, 0x01A8+0x0800, 0x01A8+0x0C00, 0x01A8+0x1000, 0x01A8+0x1400, 0x01A8+0x1800, 0x01A8+0x1C00, 
      0x0228+0x0000, 0x0228+0x0400, 0x0228+0x0800, 0x0228+0x0C00, 0x0228+0x1000, 0x0228+0x1400, 0x0228+0x1800, 0x0228+0x1C00, 
      0x02A8+0x0000, 0x02A8+0x0400, 0x02A8+0x0800, 0x02A8+0x0C00, 0x02A8+0x1000, 0x02A8+0x1400, 0x02A8+0x1800, 0x02A8+0x1C00, 
      0x0328+0x0000, 0x0328+0x0400, 0x0328+0x0800, 0x0328+0x0C00, 0x0328+0x1000, 0x0328+0x1400, 0x0328+0x1800, 0x0328+0x1C00, 
      0x03A8+0x0000, 0x03A8+0x0400, 0x03A8+0x0800, 0x03A8+0x0C00, 0x03A8+0x1000, 0x03A8+0x1400, 0x03A8+0x1800, 0x03A8+0x1C00, 
      0x0050+0x0000, 0x0050+0x0400, 0x0050+0x0800, 0x0050+0x0C00, 0x0050+0x1000, 0x0050+0x1400, 0x0050+0x1800, 0x0050+0x1C00, 
      0x00D0+0x0000, 0x00D0+0x0400, 0x00D0+0x0800, 0x00D0+0x0C00, 0x00D0+0x1000, 0x00D0+0x1400, 0x00D0+0x1800, 0x00D0+0x1C00, 
      0x0150+0x0000, 0x0150+0x0400, 0x0150+0x0800, 0x0150+0x0C00, 0x0150+0x1000, 0x0150+0x1400, 0x0150+0x1800, 0x0150+0x1C00, 
      0x01D0+0x0000, 0x01D0+0x0400, 0x01D0+0x0800, 0x01D0+0x0C00, 0x01D0+0x1000, 0x01D0+0x1400, 0x01D0+0x1800, 0x01D0+0x1C00, 
      0x0250+0x0000, 0x0250+0x0400, 0x0250+0x0800, 0x0250+0x0C00, 0x0250+0x1000, 0x0250+0x1400, 0x0250+0x1800, 0x0250+0x1C00, 
      0x02D0+0x0000, 0x02D0+0x0400, 0x02D0+0x0800, 0x02D0+0x0C00, 0x02D0+0x1000, 0x02D0+0x1400, 0x02D0+0x1800, 0x02D0+0x1C00, 
      0x0350+0x0000, 0x0350+0x0400, 0x0350+0x0800, 0x0350+0x0C00, 0x0350+0x1000, 0x0350+0x1400, 0x0350+0x1800, 0x0350+0x1C00, 
      0x03D0+0x0000, 0x03D0+0x0400, 0x03D0+0x0800, 0x03D0+0x0C00, 0x03D0+0x1000, 0x03D0+0x1400, 0x03D0+0x1800, 0x03D0+0x1C00
    };
  int tab_text40_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /** Graphic : Rcupre les donnes brutes : 8192 bytes **/
  graphichgr_data_address = (graphichgr_bank << 16) | graphichgr_address;
  ReadMemoryBlock(graphichgr_data_address,8192,graphichgr_raw_data,current_mmu);

  /** Text : Rcupre les donnes brutes : 1024 chars **/
  textpage40_data_address = (textpage40_bank << 16) | textpage40_address;
  ReadMemoryBlock(textpage40_data_address,1024,textpage40_raw_data,current_mmu);

  /** Graphic : R-organise les donnes : Linarisation (passage du block de 8192 byte en 40*192) **/
  for(y=0; y<192; y++)
    for(x=0; x<40; x++)
      graphichgr_data[x+y*40] = graphichgr_raw_data[tab_hgr_address[y]+x];

  /** Text : R-organise les donnes : Linarisation (passage du block de 1024 char en 40*24) **/
  has_flash_character = 0;
  for(y=0; y<24; y++)
    for(x=0; x<40; x++)
      {
        textpage40_data[x+y*40] = textpage40_raw_data[tab_text40_address[y]+x];
        if(textpage40_data[x+y*40] >= 0x40 && textpage40_data[x+y*40] <= 0x7F)
          has_flash_character = 1;
      }

  /** Compare les donnes **/
  if(window_graphichgrtext40->graphichgr_data != NULL && window_graphichgrtext40->textpage40_data != NULL)
    if(graphichgr_bank == window_graphichgrtext40->graphichgr_bank &&
       graphichgr_address == window_graphichgrtext40->graphichgr_address &&
       memcmp(&graphichgr_data[0],&window_graphichgrtext40->graphichgr_data[0],40*192) == 0 && 
       textpage40_bank == window_graphichgrtext40->textpage40_bank &&
       textpage40_address == window_graphichgrtext40->textpage40_address &&
       memcmp(&textpage40_data[0],&window_graphichgrtext40->textpage40_data[0],40*24) == 0 && 
       (window_graphichgrtext40->flash == 1 && has_flash_character == 0))
      return(0);

  /** Les donnes sont diffrentes => MAJ **/
  /* Graphic */
  window_graphichgrtext40->graphichgr_bank = graphichgr_bank;
  window_graphichgrtext40->graphichgr_address = graphichgr_address;
  memcpy(window_graphichgrtext40->graphichgr_data,graphichgr_data,40*192);
  /* Text */
  window_graphichgrtext40->textpage40_bank = textpage40_bank;
  window_graphichgrtext40->textpage40_address = textpage40_address;
  memcpy(window_graphichgrtext40->textpage40_data,textpage40_data,40*24);
  window_graphichgrtext40->has_flash_character = has_flash_character;

  /* Donnes diffrentes ou FLASH */
  return(1);
}


/***********************************************************************************/
/*  ResizeGraphicHGRText40Window() : Resize GraphicHGRText40 window by given size. */
/***********************************************************************************/
static void ResizeGraphicHGRText40Window(struct window_graphichgrtext40 *window_graphichgrtext40)
{
  HWND hwnd_group;
  RECT window_rect, client_rect, group_rect;
  int window_x, window_y;
  
  /** On va ajuster la fentre par rapport au Zoom demand **/  
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_graphichgrtext40->hwnd_graphichgrtext40,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_graphichgrtext40->hwnd_graphichgrtext40,&client_rect);
  window_graphichgrtext40->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select */
  hwnd_group = GetDlgItem(window_graphichgrtext40->hwnd_graphichgrtext40,IDC_GROUP_GRAPHICHGRTEXT40);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_GRAPHICHGRTEXT40 */
  GetWindowRect(hwnd_group,&group_rect);
  window_graphichgrtext40->group_data_x = (group_rect.left - window_rect.left) - 3;
  window_graphichgrtext40->group_data_y = ((group_rect.top - window_rect.top) - window_graphichgrtext40->title_height) + 3;

  /* Position de la Zone BipMap dans la fenetre */
  window_graphichgrtext40->bitmap_graphichgrtext40_x = window_graphichgrtext40->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_graphichgrtext40->bitmap_graphichgrtext40_y = window_graphichgrtext40->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_graphichgrtext40->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_graphichgrtext40->bitmap_graphichgrtext40_width*window_graphichgrtext40->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_graphichgrtext40->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphichgrtext40->bitmap_graphichgrtext40_height*window_graphichgrtext40->zoom_level + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_graphichgrtext40->group_data_x,window_graphichgrtext40->group_data_y,window_graphichgrtext40->group_data_width,window_graphichgrtext40->group_data_height,TRUE);

  /* Largeur de la fentre */
  window_graphichgrtext40->window_width = window_graphichgrtext40->group_data_x + window_graphichgrtext40->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* On doit garantir une largeur minimale de la fentre */
  if(window_graphichgrtext40->window_width < GRAPHICHGRTEXT40_DIALOGBOX_WIDTH)
    window_graphichgrtext40->window_width = GRAPHICHGRTEXT40_DIALOGBOX_WIDTH;

  /* Hauteur de la fentre */
  window_graphichgrtext40->window_height = window_graphichgrtext40->title_height + window_graphichgrtext40->group_data_y + window_graphichgrtext40->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Resize window */
  MoveWindow(window_graphichgrtext40->hwnd_graphichgrtext40,window_x,window_y,window_graphichgrtext40->window_width,window_graphichgrtext40->window_height,TRUE);
}


/******************************************************************************************************************/
/*  BuildGraphicHGRText40FromData() :  Construire la page Gaphique + Text  afficher  partir des donnes brutes. */
/******************************************************************************************************************/
static void BuildGraphicHGRText40FromData(struct window_graphichgrtext40 *window_graphichgrtext40)
{
  int one_byte, flash_offset_low, flash_offset_high, hundred_sec;
  int x, y, pixmap_char_x, pixmap_char_y, pixel_x, pixel_y, offset_pixmap_char, offset_pixmap_screen, offset;
  int bit_previous, bit_0, bit_1, bit_2, bit_3, bit_4, bit_5, bit_6, bit_next, bit_color;
  char buffer_title[256];
  DWORD color_text, color_background, color_pixmap, color_screen;
  DWORD tab_rgb_color_graphic[16] = 
    {
      /* 8 couleurs pour le mode HGR */
      MAKECOLOR(0,0,0), MAKECOLOR(0,221,0),   MAKECOLOR(221,0,221), MAKECOLOR(255,255,255),
      MAKECOLOR(0,0,0), MAKECOLOR(255,102,0), MAKECOLOR(34,34,255), MAKECOLOR(255,255,255)
    };
  DWORD tab_rgb_color_text[16] = 
    {
      /* 16 couleurs de la Bordure / Fond Ecran / Text */
      MAKECOLOR(0,0,0), MAKECOLOR(221,0,51), MAKECOLOR(0,0,153), MAKECOLOR(221,34,221),
      MAKECOLOR(0,119,34), MAKECOLOR(85,85,85), MAKECOLOR(34,34,255), MAKECOLOR(102,170,255),
      MAKECOLOR(136,85,0), MAKECOLOR(255,102,0), MAKECOLOR(170,170,170), MAKECOLOR(255,153,136),
      MAKECOLOR(17,221,0), MAKECOLOR(255,255,0), MAKECOLOR(68,255,153), MAKECOLOR(255,255,255)
    };
  struct _timeb timebuffer;
  struct picture_true *picture_char;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rien  faire */
  if(window_graphichgrtext40->bitmap_data_graphichgrtext40 == NULL)
    return;

  /* Dtermine le changement pour les FLASH */
  _ftime(&timebuffer);
  hundred_sec = timebuffer.millitm / 10;
  if(hundred_sec < 50)
    {
      /* INVERSE */
      flash_offset_low = (-1)*(0x40);
      flash_offset_high = (-1)*(0x40);
    }
  else
    {
      /* NORMAL */
      flash_offset_low = (0xA0 - 0x40 + 0x20);
      flash_offset_high = (0xA0 - 0x40 - 0x20);
    }

  /* Adresse de la picture */
  if(window_graphichgrtext40->display_language == LANGUAGE_IIGS_USA)
    picture_char = param->picture_char_us;
  else if(window_graphichgrtext40->display_language == LANGUAGE_IIGS_UK)
    picture_char = param->picture_char_uk;
  else if(window_graphichgrtext40->display_language == LANGUAGE_IIGS_FRENCH)
    picture_char = param->picture_char_fr;
  else if(window_graphichgrtext40->display_language == LANGUAGE_IIGS_DANISH)
    picture_char = param->picture_char_dk;
  else if(window_graphichgrtext40->display_language == LANGUAGE_IIGS_SPANISH)
    picture_char = param->picture_char_se;
  else if(window_graphichgrtext40->display_language == LANGUAGE_IIGS_ITALIAN)
    picture_char = param->picture_char_it;
  else if(window_graphichgrtext40->display_language == LANGUAGE_IIGS_GERMAN)
    picture_char = param->picture_char_de;
  else if(window_graphichgrtext40->display_language == LANGUAGE_IIGS_SWEDISH)
    picture_char = param->picture_char_se;
  else if(window_graphichgrtext40->display_language == LANGUAGE_IIGS_MOUSETEXT)
    picture_char = param->picture_char_mousetext;
    
  /* Couleur du Texte */
  color_text = tab_rgb_color_text[window_graphichgrtext40->color_text];

  /* Couleur du Fond */
  color_background = tab_rgb_color_text[window_graphichgrtext40->color_background];

  /** Graphic : Cration de la bitmap 280*192 correspondant au 40*192 du HGR **/
  bit_previous = 0;
  offset = 0;
  for(y=0; y<160; y++)    /* 192 - 4*8 = 160 */
    {
      for(x=0; x<40; x++)
        {
          /* Bit Suivant */
          if(x == 39 && y == 191)
            bit_next = 0;                                                             /* Fin de l'image */
          else if(x == 39)
            bit_next = (window_graphichgrtext40->graphichgr_data[(y+1)*40+0] & 0x01);       /* Ligne suivante, Byte #0 */
          else
            bit_next = (window_graphichgrtext40->graphichgr_data[y*40+(x+1)] & 0x01);       /* Byte suivant sur la mme ligne */

          /* 1 Byte = 7 pixels */
          one_byte = window_graphichgrtext40->graphichgr_data[y*40+x];
          bit_color = (one_byte & 0x80) >> 7;                                         /* 0=Vert+Violet, 1=Bleu+Orange */
          bit_6 = (one_byte & 0x40) >> 6;
          bit_5 = (one_byte & 0x20) >> 5;
          bit_4 = (one_byte & 0x10) >> 4;
          bit_3 = (one_byte & 0x08) >> 3;
          bit_2 = (one_byte & 0x04) >> 2;
          bit_1 = (one_byte & 0x02) >> 1;
          bit_0 = (one_byte & 0x01);
                  
          /* Codage des points */
          window_graphichgrtext40->bitmap_data_graphichgrtext40[offset++] = tab_rgb_color_graphic[GetHGRColor(bit_previous,bit_0,bit_1,bit_color,x*7+0)];
          window_graphichgrtext40->bitmap_data_graphichgrtext40[offset++] = tab_rgb_color_graphic[GetHGRColor(bit_0,bit_1,bit_2,bit_color,x*7+1)];
          window_graphichgrtext40->bitmap_data_graphichgrtext40[offset++] = tab_rgb_color_graphic[GetHGRColor(bit_1,bit_2,bit_3,bit_color,x*7+2)];
          window_graphichgrtext40->bitmap_data_graphichgrtext40[offset++] = tab_rgb_color_graphic[GetHGRColor(bit_2,bit_3,bit_4,bit_color,x*7+3)];
          window_graphichgrtext40->bitmap_data_graphichgrtext40[offset++] = tab_rgb_color_graphic[GetHGRColor(bit_3,bit_4,bit_5,bit_color,x*7+4)];
          window_graphichgrtext40->bitmap_data_graphichgrtext40[offset++] = tab_rgb_color_graphic[GetHGRColor(bit_4,bit_5,bit_6,bit_color,x*7+5)];
          window_graphichgrtext40->bitmap_data_graphichgrtext40[offset++] = tab_rgb_color_graphic[GetHGRColor(bit_5,bit_6,bit_next,bit_color,x*7+6)];
                  
          /* Conserve le Bit prcdant */
          bit_previous = bit_6;
        }
    }

  /** Text : Cration du bas de la bitmap 280*192 correspondant au 40*24 **/
  for(y=20; y<24; y++)
    {
      for(x=0; x<40; x++)
        {
          /* Caractre */
          one_byte = (int) window_graphichgrtext40->textpage40_data[x + y*40];

          /* Si le caractre FLASH, on switch entre INVERSE et NORMAL (Pas de FLASH en MouseText) */
          if(window_graphichgrtext40->display_language != LANGUAGE_IIGS_MOUSETEXT)
            {
              if(one_byte >= 0x40 && one_byte <= 0x5F)
                one_byte += flash_offset_low;
              else if(one_byte >= 0x60 && one_byte <= 0x7F)
                one_byte += flash_offset_high;
            }

          /* Offset dans la Pixmap */
          pixmap_char_y = one_byte / 40;
          pixmap_char_x = one_byte - (pixmap_char_y*40);

          /* Copie ce bloc de 7*8 */
          offset_pixmap_char  = pixmap_char_x*7 + pixmap_char_y*(40*7)*8;
          offset_pixmap_screen = x*7 + y*(40*7)*8;
          for(pixel_y=0; pixel_y<8; pixel_y++)
            for(pixel_x=0; pixel_x<7; pixel_x++)
              {
                /* Pixel dans la Char Tab */
                color_pixmap = picture_char->data_dwd[offset_pixmap_char + pixel_x + pixel_y*40*7];

                /* Couleur du Pixel final */
                color_screen = (color_pixmap == 0xFFFFFF) ? color_text : color_background;

                /* Place le point dans la Pixmap */
                window_graphichgrtext40->bitmap_data_graphichgrtext40[offset_pixmap_screen + pixel_x + pixel_y*40*7] = color_screen;
              }
        }
    }

  /* Titre de la fentre */
  sprintf(buffer_title,"Graphic HGR Text 40 Column : %02X/%04X   %02X/%04X",window_graphichgrtext40->graphichgr_bank,window_graphichgrtext40->graphichgr_address,
                                                                            window_graphichgrtext40->textpage40_bank,window_graphichgrtext40->textpage40_address);
  SetWindowText(window_graphichgrtext40->hwnd_graphichgrtext40,buffer_title);
}


/******************************************************************************/
/*  DeleteGraphicHGRText40ChildWindows() :  Destruction des fentres enfants. */
/******************************************************************************/
static void DeleteGraphicHGRText40ChildWindows(struct window_graphichgrtext40 *window_graphichgrtext40)
{
  DeleteObject(window_graphichgrtext40->hBitmap_graphichgrtext40);
  DeleteObject(window_graphichgrtext40->hwnd_combo_color_text);
  DeleteObject(window_graphichgrtext40->hwnd_combo_color_background);
  DeleteObject(window_graphichgrtext40->hwnd_combo_display_language);
  DeleteObject(window_graphichgrtext40->hFont);
  ReleaseDC(window_graphichgrtext40->hwnd_graphichgrtext40,window_graphichgrtext40->hDC);
  DeleteDC(window_graphichgrtext40->hCompatibleDC);
}


/********************************************************************************/
/*  mem_free_window_graphichgrtext40() :  Libration mmoire pour la structure. */
/********************************************************************************/
void mem_free_window_graphichgrtext40(struct window_graphichgrtext40 *window_graphichgrtext40)
{
  if(window_graphichgrtext40)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphichgrtext40->current_notification,NULL);
      window_graphichgrtext40->current_notification = NULL;
      
      free(window_graphichgrtext40);
    }
}

/**********************************************************************/
