/************************************************************************/
/*                                                                      */
/*  Window_GraphicSHR.h :  Header de gestion de la fentre GraphicSHR.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI   *  Brutal Deluxe  *   Date :  Aout 2018  */
/************************************************************************/

/* Un message destin  la fentre Graphic SHR pour demander la maj de l'affichage */
#define HM_GRAPHICSHR_REFRESHWINDOW               (WM_USER + 24)
#define IDC_COMBOBOX_SCB_PALETTE                  1628
#define IDC_COMBOBOX_BORDER_COLOR_BACKGROUND      1629

/* Couleur de fond de la fentre */
#define FILLMODE_BACKGROUND_COLOR  MAKECOLOR(192,255,192)    /* Vert Clair */

#define GRAPHICSHR_REFRESH_AUTOMATIC    0
#define GRAPHICSHR_REFRESH_MANUAL       1

#define GRAPHICSHR_ZOOM_X1              1
#define GRAPHICSHR_ZOOM_X2              2
#define GRAPHICSHR_ZOOM_X3              3
#define GRAPHICSHR_ZOOM_X4              4

#define GRAPHICSHR_SCBPALETTE_CURRENT   0     /* Choix de la palette */
#define GRAPHICSHR_SCBPALETTE_E12000    1
#define GRAPHICSHR_SCBPALETTE_012000    2
#define GRAPHICSHR_SCBPALETTE_FIXED     3

#define GRAPHICSHR_COLOR_DYNAMIC        0
#define GRAPHICSHR_COLOR_BLACK          1
#define GRAPHICSHR_COLOR_DEEP_RED      2
#define GRAPHICSHR_COLOR_DARK_BLUE      3
#define GRAPHICSHR_COLOR_PURPLE         4
#define GRAPHICSHR_COLOR_DARK_GREEN     5
#define GRAPHICSHR_COLOR_DARK_GRAY      6
#define GRAPHICSHR_COLOR_MEDIUM_BLUE    7
#define GRAPHICSHR_COLOR_LIGHT_BLUE     8
#define GRAPHICSHR_COLOR_BROWN          9
#define GRAPHICSHR_COLOR_ORANGE        10
#define GRAPHICSHR_COLOR_LIGHT_GRAY    11
#define GRAPHICSHR_COLOR_PINK          12
#define GRAPHICSHR_COLOR_LIGHT_GREEN   13
#define GRAPHICSHR_COLOR_YELLOW        14
#define GRAPHICSHR_COLOR_AQUAMARINE    15
#define GRAPHICSHR_COLOR_WHITE         16

#define GRAPHICSHR_EDIT_FONT_HEIGHT    14    /* Taille de la Fonte en Pixel */

#define GRAPHICSHR_DIALOGBOX_WIDTH     450   /* Largeur minimale en Pixel de la fentre */

#define GRAPHICSHR_PIXEL_OFFSET        0
#define GRAPHICSHR_SCB_OFFSET          32000
#define GRAPHICSHR_PALETTE_OFFSET      32256

#define GRAPHICSHR_PICTURE_FORMAT_320_200         0
#define GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER  1
#define GRAPHICSHR_PICTURE_FORMAT_640_200         2
#define GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER  3
#define GRAPHICSHR_PICTURE_FORMAT_640_400         4
#define GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER  5

#define GRAPHICSHR_BORDER_SIZE_TOP     19    /* Picture Border Size */
#define GRAPHICSHR_BORDER_SIZE_BOTTOM  21
#define GRAPHICSHR_BORDER_SIZE_LEFT    24    /* x2 si 640 */
#define GRAPHICSHR_BORDER_SIZE_RIGHT   28

struct window_graphicshr
{
  HWND hwnd_graphicshr;
  HWND hwnd_combo_scb_palette;        /* ComboBox pour le SCB Palette */
  HWND hwnd_combo_color_background;   /* ComboBox pour le Color Background */
  HFONT hFont;

  DWORD thread_id;              /* Le Thread ID va permettre de diffrencier les diffrentes fentres */

  /** Graphical element attributes **/
  HDC hDC;
  HDC hCompatibleDC;
  HBITMAP hBitmap_palette;                        /* Bitmap prsentant la palette */
  DWORD *bitmap_data_palette;
  HBITMAP hBitmap_graphicshr_320_200;             /* Bitmap affchant la zone de slecion 320*200 */
  DWORD *bitmap_data_graphicshr_320_200;
  HBITMAP hBitmap_graphicshr_320_200_BORDER;      /* Bitmap affchant la zone de slecion 320*200 + Border */
  DWORD *bitmap_data_graphicshr_320_200_BORDER;
  HBITMAP hBitmap_graphicshr_640_200;             /* Bitmap affchant la zone de slecion 640*200 */
  DWORD *bitmap_data_graphicshr_640_200;
  HBITMAP hBitmap_graphicshr_640_200_BORDER;      /* Bitmap affchant la zone de slecion 640*200 + Border */
  DWORD *bitmap_data_graphicshr_640_200_BORDER;
  HBITMAP hBitmap_graphicshr_640_400;             /* Bitmap affchant la zone de slecion 640*400 */
  DWORD *bitmap_data_graphicshr_640_400;
  HBITMAP hBitmap_graphicshr_640_400_BORDER;      /* Bitmap affchant la zone de slecion 640*400 + Border */
  DWORD *bitmap_data_graphicshr_640_400_BORDER;

  /* Surchage des Control Edit pour rcuprer les vnements Souris + Entre */
  WNDPROC oldEditProcBank;
  WNDPROC oldEditProcAddress;
  
  /* Mode de rafraichissement de la fentre */
  int refresh_mode;        /* GRAPHICSHR_REFRESH_XXX */
    
  /* Format de l'image */
  int picture_format;      /* GRAPHICSHR_PICTURE_FORMAT_XXX */

  /* Zoom Level */
  int zoom_level;

  /* Type de SCB Palette */
  int display_scb;
  int scb_palette;

  /* Affiche le Spot */
  int display_video_spot;
  BYTE C02E;
  BYTE C02F;

  /* Fill Mode */
  int fill_mode;

  /* Display Border */
  int display_border;
  int border_color;      /* 0-N */
  DWORD border_color_rgb;

  /* Double Height */
  int double_height;     /* Uniquement en 640 */

  /* Position / Taille du Control Group Box */
  int group_data_x;
  int group_data_y;
  int group_data_width;
  int group_data_height;

  /* Dimension + Position de la Bitmap Palette */
  int bitmap_palette_x;
  int bitmap_palette_y;
  int bitmap_palette_width;    /* COLOR_BORDER + 16*(COLOR_SIZE + COLOR_BORDER) */
  int bitmap_palette_height;   /* COLOR_BORDER + COLOR_SIZE + COLOR_BORDER */

  /* Position / Taille de la Bitmap SHR Data */
  int bitmap_graphicshr_x;
  int bitmap_graphicshr_y;
  int bitmap_graphicshr_width_320;          /* 320 */
  int bitmap_graphicshr_width_320_BORDER;   /* 320 + BORDER */
  int bitmap_graphicshr_width_640;          /* 640 */
  int bitmap_graphicshr_width_640_BORDER;   /* 640 + BORDER */
  int bitmap_graphicshr_height_200;         /* 200 */
  int bitmap_graphicshr_height_200_BORDER;  /* 200 + BORDER */
  int bitmap_graphicshr_height_400;         /* 400 */
  int bitmap_graphicshr_height_400_BORDER;  /* 400 + BORDER */

  /***************/
  /**  Palette  **/
  int nb_palette;
  struct palette *first_palette;
  struct palette *last_palette;

  /** Palette en cours **/
  int num_palette;          /* 1 - nb_palette */
  char palette_name[256];   /* Nom de la palette */

  /* RGB */
  int nb_color;
  int palette_red[256];
  int palette_green[256];
  int palette_blue[256];

  /* Coodonnes (X0,Y0)(X1,Y1) des couleurs de la palette dans la Pixmap */
  int palette_x0[256];
  int palette_y0[256];
  int palette_x1[256];
  int palette_y1[256];

  /** Donnes actuellement affiches **/
  int graphicshr_bank;
  int graphicshr_address;
  unsigned char graphicshr_data[32768];     /* Pixels + SBC + Palette */

  /* Dimension de la fentre = Title + Zone Texte */
  int title_height;
  int window_width;
  int window_height;

  /* Buffer */
  char buffer_log[BUFFER_SIZE];

  /* Notification */
  struct notification *current_notification;

  /* Liste chaine des Graphic SHR */
  struct window_graphicshr *next;
};

struct window_graphicshr *BuildGraphicSHRWindow(HINSTANCE,int);
void mem_free_window_graphicshr(struct window_graphicshr *);

/***************************************************************/
