/*******************************************************************************/
/*                                                                             */
/*  Window_IPC.h :  Header de gestion de la fentre IPC.                       */
/*                                                                             */
/*******************************************************************************/
/*  Auteur : Olivier ZARDINI   *  Brutal Deluxe Software  *   Date : May 2023  */
/*******************************************************************************/

#define CYRENE_IPC_WINDOW_CLASS   "CyreneIPC"
#define CYRENE_IPC_WINDOW_NAME    "Cyrene_%d"

struct window_ipc
{
  /* Handle de la fentre IPC */
  HWND hwnd_cyrene_ipc;
  
  /* Device Context */
  HDC hDC;
  HDC hCompatibleDC;

  /* Titre height en pixels */
  int title_height;
  int window_width;
  int window_height;

  /* Titre de la fenetre IPC de Cyrene */
  char cyrene_ipc_window_name[256];

  /** CYRENE IPC Shared Memory **/
  char shared_memory_path[256];           /* Local\cyrene_ipc_window_name */
  HANDLE hSharedMemory;
  unsigned char *shared_memory_ptr;  
};

/* Taille Minimum de la Fentre IPC */
#define IPC_WINDOW_MIN_WIDTH                   240
#define IPC_WINDOW_MIN_HEIGHT                  120

/** IPC Shared Memory **/
#define IPC_SHARED_MEMORY_HEADER_SIZE                (1*1024)       /* 1 KB */
#define IPC_SHARED_MEMORY_BREAKPOINT_SIZE            (4*1024)       /* 4 KB */
#define IPC_SHARED_MEMORY_OUTPUT_SIZE                (1024*1024)    /* 1 MB */
#define IPC_SHARED_MEMORY_TOTAL_SIZE                 (IPC_SHARED_MEMORY_HEADER_SIZE+IPC_SHARED_MEMORY_BREAKPOINT_SIZE+IPC_SHARED_MEMORY_OUTPUT_SIZE)
/* Block Offset */
#define IPC_SHARED_MEMORY_OFFSET_DATA_HEADER         0
#define IPC_SHARED_MEMORY_OFFSET_DATA_BREAKPOINT     IPC_SHARED_MEMORY_HEADER_SIZE
#define IPC_SHARED_MEMORY_OFFSET_DATA_OUTPUT         (IPC_SHARED_MEMORY_HEADER_SIZE+IPC_SHARED_MEMORY_BREAKPOINT_SIZE)
/* Header */
#define IPC_SHARED_MEMORY_OFFSET_GOID                 0   /* QWORD */
#define IPC_SHARED_MEMORY_OFFSET_STATUS_BREAKPOINT    8   /* DWORD 0:Cyrene can Write/1:Kegs can read */
#define IPC_SHARED_MEMORY_OFFSET_NB_BREAKPOINT       12   /* DWORD 0-N */
#define IPC_SHARED_MEMORY_OFFSET_OUTPUT_READ         16   /* DWORD */
#define IPC_SHARED_MEMORY_OFFSET_OUTPUT_WRITE        20   /* DWORD */
#define IPC_SHARED_MEMORY_OUTPUT_WRITE_MISSED        24   /* DWORD */
#define IPC_SHARED_MEMORY_OFFSET_OUTPUT_SYSCALL      28   /* BYTE : 0/1 Trace System Calls */

int StartWindowIPCThread(void);
void UpdateBreapointTableInSharedMemory(void);
void UpdateTraceSystemCallInSharedMemory(void);
void mem_free_window_ipc(struct window_ipc *);

/*********************************************************************/
