/*********************************************************************************/
/*                                                                               */
/*  Window_Internals.h :  Header de gestion de la fentre Apple IIgs Internals.  */
/*                                                                               */
/*********************************************************************************/
/*  Auteur : Olivier ZARDINI     *     Brutal Deluxe     *     Date : Aot 2023  */
/*********************************************************************************/

/* Un message destin  la fentre Apple IIgs Internams pour demander la maj de l'affichage */
#define HM_INTERNALS_REFRESHWINDOW         (WM_USER + 22)
#define IDC_GLANCE_EDIT_DATA               1623
#define IDC_INTERNALS_TABCONTROL           1624
#define IDC_INTERNALS_LISTVIEW_USED        1625
#define IDC_INTERNALS_LISTVIEW_PURGED      1626
#define IDC_INTERNALS_LISTVIEW_FREE        1627
#define IDC_INTERNALS_LISTVIEW_CDA         1628
#define IDC_INTERNALS_LISTVIEW_NDA         1629
#define IDC_INTERNALS_LISTVIEW_QUICKDRAW   1630
#define IDC_INTERNALS_LISTVIEW_INTERRUPT   1631
#define IDC_INTERNALS_LISTVIEW_FILE        1632
#define IDC_INTERNALS_LISTVIEW_INIT        1633
#define IDC_INTERNALS_LISTVIEW_DRIVER      1634
#define IDC_INTERNALS_LISTVIEW_FST         1635

#define INTERNALS_REFRESH_AUTOMATIC     0
#define INTERNALS_REFRESH_MANUAL        1

/* Handle */
#define TYPE_HANDLE_USED                1
#define TYPE_HANDLE_PURGED              2
#define TYPE_HANDLE_FREE                3

/* Onglet */
#define INTERNALS_TAB_COUNT            12    /* Total Number of Tabs */
#define INTERNALS_TAB_GLANCE_INDEX     12

#define INTERNALS_TAB_INTERRUPT         1
#define INTERNALS_TAB_QUICKDRAW         2
#define INTERNALS_TAB_FILE              3
#define INTERNALS_TAB_CDA               4
#define INTERNALS_TAB_NDA               5
#define INTERNALS_TAB_INIT              6
#define INTERNALS_TAB_DRIVER            7
#define INTERNALS_TAB_FST               8
#define INTERNALS_TAB_USED              9
#define INTERNALS_TAB_PURGED           10
#define INTERNALS_TAB_FREE             11
#define INTERNALS_TAB_GLANCE           12

#define INTERNALS_EDIT_FONT_HEIGHT     14    /* Taille de la Fonte en Pixel */

#define INTERNALS_LISTVIEW_LINE_COUNT  38    /* Nombre de lignes dans les ListView */

struct window_internals
{
  HWND hwnd_internals;
  HWND hwnd_tabcontrol;         /* Tableau  Onglets */
  HWND hwnd_edit_data;          /* Control Text Multi Ligne */

  /* Polices de caratres de la fentre */
  HFONT hFont;
  HFONT hFont_courrier;

  /* Brush avec la couleur de Fond */
  HBRUSH hBrush;

  /* Surchage des Control Edit pour rcuprer les vnements Souris */
  WNDPROC oldEditProcInternalsData;
  
  /* Mode de rafraichissement de la fentre */
  int refresh_mode;        /* INTERNALS_REFRESH_XXX */

  /* Numro de l'onglet affich (1-N) */
  int tab_index;

  /*******************/
  /**  [INTERRUPT]  **/
  /*******************/
  struct list_view listview_interrupt;

  /*******************/
  /**  [QUICKDRAW]  **/
  /*******************/
  struct list_view listview_quickdraw;

  /*************/
  /**  [FILE]  **/
  /*************/
  struct list_view listview_file;

  /*************/
  /**  [CDA]  **/
  /*************/
  struct list_view listview_cda;

  /*************/
  /**  [NDA]  **/
  /*************/
  struct list_view listview_nda;

  /***************/
  /**  [INIT]  **/
  /***************/
  struct list_view listview_init;

  /****************/
  /**  [DRIVER]  **/
  /****************/
  struct list_view listview_driver;

  /*************/
  /**  [FST]  **/
  /*************/
  struct list_view listview_fst;

  /**************/
  /**  [USED]  **/
  /**************/
  struct list_view listview_used;

  /****************/
  /**  [PURGED]  **/
  /****************/
  struct list_view listview_purged;

  /**************/
  /**  [FREE]  **/
  /**************/
  struct list_view listview_free;

  /****************/
  /**  [GLANCE]  **/
  /****************/
  struct gs_handle *current_handle;
  unsigned char *handle_data;

  /** Control Edit : Zone Texte o sont affiches les Data **/   
  int text_data_column;            /* Nombre de colonne Data = 16 */
  int text_data_row;               /* Nombre de ligne Data = 16 */
  int text_data_width;             /* "00/0000___00_00_00_00_00_00_00_00.00_00_00_00_00_00_00_00__ASCII\r\n" */
  int text_data_size;              /* text_data_width*column */
  char text_data_hexadecimal[4096]; /* Zone Texte column*line */ 

  /** Donnes actuellement affiches **/
  int data_offset;                 /* 0 - current_handle->size */
  int nb_column;                   /* 16 */
  int nb_row;                      /* 16 */
  unsigned char memoryarea_data[256];

  /* Dimension de la fentre = Title + Zone Texte */
  int title_height;
  int window_width;
  int window_height;
  
  /* Buffer */
  char buffer_log[BUFFER_SIZE];

  /* Notification */
  struct notification *current_notification;
};

struct window_internals *BuildInternalsWindow(HINSTANCE,int);
void mem_free_window_internals(struct window_internals *);

/********************************************************/
