/************************************************************************/
/*                                                                      */
/*  Window_LoadFile.c :  Module de gestion de la fentre LoadFile.      */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI   *   Brutal Deluxe   *   Date : Dec 2018  */
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Dc_Request.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_LoadFile.h"

static DWORD BuildLoadFileWindowThread(void *);
LRESULT CALLBACK ProcHandlerLoadFileWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateLoadFileChildWindows(HWND,struct window_loadfile *);
LRESULT CALLBACK subEditLoadFileProcControl(HWND,UINT,WPARAM,LPARAM);
static void UpdateLoadFileRamType(struct window_loadfile *,int);
static void LoadDataFileToMemory(struct window_loadfile *);
static void DeleteLoadFileChildWindows(struct window_loadfile *);


/******************************************************************************/
/*  BuildLoadFileWindow() :  Construction / Affichage de la fentre LoadFile. */
/******************************************************************************/
struct window_loadfile *BuildLoadFileWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_loadfile *window_loadfile;

  /** Allocation de la structure window_loadfile **/
  window_loadfile = (struct window_loadfile *) calloc(1,sizeof(struct window_loadfile));
  if(window_loadfile == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_LOADFILE,window_loadfile,NULL);

  /* Lance un thread pour grer la fentre LoadFile */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildLoadFileWindowThread,window_loadfile,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_loadfile);
}


/****************************************************************************************/
/*  BuildLoadFileWindowThread() :  Cration de la fentre LoadFile via un Thread ddi. */
/****************************************************************************************/
static DWORD BuildLoadFileWindowThread(void *window_loadfile_param)
{
  HWND hwndLoadFile;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_loadfile *window_loadfile;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_loadfile = (struct window_loadfile *) window_loadfile_param;
  window_loadfile->thread_id = GetCurrentThreadId();         /* Le ThreadID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Load File **/
  hwndLoadFile = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_LOADFILE),window_main->hwnd_main,(DLGPROC)ProcHandlerLoadFileWindow);
  if(hwndLoadFile == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndLoadFile,SW_SHOWDEFAULT);
  UpdateWindow(hwndLoadFile);

  /** Message Loop pour toutes les fentres de ce Thread (fentre LoadFile...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/*************************************************************************/
/*  ProcHandlerLoadFileWindow() :  Gestion des vnements de la fentre. */
/*************************************************************************/
LRESULT CALLBACK ProcHandlerLoadFileWindow(HWND hwndLoadFile, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  int file_size;
  struct window_loadfile *window_loadfile;
  struct parameter *param;
  char file_path[1024];
  char buffer[1024];
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_LOADFILE,&window_loadfile,NULL);
  if(window_loadfile == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateLoadFileChildWindows(hwndLoadFile,window_loadfile);
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la destination des donnes (Ram ou Doc Ram) **/
            case IDC_RADIO_RAM :
              if(HIWORD(wParam) == BN_CLICKED && window_loadfile->ram_type != LOADFILE_RAMTYPE_RAM)
                UpdateLoadFileRamType(window_loadfile,LOADFILE_RAMTYPE_RAM);
              break;

            case IDC_RADIO_DOCRAM :
              if(HIWORD(wParam) == BN_CLICKED && window_loadfile->ram_type != LOADFILE_RAMTYPE_DOCRAM)
                UpdateLoadFileRamType(window_loadfile,LOADFILE_RAMTYPE_DOCRAM);
              break;

            /** Bouton de rcupration du chemin du fichier **/
            case IDC_BUTTON_FILE_GET_PATH :
              GetOneFilePath(window_loadfile->hwnd_loadfile,"Select Data File","All Files (*.*)\0*.*\0\0",window_loadfile->file_path,file_path);
              if(strlen(file_path) > 0)
                {
                  /* Chemin du fichier */
                  strcpy(window_loadfile->file_path,file_path);
                  SetWindowText(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_FILE_PATH),window_loadfile->file_path);

                  /* Taille du fichier */
                  file_size = GetFileSizeInt(window_loadfile->file_path);
                  sprintf(buffer,"0x%X",file_size);
                  SetWindowText(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_FILE_DATA_LENGTH),buffer);

                  /* Offset */
                  SetWindowText(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_FILE_DATA_OFFSET),"0x0");
                }
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_LOADFILE :
              LoadDataFileToMemory(window_loadfile);
              break;
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre LoadFile */
        DestroyWindow(hwndLoadFile);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteLoadFileChildWindows(window_loadfile);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_LOADFILE,window_loadfile,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*******************************************************************************************************/
/*  CreateLoadFileChildWindows() : Crer les fentres enfants et remplir la structure window_loadfile. */
/*******************************************************************************************************/
static int CreateLoadFileChildWindows(HWND hwndLoadFile, struct window_loadfile *window_loadfile)
{
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_loadfile->hwnd_loadfile = hwndLoadFile;

  /** Surchage des Control Edit pour capter les RETURN lors des saisies **/
  window_loadfile->oldEditProcFilePath =   (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndLoadFile,IDC_EDIT_FILE_PATH),GWLP_WNDPROC,(LONG_PTR)subEditLoadFileProcControl);
  window_loadfile->oldEditProcFileOffset = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndLoadFile,IDC_EDIT_FILE_DATA_OFFSET),GWLP_WNDPROC,(LONG_PTR)subEditLoadFileProcControl);
  window_loadfile->oldEditProcFileLength = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndLoadFile,IDC_EDIT_FILE_DATA_LENGTH),GWLP_WNDPROC,(LONG_PTR)subEditLoadFileProcControl);
  window_loadfile->oldEditProcRamBank =    (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndLoadFile,IDC_EDIT_RAM_BANK),GWLP_WNDPROC,(LONG_PTR)subEditLoadFileProcControl);
  window_loadfile->oldEditProcRamAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndLoadFile,IDC_EDIT_RAM_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditLoadFileProcControl);

  /* Fichier de Data */
  window_loadfile->file_data_offset = 0;
  window_loadfile->file_data_length = 0;           /* 0 = Tout le fichier */
  SetDlgItemText(window_loadfile->hwnd_loadfile,IDC_EDIT_FILE_DATA_OFFSET,"0");
  SetDlgItemText(window_loadfile->hwnd_loadfile,IDC_EDIT_FILE_DATA_LENGTH,"");

  /* Zone Mmoire */
  window_loadfile->ram_type = LOADFILE_RAMTYPE_RAM;
  window_loadfile->bank = 0x01;
  window_loadfile->address = 0x2000;
  sprintf(buffer,"%02X",window_loadfile->bank);
  SetDlgItemText(window_loadfile->hwnd_loadfile,IDC_EDIT_RAM_BANK,buffer);
  sprintf(buffer,"%04X",window_loadfile->address);
  SetDlgItemText(window_loadfile->hwnd_loadfile,IDC_EDIT_RAM_ADDRESS,buffer);

  /* Ram Type : RAM */
  UpdateLoadFileRamType(window_loadfile,LOADFILE_RAMTYPE_RAM);
  
  /* OK */
  return(0);
}


/***********************************************************************************************/
/*  subEditLoadFileProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/***********************************************************************************************/
LRESULT CALLBACK subEditLoadFileProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_loadfile *window_loadfile;
  my_Memory(MEMORY_GET_WINDOW_LOADFILE,&window_loadfile,NULL);

  /* Protection */
  if(window_loadfile == NULL)
    return(0);

  /** Identifie le Control **/
  if(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_FILE_PATH) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_FILE_DATA_OFFSET) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_FILE_DATA_LENGTH) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_RAM_BANK) == hwnd_control)
    index = 4;
  else if(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_RAM_ADDRESS) == hwnd_control)
    index = 5;
  else
    return(0);      /* Pas trouv */

  /** Dtecte la touche RETURN pour valider la donne **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_loadfile->hwnd_loadfile);
    }
  else if(index == 1)
    return(CallWindowProc(window_loadfile->oldEditProcFilePath,hwnd_control,msg,wParam,lParam));
  else if(index == 2)
    return(CallWindowProc(window_loadfile->oldEditProcFileOffset,hwnd_control,msg,wParam,lParam));
  else if(index == 3)
    return(CallWindowProc(window_loadfile->oldEditProcFileLength,hwnd_control,msg,wParam,lParam));
  else if(index == 4)
    return(CallWindowProc(window_loadfile->oldEditProcRamBank,hwnd_control,msg,wParam,lParam));
  else if(index == 5)
    return(CallWindowProc(window_loadfile->oldEditProcRamAddress,hwnd_control,msg,wParam,lParam));

  /* OK */
  return(0);
} 


/******************************************************************************/
/*  UpdateLoadFileRamType() :  Change le type de la RAM recevant les donnes. */
/******************************************************************************/
static void UpdateLoadFileRamType(struct window_loadfile *window_loadfile, int ram_type)
{
  if(ram_type == LOADFILE_RAMTYPE_RAM)
    {
      /* Visuel */
      window_loadfile->ram_type = LOADFILE_RAMTYPE_RAM;
      SendDlgItemMessage(window_loadfile->hwnd_loadfile,IDC_RADIO_RAM,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_loadfile->hwnd_loadfile,IDC_RADIO_DOCRAM,BM_SETCHECK,0,0);

      /* Enable BANK */
      EnableWindow(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_RAM_BANK),TRUE);
    }
  else
    {
      /* Visuel */
      window_loadfile->ram_type = LOADFILE_RAMTYPE_DOCRAM;
      SendDlgItemMessage(window_loadfile->hwnd_loadfile,IDC_RADIO_RAM,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_loadfile->hwnd_loadfile,IDC_RADIO_DOCRAM,BM_SETCHECK,1,0);

      /* Disable BANK */
      EnableWindow(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_RAM_BANK),FALSE);
    }
}


/************************************************************************/
/*  LoadDataFileToMemory() :  Charge en mmoire les donnes du fichier. */
/************************************************************************/
static void LoadDataFileToMemory(struct window_loadfile *window_loadfile)
{
  unsigned char *file_data;
  DWORD address, bank, bank_address;
  int i, file_size, is_error;
  int file_data_offset, file_data_length;
  char file_path[1024];
  char value_buffer[1024];
  char error_buffer[1024];
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /** Rcupre les valeurs **/
  GetWindowText(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_FILE_PATH),file_path,1024-1);
  if(strlen(file_path) == 0)
    {
      MessageBox(window_loadfile->hwnd_loadfile,"Empty File Path !","Source Data Error",MB_OK|MB_ICONWARNING);
      return;
    }

  /** Charge le fichier en mmoire **/
  file_data = LoadBinaryFile(file_path,&file_size,&error_buffer[0]);
  if(file_data == NULL)
    {
      MessageBox(window_loadfile->hwnd_loadfile,"Error, Impossible to load File data !","Source Data Error",MB_OK|MB_ICONWARNING);
      return;
    }
  if(file_size == 0)
    {
      free(file_data);
      MessageBox(window_loadfile->hwnd_loadfile,"Error, The data File is empty !","Source Data Error",MB_OK|MB_ICONWARNING);
      return;
    }

  /** Offset + Length du fichier de Data **/
  file_data_offset = EvaluateAsOffsetOrLength(window_loadfile->hwnd_loadfile,IDC_EDIT_FILE_DATA_OFFSET);
  file_data_length = EvaluateAsOffsetOrLength(window_loadfile->hwnd_loadfile,IDC_EDIT_FILE_DATA_LENGTH);
  if(file_data_offset >= file_size)
    {
      free(file_data);
      MessageBox(window_loadfile->hwnd_loadfile,"Error, The Offset value is greater than the File size !","Source Data Error",MB_OK|MB_ICONWARNING);
      return;
    }
  if((file_data_offset + file_data_length) > file_size)
    {
      free(file_data);
      MessageBox(window_loadfile->hwnd_loadfile,"Error, The Offset+Length value is greater than the File size !","Source Data Error",MB_OK|MB_ICONWARNING);
      return;
    }

  /* On ajuste la longueur si 0 */
  if(file_data_length == 0)
    file_data_length = (file_size - file_data_offset);

  /** Addresse Mmoire **/
  if(window_loadfile->ram_type == LOADFILE_RAMTYPE_RAM)
    {
      /* Rcupre l'adresse cible */
      GetWindowText(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_RAM_BANK),value_buffer,1024-1);
      bank = EvaluateAsByte(value_buffer,&is_error);
      GetWindowText(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_RAM_ADDRESS),value_buffer,1024-1);
      address = EvaluateAsWord(value_buffer,&is_error);
      bank_address = (bank << 16) | address;

      /* Transfre les bytes dans la mmoire */
      for(i=0; i<file_data_length; i++)
        WriteMemoryByte(bank_address+i,file_data[file_data_offset+i],current_mmu);

      /* Transfre les donnes dans la mmoire de l'mulateur */
      WriteDataToEmulatorMemory(RAMTYPE_RAM,bank_address,file_data_length,&file_data[file_data_offset]);
    }
  else   /* Doc RAM */
    {
      /* Rcupre l'adresse DOC cible */
      GetWindowText(GetDlgItem(window_loadfile->hwnd_loadfile,IDC_EDIT_RAM_ADDRESS),value_buffer,1024-1);
      address = EvaluateAsWord(value_buffer,&is_error);

      /* Transfre les bytes dans la DOC Ram */
      for(i=0; i<file_data_length; i++)
        WriteDOCRAMByte(address+i,file_data[file_data_offset+i],current_mmu);

      /* Transfre les donnes dans la mmoire de l'mulateur */
      WriteDataToEmulatorMemory(RAMTYPE_DOC,address,file_data_length,&file_data[file_data_offset]);
    }

  /* Libration mmoire */
  free(file_data);
}


/**********************************************************************/
/*  DeleteLoadFileChildWindows() :  Destruction des fentres enfants. */
/**********************************************************************/
static void DeleteLoadFileChildWindows(struct window_loadfile *window_loadfile)
{
  /** Libration des ressources **/  
}


/************************************************************************/
/*  mem_free_window_loadfile() :  Libration mmoire pour la structure. */
/************************************************************************/
void mem_free_window_loadfile(struct window_loadfile *window_loadfile)
{
  if(window_loadfile)
    {      
      free(window_loadfile);
    }
}

/**********************************************************************/
