/*************************************************************************/
/*                                                                       */
/*   Window_Main_Menu.c :  Module de gestion du Menu de la fentre Main. */
/*                                                                       */
/*************************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : Avril 2018  */
/*************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <windows.h>
#include <commctrl.h>
#include <Commdlg.h>
#include "resource.h"

#include "Dc_MMU.h"
#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Snapshot.h"
#include "Dc_Request.h"
#include "Cyrene_Engine.h"
#include "Window_About.h"
#include "Window_Stack.h"
#include "Window_DirectPage.h"
#include "Window_Softswitch.h"
#include "Window_Processor.h"
#include "Window_MemoryArea.h"
#include "Window_RamSound.h"
#include "Window_TextPage40.h"
#include "Window_TextPage80.h"
#include "Window_GraphicGR.h"
#include "Window_GraphicHGR.h"
#include "Window_GraphicDHGRMono.h"
#include "Window_GraphicDHGRColor.h"
#include "Window_GraphicDGR.h"
#include "Window_GraphicHGRText40.h"
#include "Window_GraphicHGRText80.h"
#include "Window_GraphicGRText40.h"
#include "Window_GraphicGRText80.h"
#include "Window_GraphicDGRText80.h"
#include "Window_GraphicDHGRMonoText80.h"
#include "Window_GraphicDHGRColorText80.h"
#include "Window_GraphicSHR.h"
#include "Window_Disassembler.h"
#include "Window_MemoryMap.h"
#include "Window_DOCRegister.h"
#include "Window_Search.h"
#include "Window_Replace.h"
#include "Window_LoadFile.h"
#include "Window_SaveMemory.h"
#include "Window_Compare.h"
#include "Window_QuickWatch.h"
#include "Window_Modify.h"
#include "Window_Move.h"
#include "Window_GraphicScreen.h"
#include "Window_Attach.h"
#include "Window_BatteryRam.h"
#include "Window_Internals.h"
#include "Window_Symbol.h"
#include "Window_IPC.h"
#include "Window_Breakpoint.h"
#include "Window_Statistic.h"
#include "Window_Profile.h"
#include "Window_Preference.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"

static void MenuFileAttachKegs(struct window_main *);
static void MenuFileMemorySnapshot(struct window_main *,int);
static void MenuFileLoadSymbols(struct window_main *);
static void MenuFileLoadBreakpoints(struct window_main *);
static void MenuEditSearch(struct window_main *);
static void MenuEditReplace(struct window_main *);
static void MenuEditMove(struct window_main *);
static void MenuEditModify(struct window_main *);
static void MenuEditCompare(struct window_main *);
static void MenuEditLoadFile(struct window_main *);
static void MenuEditSaveMemoryArea(struct window_main *);
static void MenuEditPreferences(struct window_main *);
static void MenuViewProcessor(struct window_main *);
static void MenuViewStack(struct window_main *);
static void MenuViewDirectPage(struct window_main *);
static void MenuViewSoftswitch(struct window_main *);
static void MenuViewMemoryArea(struct window_main *);
static void MenuViewQuickWatch(struct window_main *);
static void MenuViewTextPage40Col(struct window_main *);
static void MenuViewTextPage80Col(struct window_main *);
static void MenuViewGraphicGR(struct window_main *);
static void MenuViewGraphicGRText40Col(struct window_main *);
static void MenuViewGraphicGRText80Col(struct window_main *);
static void MenuViewGraphicDGR(struct window_main *);
static void MenuViewGraphicDGRText80Col(struct window_main *);
static void MenuViewGraphicHGR(struct window_main *);
static void MenuViewGraphicHGRText40Col(struct window_main *);
static void MenuViewGraphicHGRText80Col(struct window_main *);
static void MenuViewGraphicDHGRColor140(struct window_main *);
static void MenuViewGraphicDHGRColorText80Col(struct window_main *);
static void MenuViewGraphicDHGRMono560(struct window_main *);
static void MenuViewGraphicDHGRMonoText80Col(struct window_main *);
static void MenuViewGraphicSHR(struct window_main *);
static void MenuViewGraphicScreen(struct window_main *);
static void MenuViewRamSound(struct window_main *);
static void MenuViewDOCRegister(struct window_main *);
static void MenuViewBatteryRam(struct window_main *);
static void MenuViewCodeDisassembler(struct window_main *);
static void MenuViewMemoryMap(struct window_main *);
static void MenuViewInternals(struct window_main *);
static void MenuCodeSymbol(struct window_main *);
static void MenuCodeBreakpoint(struct window_main *);
static void MenuExtraFunctionStatistics(struct window_main *);
static void MenuExtraFunctionProfiler(struct window_main *);
static void MenuReset(struct window_main *);


/***********************************************************/
/*  MainMenuToolbarCommand() : Evnements du Menu+Toolbar. */
/***********************************************************/
int MainMenuToolbarCommand(HWND hwndMain, int menu_id, HWND g_wnd_ctl, UINT code_notify)
{
  struct parameter *param;
  struct window_main *window_main;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  
  switch(menu_id)
    {
      /***************/
      /*  Menu File  */
      /***************/
      /* Attach Kegs */
      case IDH_BUTTON_ATTACH :
      case ID_MENU_FILE_ATTACH_KEGS :
        MenuFileAttachKegs(window_main);
        break;

      case IDH_BUTTON_DETACH :
      case ID_MENU_FILE_DETACH_KEGS :
        MenuFileDetachKegs(window_main);
        break;

      /* Save Memory Snapshot Apple II 8 bit */
      case ID_MENU_FILE_SAVE_SNAPSHOT_II :
        MenuFileMemorySnapshot(window_main,ID_MENU_FILE_SAVE_SNAPSHOT_II);
        break;

      /* Save Memory Snapshot Apple IIgs 16 bit */
      case IDH_BUTTON_SAVE_SNAPSHOT :
      case ID_MENU_FILE_SAVE_SNAPSHOT_GS :
        MenuFileMemorySnapshot(window_main,ID_MENU_FILE_SAVE_SNAPSHOT_GS);
        break;

      /* Load Memory Snapshot Apple IIgs 16 bit */
      case IDH_BUTTON_LOAD_SNAPSHOT :
      case ID_MENU_FILE_LOAD_SNAPSHOT_GS :
        MenuFileMemorySnapshot(window_main,ID_MENU_FILE_LOAD_SNAPSHOT_GS);
        break;

      /* Load Symbols file */
      case IDH_BUTTON_LOAD_SYMBOL :
      case ID_MENU_FILE_LOAD_SYMBOLS :
        MenuFileLoadSymbols(window_main);
        break;

      /* Load Breakpoint file */
      case IDH_BUTTON_LOAD_BREAKPOINT :
      case ID_MENU_FILE_LOAD_BREAKPOINTS :
        MenuFileLoadBreakpoints(window_main);
        break;

      /* Exit */
      case ID_MENU_FILE_EXIT :
        SendMessage(window_main->hwnd_main,WM_CLOSE,0,0);
        break;

      /***************/
      /*  Menu Edit  */
      /***************/
      /* Recherche */
      case ID_MENU_EDIT_SEARCH :
        MenuEditSearch(window_main);
        break;

      /* Remplace */
      case ID_MENU_EDIT_REPLACE :
        MenuEditReplace(window_main);
        break;

      /* Compare */
      case ID_MENU_EDIT_COMPARE :
        MenuEditCompare(window_main);
        break;

      /* Move */
      case ID_MENU_EDIT_MOVE :
        MenuEditMove(window_main);
        break;

      /* Modify */
      case ID_MENU_EDIT_MODIFY :
        MenuEditModify(window_main);
        break;

      /* Load File Data into Memory */
      case ID_MENU_EDIT_LOADFILE :
        MenuEditLoadFile(window_main);
        break;
        
      /* Save Memory Area Data to File */
      case ID_MENU_EDIT_SAVEMEMORYAREA :
        MenuEditSaveMemoryArea(window_main);
        break;

      /* Preferences */
      case ID_MENU_EDIT_PREFERENCES :
        MenuEditPreferences(window_main);
        break;

      /***************/
      /*  Menu View  */
      /***************/
      /* Processor */
      case ID_MENU_VIEW_PROCESSOR :
        MenuViewProcessor(window_main);
        break;

      /* Stack */
      case ID_MENU_VIEW_STACK :
        MenuViewStack(window_main);
        break;

      /* Direct Page */
      case ID_MENU_VIEW_DIRECTPAGE :
        MenuViewDirectPage(window_main);
        break;

      /* Softswitch */
      case ID_MENU_VIEW_SOFTSWITCH :
        MenuViewSoftswitch(window_main);
        break;

      /* Memory Area */
      case ID_MENU_VIEW_MEMORYAREA :
        MenuViewMemoryArea(window_main);
        break;

      /* Quick Watch */
      case ID_MENU_VIEW_QUICKWATCH :
        MenuViewQuickWatch(window_main);
        break;

      /* Text Page 40 Colonnes */
      case ID_MENU_VIEW_TEXTPAGE40COL :
        MenuViewTextPage40Col(window_main);
        break;      

      /* Text Page 80 Colonnes */
      case ID_MENU_VIEW_TEXTPAGE80COL :
        MenuViewTextPage80Col(window_main);
        break;  
        
      /* Graphic GR */
      case ID_MENU_VIEW_GRAPHICGR :
        MenuViewGraphicGR(window_main);
        break;  

      /* Graphic GR Text 40 Colonnes */
      case ID_MENU_VIEW_GRAPHICGRTEXT40COL :
        MenuViewGraphicGRText40Col(window_main);
        break;  

      /* Graphic GR Text 80 Colonnes */
      case ID_MENU_VIEW_GRAPHICGRTEXT80COL :
        MenuViewGraphicGRText80Col(window_main);
        break;  

      /* Graphic DGR */
      case ID_MENU_VIEW_GRAPHICDGR :
        MenuViewGraphicDGR(window_main);
        break;  

      /* Graphic DGR Text 80 Colonnes */
      case ID_MENU_VIEW_GRAPHICDGRTEXT80COL :
        MenuViewGraphicDGRText80Col(window_main);
        break;  

      /* Graphic HGR */
      case ID_MENU_VIEW_GRAPHICHGR :
        MenuViewGraphicHGR(window_main);
        break;

      /* Graphic HGR Text 40 Colonnes */
      case ID_MENU_VIEW_GRAPHICHGRTEXT40COL :
        MenuViewGraphicHGRText40Col(window_main);
        break;  

      /* Graphic HGR Text 80 Colonnes */
      case ID_MENU_VIEW_GRAPHICHGRTEXT80COL :
        MenuViewGraphicHGRText80Col(window_main);
        break;  

      /* Graphic DHGR Color 140 */
      case ID_MENU_VIEW_GRAPHICDHGRCOLOR140 :
        MenuViewGraphicDHGRColor140(window_main);
        break;

      /* Graphic DHGR Color Text 80 */
      case ID_MENU_VIEW_GRAPHICDHGRCOLORTEXT80COL :
        MenuViewGraphicDHGRColorText80Col(window_main);
        break; 

      /* Graphic DHGR Mono 560 */
      case ID_MENU_VIEW_GRAPHICDHGRMONO560 :
        MenuViewGraphicDHGRMono560(window_main);
        break; 

      /* Graphic DHGR Mono Text 80 */
      case ID_MENU_VIEW_GRAPHICDHGRMONOTEXT80COL :
        MenuViewGraphicDHGRMonoText80Col(window_main);
        break; 

      /* Graphic SHR */
      case IDH_BUTTON_VIEW_SHR :
      case ID_MENU_VIEW_GRAPHICSHR :
        MenuViewGraphicSHR(window_main);
        break; 
        
      /* Graphic Screen */
      case ID_MENU_VIEW_GRAPHICSCREEN :
        MenuViewGraphicScreen(window_main);
        break; 
        
      /* Ram Sound */
      case ID_MENU_VIEW_RAMSOUND :
        MenuViewRamSound(window_main);
        break; 
        
      /* DOC Registers */
      case ID_MENU_VIEW_DOCREGISTER :
        MenuViewDOCRegister(window_main);
        break; 

      /* Battery Ram */
      case ID_MENU_VIEW_BATTERYRAM :
        MenuViewBatteryRam(window_main);
        break; 

      /* Code Disassembler */
      case IDH_BUTTON_VIEW_CODE :
      case ID_MENU_VIEW_CODEDISASSEMBLER :
        MenuViewCodeDisassembler(window_main);
        break; 

      /* Memory Map */
      case IDH_BUTTON_VIEW_MEMORYMAP :
      case ID_MENU_VIEW_MEMORYMAP :
        MenuViewMemoryMap(window_main);
        break;

      /* Apple IIgs Internals */
      case IDH_BUTTON_VIEW_INTERNALS :
      case ID_MENU_VIEW_INTERNALS :
        MenuViewInternals(window_main);
        break;

      /***************/
      /*  Menu Code  */
      /***************/
      /* Symbol */
      case ID_MENU_CODE_SYMBOL :
        MenuCodeSymbol(window_main);
        break;

      /* Breakpoint */
      case ID_MENU_CODE_BREAKPOINT :
        MenuCodeBreakpoint(window_main);
        break;

      /* Run */
      case ID_MENU_CODE_RUN :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_RUN);
        break;

      /* Step In */
      case ID_MENU_CODE_STEP_IN :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_IN);
        break;

      /* Step Over */
      case ID_MENU_CODE_STEP_OVER :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_OVER);
        break;

      /* Next 10 */
      case ID_MENU_CODE_10 :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_10X);
        break;

      /* Next 100 */
      case ID_MENU_CODE_100 :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_100X);
        break;

      /* Next 1000 */
      case ID_MENU_CODE_1000 :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_1000X);
        break;

      /* Next 10000 */
      case ID_MENU_CODE_10000 :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_10000X);
        break;

      /* Next JMP */
      case ID_MENU_CODE_JMP :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_JMP);
        break;

      /* Next JSR */
      case ID_MENU_CODE_JSR :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_JSR);
        break;

      /* Next RTS */
      case ID_MENU_CODE_RTS :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_RTS);
        break;

      /* Next VBL */
      case ID_MENU_CODE_VBL :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_VBL);
        break;

      /* Next INTERRUPT */
      case ID_MENU_CODE_INTERRUPT :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_INTERRUPT);
        break;

      /* Next TOOLBOX */
      case ID_MENU_CODE_TOOLBOX :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_TOOLBOX);
        break;

      /* Next OS */
      case ID_MENU_CODE_GSOS :
        ProcessOperationButton(window_main,IDC_BUTTON_CODE_GSOS);
        break;

      /****************/
      /*  Menu Extra  */
      /****************/
      /* Fonction Statistics */
      case IDH_BUTTON_STATISTIC :
      case ID_MENU_EXTRA_STATISTIC :
        MenuExtraFunctionStatistics(window_main);
        break;

      /* Fonctions Profiler */
      case IDH_BUTTON_PROFILE :
      case ID_MENU_EXTRA_PROFILE :
        MenuExtraFunctionProfiler(window_main);
        break;
        
      /* Clear All Recording */
      case IDH_BUTTON_RESET :
        MenuReset(window_main);
        break;

      /************/
      /*  Menu ?  */
      /************/
      case ID_MENU_ABOUT :
        MenuAbout(window_main);
        break;

      /**********************/
      /*  Barre de boutons  */
      /**********************/
      default :
        break;
    }

  return(0);
}


/**************************************************************/
/*  CheckMainWindowMenuItem() : Check or uncheck a menu item. */
/**************************************************************/
void CheckMainWindowMenuItem(int cmdID, BOOL doCheck)
{
  struct window_main *window_main;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /* Positonne ou Enlve la marque d'une lement du Menu */
  CheckMenuItem(window_main->hmenu_menu,cmdID,MF_BYCOMMAND|(doCheck?MF_CHECKED:MF_UNCHECKED));
}


/*******************************************************************************/
/*  MenuFileAttachKegs() :  On vient de slectionner Attach Kegs du menu File. */
/*******************************************************************************/
static void MenuFileAttachKegs(struct window_main *window_main)
{
  struct window_attach *window_attach;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_ATTACH,&window_attach,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Attach  la fois **/
  if(window_attach == NULL)
    {
      /** Cration de la fentre Attach **/
      window_attach = BuildAttachWindow(param->hInstance,ID_MENU_FILE_ATTACH_KEGS);
      if(window_attach == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_FILE_ATTACH_KEGS,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Attach **/
      SendMessage(window_attach->hwnd_attach,WM_CLOSE,0,0);
    }
}


/*******************************************************************************/
/*  MenuFileDetachKegs() :  On vient de slectionner Detach Kegs du menu File. */
/*******************************************************************************/
void MenuFileDetachKegs(struct window_main *window_main)
{
  struct window_ipc *window_ipc;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre IPC de Cyrene */
  my_Memory(MEMORY_GET_WINDOW_IPC,&window_ipc,NULL);

  /** On va se dconnecter de KEGS **/
  /* On indique  Kegs la fin des changes */
  SendDataToKegs(IPC_REQUEST_TYPE_C2K_CLOSE_CONNECTION,(int)strlen("Bye")+1,(unsigned char *)"Bye");

  /* Fin de la connection */
  param->kegs_connection_status = CONNECTION_STATUS_CLOSED;

  /* Close Shared Memory */
  EnterCriticalSection(&param->cs_shared_memory);
  if(window_ipc->shared_memory_ptr != NULL)
    UnmapViewOfFile(window_ipc->shared_memory_ptr);
  window_ipc->shared_memory_ptr = NULL;
  if(window_ipc->hSharedMemory != 0)
    CloseHandle(window_ipc->hSharedMemory);
  window_ipc->hSharedMemory = NULL;
  LeaveCriticalSection(&param->cs_shared_memory);

  /* Grise / Dgrise les options du Menu */
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_FILE_ATTACH_KEGS,MF_BYCOMMAND | MF_ENABLED);
  EnableMenuItem(window_main->hmenu_menu,IDH_BUTTON_DETACH,MF_BYCOMMAND | MF_GRAYED);

  /* MAJ de la Status Bar */
  UpdateWindowMainStatusBar();

  /* Nous ne sommes plus connect  Kegs */
  window_main->code_status = CYRENE_STATUS_NOT_CONNECTED;

  /* On demande la MAJ de la fentre Main : Operation History + Code + CallStack + Output */
  SendMessage(window_main->hwnd_main,HM_OPERATION_CODE_REFRESHWINDOW,0,0);
}


/***************************************************************************************/
/*  UpdateWindowMainMenuItemStatus() : Activation / Desactivation des items des menus. */
/***************************************************************************************/
void UpdateWindowMainMenuItemStatus(void)
{
  DWORD menu_data_status, menu_connection_status;
  int toolbar_data_status, toolbar_connection_status;
  struct s65816 *current_s65816;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_S65816,&current_s65816,NULL);

  /* Menu Items Visibles ou Griss ? */
  menu_data_status = (current_s65816 == NULL) ? (MF_BYCOMMAND | MF_GRAYED) : (MF_BYCOMMAND | MF_ENABLED);
  menu_connection_status = (param->kegs_connection_status != CONNECTION_STATUS_OPEN) ? (MF_BYCOMMAND | MF_GRAYED) : (MF_BYCOMMAND | MF_ENABLED);
  toolbar_data_status = (current_s65816 == NULL) ? FALSE : TRUE;
  toolbar_connection_status = (param->kegs_connection_status != CONNECTION_STATUS_OPEN) ? FALSE : TRUE;

  /** Modification du status des Items des Menus pouvant tre griss (pas encore de Data charges) **/
  /* File */
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_FILE_DETACH_KEGS,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_FILE_SAVE_SNAPSHOT_II,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_FILE_SAVE_SNAPSHOT_GS,menu_data_status);

  /* Edit */
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_EDIT_SEARCH,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_EDIT_REPLACE,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_EDIT_COMPARE,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_EDIT_MOVE,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_EDIT_MODIFY,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_EDIT_LOADFILE,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_EDIT_SAVEMEMORYAREA,menu_data_status);

  /* View */
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_PROCESSOR,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_STACK,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_DIRECTPAGE,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_SOFTSWITCH,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_MEMORYAREA,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_QUICKWATCH,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_TEXTPAGE40COL,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_TEXTPAGE80COL,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICGR,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICGRTEXT40COL,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICGRTEXT80COL,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICDGR,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICDGRTEXT80COL,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICHGR,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICHGRTEXT40COL,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICHGRTEXT80COL,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICDHGRCOLOR140,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICDHGRCOLORTEXT80COL,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICDHGRMONO560,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICDHGRMONOTEXT80COL,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICSHR,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_GRAPHICSCREEN,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_RAMSOUND,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_DOCREGISTER,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_CODEDISASSEMBLER,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_MEMORYMAP,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_BATTERYRAM,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_VIEW_INTERNALS,menu_data_status);

  /* Code */
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_SYMBOL,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_BREAKPOINT,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_RUN,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_STEP_IN,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_STEP_OVER,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_10,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_100,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_1000,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_10000,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_JMP,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_JSR,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_RTS,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_VBL,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_INTERRUPT,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_TOOLBOX,menu_connection_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_GSOS,menu_connection_status);

  /* Extra */
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_EXTRA_STATISTIC,menu_data_status);
  EnableMenuItem(window_main->hmenu_menu,ID_MENU_EXTRA_PROFILE,menu_data_status);

  /* About */

  /** Active / Dsactive les boutons de la Toolbar **/
  SendMessage(window_main->hwnd_toolbar,TB_ENABLEBUTTON,IDH_BUTTON_DETACH,(LPARAM)MAKELONG(toolbar_connection_status,0));
  SendMessage(window_main->hwnd_toolbar,TB_ENABLEBUTTON,IDH_BUTTON_SAVE_SNAPSHOT,(LPARAM)MAKELONG(toolbar_data_status,0));
  SendMessage(window_main->hwnd_toolbar,TB_ENABLEBUTTON,IDH_BUTTON_VIEW_SHR,(LPARAM)MAKELONG(toolbar_data_status,0));
  SendMessage(window_main->hwnd_toolbar,TB_ENABLEBUTTON,IDH_BUTTON_VIEW_CODE,(LPARAM)MAKELONG(toolbar_data_status,0));
  SendMessage(window_main->hwnd_toolbar,TB_ENABLEBUTTON,IDH_BUTTON_VIEW_INTERNALS,(LPARAM)MAKELONG(toolbar_data_status,0));
  SendMessage(window_main->hwnd_toolbar,TB_ENABLEBUTTON,IDH_BUTTON_VIEW_MEMORYMAP,(LPARAM)MAKELONG(toolbar_data_status,0));
  SendMessage(window_main->hwnd_toolbar,TB_ENABLEBUTTON,IDH_BUTTON_STATISTIC,(LPARAM)MAKELONG(toolbar_data_status,0));
  SendMessage(window_main->hwnd_toolbar,TB_ENABLEBUTTON,IDH_BUTTON_PROFILE,(LPARAM)MAKELONG(toolbar_data_status,0));
  SendMessage(window_main->hwnd_toolbar,TB_ENABLEBUTTON,IDH_BUTTON_RESET,(LPARAM)MAKELONG(toolbar_data_status,0));
}


/*******************************************************************************************/
/*  MenuFileMemorySnapshot() :  On vient de slectionner Save MemorySnapshot du menu View. */
/*******************************************************************************************/
static void MenuFileMemorySnapshot(struct window_main *window_main, int menu_id)
{
  struct snapshot *current_snapshot;

  /** Cration du fichier Snapshot sur disque **/
  if(menu_id == ID_MENU_FILE_SAVE_SNAPSHOT_II)
    SaveSnapshotFile(SNAPSHOT_TYPE_II);
  else if(menu_id == ID_MENU_FILE_SAVE_SNAPSHOT_GS)
    SaveSnapshotFile(SNAPSHOT_TYPE_GS);
  else if(menu_id == ID_MENU_FILE_LOAD_SNAPSHOT_GS)
    {
      /** Rcupre les donnes d'un fichier Snapshot GS **/
      current_snapshot = LoadSnapshotFile(SNAPSHOT_TYPE_GS);
      if(current_snapshot)
        {
          /** On va initialiser/MAJ la mmoire avec les donnes du Snapshot **/
          Initialize65816Engine(current_snapshot);

          /* Libration du snapshot */
          mem_free_snapshot(current_snapshot);

          /* Activation de tous les menus */
          UpdateWindowMainMenuItemStatus();

          /* MAJ de la Status Bar */
          UpdateWindowMainStatusBar();
        }
    }
}


/*******************************************************************/
/*  MenuFileLoadSymbols() :  Charge le fichier Symbols en mmoire .*/
/*******************************************************************/
static void MenuFileLoadSymbols(struct window_main *window_main)
{
  /** Charge en mmoire le fichier **/
  LoadAppendSymbolFile(NULL,NULL);

  /* Active le Menu Symbol */
  if(window_main->current_symbol_table != NULL)
    EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_SYMBOL,MF_BYCOMMAND | MF_ENABLED);

  /* MAJ de la Status Bar */
  UpdateWindowMainStatusBar();
}


/***************************************************************************/
/*  MenuFileLoadBreakpoints() :  Charge le fichier Breakpoints en mmoire .*/
/***************************************************************************/
static void MenuFileLoadBreakpoints(struct window_main *window_main)
{
  /** Charge en mmoire le fichier **/
  LoadAppendBreakpointFile(NULL,NULL);

  /* Active le Menu Breakpoint */
  if(window_main->current_breakpoint_table != NULL)
    EnableMenuItem(window_main->hmenu_menu,ID_MENU_CODE_BREAKPOINT,MF_BYCOMMAND | MF_ENABLED);

  /* MAJ de la Status Bar */
  UpdateWindowMainStatusBar();
}


/**********************************************************************/
/*  MenuEditSearch() :  On vient de slectionner Search du menu Edit. */
/**********************************************************************/
static void MenuEditSearch(struct window_main *window_main)
{
  struct window_search *window_search;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_SEARCH,&window_search,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Search  la fois **/
  if(window_search == NULL)
    {
      /** Cration de la fentre Search **/
      window_search = BuildSearchWindow(param->hInstance,ID_MENU_EDIT_SEARCH);
      if(window_search == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_EDIT_SEARCH,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Search **/
      SendMessage(window_search->hwnd_search,WM_CLOSE,0,0);
    }
}


/************************************************************************/
/*  MenuEditReplace() :  On vient de slectionner Replace du menu Edit. */
/************************************************************************/
static void MenuEditReplace(struct window_main *window_main)
{
  struct window_replace *window_replace;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_REPLACE,&window_replace,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Replace  la fois **/
  if(window_replace == NULL)
    {
      /** Cration de la fentre Replace **/
      window_replace = BuildReplaceWindow(param->hInstance,ID_MENU_EDIT_REPLACE);
      if(window_replace == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_EDIT_REPLACE,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Replace **/
      SendMessage(window_replace->hwnd_replace,WM_CLOSE,0,0);
    }
}


/**************************************************************************/
/*  MenuEditCompare() :  On vient de slectionner Compare du menu Edit. */
/**************************************************************************/
static void MenuEditCompare(struct window_main *window_main)
{
  struct window_compare *window_compare;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Compare (On peut avoir plusieurs fentres Compare) **/
  window_compare = BuildCompareWindow(param->hInstance,ID_MENU_EDIT_COMPARE);
  if(window_compare == NULL)
    return;
}


/******************************************************************/
/*  MenuEditMove() :  On vient de slectionner Move du menu Edit. */
/******************************************************************/
static void MenuEditMove(struct window_main *window_main)
{
  struct window_move *window_move;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Move (On peut avoir plusieurs fentres Move) **/
  window_move = BuildMoveWindow(param->hInstance,ID_MENU_EDIT_MOVE);
  if(window_move == NULL)
    return;
}


/**********************************************************************/
/*  MenuEditModify() :  On vient de slectionner Modify du menu Edit. */
/**********************************************************************/
static void MenuEditModify(struct window_main *window_main)
{
  struct window_modify *window_modify;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Modify (On peut avoir plusieurs fentres Modify) **/
  window_modify = BuildModifyWindow(param->hInstance,ID_MENU_EDIT_MODIFY);
  if(window_modify == NULL)
    return;
}


/**************************************************************************/
/*  MenuEditLoadFile() :  On vient de slectionner LoadFile du menu Edit. */
/**************************************************************************/
static void MenuEditLoadFile(struct window_main *window_main)
{
  struct window_loadfile *window_loadfile;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Load File (On peut avoir plusieurs fentres Load File) **/
  window_loadfile = BuildLoadFileWindow(param->hInstance,ID_MENU_EDIT_LOADFILE);
  if(window_loadfile == NULL)
    return;
}


/**************************************************************************************/
/*  MenuEditSaveMemoryArea() :  On vient de slectionner SaveMemoryArea du menu Edit. */
/**************************************************************************************/
static void MenuEditSaveMemoryArea(struct window_main *window_main)
{
  struct window_savememory *window_savememory;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Save Memory (On peut avoir plusieurs fentres Save Memory) **/
  window_savememory = BuildSaveMemoryWindow(param->hInstance,ID_MENU_EDIT_SAVEMEMORYAREA);
  if(window_savememory == NULL)
    return;
}


/********************************************************************************/
/*  MenuEditPreferences() :  On vient de slectionner Preferences du menu Edit. */
/********************************************************************************/
static void MenuEditPreferences(struct window_main *window_main)
{
  struct window_preference *window_preference;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_PREFERENCE,&window_preference,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Preference  la fois **/
  if(window_preference == NULL)
    {
      /** Cration de la fentre Preference **/
      window_preference = BuildPreferenceWindow(param->hInstance,ID_MENU_EDIT_PREFERENCES);
      if(window_preference == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_EDIT_PREFERENCES,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Preference **/
      SendMessage(window_preference->hwnd_preference,WM_CLOSE,0,0);
    }
}


/****************************************************************************/
/*  MenuViewProcessor() :  On vient de slectionner Processor du menu View. */
/****************************************************************************/
static void MenuViewProcessor(struct window_main *window_main)
{
  struct window_processor *window_processor;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_PROCESSOR,&window_processor,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Processor  la fois **/
  if(window_processor == NULL)
    {
      /** Cration de la fentre Processor **/
      window_processor = BuildProcessorWindow(param->hInstance,ID_MENU_VIEW_PROCESSOR);
      if(window_processor == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_VIEW_PROCESSOR,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Processor **/
      SendMessage(window_processor->hwnd_processor,WM_CLOSE,0,0);
    }
}


/********************************************************************/
/*  MenuViewStack() :  On vient de slectionner Stack du menu View. */
/********************************************************************/
static void MenuViewStack(struct window_main *window_main)
{
  struct window_stack *window_stack;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_STACK,&window_stack,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Stack  la fois **/
  if(window_stack == NULL)
    {
      /** Cration de la fentre Stack **/
      window_stack = BuildStackWindow(param->hInstance,ID_MENU_VIEW_STACK);
      if(window_stack == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_VIEW_STACK,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Stack **/
      SendMessage(window_stack->hwnd_stack,WM_CLOSE,0,0);
    }
}


/******************************************************************************/
/*  MenuViewDirectPage() :  On vient de slectionner DirectPage du menu View. */
/******************************************************************************/
static void MenuViewDirectPage(struct window_main *window_main)
{
  struct window_directpage *window_directpage;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_DIRECTPAGE,&window_directpage,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Direct Page  la fois **/
  if(window_directpage == NULL)
    {
      /** Cration de la fentre Direct Page **/
      window_directpage = BuildDirectPageWindow(param->hInstance,ID_MENU_VIEW_DIRECTPAGE);
      if(window_directpage == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_VIEW_DIRECTPAGE,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Direct Page **/
      SendMessage(window_directpage->hwnd_directpage,WM_CLOSE,0,0);
    }
}


/******************************************************************************/
/*  MenuViewSoftswitch() :  On vient de slectionner Softswitch du menu View. */
/******************************************************************************/
static void MenuViewSoftswitch(struct window_main *window_main)
{
  struct window_softswitch *window_softswitch;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_SOFTSWITCH,&window_softswitch,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Softswitch  la fois **/
  if(window_softswitch == NULL)
    {
      /** Cration de la fentre Softswitch **/
      window_softswitch = BuildSoftswitchWindow(param->hInstance,ID_MENU_VIEW_SOFTSWITCH);
      if(window_softswitch == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_VIEW_SOFTSWITCH,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Softswitch **/
      SendMessage(window_softswitch->hwnd_softswitch,WM_CLOSE,0,0);
    }
}


/******************************************************************************/
/*  MenuViewMemoryArea() :  On vient de slectionner MemoryArea du menu View. */
/******************************************************************************/
static void MenuViewMemoryArea(struct window_main *window_main)
{
  struct window_memoryarea *window_memoryarea;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Memory Area (On peut avoir plusieurs fentres Memory Area) **/
  window_memoryarea = BuildMemoryAreaWindow(param->hInstance,ID_MENU_VIEW_MEMORYAREA,0xFFFFFFFF);
  if(window_memoryarea == NULL)
    return;
}


/******************************************************************************/
/*  MenuViewQuickWatch() :  On vient de slectionner QuickWatch du menu View. */
/******************************************************************************/
static void MenuViewQuickWatch(struct window_main *window_main)
{
  struct window_quickwatch *window_quickwatch;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_QUICKWATCH,&window_quickwatch,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Softswitch  la fois **/
  if(window_quickwatch == NULL)
    {
      /** Cration de la fentre Quickwatch **/
      window_quickwatch = BuildQuickWatchWindow(param->hInstance,ID_MENU_VIEW_QUICKWATCH);
      if(window_quickwatch == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_VIEW_QUICKWATCH,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Quickwatch **/
      SendMessage(window_quickwatch->hwnd_quickwatch,WM_CLOSE,0,0);
    }
}


/************************************************************************************/
/*  MenuViewTextPage40Col() :  On vient de slectionner TextPage40Col du menu View. */
/************************************************************************************/
static void MenuViewTextPage40Col(struct window_main *window_main)
{
  struct window_textpage40 *window_textpage40;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre TextPage40 (On peut avoir plusieurs fentres TextPage40) **/
  window_textpage40 = BuildTextPage40Window(param->hInstance,ID_MENU_VIEW_TEXTPAGE40COL);
  if(window_textpage40 == NULL)
    return;
}


/************************************************************************************/
/*  MenuViewTextPage80Col() :  On vient de slectionner TextPage80Col du menu View. */
/************************************************************************************/
static void MenuViewTextPage80Col(struct window_main *window_main)
{
  struct window_textpage80 *window_textpage80;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre TextPage80 (On peut avoir plusieurs fentres TextPage80) **/
  window_textpage80 = BuildTextPage80Window(param->hInstance,ID_MENU_VIEW_TEXTPAGE80COL);
  if(window_textpage80 == NULL)
    return;
}


/****************************************************************************/
/*  MenuViewGraphicGR() :  On vient de slectionner GraphicGR du menu View. */
/****************************************************************************/
static void MenuViewGraphicGR(struct window_main *window_main)
{
  struct window_graphicgr *window_graphicgr;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre GraphicGR (On peut avoir plusieurs fentres GraphicGR) **/
  window_graphicgr = BuildGraphicGRWindow(param->hInstance,ID_MENU_VIEW_GRAPHICGR);
  if(window_graphicgr == NULL)
    return;
}


/**********************************************************************************************/
/*  MenuViewGraphicGRText40Col() :  On vient de slectionner GraphicGRText40Col du menu View. */
/**********************************************************************************************/
static void MenuViewGraphicGRText40Col(struct window_main *window_main)
{
  struct window_graphicgrtext40 *window_graphicgrtext40;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre GraphicGRText40 (On peut avoir plusieurs fentres GraphicGRText40) **/
  window_graphicgrtext40 = BuildGraphicGRText40Window(param->hInstance,ID_MENU_VIEW_GRAPHICGRTEXT40COL);
  if(window_graphicgrtext40 == NULL)
    return;
}


/**********************************************************************************************/
/*  MenuViewGraphicGRText80Col() :  On vient de slectionner GraphicGRText80Col du menu View. */
/**********************************************************************************************/
static void MenuViewGraphicGRText80Col(struct window_main *window_main)
{
  struct window_graphicgrtext80 *window_graphicgrtext80;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre GraphicGRText80 (On peut avoir plusieurs fentres GraphicGRText80) **/
  window_graphicgrtext80 = BuildGraphicGRText80Window(param->hInstance,ID_MENU_VIEW_GRAPHICGRTEXT80COL);
  if(window_graphicgrtext80 == NULL)
    return;
}


/******************************************************************************/
/*  MenuViewGraphicDGR() :  On vient de slectionner GraphicDGR du menu View. */
/******************************************************************************/
static void MenuViewGraphicDGR(struct window_main *window_main)
{
  struct window_graphicdgr *window_graphicdgr;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre GraphicDGR (On peut avoir plusieurs fentres GraphicDGR) **/
  window_graphicdgr = BuildGraphicDGRWindow(param->hInstance,ID_MENU_VIEW_GRAPHICDGR);
  if(window_graphicdgr == NULL)
    return;
}


/************************************************************************************************/
/*  MenuViewGraphicDGRText80Col() :  On vient de slectionner GraphicDGRText80Col du menu View. */
/************************************************************************************************/
static void MenuViewGraphicDGRText80Col(struct window_main *window_main)
{
  struct window_graphicdgrtext80 *window_graphicdgrtext80;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre GraphicDGRText80 (On peut avoir plusieurs fentres GraphicDGRText80) **/
  window_graphicdgrtext80 = BuildGraphicDGRText80Window(param->hInstance,ID_MENU_VIEW_GRAPHICDGRTEXT80COL);
  if(window_graphicdgrtext80 == NULL)
    return;
}


/******************************************************************************/
/*  MenuViewGraphicHGR() :  On vient de slectionner GraphicHGR du menu View. */
/******************************************************************************/
static void MenuViewGraphicHGR(struct window_main *window_main)
{
  struct window_graphichgr *window_graphichgr;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre GraphicHGR (On peut avoir plusieurs fentres GraphicHGR) **/
  window_graphichgr = BuildGraphicHGRWindow(param->hInstance,ID_MENU_VIEW_GRAPHICHGR);
  if(window_graphichgr == NULL)
    return;
}


/************************************************************************************************/
/*  MenuViewGraphicHGRText40Col() :  On vient de slectionner GraphicHGRText40Col du menu View. */
/************************************************************************************************/
static void MenuViewGraphicHGRText40Col(struct window_main *window_main)
{
  struct window_graphichgrtext40 *window_graphichgrtext40;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre GraphicHGRText40 (On peut avoir plusieurs fentres GraphicHGRText40) **/
  window_graphichgrtext40 = BuildGraphicHGRText40Window(param->hInstance,ID_MENU_VIEW_GRAPHICHGRTEXT40COL);
  if(window_graphichgrtext40 == NULL)
    return;
}


/************************************************************************************************/
/*  MenuViewGraphicHGRText80Col() :  On vient de slectionner GraphicHGRText80Col du menu View. */
/************************************************************************************************/
static void MenuViewGraphicHGRText80Col(struct window_main *window_main)
{
  struct window_graphichgrtext80 *window_graphichgrtext80;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre GraphicHGRText80 (On peut avoir plusieurs fentres GraphicHGRText80) **/
  window_graphichgrtext80 = BuildGraphicHGRText80Window(param->hInstance,ID_MENU_VIEW_GRAPHICHGRTEXT80COL);
  if(window_graphichgrtext80 == NULL)
    return;
}


/************************************************************************************************/
/*  MenuViewGraphicDHGRColor140() :  On vient de slectionner GraphicDHGRColor140 du menu View. */
/************************************************************************************************/
static void MenuViewGraphicDHGRColor140(struct window_main *window_main)
{
  struct window_graphicdhgrcolor *window_graphicdhgrcolor;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre GraphicDHGRColor (On peut avoir plusieurs fentres GraphicDHGRColor) **/
  window_graphicdhgrcolor = BuildGraphicDHGRColorWindow(param->hInstance,ID_MENU_VIEW_GRAPHICDHGRCOLOR140);
  if(window_graphicdhgrcolor == NULL)
    return;
}


/*********************************************************************************************************/
/*  MenuViewGraphicDHGRColorText80Col() :  On vient de slectionner GraphicDHGRColorText80 du menu View. */
/*********************************************************************************************************/
static void MenuViewGraphicDHGRColorText80Col(struct window_main *window_main)
{
  struct window_graphicdhgrcolortext80 *window_graphicdhgrcolortext80;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre GraphicDHGRColorText80 (On peut avoir plusieurs fentres GraphicDHGRColorText80) **/
  window_graphicdhgrcolortext80 = BuildGraphicDHGRColorText80Window(param->hInstance,ID_MENU_VIEW_GRAPHICDHGRCOLORTEXT80COL);
  if(window_graphicdhgrcolortext80 == NULL)
    return;
}


/**********************************************************************************************/
/*  MenuViewGraphicDHGRMono560() :  On vient de slectionner GraphicDHGRMono560 du menu View. */
/**********************************************************************************************/
static void MenuViewGraphicDHGRMono560(struct window_main *window_main)
{
  struct window_graphicdhgrmono *window_graphicdhgrmono;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre GraphicDHGRMono (On peut avoir plusieurs fentres GraphicDHGRMono) **/
  window_graphicdhgrmono = BuildGraphicDHGRMonoWindow(param->hInstance,ID_MENU_VIEW_GRAPHICDHGRMONO560);
  if(window_graphicdhgrmono == NULL)
    return;
}


/*******************************************************************************************************/
/*  MenuViewGraphicDHGRMonoText80Col() :  On vient de slectionner GraphicDHGRMonoText80 du menu View. */
/*******************************************************************************************************/
static void MenuViewGraphicDHGRMonoText80Col(struct window_main *window_main)
{
  struct window_graphicdhgrmonotext80 *window_graphicdhgrmonotext80;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre GraphicDHGRMonoText80 (On peut avoir plusieurs fentres GraphicDHGRMonoText80) **/
  window_graphicdhgrmonotext80 = BuildGraphicDHGRMonoText80Window(param->hInstance,ID_MENU_VIEW_GRAPHICDHGRMONOTEXT80COL);
  if(window_graphicdhgrmonotext80 == NULL)
    return;
}


/******************************************************************************/
/*  MenuViewGraphicSHR() :  On vient de slectionner GraphicSHR du menu View. */
/******************************************************************************/
static void MenuViewGraphicSHR(struct window_main *window_main)
{
  struct window_graphicshr *window_graphicshr;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre GraphicSHR (On peut avoir plusieurs fentres GraphicSHR) **/
  window_graphicshr = BuildGraphicSHRWindow(param->hInstance,ID_MENU_VIEW_GRAPHICSHR);
  if(window_graphicshr == NULL)
    return;
}


/************************************************************************************/
/*  MenuViewGraphicScreen() :  On vient de slectionner GraphicScreen du menu View. */
/************************************************************************************/
static void MenuViewGraphicScreen(struct window_main *window_main)
{
  struct window_graphicscreen *window_graphicscreen;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_GRAPHICSCREEN,&window_graphicscreen,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre GraphicScreen Register  la fois **/
  if(window_graphicscreen == NULL)
    {
      /** Cration de la fentre Graphic Screen **/
      window_graphicscreen = BuildGraphicScreenWindow(param->hInstance,ID_MENU_VIEW_GRAPHICSCREEN);
      if(window_graphicscreen == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_VIEW_GRAPHICSCREEN,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Graphic Screen **/
      SendMessage(window_graphicscreen->hwnd_graphicscreen,WM_CLOSE,0,0);
    }
}


/**************************************************************************/
/*  MenuViewRamSound() :  On vient de slectionner RamSound du menu View. */
/**************************************************************************/
static void MenuViewRamSound(struct window_main *window_main)
{
  struct window_ramsound *window_ramsound;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Ram Sound (On peut avoir plusieurs fentres Ram Sound) **/
  window_ramsound = BuildRamSoundWindow(param->hInstance,ID_MENU_VIEW_RAMSOUND);
  if(window_ramsound == NULL)
    return;
}


/********************************************************************************/
/*  MenuViewDOCRegister() :  On vient de slectionner DocRegister du menu View. */
/********************************************************************************/
static void MenuViewDOCRegister(struct window_main *window_main)
{
  struct window_docregister *window_docregister;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_DOCREGISTER,&window_docregister,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre DOC Register  la fois **/
  if(window_docregister == NULL)
    {
      /** Cration de la fentre DOC Register **/
      window_docregister = BuildDOCRegisterWindow(param->hInstance,ID_MENU_VIEW_DOCREGISTER);
      if(window_docregister == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_VIEW_DOCREGISTER,TRUE);
    }
  else
    {
      /** Fermeture de la fentre DOC Register **/
      SendMessage(window_docregister->hwnd_docregister,WM_CLOSE,0,0);
    }
}


/******************************************************************************/
/*  MenuViewBatteryRam() :  On vient de slectionner BatteryRam du menu View. */
/******************************************************************************/
static void MenuViewBatteryRam(struct window_main *window_main)
{
  struct window_batteryram *window_batteryram;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_BATTERYRAM,&window_batteryram,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Battery Ram  la fois **/
  if(window_batteryram == NULL)
    {
      /** Cration de la fentre Battery Ram **/
      window_batteryram = BuildBatteryRamWindow(param->hInstance,ID_MENU_VIEW_BATTERYRAM);
      if(window_batteryram == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_VIEW_BATTERYRAM,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Battery Ram **/
      SendMessage(window_batteryram->hwnd_batteryram,WM_CLOSE,0,0);
    }
}


/******************************************************************************************/
/*  MenuViewCodeDisassembler() :  On vient de slectionner CodeDisassembler du menu View. */
/******************************************************************************************/
static void MenuViewCodeDisassembler(struct window_main *window_main)
{
  struct window_disassembler *window_disassembler;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Disassembler (On peut avoir plusieurs fentres Disassembler) **/
  window_disassembler = BuildDisassemblerWindow(param->hInstance,ID_MENU_VIEW_CODEDISASSEMBLER,0xFFFFFFFF);
  if(window_disassembler == NULL)
    return;
}


/****************************************************************************/
/*  MenuViewMemoryMap() :  On vient de slectionner MemoryMap du menu View. */
/****************************************************************************/
static void MenuViewMemoryMap(struct window_main *window_main)
{
  struct window_memorymap *window_memorymap;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MEMORYMAP,&window_memorymap,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Memory Map  la fois **/
  if(window_memorymap == NULL)
    {
      /** Cration de la fentre Memory Map **/
      window_memorymap = BuildMemoryMapWindow(param->hInstance,ID_MENU_VIEW_MEMORYMAP);
      if(window_memorymap == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_VIEW_MEMORYMAP,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Memory Map **/
      SendMessage(window_memorymap->hwnd_memorymap,WM_CLOSE,0,0);
    }
}


/****************************************************************************/
/*  MenuViewInternals() :  On vient de slectionner Internals du menu View. */
/****************************************************************************/
static void MenuViewInternals(struct window_main *window_main)
{
  struct window_internals *window_internals;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_INTERNALS,&window_internals,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Apple IIgs Internals  la fois **/
  if(window_internals == NULL)
    {
      /** Cration de la fentre Apple IIgs Internals **/
      window_internals = BuildInternalsWindow(param->hInstance,ID_MENU_VIEW_INTERNALS);
      if(window_internals == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_VIEW_INTERNALS,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Apple IIgs Internals **/
      SendMessage(window_internals->hwnd_internals,WM_CLOSE,0,0);
    }
}


/**********************************************************************/
/*  MenuCodeSymbol() :  On vient de slectionner Symbol du menu Code. */
/**********************************************************************/
static void MenuCodeSymbol(struct window_main *window_main)
{
  struct window_symbol *window_symbol;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_SYMBOL,&window_symbol,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Symbol  la fois **/
  if(window_symbol == NULL)
    {
      /** Cration de la fentre Symbol **/
      window_symbol = BuildSymbolWindow(param->hInstance,ID_MENU_CODE_SYMBOL);
      if(window_symbol == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_CODE_SYMBOL,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Symbol **/
      SendMessage(window_symbol->hwnd_symbol,WM_CLOSE,0,0);
    }
}


/******************************************************************************/
/*  MenuCodeBreakpoint() :  On vient de slectionner Breakpoint du menu Code. */
/******************************************************************************/
static void MenuCodeBreakpoint(struct window_main *window_main)
{
  struct window_breakpoint *window_breakpoint;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_BREAKPOINT,&window_breakpoint,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Breakpoint  la fois **/
  if(window_breakpoint == NULL)
    {
      /** Cration de la fentre Breakpoint **/
      window_breakpoint = BuildBreakpointWindow(param->hInstance,ID_MENU_CODE_BREAKPOINT);
      if(window_breakpoint == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_CODE_BREAKPOINT,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Breakpoint **/
      SendMessage(window_breakpoint->hwnd_breakpoint,WM_CLOSE,0,0);
    }
}


/************************************************************************************************/
/*  MenuExtraFunctionStatistics() :  On vient de slectionner FunctionStatistics du menu Extra. */
/************************************************************************************************/
static void MenuExtraFunctionStatistics(struct window_main *window_main)
{
  struct window_statistic *window_statistic;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_STATISTIC,&window_statistic,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Statistic  la fois **/
  if(window_statistic == NULL)
    {
      /** Cration de la fentre Statistic **/
      window_statistic = BuildStatisticWindow(param->hInstance,ID_MENU_EXTRA_STATISTIC);
      if(window_statistic == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_EXTRA_STATISTIC,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Statistic **/
      SendMessage(window_statistic->hwnd_statistic,WM_CLOSE,0,0);
    }
}


/********************************************************************************************/
/*  MenuExtraFunctionProfiler() :  On vient de slectionner FunctionProfiler du menu Extra. */
/********************************************************************************************/
static void MenuExtraFunctionProfiler(struct window_main *window_main)
{
  struct window_profile *window_profile;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_PROFILE,&window_profile,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre Profile  la fois **/
  if(window_profile == NULL)
    {
      /** Cration de la fentre Profile **/
      window_profile = BuildProfileWindow(param->hInstance,ID_MENU_EXTRA_PROFILE);
      if(window_profile == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_EXTRA_PROFILE,TRUE);
    }
  else
    {
      /** Fermeture de la fentre Profile **/
      SendMessage(window_profile->hwnd_profile,WM_CLOSE,0,0);
    }
}


/*****************************************************************/
/*  MenuAbout() :  On vient de slectionner About du menu About. */
/*****************************************************************/
void MenuAbout(struct window_main *window_main)
{
  struct window_about *window_about;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_ABOUT,&window_about,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** On ne peut avoir qu'une seule fentre About  la fois **/
  if(window_about == NULL)
    {
      /** Cration de la fentre About **/
      window_about = BuildAboutWindow(param->hInstance,ID_MENU_ABOUT);
      if(window_about == NULL)
        return;

      /* Coche le menu */
      CheckMainWindowMenuItem(ID_MENU_ABOUT,TRUE);
    }
  else
    {
      /** Fermeture de la fentre About **/
      SendMessage(window_about->hwnd_about,WM_CLOSE,0,0);
    }
}


/*****************************************************************/
/*  MenuReset() :  On va faire un Reset global de l'application. */
/*****************************************************************/
static void MenuReset(struct window_main *window_main)
{
  int i;
//  char buffer_log[1024];
  struct window_breakpoint *window_breakpoint;
  struct window_symbol *window_symbol;
  struct window_statistic *window_statistic;
  struct window_profile *window_profile;
  struct window_quickwatch *window_quickwatch;
  int tab_bank_id[10]    = {IDC_EDIT_RAM_BANK1,IDC_EDIT_RAM_BANK2,IDC_EDIT_RAM_BANK3,IDC_EDIT_RAM_BANK4,IDC_EDIT_RAM_BANK5,IDC_EDIT_RAM_BANK6,IDC_EDIT_RAM_BANK7,IDC_EDIT_RAM_BANK8,IDC_EDIT_RAM_BANK9,IDC_EDIT_RAM_BANK10};
  int tab_address_id[10] = {IDC_EDIT_RAM_ADDRESS1,IDC_EDIT_RAM_ADDRESS2,IDC_EDIT_RAM_ADDRESS3,IDC_EDIT_RAM_ADDRESS4,IDC_EDIT_RAM_ADDRESS5,IDC_EDIT_RAM_ADDRESS6,IDC_EDIT_RAM_ADDRESS7,IDC_EDIT_RAM_ADDRESS8,IDC_EDIT_RAM_ADDRESS9,IDC_EDIT_RAM_ADDRESS10};
  int tab_name_id[10]    = {IDC_EDIT_LABEL1,IDC_EDIT_LABEL2,IDC_EDIT_LABEL3,IDC_EDIT_LABEL4,IDC_EDIT_LABEL5,IDC_EDIT_LABEL6,IDC_EDIT_LABEL7,IDC_EDIT_LABEL8,IDC_EDIT_LABEL9,IDC_EDIT_LABEL10};
  int tab_value_id[10]   = {IDC_EDIT_VALUE1,IDC_EDIT_VALUE2,IDC_EDIT_VALUE3,IDC_EDIT_VALUE4,IDC_EDIT_VALUE5,IDC_EDIT_VALUE6,IDC_EDIT_VALUE7,IDC_EDIT_VALUE8,IDC_EDIT_VALUE9,IDC_EDIT_VALUE10};
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* On va se dtacher de KEGS */
  if(param->kegs_connection_status == CONNECTION_STATUS_OPEN)
    MenuFileDetachKegs(window_main);

  /* Reset Stop Condition */
  window_main->stop_condition = 0;

  /* On va faire un Reset de la fentre Main */
  SendMessage(window_main->hwnd_main,HM_WINDOW_MAIN_RESET,0,0);

  /** RAZ Symbol : La Table des Symbol est rattache  la fentre Main pour ne pas la perdre **/
  my_Memory(MEMORY_GET_WINDOW_SYMBOL,&window_symbol,NULL);
  if(window_symbol != NULL)
    ClearSymbolTable(window_symbol);
  else
    {
      mem_free_symbol_table(window_main->current_symbol_table);
      window_main->current_symbol_table = NULL;
    }

  /** RAZ Breakpoint : La Table des Breakpoint est rattache  la fentre Main pour ne pas la perdre **/
  my_Memory(MEMORY_GET_WINDOW_BREAKPOINT,&window_breakpoint,NULL);
  if(window_breakpoint != NULL)
    ClearBreakpointTable(window_breakpoint);
  else
    {
      mem_free_breakpoint_table(window_main->current_breakpoint_table);
      window_main->current_breakpoint_table = NULL;
    }

  /** RAZ QuickWatch : La Table des QuickWatch est rattache  la fentre Main pour ne pas la perdre **/
  my_Memory(MEMORY_GET_WINDOW_QUICKWATCH,&window_quickwatch,NULL);
  for(i=0; i<10; i++)
    {
      /* RAZ Data */
      memset(&window_main->tab_quickwatch_data[i],0,sizeof(struct quickwatch_data));
      window_main->tab_quickwatch_data[i].ram_type               = QUICKWATCH_RAMTYPE_RAM;
      window_main->tab_quickwatch_data[i].is_data_error          = 1;
      window_main->tab_quickwatch_data[i].combo_length_alignment = QUICKWATCH_COMBO_8;
      window_main->tab_quickwatch_data[i].data_length            = QUICKWATCH_LENGTH_8;
      window_main->tab_quickwatch_data[i].data_alignment         = QUICKWATCH_ALIGNMENT_INTEL;

      /* RAZ Interface */
      if(window_quickwatch != NULL)
        {
          UpdateQuickWatchRamType(window_quickwatch,i,window_main->tab_quickwatch_data[i].ram_type,0);
          SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[i],CB_SETCURSEL,window_main->tab_quickwatch_data[i].combo_length_alignment,0);
          SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_bank_id[i]),"");
          SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_address_id[i]),"");
          SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_name_id[i]),"");
          SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_value_id[i]),"");
        }
    }

  /** RAZ Statistic : Nettoyage de la La Table des Statistic **/
  my_Memory(MEMORY_GET_WINDOW_STATISTIC,&window_statistic,NULL);
  if(window_statistic != NULL)
    ClearStatisticTable(window_statistic);
  else
    {
      mem_free_statistic_table(window_main->current_statistic_table);
      window_main->current_statistic_table = NULL;
    }

  /** RAZ Profile : La Table des Profile est rattache  la fentre Main pour ne pas la perdre **/
  my_Memory(MEMORY_GET_WINDOW_PROFILE,&window_profile,NULL);
  if(window_profile != NULL)
    ClearProfileTable(window_profile);
  else
    {
      mem_free_profile_table(window_main->current_profile_table);
      window_main->current_profile_table = NULL;
    }

  /** MAJ des Menus **/
  UpdateWindowMainMenuItemStatus();

  /** MAJ de la StatusBar **/
  UpdateWindowMainStatusBar();

  /** On charge les fichiers Preference : Breakpoint / Quickwatch / Symbol / SystemCall **/
  LoadInitPreferenceFiles();

  /** On met  jour les fentres **/
  my_Memory(MEMORY_GET_WINDOW_SYMBOL,&window_symbol,NULL);
  if(window_symbol != NULL)
    RefreshSymbolWindow(window_symbol);
  my_Memory(MEMORY_GET_WINDOW_BREAKPOINT,&window_breakpoint,NULL);
  if(window_breakpoint != NULL)
    RefreshBreakpointWindow(window_breakpoint);
  my_Memory(MEMORY_GET_WINDOW_QUICKWATCH,&window_quickwatch,NULL);
  if(window_quickwatch != NULL)
    RefreshQuickWatchWindow(window_quickwatch);

  /* Sauvegarde des fichiers *
  param->buffer_ipc_log[param->buffer_ipc_offset] = '\0';
  SaveTextFile("c:\\AppleIIgs\\WindowIPC_log.txt",param->buffer_ipc_log,buffer_log);
  param->buffer_request_log[param->buffer_request_offset] = '\0';
  SaveTextFile("c:\\AppleIIgs\\WindowRequest_log.txt",param->buffer_request_log,buffer_log);
  param->buffer_request_offset = 0; */
  param->buffer_ipc_offset = 0;
  param->buffer_request_offset = 0;
}

/*************************************************************************/
