/**************************************************************************/
/*                                                                        */
/*  Window_MemoryMap.h :  Header de gestion de la fentre MemoryMap.      */
/*                                                                        */
/**************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Septembre 2018  */
/**************************************************************************/

/* Un message destin  la fentre Memory Map pour demander la maj de l'affichage */
#define HM_MEMORYMAP_REFRESHWINDOW      (WM_USER + 24)
#define IDC_COMBOBOX_HANDLE             1038

#define MEMORYMAP_REFRESH_AUTOMATIC    0
#define MEMORYMAP_REFRESH_MANUAL       1

#define MEMORYMAP_EDIT_FONT_HEIGHT    14   /* Taille de la Fonte en Pixel */

#define MEMORYMAP_DIALOGBOX_WIDTH    655   /* Largeur minimale en Pixel de la fentre */

#define MEMORYMAP_BITMAP_WIDTH        32   /* Taille Bitmap d'un banc mmoire */
#define MEMORYMAP_BITMAP_WIDTH_8BIT  256
#define MEMORYMAP_BITMAP_HEIGHT      256

#define MEMORYMAP_BITMAP_BORDER        5   /* Espace entre les Bitmap */

#define MEMORYMAP_TEXT_HEIGHT          8   /* Taille du Texte sous les BitMap */
#define MEMORYMAP_TEXT_BORDER_HEIGHT   3   /* Espace Sous / En Dessous du Texte sous les BitMap */

#define MEMORYMAP_GROUP_BORDER        10   /* Espace entre les Group Box */
#define MEMORYMAP_GROUP_MARGIN         3   /* Marge interne des Group Box */

#define MEMORYMAP_BITMAP_COLOR_SIZE   16   /* Taille du carr de la couleur */
#define MEMORYMAP_COLOR_BORDER_SIZE    8   /* Espace Vertical entre les couleurs */

#define MEMORYMAP_LABEL_WIDTH        150   /* Largeur de la Zone Libell / Couleur */

#define MEMORYMAP_EMPTY_COLOR          MAKECOLOR(200,200,200)      /* Gris Clair      : Zone mmoire */
#define MEMORYMAP_DATA_COLOR           MAKECOLOR(150,150,150)      /* Gris Moyen      : Zone mmoire o la data n'est pas  0 */
#define MEMORYMAP_SELECTED_COLOR       MAKECOLOR(20,20,20)         /* Gris Trs Fonc : Zone mmoire de la data Selectionne */
#define MEMORYMAP_FRIEND_COLOR         MAKECOLOR(80,80,80)         /* Gris Fonc      : Zone mmoire des 'amis' de la Data Slectionne */

#define MEMORYMAP_OPERATION_READ       0x01   /* Read */
#define MEMORYMAP_OPERATION_WRITE      0x02   /* Write */
#define MEMORYMAP_OPERATION_EXECUTE    0x04   /* Execute */
#define MEMORYMAP_OPERATION_SELECTED   0x08   /* Selected Handle */
#define MEMORYMAP_OPERATION_FRIEND     0x10   /* Selected Handle Friend (Same ID, Different Address) */

struct window_memorymap
{
  HWND hwnd_memorymap;
  HFONT hFont;
  HFONT hFont_courier;
  HWND hwnd_combo_handle;       /* ComboList */

  DWORD thread_id;              /* Le ThreadID va permettre de diffrencier les diffrentes fentres */

  /* Taille de la mmoire */
  int nb_fastram_bank;          /* Apple IIe=2 */
  int nb_rom_bank;              /* Apple IIe=0, Rom01=2, Rom03=4 */
  
  int nb_fastram_bank_display;  /* Nombre de bank Fast ram affich dans la fentre */
  int current_fastram_bank;

  /** Libells sous les Bitmap **/
  HWND hwnd_fastram_label[16];
  HWND hwnd_slowram_label[2];
  HWND hwnd_rom_label[4];
  HWND hwnd_docram_label;

  /** Graphical element attributes **/
  HDC hDC;
  HDC hCompatibleDC;
  /* Fast Ram */
  HBITMAP hBitmap_fastram[224];        /* 00 01 ... DF */
  DWORD *bitmap_data_fastram[224];
  /* Slow Ram */
  HBITMAP hBitmap_slowram[2];          /* E0 E1 */
  DWORD *bitmap_data_slowram[2];
  /* Rom */
  HBITMAP hBitmap_rom[4];              /* [FC FD] FE FF */
  DWORD *bitmap_data_rom[4];
  /* DOC Ram */
  HBITMAP hBitmap_docram;
  DWORD *bitmap_data_docram;

  /* 7 Couleurs pour les libells */
  HBITMAP hBitmap_color[7];
  DWORD *bitmap_data_color[7];
  
  /* Image affichant les 3 couleurs */
  HBITMAP hBitmap_color_circle;
  DWORD *bitmap_data_color_circle;  
  struct picture_true *color_circle_picture;

  /* Handle slectionn dans la ComboBox  */
  int selected_handle;
  int nb_handle;
  struct memorymap_handle **tab_handle;

  /* Mode de rafraichissement de la fentre */
  int refresh_mode;        /* MEMORYMAP_REFRESH_XXX */

  /** Position des Group Box dans la fentre **/
  /* Position / Taille du Control Group Box Fast Ram */
  int group_fastram_x;
  int group_fastram_y;
  int group_fastram_width;
  int group_fastram_height;

  /* Position / Taille du Control Group Box Slow Ram */
  int group_slowram_x;
  int group_slowram_y;
  int group_slowram_width;
  int group_slowram_height;
  
  /* Position / Taille du Control Group Box Rom */
  int group_rom_x;
  int group_rom_y;
  int group_rom_width;
  int group_rom_height;

  /* Position / Taille du Control Group Box DOC Ram */
  int group_docram_x;
  int group_docram_y;
  int group_docram_width;
  int group_docram_height;

  /** Position des Bitmap dans la fentre **/
  /* Position / Taille des Bitmaps Fast Ram */
  int bitmap_fastram_x[64];
  int bitmap_fastram_y[64];

  /* Position / Taille des Bitmaps Slow Ram */
  int bitmap_slowram_x[2];
  int bitmap_slowram_y[2];

  /* Position / Taille des Bitmaps Rom */
  int bitmap_rom_x[4];
  int bitmap_rom_y[4];

  /* Position / Taille de la Bitmap DOC Ram */
  int bitmap_docram_x;
  int bitmap_docram_y;

  /* Position / Taille des Bitmaps Color */
  int bitmap_color_x[7];
  int bitmap_color_y[7];

  /* Taille de la Bitmap Colors */
  int bitmap_color_circle_width;
  int bitmap_color_circle_height;
  
  /* Position de la Bitmap Colors */
  int bitmap_color_circle_x;
  int bitmap_color_circle_y;

  /* Dimension de la fentre = Title + Zone Texte */
  int title_height;
  int window_width;
  int window_height;

  /* Buffer */
  char buffer_log[BUFFER_SIZE];

  /* Notification */
  struct notification *current_notification;
};

struct memorymap_handle
{
  DWORD address;
  DWORD length;

  WORD id;              /* 0001 = RAM Disk */

  char type[256];
  char information[512];

  struct memorymap_handle *next;
};

struct window_memorymap *BuildMemoryMapWindow(HINSTANCE,int);
void mem_free_window_memorymap(struct window_memorymap *);

/***************************************************************/
