/*********************************************************************/
/*                                                                   */
/*  Window_Modify.c :  Module de gestion de la fentre Modify.       */
/*                                                                   */
/*********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Janv 2019  */
/*********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Dc_Request.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Modify.h"

static DWORD BuildModifyWindowThread(void *);
LRESULT CALLBACK ProcHandlerModifyWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateModifyChildWindows(HWND,struct window_modify *);
LRESULT CALLBACK subEditModifyProcControl(HWND,UINT,WPARAM,LPARAM);
static void UpdateModifyMode(struct window_modify *,int);
static void UpdateModifyDataLengthAlignment(struct window_modify *,int,int);
static void UpdateModifyRamType(struct window_modify *,int);
static void ResizeModifyWindow(struct window_modify *);
static void ModifyMemory(struct window_modify *);
static struct modify_value *DecodeModifyValueLine(char *,struct window_modify *);
static void mem_free_valuelist(struct modify_value *);
static void DeleteModifyChildWindows(struct window_modify *);


/**************************************************************************/
/*  BuildModifyWindow() :  Construction / Affichage de la fentre Modify. */
/**************************************************************************/
struct window_modify *BuildModifyWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_modify *window_modify;

  /** Allocation de la structure window_modify **/
  window_modify = (struct window_modify *) calloc(1,sizeof(struct window_modify));
  if(window_modify == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_MODIFY,window_modify,NULL);

  /* Lance un thread pour grer la fentre Modify */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildModifyWindowThread,window_modify,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_modify);
}


/************************************************************************************/
/*  BuildModifyWindowThread() :  Cration de la fentre Modify via un Thread ddi. */
/************************************************************************************/
static DWORD BuildModifyWindowThread(void *window_modify_param)
{
  HWND hwndModify;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_modify *window_modify;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_modify = (struct window_modify *) window_modify_param;
  window_modify->thread_id = GetCurrentThreadId();         /* Le ThreadID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Modify **/
  hwndModify = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_MODIFY),window_main->hwnd_main,(DLGPROC)ProcHandlerModifyWindow);
  if(hwndModify == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndModify,SW_SHOWDEFAULT);
  UpdateWindow(hwndModify);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Modify...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***********************************************************************/
/*  ProcHandlerModifyWindow() :  Gestion des vnements de la fentre. */
/***********************************************************************/
LRESULT CALLBACK ProcHandlerModifyWindow(HWND hwndModify, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  LRESULT result;
  struct window_modify *window_modify;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_MODIFY,&window_modify,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateModifyChildWindows(hwndModify,window_modify);
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de modification **/
            case IDC_RADIO_BYTE :
              if(HIWORD(wParam) == BN_CLICKED && window_modify->mode != MODIFY_MODE_BYTE)
                UpdateModifyMode(window_modify,MODIFY_MODE_BYTE);
              break;

            case IDC_RADIO_PATTERN :
              if(HIWORD(wParam) == BN_CLICKED && window_modify->mode != MODIFY_MODE_PATTERN)
                UpdateModifyMode(window_modify,MODIFY_MODE_PATTERN);
              break;

            /** Bouton Radio : On change le type de la Ram **/
            case IDC_RADIO_RAM :
              if(HIWORD(wParam) == BN_CLICKED && window_modify->ram_type != MODIFY_RAMTYPE_RAM)
                UpdateModifyRamType(window_modify,MODIFY_RAMTYPE_RAM);
              break;
            case IDC_RADIO_DOCRAM :
              if(HIWORD(wParam) == BN_CLICKED && window_modify->ram_type != MODIFY_RAMTYPE_DOCRAM)
                UpdateModifyRamType(window_modify,MODIFY_RAMTYPE_DOCRAM);
              break;

            /** Slection dans les ComboBox Data Length Alignment **/
            case IDC_COMBOBOX_MODIFY_DATALENGTH_1 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_modify->hwnd_combo_length_alignment1,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_modify->tab_pattern[0].combo_length_alignment)
                      UpdateModifyDataLengthAlignment(window_modify,1,(int)result);
                }
              break;

            case IDC_COMBOBOX_MODIFY_DATALENGTH_2 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_modify->hwnd_combo_length_alignment2,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_modify->tab_pattern[1].combo_length_alignment)
                      UpdateModifyDataLengthAlignment(window_modify,2,(int)result);
                }
              break;

            case IDC_COMBOBOX_MODIFY_DATALENGTH_3 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_modify->hwnd_combo_length_alignment3,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_modify->tab_pattern[2].combo_length_alignment)
                      UpdateModifyDataLengthAlignment(window_modify,3,(int)result);
                }
              break;

            case IDC_COMBOBOX_MODIFY_DATALENGTH_4 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_modify->hwnd_combo_length_alignment4,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_modify->tab_pattern[3].combo_length_alignment)
                      UpdateModifyDataLengthAlignment(window_modify,4,(int)result);
                }
              break;

            case IDC_COMBOBOX_MODIFY_DATALENGTH_5 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_modify->hwnd_combo_length_alignment5,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_modify->tab_pattern[4].combo_length_alignment)
                      UpdateModifyDataLengthAlignment(window_modify,5,(int)result);
                }
              break;

            /** Bouton de modification de la mmoire **/
            case IDC_BUTTON_MODIFY :
              ModifyMemory(window_modify);
              break;
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Modify */
        DestroyWindow(hwndModify);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteModifyChildWindows(window_modify);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_MODIFY,window_modify,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***************************************************************************************************/
/*  CreateModifyChildWindows() : Crer les fentres enfants et remplir la structure window_modify. */
/***************************************************************************************************/
static int CreateModifyChildWindows(HWND hwndModify, struct window_modify *window_modify)
{
  int i;
  char *data_length_alignment[7] = {"8 Bit","16 Bit / Intel","24 Bit / Intel","32 Bit / Intel","16 Bit / Motorola","24 Bit / Motorola","32 Bit / Motorola"};
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_modify->hwnd_modify = hwndModify;

  /* Modify Mode : Byte */
  UpdateModifyMode(window_modify,MODIFY_MODE_BYTE);

  /* Ram Type : RAM */
  UpdateModifyRamType(window_modify,MODIFY_RAMTYPE_RAM);

  /** Surchage des Control Edit pour capter les RETURN lors des saisies **/
  window_modify->oldEditProcBank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_RAM_BANK),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_RAM_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcData = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_DATA),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcRepeat = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_REPEAT),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcName1 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_NAME1),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcName2 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_NAME2),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcName3 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_NAME3),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcName4 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_NAME4),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcName5 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_NAME5),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcValue1 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_VALUE1),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcValue2 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_VALUE2),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcValue3 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_VALUE3),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcValue4 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_VALUE4),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcValue5 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_VALUE5),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcIncrement1 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_INCREMENT1),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcIncrement2 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_INCREMENT2),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcIncrement3 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_INCREMENT3),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcIncrement4 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_INCREMENT4),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);
  window_modify->oldEditProcIncrement5 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndModify,IDC_EDIT_PATTERN_INCREMENT5),GWLP_WNDPROC,(LONG_PTR)subEditModifyProcControl);  

  /** Cration des ComboBox Data Length Alignment **/
  window_modify->hwnd_combo_length_alignment1 = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                               150,179,120,250,hwndModify,(HMENU)IDC_COMBOBOX_MODIFY_DATALENGTH_1,param->hInstance,0);
  window_modify->hwnd_combo_length_alignment2 = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                               150,208,120,250,hwndModify,(HMENU)IDC_COMBOBOX_MODIFY_DATALENGTH_2,param->hInstance,0);
  window_modify->hwnd_combo_length_alignment3 = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                               150,238,120,250,hwndModify,(HMENU)IDC_COMBOBOX_MODIFY_DATALENGTH_3,param->hInstance,0);
  window_modify->hwnd_combo_length_alignment4 = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                               150,267,120,250,hwndModify,(HMENU)IDC_COMBOBOX_MODIFY_DATALENGTH_4,param->hInstance,0);
  window_modify->hwnd_combo_length_alignment5 = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                               150,296,120,250,hwndModify,(HMENU)IDC_COMBOBOX_MODIFY_DATALENGTH_5,param->hInstance,0);
  for(i=0; i<7; i++)
    {
      SendMessage(window_modify->hwnd_combo_length_alignment1,CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
      SendMessage(window_modify->hwnd_combo_length_alignment2,CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
      SendMessage(window_modify->hwnd_combo_length_alignment3,CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
      SendMessage(window_modify->hwnd_combo_length_alignment4,CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
      SendMessage(window_modify->hwnd_combo_length_alignment5,CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
    }

  /** 1res valeurs **/
  for(i=0; i<5; i++)
    {
      window_modify->tab_pattern[0].combo_length_alignment = MODIFY_COMBO_8;
      window_modify->tab_pattern[i].data_length = MODIFY_LENGTH_8;
      window_modify->tab_pattern[i].data_alignment = MODIFY_ALIGNMENT_INTEL;
    }
  SendMessage(window_modify->hwnd_combo_length_alignment1,CB_SETCURSEL,window_modify->tab_pattern[0].combo_length_alignment,0);
  SendMessage(window_modify->hwnd_combo_length_alignment2,CB_SETCURSEL,window_modify->tab_pattern[1].combo_length_alignment,0);
  SendMessage(window_modify->hwnd_combo_length_alignment3,CB_SETCURSEL,window_modify->tab_pattern[2].combo_length_alignment,0);
  SendMessage(window_modify->hwnd_combo_length_alignment4,CB_SETCURSEL,window_modify->tab_pattern[3].combo_length_alignment,0);
  SendMessage(window_modify->hwnd_combo_length_alignment5,CB_SETCURSEL,window_modify->tab_pattern[4].combo_length_alignment,0);

  /** Cration de la Police de caractre + Affectation de la police au Control Combo **/
  window_modify->hFont_conbo = CreateFont(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");
  SendMessage(window_modify->hwnd_combo_length_alignment1,WM_SETFONT,(WPARAM)window_modify->hFont_conbo,TRUE);
  SendMessage(window_modify->hwnd_combo_length_alignment2,WM_SETFONT,(WPARAM)window_modify->hFont_conbo,TRUE);
  SendMessage(window_modify->hwnd_combo_length_alignment3,WM_SETFONT,(WPARAM)window_modify->hFont_conbo,TRUE);
  SendMessage(window_modify->hwnd_combo_length_alignment4,WM_SETFONT,(WPARAM)window_modify->hFont_conbo,TRUE);
  SendMessage(window_modify->hwnd_combo_length_alignment5,WM_SETFONT,(WPARAM)window_modify->hFont_conbo,TRUE);

  /* OK */
  return(0);
}


/***************************************************************************************/
/*  subEditModifyProcControl() :  Rcupre le RETURN pour valider les donnes saisies. */
/***************************************************************************************/
LRESULT CALLBACK subEditModifyProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_modify *window_modify;
  my_Memory(MEMORY_GET_WINDOW_MODIFY,&window_modify,NULL);

  /* Protection */
  if(window_modify == NULL)
    return(0);

  /** Identifie le Control **/
  if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_RAM_BANK) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_RAM_ADDRESS) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_DATA) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_REPEAT) == hwnd_control)
    index = 4;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_NAME1) == hwnd_control)
    index = 5;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_NAME2) == hwnd_control)
    index = 6;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_NAME3) == hwnd_control)
    index = 7;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_NAME4) == hwnd_control)
    index = 8;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_NAME5) == hwnd_control)
    index = 9;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_VALUE1) == hwnd_control)
    index = 10;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_VALUE2) == hwnd_control)
    index = 11;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_VALUE3) == hwnd_control)
    index = 12;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_VALUE4) == hwnd_control)
    index = 13;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_VALUE5) == hwnd_control)
    index = 14;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_INCREMENT1) == hwnd_control)
    index = 15;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_INCREMENT2) == hwnd_control)
    index = 16;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_INCREMENT3) == hwnd_control)
    index = 17;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_INCREMENT4) == hwnd_control)
    index = 18;
  else if(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_INCREMENT5) == hwnd_control)
    index = 19;
  else
    return(0);      /* Pas trouv */

  /** Dtecte la touche RETURN pour valider la donne **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_modify->hwnd_modify);
    }
  else if(index == 1)
    return(CallWindowProc(window_modify->oldEditProcBank,hwnd_control,msg,wParam,lParam));
  else if(index == 2)
    return(CallWindowProc(window_modify->oldEditProcAddress,hwnd_control,msg,wParam,lParam));
  else if(index == 3)
    return(CallWindowProc(window_modify->oldEditProcData,hwnd_control,msg,wParam,lParam));
  else if(index == 4)
    return(CallWindowProc(window_modify->oldEditProcRepeat,hwnd_control,msg,wParam,lParam));
  else if(index == 5)
    return(CallWindowProc(window_modify->oldEditProcName1,hwnd_control,msg,wParam,lParam));
  else if(index == 6)
    return(CallWindowProc(window_modify->oldEditProcName2,hwnd_control,msg,wParam,lParam));
  else if(index == 7)
    return(CallWindowProc(window_modify->oldEditProcName3,hwnd_control,msg,wParam,lParam));
  else if(index == 8)
    return(CallWindowProc(window_modify->oldEditProcName4,hwnd_control,msg,wParam,lParam));
  else if(index == 9)
    return(CallWindowProc(window_modify->oldEditProcName5,hwnd_control,msg,wParam,lParam));
  else if(index == 10)
    return(CallWindowProc(window_modify->oldEditProcValue1,hwnd_control,msg,wParam,lParam));
  else if(index == 11)
    return(CallWindowProc(window_modify->oldEditProcValue2,hwnd_control,msg,wParam,lParam));    
  else if(index == 12)
    return(CallWindowProc(window_modify->oldEditProcValue3,hwnd_control,msg,wParam,lParam));
  else if(index == 13)
    return(CallWindowProc(window_modify->oldEditProcValue4,hwnd_control,msg,wParam,lParam));    
  else if(index == 14)
    return(CallWindowProc(window_modify->oldEditProcValue5,hwnd_control,msg,wParam,lParam));
  else if(index == 15)
    return(CallWindowProc(window_modify->oldEditProcIncrement1,hwnd_control,msg,wParam,lParam));    
  else if(index == 16)
    return(CallWindowProc(window_modify->oldEditProcIncrement2,hwnd_control,msg,wParam,lParam));  
  else if(index == 17)
    return(CallWindowProc(window_modify->oldEditProcIncrement3,hwnd_control,msg,wParam,lParam));      
  else if(index == 18)
    return(CallWindowProc(window_modify->oldEditProcIncrement4,hwnd_control,msg,wParam,lParam));  
  else if(index == 19)
    return(CallWindowProc(window_modify->oldEditProcIncrement5,hwnd_control,msg,wParam,lParam));       

  /* OK */
  return(0);
}


/****************************************************************************/
/*  UpdateModifyMode() :  MAJ du mode de modification de la fentre Modify. */
/****************************************************************************/
static void UpdateModifyMode(struct window_modify *window_modify, int mode)
{
  if(mode == MODIFY_MODE_BYTE)
    {
      /* Visuel */
      window_modify->mode = MODIFY_MODE_BYTE;
      SendDlgItemMessage(window_modify->hwnd_modify,IDC_RADIO_BYTE,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_modify->hwnd_modify,IDC_RADIO_PATTERN,BM_SETCHECK,0,0);

      /* Compacte la fentre */
      ResizeModifyWindow(window_modify);
      ShowWindow(GetDlgItem(window_modify->hwnd_modify,IDC_GROUP_PATTERN),SW_HIDE);
    }
  else
    {
      /* Visuel */
      window_modify->mode = MODIFY_MODE_PATTERN;
      SendDlgItemMessage(window_modify->hwnd_modify,IDC_RADIO_BYTE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_modify->hwnd_modify,IDC_RADIO_PATTERN,BM_SETCHECK,1,0);

      /* Expand la fentre */
      ResizeModifyWindow(window_modify);
      ShowWindow(GetDlgItem(window_modify->hwnd_modify,IDC_GROUP_PATTERN),SW_SHOW);      
    }
}


/*****************************************************************************************************/
/*  UpdateModifyDataLengthAlignment() :  MAJ du ComboBox Data Length Alignment de la fentre Modify. */
/*****************************************************************************************************/
static void UpdateModifyDataLengthAlignment(struct window_modify *window_modify, int combo_index, int data_length_alignment)
{
  /* MAJ de la valeur */
  window_modify->tab_pattern[combo_index-1].combo_length_alignment = data_length_alignment;

  /* Valeurs */
  if(data_length_alignment == MODIFY_COMBO_8)
    {
      window_modify->tab_pattern[combo_index-1].data_length = MODIFY_LENGTH_8;
      window_modify->tab_pattern[combo_index-1].data_alignment = MODIFY_ALIGNMENT_INTEL;
    }
  else if(data_length_alignment == MODIFY_COMBO_16_INTEL)
    {
      window_modify->tab_pattern[combo_index-1].data_length = MODIFY_LENGTH_16;
      window_modify->tab_pattern[combo_index-1].data_alignment = MODIFY_ALIGNMENT_INTEL;
    }
  else if(data_length_alignment == MODIFY_COMBO_24_INTEL)
    {
      window_modify->tab_pattern[combo_index-1].data_length = MODIFY_LENGTH_24;
      window_modify->tab_pattern[combo_index-1].data_alignment = MODIFY_ALIGNMENT_INTEL;
    }
  else if(data_length_alignment == MODIFY_COMBO_32_INTEL)
    {
      window_modify->tab_pattern[combo_index-1].data_length = MODIFY_LENGTH_32;
      window_modify->tab_pattern[combo_index-1].data_alignment = MODIFY_ALIGNMENT_INTEL;
    }
  else if(data_length_alignment == MODIFY_COMBO_16_MOTOROLA)
    {
      window_modify->tab_pattern[combo_index-1].data_length = MODIFY_LENGTH_16;
      window_modify->tab_pattern[combo_index-1].data_alignment = MODIFY_ALIGNMENT_MOTOROLA;
    }
  else if(data_length_alignment == MODIFY_COMBO_24_MOTOROLA)
    {
      window_modify->tab_pattern[combo_index-1].data_length = MODIFY_LENGTH_24;
      window_modify->tab_pattern[combo_index-1].data_alignment = MODIFY_ALIGNMENT_MOTOROLA;
    }
  else if(data_length_alignment == MODIFY_COMBO_32_MOTOROLA)
    {
      window_modify->tab_pattern[combo_index-1].data_length = MODIFY_LENGTH_32;
      window_modify->tab_pattern[combo_index-1].data_alignment = MODIFY_ALIGNMENT_MOTOROLA;
    }
}


/*******************************************************************************/
/*  UpdateModifyRamType() :  MAJ du Radio Bouton RamType de la fentre Modify. */
/*******************************************************************************/
static void UpdateModifyRamType(struct window_modify *window_modify, int ram_type)
{
  window_modify->ram_type = ram_type;
  if(ram_type == MODIFY_RAMTYPE_RAM)
    {
      /* Visuel */
      SendDlgItemMessage(window_modify->hwnd_modify,IDC_RADIO_RAM,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_modify->hwnd_modify,IDC_RADIO_DOCRAM,BM_SETCHECK,0,0);

      /* Enable BANK */
      EnableWindow(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_RAM_BANK),TRUE);
    }
  else
    {
      /* Visuel */
      SendDlgItemMessage(window_modify->hwnd_modify,IDC_RADIO_RAM,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_modify->hwnd_modify,IDC_RADIO_DOCRAM,BM_SETCHECK,1,0);

      /* Disable BANK */
      EnableWindow(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_RAM_BANK),FALSE);
    }
}


/***************************************************************/
/*  ResizeModifyWindow() : Resize Modify Window by given size. */
/***************************************************************/
static void ResizeModifyWindow(struct window_modify *window_modify)
{
  RECT window_rect, client_rect;
  int window_x, window_y, window_width, window_height, title_height;
   
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_modify->hwnd_modify,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_modify->hwnd_modify,&client_rect);
  title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Largeur de la fentre */
  window_width = window_rect.right - window_rect.left;

  /* Hauteur de la fentre */
  window_height = window_rect.bottom - window_rect.top;
  if(window_modify->mode == MODIFY_MODE_BYTE)
    window_height -= 191;
  else
    window_height += 191;

  /* Resize window */
  MoveWindow(window_modify->hwnd_modify,window_x,window_y,window_width,window_height,TRUE);
}


/*************************************************************************/
/*  ModifyMemory() :  Modifie la mmoire avec les donnes de la fentre. */
/*************************************************************************/
static void ModifyMemory(struct window_modify *window_modify)
{
  int i, j, bank, address, bank_address, repeat, is_error, offset, sign;
  struct modify_value *value_list;
  struct modify_value *current_value;
  struct mmu *current_mmu;
  char buffer_value[1024];
  char buffer_error[1024];
  unsigned char one_byte;
  unsigned short one_word;
  unsigned int one_dword;
  unsigned char data[16];
  unsigned char data_byte[16];
  char *value;
  int tab_value_id[5]     = {IDC_EDIT_PATTERN_VALUE1, IDC_EDIT_PATTERN_VALUE2, IDC_EDIT_PATTERN_VALUE3, IDC_EDIT_PATTERN_VALUE4, IDC_EDIT_PATTERN_VALUE5};
  int tab_increment_id[5] = {IDC_EDIT_PATTERN_INCREMENT1, IDC_EDIT_PATTERN_INCREMENT2, IDC_EDIT_PATTERN_INCREMENT3, IDC_EDIT_PATTERN_INCREMENT4, IDC_EDIT_PATTERN_INCREMENT5};

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /** Adresse de Destination des Data **/
  if(window_modify->ram_type == MODIFY_RAMTYPE_RAM)
    {
      /* Rcupre l'adresse cible */
      GetWindowText(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_RAM_BANK),buffer_value,1024-1);
      if(strlen(buffer_value) == 0)
        {
          MessageBox(window_modify->hwnd_modify,"Error, The Source Bank is Empty !","Modify Memory Error",MB_OK|MB_ICONWARNING);
          return;
        }
      bank = EvaluateAsByte(buffer_value,&is_error);
      GetWindowText(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_RAM_ADDRESS),buffer_value,1024-1);
      if(strlen(buffer_value) == 0)
        {
          MessageBox(window_modify->hwnd_modify,"Error, The Source Address is Empty !","Modify Memory Error",MB_OK|MB_ICONWARNING);
          return;
        }
      address = EvaluateAsWord(buffer_value,&is_error);
      bank_address = (bank << 16) | address;
    }
  else   /* Doc RAM */
    {
      /* Rcupre l'adresse DOC cible */
      GetWindowText(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_RAM_ADDRESS),buffer_value,1024-1);
      if(strlen(buffer_value) == 0)
        {
          MessageBox(window_modify->hwnd_modify,"Error, The Source Address is Empty !","Modify Memory Error",MB_OK|MB_ICONWARNING);
          return;
        }
      bank_address = EvaluateAsWord(buffer_value,&is_error);
    }

  /** Facteur de rptition **/
  GetWindowText(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_REPEAT),buffer_value,1024-1);
  if(strlen(buffer_value) == 0)
    repeat = 1;
  else
    {
      if(buffer_value[0] == '0' && toupper(buffer_value[1]) == 'X')
        sscanf(&buffer_value[2],"%X",&repeat);
      else
        repeat = atoi(buffer_value);
    }

  /* RAZ */
  for(i=0; i<5; i++)
    {
      strcpy(window_modify->tab_pattern[i].name,"");
      window_modify->tab_pattern[i].in_use = 0;
    }

  /** Rcupre la liste des Pattern **/
  if(window_modify->mode == MODIFY_MODE_PATTERN)
    {
      GetWindowText(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_NAME1),window_modify->tab_pattern[0].name,256-1);
      GetWindowText(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_NAME2),window_modify->tab_pattern[1].name,256-1);
      GetWindowText(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_NAME3),window_modify->tab_pattern[2].name,256-1);
      GetWindowText(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_NAME4),window_modify->tab_pattern[3].name,256-1);
      GetWindowText(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_PATTERN_NAME5),window_modify->tab_pattern[4].name,256-1);
    }

  /** Rcupre et dcode la ligne de valeurs **/
  GetWindowText(GetDlgItem(window_modify->hwnd_modify,IDC_EDIT_DATA),buffer_value,1024-1);
  if(strlen(buffer_value) == 0)
    {
      MessageBox(window_modify->hwnd_modify,"Error, The Data Byte list is Empty !","Modify Memory Error",MB_OK|MB_ICONWARNING);
      return;
    }
  value_list = DecodeModifyValueLine(buffer_value,window_modify);
  if(value_list == NULL)
    return;

  /** Rcupre les valeurs des Patterns utilises **/
  if(window_modify->mode == MODIFY_MODE_PATTERN)
    {
      /** Pattern #1 -> #5 **/
      for(j=0; j<5; j++)
        if(window_modify->tab_pattern[j].in_use == 1)
          {
            /* Init Value */
            GetWindowText(GetDlgItem(window_modify->hwnd_modify,tab_value_id[j]),buffer_value,1024-1);
            if(strlen(buffer_value) == 0)
              {
                sprintf(buffer_error,"Error, Init Value for Pattern #%d '%s' is Empty !",j+1,window_modify->tab_pattern[j].name);
                MessageBox(window_modify->hwnd_modify,buffer_error,"Modify Memory Error",MB_OK|MB_ICONWARNING);
                mem_free_valuelist(value_list);
                return;
              }
            /* Hexa */
            if(buffer_value[0] == '0' && toupper(buffer_value[1]) == 'X')
              sscanf(&buffer_value[2],"%X",&window_modify->tab_pattern[j].value);
            else
              window_modify->tab_pattern[j].value = atoi(buffer_value);

            /* Increment */
            sign = 1;
            window_modify->tab_pattern[j].increment = 0;
            GetWindowText(GetDlgItem(window_modify->hwnd_modify,tab_increment_id[j]),buffer_value,1024-1);
            if(strlen(buffer_value) > 0)
              {
                value = &buffer_value[0];
                /* Saut le Signe */
                if(*value == '-')
                  {
                    sign = -1;
                    value++;
                  }
                /* Hexa */
                if(value[0] == '0' && toupper(value[1]) == 'X')
                  sscanf(&value[2],"%X",&window_modify->tab_pattern[j].increment);
                else
                  window_modify->tab_pattern[j].increment = atoi(value);

                /* Positionne le signe */
                window_modify->tab_pattern[j].increment *= sign;
              }
          }
    }

  /* Init Value */
  for(current_value = value_list; current_value; current_value = current_value->next)
    if(current_value->type == MODIFY_VALUE_PATTERN)
      current_value->value = current_value->pattern->value;

  /** On va modifier la mmoire **/
  for(i=0, offset=0; i<repeat; i++)
    {
      /** On va placer toutes les valeur de la Byte List **/
      for(current_value = value_list; current_value; current_value = current_value->next)
        {
          /* Transfert 1 Byte */
          if(current_value->type == MODIFY_VALUE_BYTE)
            {
              if(window_modify->ram_type == MODIFY_RAMTYPE_RAM)
                {
                  WriteMemoryByte(bank_address+offset,current_value->byte,current_mmu);

                  /* Transfre les donnes dans la mmoire de l'mulateur */
                  data_byte[0] = current_value->byte;
                  WriteDataToEmulatorMemory(RAMTYPE_RAM,bank_address+offset,1,&data_byte[0]);
                }
              else
                {
                  WriteDOCRAMByte(bank_address+offset,current_value->byte,current_mmu);

                  /* Transfre les donnes dans la mmoire de l'mulateur */
                  data_byte[0] = current_value->byte;
                  WriteDataToEmulatorMemory(RAMTYPE_DOC,bank_address+offset,1,&data_byte[0]);
                }
              offset++;
            }
          else
            {
              /* Transfert une Pattern */
              if(current_value->pattern->data_length == MODIFY_LENGTH_8)
                one_byte = (unsigned char) current_value->value;
              else if(current_value->pattern->data_length == MODIFY_LENGTH_16)
                one_word = (unsigned short) current_value->value;
              else if(current_value->pattern->data_length == MODIFY_LENGTH_24)
                one_dword = (unsigned int) current_value->value;
              else if(current_value->pattern->data_length == MODIFY_LENGTH_32)
                one_dword = (unsigned int) current_value->value;

              /* Recopie les bytes dans l'ordre attendu */
              if(current_value->pattern->combo_length_alignment == MODIFY_COMBO_8)
                data[0] = one_byte;
              else if(current_value->pattern->combo_length_alignment == MODIFY_COMBO_16_INTEL)
                memcpy(&data[0],&one_word,sizeof(short));
              else if(current_value->pattern->combo_length_alignment == MODIFY_COMBO_24_INTEL)
                memcpy(&data[0],&one_dword,sizeof(int));
              else if(current_value->pattern->combo_length_alignment == MODIFY_COMBO_32_INTEL)
                memcpy(&data[0],&one_dword,sizeof(int));
              else if(current_value->pattern->combo_length_alignment == MODIFY_COMBO_16_MOTOROLA)
                {
                  memcpy(&data[0],&one_word,sizeof(short));
                  ExchangeByte(&data[0],2);
                }
              else if(current_value->pattern->combo_length_alignment == MODIFY_COMBO_24_MOTOROLA)
                {
                  memcpy(&data[0],&one_dword,sizeof(int));
                  ExchangeByte(&data[0],3);
                }
              else if(current_value->pattern->combo_length_alignment == MODIFY_COMBO_32_MOTOROLA)
                {
                  memcpy(&data[0],&one_dword,sizeof(int));
                  ExchangeByte(&data[0],4);
                }

              /** Transfre les donnes dans la mmoire de l'mulateur **/
              if(window_modify->ram_type == MODIFY_RAMTYPE_RAM)
                WriteDataToEmulatorMemory(RAMTYPE_RAM,bank_address+offset,current_value->pattern->data_length,data);
              else
                WriteDataToEmulatorMemory(RAMTYPE_DOC,bank_address+offset,current_value->pattern->data_length,data);

              /** Transfer les bytes **/
              for(j=0; j<current_value->pattern->data_length; j++)
                {
                  if(window_modify->ram_type == MODIFY_RAMTYPE_RAM)
                    WriteMemoryByte(bank_address+offset,data[j],current_mmu);
                  else
                    WriteDOCRAMByte(bank_address+offset,data[j],current_mmu);
                  offset++;
                }

              /* Post incrmentation de la valeur */
              current_value->value += current_value->pattern->increment;
            }
        }
    }

  /* Libration mmoire */
  mem_free_valuelist(value_list);
}


/*************************************************************************************/
/*  DecodeModifyValueLine() :  Dcode les diffrentes valeurs de la liste Data Byte. */
/*************************************************************************************/
static struct modify_value *DecodeModifyValueLine(char *data_byte, struct window_modify *window_modify)
{
  int i, is_error;
  struct modify_value *first_value;
  struct modify_value *last_value;
  struct modify_value *current_value;
  char buffer_value[1024];
  char buffer_error[2048];

  /* Init */
  first_value = NULL;
  last_value = NULL;
  buffer_value[0] = '\0';

  /** Dcoupe la chaine en byte **/
  for(i=0; i<(int)strlen(data_byte); i++)
    {
      /* Espace */
      if(data_byte[i] == ' ' && strlen(buffer_value) == 0)
        continue;

      /* Fin de valeur */
      if(data_byte[i] == ' ' && strlen(buffer_value) > 0)
        {
          /* Allocation + Remplissage */
          current_value = (struct modify_value *) calloc(1,sizeof(struct modify_value));
          if(current_value == NULL)
            {
              MessageBox(window_modify->hwnd_modify,"Error, Can't allocate memory to decode Data Byte list !","Modify Memory Error",MB_OK|MB_ICONWARNING);
              mem_free_valuelist(current_value);
              return(NULL);
            }
          strcpy(current_value->value_txt,buffer_value);
          /* Rattachement */
          if(first_value == NULL)
            first_value = current_value;
          else
            last_value->next = current_value;
          last_value = current_value;
          /* Init */
          buffer_value[0] = '\0';
          continue;
        }

      /* Valeurs en cours de rcupration */
      buffer_value[strlen(buffer_value)+1] = '\0';
      buffer_value[strlen(buffer_value)] = data_byte[i];
    }
  if(strlen(buffer_value) > 0)
    {
      /* Allocation + Remplissage */
      current_value = (struct modify_value *) calloc(1,sizeof(struct modify_value));
      if(current_value == NULL)
        {
          MessageBox(window_modify->hwnd_modify,"Error, Can't allocate memory to decode Data Byte list !","Modify Memory Error",MB_OK|MB_ICONWARNING);
          mem_free_valuelist(current_value);
          return(NULL);
        }
      strcpy(current_value->value_txt,buffer_value);
      /* Rattachement */
      if(first_value == NULL)
        first_value = current_value;
      else
        last_value->next = current_value;
      last_value = current_value;
      /* Init */
      buffer_value[0] = '\0';
    }

  /** Vrifie les types de donnes **/
  for(current_value = first_value; current_value; current_value = current_value->next)
    {
      /** Est-ce un octet cod en 0xXX ? **/
      if(strlen(current_value->value_txt) == 4 && 
         current_value->value_txt[0] == '0' && toupper(current_value->value_txt[1]) == 'X' &&
         IsHexaDecimalChar(current_value->value_txt[2]) && IsHexaDecimalChar(current_value->value_txt[3]))
        {
          current_value->type = MODIFY_VALUE_BYTE;
          current_value->byte = EvaluateAsByte(&current_value->value_txt[2],&is_error);
          continue;
        }

      /** Est_ce un octet Hexa cod XX **/
      if(strlen(current_value->value_txt) == 2 && 
         IsHexaDecimalChar(current_value->value_txt[0]) && IsHexaDecimalChar(current_value->value_txt[1]) &&
         (current_value->value_txt[0] == '0' || 
          (toupper(current_value->value_txt[0]) >= 'A' && toupper(current_value->value_txt[0]) <= 'F') ||
          (toupper(current_value->value_txt[1]) >= 'A' && toupper(current_value->value_txt[1]) <= 'F')
         ))
        {
          current_value->type = MODIFY_VALUE_BYTE;
          current_value->byte = EvaluateAsByte(&current_value->value_txt[0],&is_error);
          continue;
        }

      /** Est-ce un entier ? **/
      if((strlen(current_value->value_txt) == 1 && IsDecimalChar(current_value->value_txt[0])) ||
         (strlen(current_value->value_txt) == 2 && IsDecimalChar(current_value->value_txt[0]) && IsDecimalChar(current_value->value_txt[1])) || 
         (strlen(current_value->value_txt) == 3 && IsDecimalChar(current_value->value_txt[0]) && IsDecimalChar(current_value->value_txt[1]) && IsDecimalChar(current_value->value_txt[2])))
        {
          if(atoi(current_value->value_txt) < 256)
            {
              current_value->type = MODIFY_VALUE_BYTE;
              current_value->byte = atoi(current_value->value_txt);
              continue;
            }
        }

      /** Est-ce une des Pattern **/
      if(window_modify->mode == MODIFY_MODE_PATTERN)
        {
          for(i=0; i<5; i++)
            if(!stricmp(window_modify->tab_pattern[i].name,current_value->value_txt))
              {
                current_value->type = MODIFY_VALUE_PATTERN;
                current_value->pattern = &window_modify->tab_pattern[i];
                window_modify->tab_pattern[i].in_use = 1;
                break;
              }
        }

      /* Rien trouv : Erreur */
      if(current_value->type == MODIFY_VALUE_UNKNOWN)
        {
          sprintf(buffer_error,"Error, Can't decode Data Byte value '%s' !",current_value->value_txt);
          MessageBox(window_modify->hwnd_modify,buffer_error,"Modify Memory Error",MB_OK|MB_ICONWARNING);
          mem_free_valuelist(first_value);
          return(NULL);
        }
    }

  /* Rien  renvoyer ? */
  if(first_value == NULL)
    MessageBox(window_modify->hwnd_modify,"Error, The Data Byte list is void !","Modify Memory Error",MB_OK|MB_ICONWARNING);

  /* Renvoi la liste */
  return(first_value);
}


/*******************************************************************/
/*  mem_free_valuelist() :  Libration mmoire liste de structure. */
/*******************************************************************/
static void mem_free_valuelist(struct modify_value *first_value)
{
  struct modify_value *current_value;
  struct modify_value *next_value;

  /** Libration **/
  for(current_value = first_value; current_value; current_value = next_value)
    {
      next_value = current_value->next;
      free(current_value);
    }
}

/*
#define MODIFY_VALUE_BYTE          0
#define MODIFY_VALUE_PATTERN       1

struct modify_value
{
  int type;

  /* Byte *
  unsigned char byte;

  /* Pattern *
  int value;
  struct modify_pattern *pattern;

  struct modify_value *next;
};
*/

/********************************************************************/
/*  DeleteModifyChildWindows() :  Destruction des fentres enfants. */
/********************************************************************/
static void DeleteModifyChildWindows(struct window_modify *window_modify)
{
}


/**********************************************************************/
/*  mem_free_window_modify() :  Libration mmoire pour la structure. */
/**********************************************************************/
void mem_free_window_modify(struct window_modify *window_modify)
{
  if(window_modify)
    {      
      free(window_modify);
    }
}

/**********************************************************************/
