/***********************************************************************/
/*                                                                     */
/*  Window_Profile.h :  Header de gestion de la fentre Profile.       */
/*                                                                     */
/***********************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : May 2024  */
/***********************************************************************/

/* Un message destin  la fentre Profile pour demander la maj de l'affichage */
#define HM_PROFILE_REFRESHWINDOW   (WM_USER + 24)

#define PROFILE_REFRESH_AUTOMATIC       0
#define PROFILE_REFRESH_MANUAL          1

#define PROFILE_LISTVIEW_ROW_HEIGHT    15    /* Taille d'une ligne de la ListView (en Pixel) */

#define IDC_LISTVIEW_PROFILE         1033

#define PROFILE_LISTVIEW_NB_ROW        20    /* Nombre de ligne de donnes dans la ListView  */
#define PROFILE_LISTVIEW_MIN_NB_ROW     2    /* Nombre de ligne de donnes Min dans la ListView  */

struct profile_stack
{
  /** Point d'entre de cette fonction **/
  DWORD function_address;         /* Adresse de base de la fonction en cours = Jump Address de la callstack prcdente */
  WORD call_number;               /* Complment d'appel  la fonction Tool / GS/OS pour nommer la fonction */
  QWORD gcc;                      /* Global Cycle Count */
  QWORD goid;                     /* Global Operation ID */
  DWORD function_end_address;     /* Adresse de retour de fin de cette fonction (0xFFFFFFFF si ROOT) */
  int is_interrupt;               /* On vient de rentrer dans une interruption */
  int depth;                      /* Niveau de profondeur des appels 0-N */
  int function_under_interrupt;   /* Cette fonction est appele sous interruption (la fonction qui appelle celle-ci est sous interruption) */

  struct profile_stack *previous;
  struct profile_stack *next;
};

struct profile_function
{
  DWORD address;                               /* Adresse de base de la fonction en cours = Jump Address de la callstack prcdente */
  WORD call_number;                            /* Complment d'appel  la fonction Tool / GS/OS pour nommer la fonction */
  QWORD goid_init;                             /* Global Operation ID lors de l'entre dans cette fonction */
  QWORD gcc_init;                              /* Global Cycle Count lors de l'entre dans cette fonction */
  int depth;                                   /* Niveau de profondeur des appels 0-N */
  int is_interrupt;                            /* Cette fonction est sous interruption */
  int is_current_path;                         /* Ce chemin est celui de la fonction courrante */
  DWORD nb_call;                               /* Nombre d'appel cette fonction */

  QWORD total_cycle_local;                     /* Nombre de cycle pour cette fonction (sauf les sous-fonctions) */
  QWORD total_cycle_local_hide_interrupt;      /* Nombre de cycle pour cette fonction (sauf les sous-fonctions) sous interruption */

  QWORD total_cycle_sub;                       /* Total des cycles des sous fonctions lors de cet appel */
  QWORD total_cycle_sub_hide_interrupt;        /* Total des cycles des sous fonctions lors de cet appel sous interruption */

  struct profile_function *parent;             /* On remonte vers le Pre */
  int nb_sub_function;
  struct profile_function *first_sub_function; /* Liste chaine des Sous Fonctions */
  struct profile_function *last_sub_function;

  struct profile_function* next;               /* Fonction Suivante de mme niveau */

  /** Pour le fichier Profiler **/
  int file_x;                                  /* 1-N */
  int file_y;                                  /* 1-N */
  int width;
  char full_name[BUFFER_SIZE];                 /* 00/0000 [0000] FunctionName */
};

struct profile_table
{
  /* Arbre des Fonctions */
  struct profile_function *first_profile_function;
  struct profile_function *current_profile_function;     /* Pointeur vers la Fonction en cours */


  /* Pile d'appel des Fonctions */
  int nb_profile_stack;
  struct profile_stack *first_profile_stack;
  struct profile_stack *last_profile_stack;

  /* Valeurs Max pour X/Y dans le fichier */
  int file_max_x;
  int file_max_y;
  int *tab_max_width;              /* Largueur Max de chaque colonne */
};

struct window_profile
{
  HWND hwnd_profile;
  HFONT hFont;

  /* Mode de retaille de la fentre (user/executable) */
  int user_resize;

  /* Dimension de la fentre  la cration */
  int window_init_width;
  int window_init_height;
  int nb_init_data_row;

  /* Mode de rafraichissement de la fentre */
  int refresh_mode;                /* PROFILE_REFRESH_XXX */
  int hide_interrupt;              /* A 1 si on n'affiche pas les Interrupt */

  /* List View */
  int nb_row;                      /* Nombre de lignes affiches dans la ListBox */
  int profile_x0;
  int profile_y0;
  struct list_view listview_profile;
  
  /* Pointeur vers la Profile Function ROOT en cours d'affichage */
  struct profile_function *current_profile_function;

  /* Ligne slectionne dans la ListView */
  DWORD selected_profile_address;
  WORD selected_profile_call_number;
  struct profile_function *selected_profile_function;

  /* Buffer */
  char buffer_log[BUFFER_SIZE];

  /* Notification */
  struct notification *current_notification;
};

struct window_profile *BuildProfileWindow(HINSTANCE,int);
void ProcessProfileOperation(struct window_main *,int);
void ClearProfileTable(struct window_profile *);
void mem_free_window_profile(struct window_profile *);
void mem_free_profile_table(struct profile_table *);

/********************************************************/
