/**********************************************************************/
/*                                                                    */
/*   Window_Project.c :  Module de gestion de la fentre Project.     */
/*                                                                    */
/**********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Avril 2018  */
/**********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Project.h"
#include "Cyrene_Engine.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Project.h"

LRESULT CALLBACK ProcHandlerProjectWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateProjectChildWindows(HWND,struct window_project *);
LRESULT CALLBACK subEditProjectProcControl(HWND,UINT,WPARAM,LPARAM);
static struct project *GetProjectValueFromWindow(struct window_project *);
static void DeleteProjectChildWindows(struct window_project *);


/****************************************************************************/
/*  BuildProjectWindow() :  Construction / Affichage de la fentre Project. */
/****************************************************************************/
struct window_project *BuildProjectWindow(HINSTANCE hInstance, int menu_id, char *current_project_filepath)
{
  HWND hwndProject;
  char project_file_path[BUFFER_SIZE];
  char buffer_error[BUFFER_SIZE];
  struct project *previous_project;
  struct project *current_project;
  struct window_main *window_main;
  struct window_project *window_project;
  struct parameter *param;

  /* Init */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  current_project = NULL;
  project_file_path[0] = '\0';

  /** On demande  charger un Projet en mmoire **/
  if(menu_id == ID_MENU_FILE_OPENPROJECT)
    {
      /* Rcupre le chemin du fichier */
      GetOneFilePath(window_main->hwnd_main,"All Files (*.*)\0*.*\0\0","",project_file_path);
      if(strlen(project_file_path) == 0)
        return(NULL);

      /* Charge le fichier en mmoire */
      current_project = LoadProjectFile(project_file_path,&buffer_error[0]);
      if(current_project == NULL)
        {
          sprintf(window_main->buffer_log,"Error : %s",buffer_error);
          MessageBox(window_main->hwnd_main,window_main->buffer_log,"Open Project file",MB_OK|MB_ICONWARNING);
          return(NULL);
        }
    }
  else if(menu_id == ID_MENU_FILE_RELOADPROJECT)
    {
      /* Rcupre le projet en cours */
      my_Memory(MEMORY_GET_PROJECT,&previous_project,NULL);
      strcpy(current_project_filepath,previous_project->project_file_path);

      /* Charge le fichier en mmoire */
      current_project = LoadProjectFile(current_project_filepath,&buffer_error[0]);
      if(current_project == NULL)
        {
          sprintf(window_main->buffer_log,"Error : %s",buffer_error);
          MessageBox(window_main->hwnd_main,window_main->buffer_log,"Reload Project",MB_OK|MB_ICONWARNING);
          return(NULL);
        }
    }
  else if(menu_id == ID_MENU_FILE_SAVEPROJECTAS)
    {
      /* Rcupre le projet en cours */
      my_Memory(MEMORY_GET_PROJECT,&current_project,NULL);
    }
  else if(menu_id == ID_MENU_FILE_EDITPROJECT)
    {
      /* Rcupre le projet en cours */
      my_Memory(MEMORY_GET_PROJECT,&current_project,NULL);
      if(current_project == NULL)
        return(NULL);

      /* Rcupre le nom du fichier */
      strcpy(project_file_path,current_project->project_file_path);
    }

  /** Allocation de la structure window_project **/
  window_project = (struct window_project *) calloc(1,sizeof(struct window_project));
  if(window_project == NULL)
    return(NULL);
  window_project->menu_id = menu_id;
  window_project->current_project = current_project;
  my_Memory(MEMORY_SET_WINDOW_PROJECT,window_project,NULL);

  /** Cration de la fentre Project **/
  hwndProject = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_PROJECT),window_main->hwnd_main,(DLGPROC)ProcHandlerProjectWindow);
  if(hwndProject == NULL)
    {
      my_Memory(MEMORY_FREE_WINDOW_PROJECT,NULL,NULL);
      return(NULL);
    }

  /** Show Window + Paint Content **/
  ShowWindow(hwndProject,SW_SHOWDEFAULT);
  UpdateWindow(hwndProject);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_project);
}


/************************************************************************/
/*  ProcHandlerProjectWindow() :  Gestion des vnements de la fentre. */
/************************************************************************/
LRESULT CALLBACK ProcHandlerProjectWindow(HWND hwndProject, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  BOOL is_OK;
  int result, nb_file;
  char file_path[BUFFER_SIZE];
  struct project *previous_project;
  struct project *current_project;
  struct project *saveas_project;
  struct window_project *window_project;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_PROJECT,&window_project,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateProjectChildWindows(hwndProject,window_project);
        return(TRUE);

      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** On appuie sur un des boutons de rcupration du chemin des Files **/
            case IDC_BUTTON_FILE0_GET_PATH :
              GetOneFilePath(window_project->hwnd_project,"All Files (*.*)\0*.*\0\0",window_project->file0_path,file_path);
              if(strlen(file_path) > 0)
                {
                  strcpy(window_project->file0_path,file_path);
                  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE0_PATH),window_project->file0_path);
                }
              break;
            case IDC_BUTTON_FILE1_GET_PATH :
              GetOneFilePath(window_project->hwnd_project,"All Files (*.*)\0*.*\0\0",window_project->file1_path,file_path);
              if(strlen(file_path) > 0)
                {
                  strcpy(window_project->file1_path,file_path);
                  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE1_PATH),window_project->file1_path);
                }
              break;
            case IDC_BUTTON_FILE2_GET_PATH :
              GetOneFilePath(window_project->hwnd_project,"All Files (*.*)\0*.*\0\0",window_project->file2_path,file_path);
              if(strlen(file_path) > 0)
                {
                  strcpy(window_project->file2_path,file_path);
                  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE2_PATH),window_project->file2_path);
                }
              break;
            case IDC_BUTTON_FILE3_GET_PATH :
              GetOneFilePath(window_project->hwnd_project,"All Files (*.*)\0*.*\0\0",window_project->file3_path,file_path);
              if(strlen(file_path) > 0)
                {
                  strcpy(window_project->file3_path,file_path);
                  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE3_PATH),window_project->file3_path);
                }
              break;
            case IDC_BUTTON_FILE4_GET_PATH :
              GetOneFilePath(window_project->hwnd_project,"All Files (*.*)\0*.*\0\0",window_project->file4_path,file_path);
              if(strlen(file_path) > 0)
                {
                  strcpy(window_project->file4_path,file_path);
                  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE4_PATH),window_project->file4_path);
                }
              break;
            case IDC_BUTTON_FILE5_GET_PATH :
              GetOneFilePath(window_project->hwnd_project,"All Files (*.*)\0*.*\0\0",window_project->file5_path,file_path);
              if(strlen(file_path) > 0)
                {
                  strcpy(window_project->file5_path,file_path);
                  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE5_PATH),window_project->file5_path);
                }
              break;
            case IDC_BUTTON_FILE6_GET_PATH :
              GetOneFilePath(window_project->hwnd_project,"All Files (*.*)\0*.*\0\0",window_project->file6_path,file_path);
              if(strlen(file_path) > 0)
                {
                  strcpy(window_project->file6_path,file_path);
                  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE6_PATH),window_project->file6_path);
                }
              break;
            case IDC_BUTTON_FILE7_GET_PATH :
              GetOneFilePath(window_project->hwnd_project,"All Files (*.*)\0*.*\0\0",window_project->file7_path,file_path);
              if(strlen(file_path) > 0)
                {
                  strcpy(window_project->file7_path,file_path);
                  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE7_PATH),window_project->file7_path);
                }
              break;
            case IDC_BUTTON_FILE8_GET_PATH :
              GetOneFilePath(window_project->hwnd_project,"All Files (*.*)\0*.*\0\0",window_project->file8_path,file_path);
              if(strlen(file_path) > 0)
                {
                  strcpy(window_project->file8_path,file_path);
                  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE8_PATH),window_project->file8_path);
                }
              break;
                         
            /** File Sound Ram **/
            case IDC_BUTTON_FILESOUND_GET_PATH :            
              GetOneFilePath(window_project->hwnd_project,"All Files (*.*)\0*.*\0\0",window_project->file_sound_path,file_path);
              if(strlen(file_path) > 0)
                {
                  strcpy(window_project->file_sound_path,file_path);
                  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILESOUND_PATH),window_project->file_sound_path);
                }
              break;

            /** Files Merlin 32 **/
            case IDC_BUTTON_FILEMERLIN32_GET_PATH :
              GetOneFilePath(window_project->hwnd_project,"All Files (*.*)\0*.*\0\0",window_project->file_merlin32_path,file_path);
              if(strlen(file_path) > 0)
                {
                  strcpy(window_project->file_merlin32_path,file_path);
                  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILEMERLIN32_PATH),window_project->file_merlin32_path);
                }
              break;

            /** On appuie sur un des boutons de rcupration du nombre de fichiers  **/
            case IDC_BUTTON_FILEMERLIN32_COUNT :
              GetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILEMERLIN32_PATH),file_path,1023);
              nb_file = GetFileNumber(file_path,1);
              SetDlgItemInt(window_project->hwnd_project,IDC_STATIC_FILEMERLIN32_NB,nb_file,FALSE);
              break;
                         
            /** Bouton CheckBox : Apple II Rom **/ 
            case IDC_CHECK_APPLEII_ROM :
              if(HIWORD(wParam) == BN_CLICKED)
                {
                  if(window_project->appleii_rom == 0)
                    window_project->appleii_rom = 1;
                  else
                    window_project->appleii_rom = 0;
                }
              break;

            /** Bouton Radio : On change la valeur de la Rom Apple IIgs **/
            case IDC_RADIO_APPLEIIGS_ROM01 :
              if(HIWORD(wParam) == BN_CLICKED && window_project->appleiigs_rom != PROJECT_APPLEIIGS_ROM01)
                {
                  window_project->appleiigs_rom = PROJECT_APPLEIIGS_ROM01;
                  SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_ROM01,BM_SETCHECK,1,0);
                  SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_ROM03,BM_SETCHECK,0,0);
                  SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_NOROM,BM_SETCHECK,0,0);
                }
              break;
            case IDC_RADIO_APPLEIIGS_ROM03 :
              if(HIWORD(wParam) == BN_CLICKED && window_project->appleiigs_rom != PROJECT_APPLEIIGS_ROM03)
                {
                  window_project->appleiigs_rom = PROJECT_APPLEIIGS_ROM03;
                  SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_ROM01,BM_SETCHECK,0,0);
                  SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_ROM03,BM_SETCHECK,1,0);
                  SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_NOROM,BM_SETCHECK,0,0);
                }
              break;
            case IDC_RADIO_APPLEIIGS_NOROM :
              if(HIWORD(wParam) == BN_CLICKED && window_project->appleiigs_rom != PROJECT_APPLEIIGS_NOROM)
                {
                  window_project->appleiigs_rom = PROJECT_APPLEIIGS_NOROM;
                  SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_ROM01,BM_SETCHECK,0,0);
                  SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_ROM03,BM_SETCHECK,0,0);
                  SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_NOROM,BM_SETCHECK,1,0);
                }
              break;

            case IDC_BUTTON_SAVEPROJECTAS :
              /* Rcupre les valeurs de la fentre (avec validation) */
              saveas_project = GetProjectValueFromWindow(window_project);
              if(saveas_project == NULL)
                break;
                
              /* Demande un chemin de sauvegarde */
              if(window_project->current_project != NULL)
                strcpy(file_path,window_project->current_project->project_file_path);
              else
                strcpy(file_path,"");
              is_OK = GetSaveFilePath(window_project->hwnd_project,"All Files (*.*)\0*.*\0\0\0",file_path);
              if(!is_OK)
                {
                  /* Si l'utilisateur l'annule */
                  mem_free_project(saveas_project);
                  if(window_project->menu_id == ID_MENU_FILE_SAVEPROJECTAS)
                    SendMessage(hwndProject,WM_CLOSE,0,0);    /* Quite la fentre */
                  break;     
                }
              strcpy(saveas_project->project_file_path,file_path);

              /** Sauvegarde des informations du Projet dans un fichier texte **/
              result = SaveProjectFile(saveas_project,saveas_project->project_file_path);
              if(result)
                {
                  sprintf(window_project->buffer_log,"Impossible to save File '%s'",saveas_project->project_file_path);
                  MessageBox(window_project->hwnd_project,window_project->buffer_log,"Save Project values",MB_OK|MB_ICONWARNING);
                  mem_free_project(saveas_project);
                  if(window_project->menu_id == ID_MENU_FILE_SAVEPROJECTAS)
                    SendMessage(hwndProject,WM_CLOSE,0,0);    /* Quite la fentre */
                  break;
                }

              /* Conserve le nouveau projet */
              my_Memory(MEMORY_GET_PROJECT,&previous_project,NULL);
              if(window_project->current_project != NULL)
                {
                  if(previous_project != window_project->current_project)
                    mem_free_project(window_project->current_project);
                }
              window_project->current_project = saveas_project;

              /* Dgrise les Menu Project */
              UpdateWindowMainMenuItemStatus();

              /* Ferme la fentre */
              if(window_project->menu_id == ID_MENU_FILE_SAVEPROJECTAS)
                SendMessage(hwndProject,WM_CLOSE,0,0);     /* Quite la fentre */
              break;                    
                                      
            case IDC_BUTTON_RUNPROJECT :
              /* Rcupre les valeurs de la fentre (avec validation) */
              current_project = GetProjectValueFromWindow(window_project);
              if(current_project == NULL)
                break;

              /* Libration du projet en cours */
              my_Memory(MEMORY_GET_PROJECT,&previous_project,NULL);
              if(previous_project != NULL)
                {
                  if(window_project->current_project == previous_project)
                    window_project->current_project = NULL;
                  my_Memory(MEMORY_FREE_PROJECT,NULL,NULL);
                }

              /* Sauvegarde du Projet */
              my_Memory(MEMORY_SET_PROJECT,current_project,NULL);

              /* Dgrise les Menu Project */
              UpdateWindowMainMenuItemStatus();

              /* Mise  jour de la Status Bar */
              UpdateWindowMainStatusBar();

              /* Init Engine */
              Initialize65816Engine(current_project,NULL);

              /* Quite la fentre */
              SendMessage(hwndProject,WM_CLOSE,0,0);
              break;

            case IDC_BUTTON_CANCEL :
              /* Le projet rattach  window_project a t'il dj t sauvegard ? */
              if(window_project->current_project != NULL)
                {
                  /* Rcupre le Projet sauvegard */
                  my_Memory(MEMORY_GET_PROJECT,&current_project,NULL);
                  if(window_project->current_project == current_project)
                    window_project->current_project = NULL;   /* Dsolidarise pour empcher sa libration */
                }

              /* Quite la fentre */
              SendMessage(hwndProject,WM_CLOSE,0,0);
              break;
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Project */
        DestroyWindow(hwndProject);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteProjectChildWindows(window_project);

        /* Dcoche l'option dans le Menu File */
        CheckMainWindowMenuItem(window_project->menu_id,FALSE);

        /* Libration mmoire de la structure */
        my_Memory(MEMORY_FREE_WINDOW_PROJECT,NULL,NULL);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*****************************************************************************************************/
/*  CreateProjectChildWindows() : Crer les fentres enfants et remplir la structure window_project. */
/*****************************************************************************************************/
static int CreateProjectChildWindows(HWND hwndProject, struct window_project *window_project)
{
  int i;
  char *tab_color_name[] = {"Black","Deep Red","Dark Blue","Purple","Dark Green","Dark Gray","Medium Blue","Light Blue","Brown","Orange","Light Gray","Pink","Light Green","Yellow","Aquamarine","White"};
  char buffer_hexa[256];
  int tab_edit_file_path[] = {IDC_EDIT_FILE0_PATH,IDC_EDIT_FILE1_PATH,IDC_EDIT_FILE2_PATH,IDC_EDIT_FILE3_PATH,IDC_EDIT_FILE4_PATH,IDC_EDIT_FILE5_PATH,IDC_EDIT_FILE6_PATH,IDC_EDIT_FILE7_PATH,IDC_EDIT_FILE8_PATH};
  int tab_edit_file_bank[] = {IDC_EDIT_FILE0_BANK,IDC_EDIT_FILE1_BANK,IDC_EDIT_FILE2_BANK,IDC_EDIT_FILE3_BANK,IDC_EDIT_FILE4_BANK,IDC_EDIT_FILE5_BANK,IDC_EDIT_FILE6_BANK,IDC_EDIT_FILE7_BANK,IDC_EDIT_FILE8_BANK};
  int tab_edit_file_address[] = {IDC_EDIT_FILE0_ADDRESS,IDC_EDIT_FILE1_ADDRESS,IDC_EDIT_FILE2_ADDRESS,IDC_EDIT_FILE3_ADDRESS,IDC_EDIT_FILE4_ADDRESS,IDC_EDIT_FILE5_ADDRESS,IDC_EDIT_FILE6_ADDRESS,IDC_EDIT_FILE7_ADDRESS,IDC_EDIT_FILE8_ADDRESS};
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL); 

  /** Window Handle **/
  window_project->hwnd_project = hwndProject;

  /** Titre de la fentre **/
  if(window_project->menu_id == ID_MENU_FILE_NEWPROJECT)
    SetWindowText(window_project->hwnd_project,"New Project");
  else if(window_project->menu_id == ID_MENU_FILE_OPENPROJECT)
    SetWindowText(window_project->hwnd_project,"Open Project");
  else if(window_project->menu_id == ID_MENU_FILE_RELOADPROJECT)
    SetWindowText(window_project->hwnd_project,"Reload Project");
  else if(window_project->menu_id == ID_MENU_FILE_EDITPROJECT)
    SetWindowText(window_project->hwnd_project,"Edit Project");
  else if(window_project->menu_id == ID_MENU_FILE_SAVEPROJECTAS)
    SetWindowText(window_project->hwnd_project,"Save Project As...");

  /** Boutons en bas  froite de la fentre **/
  if(window_project->menu_id == ID_MENU_FILE_NEWPROJECT)
    {
      SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_BUTTON_SAVEPROJECTAS),"Save As...");
      SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_BUTTON_RUNPROJECT),"Run Project");
    }
  else if(window_project->menu_id == ID_MENU_FILE_OPENPROJECT)
    {
      SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_BUTTON_SAVEPROJECTAS),"Save As...");
      SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_BUTTON_RUNPROJECT),"Run Project");
    }
  else if(window_project->menu_id == ID_MENU_FILE_RELOADPROJECT)
    {
      SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_BUTTON_SAVEPROJECTAS),"Save As...");
      SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_BUTTON_RUNPROJECT),"Run Project");
    }
  else if(window_project->menu_id == ID_MENU_FILE_EDITPROJECT)
    {
      SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_BUTTON_SAVEPROJECTAS),"Save As...");
      SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_BUTTON_RUNPROJECT),"Run Project");
    }
  else if(window_project->menu_id == ID_MENU_FILE_SAVEPROJECTAS)
    {
      SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_BUTTON_SAVEPROJECTAS),"Save Project");
      ShowWindow(GetDlgItem(window_project->hwnd_project,IDC_BUTTON_RUNPROJECT), SW_HIDE);
    }

  /** Surchage des Control Edit pour capter les RETURN lors des saisies **/
  window_project->oldEditProcProcA = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_PROC_A),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcProcX = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_PROC_X),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcProcY = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_PROC_Y),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcProcDBR = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_PROC_DBR),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcProcPBR = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_PROC_PBR),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcProcPC = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_PROC_PC),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcProcStack = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_PROC_STACK),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcProcDPage = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_PROC_DPAGE),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcStatusC = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_STATUS_C),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcStatusZ = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_STATUS_Z),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcStatusI = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_STATUS_I),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcStatusD = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_STATUS_D),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcStatusX = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_STATUS_X),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcStatusM = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_STATUS_M),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcStatusV = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_STATUS_V),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcStatusN = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_STATUS_N),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcStatusE = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_STATUS_E),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcStatusB = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_STATUS_B),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcRamSize = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_RAM_SIZE),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile0Bank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE0_BANK),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile0Address = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE0_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile0Path = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE0_PATH),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile1Bank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE1_BANK),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile1Address = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE1_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile1Path = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE1_PATH),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile2Bank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE2_BANK),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile2Address = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE2_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile2Path = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE2_PATH),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile3Bank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE3_BANK),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile3Address = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE3_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile3Path = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE3_PATH),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile4Bank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE4_BANK),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile4Address = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE4_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile4Path = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE4_PATH),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile5Bank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE5_BANK),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile5Address = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE5_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile5Path = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE5_PATH),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile6Bank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE6_BANK),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile6Address = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE6_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile6Path = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE6_PATH),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile7Bank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE7_BANK),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile7Address = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE7_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile7Path = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE7_PATH),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile8Bank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE8_BANK),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile8Address = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE8_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFile8Path = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILE8_PATH),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFileSoundAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILESOUND_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFileSoundPath = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILESOUND_PATH),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);
  window_project->oldEditProcFileMerlin32Path = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndProject,IDC_EDIT_FILEMERLIN32_PATH),GWLP_WNDPROC,(LONG_PTR)subEditProjectProcControl);

  /** Remplissage de Files 0-8 **/
  if(window_project->current_project != NULL)
    {
      for(i=0; i<9; i++)
        {
          if(window_project->current_project->tab_file[i] != NULL)
            {
              SetWindowText(GetDlgItem(window_project->hwnd_project,tab_edit_file_path[i]),window_project->current_project->tab_file[i]->path);
              sprintf(buffer_hexa,"%02X",window_project->current_project->tab_file_bank[i]);
              SetWindowText(GetDlgItem(window_project->hwnd_project,tab_edit_file_bank[i]),buffer_hexa);
              sprintf(buffer_hexa,"%04X",window_project->current_project->tab_file_address[i]);
              SetWindowText(GetDlgItem(window_project->hwnd_project,tab_edit_file_address[i]),buffer_hexa);
            }
        } 
    }

  /** Remplissage File Sound **/
  if(window_project->current_project != NULL)
    if(window_project->current_project->file_sound != NULL)
      {
        SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILESOUND_PATH),window_project->current_project->file_sound->path);
        sprintf(buffer_hexa,"%04X",window_project->current_project->file_sound_address);
        SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILESOUND_ADDRESS),buffer_hexa);
      }

  /** Remplissage File Merlin 32 **/
  if(window_project->current_project != NULL)
    if(strlen(window_project->current_project->file_merlin32_path) > 0)
      SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILEMERLIN32_PATH),window_project->current_project->file_merlin32_path);

  /** Cration des ComboBox Couleur **/
  window_project->hwnd_combo_text_color = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                         571,302,121,300,hwndProject,(HMENU)IDC_COMBOBOX_NEWPROJECT_TEXT_COLOR,param->hInstance,0);
  window_project->hwnd_combo_background_color = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                               571,333,121,300,hwndProject,(HMENU)IDC_COMBOBOX_NEWPROJECT_BACKGROUND_COLOR,param->hInstance,0); 
  window_project->hwnd_combo_border_color = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                           571,364,121,300,hwndProject,(HMENU)IDC_COMBOBOX_NEWPROJECT_BORDER_COLOR,param->hInstance,0); 
  for(i=0; i<16; i++)
    {
      SendMessage(window_project->hwnd_combo_text_color,CB_ADDSTRING,0,(LPARAM)tab_color_name[i]); 
      SendMessage(window_project->hwnd_combo_background_color,CB_ADDSTRING,0,(LPARAM)tab_color_name[i]); 
      SendMessage(window_project->hwnd_combo_border_color,CB_ADDSTRING,0,(LPARAM)tab_color_name[i]); 
    }

  /* Premire valeur des Combo couleurs */
  if(window_project->current_project == NULL)
    {
      /* Valeurs par dfaut */
      window_project->color_text = COLOR_DISPLAY_CODE_WHITE;
      window_project->color_background = COLOR_DISPLAY_CODE_MEDIUMBLUE;
      window_project->color_border = COLOR_DISPLAY_CODE_MEDIUMBLUE;
    }
  else
    {
      /* Valeur du projet */
      window_project->color_text = window_project->current_project->color_text;
      window_project->color_background = window_project->current_project->color_background;
      window_project->color_border = window_project->current_project->color_border;
    }
  SendMessage(window_project->hwnd_combo_text_color,CB_SETCURSEL,window_project->color_text,0);  
  SendMessage(window_project->hwnd_combo_background_color,CB_SETCURSEL,window_project->color_background,0);  
  SendMessage(window_project->hwnd_combo_border_color,CB_SETCURSEL,window_project->color_border,0);  

  /* Font */
  window_project->hFont = CreateFont(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,
                                     OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");
  SendMessage(window_project->hwnd_combo_text_color,WM_SETFONT,(WPARAM)window_project->hFont,TRUE);
  SendMessage(window_project->hwnd_combo_background_color,WM_SETFONT,(WPARAM)window_project->hFont,TRUE);
  SendMessage(window_project->hwnd_combo_border_color,WM_SETFONT,(WPARAM)window_project->hFont,TRUE);

  /* Processeur */
  if(window_project->current_project == NULL)
    {
      /* Valeurs par dfaut */
      window_project->proc_a = 0;
      window_project->proc_x = 0;
      window_project->proc_y = 0;
      window_project->proc_dbr = 0;
      window_project->proc_pbr = 0;
      window_project->proc_pc = 0x0300;
      window_project->proc_stack = 0x01FF;
      window_project->proc_directpage = 0;
    }
  else
    {
      /* Valeurs du projet */
      window_project->proc_a = window_project->current_project->proc_a;
      window_project->proc_x = window_project->current_project->proc_x;
      window_project->proc_y = window_project->current_project->proc_y;
      window_project->proc_dbr = window_project->current_project->proc_dbr;
      window_project->proc_pbr = window_project->current_project->proc_pbr;
      window_project->proc_pc = window_project->current_project->proc_pc;
      window_project->proc_stack = window_project->current_project->proc_stack;
      window_project->proc_directpage = window_project->current_project->proc_directpage;
    }
  sprintf(buffer_hexa,"%04X",window_project->proc_a);
  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_A),buffer_hexa);
  sprintf(buffer_hexa,"%04X",window_project->proc_x);
  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_X),buffer_hexa);
  sprintf(buffer_hexa,"%04X",window_project->proc_y);
  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_Y),buffer_hexa);
  sprintf(buffer_hexa,"%02X",window_project->proc_dbr);
  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_DBR),buffer_hexa);
  sprintf(buffer_hexa,"%02X",window_project->proc_pbr);
  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_PBR),buffer_hexa);
  sprintf(buffer_hexa,"%04X",window_project->proc_pc);
  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_PC),buffer_hexa);
  sprintf(buffer_hexa,"%04X",window_project->proc_stack);
  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_STACK),buffer_hexa);
  sprintf(buffer_hexa,"%04X",window_project->proc_directpage);
  SetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_DPAGE),buffer_hexa);

  /* Status Register */
  if(window_project->current_project == NULL)
    {
      /* Valeurs par dfaut */
      window_project->status_c = 0;
      window_project->status_z = 0;
      window_project->status_i = 1;
      window_project->status_d = 0;
      window_project->status_x = 1;
      window_project->status_m = 1;
      window_project->status_v = 0;
      window_project->status_n = 0;
      window_project->status_e = 1;
      window_project->status_b = 0;
    }
  else
    {
      /* Valeurs du projet */
      window_project->status_c = window_project->current_project->status_c;
      window_project->status_z = window_project->current_project->status_z;
      window_project->status_i = window_project->current_project->status_i;
      window_project->status_d = window_project->current_project->status_d;
      window_project->status_x = window_project->current_project->status_x;
      window_project->status_m = window_project->current_project->status_m;
      window_project->status_v = window_project->current_project->status_v;
      window_project->status_n = window_project->current_project->status_n;
      window_project->status_e = window_project->current_project->status_e;
      window_project->status_b = window_project->current_project->status_b;
    }
  SetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_C,window_project->status_c,FALSE);
  SetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_Z,window_project->status_z,FALSE);
  SetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_I,window_project->status_i,FALSE);
  SetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_D,window_project->status_d,FALSE);
  SetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_X,window_project->status_x,FALSE);
  SetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_M,window_project->status_m,FALSE);
  SetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_V,window_project->status_v,FALSE);
  SetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_N,window_project->status_n,FALSE);
  SetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_E,window_project->status_e,FALSE);
  SetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_B,window_project->status_b,FALSE);

  /* RAM Size */
  if(window_project->current_project == NULL)
    window_project->ram_size = 16;     /* 1024 KB */
  else
    window_project->ram_size = window_project->current_project->ram_size;
  SetDlgItemInt(window_project->hwnd_project,IDC_EDIT_RAM_SIZE,window_project->ram_size,FALSE);

  /* Apple II ROM */
  if(window_project->current_project == NULL)
    window_project->appleii_rom = 1;
  else
    window_project->appleii_rom = window_project->current_project->appleii_rom;
  if(window_project->appleii_rom == 1)
    SendDlgItemMessage(window_project->hwnd_project,IDC_CHECK_APPLEII_ROM,BM_SETCHECK,1,0);
  else
    SendDlgItemMessage(window_project->hwnd_project,IDC_CHECK_APPLEII_ROM,BM_SETCHECK,0,0);

  /* Apple IIgs ROM */
  if(window_project->current_project == NULL)
    window_project->appleiigs_rom = PROJECT_APPLEIIGS_ROM01;
  else
    window_project->appleiigs_rom = window_project->current_project->appleiigs_rom;
  if(window_project->appleiigs_rom == PROJECT_APPLEIIGS_ROM01)
    {
      SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_ROM01,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_ROM03,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_NOROM,BM_SETCHECK,0,0);
    }
  else if(window_project->appleiigs_rom == PROJECT_APPLEIIGS_ROM03)
    {
      SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_ROM01,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_ROM03,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_NOROM,BM_SETCHECK,0,0);
    }
  else    /* NO ROM */
    {
      SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_ROM01,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_ROM03,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_project->hwnd_project,IDC_RADIO_APPLEIIGS_NOROM,BM_SETCHECK,1,0);
    }

  /* OK */
  return(0);
}


/****************************************************************************************/
/*  subEditProjectProcControl() :  Rcupre le RETURN pour valider les donnes saisies. */
/****************************************************************************************/
LRESULT CALLBACK subEditProjectProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_project *window_project;
  my_Memory(MEMORY_GET_WINDOW_PROJECT,&window_project,NULL);

  /* Rien  faire */
  if(window_project == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_A) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_X) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_Y) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_DBR) == hwnd_control)
    index = 4;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_PBR) == hwnd_control)
    index = 5;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_PC) == hwnd_control)
    index = 6;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_STACK) == hwnd_control)
    index = 7;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_DPAGE) == hwnd_control)
    index = 8;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_STATUS_C) == hwnd_control)
    index = 9;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_STATUS_Z) == hwnd_control)
    index = 10;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_STATUS_I) == hwnd_control)
    index = 11;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_STATUS_D) == hwnd_control)
    index = 12;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_STATUS_X) == hwnd_control)
    index = 13;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_STATUS_M) == hwnd_control)
    index = 14;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_STATUS_V) == hwnd_control)
    index = 15;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_STATUS_N) == hwnd_control)
    index = 16;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_STATUS_E) == hwnd_control)
    index = 17;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_STATUS_B) == hwnd_control)
    index = 18;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_RAM_SIZE) == hwnd_control)
    index = 19;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE0_BANK) == hwnd_control)
    index = 20;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE0_ADDRESS) == hwnd_control)
    index = 21;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE0_PATH) == hwnd_control)
    index = 22;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE1_BANK) == hwnd_control)
    index = 23;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE1_ADDRESS) == hwnd_control)
    index = 24;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE1_PATH) == hwnd_control)
    index = 25;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE2_BANK) == hwnd_control)
    index = 26;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE2_ADDRESS) == hwnd_control)
    index = 27;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE2_PATH) == hwnd_control)
    index = 28;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE3_BANK) == hwnd_control)
    index = 29;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE3_ADDRESS) == hwnd_control)
    index = 30;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE3_PATH) == hwnd_control)
    index = 31;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE4_BANK) == hwnd_control)
    index = 32;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE4_ADDRESS) == hwnd_control)
    index = 33;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE4_PATH) == hwnd_control)
    index = 34;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE5_BANK) == hwnd_control)
    index = 35;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE5_ADDRESS) == hwnd_control)
    index = 36;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE5_PATH) == hwnd_control)
    index = 37;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE6_BANK) == hwnd_control)
    index = 38;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE6_ADDRESS) == hwnd_control)
    index = 39;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE6_PATH) == hwnd_control)
    index = 40;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE7_BANK) == hwnd_control)
    index = 41;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE7_ADDRESS) == hwnd_control)
    index = 42;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE7_PATH) == hwnd_control)
    index = 43;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE8_BANK) == hwnd_control)
    index = 44;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE8_ADDRESS) == hwnd_control)
    index = 45;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILE8_PATH) == hwnd_control)
    index = 46;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILESOUND_ADDRESS) == hwnd_control)
    index = 47;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILESOUND_PATH) == hwnd_control)
    index = 48;
  else if(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILEMERLIN32_PATH) == hwnd_control)
    index = 49;
  else
    return(0);      /* Pas trouv */

  /** Dtecte la touche RETURN pour valider la donne **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_project->hwnd_project);
    }
  else if(index == 1)
    return(CallWindowProc(window_project->oldEditProcProcA,hwnd_control,msg,wParam,lParam));
  else if(index == 2)
    return(CallWindowProc(window_project->oldEditProcProcX,hwnd_control,msg,wParam,lParam));
  else if(index == 3)
    return(CallWindowProc(window_project->oldEditProcProcY,hwnd_control,msg,wParam,lParam));
  else if(index == 4)
    return(CallWindowProc(window_project->oldEditProcProcDBR,hwnd_control,msg,wParam,lParam));
  else if(index == 5)
    return(CallWindowProc(window_project->oldEditProcProcPBR,hwnd_control,msg,wParam,lParam));
  else if(index == 6)
    return(CallWindowProc(window_project->oldEditProcProcPC,hwnd_control,msg,wParam,lParam));
  else if(index == 7)
    return(CallWindowProc(window_project->oldEditProcProcStack,hwnd_control,msg,wParam,lParam));  
  else if(index == 8)
    return(CallWindowProc(window_project->oldEditProcProcDPage,hwnd_control,msg,wParam,lParam));
  else if(index == 9)
    return(CallWindowProc(window_project->oldEditProcStatusC,hwnd_control,msg,wParam,lParam));
  else if(index == 10)
    return(CallWindowProc(window_project->oldEditProcStatusZ,hwnd_control,msg,wParam,lParam));
  else if(index == 11)
    return(CallWindowProc(window_project->oldEditProcStatusI,hwnd_control,msg,wParam,lParam));
  else if(index == 12)
    return(CallWindowProc(window_project->oldEditProcStatusD,hwnd_control,msg,wParam,lParam));
  else if(index == 13)
    return(CallWindowProc(window_project->oldEditProcStatusX,hwnd_control,msg,wParam,lParam));
  else if(index == 14)
    return(CallWindowProc(window_project->oldEditProcStatusM,hwnd_control,msg,wParam,lParam));
  else if(index == 15)
    return(CallWindowProc(window_project->oldEditProcStatusV,hwnd_control,msg,wParam,lParam));
  else if(index == 16)
    return(CallWindowProc(window_project->oldEditProcStatusN,hwnd_control,msg,wParam,lParam));
  else if(index == 17)
    return(CallWindowProc(window_project->oldEditProcStatusE,hwnd_control,msg,wParam,lParam));
  else if(index == 18)
    return(CallWindowProc(window_project->oldEditProcStatusB,hwnd_control,msg,wParam,lParam));
  else if(index == 19)
    return(CallWindowProc(window_project->oldEditProcRamSize,hwnd_control,msg,wParam,lParam));
  else if(index == 20)
    return(CallWindowProc(window_project->oldEditProcFile0Bank,hwnd_control,msg,wParam,lParam));
  else if(index == 21)
    return(CallWindowProc(window_project->oldEditProcFile0Address,hwnd_control,msg,wParam,lParam));
  else if(index == 22)
    return(CallWindowProc(window_project->oldEditProcFile0Path,hwnd_control,msg,wParam,lParam));
  else if(index == 23)
    return(CallWindowProc(window_project->oldEditProcFile1Bank,hwnd_control,msg,wParam,lParam));
  else if(index == 24)
    return(CallWindowProc(window_project->oldEditProcFile1Address,hwnd_control,msg,wParam,lParam));
  else if(index == 25)
    return(CallWindowProc(window_project->oldEditProcFile1Path,hwnd_control,msg,wParam,lParam));
  else if(index == 26)
    return(CallWindowProc(window_project->oldEditProcFile2Bank,hwnd_control,msg,wParam,lParam));
  else if(index == 27)
    return(CallWindowProc(window_project->oldEditProcFile2Address,hwnd_control,msg,wParam,lParam));
  else if(index == 28)
    return(CallWindowProc(window_project->oldEditProcFile2Path,hwnd_control,msg,wParam,lParam));
  else if(index == 29)
    return(CallWindowProc(window_project->oldEditProcFile3Bank,hwnd_control,msg,wParam,lParam));
  else if(index == 30)
    return(CallWindowProc(window_project->oldEditProcFile3Address,hwnd_control,msg,wParam,lParam));
  else if(index == 31)
    return(CallWindowProc(window_project->oldEditProcFile3Path,hwnd_control,msg,wParam,lParam));
  else if(index == 32)
    return(CallWindowProc(window_project->oldEditProcFile4Bank,hwnd_control,msg,wParam,lParam));
  else if(index == 33)
    return(CallWindowProc(window_project->oldEditProcFile4Address,hwnd_control,msg,wParam,lParam));
  else if(index == 34)
    return(CallWindowProc(window_project->oldEditProcFile4Path,hwnd_control,msg,wParam,lParam));
  else if(index == 35)
    return(CallWindowProc(window_project->oldEditProcFile5Bank,hwnd_control,msg,wParam,lParam));
  else if(index == 36)
    return(CallWindowProc(window_project->oldEditProcFile5Address,hwnd_control,msg,wParam,lParam));
  else if(index == 37)
    return(CallWindowProc(window_project->oldEditProcFile5Path,hwnd_control,msg,wParam,lParam));
  else if(index == 38)
    return(CallWindowProc(window_project->oldEditProcFile6Bank,hwnd_control,msg,wParam,lParam));
  else if(index == 39)
    return(CallWindowProc(window_project->oldEditProcFile6Address,hwnd_control,msg,wParam,lParam));
  else if(index == 40)
    return(CallWindowProc(window_project->oldEditProcFile6Path,hwnd_control,msg,wParam,lParam));
  else if(index == 41)
    return(CallWindowProc(window_project->oldEditProcFile7Bank,hwnd_control,msg,wParam,lParam));
  else if(index == 42)
    return(CallWindowProc(window_project->oldEditProcFile7Address,hwnd_control,msg,wParam,lParam));
  else if(index == 43)
    return(CallWindowProc(window_project->oldEditProcFile7Path,hwnd_control,msg,wParam,lParam));
  else if(index == 44)
    return(CallWindowProc(window_project->oldEditProcFile8Bank,hwnd_control,msg,wParam,lParam));
  else if(index == 45)
    return(CallWindowProc(window_project->oldEditProcFile8Address,hwnd_control,msg,wParam,lParam));
  else if(index == 46)
    return(CallWindowProc(window_project->oldEditProcFile8Path,hwnd_control,msg,wParam,lParam));
  else if(index == 47)
    return(CallWindowProc(window_project->oldEditProcFileSoundAddress,hwnd_control,msg,wParam,lParam));
  else if(index == 48)
    return(CallWindowProc(window_project->oldEditProcFileSoundPath,hwnd_control,msg,wParam,lParam));
  else if(index == 49)
    return(CallWindowProc(window_project->oldEditProcFileMerlin32Path,hwnd_control,msg,wParam,lParam));
    
  /* OK */
  return(0);
}


/*********************************************************************/
/*  GetProjectValueFromWindow() :  Destruction des fentres enfants. */
/*********************************************************************/
static struct project *GetProjectValueFromWindow(struct window_project *window_project)
{
  int i, j, number, is_error, nb_file, highest_address, pc_address, pc_in_file;
  int file_address_min, file_address_max, file_other_address_min, file_other_address_max, has_recover_file;
  BYTE byte;
  WORD word;
  LRESULT result;
  char buffer[BUFFER_SIZE];
  char buffer_file_path[BUFFER_SIZE];
  char error_buffer[BUFFER_SIZE];
  struct project *current_project;
  struct file_path *current_filepath;
  int tab_edit_file_path[9] = 
    {
      IDC_EDIT_FILE0_PATH, IDC_EDIT_FILE1_PATH, IDC_EDIT_FILE2_PATH,
      IDC_EDIT_FILE3_PATH, IDC_EDIT_FILE4_PATH, IDC_EDIT_FILE5_PATH,
      IDC_EDIT_FILE6_PATH, IDC_EDIT_FILE7_PATH, IDC_EDIT_FILE8_PATH
    };
  int tab_edit_file_bank[9] = 
    {
      IDC_EDIT_FILE0_BANK, IDC_EDIT_FILE1_BANK, IDC_EDIT_FILE2_BANK,
      IDC_EDIT_FILE3_BANK, IDC_EDIT_FILE4_BANK, IDC_EDIT_FILE5_BANK,
      IDC_EDIT_FILE6_BANK, IDC_EDIT_FILE7_BANK, IDC_EDIT_FILE8_BANK
    };
  int tab_edit_file_address[9] = 
    {
      IDC_EDIT_FILE0_ADDRESS, IDC_EDIT_FILE1_ADDRESS, IDC_EDIT_FILE2_ADDRESS,
      IDC_EDIT_FILE3_ADDRESS, IDC_EDIT_FILE4_ADDRESS, IDC_EDIT_FILE5_ADDRESS,
      IDC_EDIT_FILE6_ADDRESS, IDC_EDIT_FILE7_ADDRESS, IDC_EDIT_FILE8_ADDRESS
    };
 
  /* Allocation mmoire */
  current_project = (struct project *) calloc(1,sizeof(struct project));
  if(current_project == NULL)
    return(NULL);

  /** File 0-8 **/
  for(i=0; i<9; i++)
    {
      /* File Path */
      GetWindowText(GetDlgItem(window_project->hwnd_project,tab_edit_file_path[i]),buffer_file_path,BUFFER_SIZE-1);
      if(strlen(buffer_file_path) > 0)
        {
          /* Fichier */
          current_filepath = BuildFilePath(buffer_file_path);
          if(current_filepath == NULL)
            {
              sprintf(window_project->buffer_log,"Impossible to load File '%s'",buffer_file_path);
              MessageBox(window_project->hwnd_project,window_project->buffer_log,"Load Project values",MB_OK|MB_ICONWARNING);
              mem_free_project(current_project);
              return(NULL);
            }
          current_project->tab_file[i] = current_filepath;

          /* Bank */
          GetWindowText(GetDlgItem(window_project->hwnd_project,tab_edit_file_bank[i]),buffer,BUFFER_SIZE-1);
          byte = EvaluateAsByte(buffer,&is_error);
          if(is_error)
            {
              sprintf(window_project->buffer_log,"Invalid Bank Value for File '%s' (8 bit hexadecimal is expected)",buffer_file_path);
              MessageBox(window_project->hwnd_project,window_project->buffer_log,"Load Project values",MB_OK|MB_ICONWARNING);
              mem_free_project(current_project);
              return(NULL);
            }
          current_project->tab_file_bank[i] = byte;
            
          /* Address */
          GetWindowText(GetDlgItem(window_project->hwnd_project,tab_edit_file_address[i]),buffer,BUFFER_SIZE-1);
          word = EvaluateAsWord(buffer,&is_error);
          if(is_error)
            {
              sprintf(window_project->buffer_log,"Invalid Address Value for File '%s' (16 bit hexadecimal is expected)",buffer_file_path);
              MessageBox(window_project->hwnd_project,window_project->buffer_log,"Load Project values",MB_OK|MB_ICONWARNING);
              mem_free_project(current_project);
              return(NULL);
            }
          current_project->tab_file_address[i] = word;      
        }
    }
    
  /** Ram Sound **/
  GetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILESOUND_PATH),buffer_file_path,BUFFER_SIZE-1);
  if(strlen(buffer_file_path) > 0)
    {
      /* Fichier */
      current_filepath = BuildFilePath(buffer_file_path);
      if(current_filepath == NULL)
        {
          sprintf(window_project->buffer_log,"Impossible to load File Sound '%s'",buffer_file_path);
          MessageBox(window_project->hwnd_project,window_project->buffer_log,"Load Project values",MB_OK|MB_ICONWARNING);
          mem_free_project(current_project);
          return(NULL);
        }
      current_project->file_sound = current_filepath;
        
      /* Address */
      GetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILESOUND_ADDRESS),buffer,BUFFER_SIZE-1);
      word = EvaluateAsWord(buffer,&is_error);
      if(is_error)
        {
          sprintf(window_project->buffer_log,"Invalid Address Value for File Sound '%s' (16 bit hexadecimal is expected)",buffer_file_path);
          MessageBox(window_project->hwnd_project,window_project->buffer_log,"Load Project values",MB_OK|MB_ICONWARNING);
          mem_free_project(current_project);
          return(NULL);
        }
      current_project->file_sound_address = word;
    } 

  /** Merlin 32 Files **/
  GetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_FILEMERLIN32_PATH),buffer_file_path,BUFFER_SIZE-1);  
  nb_file = GetFileNumber(buffer_file_path,0);
  my_Memory(MEMORY_GET_FILE_NB,&nb_file,NULL);
  for(i=1; i<=nb_file; i++)
    {
      my_Memory(MEMORY_GET_FILE,&i,&current_filepath);
      
      /* Charge les donnes du fichier */
      current_filepath->data = LoadTextFile(current_filepath->path,&current_filepath->size,&error_buffer[0]);
      if(current_filepath->data == NULL)
        {
          sprintf(window_project->buffer_log,"Impossible to load Merlin 32 File '%s'",current_filepath->path);
          MessageBox(window_project->hwnd_project,window_project->buffer_log,"Load Project values",MB_OK|MB_ICONWARNING);
          mem_free_project(current_project);
          return(NULL);
        }  
    }
  /* On va rattacher les file_path  la structure project */
  current_project->nb_merlin32_file = nb_file;
  if(nb_file > 0)
    {
      /* First */
      i = 1;
      my_Memory(MEMORY_GET_FILE,&i,&current_filepath);
      current_project->first_merlin32_file = current_filepath;
      
      /* Last */
      i = nb_file;
      my_Memory(MEMORY_GET_FILE,&i,&current_filepath);
      current_project->last_merlin32_file = current_filepath;
      
      /* Dtache cette liste de my_Memory() */
      my_Memory(MEMORY_INIT_FILE,NULL,NULL);
    }

  /** Processor **/
  /* A */
  GetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_A),buffer,BUFFER_SIZE-1);
  word = EvaluateAsWord(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_project->hwnd_project,"Invalid Processor Value : A (16 bit hexadecimal is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->proc_a = word;
  /* X */
  GetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_X),buffer,BUFFER_SIZE-1);
  word = EvaluateAsWord(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_project->hwnd_project,"Invalid Processor Value : X (16 bit hexadecimal is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->proc_x = word;
  /* Y */
  GetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_Y),buffer,BUFFER_SIZE-1);
  word = EvaluateAsWord(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_project->hwnd_project,"Invalid Processor Value : Y (16 bit hexadecimal is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->proc_y = word;
  /* DBR */
  GetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_DBR),buffer,BUFFER_SIZE-1);
  byte = EvaluateAsByte(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_project->hwnd_project,"Invalid Processor Value : DBR (8 bit hexadecimal is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->proc_dbr = byte;
  /* PBR */
  GetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_PBR),buffer,BUFFER_SIZE-1);
  byte = EvaluateAsByte(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_project->hwnd_project,"Invalid Processor Value : PBR (8 bit hexadecimal is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->proc_pbr = byte;
  /* PC */
  GetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_PC),buffer,BUFFER_SIZE-1);
  word = EvaluateAsWord(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_project->hwnd_project,"Invalid Processor Value : PC (16 bit hexadecimal is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->proc_pc = word;  
  /* Stack */
  GetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_STACK),buffer,BUFFER_SIZE-1);
  word = EvaluateAsWord(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_project->hwnd_project,"Invalid Processor Value : Stack (16 bit hexadecimal is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->proc_stack = word;
  /* Direct Page */
  GetWindowText(GetDlgItem(window_project->hwnd_project,IDC_EDIT_PROC_DPAGE),buffer,BUFFER_SIZE-1);
  word = EvaluateAsWord(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_project->hwnd_project,"Invalid Processor Value : Direct Page (16 bit hexadecimal is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->proc_directpage = word;
  
  /** Status Register **/
  /* C */
  number = GetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_C,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_project->hwnd_project,"Invalid Status Register Value : C (0 or 1 is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->status_c = number;
  /* Z */
  number = GetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_Z,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_project->hwnd_project,"Invalid Status Register Value : Z (0 or 1 is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->status_z = number;
  /* I */
  number = GetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_I,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_project->hwnd_project,"Invalid Status Register Value : I (0 or 1 is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->status_i = number;
  /* D */
  number = GetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_D,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_project->hwnd_project,"Invalid Status Register Value : D (0 or 1 is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->status_d = number;
  /* X */
  number = GetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_X,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_project->hwnd_project,"Invalid Status Register Value : X (0 or 1 is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->status_x = number;
  /* M */
  number = GetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_M,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_project->hwnd_project,"Invalid Status Register Value : M (0 or 1 is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->status_m = number;
  /* V */
  number = GetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_V,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_project->hwnd_project,"Invalid Status Register Value : V (0 or 1 is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->status_v = number;
  /* N */
  number = GetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_N,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_project->hwnd_project,"Invalid Status Register Value : N (0 or 1 is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->status_n = number;
  /* E */
  number = GetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_E,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_project->hwnd_project,"Invalid Status Register Value : E (0 or 1 is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->status_e = number;
  /* B */
  number = GetDlgItemInt(window_project->hwnd_project,IDC_EDIT_STATUS_B,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_project->hwnd_project,"Invalid Status Register Value : B (0 or 1 is expected)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->status_b = number;
  
  /** Ram Size **/
  number = GetDlgItemInt(window_project->hwnd_project,IDC_EDIT_RAM_SIZE,NULL,FALSE);
  if(number > 128)
    {
      MessageBox(window_project->hwnd_project,"Invalid Ram Size value : 128 is the maximum value (8 MB)","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
  current_project->ram_size = number;
  
  /* Apple II Rom */
  current_project->appleii_rom = window_project->appleii_rom;

  /* Apple IIgs ROM */
  current_project->appleiigs_rom = window_project->appleiigs_rom;

  /** Display Preferences **/
  /* Text Color */
  current_project->color_text = COLOR_DISPLAY_CODE_WHITE;
  result = SendMessage(window_project->hwnd_combo_text_color,CB_GETCURSEL,0,0);
  if(result != CB_ERR)
    current_project->color_text = result;
  /* Background Color */
  current_project->color_background = COLOR_DISPLAY_CODE_MEDIUMBLUE;
  result = SendMessage(window_project->hwnd_combo_background_color,CB_GETCURSEL,0,0);
  if(result != CB_ERR)
    current_project->color_background = result;
  /* Border Color */
  current_project->color_border = COLOR_DISPLAY_CODE_MEDIUMBLUE;
  result = SendMessage(window_project->hwnd_combo_border_color,CB_GETCURSEL,0,0);
  if(result != CB_ERR)
    current_project->color_border = result;
  
  /** Y a t'il au moins 1 fichier de Data charg en mmoire ? **/
  if(current_project->tab_file[0] == NULL && current_project->tab_file[1] == NULL && current_project->tab_file[2] == NULL && 
     current_project->tab_file[3] == NULL && current_project->tab_file[4] == NULL && current_project->tab_file[5] == NULL && 
     current_project->tab_file[6] == NULL && current_project->tab_file[7] == NULL && current_project->tab_file[8] == NULL)
    {
      MessageBox(window_project->hwnd_project,"You need to load at least one file in memory !","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }

  /** Vrifie la cohrence entre la taille mmoire et les addresses de chargement des fichiers **/
  highest_address = 2*64*1024 + current_project->ram_size*64*1024;
  for(i=0; i<9; i++)
    {
      if(current_project->tab_file[i] != NULL)
        {
          file_address_max = current_project->tab_file_bank[i]*64*1024 + current_project->tab_file_address[i] + current_project->tab_file[i]->size;
          if(file_address_max > highest_address)
            {
              sprintf(window_project->buffer_log,"The loading address for file '%s' is incompatible with Ram size",current_project->tab_file[i]->path);
              MessageBox(window_project->hwnd_project,window_project->buffer_log,"Load Project values",MB_OK|MB_ICONWARNING);
              mem_free_project(current_project);
              return(NULL);
            }
        }
    }

  /** Vrifie les recouvrements des donnes des fichiers en mmoire **/
  has_recover_file = 0;
  for(i=0; i<9; i++)
    {
      /* Recherche les recouvrement pour ce fichier */
      if(current_project->tab_file[i] != NULL)
        {
          file_address_min = current_project->tab_file_bank[i]*64*1024 + current_project->tab_file_address[i];
          file_address_max = file_address_min + current_project->tab_file[i]->size;
                
          /* Regarde dans les fichiers suivants */    
          for(j=i+1; j<9; j++)
            {
              if(current_project->tab_file[j] != NULL)
                {
                  file_other_address_min = current_project->tab_file_bank[j]*64*1024 + current_project->tab_file_address[j];
                  file_other_address_max = file_other_address_min + current_project->tab_file[j]->size;
                  
                  /* Recherche de recouvrement */
                  if((file_other_address_min >= file_address_min && file_other_address_min <= file_address_max) ||
                     (file_other_address_max >= file_address_min && file_other_address_max <= file_address_max))
                    {
                      has_recover_file = 1;
                      break;
                    }
                }
            }
          if(has_recover_file == 1)
            break;
        }
    }
  if(has_recover_file == 1)
    MessageBox(window_project->hwnd_project,"Warning ! Some files data are overlapping in memory","Load Project values",MB_OK|MB_ICONWARNING);

  /** Le PC pointe t'il vers une zone mmoire o a t tlcharg un fichier ? **/
  pc_in_file = 0;
  pc_address = current_project->proc_pbr*64*1024 + current_project->proc_pc;
  for(i=0; i<9; i++)
    {
      if(current_project->tab_file[i] != NULL)
        {
          file_address_min = current_project->tab_file_bank[i]*64*1024 + current_project->tab_file_address[i];
          file_address_max = file_address_min + current_project->tab_file[i]->size;
          if(pc_address >= file_address_min && pc_address < file_address_max)
            pc_in_file = 1;
        }
    }
  if(pc_in_file == 0)
    {
      MessageBox(window_project->hwnd_project,"The PC value is not located in one of the loaded file !","Load Project values",MB_OK|MB_ICONWARNING);
      mem_free_project(current_project);
      return(NULL);
    }
    
  /* Nom du fichier (provenant d'un projet charg en mmoire) */
  if(window_project->current_project != NULL)
    if(strlen(window_project->current_project->project_file_path) > 0)
      strcpy(current_project->project_file_path,window_project->current_project->project_file_path);

  /* Renvoi la structure */
  return(current_project);
}


/*********************************************************************/
/*  DeleteProjectChildWindows() :  Destruction des fentres enfants. */
/*********************************************************************/
static void DeleteProjectChildWindows(struct window_project *window_project)
{
  DeleteObject(window_project->hwnd_combo_text_color);
  DeleteObject(window_project->hwnd_combo_background_color);
  DeleteObject(window_project->hwnd_combo_border_color);
  DeleteObject(window_project->hFont);
}


/***********************************************************************/
/*  mem_free_window_project() :  Libration mmoire pour la structure. */
/***********************************************************************/
void mem_free_window_project(struct window_project *window_project)
{
  if(window_project)
    {
      if(window_project->current_project != NULL)
        mem_free_project(window_project->current_project);

      free(window_project);
    }
}

/**********************************************************************/
