/*********************************************************************/
/*                                                                   */
/*  Window_RamSound.c :  Module de gestion de la fentre RamSound.   */
/*                                                                   */
/********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Juin 2018  */
/********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_RamSound.h"

static DWORD BuildRamSoundWindowThread(void *);
LRESULT CALLBACK ProcHandlerRamSoundWindow(HWND,UINT,WPARAM,LPARAM);
LRESULT CALLBACK subEditRamSoundProcControl(HWND,UINT,WPARAM,LPARAM);
static int CreateRamSoundChildWindows(HWND,struct window_ramsound *);
static void UpdateRamSoundContextInformation(struct window_ramsound *,int,int);
static void UpdateRamSoundAddress(struct window_ramsound *,int);
static void UpdateRamSoundRefreshMode(struct window_ramsound *,int);
static void RefreshRamSoundWindowData(struct window_ramsound *);
static int IsRamSoundDataModified(int,int,int,struct window_ramsound *);
static void UserResizeRamSoundWindow(struct window_ramsound *);
static void ResizeRamSoundWindow(struct window_ramsound *);
static void BuildRamSoundFromData(struct window_ramsound *);
static DWORD CALLBACK RamSoundRichEditStreamCallback(DWORD *,LPBYTE,LONG,LONG *);
static void DeleteRamSoundChildWindows(struct window_ramsound *);


/******************************************************************************/
/*  BuildRamSoundWindow() :  Construction / Affichage de la fentre RamSound. */
/******************************************************************************/
struct window_ramsound *BuildRamSoundWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_ramsound *window_ramsound;

  /** Allocation de la structure window_ramsound **/
  window_ramsound = (struct window_ramsound *) calloc(1,sizeof(struct window_ramsound));
  if(window_ramsound == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_RAMSOUND,window_ramsound,NULL);

  /* Lance un thread pour grer la fentre RamSound */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildRamSoundWindowThread,window_ramsound,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_ramsound);
}


/****************************************************************************************/
/*  BuildRamSoundWindowThread() :  Cration de la fentre RamSound via un Thread ddi. */
/****************************************************************************************/
static DWORD BuildRamSoundWindowThread(void *window_ramsound_param)
{
  HWND hwndRamSound;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_ramsound *window_ramsound;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_ramsound = (struct window_ramsound *) window_ramsound_param;
  window_ramsound->thread_id = GetCurrentThreadId();         /* Le ThreaID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Memory Area **/
  hwndRamSound = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_RAMSOUND),window_main->hwnd_main,(DLGPROC)ProcHandlerRamSoundWindow);
  if(hwndRamSound == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndRamSound,SW_SHOWDEFAULT);
  UpdateWindow(hwndRamSound);

  /** Message Loop pour toutes les fentres de ce Thread (fentre RamSound...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/*************************************************************************/
/*  ProcHandlerRamSoundWindow() :  Gestion des vnements de la fentre. */
/*************************************************************************/
LRESULT CALLBACK ProcHandlerRamSoundWindow(HWND hwndRamSound, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  int mouse_x, mouse_y;
  MSGFILTER *message_filter;
  struct window_ramsound *window_ramsound;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_RAMSOUND,&window_ramsound,NULL);
  if(window_ramsound == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateRamSoundChildWindows(hwndRamSound,window_ramsound);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        SetWindowText(GetDlgItem(window_ramsound->hwnd_ramsound,IDC_STATIC_CONTEXT),"");
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_ramsound->refresh_mode != RAMSOUND_REFRESH_AUTOMATIC)
                UpdateRamSoundRefreshMode(window_ramsound,RAMSOUND_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_ramsound->refresh_mode != RAMSOUND_REFRESH_MANUAL)
                UpdateRamSoundRefreshMode(window_ramsound,RAMSOUND_REFRESH_MANUAL);
              break;

            /** Bouton de dplacement de l'adresse **/
            case IDC_BUTTON_LEFT :
              UpdateRamSoundAddress(window_ramsound,-1);
              break;

            case IDC_BUTTON_RIGHT :
              UpdateRamSoundAddress(window_ramsound,1);
              break;

            case IDC_BUTTON_UP :
              UpdateRamSoundAddress(window_ramsound,(-1)*window_ramsound->nb_column);
              break;

            case IDC_BUTTON_DOWN :
              UpdateRamSoundAddress(window_ramsound,window_ramsound->nb_column);
              break;

            case IDC_BUTTON_PAGEUP :
              UpdateRamSoundAddress(window_ramsound,(-1)*window_ramsound->nb_column*window_ramsound->nb_row);
              break;

            case IDC_BUTTON_PAGEDOWN :
              UpdateRamSoundAddress(window_ramsound,window_ramsound->nb_column*window_ramsound->nb_row);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshRamSoundWindowData(window_ramsound);
              break;
          }
        return(TRUE);

      /**********************************************/
      /* Evnements des Controles  la fentre mre */
      /**********************************************/
      case WM_NOTIFY :
        message_filter = (MSGFILTER *) lParam;
        
        /** Evnement de la souris dans le Rich Edit transmis  la fentre mre **/
        if(message_filter && message_filter->nmhdr.hwndFrom == window_ramsound->hwnd_richedit_data)
          {
            mouse_x = GET_X_LPARAM(message_filter->lParam) / 7;
            mouse_y = GET_Y_LPARAM(message_filter->lParam) / RAMSOUND_RICHEDIT_FONT_HEIGHT;
            UpdateRamSoundContextInformation(window_ramsound,mouse_x,mouse_y);
          }
        return(FALSE);
        
      /**************************************/
      /*  On vient de retailler la fentre  */
      /**************************************/
      case WM_SIZE :
        /* On va tendre la fentre de donne mmoire si c'est un Resize User */
        if(window_ramsound->user_resize == 1)
          UserResizeRamSoundWindow(window_ramsound);
        window_ramsound->user_resize = 1;
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_RAMSOUND_REFRESHWINDOW :
        RefreshRamSoundWindowData(window_ramsound);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre RamSound */
        DestroyWindow(hwndRamSound);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteRamSoundChildWindows(window_ramsound);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_RAMSOUND,window_ramsound,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*******************************************************************************************************/
/*  CreateRamSoundChildWindows() : Crer les fentres enfants et remplir la structure window_ramsound. */
/*******************************************************************************************************/
static int CreateRamSoundChildWindows(HWND hwndRamSound, struct window_ramsound *window_ramsound)
{
  HWND hwnd_group;
  DWORD dialog_background_color;
  RECT client_rect, window_rect, group_rect;
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_ramsound->hwnd_ramsound = hwndRamSound;

  /** Surchage des Control Edit pour capter les vnements **/
  window_ramsound->oldEditProcAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndRamSound,IDC_EDIT_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditRamSoundProcControl);
  window_ramsound->oldEditProcColumn  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndRamSound,IDC_EDIT_COLUMN),GWLP_WNDPROC,(LONG_PTR)subEditRamSoundProcControl);
  window_ramsound->oldEditProcRow     = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndRamSound,IDC_EDIT_ROW),GWLP_WNDPROC,(LONG_PTR)subEditRamSoundProcControl);

  /** Ajoute les images sur les boutons **/
  SendMessage(GetDlgItem(hwndRamSound,IDC_BUTTON_LEFT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndRamSound,IDC_BUTTON_RIGHT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndRamSound,IDC_BUTTON_UP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndRamSound,IDC_BUTTON_DOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndRamSound,IDC_BUTTON_PAGEUP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndRamSound,IDC_BUTTON_PAGEDOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Adresse de la Zone de Data **/
  window_ramsound->ramsound_address = 0x0000; 
  sprintf(buffer,"%04X",window_ramsound->ramsound_address);
  SetDlgItemText(window_ramsound->hwnd_ramsound,IDC_EDIT_ADDRESS,buffer);

  /** Zone d'affichage des Ram Sound Data **/
  /* Taille minimale */
  window_ramsound->text_min_data_column = 1;
  window_ramsound->text_min_data_row    = 1;
  /* Taille par dfaut */
  window_ramsound->nb_column = 16;
  window_ramsound->nb_row = 16; 
  SetDlgItemInt(window_ramsound->hwnd_ramsound,IDC_EDIT_COLUMN,window_ramsound->nb_column,FALSE);
  SetDlgItemInt(window_ramsound->hwnd_ramsound,IDC_EDIT_ROW,window_ramsound->nb_row,FALSE);

  /* Nombre caractres visibles sur la ligne : Address Data Ascii (on ne compte pas le \r\n final) */
  window_ramsound->text_data_width = (int)(strlen("0000   ") + window_ramsound->nb_column*3 - 1);   /* Enlve le dernier sparateur */

  /* Refresh Mode : Automatic */
  UpdateRamSoundRefreshMode(window_ramsound,RAMSOUND_REFRESH_AUTOMATIC);

  /* Hauteur Titre */
  GetWindowRect(window_ramsound->hwnd_ramsound,&window_rect);
  GetClientRect(window_ramsound->hwnd_ramsound,&client_rect);
  window_ramsound->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select + GroupBox Palette */
  hwnd_group = GetDlgItem(window_ramsound->hwnd_ramsound,IDC_GROUP_RAMSOUND);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_RAMSOUND */
  GetWindowRect(hwnd_group,&group_rect);
  window_ramsound->group_data_x = group_rect.left - window_rect.left;
  window_ramsound->group_data_y = (group_rect.top - window_rect.top) - window_ramsound->title_height;

  /* Position de la Zone Text Direct Page Data dans la fenetre */
  window_ramsound->ramsound_data_x = window_ramsound->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_ramsound->ramsound_data_y = window_ramsound->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Text RamSound Data */
  window_ramsound->ramsound_data_width = window_ramsound->text_data_width*7 + 2;
  window_ramsound->ramsound_data_height = window_ramsound->nb_row*RAMSOUND_EDIT_FONT_HEIGHT;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_ramsound->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_ramsound->ramsound_data_width + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_ramsound->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_ramsound->ramsound_data_height + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_ramsound->group_data_x,window_ramsound->group_data_y,window_ramsound->group_data_width,window_ramsound->group_data_height,FALSE);

  /** Cration du Rich Edit **/
  window_ramsound->hwnd_richedit_data = CreateWindowEx(0,RICHEDIT_CLASS,"",WS_CHILD | WS_VISIBLE | ES_SAVESEL | ES_MULTILINE | ES_READONLY, 
                                                       window_ramsound->ramsound_data_x, window_ramsound->ramsound_data_y, window_ramsound->ramsound_data_width, window_ramsound->ramsound_data_height,   // set size in WM_SIZE message 
                                                       window_ramsound->hwnd_ramsound,
                                                       (HMENU)IDC_RICHEDIT_RAMSOUND_DATA,
                                                       param->hInstance,
                                                       NULL);
  
  /* Couleur de Fond du control Rich Edit : Gris comme la fentre d'accueil */
  dialog_background_color = GetSysColor(COLOR_3DFACE);
  SendMessage(window_ramsound->hwnd_richedit_data,EM_SETBKGNDCOLOR,0,(LPARAM)dialog_background_color); 
  SendMessage(window_ramsound->hwnd_richedit_data,EM_SETEVENTMASK,0,(LPARAM)ENM_MOUSEEVENTS);

  /* Retaille la fentre */
  ResizeRamSoundWindow(window_ramsound);

  /** Construction du Texte + Rafraichissement de la fentre **/
  RefreshRamSoundWindowData(window_ramsound);
    
  /* OK */
  return(0);
}


/***********************************************************************************************/
/*  subEditRamSoundProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/***********************************************************************************************/
LRESULT CALLBACK subEditRamSoundProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_ramsound *window_ramsound;
  my_Memory(MEMORY_GET_WINDOW_RAMSOUND,&window_ramsound,NULL);

  /* Rien  faire */
  if(window_ramsound == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_ramsound->hwnd_ramsound,IDC_EDIT_ADDRESS) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_ramsound->hwnd_ramsound,IDC_EDIT_COLUMN) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_ramsound->hwnd_ramsound,IDC_EDIT_ROW) == hwnd_control)
    index = 3;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement de la souris dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_ramsound->hwnd_ramsound);

      /* On force la MAJ de la fentre */
      RefreshRamSoundWindowData(window_ramsound);
    }
  else if(index == 1)
    return(CallWindowProc(window_ramsound->oldEditProcAddress,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_ramsound->oldEditProcColumn,hwnd_control,msg,wParam,lParam)); 
  else if(index == 3)
    return(CallWindowProc(window_ramsound->oldEditProcRow,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
}


/***************************************************************************/
/*  UpdateRamSoundAddress() :  Dplace l'adresse pour la fentre RamSound. */
/***************************************************************************/
static void UpdateRamSoundAddress(struct window_ramsound *window_ramsound, int delta)
{
  int current_address, new_address, ramsound_address, data_size;
  char buffer[BUFFER_SIZE];

  /* Taille des Data */
  data_size = window_ramsound->nb_column*window_ramsound->nb_row;

  /* Adresse actuelle */
  current_address = window_ramsound->ramsound_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFF)
    return;
  if((current_address + delta + data_size) > 0x010000)   /* 64 KB */
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  ramsound_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%04X",ramsound_address);
  SetDlgItemText(window_ramsound->hwnd_ramsound,IDC_EDIT_ADDRESS,buffer);

  /* MAJ de la fentre */
  RefreshRamSoundWindowData(window_ramsound);
}


/***************************************************************************************************/
/*  UpdateRamSoundContextInformation() :  MAJ des information de contexte de la fenetre RamSound. */
/***************************************************************************************************/
static void UpdateRamSoundContextInformation(struct window_ramsound *window_ramsound, int grid_x, int grid_y)
{
  int grid_ramsound_x, ramsound_data_address, grid_ramsound_offset, ramsound_data_offset;
  unsigned char ramsound_data;

  /* Rien  faire */
  if(window_ramsound == NULL)
    return;

  /* En dehors de la zone */
  if(grid_x < 7 || grid_x > (7+window_ramsound->nb_column*3-1))
    {
      SetWindowText(GetDlgItem(window_ramsound->hwnd_ramsound,IDC_STATIC_CONTEXT),"");
      return;
    }
  grid_ramsound_x = grid_x - 7;     /* Enlve les 7 caractres de gauche */
  if(((grid_ramsound_x+1) % 3) == 0)
    {
      SetWindowText(GetDlgItem(window_ramsound->hwnd_ramsound,IDC_STATIC_CONTEXT),"");
      return;
    }
  grid_ramsound_offset = (grid_ramsound_x+1) / 3;

  /* Rcupre l'adresse de la cellule */
  ramsound_data_address = window_ramsound->ramsound_address + grid_y*window_ramsound->nb_column;
  ramsound_data_address += grid_ramsound_offset;

  /* Rcupre la valeur de la cellule */
  ramsound_data_offset = ramsound_data_address - window_ramsound->ramsound_address;
  if(ramsound_data_offset > (window_ramsound->nb_column*window_ramsound->nb_row-1))
    {
      SetWindowText(GetDlgItem(window_ramsound->hwnd_ramsound,IDC_STATIC_CONTEXT),"");
      return;
    }
  ramsound_data = window_ramsound->ramsound_data[ramsound_data_offset];

  /* Coordonnes de la souris en coodonnes grid_x,grid_y */
  sprintf(window_ramsound->buffer_log,"Memory Data  %04X : %02X",ramsound_data_address&0xFFFF,ramsound_data);
  SetWindowText(GetDlgItem(window_ramsound->hwnd_ramsound,IDC_STATIC_CONTEXT),window_ramsound->buffer_log);
}


/********************************************************************************************/
/*  UpdateRamSoundRefreshMode() :  MAJ du mode de rafraichissement de la fenetre RamSound. */
/********************************************************************************************/
static void UpdateRamSoundRefreshMode(struct window_ramsound *window_ramsound, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == RAMSOUND_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_ramsound->refresh_mode = RAMSOUND_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_ramsound->hwnd_ramsound,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_ramsound->hwnd_ramsound,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_ramsound->hwnd_ramsound;
      current_notification->message = HM_RAMSOUND_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_RAMSOUND;

      /* Ajout de la notification */
      window_ramsound->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_ramsound->refresh_mode = RAMSOUND_REFRESH_MANUAL;
      SendDlgItemMessage(window_ramsound->hwnd_ramsound,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_ramsound->hwnd_ramsound,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_ramsound->current_notification,NULL);
      window_ramsound->current_notification = NULL;
    }
}


/***********************************************************************/
/*  UserResizeRamSoundWindow() :  Retaille la fentre de zone mmoire. */
/***********************************************************************/
static void UserResizeRamSoundWindow(struct window_ramsound *window_ramsound)
{
  RECT window_rect;
  int window_new_width, window_new_height, delta_pixel_width, delta_pixel_height, delta_char_width, delta_char_height;
  int nb_new_text_column, nb_new_text_row, nb_new_data_column, nb_new_data_row;

  /* Position + Dimension actuelle de la fentre retaille dans l'cran */
  GetWindowRect(window_ramsound->hwnd_ramsound,&window_rect);

  /* Nouvelle Dimension de la fentre (avec la barre de titre et les bordures) */
  window_new_width  = window_rect.right  - window_rect.left;
  window_new_height = window_rect.bottom - window_rect.top;

  /* Calcule le Delta en Pixel par rapport  la taille prcdente */
  delta_pixel_width  = window_new_width  - window_ramsound->window_metric_width;
  delta_pixel_height = window_new_height - window_ramsound->window_metric_height;

  /* Calcule le Delta en Charactres par rapport  la taille prcdente */
  delta_char_width  = delta_pixel_width / RAMSOUND_EDIT_FONT_WIDTH;
  delta_char_height = delta_pixel_height / RAMSOUND_EDIT_FONT_HEIGHT;

  /* Nouvelle dimension de la zone Texte (Adresse + Data Hexa + Data Ascii) */
  nb_new_text_column = (window_ramsound->window_metric_column*3)+6 + delta_char_width;
  nb_new_text_row    = window_ramsound->window_metric_row           + delta_char_height;

  /* On dtermine la nouvelle dimension de la zone Data (Data Hexa) */
  nb_new_data_column = (nb_new_text_column - 6) / 3;   /* 4 bytes par valeur */
  nb_new_data_row    = nb_new_text_row;

  /** Teste les valeurs minimales autorises **/
  if(nb_new_data_column < window_ramsound->text_min_data_column)
    nb_new_data_column = window_ramsound->text_min_data_column;
  if(nb_new_data_row < window_ramsound->text_min_data_row)
    nb_new_data_row = window_ramsound->text_min_data_row;

  /* MAJ du nombre de Data colonnes + lignes affiches */
  SetDlgItemInt(window_ramsound->hwnd_ramsound,IDC_EDIT_COLUMN,nb_new_data_column,FALSE);
  SetDlgItemInt(window_ramsound->hwnd_ramsound,IDC_EDIT_ROW,nb_new_data_row,FALSE);

  /** Reconstruit les lements de la fentre avec les nouvelles valeurs du Rich Edit **/
  RefreshRamSoundWindowData(window_ramsound);

  /* Retaille la fentre pour viter les tailles entre deux */
  ResizeRamSoundWindow(window_ramsound);
}


/**************************************************************************/
/*  RefreshRamSoundWindowData() :  MAJ du contenu de la fentre RamSound. */
/**************************************************************************/
static void RefreshRamSoundWindowData(struct window_ramsound *window_ramsound)
{
  int length, is_modified;
  int ramsound_address, nb_column, nb_row;
  char buffer[1024];

  /* Rcupre la valeur du RamSound Pointer */
  length = GetDlgItemText(window_ramsound->hwnd_ramsound,IDC_EDIT_ADDRESS,buffer,1024);
  sscanf(buffer,"%X",&ramsound_address);
  nb_column = GetDlgItemInt(window_ramsound->hwnd_ramsound,IDC_EDIT_COLUMN,NULL,FALSE);
  if(nb_column < 1)
    nb_column = 1;
  nb_row = GetDlgItemInt(window_ramsound->hwnd_ramsound,IDC_EDIT_ROW,NULL,FALSE);
  if(nb_row < 1)
    nb_row = 1;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%04X",ramsound_address);
  SetDlgItemText(window_ramsound->hwnd_ramsound,IDC_EDIT_ADDRESS,buffer);
  SetDlgItemInt(window_ramsound->hwnd_ramsound,IDC_EDIT_COLUMN,nb_column,FALSE);
  SetDlgItemInt(window_ramsound->hwnd_ramsound,IDC_EDIT_ROW,nb_row,FALSE);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsRamSoundDataModified(ramsound_address,nb_column,nb_row,window_ramsound);
  if(is_modified == 0)
    return;

  /** Cration de la Zone Text Edit  partir des nouvelles donnes **/
  BuildRamSoundFromData(window_ramsound);

  /* Rafraichit la fentre */
  InvalidateRect(window_ramsound->hwnd_ramsound,NULL,TRUE);
  UpdateWindow(window_ramsound->hwnd_ramsound);
}


/*********************************************************************************************************/
/*  IsRamSoundDataModified() :  Les donnes RamSound ont t'elles t modifies depuis la dernirer fois. */
/*********************************************************************************************************/
static int IsRamSoundDataModified(int ramsound_address, int nb_column, int nb_row, struct window_ramsound *window_ramsound)
{
  int ramsound_data_address;
  unsigned char *ramsound_data;
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
  
  /* Allocation mmoire */
  ramsound_data = (unsigned char *) calloc(nb_column*nb_row+1,sizeof(unsigned char));
  if(ramsound_data == NULL)
    return(0);

  /* Rcupre les donnes */
  ramsound_data_address = ramsound_address;
  ReadDOCRAMBlock(ramsound_data_address,nb_column*nb_row,ramsound_data,current_mmu);
  
  /* Compare les donnes */
  if(window_ramsound->ramsound_data != NULL)
    if(ramsound_address == window_ramsound->ramsound_address &&
       nb_column == window_ramsound->nb_column &&
       nb_row == window_ramsound->nb_row &&
       memcmp(&ramsound_data[0],&window_ramsound->ramsound_data[0],MY_MIN(nb_column*nb_row,window_ramsound->nb_column*window_ramsound->nb_row)) == 0)
      return(0);
    
  /* Les donnes sont diffrentes => MAJ */
  window_ramsound->ramsound_address = ramsound_address;
  window_ramsound->nb_column = nb_column;
  window_ramsound->nb_row = nb_row;
  if(window_ramsound->ramsound_data != NULL)
    free(window_ramsound->ramsound_data);
  window_ramsound->ramsound_data = ramsound_data;

  /* Donnes diffrentes */
  return(1);
}


/*******************************************************************/
/*  ResizeRamSoundWindow() : Resize RamSound window by given size. */
/*******************************************************************/
static void ResizeRamSoundWindow(struct window_ramsound *window_ramsound)
{
  RECT window_rect;
  int window_x, window_y;
  
  /* Largeur de la partie Client de la fentre */
  window_ramsound->window_width = window_ramsound->group_data_x + window_ramsound->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_EXTRA_RESIZE_RIGHT_BORDER_SIZE;

  /* Hauteur der la partie Client de la fentre */
  window_ramsound->window_height = window_ramsound->group_data_y + window_ramsound->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_ramsound->hwnd_ramsound,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;

  /* Resize window */
  window_ramsound->user_resize = 0;
  MoveWindow(window_ramsound->hwnd_ramsound,window_x,window_y,window_ramsound->window_width,window_ramsound->title_height+window_ramsound->window_height,FALSE);

  /* On va conserver les mtriques de la fentre actuellement affiche */
  window_ramsound->window_metric_width  = window_ramsound->window_width;
  window_ramsound->window_metric_height = window_ramsound->title_height + window_ramsound->window_height;
  window_ramsound->window_metric_column = window_ramsound->nb_column;
  window_ramsound->window_metric_row    = window_ramsound->nb_row;
}


/**********************************************************************************************************/
/*  BuildRamSoundFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/**********************************************************************************************************/
static void BuildRamSoundFromData(struct window_ramsound *window_ramsound)
{
  HWND hwnd_group;
  EDITSTREAM edit_stream;
  int i, j, offset, offset_rtf, nb_byte_to_process, address_data, text_data_width_min;
  char buffer[BUFFER_SIZE];
  char buffer_title[256];

  /** On doit peut tre redimensionner la zone Text Edit dynamique **/
  if(window_ramsound->rtf_data_hexadecimal == NULL || 
     window_ramsound->text_data_column != window_ramsound->nb_column ||
     window_ramsound->text_data_row != window_ramsound->nb_row)
    {
      /* MAJ */
      window_ramsound->text_data_column = window_ramsound->nb_column;
      window_ramsound->text_data_row = window_ramsound->nb_row;

      /* Libration mmoire */
      if(window_ramsound->rtf_data_hexadecimal)
        free(window_ramsound->rtf_data_hexadecimal);
      window_ramsound->rtf_data_hexadecimal = NULL;

      /* Nombre caractres visibles sur la ligne : Address Data Ascii (on ne compte pas le \r\n final) */
      window_ramsound->text_data_width = (int)(strlen("0000   ") + window_ramsound->nb_column*3 - 1);   /* Enlve le dernier sparateur */

      /* Allocation mmoire */
      window_ramsound->rtf_data_size = (window_ramsound->text_data_width + (int)strlen("\r\n"))*window_ramsound->text_data_row;
      window_ramsound->rtf_data_hexadecimal = (char *) calloc(30*window_ramsound->rtf_data_size+1,sizeof(char));
      if(window_ramsound->rtf_data_hexadecimal == NULL)
        return;

      /* Dimension en Pixel du Control Text RamSound Data */
      text_data_width_min = (int) strlen("0300   26 2F 00 00 00 00 00 00.00 00 00 00 00 00 ");
      window_ramsound->ramsound_data_width = MY_MAX(window_ramsound->text_data_width,text_data_width_min)*7 + 2;
      window_ramsound->ramsound_data_height = window_ramsound->nb_row*RAMSOUND_EDIT_FONT_HEIGHT;
      MoveWindow(window_ramsound->hwnd_richedit_data,window_ramsound->ramsound_data_x,window_ramsound->ramsound_data_y,window_ramsound->ramsound_data_width,window_ramsound->ramsound_data_height,FALSE);

      /* Dimension en Pixel du Control Group Box + Resize */  
      hwnd_group = GetDlgItem(window_ramsound->hwnd_ramsound,IDC_GROUP_RAMSOUND);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_RAMSOUND */
      window_ramsound->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_ramsound->ramsound_data_width + WINDOW_GROUP_BORDER_RIGHT_SIZE;
      window_ramsound->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_ramsound->ramsound_data_height + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
      MoveWindow(hwnd_group,window_ramsound->group_data_x,window_ramsound->group_data_y,window_ramsound->group_data_width,window_ramsound->group_data_height,FALSE);

      /* Retaille la fentre */
      ResizeRamSoundWindow(window_ramsound);
    }

  /* Adresse des donnes */
  address_data = window_ramsound->ramsound_address;

  /** Cration d'un buffer RTF rpentant les donnes **/
  window_ramsound->rtf_data_hexadecimal[0] = '\0';
  strcat(window_ramsound->rtf_data_hexadecimal,"{\\rtf1\\ansi\\ansicpg1252\\deff0\\deflang1036{\\fonttbl{\\f0\\fnil\\fcharset0 Courier New;}}\r\n");
  strcat(window_ramsound->rtf_data_hexadecimal,"{\\colortbl ;\\red51\\green153\\blue255;}\r\n");
  strcat(window_ramsound->rtf_data_hexadecimal,"{\\*\\generator Msftedit 5.41.21.2510;}\\viewkind4\\uc1\\pard\\sl240\\slmult1\\lang12\\f0\\fs");
  strcat(window_ramsound->rtf_data_hexadecimal,RAMSOUND_RICHEDIT_FONT_HEIGHT_RTF);  /* Taille de la Police : Si 8 dans WordPad => 16 ici (x2)*/
  strcat(window_ramsound->rtf_data_hexadecimal," ");
  offset_rtf = (int) strlen(window_ramsound->rtf_data_hexadecimal);

  /*** Transformer les donnes BYTE en Hexa Ascii ***/
  nb_byte_to_process = window_ramsound->nb_column*window_ramsound->nb_row;
  for(i=0,offset=0; i<window_ramsound->nb_row; i++,offset+=window_ramsound->nb_column)
    {
      /* Dbut de ligne : Adresse */
      sprintf(&window_ramsound->rtf_data_hexadecimal[offset_rtf],"%04X   ",(offset+address_data) & 0xFFFF);
      offset_rtf += (int) strlen("0000   ");

      /** Encode les Byte en Hexa **/
      for(j=0; j<window_ramsound->nb_column; j++)
        {
          /* On met les 00 en inverse */
          if(window_ramsound->ramsound_data[offset+j] == 0x00)
            {
              /* Passe ce Byte en Rouge */
              sprintf(buffer,"\\highlight1 %02X\\highlight0 ",window_ramsound->ramsound_data[offset+j]);
              strcpy(&window_ramsound->rtf_data_hexadecimal[offset_rtf],buffer);
              offset_rtf += (int) strlen(buffer);
            }
          else if(i*window_ramsound->nb_column+j < nb_byte_to_process)
            {
              /* Byte en noir */
              sprintf(&window_ramsound->rtf_data_hexadecimal[offset_rtf],"%02X",window_ramsound->ramsound_data[offset+j]);
              offset_rtf += 2;
            }
          else
            {
              /* Aucune donne  afficher */
              strcpy(&window_ramsound->rtf_data_hexadecimal[offset_rtf],"  ");
              offset_rtf += 2;
            }

          /* Sparateur */
          if(j < (window_ramsound->nb_column-1))
            {
              window_ramsound->rtf_data_hexadecimal[offset_rtf] = (j==7 && (i*window_ramsound->nb_column+j < nb_byte_to_process)) ? '.' : ' ';
              offset_rtf++;
            }
        }

      /* Fin de ligne */
      strcpy(&window_ramsound->rtf_data_hexadecimal[offset_rtf],"\\par\r\n");
      offset_rtf += (int) strlen("\\par\r\n");
    }
    
  /** Fin du RTF **/
  strcpy(&window_ramsound->rtf_data_hexadecimal[offset_rtf],"}\r\n");
  offset_rtf += (int) strlen("}\r\n");
  window_ramsound->rtf_data_hexadecimal[offset_rtf] = '\0';
  
  /* Taille du RTF (le \0 final compte) */
  window_ramsound->rtf_data_size = (int) strlen(window_ramsound->rtf_data_hexadecimal) + 1;    
    
  /** Afficher le rsultat en RTF dans le control Rich Edit **/
  window_ramsound->rtf_write_offset = 0;
  memset(&edit_stream,0,sizeof(EDITSTREAM));
  edit_stream.pfnCallback = (EDITSTREAMCALLBACK) RamSoundRichEditStreamCallback;
  edit_stream.dwCookie = (DWORD_PTR) window_ramsound;          /* Paramtre  passer  la fonction Callback */
  SendMessage(window_ramsound->hwnd_richedit_data,EM_STREAMIN,SF_RTF,(LPARAM)&edit_stream);
  
  /* Titre de la fentre */
  sprintf(buffer_title,"RAM Sound : %04X",window_ramsound->ramsound_address);
  SetWindowText(window_ramsound->hwnd_ramsound,buffer_title);
}


/**************************************************************************************/
/*  RamSoundRichEditStreamCallback() :  Fonction Call back pour remplir le Rich Edit. */
/**************************************************************************************/
static DWORD CALLBACK RamSoundRichEditStreamCallback(DWORD *dwCookie, LPBYTE output_buffer, LONG nb_write_max, LONG *nb_write_rtn)
{
  int nb_remain_byte, nb_copy_byte;
  struct window_ramsound *window_ramsound;

  /* Rcupration du paramtre */
	window_ramsound = (struct window_ramsound *) dwCookie;

  /* Taille des donnes encore  copier */
  nb_remain_byte = window_ramsound->rtf_data_size - window_ramsound->rtf_write_offset;
  if(nb_remain_byte == 0)
    {
      /* Plus rien  copier */
      *nb_write_rtn = 0;
      return(0);
    }

  /* Taille des donnes que l'on est autoris  copier */
  nb_copy_byte = (nb_write_max >= nb_remain_byte) ? nb_remain_byte : nb_write_max;

	/* Nombre de byte transfrs */
	*nb_write_rtn = nb_copy_byte;
		  
	/** Copie les Byte dans le output buffer **/
  memcpy(output_buffer,&window_ramsound->rtf_data_hexadecimal[window_ramsound->rtf_write_offset],nb_copy_byte);

  /* Dplace l'offset */
  window_ramsound->rtf_write_offset += nb_copy_byte;

  /* OK */
	return(0);
}


/**********************************************************************/
/*  DeleteRamSoundChildWindows() :  Destruction des fentres enfants. */
/**********************************************************************/
static void DeleteRamSoundChildWindows(struct window_ramsound *window_ramsound)
{
  /** Libration des ressources **/  
  if(window_ramsound->rtf_data_hexadecimal != NULL)
    free(window_ramsound->rtf_data_hexadecimal);
  window_ramsound->rtf_data_hexadecimal = NULL;

  if(window_ramsound->ramsound_data != NULL)
    free(window_ramsound->ramsound_data);
  window_ramsound->ramsound_data = NULL;
}


/************************************************************************/
/*  mem_free_window_ramsound() :  Libration mmoire pour la structure. */
/************************************************************************/
void mem_free_window_ramsound(struct window_ramsound *window_ramsound)
{
  if(window_ramsound)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_ramsound->current_notification,NULL);
      window_ramsound->current_notification = NULL;
      
      free(window_ramsound);
    }
}

/**********************************************************************/
