/************************************************************************/
/*                                                                      */
/*  Window_Replace.h :  Header de gestion de la fentre Replace.        */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Octobre 2018  */
/************************************************************************/

/* Control Edit pour les Replace Result */
#define IDC_RICHEDIT_REPLACE_DATA        1624

#define REPLACE_EDIT_FONT_HEIGHT          14   /* Taille de la Fonte en Pixel */

#define REPLACE_RICHEDIT_FONT_HEIGHT      14   /* Taille de la Fonte en Pixel */
#define REPLACE_RICHEDIT_FONT_HEIGHT_RTF "16"  /* Taille de la Fonte dans le RTF */

#define REPLACE_RICHEDIT_HEADER_SIZE    1024   /* Nombre d'octet MAX pour encoder le header du RichEdit */
#define REPLACE_RICHEDIT_MAX_CHAR_SIZE    24   /* Nombre d'octet MAX pour encoder un caractre (y compris sa couleur...) */
#define REPLACE_RICHEDIT_FOOTER_SIZE     512   /* Nombre d'octet MAX pour encoder le footer du RichEdit */

#define REPLACE_RICHEDIT_COLOR_BLACK       0   /* Couleurs pour le RichEdit */
#define REPLACE_RICHEDIT_COLOR_RED         1
#define REPLACE_RICHEDIT_COLOR_BLUE        2
#define REPLACE_RICHEDIT_COLOR_GREEN       3
#define REPLACE_RICHEDIT_COLOR_DARKRED     4
#define REPLACE_RICHEDIT_COLOR_DARKGREY    5

#define REPLACE_TYPE_HEXA               1
#define REPLACE_TYPE_DECIMAL            2
#define REPLACE_TYPE_TEXT               3
#define REPLACE_TYPE_BYTES              4

#define REPLACE_SIZE_8                  1
#define REPLACE_SIZE_16                 2
#define REPLACE_SIZE_24                 3
#define REPLACE_SIZE_32                 4

#define REPLACE_ALIGN_INTEL             1
#define REPLACE_ALIGN_MOTOROLA          2

#define REPLACE_CHARSET_ASCII           1
#define REPLACE_CHARSET_HIGHASCII       2
#define REPLACE_CHARSET_INVERSE         3
#define REPLACE_CHARSET_FLASH           4

#define REPLACE_OPTIONS_CASESENSITIVE   1
#define REPLACE_OPTIONS_REVERSEORDER    2
#define REPLACE_OPTIONS_WILDCARD        3

#define REPLACE_AREA_FASTRAM            1
#define REPLACE_AREA_SLOWRAM            2
#define REPLACE_AREA_ROM                3
#define REPLACE_AREA_DOCRAM             4

#define REPLACE_CURRENT_RESULT          1
#define REPLACE_ALL_RESULT              2

struct replace_result
{ 
  int replace_area;        /* REPLACE_AREA_XXX */
  
  int bank;                /* Addresse o se trouve les donnes */
  int address;
  
  int data_size;           /* Taille des donnes recherches */
  
  int replaced;            /* Le remplacement a dj t effectu */

  struct replace_result *next;
};

struct window_replace
{
  HWND hwnd_replace;
  HWND hwnd_richedit_data;      /* Control multi-lignes RTF */

  /** Surcharge des Control Edit **/
  WNDPROC oldEditProcSearch;
  WNDPROC oldEditProcFastramFromBank;
  WNDPROC oldEditProcFastramFromAddress;
  WNDPROC oldEditProcFastramToBank;
  WNDPROC oldEditProcFastramToAddress;
  WNDPROC oldEditProcSlowramFromBank;
  WNDPROC oldEditProcSlowramFromAddress;
  WNDPROC oldEditProcSlowramToBank;
  WNDPROC oldEditProcSlowramToAddress;
  WNDPROC oldEditProcRomFromBank;
  WNDPROC oldEditProcRomFromAddress;
  WNDPROC oldEditProcRomToBank;
  WNDPROC oldEditProcRomToAddress;
  WNDPROC oldEditProcDocramFromAddress;
  WNDPROC oldEditProcDocramToAddress;
  WNDPROC oldEditProcReplace;

  /* Modle mmoire */
  int nb_fastram_bank;
  int nb_rom_bank;

  /** Critres de recherche **/
  int search_type;       /* REPLACE_TYPE_XXX */
  int search_size;       /* REPLACE_SIZE_XXX */
  int search_align;      /* REPLACE_ALIGN_XXX */
  int search_area_fastram;
  int search_area_slowram;
  int search_area_rom;
  int search_area_docram;

  /* Charset */
  int charset_ascii;
  int charset_highascii;
  int charset_inverse;
  int charset_flash;

  /* Options */
  int options_casesensitive;
  int options_reverseorder;
  int options_wildcard;

  /** Zones de Recherche **/
  int fastram_from_bank;
  int fastram_from_address;
  int fastram_to_bank;
  int fastram_to_address;

  int slowram_from_bank;
  int slowram_from_address;
  int slowram_to_bank;
  int slowram_to_address;

  int rom_from_bank;
  int rom_from_address;
  int rom_to_bank;
  int rom_to_address;

  int docram_from_address;
  int docram_to_address;

  /** Replace settings **/
  int replace_type;       /* REPLACE_TYPE_XXX */
  int replace_size;       /* REPLACE_SIZE_XXX */
  int replace_align;      /* REPLACE_ALIGN_XXX */
  int replace_charset;    /* REPLACE_CHARSET_XXX */

  /** Search + Replace Query **/
  int search_query_size;
  unsigned char search_query_data[2048];
  int replace_query_size;
  unsigned char replace_query_data[2048];

  /** Rsultat de la recherche **/
  int nb_found;
  int nb_replaced;
  int current_result;
  struct replace_result *first_replaceresult;
  struct replace_result *last_replaceresult;

  /* Posiiton / Taille du Control Group Box */
  int group_data_x;
  int group_data_y;

  /* Position / Taille du Control Text Zone Replace Result Data */
  int replace_data_x;
  int replace_data_y;
  int replace_data_width;
  int replace_data_height;

  /** Control Edit : Zone Texte o sont affiches les Data **/  
  int nb_line;                     /* 2 */  
  int nb_column;                   /* 16 */
  int text_data_width;             /* "00/0000___00_00_00_00_00_00_00_00.00_00_00_00_00_00_00_00__0123456789ABCDEF\r\n" */
  int rtf_data_size;               /* Taille de la Zone Texte RTF */
  char *rtf_data_hexadecimal;      /* Zone Texte RTF */  
  int rtf_write_offset;            /* Offset de l'criture de la Zone Texte RTF vers le Rich Edit */

  /* Dimension de la fentre = Title + Zone Texte */
  int title_height;
  int window_width;
  int window_height;
  
  /* Buffer */
  char buffer_log[BUFFER_SIZE];
};

struct window_replace *BuildReplaceWindow(HINSTANCE,int);
void mem_free_replaceresult(struct replace_result *);
void mem_free_window_replace(struct window_replace *);

/********************************************************/
