/************************************************************************/
/*                                                                      */
/*  Window_SaveMemory.c :  Module de gestion de la fentre SaveMemory.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI   *   Brutal Deluxe   *   Date : Jan 2019  */
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_SaveMemory.h"

static DWORD BuildSaveMemoryWindowThread(void *);
LRESULT CALLBACK ProcHandlerSaveMemoryWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateSaveMemoryChildWindows(HWND,struct window_savememory *);
LRESULT CALLBACK subEditSaveMemoryProcControl(HWND,UINT,WPARAM,LPARAM);
static void UpdateSaveMemoryRamType(struct window_savememory *,int);
static void UpdateSaveMemoryFileMode(struct window_savememory *,int);
static void SaveMemoryDataToFile(struct window_savememory *);
static void DeleteSaveMemoryChildWindows(struct window_savememory *);


/**********************************************************************************/
/*  BuildSaveMemoryWindow() :  Construction / Affichage de la fentre SaveMemory. */
/**********************************************************************************/
struct window_savememory *BuildSaveMemoryWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_savememory *window_savememory;

  /** Allocation de la structure window_savememory **/
  window_savememory = (struct window_savememory *) calloc(1,sizeof(struct window_savememory));
  if(window_savememory == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_SAVEMEMORY,window_savememory,NULL);

  /* Lance un thread pour grer la fentre SaveMemory */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildSaveMemoryWindowThread,window_savememory,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_savememory);
}


/********************************************************************************************/
/*  BuildSaveMemoryWindowThread() :  Cration de la fentre SaveMemory via un Thread ddi. */
/********************************************************************************************/
static DWORD BuildSaveMemoryWindowThread(void *window_savememory_param)
{
  HWND hwndSaveMemory;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_savememory *window_savememory;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_savememory = (struct window_savememory *) window_savememory_param;
  window_savememory->thread_id = GetCurrentThreadId();         /* Le ThreadID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Save Memory **/
  hwndSaveMemory = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_SAVEMEMORY),window_main->hwnd_main,(DLGPROC)ProcHandlerSaveMemoryWindow);
  if(hwndSaveMemory == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndSaveMemory,SW_SHOWDEFAULT);
  UpdateWindow(hwndSaveMemory);

  /** Message Loop pour toutes les fentres de ce Thread (fentre SaveMemory...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***************************************************************************/
/*  ProcHandlerSaveMemoryWindow() :  Gestion des vnements de la fentre. */
/***************************************************************************/
LRESULT CALLBACK ProcHandlerSaveMemoryWindow(HWND hwndSaveMemory, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  struct window_savememory *window_savememory;
  struct parameter *param;
  char file_path[1024];
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_SAVEMEMORY,&window_savememory,NULL);
  if(window_savememory == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateSaveMemoryChildWindows(hwndSaveMemory,window_savememory);
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la source des donnes (Ram ou Doc Ram) **/
            case IDC_RADIO_RAM :
              if(HIWORD(wParam) == BN_CLICKED && window_savememory->ram_type != SAVEMEMORY_RAMTYPE_RAM)
                UpdateSaveMemoryRamType(window_savememory,SAVEMEMORY_RAMTYPE_RAM);
              break;

            case IDC_RADIO_DOCRAM :
              if(HIWORD(wParam) == BN_CLICKED && window_savememory->ram_type != SAVEMEMORY_RAMTYPE_DOCRAM)
                UpdateSaveMemoryRamType(window_savememory,SAVEMEMORY_RAMTYPE_DOCRAM);
              break;

            /** Bouton Radio : On change le mode d'criture des donnes (New ou Append) **/
            case IDC_RADIO_NEW :
              if(HIWORD(wParam) == BN_CLICKED && window_savememory->ram_type != SAVEMEMORY_FILEMODE_NEW)
                UpdateSaveMemoryFileMode(window_savememory,SAVEMEMORY_FILEMODE_NEW);
              break;

            case IDC_RADIO_APPEND :
              if(HIWORD(wParam) == BN_CLICKED && window_savememory->ram_type != SAVEMEMORY_FILEMODE_APPEND)
                UpdateSaveMemoryFileMode(window_savememory,SAVEMEMORY_FILEMODE_APPEND);
              break;

            /** Bouton de rcupration du chemin du fichier **/
            case IDC_BUTTON_FILE_GET_PATH :
              GetOneFilePath(window_savememory->hwnd_savememory,"Select Data File","All Files (*.*)\0*.*\0\0",window_savememory->file_path,file_path);
              if(strlen(file_path) > 0)
                {
                  /* Chemin du fichier */
                  strcpy(window_savememory->file_path,file_path);
                  SetWindowText(GetDlgItem(window_savememory->hwnd_savememory,IDC_EDIT_FILE_PATH),window_savememory->file_path);
                }
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_SAVEMEMORY :
              SaveMemoryDataToFile(window_savememory);
              break;
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre SaveMemory */
        DestroyWindow(hwndSaveMemory);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteSaveMemoryChildWindows(window_savememory);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_SAVEMEMORY,window_savememory,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************/
/*  CreateSaveMemoryChildWindows() : Crer les fentres enfants et remplir la structure window_savememory. */
/***********************************************************************************************************/
static int CreateSaveMemoryChildWindows(HWND hwndSaveMemory, struct window_savememory *window_savememory)
{
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_savememory->hwnd_savememory = hwndSaveMemory;

  /** Surchage des Control Edit pour capter les RETURN lors des saisies **/
  window_savememory->oldEditProcRamBank =    (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSaveMemory,IDC_EDIT_RAM_BANK),GWLP_WNDPROC,(LONG_PTR)subEditSaveMemoryProcControl);
  window_savememory->oldEditProcRamAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSaveMemory,IDC_EDIT_RAM_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditSaveMemoryProcControl);
  window_savememory->oldEditProcDataLength = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSaveMemory,IDC_EDIT_DATA_LENGTH),GWLP_WNDPROC,(LONG_PTR)subEditSaveMemoryProcControl);  
  window_savememory->oldEditProcFilePath =   (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSaveMemory,IDC_EDIT_FILE_PATH),GWLP_WNDPROC,(LONG_PTR)subEditSaveMemoryProcControl);

  /* Fichier de Data */
  window_savememory->data_length = 0x8000;
  SetDlgItemText(window_savememory->hwnd_savememory,IDC_EDIT_DATA_LENGTH,"0x8000");

  /* Zone Mmoire */
  window_savememory->ram_type = SAVEMEMORY_RAMTYPE_RAM;
  window_savememory->bank = 0x01;
  window_savememory->address = 0x2000;
  sprintf(buffer,"%02X",window_savememory->bank);
  SetDlgItemText(window_savememory->hwnd_savememory,IDC_EDIT_RAM_BANK,buffer);
  sprintf(buffer,"%04X",window_savememory->address);
  SetDlgItemText(window_savememory->hwnd_savememory,IDC_EDIT_RAM_ADDRESS,buffer);

  /* Ram Type : RAM */
  UpdateSaveMemoryRamType(window_savememory,SAVEMEMORY_RAMTYPE_RAM);

  /* File Mode : NEW */
  UpdateSaveMemoryFileMode(window_savememory,SAVEMEMORY_FILEMODE_NEW);
  
  /* OK */
  return(0);
}


/*************************************************************************************************/
/*  subEditSaveMemoryProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*************************************************************************************************/
LRESULT CALLBACK subEditSaveMemoryProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_savememory *window_savememory;
  my_Memory(MEMORY_GET_WINDOW_SAVEMEMORY,&window_savememory,NULL);

  /* Protection */
  if(window_savememory == NULL)
    return(0);

  /** Identifie le Control **/
  if(GetDlgItem(window_savememory->hwnd_savememory,IDC_EDIT_RAM_BANK) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_savememory->hwnd_savememory,IDC_EDIT_RAM_ADDRESS) == hwnd_control)
    index = 2;  
  else if(GetDlgItem(window_savememory->hwnd_savememory,IDC_EDIT_DATA_LENGTH) == hwnd_control)
    index = 3;    
  else if(GetDlgItem(window_savememory->hwnd_savememory,IDC_EDIT_FILE_PATH) == hwnd_control)
    index = 4;
  else
    return(0);      /* Pas trouv */

  /** Dtecte la touche RETURN pour valider la donne **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_savememory->hwnd_savememory);
    }
  else if(index == 1)
    return(CallWindowProc(window_savememory->oldEditProcRamBank,hwnd_control,msg,wParam,lParam));
  else if(index == 2)
    return(CallWindowProc(window_savememory->oldEditProcRamAddress,hwnd_control,msg,wParam,lParam));    
  else if(index == 3)
    return(CallWindowProc(window_savememory->oldEditProcDataLength,hwnd_control,msg,wParam,lParam));    
  else if(index == 4)
    return(CallWindowProc(window_savememory->oldEditProcFilePath,hwnd_control,msg,wParam,lParam));

  /* OK */
  return(0);
} 


/*********************************************************************************/
/*  UpdateSaveMemoryRamType() :  Change le type de la RAM contenant les donnes. */
/*********************************************************************************/
static void UpdateSaveMemoryRamType(struct window_savememory *window_savememory, int ram_type)
{
  if(ram_type == SAVEMEMORY_RAMTYPE_RAM)
    {
      /* Visuel */
      window_savememory->ram_type = SAVEMEMORY_RAMTYPE_RAM;
      SendDlgItemMessage(window_savememory->hwnd_savememory,IDC_RADIO_RAM,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_savememory->hwnd_savememory,IDC_RADIO_DOCRAM,BM_SETCHECK,0,0);

      /* Enable BANK */
      EnableWindow(GetDlgItem(window_savememory->hwnd_savememory,IDC_EDIT_RAM_BANK),TRUE);
    }
  else
    {
      /* Visuel */
      window_savememory->ram_type = SAVEMEMORY_RAMTYPE_DOCRAM;
      SendDlgItemMessage(window_savememory->hwnd_savememory,IDC_RADIO_RAM,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_savememory->hwnd_savememory,IDC_RADIO_DOCRAM,BM_SETCHECK,1,0);

      /* Disable BANK */
      EnableWindow(GetDlgItem(window_savememory->hwnd_savememory,IDC_EDIT_RAM_BANK),FALSE);
    }
}


/*****************************************************************************************/
/*  UpdateSaveMemoryFileMode() :  Change le mode d'criture des donnes dans le fichier. */
/*****************************************************************************************/
static void UpdateSaveMemoryFileMode(struct window_savememory *window_savememory, int file_mode)
{
  if(file_mode == SAVEMEMORY_FILEMODE_NEW)
    {
      /* Visuel */
      window_savememory->file_mode = SAVEMEMORY_FILEMODE_NEW;
      SendDlgItemMessage(window_savememory->hwnd_savememory,IDC_RADIO_NEW,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_savememory->hwnd_savememory,IDC_RADIO_APPEND,BM_SETCHECK,0,0);
    }
  else
    {
      /* Visuel */
      window_savememory->file_mode = SAVEMEMORY_FILEMODE_APPEND;
      SendDlgItemMessage(window_savememory->hwnd_savememory,IDC_RADIO_NEW,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_savememory->hwnd_savememory,IDC_RADIO_APPEND,BM_SETCHECK,1,0);
    }
}


/*************************************************************************/
/*  SaveMemoryDataToFile() :  Sauvegarde des donnes mmoire sur disque. */
/*************************************************************************/
static void SaveMemoryDataToFile(struct window_savememory *window_savememory)
{
  FILE *fd;
  unsigned char *file_data;
  int data_length, address, bank, bank_address, is_error;
  char file_path[1024];
  char value_buffer[1024];
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /** Length des Data **/
  data_length = EvaluateAsOffsetOrLength(window_savememory->hwnd_savememory,IDC_EDIT_DATA_LENGTH);
  if(data_length == 0)
    {
      MessageBox(window_savememory->hwnd_savememory,"Error, The Data length is Zero !","Source Data Error",MB_OK|MB_ICONWARNING);
      return;
    }
  file_data = (unsigned char *) calloc(data_length+1,sizeof(unsigned char));
  if(file_data == NULL)
    {
      MessageBox(window_savememory->hwnd_savememory,"Error, Can't allocate memory !","Internal Error",MB_OK|MB_ICONWARNING);
      return;
    }  

  /** Chemin du fichier **/
  GetWindowText(GetDlgItem(window_savememory->hwnd_savememory,IDC_EDIT_FILE_PATH),file_path,1024-1);
  if(strlen(file_path) == 0)
    {
      MessageBox(window_savememory->hwnd_savememory,"Empty File Path !","Target File Error",MB_OK|MB_ICONWARNING);
      return;
    }

  /** Addresse Mmoire **/
  if(window_savememory->ram_type == SAVEMEMORY_RAMTYPE_RAM)
    {
      /* Rcupre l'adresse cible */
      GetWindowText(GetDlgItem(window_savememory->hwnd_savememory,IDC_EDIT_RAM_BANK),value_buffer,1024-1);
      bank = EvaluateAsByte(value_buffer,&is_error);
      GetWindowText(GetDlgItem(window_savememory->hwnd_savememory,IDC_EDIT_RAM_ADDRESS),value_buffer,1024-1);
      address = EvaluateAsWord(value_buffer,&is_error);
      bank_address = (bank << 16) | address;

      /* Copie les bytes de la mmoire */
      ReadMemoryBlock(bank_address,data_length,file_data,current_mmu);
    }
  else   /* Doc RAM */
    {
      /* Rcupre l'adresse DOC cible */
      GetWindowText(GetDlgItem(window_savememory->hwnd_savememory,IDC_EDIT_RAM_ADDRESS),value_buffer,1024-1);
      address = EvaluateAsWord(value_buffer,&is_error);

      /* Copie les bytes de la mmoire */
      ReadDOCRAMBlock(address,data_length,file_data,current_mmu);
    }

  /** Ecriture dans le fichier **/
  if(window_savememory->file_mode == SAVEMEMORY_FILEMODE_NEW)
    {
      /* Nouveau fichier */
      fd = fopen(file_path,"wb");
      if(fd == NULL)
        {
          free(file_data);
          MessageBox(window_savememory->hwnd_savememory,"Can't create file on disk !","Target File Error",MB_OK|MB_ICONWARNING);
          return;
        }
      
      /* Ecriture */
      fwrite(file_data,data_length,1,fd);
      
      /* Fermeture */
      fclose(fd);
    }
  else 
    {
      /* Nouveau fichier */
      fd = fopen(file_path,"ab");
      if(fd == NULL)
        {
          free(file_data);
          MessageBox(window_savememory->hwnd_savememory,"Can't open file on disk !","Target File Error",MB_OK|MB_ICONWARNING);
          return;
        }
      
      /* Ecriture */
      fwrite(file_data,data_length,1,fd);
      
      /* Fermeture */
      fclose(fd);      
    }

  /* Libration mmoire */
  free(file_data);
}


/************************************************************************/
/*  DeleteSaveMemoryChildWindows() :  Destruction des fentres enfants. */
/************************************************************************/
static void DeleteSaveMemoryChildWindows(struct window_savememory *window_savememory)
{
  /** Libration des ressources **/  
}


/**************************************************************************/
/*  mem_free_window_savememory() :  Libration mmoire pour la structure. */
/**************************************************************************/
void mem_free_window_savememory(struct window_savememory *window_savememory)
{
  if(window_savememory)
    {      
      free(window_savememory);
    }
}

/**********************************************************************/
