/*********************************************************************/
/*                                                                   */
/*  Window_Search.h :  Header de gestion de la fentre Search.       */
/*                                                                   */
/*********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Juin 2018  */
/*********************************************************************/

/* Control Edit pour les Search Result */
#define IDC_RICHEDIT_SEARCH_DATA        1624

#define SEARCH_EDIT_FONT_HEIGHT       14    /* Taille de la Fonte en Pixel */

#define SEARCH_RICHEDIT_FONT_HEIGHT      14   /* Taille de la Fonte en Pixel */
#define SEARCH_RICHEDIT_FONT_HEIGHT_RTF "16"  /* Taille de la Fonte dans le RTF */

#define SEARCH_RICHEDIT_HEADER_SIZE    1024   /* Nombre d'octet MAX pour encoder le header du RichEdit */
#define SEARCH_RICHEDIT_MAX_CHAR_SIZE    24   /* Nombre d'octet MAX pour encoder un caractre (y compris sa couleur...) */
#define SEARCH_RICHEDIT_FOOTER_SIZE     512   /* Nombre d'octet MAX pour encoder le footer du RichEdit */

#define SEARCH_RICHEDIT_COLOR_BLACK       0   /* Couleurs pour le RichEdit */
#define SEARCH_RICHEDIT_COLOR_RED         1
#define SEARCH_RICHEDIT_COLOR_BLUE        2
#define SEARCH_RICHEDIT_COLOR_GREEN       3
#define SEARCH_RICHEDIT_COLOR_DARKRED     4
#define SEARCH_RICHEDIT_COLOR_DARKGREY    5

#define SEARCH_TYPE_HEXA               1
#define SEARCH_TYPE_DECIMAL            2
#define SEARCH_TYPE_TEXT               3
#define SEARCH_TYPE_BYTES              4

#define SEARCH_SIZE_8                  1
#define SEARCH_SIZE_16                 2
#define SEARCH_SIZE_24                 3
#define SEARCH_SIZE_32                 4

#define SEARCH_ALIGN_INTEL             1
#define SEARCH_ALIGN_MOTOROLA          2

#define SEARCH_CHARSET_ASCII           1
#define SEARCH_CHARSET_HIGHASCII       2
#define SEARCH_CHARSET_INVERSE         3
#define SEARCH_CHARSET_FLASH           4

#define SEARCH_OPTIONS_CASESENSITIVE   1
#define SEARCH_OPTIONS_REVERSEORDER    2
#define SEARCH_OPTIONS_WILDCARD        3

#define SEARCH_AREA_FASTRAM            1
#define SEARCH_AREA_SLOWRAM            2
#define SEARCH_AREA_ROM                3
#define SEARCH_AREA_DOCRAM             4

struct search_result
{ 
  int search_area;         /* SEARCH_AREA_XXX */
  
  int bank;                /* Addresse o se trouve les donnes */
  int address;
  
  int data_size;           /* Taille des donnes recherches */
  
  struct search_result *next;
};

struct window_search
{
  HWND hwnd_search;
  HWND hwnd_richedit_data;      /* Control multi-lignes RTF */

  /** Surcharge des Control Edit **/
  WNDPROC oldEditProcSearch;
  WNDPROC oldEditProcFastramFromBank;
  WNDPROC oldEditProcFastramFromAddress;
  WNDPROC oldEditProcFastramToBank;
  WNDPROC oldEditProcFastramToAddress;
  WNDPROC oldEditProcSlowramFromBank;
  WNDPROC oldEditProcSlowramFromAddress;
  WNDPROC oldEditProcSlowramToBank;
  WNDPROC oldEditProcSlowramToAddress;
  WNDPROC oldEditProcRomFromBank;
  WNDPROC oldEditProcRomFromAddress;
  WNDPROC oldEditProcRomToBank;
  WNDPROC oldEditProcRomToAddress;
  WNDPROC oldEditProcDocramFromAddress;
  WNDPROC oldEditProcDocramToAddress;

  /* Modle mmoire */
  int nb_fastram_bank;
  int nb_rom_bank;

  /** Critres de recherche **/
  int search_type;       /* SEARCH_TYPE_XXX */
  int search_size;       /* SEARCH_SIZE_XXX */
  int search_align;      /* SEARCH_ALIGN_XXX */
  int search_area_fastram;
  int search_area_slowram;
  int search_area_rom;
  int search_area_docram;

  /* Charset */
  int charset_ascii;
  int charset_highascii;
  int charset_inverse;
  int charset_flash;

  /* Options */
  int options_casesensitive;
  int options_reverseorder;
  int options_wildcard;

  /** Zones de Recherche **/
  int fastram_from_bank;
  int fastram_from_address;
  int fastram_to_bank;
  int fastram_to_address;

  int slowram_from_bank;
  int slowram_from_address;
  int slowram_to_bank;
  int slowram_to_address;

  int rom_from_bank;
  int rom_from_address;
  int rom_to_bank;
  int rom_to_address;

  int docram_from_address;
  int docram_to_address;

  /** Search Query **/
  int search_query_size;
  unsigned char search_query_data[2048];

  /** Rsultat de la recherche **/
  int nb_found;
  int current_result;
  struct search_result *first_searchresult;
  struct search_result *last_searchresult;

  /* Posiiton / Taille du Control Group Box */
  int group_data_x;
  int group_data_y;

  /* Position / Taille du Control Text Zone Search Result Data */
  int search_data_x;
  int search_data_y;
  int search_data_width;
  int search_data_height;

  /** Control Edit : Zone Texte o sont affiches les Data **/  
  int nb_line;                     /* 2 */  
  int nb_column;                   /* 16 */
  int text_data_width;             /* "00/0000___00_00_00_00_00_00_00_00.00_00_00_00_00_00_00_00__0123456789ABCDEF\r\n" */
  int rtf_data_size;               /* Taille de la Zone Texte RTF */
  char *rtf_data_hexadecimal;      /* Zone Texte RTF */  
  int rtf_write_offset;            /* Offset de l'criture de la Zone Texte RTF vers le Rich Edit */

  /* Dimension de la fentre = Title + Zone Texte */
  int title_height;
  int window_width;
  int window_height;
  
  /* Buffer */
  char buffer_log[BUFFER_SIZE];
};

struct window_search *BuildSearchWindow(HINSTANCE,int);
void mem_free_searchresult(struct search_result *);
void mem_free_window_search(struct window_search *);

/********************************************************/
