/**********************************************************************/
/*                                                                    */
/*   Window_Snapshot.c :  Module de gestion de la fentre Snapshot.   */
/*                                                                    */
/**********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Avril 2018  */
/**********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Snapshot.h"
#include "Cyrene_Engine.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Snapshot.h"

LRESULT CALLBACK ProcHandlerSnapshotWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateSnapshotChildWindows(HWND,struct window_snapshot *);
LRESULT CALLBACK subEditSnapshotProcControl(HWND,UINT,WPARAM,LPARAM);
static struct snapshot *GetSnapshotValueFromWindow(struct window_snapshot *);
static void DeleteSnapshotChildWindows(struct window_snapshot *);


/******************************************************************************/
/*  BuildSnapshotWindow() :  Construction / Affichage de la fentre Snapshot. */
/******************************************************************************/
struct window_snapshot *BuildSnapshotWindow(HINSTANCE hInstance, int menu_id, char *current_snapshot_filepath)
{
  HWND hwndSnapshot;
  char snapshot_file_path[BUFFER_SIZE];
  char buffer_error[BUFFER_SIZE];
  struct snapshot *current_snapshot;
  struct window_main *window_main;
  struct window_snapshot *window_snapshot;
  struct parameter *param;

  /* Init */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  current_snapshot = NULL;
  snapshot_file_path[0] = '\0';

  /** On demande  charger un Snapshot en mmoire **/
  if(menu_id == ID_MENU_FILE_LOADMEMORYSNAPSHOT)
    {
      /* Rcupre le chemin du fichier */
      GetOneFilePath(window_main->hwnd_main,"All Files (*.*)\0*.*\0Apple II - Memory Snapshot (*.IIM)\0*.IIM\0Apple IIgs - Memory Snapshot (*.GSM)\0*.GSM\0\0\0","",snapshot_file_path);
      if(strlen(snapshot_file_path) == 0)
        return(NULL);

      /* Charge le fichier en mmoire */
      current_snapshot = LoadSnapshotFile(snapshot_file_path,&buffer_error[0]);
      if(current_snapshot == NULL)
        {
          sprintf(window_main->buffer_log,"Error : %s",buffer_error);
          MessageBox(window_main->hwnd_main,window_main->buffer_log,"Open Snapshot file",MB_OK|MB_ICONWARNING);
          return(NULL);
        }
    }
  else if(menu_id == ID_MENU_FILE_SAVEMEMORYSNAPSHOT)
    {
      /* Rcupre le Snapshot en cours */
      my_Memory(MEMORY_GET_SNAPSHOT,&current_snapshot,NULL);
    }

  /** Allocation de la structure window_snapshot **/
  window_snapshot = (struct window_snapshot *) calloc(1,sizeof(struct window_snapshot));
  if(window_snapshot == NULL)
    return(NULL);
  window_snapshot->menu_id = menu_id;
  window_snapshot->current_snapshot = current_snapshot;
  my_Memory(MEMORY_SET_WINDOW_SNAPSHOT,window_snapshot,NULL);

  /** Cration de la fentre Snapshot **/
  hwndSnapshot = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_SNAPSHOT),window_main->hwnd_main,(DLGPROC)ProcHandlerSnapshotWindow);
  if(hwndSnapshot == NULL)
    {
      my_Memory(MEMORY_FREE_WINDOW_SNAPSHOT,NULL,NULL);
      return(NULL);
    }

  /** Show Window + Paint Content **/
  ShowWindow(hwndSnapshot,SW_SHOWDEFAULT);
  UpdateWindow(hwndSnapshot);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_snapshot);
}


/*************************************************************************/
/*  ProcHandlerSnapshotWindow() :  Gestion des vnements de la fentre. */
/*************************************************************************/
LRESULT CALLBACK ProcHandlerSnapshotWindow(HWND hwndSnapshot, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  BOOL is_OK;
  int result;
  char file_path[BUFFER_SIZE];
  struct snapshot *previous_snapshot;
  struct snapshot *current_snapshot;
  struct snapshot *saveas_snapshot;
  struct window_snapshot *window_snapshot;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_SNAPSHOT,&window_snapshot,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateSnapshotChildWindows(hwndSnapshot,window_snapshot);
        return(TRUE);

      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : Type de la Rom **/
            case IDC_RADIO_APPLEII_ROM :
            case IDC_RADIO_APPLEIIGS_ROM01 :
            case IDC_RADIO_APPLEIIGS_ROM03 :
              break;

            case IDC_BUTTON_SAVESNAPSHOTAS :
              /* Rcupre les valeurs de la fentre (avec validation) */
              saveas_snapshot = GetSnapshotValueFromWindow(window_snapshot);
              if(saveas_snapshot == NULL)
                break;
                
              /* Demande un chemin de sauvegarde */
              if(window_snapshot->current_snapshot != NULL)
                strcpy(file_path,window_snapshot->current_snapshot->snapshot_file_path);
              else
                strcpy(file_path,"");
              is_OK = GetSaveFilePath(window_snapshot->hwnd_snapshot,"All Files (*.*)\0*.*\0\0\0",file_path);
              if(!is_OK)
                {
                  /* Si l'utilisateur l'annule */
                  mem_free_snapshot(saveas_snapshot);
                  if(window_snapshot->menu_id == ID_MENU_FILE_SAVEMEMORYSNAPSHOT)
                    SendMessage(hwndSnapshot,WM_CLOSE,0,0);    /* Quite la fentre */
                  break;     
                }
              strcpy(saveas_snapshot->snapshot_file_path,file_path);

              /** Sauvegarde des informations du Projet dans un fichier texte **/
              result = SaveSnapshotFile(saveas_snapshot,saveas_snapshot->snapshot_file_path);
              if(result)
                {
                  sprintf(window_snapshot->buffer_log,"Impossible to save File '%s'",saveas_snapshot->snapshot_file_path);
                  MessageBox(window_snapshot->hwnd_snapshot,window_snapshot->buffer_log,"Save Snapshot values",MB_OK|MB_ICONWARNING);
                  mem_free_snapshot(saveas_snapshot);
                  if(window_snapshot->menu_id == ID_MENU_FILE_SAVEMEMORYSNAPSHOT)
                    SendMessage(hwndSnapshot,WM_CLOSE,0,0);    /* Quite la fentre */
                  break;
                }

              /* Conserve le nouveau projet */
              my_Memory(MEMORY_GET_SNAPSHOT,&previous_snapshot,NULL);
              if(window_snapshot->current_snapshot != NULL)
                {
                  if(previous_snapshot != window_snapshot->current_snapshot)
                    mem_free_snapshot(window_snapshot->current_snapshot);
                }
              window_snapshot->current_snapshot = saveas_snapshot;

              /* Dgrise les Menu Snapshot */
              UpdateWindowMainMenuItemStatus();

              /* Ferme la fentre */
              if(window_snapshot->menu_id == ID_MENU_FILE_SAVEMEMORYSNAPSHOT)
                SendMessage(hwndSnapshot,WM_CLOSE,0,0);     /* Quite la fentre */
              break;                    
                                      
            case IDC_BUTTON_RUNSNAPSHOT :
              /* Rcupre les valeurs de la fentre (avec validation) */
              current_snapshot = GetSnapshotValueFromWindow(window_snapshot);
              if(current_snapshot == NULL)
                break;

              /* Libration du projet en cours */
              my_Memory(MEMORY_GET_SNAPSHOT,&previous_snapshot,NULL);
              if(previous_snapshot != NULL)
                {
                  if(window_snapshot->current_snapshot == previous_snapshot)
                    window_snapshot->current_snapshot = NULL;
                  my_Memory(MEMORY_FREE_SNAPSHOT,NULL,NULL);
                }

              /* Sauvegarde du Snapshot */
              my_Memory(MEMORY_SET_SNAPSHOT,current_snapshot,NULL);

              /* Dgrise les Menu Snapshot */
              UpdateWindowMainMenuItemStatus();

              /* Mise  jour de la Status Bar */
              UpdateWindowMainStatusBar();

              /* Init Engine */
              Initialize65816Engine(current_snapshot);

              /* Quite la fentre */
              SendMessage(hwndSnapshot,WM_CLOSE,0,0);
              break;

            case IDC_BUTTON_CANCEL :
              /* Le projet rattach  window_snapshot a t'il dj t sauvegard ? */
              if(window_snapshot->current_snapshot != NULL)
                {
                  /* Rcupre le Snapshot sauvegard */
                  my_Memory(MEMORY_GET_SNAPSHOT,&current_snapshot,NULL);
                  if(window_snapshot->current_snapshot == current_snapshot)
                    window_snapshot->current_snapshot = NULL;   /* Dsolidarise pour empcher sa libration */
                }

              /* Quite la fentre */
              SendMessage(hwndSnapshot,WM_CLOSE,0,0);
              break;
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Snapshot */
        DestroyWindow(hwndSnapshot);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteSnapshotChildWindows(window_snapshot);

        /* Dcoche l'option dans le Menu File */
        CheckMainWindowMenuItem(window_snapshot->menu_id,FALSE);

        /* Libration mmoire de la structure */
        my_Memory(MEMORY_FREE_WINDOW_SNAPSHOT,NULL,NULL);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*******************************************************************************************************/
/*  CreateSnapshotChildWindows() : Crer les fentres enfants et remplir la structure window_snapshot. */
/*******************************************************************************************************/
static int CreateSnapshotChildWindows(HWND hwndSnapshot, struct window_snapshot *window_snapshot)
{
  char buffer_hexa[256];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL); 

  /** Window Handle **/
  window_snapshot->hwnd_snapshot = hwndSnapshot;

  /** Surchage des Control Edit pour capter les RETURN lors des saisies **/
  window_snapshot->oldEditProcProcA = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_PROC_A),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcProcX = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_PROC_X),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcProcY = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_PROC_Y),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcProcDBR = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_PROC_DBR),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcProcPBR = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_PROC_PBR),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcProcPC = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_PROC_PC),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcProcStack = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_PROC_STACK),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcProcDPage = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_PROC_DPAGE),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcStatusC = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_STATUS_C),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcStatusZ = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_STATUS_Z),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcStatusI = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_STATUS_I),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcStatusD = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_STATUS_D),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcStatusX = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_STATUS_X),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcStatusM = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_STATUS_M),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcStatusV = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_STATUS_V),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcStatusN = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_STATUS_N),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcStatusE = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_STATUS_E),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);
  window_snapshot->oldEditProcStatusB = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndSnapshot,IDC_EDIT_STATUS_B),GWLP_WNDPROC,(LONG_PTR)subEditSnapshotProcControl);

  /* Processeur */
  window_snapshot->proc_a = window_snapshot->current_snapshot->proc_a;
  window_snapshot->proc_x = window_snapshot->current_snapshot->proc_x;
  window_snapshot->proc_y = window_snapshot->current_snapshot->proc_y;
  window_snapshot->proc_dbr = window_snapshot->current_snapshot->proc_dbr;
  window_snapshot->proc_pbr = window_snapshot->current_snapshot->proc_pbr;
  window_snapshot->proc_pc = window_snapshot->current_snapshot->proc_pc;
  window_snapshot->proc_stack = window_snapshot->current_snapshot->proc_stack;
  window_snapshot->proc_directpage = window_snapshot->current_snapshot->proc_directpage;
  sprintf(buffer_hexa,"%04X",window_snapshot->proc_a);
  SetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_A),buffer_hexa);
  sprintf(buffer_hexa,"%04X",window_snapshot->proc_x);
  SetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_X),buffer_hexa);
  sprintf(buffer_hexa,"%04X",window_snapshot->proc_y);
  SetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_Y),buffer_hexa);
  sprintf(buffer_hexa,"%02X",window_snapshot->proc_dbr);
  SetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_DBR),buffer_hexa);
  sprintf(buffer_hexa,"%02X",window_snapshot->proc_pbr);
  SetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_PBR),buffer_hexa);
  sprintf(buffer_hexa,"%04X",window_snapshot->proc_pc);
  SetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_PC),buffer_hexa);
  sprintf(buffer_hexa,"%04X",window_snapshot->proc_stack);
  SetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_STACK),buffer_hexa);
  sprintf(buffer_hexa,"%04X",window_snapshot->proc_directpage);
  SetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_DPAGE),buffer_hexa);

  /* Status Register */
  window_snapshot->status_c = window_snapshot->current_snapshot->status_c;
  window_snapshot->status_z = window_snapshot->current_snapshot->status_z;
  window_snapshot->status_i = window_snapshot->current_snapshot->status_i;
  window_snapshot->status_d = window_snapshot->current_snapshot->status_d;
  window_snapshot->status_x = window_snapshot->current_snapshot->status_x;
  window_snapshot->status_m = window_snapshot->current_snapshot->status_m;
  window_snapshot->status_v = window_snapshot->current_snapshot->status_v;
  window_snapshot->status_n = window_snapshot->current_snapshot->status_n;
  window_snapshot->status_e = window_snapshot->current_snapshot->status_e;
  window_snapshot->status_b = window_snapshot->current_snapshot->status_b;
  SetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_C,window_snapshot->status_c,FALSE);
  SetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_Z,window_snapshot->status_z,FALSE);
  SetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_I,window_snapshot->status_i,FALSE);
  SetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_D,window_snapshot->status_d,FALSE);
  SetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_X,window_snapshot->status_x,FALSE);
  SetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_M,window_snapshot->status_m,FALSE);
  SetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_V,window_snapshot->status_v,FALSE);
  SetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_N,window_snapshot->status_n,FALSE);
  SetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_E,window_snapshot->status_e,FALSE);
  SetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_B,window_snapshot->status_b,FALSE);

  /* RAM Size */
  window_snapshot->ram_size = window_snapshot->current_snapshot->ram_size;
  SetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_RAM_SIZE,window_snapshot->ram_size,FALSE);

  /* Apple II ROM / Apple IIgs ROM */
  window_snapshot->rom_type = window_snapshot->current_snapshot->rom_type;
  if(window_snapshot->rom_type == SNAPSHOT_APPLEII_ROM)
    {
      SendDlgItemMessage(window_snapshot->hwnd_snapshot,IDC_RADIO_APPLEII_ROM,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_snapshot->hwnd_snapshot,IDC_RADIO_APPLEIIGS_ROM01,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_snapshot->hwnd_snapshot,IDC_RADIO_APPLEIIGS_ROM03,BM_SETCHECK,0,0);    
    }
  else if(window_snapshot->rom_type == SNAPSHOT_APPLEIIGS_ROM01)
    {
      SendDlgItemMessage(window_snapshot->hwnd_snapshot,IDC_RADIO_APPLEII_ROM,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_snapshot->hwnd_snapshot,IDC_RADIO_APPLEIIGS_ROM01,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_snapshot->hwnd_snapshot,IDC_RADIO_APPLEIIGS_ROM03,BM_SETCHECK,0,0);     
    }
  else
    {
      SendDlgItemMessage(window_snapshot->hwnd_snapshot,IDC_RADIO_APPLEII_ROM,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_snapshot->hwnd_snapshot,IDC_RADIO_APPLEIIGS_ROM01,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_snapshot->hwnd_snapshot,IDC_RADIO_APPLEIIGS_ROM03,BM_SETCHECK,1,0);     
    }

  /* OK */
  return(0);
}


/****************************************************************************************/
/*  subEditSnapshotProcControl() :  Rcupre le RETURN pour valider les donnes saisies. */
/****************************************************************************************/
LRESULT CALLBACK subEditSnapshotProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_snapshot *window_snapshot;
  my_Memory(MEMORY_GET_WINDOW_SNAPSHOT,&window_snapshot,NULL);

  /* Protection */
  if(window_snapshot == NULL)
    return(0);

  /** Identifie le Control **/
  if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_A) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_X) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_Y) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_DBR) == hwnd_control)
    index = 4;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_PBR) == hwnd_control)
    index = 5;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_PC) == hwnd_control)
    index = 6;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_STACK) == hwnd_control)
    index = 7;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_DPAGE) == hwnd_control)
    index = 8;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_C) == hwnd_control)
    index = 9;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_Z) == hwnd_control)
    index = 10;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_I) == hwnd_control)
    index = 11;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_D) == hwnd_control)
    index = 12;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_X) == hwnd_control)
    index = 13;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_M) == hwnd_control)
    index = 14;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_V) == hwnd_control)
    index = 15;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_N) == hwnd_control)
    index = 16;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_E) == hwnd_control)
    index = 17;
  else if(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_B) == hwnd_control)
    index = 18;
  else
    return(0);      /* Pas trouv */

  /** Dtecte la touche RETURN pour valider la donne **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_snapshot->hwnd_snapshot);
    }
  else if(index == 1)
    return(CallWindowProc(window_snapshot->oldEditProcProcA,hwnd_control,msg,wParam,lParam));
  else if(index == 2)
    return(CallWindowProc(window_snapshot->oldEditProcProcX,hwnd_control,msg,wParam,lParam));
  else if(index == 3)
    return(CallWindowProc(window_snapshot->oldEditProcProcY,hwnd_control,msg,wParam,lParam));
  else if(index == 4)
    return(CallWindowProc(window_snapshot->oldEditProcProcDBR,hwnd_control,msg,wParam,lParam));
  else if(index == 5)
    return(CallWindowProc(window_snapshot->oldEditProcProcPBR,hwnd_control,msg,wParam,lParam));
  else if(index == 6)
    return(CallWindowProc(window_snapshot->oldEditProcProcPC,hwnd_control,msg,wParam,lParam));
  else if(index == 7)
    return(CallWindowProc(window_snapshot->oldEditProcProcStack,hwnd_control,msg,wParam,lParam));  
  else if(index == 8)
    return(CallWindowProc(window_snapshot->oldEditProcProcDPage,hwnd_control,msg,wParam,lParam));
  else if(index == 9)
    return(CallWindowProc(window_snapshot->oldEditProcStatusC,hwnd_control,msg,wParam,lParam));
  else if(index == 10)
    return(CallWindowProc(window_snapshot->oldEditProcStatusZ,hwnd_control,msg,wParam,lParam));
  else if(index == 11)
    return(CallWindowProc(window_snapshot->oldEditProcStatusI,hwnd_control,msg,wParam,lParam));
  else if(index == 12)
    return(CallWindowProc(window_snapshot->oldEditProcStatusD,hwnd_control,msg,wParam,lParam));
  else if(index == 13)
    return(CallWindowProc(window_snapshot->oldEditProcStatusX,hwnd_control,msg,wParam,lParam));
  else if(index == 14)
    return(CallWindowProc(window_snapshot->oldEditProcStatusM,hwnd_control,msg,wParam,lParam));
  else if(index == 15)
    return(CallWindowProc(window_snapshot->oldEditProcStatusV,hwnd_control,msg,wParam,lParam));
  else if(index == 16)
    return(CallWindowProc(window_snapshot->oldEditProcStatusN,hwnd_control,msg,wParam,lParam));
  else if(index == 17)
    return(CallWindowProc(window_snapshot->oldEditProcStatusE,hwnd_control,msg,wParam,lParam));
  else if(index == 18)
    return(CallWindowProc(window_snapshot->oldEditProcStatusB,hwnd_control,msg,wParam,lParam));

  /* OK */
  return(0);
}


/*********************************************************************/
/*  GetSnapshotValueFromWindow() :  Destruction des fentres enfants. */
/*********************************************************************/
static struct snapshot *GetSnapshotValueFromWindow(struct window_snapshot *window_snapshot)
{
  int number, is_error, pc_address, pc_in_memory;
  BYTE byte;
  WORD word;
  char buffer[BUFFER_SIZE];
  struct snapshot *current_snapshot;
 
  /* Allocation mmoire */
  current_snapshot = (struct snapshot *) calloc(1,sizeof(struct snapshot));
  if(current_snapshot == NULL)
    return(NULL);

  /** Processor **/
  /* A */
  GetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_A),buffer,BUFFER_SIZE-1);
  word = EvaluateAsWord(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Processor Value : A (16 bit hexadecimal is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->proc_a = word;
  /* X */
  GetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_X),buffer,BUFFER_SIZE-1);
  word = EvaluateAsWord(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Processor Value : X (16 bit hexadecimal is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->proc_x = word;
  /* Y */
  GetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_Y),buffer,BUFFER_SIZE-1);
  word = EvaluateAsWord(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Processor Value : Y (16 bit hexadecimal is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->proc_y = word;
  /* DBR */
  GetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_DBR),buffer,BUFFER_SIZE-1);
  byte = EvaluateAsByte(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Processor Value : DBR (8 bit hexadecimal is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->proc_dbr = byte;
  /* PBR */
  GetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_PBR),buffer,BUFFER_SIZE-1);
  byte = EvaluateAsByte(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Processor Value : PBR (8 bit hexadecimal is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->proc_pbr = byte;
  /* PC */
  GetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_PC),buffer,BUFFER_SIZE-1);
  word = EvaluateAsWord(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Processor Value : PC (16 bit hexadecimal is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->proc_pc = word;  
  /* Stack */
  GetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_STACK),buffer,BUFFER_SIZE-1);
  word = EvaluateAsWord(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Processor Value : Stack (16 bit hexadecimal is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->proc_stack = word;
  /* Direct Page */
  GetWindowText(GetDlgItem(window_snapshot->hwnd_snapshot,IDC_EDIT_PROC_DPAGE),buffer,BUFFER_SIZE-1);
  word = EvaluateAsWord(buffer,&is_error);
  if(is_error)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Processor Value : Direct Page (16 bit hexadecimal is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->proc_directpage = word;
  
  /** Status Register **/
  /* C */
  number = GetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_C,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Status Register Value : C (0 or 1 is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->status_c = number;
  /* Z */
  number = GetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_Z,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Status Register Value : Z (0 or 1 is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->status_z = number;
  /* I */
  number = GetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_I,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Status Register Value : I (0 or 1 is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->status_i = number;
  /* D */
  number = GetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_D,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Status Register Value : D (0 or 1 is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->status_d = number;
  /* X */
  number = GetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_X,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Status Register Value : X (0 or 1 is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->status_x = number;
  /* M */
  number = GetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_M,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Status Register Value : M (0 or 1 is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->status_m = number;
  /* V */
  number = GetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_V,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Status Register Value : V (0 or 1 is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->status_v = number;
  /* N */
  number = GetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_N,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Status Register Value : N (0 or 1 is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->status_n = number;
  /* E */
  number = GetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_E,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Status Register Value : E (0 or 1 is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->status_e = number;
  /* B */
  number = GetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_STATUS_B,NULL,FALSE);
  if(number != 0 && number != 1)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Status Register Value : B (0 or 1 is expected)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->status_b = number;
  
  /** Ram Size **/
  number = GetDlgItemInt(window_snapshot->hwnd_snapshot,IDC_EDIT_RAM_SIZE,NULL,FALSE);
  if(number > 128)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"Invalid Ram Size value : 128 is the maximum value (8 MB)","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
  current_snapshot->ram_size = number;
  
  /* Rom : Apple II / Apple IIgs */
  current_snapshot->rom_type = window_snapshot->rom_type;

  /** Le PC pointe t'il vers une zone mmoire existante ? **/
  pc_in_memory = 0;
  pc_address = current_snapshot->proc_pbr*64*1024 + current_snapshot->proc_pc;
  /* Ram */
  if(pc_address >= 0x000000 && pc_address < (2+current_snapshot->ram_size)*64*1024)
    pc_in_memory = 1;
  /* Ram E0 / E1 */
  if(pc_address >= 0xE00000 && pc_address <= 0xE1FFFF)
    pc_in_memory = 1;
  /* Rom 01 : FE-FF */
  if(current_snapshot->rom_type == SNAPSHOT_APPLEIIGS_ROM01)
    if(pc_address >= 0xFE0000 && pc_address <= 0xFFFFFF)
      pc_in_memory = 1;
  /* Rom 03 : FC-FF */
  if(current_snapshot->rom_type == SNAPSHOT_APPLEIIGS_ROM03)
    if(pc_address >= 0xFC0000 && pc_address <= 0xFFFFFF)
      pc_in_memory = 1;  
  if(pc_in_memory == 0)
    {
      MessageBox(window_snapshot->hwnd_snapshot,"The PC value is not located at a valid memory address !","Load Snapshot values",MB_OK|MB_ICONWARNING);
      mem_free_snapshot(current_snapshot);
      return(NULL);
    }
    
  /** Data : Header + Data + file Path **/
  if(window_snapshot->current_snapshot != NULL)
    {
      /* File Path */
      strcpy(current_snapshot->snapshot_file_path,window_snapshot->current_snapshot->snapshot_file_path);
      
      /* Header */
      current_snapshot->snapshot_type = window_snapshot->current_snapshot->snapshot_type;
      memcpy(current_snapshot->snapshot_header,window_snapshot->current_snapshot->snapshot_header,SNAPSHOT_HEADER_SIZE);
      
      /* Data */
      current_snapshot->snapshot_length = window_snapshot->current_snapshot->snapshot_length;
      current_snapshot->snapshot_data = (unsigned char *) calloc(current_snapshot->snapshot_length,sizeof(unsigned char));
      if(current_snapshot->snapshot_data == NULL)
        {
          mem_free_snapshot(current_snapshot);
          return(NULL);    
        }
      memcpy(current_snapshot->snapshot_data,window_snapshot->current_snapshot->snapshot_data,current_snapshot->snapshot_length);
    }

  /* Renvoi la structure */
  return(current_snapshot);
}


/**********************************************************************/
/*  DeleteSnapshotChildWindows() :  Destruction des fentres enfants. */
/**********************************************************************/
static void DeleteSnapshotChildWindows(struct window_snapshot *window_snapshot)
{
}


/************************************************************************/
/*  mem_free_window_snapshot() :  Libration mmoire pour la structure. */
/************************************************************************/
void mem_free_window_snapshot(struct window_snapshot *window_snapshot)
{
  if(window_snapshot)
    {
      if(window_snapshot->current_snapshot != NULL)
        mem_free_snapshot(window_snapshot->current_snapshot);

      free(window_snapshot);
    }
}

/**********************************************************************/
