/***********************************************************************/
/*                                                                     */
/*  Window_Statistic.h :  Header de gestion de la fentre Statistic.   */
/*                                                                     */
/***********************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : May 2024  */
/***********************************************************************/

/* Un message destin  la fentre Statistic pour demander la maj de l'affichage */
#define HM_STATISTIC_REFRESHWINDOW   (WM_USER + 24)

#define STATISTIC_REFRESH_AUTOMATIC       0
#define STATISTIC_REFRESH_MANUAL          1

#define STATISTIC_LISTVIEW_ROW_HEIGHT    15    /* Taille d'une ligne de la ListView (en Pixel) */

#define IDC_LISTVIEW_STATISTIC         1033

#define STATISTIC_LISTVIEW_NB_ROW        20    /* Nombre de ligne de donnes dans la ListView  */
#define STATISTIC_LISTVIEW_MIN_NB_ROW     2    /* Nombre de ligne de donnes Min dans la ListView  */

struct statistic_stack
{
  DWORD function_address;      /* Adresse de la fonction en mmoire */
  WORD call_number;            /* Numro d'appel pour Toolbox | GS/OS */
  DWORD function_end_address;  /* Adresse indiquant la fin de la fonction */

  QWORD gcc;                   /* Valeur de GCC  l'appel de la fonction */
  QWORD goid;                  /* Valeur de GOID  l'appel de la fonction */

  int is_interrupt;            /* On vient de rentrer dans une interruption */

  int depth;                   /* Niveau de profondeur de l'appel */

  struct statistic_stack *previous;
  struct statistic_stack *next;
};

struct statistic_function
{
  DWORD address;               /* Adresse de la fonction en mmoire */
  WORD call_number;            /* Numro d'appel pour Toolbox | GS/OS */
                               /* Nom de la fonction dans les Symbol */
                               /* Nom de l'application dans Internals */

  DWORD nb_call;               /* Nombre d'appel de la fonction */

  QWORD total_cycle;           /* Nombre Total de cycle pour tous les appels  */
  DWORD max_cycle;             /* Nombre Max de cycle pour 1 appel */
  DWORD min_cycle;             /* Nombre Min de cycle pour 1 appel */
                               /* Nombre Moyen de cycle pour 1 appel = TotalCycle/NbCall */

  struct statistic_function *next;
};

struct statistic_table
{
  /* Liste de toutes les fonctions */
  int nb_statistic_func;
  struct statistic_function *first_statistic_func;
  struct statistic_function *last_statistic_func;
  struct statistic_function **tab_statistic_func;      /* Tableau tri par adresse|call_number */
  DWORD total_call_statistic_func;                     /* Total des #Call */
  QWORD total_cycle_statistic_func;                    /* Total des #Cycle */

  /* Pile d'appel de toutes les fonctions */
  struct statistic_stack *first_statistic_stack;
  struct statistic_stack *last_statistic_stack;

  /* Historique de toutes les fonctions */
  int nb_fonction_history;
  struct fonction_history *first_fonction_history;
  struct fonction_history *last_fonction_history;

  QWORD gcc_init;                        /* Nombre de cycle au dmarrage de la collecte des statistiques */
};

#define FUNCTION_HISTORY_TYPE_ENTER_FUNCTION   1
#define FUNCTION_HISTORY_TYPE_ENTER_INTERRUPT  2
#define FUNCTION_HISTORY_TYPE_EXIT             3

struct fonction_history
{
  int type;              /* FUNCTION_HISTORY_TYPE_XXX */

  QWORD goid;
  QWORD cycle;
  DWORD function_address;
  WORD call_number;
  BYTE operation_PB;
  WORD operation_PC;
  BYTE prev_operation_PB;
  WORD prev_operation_PC;
  DWORD function_end_address;
  int depth;

  int exit_interrupt;
  int nb_unstack;
  DWORD nb_operation;
  DWORD nb_cycle;
  int nb_exit;

  struct fonction_history *next;
};

struct window_statistic
{
  HWND hwnd_statistic;
  HFONT hFont;

  /* Mode de retaille de la fentre (user/executable) */
  int user_resize;

  /* Dimension de la fentre  la cration */
  int window_init_width;
  int window_init_height;
  int nb_init_data_row;

  /* Mode de rafraichissement de la fentre */
  int refresh_mode;        /* STATISTIC_REFRESH_XXX */

  /* Fonction Slectionne */
  DWORD selected_statistic_address;     /* 0xFFFFFFFF si rien n'est slectionn */
  WORD selected_statistic_call_number;

  /* List View */
  int nb_row;                      /* Nombre de lignes affiches dans la ListBox */
  int statistic_x0;
  int statistic_y0;
  struct list_view listview_statistic;
  
  /* Buffer */
  char buffer_log[BUFFER_SIZE];

  /* Notification */
  struct notification *current_notification;
};

struct window_statistic *BuildStatisticWindow(HINSTANCE,int);
void ProcessStatisticOperation(struct window_main *,int);
void ClearStatisticTable(struct window_statistic *);
void mem_free_window_statistic(struct window_statistic *);
void mem_free_statistic(struct statistic_function *);
void mem_free_statistic_table(struct statistic_table *);

/********************************************************/
