/***********************************************************************/
/*                                                                     */
/*  Window_Symbol.h :  Header de gestion de la fentre Symbol.         */
/*                                                                     */
/***********************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : Oct 2023  */
/***********************************************************************/

#define SYMBOL_LISTVIEW_ROW_HEIGHT    15    /* Taille d'une ligne de la ListView (en Pixel) */

#define IDC_LISTVIEW_SYMBOL         1033

#define SYMBOL_LISTVIEW_NB_ROW        12    /* Nombre de ligne de donnes dans la ListView  */
#define SYMBOL_LISTVIEW_MIN_NB_ROW     2    /* Nombre de ligne de donnes Min dans la ListView  */

struct symbol
{
  /* Segment Load Address */
  DWORD segment_begin_address;     /* Adresse en mmoire du dbut du Segment auquel est attach ce Symbol */

  /* Nom de l'excutable */
  char *program_name;

  /* Donnes du fichier Symbol */
  char *segment_name;
  int segment_line;
  char *source_file_name;
  int source_file_line;
  DWORD symbol_relative_address;   /* Adresse non reloge du Symbol */
  char *name;                      /* Nom du Symbol */
  char *type;                      /* Type : Code / Data */
  char *data_type;                 /* Type de Data (DA,DS,DW,HEX,ASC,STRL,ADRL) */
  int size;                        /* Taille de la ligne (Opcode + Data) */
  char m;                          /* M : 0/1/? */
  char x;                          /* X : 0/1/? */
  char reloc[16];                  /* Reloc : 2,3,E 3,1 >>16,2 >> 8 */

  /* Ce Symbol a besoin d'tre relog */
  int require_relocate;            /* 0:No (Fixed Address) / 1:Yes (Rel Address) */
  int is_valid;                    /* 1 si valid */

  /* Adresse en mmoire */
  DWORD symbol_relocated_address;  /* Adresse Reloge = Segment Load Address + Adress non reloge */

  /* Liste Chane */
  struct symbol *next;
};

struct reloc
{
  /* Segment Load Address */
  DWORD segment_begin_address;     /* Adresse en mmoire du dbut du Segment auquel est attach ce Symbol */

  /* Nom de l'excutable */
  char *program_name;

  /* Donnes du fichier Symbol */
  char *segment_name;
  DWORD symbol_relative_address;   /* Adresse non reloge */
  char reloc[16];                  /* Reloc : 2,3,E 3,1 >>16,2 >> 8 */

  /* Ce Reloc a besoin d'tre relog */
  int require_relocate;            /* 0:No (Fixed Address) / 1:Yes (Rel Address) */
  int is_valid;                    /* 1 si valid */

  /* Adresse en mmoire */
  DWORD symbol_relocated_address;  /* Adresse Reloge = Segment Load Address + Adresse non reloge */

  struct reloc *next;
};

struct symbol_table
{
  /** Liste des Symbol **/
  int nb_symbol;
  struct symbol *first_symbol;
  struct symbol *last_symbol;
  struct symbol **tab_symbol;       /* Tableau tri par adresse */
  struct symbol **tab_symbol_name;  /* Tableau tri par name|segment|program */

  /** Liste des Symbol valides **/
  int nb_symbol_valid;
  struct symbol **tab_symbol_valid; /* Tableau tri par adresse */

  /** Liste des Reloc **/
  int nb_reloc;
  struct reloc *first_reloc;
  struct reloc *last_reloc;
  struct reloc **tab_reloc;         /* Tableau tri par adresse */

  /** Liste des Reloc valides **/
  int nb_reloc_valid;
  struct reloc **tab_reloc_valid;   /* Tableau tri par adresse */
};

struct segment
{
  char name[256];              /* Segment Name */
  int is_rellocated;           /* 1 is contains relocated symbols */

  struct segment *next;
};

struct window_symbol
{
  HWND hwnd_symbol;
  HFONT hFont;

  /* Mode de retaille de la fentre (user/executable) */
  int user_resize;

  /* Dimension de la fentre  la cration */
  int window_init_width;
  int window_init_height;
  int nb_init_data_row;

  /* List View */
  int nb_row;                      /* Nombre de lignes affiches dans la ListBox */
  int symbol_x0;
  int symbol_y0;
  struct list_view listview_symbol;

  /* Symbol slectionn */
  char current_program_name[1024];
  char current_segment_name[1024];
  DWORD current_segment_address;

  /* Buffer */
  char buffer_log[BUFFER_SIZE];  
};

struct window_symbol *BuildSymbolWindow(HINSTANCE,int);
void LoadAppendSymbolFile(char *,struct window_symbol *);
struct symbol *GetSymbolByAddress(DWORD);
struct reloc *GetRelocByAddress(DWORD);
struct symbol *GetSymbolByPreviousAddress(DWORD);
struct symbol *GetSymbolByNameSegmentFile(char *,char *,char *);
void ClearSymbolTable(struct window_symbol *);
void RefreshSymbolWindow(struct window_symbol *);
void UpdateSymbolAddress(void);
int IsSameSegmentName(char *,char *);
void mem_free_window_symbol(struct window_symbol *);
void mem_free_symbol(struct symbol *);
void mem_free_reloc(struct reloc *);
void mem_free_symbol_table(struct symbol_table *);

/********************************************************/
