/************************************************************************/
/*                                                                      */
/*  Window_TextPage80.c :  Module de gestion de la fentre TextPage80.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Juillet 2018  */
/************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_TextPage80.h"

static DWORD BuildTextPage80WindowThread(void *);
LRESULT CALLBACK ProcHandlerTextPage80Window(HWND,UINT,WPARAM,LPARAM);
static int CreateTextPage80ChildWindows(HWND,struct window_textpage80 *);
LRESULT CALLBACK subEditTextPage80ProcControl(HWND,UINT,WPARAM,LPARAM);
void UpdateTextPage80ClientAllArea(HDC,struct window_textpage80 *);
static void UpdateTextPage80ClientBitmapArea(HDC,struct window_textpage80 *);
static void UpdateTextPage80ContextInformation(struct window_textpage80 *,int,int);
static void UpdateTextPage80Address(struct window_textpage80 *,int);
static void UpdateTextPage80RefreshMode(struct window_textpage80 *,int);
static void UpdateTextPage80ZoomLevel(struct window_textpage80 *,int);
static void RefreshTextPage80WindowData(struct window_textpage80 *,int);
static int IsTextPage80DataModified(int,int,struct window_textpage80 *);
static void ResizeTextPage80Window(struct window_textpage80 *);
static void BuildTextPage80FromData(struct window_textpage80 *);
static void DeleteTextPage80ChildWindows(struct window_textpage80 *);


/**********************************************************************************/
/*  BuildTextPage80Window() :  Construction / Affichage de la fentre TextPage80. */
/**********************************************************************************/
struct window_textpage80 *BuildTextPage80Window(HINSTANCE hInstance, int menu_id)
{
  struct window_textpage80 *window_textpage80;

  /** Allocation de la structure window_textpage80 **/
  window_textpage80 = (struct window_textpage80 *) calloc(1,sizeof(struct window_textpage80));
  if(window_textpage80 == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_TEXTPAGE80,window_textpage80,NULL);

  /* Lance un thread pour grer la fentre TextPage80 */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildTextPage80WindowThread,window_textpage80,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_textpage80);
}


/********************************************************************************************/
/*  BuildTextPage80WindowThread() :  Cration de la fentre TextPage80 via un Thread ddi. */
/********************************************************************************************/
static DWORD BuildTextPage80WindowThread(void *window_textpage80_param)
{
  HWND hwndTextPage80;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_textpage80 *window_textpage80;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_textpage80 = (struct window_textpage80 *) window_textpage80_param;
  window_textpage80->thread_id = GetCurrentThreadId();         /* Le ThreaID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Text Page 80 Column **/
  hwndTextPage80 = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_TEXTPAGE80),window_main->hwnd_main,(DLGPROC)ProcHandlerTextPage80Window);
  if(hwndTextPage80 == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndTextPage80,SW_SHOWDEFAULT);
  UpdateWindow(hwndTextPage80);

  /** Message Loop pour toutes les fentres de ce Thread (fentre TextPage80 + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***************************************************************************/
/*  ProcHandlerTextPage80Window() :  Gestion des vnements de la fentre. */
/***************************************************************************/
LRESULT CALLBACK ProcHandlerTextPage80Window(HWND hwndTextPage80, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;
  LRESULT result;
  WORD param_wd;
  int mouse_x, mouse_y;
  struct window_textpage80 *window_textpage80;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_TEXTPAGE80,&window_textpage80,NULL);
  if(window_textpage80 == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateTextPage80ChildWindows(hwndTextPage80,window_textpage80);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        UpdateTextPage80ContextInformation(window_textpage80,mouse_x,mouse_y);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_textpage80->hBitmap_textpage80 != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndTextPage80,&ps);

            /** MAJ Bitmap **/
            UpdateTextPage80ClientAllArea(hDC,window_textpage80);

            /* End Paint */
            EndPaint(hwndTextPage80,&ps);
          }
        break;
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_textpage80->refresh_mode != TEXTPAGE80_REFRESH_AUTOMATIC)
                UpdateTextPage80RefreshMode(window_textpage80,TEXTPAGE80_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_textpage80->refresh_mode != TEXTPAGE80_REFRESH_MANUAL)
                UpdateTextPage80RefreshMode(window_textpage80,TEXTPAGE80_REFRESH_MANUAL);
              break;

            /** Bouton Radio : On change le Zoom de la fentre **/
            case IDC_RADIO_ZOOM_X1 :
              if(HIWORD(wParam) == BN_CLICKED && window_textpage80->zoom_level != TEXTPAGE80_ZOOM_X1)
                UpdateTextPage80ZoomLevel(window_textpage80,TEXTPAGE80_ZOOM_X1);
              break;

            case IDC_RADIO_ZOOM_X2 :
              if(HIWORD(wParam) == BN_CLICKED && window_textpage80->zoom_level != TEXTPAGE80_ZOOM_X2)
                UpdateTextPage80ZoomLevel(window_textpage80,TEXTPAGE80_ZOOM_X2);
              break;

            /** Bouton de dplacement de l'adresse **/
            case IDC_BUTTON_LEFT :
              UpdateTextPage80Address(window_textpage80,-1);
              break;

            case IDC_BUTTON_RIGHT :
              UpdateTextPage80Address(window_textpage80,1);
              break;

            case IDC_BUTTON_UP :
              UpdateTextPage80Address(window_textpage80,-80);
              break;

            case IDC_BUTTON_DOWN :
              UpdateTextPage80Address(window_textpage80,80);
              break;

            case IDC_BUTTON_PAGEUP :
              UpdateTextPage80Address(window_textpage80,-1024);
              break;

            case IDC_BUTTON_PAGEDOWN :
              UpdateTextPage80Address(window_textpage80,1024);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshTextPage80WindowData(window_textpage80,1);
              break;
              
            /** Slection dans la ComboBox **/
            case IDC_COMBOBOX_TEXTPAGE80_COLOR_TEXT :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_textpage80->hwnd_combo_color_text,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_textpage80->color_text)
                      {
                        window_textpage80->color_text = (int) result;
                        RefreshTextPage80WindowData(window_textpage80,1);
                      }
                }
              break;              
             
            case IDC_COMBOBOX_TEXTPAGE80_COLOR_BACKGROUND :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_textpage80->hwnd_combo_color_background,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_textpage80->color_background)
                      {
                        window_textpage80->color_background = (int) result;
                        RefreshTextPage80WindowData(window_textpage80,1);
                      }
                }
              break; 
             
            case IDC_COMBOBOX_TEXTPAGE80_DISPLAY_LANGUAGE :
              param_wd = HIWORD(wParam);
              if(param_wd == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_textpage80->hwnd_combo_display_language,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_textpage80->display_language)
                      {
                        window_textpage80->display_language = (int) result;
                        RefreshTextPage80WindowData(window_textpage80,1);
                      }
                }
              break;                           
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_TEXTPAGE80_REFRESHWINDOW :
        RefreshTextPage80WindowData(window_textpage80,0);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre TextPage80 */
        DestroyWindow(hwndTextPage80);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteTextPage80ChildWindows(window_textpage80);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_TEXTPAGE80,window_textpage80,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************/
/*  CreateTextPage80ChildWindows() : Crer les fentres enfants et remplir la structure window_textpage80. */
/***********************************************************************************************************/
static int CreateTextPage80ChildWindows(HWND hwndTextPage80, struct window_textpage80 *window_textpage80)
{
  BITMAPINFO bitmap_info;
  int i;
  char buffer[BUFFER_SIZE];
  char *color_name[16] = {"Black","Deep Red","Dark Blue","Purple","Dark Green","Dark Gray","Medium Blue","Light Blue","Brown","Orange","Light Gray","Pink","Light Green","Yellow","Aquamarine","White"};
  char *display_language[9] = {"U.S.A.","U.K.","French","Danish","Spanish","Italian","German","Swedish","MouseText"};
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_textpage80->hwnd_textpage80 = hwndTextPage80;

  /** Device Context **/
  window_textpage80->hDC = GetDC(window_textpage80->hwnd_textpage80);
  window_textpage80->hCompatibleDC = CreateCompatibleDC(window_textpage80->hDC);

  /** Surchage des Control Edit pour capter les vnements **/
  window_textpage80->oldEditProcBank    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndTextPage80,IDC_EDIT_BANK),GWLP_WNDPROC,(LONG_PTR)subEditTextPage80ProcControl);
  window_textpage80->oldEditProcAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndTextPage80,IDC_EDIT_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditTextPage80ProcControl);

  /** Ajoute les images sur les boutons **/
  SendMessage(GetDlgItem(hwndTextPage80,IDC_BUTTON_LEFT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndTextPage80,IDC_BUTTON_RIGHT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndTextPage80,IDC_BUTTON_UP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndTextPage80,IDC_BUTTON_DOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndTextPage80,IDC_BUTTON_PAGEUP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndTextPage80,IDC_BUTTON_PAGEDOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Cration des ComboBox **/
  /* Text Color */
  window_textpage80->hwnd_combo_color_text = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                            96,68,100,250,hwndTextPage80,(HMENU)IDC_COMBOBOX_TEXTPAGE80_COLOR_TEXT,param->hInstance,0);
  for(i=0; i<16; i++)
    SendMessage(window_textpage80->hwnd_combo_color_text,CB_ADDSTRING,0,(LPARAM)color_name[i]);

  /* Background Color */
  window_textpage80->hwnd_combo_color_background = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                                  301,68,100,250,hwndTextPage80,(HMENU)IDC_COMBOBOX_TEXTPAGE80_COLOR_BACKGROUND,param->hInstance,0);
  for(i=0; i<16; i++)
    SendMessage(window_textpage80->hwnd_combo_color_background,CB_ADDSTRING,0,(LPARAM)color_name[i]);

  /* Display Language */
  window_textpage80->hwnd_combo_display_language = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                                  301,100,100,220,hwndTextPage80,(HMENU)IDC_COMBOBOX_TEXTPAGE80_DISPLAY_LANGUAGE,param->hInstance,0);
  for(i=0; i<9; i++)
    SendMessage(window_textpage80->hwnd_combo_display_language,CB_ADDSTRING,0,(LPARAM)display_language[i]);

  /* 1res valeurs */
  window_textpage80->color_text = COLOR_IIGS_WHITE;
  window_textpage80->color_background = COLOR_IIGS_MEDIUM_BLUE;
  window_textpage80->display_language = LANGUAGE_IIGS_USA;
  SendMessage(window_textpage80->hwnd_combo_color_text,CB_SETCURSEL,window_textpage80->color_text,0);
  SendMessage(window_textpage80->hwnd_combo_color_background,CB_SETCURSEL,window_textpage80->color_background,0);
  SendMessage(window_textpage80->hwnd_combo_display_language,CB_SETCURSEL,window_textpage80->display_language,0);

  /** Cration de la Police de caractre + Affectation de la police au Control Edit **/
  window_textpage80->hFont = CreateFont(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");
  SendMessage(window_textpage80->hwnd_combo_color_text,WM_SETFONT,(WPARAM)window_textpage80->hFont,TRUE);
  SendMessage(window_textpage80->hwnd_combo_color_background,WM_SETFONT,(WPARAM)window_textpage80->hFont,TRUE);
  SendMessage(window_textpage80->hwnd_combo_display_language,WM_SETFONT,(WPARAM)window_textpage80->hFont,TRUE);

  /** Adresse de la Zone de Data **/
  window_textpage80->textpage80_bank = 0x00;
  window_textpage80->textpage80_address = 0x0400; 
  sprintf(buffer,"%02X",window_textpage80->textpage80_bank);
  SetDlgItemText(window_textpage80->hwnd_textpage80,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",window_textpage80->textpage80_address);
  SetDlgItemText(window_textpage80->hwnd_textpage80,IDC_EDIT_ADDRESS,buffer);

  /* Refresh Mode : Automatic */
  UpdateTextPage80RefreshMode(window_textpage80,TEXTPAGE80_REFRESH_AUTOMATIC);

  /* Zoom Level : 1 */
  window_textpage80->zoom_level = TEXTPAGE80_ZOOM_X1;
  SendDlgItemMessage(window_textpage80->hwnd_textpage80,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_textpage80->hwnd_textpage80,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_textpage80->hwnd_textpage80,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);

  /* Dimension en Pixel de la Bitmap TextPage80 Data */
  window_textpage80->bitmap_textpage80_width = 560;
  window_textpage80->bitmap_textpage80_height = 192;
  
  /** Bitmap affichant la zone de slection **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_textpage80->bitmap_textpage80_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_textpage80->bitmap_textpage80_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_textpage80->hBitmap_textpage80 = CreateDIBSection(window_textpage80->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_textpage80->bitmap_data_textpage80),NULL,0);
  for(i=0; i<window_textpage80->bitmap_textpage80_width*window_textpage80->bitmap_textpage80_height; i++)
    window_textpage80->bitmap_data_textpage80[i] = BITMAP_BACKGROUND_COLOR;

  /* Retaille la fentre */
  ResizeTextPage80Window(window_textpage80);

  /** Construction de la Page Texte + Rafraichissement de la fentre **/
  RefreshTextPage80WindowData(window_textpage80,1);
    
  /* OK */
  return(0);
}


/*************************************************************************************************/
/*  subEditTextPage80ProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*************************************************************************************************/
LRESULT CALLBACK subEditTextPage80ProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_textpage80 *window_textpage80;
  my_Memory(MEMORY_GET_WINDOW_TEXTPAGE80,&window_textpage80,NULL);

  /* Rien  faire */
  if(window_textpage80 == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_textpage80->hwnd_textpage80,IDC_EDIT_BANK) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_textpage80->hwnd_textpage80,IDC_EDIT_ADDRESS) == hwnd_control)
    index = 2;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement du Return dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_textpage80->hwnd_textpage80);

      /* On force la MAJ de la fentre */
      RefreshTextPage80WindowData(window_textpage80,1);
    }
  else if(index == 1)
    return(CallWindowProc(window_textpage80->oldEditProcBank,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_textpage80->oldEditProcAddress,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
} 


/**************************************************************************************/
/*  UpdateTextPage80ClientAllArea() :  Affichage des Bitmap de la fentre TextPage80. */
/**************************************************************************************/
void UpdateTextPage80ClientAllArea(HDC hDC, struct window_textpage80 *window_textpage80)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap **/
  UpdateTextPage80ClientBitmapArea(hDC,window_textpage80);
}


/*****************************************************************************/
/*  UpdateTextPage80ClientBitmapArea() :  Affichage de la Bitmap TextPage80. */
/*****************************************************************************/
static void UpdateTextPage80ClientBitmapArea(HDC hDC, struct window_textpage80 *window_textpage80)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec la zone mmoire et la zone slectionne **/
  bitmap_old = SelectObject(window_textpage80->hCompatibleDC,window_textpage80->hBitmap_textpage80);
  StretchBlt(hDC,window_textpage80->bitmap_textpage80_x,window_textpage80->bitmap_textpage80_y,
             window_textpage80->bitmap_textpage80_width*window_textpage80->zoom_level,
             window_textpage80->bitmap_textpage80_height*window_textpage80->zoom_level*2,  /* Double la hauteur */
             window_textpage80->hCompatibleDC,0,0,window_textpage80->bitmap_textpage80_width,window_textpage80->bitmap_textpage80_height,SRCCOPY);
  SelectObject(window_textpage80->hCompatibleDC,bitmap_old);
}


/*******************************************************************************/
/*  UpdateTextPage80Address() :  Dplace l'adresse pour la fentre TextPage80. */
/*******************************************************************************/
static void UpdateTextPage80Address(struct window_textpage80 *window_textpage80, int delta)
{
  int current_address, new_address, textpage80_bank, textpage80_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_textpage80->textpage80_bank << 16) | window_textpage80->textpage80_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  textpage80_bank = (new_address >> 16) & 0xFF;
  textpage80_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",textpage80_bank);
  SetDlgItemText(window_textpage80->hwnd_textpage80,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",textpage80_address);
  SetDlgItemText(window_textpage80->hwnd_textpage80,IDC_EDIT_ADDRESS,buffer);

  /* MAJ de la fentre */
  RefreshTextPage80WindowData(window_textpage80,1);
}


/*******************************************************************************************************/
/*  UpdateTextPage80ContextInformation() :  MAJ des information de contexte de la fenetre TextPage80. */
/*******************************************************************************************************/
static void UpdateTextPage80ContextInformation(struct window_textpage80 *window_textpage80, int mouse_x, int mouse_y)
{
  int pixmap_x, pixmap_y, grid_x, grid_y;
  int textpage80_data_address;
  unsigned char textpage80_data;
  int tab_text80_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };

  /* Rien  faire */
  if(window_textpage80 == NULL)
    return;

  /* Est t'on dans l'image ? */
  if(mouse_x < window_textpage80->bitmap_textpage80_x || mouse_x >= (window_textpage80->bitmap_textpage80_x + window_textpage80->bitmap_textpage80_width*window_textpage80->zoom_level) ||
     mouse_y < window_textpage80->bitmap_textpage80_y || mouse_y >= (window_textpage80->bitmap_textpage80_y + window_textpage80->bitmap_textpage80_height*window_textpage80->zoom_level*2))
    {
      SetWindowText(GetDlgItem(window_textpage80->hwnd_textpage80,IDC_STATIC_CONTEXT),"");
      return;
    }

  /* Coordonnes dans la Pixmap */
  pixmap_x = (mouse_x - window_textpage80->bitmap_textpage80_x)/window_textpage80->zoom_level;
  pixmap_y = (mouse_y - window_textpage80->bitmap_textpage80_y)/window_textpage80->zoom_level;

  /* Coordonnes dans la grille 80x24 */
  grid_x = pixmap_x/7;
  grid_y = pixmap_y/(8*2);

  /* Valeur */
  textpage80_data = window_textpage80->textpage80_data[grid_x + grid_y*80];

  /* Adresse */
  textpage80_data_address = ((window_textpage80->textpage80_bank+(((grid_x%2)==0)?1:0)) << 16) + window_textpage80->textpage80_address;
  textpage80_data_address += (grid_x/2);
  textpage80_data_address += tab_text80_address[grid_y];

  /* Message */
  sprintf(window_textpage80->buffer_log,"Memory Data  %02X/%04X : %02X  %d,%d",textpage80_data_address>>16,textpage80_data_address&0xFFFF,textpage80_data,grid_x,grid_y);
  SetWindowText(GetDlgItem(window_textpage80->hwnd_textpage80,IDC_STATIC_CONTEXT),window_textpage80->buffer_log);
}


/************************************************************************************************/
/*  UpdateTextPage80RefreshMode() :  MAJ du mode de rafraichissement de la fenetre TextPage80. */
/************************************************************************************************/
static void UpdateTextPage80RefreshMode(struct window_textpage80 *window_textpage80, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == TEXTPAGE80_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_textpage80->refresh_mode = TEXTPAGE80_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_textpage80->hwnd_textpage80,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_textpage80->hwnd_textpage80,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_textpage80->hwnd_textpage80;
      current_notification->message = HM_TEXTPAGE80_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_TEXTPAGE80;

      /* Ajout de la notification */
      window_textpage80->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_textpage80->refresh_mode = TEXTPAGE80_REFRESH_MANUAL;
      SendDlgItemMessage(window_textpage80->hwnd_textpage80,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_textpage80->hwnd_textpage80,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_textpage80->current_notification,NULL);
      window_textpage80->current_notification = NULL;
    }
}


/***********************************************************************************/
/*  UpdateTextPage80ZoomLevel() :  MAJ du nieau de Zomm de la fenetre TextPage80. */
/***********************************************************************************/
static void UpdateTextPage80ZoomLevel(struct window_textpage80 *window_textpage80, int zoom_level)
{
  if(zoom_level == TEXTPAGE80_ZOOM_X1)
    {
      /* Visuel */
      window_textpage80->zoom_level = TEXTPAGE80_ZOOM_X1;
      SendDlgItemMessage(window_textpage80->hwnd_textpage80,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_textpage80->hwnd_textpage80,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_textpage80->hwnd_textpage80,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
    }
  else if(zoom_level == TEXTPAGE80_ZOOM_X2)
    {
      /* Visuel */
      window_textpage80->zoom_level = TEXTPAGE80_ZOOM_X2;
      SendDlgItemMessage(window_textpage80->hwnd_textpage80,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_textpage80->hwnd_textpage80,IDC_RADIO_ZOOM_X2,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_textpage80->hwnd_textpage80,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
    }
    
  /* Retaille la fentre */
  ResizeTextPage80Window(window_textpage80);  
  
  /* On force la MAJ de la fentre */  
  RefreshTextPage80WindowData(window_textpage80,1);    
}


/******************************************************************************/
/*  RefreshTextPage80WindowData() :  MAJ du contenu de la fentre TextPage80. */
/******************************************************************************/
static void RefreshTextPage80WindowData(struct window_textpage80 *window_textpage80, int force)
{
  int length, is_modified;
  int textpage80_bank, textpage80_address;
  char buffer[1024];

  /* Rcupre la valeur du TextPage80 Pointer */
  length = GetDlgItemText(window_textpage80->hwnd_textpage80,IDC_EDIT_BANK,buffer,1024);
  sscanf(buffer,"%X",&textpage80_bank);
  length = GetDlgItemText(window_textpage80->hwnd_textpage80,IDC_EDIT_ADDRESS,buffer,1024);
  sscanf(buffer,"%X",&textpage80_address);

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",textpage80_bank);
  SetDlgItemText(window_textpage80->hwnd_textpage80,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",textpage80_address);
  SetDlgItemText(window_textpage80->hwnd_textpage80,IDC_EDIT_ADDRESS,buffer);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsTextPage80DataModified(textpage80_bank,textpage80_address,window_textpage80);
  if(is_modified == 0 && force == 0)
    return;

  /** Cration de la Zone Text Graphique  partir des nouvelles donnes **/
  BuildTextPage80FromData(window_textpage80);

  /* Rafraichit la fentre */
  InvalidateRect(window_textpage80->hwnd_textpage80,NULL,TRUE);
  UpdateWindow(window_textpage80->hwnd_textpage80);
}


/*************************************************************************************************************/
/*  IsTextPage80DataModified() :  Les donnes TextPage80 ont t'elles t modifies depuis la dernirer fois. */
/*************************************************************************************************************/
static int IsTextPage80DataModified(int textpage80_bank, int textpage80_address, struct window_textpage80 *window_textpage80)
{
  int x, y, textpage80_data_address;
  unsigned char textpage80_raw_data_main[1024];
  unsigned char textpage80_raw_data_aux[1024];
  unsigned char textpage80_data[80*24];
  int tab_text80_address[24] = 
    {
      0x00,0x80,0x100,0x180,0x200,0x280,0x300,0x380,    /* Adresse du dbut des 24 lignes de la Page Text */
      0x28,0xA8,0x128,0x1A8,0x228,0x2A8,0x328,0x3A8,
      0x50,0xD0,0x150,0x1D0,0x250,0x2D0,0x350,0x3D0
    };
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /* Rcupre les donnes brutes : 2x1024 chars */
  textpage80_data_address = (textpage80_bank << 16) | textpage80_address;
  ReadMemoryBlock(textpage80_data_address,1024,textpage80_raw_data_main,current_mmu);
  textpage80_data_address = ((textpage80_bank+1) << 16) | textpage80_address;
  ReadMemoryBlock(textpage80_data_address,1024,textpage80_raw_data_aux,current_mmu);
  
  /* R-organise les donnes : Linarisation (passage du block de 1024 char en 80*24) */
  for(y=0; y<24; y++)
    for(x=0; x<40; x++)
      {
        textpage80_data[(2*x+0)+y*80] = textpage80_raw_data_aux[tab_text80_address[y]+x];
        textpage80_data[(2*x+1)+y*80] = textpage80_raw_data_main[tab_text80_address[y]+x];
      }
      
  /* Compare les donnes */
  if(window_textpage80->textpage80_data != NULL)
    if(textpage80_bank == window_textpage80->textpage80_bank &&
       textpage80_address == window_textpage80->textpage80_address &&
       memcmp(&textpage80_data[0],&window_textpage80->textpage80_data[0],80*24) == 0)
      return(0);

  /* Les donnes sont diffrentes => MAJ */
  window_textpage80->textpage80_bank = textpage80_bank;
  window_textpage80->textpage80_address = textpage80_address;
  memcpy(window_textpage80->textpage80_data,textpage80_data,80*24);

  /* Donnes diffrentes ou FLASH */
  return(1);
}


/***********************************************************************/
/*  ResizeTextPage80Window() : Resize TextPage80 window by given size. */
/***********************************************************************/
static void ResizeTextPage80Window(struct window_textpage80 *window_textpage80)
{
  HWND hwnd_group;
  RECT window_rect, client_rect, group_rect;
  int window_x, window_y;
  
  /** On va ajuster la fentre par rapport au Zoom demand **/  
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_textpage80->hwnd_textpage80,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_textpage80->hwnd_textpage80,&client_rect);
  window_textpage80->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select */
  hwnd_group = GetDlgItem(window_textpage80->hwnd_textpage80,IDC_GROUP_TEXTPAGE80);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_TEXTPAGE80 */
  GetWindowRect(hwnd_group,&group_rect);
  window_textpage80->group_data_x = (group_rect.left - window_rect.left) - 3;
  window_textpage80->group_data_y = ((group_rect.top - window_rect.top) - window_textpage80->title_height) + 3;

  /* Position de la Zone BitMap dans la fenetre */
  window_textpage80->bitmap_textpage80_x = window_textpage80->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_textpage80->bitmap_textpage80_y = window_textpage80->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_textpage80->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_textpage80->bitmap_textpage80_width*window_textpage80->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_textpage80->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_textpage80->bitmap_textpage80_height*window_textpage80->zoom_level*2 + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_textpage80->group_data_x,window_textpage80->group_data_y,window_textpage80->group_data_width,window_textpage80->group_data_height,TRUE);

  /* Largeur de la fentre */
  window_textpage80->window_width = window_textpage80->group_data_x + window_textpage80->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* On doit garantir une largeur minimale de la fentre */
  if(window_textpage80->window_width < TEXTPAGE80_DIALOGBOX_WIDTH)
    window_textpage80->window_width = TEXTPAGE80_DIALOGBOX_WIDTH;

  /* Hauteur de la fentre */
  window_textpage80->window_height = window_textpage80->title_height + window_textpage80->group_data_y + window_textpage80->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Resize window */
  MoveWindow(window_textpage80->hwnd_textpage80,window_x,window_y,window_textpage80->window_width,window_textpage80->window_height,TRUE);
}


/************************************************************************************************************/
/*  BuildTextPage80FromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/************************************************************************************************************/
static void BuildTextPage80FromData(struct window_textpage80 *window_textpage80)
{
  int one_byte;
  int x, y, pixmap_char_x, pixmap_char_y, pixel_x, pixel_y, offset_pixmap_char, offset_pixmap_screen;
  char buffer_title[256];
  DWORD color_text, color_background, color_pixmap, color_screen;
  DWORD tab_rgb_color[16] = 
    {
      /* 16 couleurs de la Bordure / Fond Ecran / Text */
      MAKECOLOR(0,0,0), MAKECOLOR(221,0,51), MAKECOLOR(0,0,153), MAKECOLOR(221,34,221),
      MAKECOLOR(0,119,34), MAKECOLOR(85,85,85), MAKECOLOR(34,34,255), MAKECOLOR(102,170,255),
      MAKECOLOR(136,85,0), MAKECOLOR(255,102,0), MAKECOLOR(170,170,170), MAKECOLOR(255,153,136),
      MAKECOLOR(17,221,0), MAKECOLOR(255,255,0), MAKECOLOR(68,255,153), MAKECOLOR(255,255,255)
    };
  struct picture_true *picture_char;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rien  faire */
  if(window_textpage80->bitmap_data_textpage80 == NULL)
    return;

  /* Adresse de la picture */
  if(window_textpage80->display_language == LANGUAGE_IIGS_USA)
    picture_char = param->picture_char_us;
  else if(window_textpage80->display_language == LANGUAGE_IIGS_UK)
    picture_char = param->picture_char_uk;
  else if(window_textpage80->display_language == LANGUAGE_IIGS_FRENCH)
    picture_char = param->picture_char_fr;
  else if(window_textpage80->display_language == LANGUAGE_IIGS_DANISH)
    picture_char = param->picture_char_dk;
  else if(window_textpage80->display_language == LANGUAGE_IIGS_SPANISH)
    picture_char = param->picture_char_se;
  else if(window_textpage80->display_language == LANGUAGE_IIGS_ITALIAN)
    picture_char = param->picture_char_it;
  else if(window_textpage80->display_language == LANGUAGE_IIGS_GERMAN)
    picture_char = param->picture_char_de;
  else if(window_textpage80->display_language == LANGUAGE_IIGS_SWEDISH)
    picture_char = param->picture_char_se;
  else if(window_textpage80->display_language == LANGUAGE_IIGS_MOUSETEXT)
    picture_char = param->picture_char_mousetext;
    
  /* Couleur du Texte */
  color_text = tab_rgb_color[window_textpage80->color_text];

  /* Couleur du Fond */
  color_background = tab_rgb_color[window_textpage80->color_background];

  /** Cration de la bitmap 560*192 correspondant au 80*24 **/
  for(y=0; y<24; y++)
    {
      for(x=0; x<80; x++)
        {
          /* Caractre */
          one_byte = (int) window_textpage80->textpage80_data[x + y*80];

          /* Offset dans la Pixmap Charset */
          pixmap_char_y = one_byte / 40;
          pixmap_char_x = one_byte - (pixmap_char_y*40);

          /* Copie ce bloc de 7*8 */
          offset_pixmap_char  = pixmap_char_x*7 + pixmap_char_y*(40*7)*8;
          offset_pixmap_screen = x*7 + y*(80*7)*8;
          for(pixel_y=0; pixel_y<8; pixel_y++)
            for(pixel_x=0; pixel_x<7; pixel_x++)
              {
                /* Pixel dans la Char Tab */
                color_pixmap = picture_char->data_dwd[offset_pixmap_char + pixel_x + pixel_y*40*7];

                /* Couleur du Pixel final */
                color_screen = (color_pixmap == 0xFFFFFF) ? color_text : color_background;

                /* Place le point dans la Pixmap */
                window_textpage80->bitmap_data_textpage80[offset_pixmap_screen + pixel_x + pixel_y*80*7] = color_screen;
              }
        }
    }

  /* Titre de la fentre */
  sprintf(buffer_title,"Text Page 80 Column : %02X/%04X",window_textpage80->textpage80_bank,window_textpage80->textpage80_address);
  SetWindowText(window_textpage80->hwnd_textpage80,buffer_title);
}


/************************************************************************/
/*  DeleteTextPage80ChildWindows() :  Destruction des fentres enfants. */
/************************************************************************/
static void DeleteTextPage80ChildWindows(struct window_textpage80 *window_textpage80)
{
  DeleteObject(window_textpage80->hBitmap_textpage80);
  DeleteObject(window_textpage80->hwnd_combo_color_text);
  DeleteObject(window_textpage80->hwnd_combo_color_background);
  DeleteObject(window_textpage80->hwnd_combo_display_language);
  DeleteObject(window_textpage80->hFont);
  ReleaseDC(window_textpage80->hwnd_textpage80,window_textpage80->hDC);
  DeleteDC(window_textpage80->hCompatibleDC);
}


/**************************************************************************/
/*  mem_free_window_textpage80() :  Libration mmoire pour la structure. */
/**************************************************************************/
void mem_free_window_textpage80(struct window_textpage80 *window_textpage80)
{
  if(window_textpage80)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_textpage80->current_notification,NULL);
      window_textpage80->current_notification = NULL;
      
      free(window_textpage80);
    }
}

/**********************************************************************/
