/********************************************************************/
/*                                                                  */
/*   Cyrene_Breakpoint.h :  Header de gestion des Breakpoints.      */
/*                                                                  */
/********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : May 2023  */
/********************************************************************/

#define BREAKPOINT_SOURCE_ADDRESS            1
#define BREAKPOINT_SOURCE_SYMBOL             2
#define BREAKPOINT_SOURCE_SYSTEMCALL         3
#define BREAKPOINT_SOURCE_REGISTER           4
#define BREAKPOINT_SOURCE_OPERATION          5

#define BREAKPOINT_TRIGGER_READ              1
#define BREAKPOINT_TRIGGER_WRITE             2
#define BREAKPOINT_TRIGGER_READWRITE         3
#define BREAKPOINT_TRIGGER_EXECUTE           4
#define BREAKPOINT_TRIGGER_HOLD              5

#define BREAKPOINT_ACTION_STOP               1
#define BREAKPOINT_ACTION_PRINT              2
#define BREAKPOINT_ACTION_FILE               3
#define BREAKPOINT_ACTION_TRACE_SYSTEM_CALL  4

#define BREAKPOINT_REPEAT_ALWAYS             1
#define BREAKPOINT_REPEAT_ONLY               2
#define BREAKPOINT_REPEAT_AFTER              3

#define BREAKPOINT_REGISTER_A                0
#define BREAKPOINT_REGISTER_X                1
#define BREAKPOINT_REGISTER_Y                2
#define BREAKPOINT_REGISTER_PC               3
#define BREAKPOINT_REGISTER_STACK            4
#define BREAKPOINT_REGISTER_DIRECT           5
#define BREAKPOINT_REGISTER_DBR              6
#define BREAKPOINT_REGISTER_PBR              7
#define BREAKPOINT_REGISTER_STATUS           8

#define BREAKPOINT_OPERATOR_EQUAL            0
#define BREAKPOINT_OPERATOR_DIFFERENT        1
#define BREAKPOINT_OPERATOR_GREATER          2
#define BREAKPOINT_OPERATOR_GREATER_EQUAL    3
#define BREAKPOINT_OPERATOR_LOWER            4
#define BREAKPOINT_OPERATOR_LOWER_EQUAL      5  

#define BREAKPOINT_ADDRESS_EMPTY             0    /* No Address Available */
#define BREAKPOINT_ADDRESS_NUM               1    /* Address fully available */
#define BREAKPOINT_ADDRESS_TXT               2    /* Address has * */

struct cyrene_breakpoint
{
  int status;                          /* 0:Disable / 1:Enable */

  DWORD id;                            /* Identifiant unique d'un Breakpoint (1-N) */

  /* Breakpoint en version Texte */
  char breakpoint_line[8192];

  /* Source */
  int source;                          /* BREAKPOINT_SOURCE_XXX */
  /* Address */
  int source_address_type;             /* BREAKPOINT_ADDRESS_XXX */
  DWORD source_address_masked;         /* Address with * replaced by 0 */
  DWORD source_address_mask;           /* Mask * nibble */
  int source_address_type_dst;         /* BREAKPOINT_ADDRESS_XXX */
  DWORD source_address_masked_dst;     /* Address with * replaced by 0 */
  DWORD source_address_mask_dst;       /* Mask * nibble */
  /* SystemCall */
  int source_systemcall_type;
  WORD source_systemcall_number;
  /* Register */
  int source_register;                 /* BREAKPOINT_REGISTER_XXX */
  /* Operation */
  char source_operation_text[256];
  int source_operation_data_nb;
  BYTE source_operation_data[4];       /* [0]=Opcode, [1,2,3]=Operand */

  /* Trigger */
  int trigger;                         /* BREAKPOINT_TRIGGER_XXX */
  int trigger_read_operator;           /* BREAKPOINT_OPERATOR_XXX */
  int trigger_read_value_size;         /* 0:None / 1:8bit / 2:16bit */
  WORD trigger_read_value;
  int trigger_write_operator;          /* BREAKPOINT_OPERATOR_XXX */
  int trigger_write_value_size;        /* 0:None / 1:8bit / 2:16bit */
  WORD trigger_write_value;
  int trigger_readwrite_operator;      /* BREAKPOINT_OPERATOR_XXX */
  int trigger_readwrite_value_size;    /* 0:None / 1:8bit / 2:16bit */
  WORD trigger_readwrite_value;
  int trigger_hold_operator;           /* BREAKPOINT_OPERATOR_XXX */
  int trigger_hold_value_size;         /* 0:None / 1:8bit / 2:16bit */
  WORD trigger_hold_value;

  /* Action */
  int action;                          /* BREAKPOINT_ACTION_XXX */
  char action_print_text[1024];        /* [A] [X] [Y] [PC] ... [012000] */
  char action_file_path[1024];         /* File Path */

  /* Repeat */
  int repeat;                          /* BREAKPOINT_REPEAT_XXX */
  DWORD repeat_only_times;

  /* Stat */
  DWORD nb_times;                      /* Nombre de fois o ce Breakpoint est lev */

  struct cyrene_breakpoint *next;
};

int CheckForBreakpointMatch(struct cyrene_breakpoint *,struct cyrene_operation *,unsigned char *);
void BuildBreakpointOutputLine(char *,struct cyrene_operation *,struct cyrene_parameter *,char *);
void UpdateBreakpointList(struct cyrene_parameter *);
void mem_free_breakpoint(struct cyrene_breakpoint *);

/********************************************************************/
