/*************************************************************************/
/*                                                                       */
/*   Cyrene_Operation.h :  Header de gestion des donnes des Oprations. */
/*                                                                       */
/*************************************************************************/
/*  Auteur : Olivier ZARDINI   *   Brutal Deluxe   *   Date : May 2023   */
/*************************************************************************/

/* Operation Flags */
#define OPERATION_FLAG_INTERRUPT             0x80   /* Interruption has occurred */

struct cyrene_operation
{
  dword64 dfcyc;            /* Current */

  WORD a_reg;               /* 65c816 Processor Registers */
  WORD x_reg;
  WORD y_reg;
  WORD pc_reg;
  BYTE pb_reg;
  BYTE db_reg;
  WORD stack_reg;
  WORD direct_reg;
  WORD status_reg;

  BYTE opcode;              /* Operation Opcode */
  BYTE operand_1;           /* Operation Operand */
  BYTE operand_2;
  BYTE operand_3;

  BYTE s_C019;              /* RDVBLBAR : VBL */
  BYTE s_C02E;              /* VERTCNT */
  BYTE s_C02F;              /* HORIZCNT */
  BYTE s_C035;              /* SHADOW : Shadowing 01 -> E1 */
  BYTE s_C068;              /* STATEREG : Stack/DirectPage in Bank 01 */

  /* Operation Extra Information */
  int nb_cycle;             /* Operation Cycle number */
  int operand_data_size;    /* Number of bytes in Operand (0-3) */
  DWORD operand_address_1;  /* Operand Address #1 (0xFFFFFFFF if not available) */
  DWORD operand_address_2;  /* Operand Address #2 for MVN/MVP (0xFFFFFFFF if not available) */
  int operand_addr_size_1;  /* Operand Address #1 size (1 or 2 bytes) : STA $20FA, 1 byte if M=1 ($20FA), 2 bytes is M=0 ($20FA-$20FB) */
  int operand_addr_size_2;  /* Operand Address #2 size (Always 1 byte because MVN/MVP impact only 1 byte on target address) */
  DWORD jump_address;       /* Execute */
  WORD call_number;         /* Toolbox / GS/OS call number */

  /* Address used in Breakpoint */
  DWORD read_address;       /* Read (0xFFFFFFFF if no READ performed) */
  WORD read_value;
  BYTE read_value_size;     /* 0-2 */
  BYTE read_value_wrap;
  DWORD write_address;      /* Write (0xFFFFFFFF if no WRITE performed) */
  WORD write_value;
  BYTE write_value_size;    /* 0-2 */
  BYTE write_value_wrap;
  DWORD execute_address;    /* Execute */

  /* Flags */
  BYTE flags;               /* Interrupt / Data Size / Address Wrap */
};

void GetOperationExtraInformation(struct cyrene_operation *,struct cyrene_parameter *);  /* Compute Cycle Count + Breakpoint information for the Operation */

/*************************************************************************/
