/********************************************************************/
/*                                                                  */
/*   Cyrene_Shared.c :  Module de gestion des fonctions partages.  */
/*                                                                  */
/********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : May 2023  */
/********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>

#include "defc.h"
#include "sound.h"
#include "Cyrene_Operation.h"
#include "Cyrene_Shared.h"
#include "Cyrene.h"


/*************************************************************/
/*  SetMemoryData() :  On modifie la mmoire de l'mulateur. */
/*************************************************************/
void SetMemoryData(int ram_type, DWORD address, DWORD data_size, unsigned char *data, struct cyrene_parameter *cyrene_param)
{
  DWORD i;

  /** RAM **/
  if(ram_type == RAMTYPE_RAM)
    {
      for(i=0; i<data_size; i++)
        SetMemoryByte(address+i,data[i],cyrene_param);
    }
  /** DOC RAM **/
  else if(ram_type == RAMTYPE_DOC)
    {
      for(i=0; i<data_size; i++)
        if(i < 0x00010000)
          cyrene_param->doc_ram[i] = data[i];
    }
}


/********************************************************/
/*  GetMemoryValue() :  Rcupre une valeur en mmoire. */
/********************************************************/
DWORD GetMemoryValue(DWORD address, int value_size, struct cyrene_operation *current_operation, struct cyrene_parameter *cyrene_param)
{
  DWORD value;
  BYTE byte_1, byte_2, byte_3, byte_4;

  /* Rcupre les donnes */
  byte_1 = GetMemoryByte(address+0,current_operation,cyrene_param);
  if(value_size >= 2)
    byte_2 = GetMemoryByte(address+1,current_operation,cyrene_param);
  if(value_size >= 3)
    byte_3 = GetMemoryByte(address+2,current_operation,cyrene_param);
  if(value_size >= 4)
    byte_4 = GetMemoryByte(address+3,current_operation,cyrene_param);

  /* Construit la valeur */
  value = (DWORD) byte_1;
  if(value_size >= 2)
    value |= ((DWORD) byte_2) << 8;
  if(value_size >= 3)
    value |= ((DWORD) byte_3) << 16;
  if(value_size >= 4)
    value |= ((DWORD) byte_4) << 24;

  /* Renvoie la valeur */
  return(value);
}


/*****************************************************/
/*  GetMemoryByte() :  Rcupre un octet en mmoire. */
/*****************************************************/
BYTE GetMemoryByte(DWORD address, struct cyrene_operation *current_operation, struct cyrene_parameter *cyrene_param)
{
  BYTE bank_address;
  WORD softswich;
  int tab_softswich[256] = 
    {/*  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F  */
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   /* $C000 - $C00F */
      0, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1,   /* $C010 - $C01F */
      0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 2, 2,   /* $C020 - $C02F */
      0, 1, 1, 1, 1, 2, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1,   /* $C030 - $C03F */
      0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   /* $C040 - $C04F */
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   /* $C050 - $C05F */
      1, 1, 1, 1, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 1,   /* $C060 - $C06F */
      0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1,   /* $C070 - $C07F */
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   /* $C080 - $C09F */
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   /* $C0A0 - $C0AF */
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   /* $C0B0 - $C0BF */
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   /* $C0C0 - $C0CF */
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   /* $C0D0 - $C0DF */
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   /* $C0E0 - $C0EF */
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0    /* $C0F0 - $C0FF */
    };

  /* Init */
  bank_address = (BYTE) ((address & 0x00FF0000) >> 16);

  /** SoftSwitch : $00|01|E0|E1/C000-C0FF **/
  if((bank_address == 0x00 || bank_address == 0x01 || bank_address == 0xE0 || bank_address == 0xE1) && (address & 0x0000FF00) == 0x0000C000)
    {
      softswich = address & 0x0000FFFF;

      /* Softswitch dj connus */
      if(softswich == 0x0000C019)
        return(current_operation->s_C019);
      else if(softswich == 0x0000C02E)
        return(current_operation->s_C02E);
      else if(softswich == 0x0000C02F)
        return(current_operation->s_C02F);
      else if(softswich == 0x0000C035)
        return(current_operation->s_C035);
      else if(softswich == 0x0000C068)
        return(current_operation->s_C068);

      /* Avoid active Softswitch */
      if(tab_softswich[softswich & 0x00FF] == 0)
        return(0x00);

      /* Use Native KEGS IORead Function */
      //printf("%04X,",softswich);
      return((BYTE) cyrene_param->_io_read(address & 0x00000fff,&current_operation->dfcyc));
    }
  /** Fast RAM : $00/0000 - $7D/FFFF **/
  else if(address < cyrene_param->max_ram_address)
    return(cyrene_param->page_info_rd_wr[(address & 0x00FFFF00) >> 8].rd_wr[address & 0x000000FF]);
  /** Slow RAM : $E0/0000-$E1/FFFF **/
  else if(address >= 0x00E00000 && address <= 0x00E1FFFF)
    return(cyrene_param->page_info_rd_wr[(address & 0x00FFFF00) >> 8].rd_wr[address & 0x000000FF]);
  /** ROM 01 : $FE/0000-$FF/FFFF **/
  else if(cyrene_param->version_rom == 1 && bank_address >= 0xFE && bank_address <= 0xFF)
    return(cyrene_param->gs_rom[(address & 0x00FFFFFF)-0x00FC0000]);
  /** ROM 03 : $FC/0000-$FF/FFFF **/
  else if(cyrene_param->version_rom == 3 && bank_address >= 0xFC && bank_address <= 0xFF)
    return(cyrene_param->gs_rom[(address & 0x00FFFFFF)-0x00FC0000]);
  else
    return(0x00);   /* Unknown or Invalid Address */
}


/**************************************************/
/*  SetMemoryByte() :  Ecrit un octet en mmoire. */
/**************************************************/
void SetMemoryByte(DWORD address, BYTE value_byte, struct cyrene_parameter *cyrene_param)
{
  BYTE bank_address;

  /* Init */
  bank_address = (BYTE) ((address & 0x00FF0000) >> 16);

  /** SoftSwitch : $00|01|E0|E1/C000-C0FF **/
  if((bank_address == 0x00 || bank_address == 0x01 || bank_address == 0xE0 || bank_address == 0xE1) && (address & 0x0000FF00) == 0x0000C000)
    return;
  /** Fast RAM : $00/0000 - $7D/FFFF **/
  else if(address < cyrene_param->max_ram_address)
    cyrene_param->page_info_rd_wr[(address & 0x00FFFF00) >> 8].rd_wr[address & 0x000000FF] = value_byte;
  /** Slow RAM : $E0/0000-$E1/FFFF **/
  else if(address >= 0x00E00000 && address <= 0x00E1FFFF)
    cyrene_param->page_info_rd_wr[(address & 0x00FFFF00) >> 8].rd_wr[address & 0x000000FF] = value_byte;
  /** ROM 01 : $FE/0000-$FF/FFFF **/
  else if(cyrene_param->version_rom == 1 && bank_address >= 0xFE && bank_address <= 0xFF)
    cyrene_param->gs_rom[(address & 0x00FFFFFF)-0x00FC0000] = value_byte;
  /** ROM 03 : $FC/0000-$FF/FFFF **/
  else if(cyrene_param->version_rom == 3 && bank_address >= 0xFC && bank_address <= 0xFF)
    cyrene_param->gs_rom[(address & 0x00FFFFFF)-0x00FC0000] = value_byte;
  else
    ;   /* Unknown or Invalid Address */
}


/***************************************************************/
/*  RefreshEmulatorWindows() :  Refresh KEGS Emulator Windows. */
/***************************************************************/
int RefreshEmulatorWindows(struct cyrene_parameter *cyrene_param)
{
  int exit_now;

  /* Init */
  exit_now = 0;

  /* Refresh KEGS Main Window */
  cyrene_param->_x_update_display(cyrene_param->g_mainwin_info);

  /* Refresh KEGS Debug Window */
  cyrene_param->_x_update_display(cyrene_param->g_debugwin_info);

  /* Process Windows Messages */
  cyrene_param->_check_input_events();

  /* Check Exit Code */
  if(*cyrene_param->g_a2_fatal_err == 0x10)
    exit_now = 1;

  /* Renvoi le status */
  return(exit_now);
}


/*******************************************************************************************/
/*  GetNamedValueText() :  Rcupre la valeur Texte d'une variable dont on fournit le nom. */
/*******************************************************************************************/
void GetNamedValueText(char *value_list, char *name, char *value_rtn)
{
  int length_value;
  char *begin_value;
  char *end_value;

  /* Init */
  strcpy(value_rtn,"");

  /* Recherche le nom */
  begin_value = strstr(value_list,name);
  if(begin_value)
    {
      begin_value += strlen(name);
      end_value = strchr(begin_value,',');
      if(end_value)
        {
          /* Traite la valeur Texte */
          length_value = (int) (end_value - begin_value);
          memcpy(value_rtn,begin_value,length_value);
          value_rtn[length_value] = '\0';
        }
    }
}


/****************************************************************************************/
/*  GetNamedValueInt() :  Rcupre la valeur Int d'une variable dont on fournit le nom. */
/****************************************************************************************/
int GetNamedValueInt(char *value_list, char *name)
{
  int length_value;
  char *begin_value;
  char *end_value;
  char value[1024];

  /* Init */
  strcpy(value,"");

  /* Recherche le nom */
  begin_value = strstr(value_list,name);
  if(begin_value)
    {
      begin_value += strlen(name);
      end_value = strchr(begin_value,',');
      if(end_value)
        {
          /* Traite la valeur Texte */
          length_value = (int) (end_value - begin_value);
          memcpy(value,begin_value,length_value);
          value[length_value] = '\0';

          /* Conversion */
          return(atoi(value));
        }
    }

  /* Pas trouv */
  return(0);
}


/**********************************************************/
/*  EvaluateAsOneByte() :  Evalue une expression en BYTE. */
/**********************************************************/
BYTE EvaluateAsOneByte(char *value_txt_param, int *is_error_rtn)
{
  int i, is_hexa, is_odd;
  BYTE value_b;
  char value_txt[3];

  /* Init */
  *is_error_rtn = 0;
  memcpy(value_txt,value_txt_param,2);
  value_txt[2] = '\0';

  /* Teste les caractres */
  is_hexa = IsHexaDecimal(value_txt,&is_odd);
  if(is_hexa == 0 || is_odd == 1)
    {
      *is_error_rtn = 1;
      return(0);
    }

  /* Conversion */
  sscanf(value_txt,"%x",&i);
  if(i < 0 || i > 255)
    {
      /* Overflow */
      *is_error_rtn = 1;
      return(0);
    }
  value_b = (BYTE) i;

  /* Renvoi la valeur */
  return(value_b);
}


/*******************************************************/
/*  EvaluateAsByte() :  Evalue une expression en BYTE. */
/*******************************************************/
BYTE EvaluateAsByte(char *value_txt, int *is_error_rtn)
{
  int i, is_hexa, is_odd;
  BYTE value_b;

  /* Init */
  *is_error_rtn = 0;

  /* Teste les caractres */
  is_hexa = IsHexaDecimal(value_txt,&is_odd);
  if(is_hexa == 0 || is_odd == 1)
    {
      *is_error_rtn = 1;
      return(0);
    }

  /* Conversion */
  sscanf(value_txt,"%x",&i);
  if(i < 0 || i > 255)
    {
      /* Overflow */
      *is_error_rtn = 1;
      return(0);
    }
  value_b = (BYTE) i;

  /* Renvoi la valeur */
  return(value_b);
}


/*******************************************************/
/*  EvaluateAsWord() :  Evalue une expression en WORD. */
/*******************************************************/
WORD EvaluateAsWord(char *value_txt, int *is_error_rtn)
{
  int i, is_hexa, is_odd;
  WORD value_wd;

  /* Init */
  *is_error_rtn = 0;

  /* Teste les caractres */
  is_hexa = IsHexaDecimal(value_txt,&is_odd);
  if(is_hexa == 0 || is_odd == 1)
    {
      *is_error_rtn = 1;
      return(0);
    }

  /* Conversion */
  sscanf(value_txt,"%x",&i);
  if(i < 0 || i > 65535)
    {
      /* Overflow */
      *is_error_rtn = 1;
      return(0);
    }
  value_wd = (WORD) i;

  /* Renvoi la valeur */
  return(value_wd);
}


/*******************************************************/
/*  IsHexaDecimal() :  Est-ce une valeur hexadcimal ? */
/*******************************************************/
int IsHexaDecimal(char *value, int *is_odd_rtn)
{
  int i;

  /* Init */
  *is_odd_rtn = 0;

  /* Valeur vide */
  if(strlen(value) == 0)
    return(0);

  /* On teste la plage de caractre 0-F */
  for(i=1; i<(int)strlen(value); i++)
    if(!((toupper(value[i]) >= '0' && toupper(value[i]) <= '9') || (toupper(value[i]) >= 'A' && toupper(value[i]) <= 'F')))
      return(0);

  /* Nombre pair d'octets */
  if((strlen(value) % 2) == 1)
    *is_odd_rtn = 1;

  /* OK */
  return(1);
}


/******************************************************************/
/*  IsHexaDecimalChar() :  Dtermine si ce caractre est en Hexa. */
/******************************************************************/
int IsHexaDecimalChar(char one_char)
{
  if(one_char >= '0' && one_char <= '9')
    return(1);
  if(toupper(one_char) >= 'A' && toupper(one_char) <= 'F')
    return(1);
  return(0);
}


/*********************************************************************************/
/*  BuildUniqueListFromText() :  Rcupre la liste des valeurs unique d'un Text. */
/*********************************************************************************/
char **BuildUniqueListFromText(char *text, char separator, int *nb_value_rtn)
{
  int i, j, nb_found, nb_value, value_length;
  char **tab;
  char buffer_value[1024];

  /* Compte le nombre de sparateur */
  nb_value = 1;
  for(i=0; text[i] != '\0'; i++)
    if(text[i] == separator)
      nb_value++;

  /* Allocation du tableau */
  tab = (char **) calloc(nb_value,sizeof(char *));
  if(tab == NULL)
    return(NULL);

  /** Lecture des valeurs **/
  nb_found = 0;
  value_length = 0;
  for(i=0; text[i] != '\0'; i++)
    {
      if(text[i] == separator)
        {
          if(value_length > 0)
            {
              /* Termine la valeur */
              buffer_value[value_length] = '\0';
              
              /* Vrifie qu'elle n'existe pas dj */
              for(j=0; j<nb_found; j++)
                if(!strcmp(tab[j],buffer_value))
                  {
                    value_length = 0;
                    break;
                  }
                  
              /* Stocke la valeur */
              if(value_length > 0)
                {
                  tab[nb_found] = strdup(buffer_value);
                  if(tab[nb_found] == NULL)
                    {
                      for(j=0; j<nb_found; j++)
                        if(tab[j] != NULL)
                          free(tab[j]);
                      free(tab);
                      return(NULL);
                    }
                  nb_found++;
                  value_length = 0;
                }
            }
        }
      else
        buffer_value[value_length++] = text[i];
    }

  /** Dernire valeur **/
  if(value_length > 0)
    {
      /* Termine la valeur */
      buffer_value[value_length] = '\0';
      
      /* Vrifie qu'elle n'existe pas dj */
      for(j=0; j<nb_found; j++)
        if(!strcmp(tab[j],buffer_value))
          {
            value_length = 0;
            break;
          }
          
      /* Stocke la valeur */
      if(value_length > 0)
        {
          tab[nb_found] = strdup(buffer_value);
          if(tab[nb_found] == NULL)
            {
              for(j=0; j<nb_found; j++)
                if(tab[j] != NULL)
                  free(tab[j]);
              free(tab);
              return(NULL);
            }
          nb_found++;
          value_length = 0;
        }
    }

  /* Renvoi le tableau */
  *nb_value_rtn = nb_found;
  return(tab);
}


/*****************************************************/
/*  CleanBuffer() : On nettoy un buffer genre @Trim. */
/*****************************************************/
void CleanBuffer(char *buffer)
{
  int i, j;
  int length;

  /* Nettoyage arrire */
  length = (int) strlen(buffer);
  for(i=length-1; i>=0; i--)
    if(buffer[i] == '\0' || buffer[i] == ' ' || buffer[i] == '\n' || buffer[i] == '\t')
      buffer[i] = '\0';
    else
      break;

  /* Nettoyage avant */
  length = (int) strlen(buffer);
  for(i=0,j=0; i<length; i++)
    if(buffer[i] == ' ' || buffer[i] == '\n' || buffer[i] == '\t')
      j++;
    else
      break;

  /* On dplace si besoin */
  if(j > 0)
    memmove(&buffer[0],&buffer[j],(length-j)+1);
}


/************************************************************************/
/*  my_stristr() :  Recherche une chaine dans une autre, sans la casse. */
/************************************************************************/
char *my_stristr(char *string1, char *string2)
{
  int i, l_1, l_2;

  /* Calcule les longueurs */
  l_1 = (int) strlen(string1);
  l_2 = (int) strlen(string2);

  /* Cas extrmes */
  if(l_2 > l_1)
    return(NULL);
  else if(l_2 == l_1)
    {
      if(!stricmp(string1,string2))
        return(string1);
      return(NULL);
    }

  /* On parcours la chaine string1 */
  for(i=0; i<(l_1-l_2+1); i++)
    if(!strnicmp(&string1[i],string2,(size_t)l_2))
      return(&string1[i]);

  /* Pas trouv */
  return(NULL);
}


/******************************************************************************/
/*  GetCurrentTime() :  On rcupre l'heure actuelle (avec les milliseconds). */
/******************************************************************************/
char *GetCurrentTimeMS(void)
{
  time_t clock;
  struct _timeb timebuffer;
  struct tm *time_date;
  static char buffer_time[256];

  /* Date actuelle */
  _ftime(&timebuffer);
  time(&clock);
  time_date = localtime(&clock);

  /* Formate l'heure */
  sprintf(buffer_time,"[%02d:%02d:%02d-%03d]",time_date->tm_hour,time_date->tm_min,time_date->tm_sec,timebuffer.millitm);

  /* Renvoi l'heure */
  return(&buffer_time[0]);
}


/************************************************************************/
/*  LogTextLineDirect() :  Ecriture d'une ligne de log dans un fichier. */
/************************************************************************/
void LogTextLineDirect(char *log_file_path, char *log_line, int add_eol)
{
  FILE *fd;

  /* Ouverture du fichier */
  fd = fopen(log_file_path,"a+");
  if(fd == NULL)
    return;

  /* Ecriture de la ligne */
  fwrite(log_line,1,strlen(log_line),fd);

  /* Sparateur */
  if(add_eol == 1)
    fwrite("\n",1,strlen("\n"),fd);

  /* Fermeture du fichier */
  fclose(fd);
}


/*************************************************************************/
/*  mem_free_list() :  libration des lments d'un tableau de pointeur. */
/*************************************************************************/
void mem_free_list(int nb_item, char **tab)
{
  int i;
  
  if(tab)
    {
      for(i=0; i<nb_item; i++)
        if(tab[i])
          free(tab[i]);

      free(tab);
    }
}

/********************************************************************/
