/*********************************************************************/
/*                                                                   */
/*   Cyrene_SystemCall.h :  Header de gestion des appels systmes.   */
/*                                                                   */
/*********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : May 2023   */
/*********************************************************************/

/* Trace System Call */
#define TRACE_SYSTEM_CALL_P8                    1
#define TRACE_SYSTEM_CALL_P16                   2
#define TRACE_SYSTEM_CALL_GSOS                  4
#define TRACE_SYSTEM_CALL_TOOLBOX               8

/* System Call Type */
#define TYPE_SYSTEM_CALL_P8                     1
#define TYPE_SYSTEM_CALL_P16                    2
#define TYPE_SYSTEM_CALL_GSOS                   3
#define TYPE_SYSTEM_CALL_TOOLBOX                4

/* Trace System Calls Origin */
#define TRACE_CALL_ORIGIN_GSOS                  1
#define TRACE_CALL_ORIGIN_P16                   2
#define TRACE_CALL_ORIGIN_P8                    3
#define TRACE_CALL_ORIGIN_TOOL                  4
#define TRACE_CALL_ORIGIN_RETURN                5

/* Valeurs de retour des appels systmes */
struct cyrene_system_call
{
  QWORD operation_goid;
  QWORD operation_gcc;
  DWORD return_address;
  WORD call_number;
  int origin;                 /* TRACE_CALL_ORIGIN_XXX */

  int nb_param;
  DWORD tab_param_address[20];
  WORD tab_param_type[20];

  struct cyrene_system_call *previous;
  struct cyrene_system_call *next;
};

void ProcessTraceSystemCall(DWORD,DWORD,DWORD,DWORD,WORD,struct cyrene_operation *,struct cyrene_parameter *);
void ProcessTraceSystemCallReturnValues(struct cyrene_system_call *,WORD,int,struct cyrene_operation *,struct cyrene_parameter *);
int IsProdos16Call(DWORD call_address, WORD call_number);
void RemoveSystemCallFromList(struct cyrene_system_call *,struct cyrene_parameter *);
void mem_free_system_call(struct cyrene_system_call *);

/*********************************************************************/
