/********************************************************************/
/*                                                                  */
/*   Cyrene_WindowIPC.h :  Header de gestion de la fentre IPC.     */
/*                                                                  */
/********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : May 2023  */
/********************************************************************/

/* Cyrene / Kegs Connection Status */
#define CONNECTION_STATUS_CLOSED                     0   /* No connection open between Cyrene and Kegs */
#define CONNECTION_STATUS_OPEN                       1   /* Connection Open between Cyrene and Kegs */

/* IPC Window Creation status */
#define IPC_WINDOW_STATUS_WAIT                       0   /* Wait for IPC Window Creation */
#define IPC_WINDOW_STATUS_READY                      1   /* IPC Window created */
#define IPC_WINDOW_STATUS_CLOSE                      2   /* IPC Window closed */
#define IPC_WINDOW_STATUS_ERROR                      3   /* IPC Window not created */

/* Kegs -> Cyrene Query */
#define CONNECTION_QUERY_NOTHING                     0   /* Cyrene is not asking for Data */
#define CONNECTION_QUERY_SNAPSHOT_DATA               1   /* Cyrene is asking for Snapshot IIgs Data */
#define CONNECTION_QUERY_OPERATION_DATA              2   /* Cyrene is asking for Next Operation Data */
#define CONNECTION_QUERY_PAUSE                       3   /* Cyrene is asking for Waiting */

/** Kegs / Cyrene Communication Protocol **/
#define IPC_REQUEST_TYPE_C2K_OPEN_CONNECTION         1   /* Cyrene -> Kegs : Open Connection (Send Connection information) */
#define IPC_REQUEST_TYPE_C2K_CLOSE_CONNECTION        2   /* Cyrene -> Kegs : Stop Connection */
#define IPC_REQUEST_TYPE_C2K_GET_SNAPSHOT            3   /* Cyrene -> Kegs : Ask for Snapshot IIgs Data */
#define IPC_REQUEST_TYPE_C2K_GET_OPERATION           4   /* Cyrene -> Kegs : Ask for Next Operation Data */
#define IPC_REQUEST_TYPE_C2K_PAUSE                   5   /* Cyrene -> Kegs : Ask for Waiting */
#define IPC_REQUEST_TYPE_C2K_WRITE_DATA              6   /* Cyrene -> Kegs : Write Data into Kegs Memory (RAM,DOC,...) */

#define IPC_REQUEST_TYPE_K2C_SEND_SNAPSHOT           6   /* Kegs -> Cyrene : Send Snapshot IIgs Data (Memory) */
#define IPC_REQUEST_TYPE_K2C_SEND_OPERATION          7   /* Kegs -> Cyrene : Send Next Operation Data (Opcode,Operand,Hardware,Register) */
#define IPC_REQUEST_TYPE_K2C_CLOSE_CONNECTION        9   /* Kegs -> Cyrene : Close Connection (Kegs is exiting) */

/** IPC Shared Memory **/
#define IPC_SHARED_MEMORY_HEADER_SIZE                (1*1024)       /* 1 KB */
#define IPC_SHARED_MEMORY_BREAKPOINT_SIZE            (4*1024)       /* 4 KB */
#define IPC_SHARED_MEMORY_OUTPUT_SIZE                (1024*1024)    /* 1 MB */
#define IPC_SHARED_MEMORY_TOTAL_SIZE                 (IPC_SHARED_MEMORY_HEADER_SIZE+IPC_SHARED_MEMORY_BREAKPOINT_SIZE+IPC_SHARED_MEMORY_OUTPUT_SIZE)
/* Block Offset */
#define IPC_SHARED_MEMORY_OFFSET_DATA_HEADER         0
#define IPC_SHARED_MEMORY_OFFSET_DATA_BREAKPOINT     IPC_SHARED_MEMORY_HEADER_SIZE
#define IPC_SHARED_MEMORY_OFFSET_DATA_OUTPUT         (IPC_SHARED_MEMORY_HEADER_SIZE+IPC_SHARED_MEMORY_BREAKPOINT_SIZE)
/* Header */
#define IPC_SHARED_MEMORY_OFFSET_GOID                 0   /* QWORD */
#define IPC_SHARED_MEMORY_OFFSET_STATUS_BREAKPOINT    8   /* DWORD 0:Cyrene can Write/1:Kegs can read */
#define IPC_SHARED_MEMORY_OFFSET_NB_BREAKPOINT       12   /* DWORD 0-N */
#define IPC_SHARED_MEMORY_OFFSET_OUTPUT_READ         16   /* DWORD */
#define IPC_SHARED_MEMORY_OFFSET_OUTPUT_WRITE        20   /* DWORD */
#define IPC_SHARED_MEMORY_OUTPUT_WRITE_MISSED        24   /* DWORD */
#define IPC_SHARED_MEMORY_OFFSET_OUTPUT_SYSCALL      28   /* BYTE : 0/1 Trace System Calls */

DWORD CyreneIPCThreadInit(void *);
void SendCyreneData(int,struct cyrene_operation *);       /* Send next operation context to Cyrene */

void CopyOutlineIntoSharedMemory(int,DWORD,int,WORD,char *,struct cyrene_parameter *);
BYTE GetSharedMemoryCircularValue(int,unsigned char *);
void SetSharedMemoryCircularValue(int,unsigned char *,BYTE);

/********************************************************************/
