// "@(#)$KmKId: engine.h,v 1.9 2023-09-11 12:55:16+00 kentd Exp $"

/************************************************************************/
/*			KEGS: Apple //gs Emulator			*/
/*			Copyright 2002-2023 by Kent Dickey		*/
/*									*/
/*	This code is covered by the GNU GPL v3				*/
/*	See the file COPYING.txt or https://www.gnu.org/licenses/	*/
/*	This program is provided with no warranty			*/
/*									*/
/*	The KEGS web page is kegs.sourceforge.net			*/
/*	You may contact the author at: kadickey@alumni.princeton.edu	*/
/************************************************************************/

int
ENGINE_TYPE (Engine_reg *engine_ptr)
{
	register byte	*ptr;
	byte	*arg_ptr;
	Pc_log	*tmp_pc_ptr;
	Fplus	*fplus_ptr;
	byte	*stat;
	dword64	dfcyc, dplus_1, dcycles_tmp1, dplus_x_m1;
	register word32	kpc, acc, xreg, yreg, direct, psr, zero, neg7, addr;
	word32	wstat, arg, stack, dbank, opcode, addr_latch, tmp1, tmp2;
	word32	getmem_tmp, save_addr, pull_tmp, tmp_bytes, dummy1;

/** Cyrene Remote Debugger Integration **/
#ifdef _WIN32
    int cyrene_connection_status, cyrene_connection_query;
    word32 cyrene_psr;
    struct cyrene_operation current_operation;
    struct cyrene_parameter *cyrene_param;
    my_Cyrene(CYRENE_GET_PARAM,&cyrene_param,NULL);
#endif

	tmp_pc_ptr = 0;
	dummy1 = 0;
	if(tmp_pc_ptr || dummy1) {	// "use" tmp_pc_ptr to avoid warning
	}

	kpc = engine_ptr->kpc;
	acc = engine_ptr->acc;
	xreg = engine_ptr->xreg;
	yreg = engine_ptr->yreg;
	stack = engine_ptr->stack;
	dbank = engine_ptr->dbank;
	direct = engine_ptr->direct;
	psr = engine_ptr->psr;
	fplus_ptr = engine_ptr->fplus_ptr;
	zero = !(psr & 2);
	neg7 = psr;

	dplus_1 = fplus_ptr->dplus_1;
	dplus_x_m1 = fplus_ptr->dplus_x_minus_1;
	dfcyc = engine_ptr->dfcyc;

	g_ret1 = 0;

	while(dfcyc <= g_dcycles_end) {

		FETCH_OPCODE;

/** Cyrene Remote Debugger Integration **/
#ifdef _WIN32
        if(cyrene_param != NULL)
          {
            /** [CYRENE] : Increment Global Operation ID + Copy into shared Mamory **/
            cyrene_param->goid++;
            *((QWORD *) &cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_GOID]) = cyrene_param->goid;

            /** [CYRENE] : Get Next Operation information **/
            current_operation.dfcyc      = dfcyc;
            current_operation.a_reg      = acc & 0xFFFF;
            current_operation.x_reg      = xreg & 0xFFFF;
            current_operation.y_reg      = yreg & 0xFFFF;
            current_operation.pc_reg     = kpc & 0xFFFF;
            current_operation.pb_reg     = (BYTE) ((kpc >> 16) & 0xFF);
            current_operation.db_reg     = (BYTE) (dbank & 0xFF);
            current_operation.stack_reg  = stack & 0xFFFF;
            current_operation.direct_reg = direct & 0xFFFF;
            cyrene_psr                   = psr & (~0x82);      /* Rebuild the Status Register with extra neg7 and zero flags */
            cyrene_psr                  |= (neg7 & 0x80);
            cyrene_psr                  |= ((!zero) << 1);
            current_operation.status_reg = cyrene_psr & 0xFFFF;
            current_operation.opcode     = (unsigned char) (opcode & 0xFF);
            current_operation.operand_1  = (arg_ptr[1] & 0xFF);
            current_operation.operand_2  = (arg_ptr[2] & 0xFF);
            current_operation.operand_3  = (arg_ptr[3] & 0xFF);

            /** Get Softswitch values + Compute Cycle Count + Breakpoint Information (Read/Write/Jump Address/Value) **/
            GetOperationExtraInformation(&current_operation,cyrene_param);

            /** [CYRENE] : Update VBL Status **/
            cyrene_param->previous_vbl_status = cyrene_param->current_vbl_status;
            cyrene_param->current_vbl_status = ((current_operation.s_C019 & 0x80) == 0x80) ? 0 : 1;

            /*** [CYRENE] : Send next operation context to Cyrene ***/
            cyrene_connection_status = (cyrene_param->hwnd_cyrene_ipc == NULL) ? CONNECTION_STATUS_CLOSED : cyrene_param->cyrene_connection_status;
            cyrene_connection_query  = (cyrene_param->hwnd_cyrene_ipc == NULL) ? CONNECTION_QUERY_NOTHING : cyrene_param->cyrene_connection_query;
            if(cyrene_connection_status == CONNECTION_STATUS_OPEN && cyrene_connection_query != CONNECTION_QUERY_NOTHING)
              {
                /** Send AppleIIgs Data to Cyrene (Next Operation / Snapshot IIgs...) **/
                SendCyreneData(cyrene_connection_query,&current_operation);
              }

            /** [CYRENE] : Increment Global Cycle Count (GCC += nb_cycle) **/
            cyrene_param->gcc += current_operation.nb_cycle;
          }
#endif

		LOG_PC_MACRO();

		switch(opcode) {
		default:
			halt_printf("acc8 unk op: %02x\n", opcode);
			arg = 9
#include "defs_instr.h"
			* 2;
			break;
#include "instable.h"
			break;
		}
		LOG_PC_MACRO2();
	}

	engine_ptr->kpc = kpc;
	engine_ptr->acc = acc;
	engine_ptr->xreg = xreg;
	engine_ptr->yreg = yreg;
	engine_ptr->stack = stack;
	engine_ptr->dbank = dbank;
	engine_ptr->direct = direct;
	engine_ptr->dfcyc = dfcyc;

	psr = psr & (~0x82);
	psr |= (neg7 & 0x80);
	psr |= ((!zero) << 1);

	engine_ptr->psr = psr;

	return g_ret1;
}
